/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Header: /plx/source/Xclients/generic/RCS/IOCascade.h,v 1.5 1996/01/19 00:38:43 drapeau Exp $ */
/* $Log: IOCascade.h,v $
 * Revision 1.5  1996/01/19  00:38:43  drapeau
 * Fixed compilation errors for g++.
 *
 * Revision 1.4  1996/01/08  19:40:56  lovvik
 * Changed PlxInputCascadeCompress() to PlxCompressInputCascade.
 * Changed PlxOutputCascadeCompress() to PlxCompressOutputCascade.
 *
 * Revision 1.3  1996/01/07  04:36:33  lovvik
 * Added function prototypes for PlxInputCascadeCompress(),
 * PlxOutputCascadeGetChild(), and PlxOutputCascadeCompress().
 *
 * Revision 1.2  1995/12/29  20:30:14  lovvik
 * Added some defines for the new function PlxInputCascadeGetChild().
 * Also added a function prototype.
 *
 * Revision 1.1  1995/10/02  22:18:51  drapeau
 * Initial revision
 * */

#ifndef _OutputCascade_h
#define _OutputCascade_h
static char outputCascadeHeaderRcsId[] = "$Header: /plx/source/Xclients/generic/RCS/IOCascade.h,v 1.5 1996/01/19 00:38:43 drapeau Exp $";


#ifdef __cplusplus
extern "C"	{
#endif

#define MAXFORMATS 8
#define MAXINSTANCES 8
#define MAXSTANDARDS 8
#define MAXINPUTS 8
#define MAXOUTPUTS 8


/*
 * The following defines are used in calls to PlxInputCascadeGetChild() and PlxOutputCascadeGetChild().
 */
#define IOC_FirstAvailable	0x10000
#define IOC_InputOff		0xF000
#define IOC_Input1		0xE000
#define IOC_Input2		0xD000
#define IOC_Input3		0xC000
#define IOC_Input4		0xB000
#define IOC_Input5		0xA000
#define IOC_Input6		0x9000
#define IOC_Input7		0x8000
#define IOC_InputMenu		0x1000

#define IOC_OutputOff		0xF000
#define IOC_Output1		0xE000
#define IOC_Output2		0xD000
#define IOC_Output3		0xC000
#define IOC_Output4		0xB000
#define IOC_Output5		0xA000
#define IOC_Output6		0x9000
#define IOC_Output7		0x8000
#define IOC_OutputMenu		0x1000

#define IOC_FormatComp		0x0D00
#define IOC_FormatYC		0x0C00
#define IOC_FormatYUV		0x0B00
#define IOC_FormatRGB		0x0A00
#define IOC_FormatMenu		0x0100

#define IOC_Instance0		0x00F0
#define IOC_Instance1		0x00E0
#define IOC_Instance2		0x00D0
#define IOC_Instance3		0x00C0
#define IOC_Instance4		0x00B0
#define IOC_Instance5		0x00A0
#define IOC_Instance6		0x0090
#define IOC_Instance7		0x0080
#define IOC_InstanceMenu	0x0010

#define IOC_StandardNTSC	0x000D
#define IOC_StandardPAL		0x000B
#define IOC_StandardSECAM	0x000A
#define IOC_StandardMenu	0x0001

/* private structures for holding output cascade data */

typedef struct _tinstance
{
	Widget w;
	Widget pw;
	Widget standards[MAXSTANDARDS];
} tinstance;

typedef struct _tformat
{
	Widget w;
	Widget pw;
	tinstance instances[MAXINSTANCES];
} tformat;

typedef struct _tinput
{
	int channel;
	Widget cascade;
	tformat formats[MAXFORMATS];
} tinput;

typedef struct _toutput
{
	int channel;
	Widget cascade;
	tformat formats[MAXFORMATS];
} toutput;

/* public structures for holding output cascade data */

typedef union _signalSpec
{
  struct
  {
	unsigned	channel:8;
	unsigned	format:8;
	unsigned	instance:8;
	unsigned	standard:8;
  } s;
  unsigned int i;
} signalSpec;

typedef struct _PlxInputCascade
{
	Widget cascade;
	tinput *inputs[MAXINPUTS];
} PlxInputCascade;

typedef struct _PlxOutputCascade
{
	Widget		cascade;
	toutput*	outputs[MAXOUTPUTS];
} PlxOutputCascade;

void PlxShowInputSelections(PlxInputCascade*	p_me,
			    signalSpec		ss);

void PlxShowOutputSelections(PlxOutputCascade*	p_me,
			     signalSpec		ss);

PlxInputCascade* PlxCreateInputCascade(Widget		mparent,
				       Widget		fparent,
				       Widget		iparent,
				       Widget		sparent,
				       plx_IO*		plxio,
				       XtCallbackProc	callback);

Widget PlxInputCascadeGetChild(Widget 	parent, 
			       int 	childId);

void PlxCompressInputCascade(Widget parent);

PlxOutputCascade* PlxCreateOutputCascade(Widget		mparent,
					 Widget		fparent,
					 Widget		iparent,
					 Widget		sparent,
					 plx_IO*	plxio,
					 XtCallbackProc	callback);

Widget PlxOutputCascadeGetChild(Widget 	parent, 
			       int 	childId);

void PlxCompressOutputCascade(Widget parent);

#ifdef __cplusplus
}
#endif
#endif /* _OutputCascade_h */
