/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Header: /plx/source/Xclients/generic/RCS/JPEGP.h,v 1.25 1996/01/19 00:43:06 drapeau Exp $ */
/* $Log: JPEGP.h,v $
 * Revision 1.25  1996/01/19  00:43:06  drapeau
 * Fixed compilation errors for g++.
 *
 * Revision 1.24  1995/09/28  05:20:50  drapeau
 * Fixed includes for shared memory definitions.
 *
 * Revision 1.23  1995/09/26  07:15:17  drapeau
 * Made the file friendly to C++ compilers.
 *
 * Revision 1.22  1995/09/21  01:43:15  drapeau
 * Added code to take advantage of enhanced server reporting
 * capabilities: the XPlxQueryConfig() function new reports more specific
 * information about the capabilities of the hardware, including whether
 * JPEG hardware is available or not.
 *
 * Revision 1.21  1995/09/19  21:01:59  drapeau
 * Added include file for XPlx library function prototypes.
 *
 * Revision 1.20  1995/06/16  19:56:43  drapeau
 * Moved the include file "sys/param.h" to assure compilation on the AIX
 * platform.
 *
 * Revision 1.19  1995/05/27  07:43:26  drapeau
 * Added support for the widget keeping track of the last JFIF input and
 * output files handled, in order to make the programmer's task simpler
 * for using the same widget to open and/or save several JFIF files.
 *
 * Revision 1.18  1995/05/02  01:43:00  drapeau
 * Added new resource to support hardware zoom when available.
 *
 * Revision 1.17  1994/10/25  23:37:31  drapeau
 * Minor cosmetic change: moved the declaration of the "drawMethod"
 * resource to the area where the rest of the public resources are
 * declared, to clarify that the resource is public and not private
 * widget state.
 *
 * Revision 1.16  1994/10/22  02:18:08  drapeau
 * Removed obsolete definition of the JPEGImage data structure, since it
 * is no longer used in any of the code.
 *
 * Revision 1.15  1994/10/14  21:15:29  drapeau
 * Added structure fields to hold information event handlers that may be
 * installed on the widget.
 *
 * Revision 1.14  1994/09/03  06:16:39  drapeau
 * Added a resource "drawMethod", allowing the application programmer to
 * draw graphics over a JPEG image.
 *
 * Revision 1.13  1994/08/25  05:12:52  drapeau
 * Fixed an incorrect type declaration (the qTable field) so as to
 * eliminate potential memory leak problems.
 *
 * Revision 1.12  1994/05/03  17:57:46  drapeau
 * Added a resource called "diagnostics"; when set, the widget will print
 * diagnostic messages about itself to aid in debugging.
 *
 * Revision 1.11  1993/12/16  01:45:45  drapeau
 * Added fields to store information about whether the shared memory
 * extension is available.  Also included files needed by the C module
 * (JPEG.c) to determine whether the shared memory extension is present.
 *
 * Revision 1.10  1993/09/10  12:14:12  drapeau
 * This is the first public release of the JPEG widget header files.
 * */
static char jpegPHeaderRcsId[] = "$Header: /plx/source/Xclients/generic/RCS/JPEGP.h,v 1.25 1996/01/19 00:43:06 drapeau Exp $";

#ifndef _JPEGP_h
#define _JPEGP_h
#ifdef __cplusplus
extern "C"	{
#endif


#include <sys/param.h>
#include <JPEG.h>
#include <X11/CoreP.h>
#include <XPlxExt.h>
#include <XPlxProtos.h>
#include <X11/extensions/XShm.h>
#include <X11/Xlibint.h>
#include <X11/extensions/shmstr.h>

/* define unique representation types not found in <X11/StringDefs.h> */

#define XtRJPEGResource "JPEGResource"

typedef struct {
    int empty;
} JPEGClassPart;

typedef struct _JPEGClassRec {
    CoreClassPart	core_class;
    JPEGClassPart	jpeg_class;
} JPEGClassRec;

extern JPEGClassRec jpegClassRec;

typedef struct
{								    /* Public resources */
  Window	sourceWin;					    /* Indicates the source window from which to grab image data */
  XRectangle*	compressSourceRect;				    /* Rectangle within source window from which to grab.. */
								    /* ...and compresse image data */
  XRectangle*	compressDestRect;				    /* Only "width" and "height" are used: size of imaginary... */
								    /* ...rectangle specified in "XPlxGetCImage()" */
  XPlxCImage*	jpegData;					    /* Pointer to the image data to be compressed/decompressed */
  Window	destWin;					    /* Window into which to decompress & display JPEG image data */
  XRectangle*	displaySourceRect;				    /* Sub-rectangle of JPEG image from which to decompress */
  XRectangle*	displayDestRect;				    /* Rectangle within dest window into which to... */
								    /* ...compress and display JPEG image data */
  Boolean	squeezeDest;					    /* Whether to crop or squeeze the destination image */
  int		qFactor;					    /* Specifies image quality; higher # means lower quality */
  Boolean	getNewImage;					    /* When set to True, widget compresses & displays new image */
  Boolean	putNewImage;					    /* When True, tells widget there's new jpegData to display */
  String	jfifInputFile;					    /* Name of a JFIF file from which to load an image */
  String	jfifOutputFile;					    /* Name of a JFIF file to which to store an image */
  Boolean	useSharedMemory;				    /* Whether it's okay to use shared memory for image storage */
  Boolean	diagnostics;					    /* Whether to print widget diagnostics */
  XtExposeProc	drawMethod;					    /* Hook for app pgmr to draw his/her own graphics in widget */
  int		zoomFactor;					    /* Current zoom ratio observed by the widget */
  Boolean	jpegAvailable;					    /* Indicates whether hardware JPEG is really available */
  /* private state */
  unsigned char* qTable;					    /* QTable for this JPEG window */
  int		qTableSize;					    /* Size in bytes of space occupied by the "qTable" data */
  GC		compressGC;
  GC		decompressGC;
  int		visualClass;
  Boolean	dataToDisplay;					    /* Whether the widget is currently holding valid JPEG data */
  XRectangle	defaultCompressSourceRect;
  XRectangle	defaultCompressDestRect;
  XRectangle	defaultDisplaySourceRect;
  XRectangle	defaultDisplayDestRect;
  Bool		shmExtensionExists;				    /* Whether the MIT-SHM extension is present */
  int		shmMajorOpcode;					    /* The major protocol request code for the MIT-SHM extension */
  Bool		eventHandlerInstalled;				    /* Keeps track of whether the widget looks for window... */
  Widget	eventHandlerWidget;				    /* ...move events, and on which widget. */
  char*		lastJFIFInputFileName;				    /* Keeps track of the names of the last JPEG input and... */
  char*		lastJFIFOutputFileName;				    /* ...output files read and written by the widget */
} JPEGPart;

typedef struct _JPEGRec
{
  CorePart	core;
  JPEGPart	jpeg;
} JPEGRec;

#ifdef __cplusplus
}
#endif
#endif /* _JPEGP_h */
