/************************************************************************************************/
/******				Compression Monitor for C-Cube Microsystems CL550 processor  
/******				January 14, 1991   						
/******             Version 1.00						
/************************************************************************************************/

/* REGISTER MAP */

#ifdef __cplusplus
extern "C"	{
#endif


typedef struct RegStruct
{
	char name[80];
	unsigned long address;
	short permission;
	short width;
} CL550RegStruct;

/* Chip registers */

#define _HPeriod		0
#define _HSync			1
#define _HDelay			2
#define _HActive		3
#define _VPeriod		4
#define _VSync			5
#define _VDelay			6
#define _VActive		7
#define _DCT			8
#define _Config			9
#define _Huff_Enable		10
#define _S_reset		11
#define _Start			12
#define _HV_Enable		13
#define _Flags			14
#define _NMRQ_Enable		15
#define _DRQ_Enable		16
#define _StartFrame		17
#define _Version		18
#define _Init_1			19
#define _Init_2			20
#define _CodecSeq		21
#define _CodecCompH		22
#define _CodecCompL		23
#define _CoderAttr		24
#define _CodingIntH		25
#define _CodingIntL		26
#define _DecLength		27
#define _DecMarker		28
#define _DecResume		29
#define _DecDPCM		30
#define _CodeOrder		31
#define _Init_3			32
#define _Quant			33
#define _QuantSelect		34
#define _QuantSync		35
#define _QuantYC		36
#define _QuantAB		37
#define _R2Y			38
#define _G2Y			39
#define _B2Y			40
#define _R2U			41
#define _G2U			42
#define _B2U			43
#define _R2V			44
#define _G2V			45
#define _B2V			46
#define _VideoLatency		47
#define _HControl		48
#define _VControl		49
#define _VertLineCount		50
#define _Init_4			51
#define _Init_7			52
#define _FIFO			53
#define _HuffYAC		54
#define _HuffYDC		55
#define _HuffCAC		56
#define _HuffCDC		57
#define _Codec			58
#define _Block			59
#define _BlockAddress		60
#define _Init_5			61
#define _Init_6			62
#define _ZeroPkFilter		63

/* Board registers */

#define _PixelDataNormal	64
#define _PixelStatus		65
#define _EPROM			66

#define NumRegister		67
#define _FirstBoardReg		64

/* Init Registers */

#define Init_1			0
#define Init_2			1
#define Init_3			2
#define Init_4			3
#define Init_5			4
#define Init_6			5
#define Init_7			6
#define	QuantSync		7		
#define	QuantYC			8		
#define	QuantAB			9		
#define	VideoLatency		10		

/* Other Registers */

#define Config	   		0
#define	CodecSeq		1						 
#define	CodecCompH		2						
#define	CodecCompL		3				
#define	CoderAttr		4	
#define	CodingIntH		5		
#define	CodingIntL		6					
#define	DecLength		7							
#define	CodeOrder		8		
#define	QuantSelect		9
#define DCT			10		

/* Flags bits */

#define _Flags_FIFO_NotFull	0x00008000L
#define _Flags_FIFO_NotEmpty	0x00004000L
#define _Flags_Codec_RegBusy	0x00002000L
#define _Flags_BusError		0x00001000L
#define _Flags_MarkerCode	0x00000800L
#define _Flags_VSync		0x00000400L
#define _Flags_VInactive	0x00000200L
#define _Flags_Field0		0x00000100L
#define _Flags_FIFO_Empty	0x00000080L
#define _Flags_FIFO_QFull	0x00000040L
#define _Flags_FIFO_HalfFull	0x00000020L
#define _Flags_FIFO_3QFull	0x00000010L
#define _Flags_FIFO_NotQFull	0x00000008L
#define _Flags_FIFO_NotHalfFull	0x00000004L
#define _Flags_FIFO_Not3QFull	0x00000002L
#define _Flags_FIFO_Late	0x00000001L

/* Interrupt bits */

#define _Int_FIFO_NotFull	0x00008000L
#define _Int_FIFO_NotEmpty	0x00004000L
#define _Int_Codec_RegBusy	0x00002000L
#define _Int_BusError		0x00001000L
#define _Int_MarkerCode		0x00000800L
#define _Int_VSync		0x00000400L
#define _Int_VInactive		0x00000200L
#define _Int_FrameEnd		0x00000100L
#define _Int_FIFO_Empty		0x00000080L
#define _Int_FIFO_QFull		0x00000040L
#define _Int_FIFO_HalfFull	0x00000020L
#define _Int_FIFO_3QFull	0x00000010L
#define _Int_FIFO_NotQFull	0x00000008L
#define _Int_FIFO_NotHalfFull	0x00000004L
#define _Int_FIFO_Not3QFull	0x00000002L
#define _Int_FIFO_Late		0x00000001L

/* Status bits */

#define _Status_DRQ		0x80000000L
#define _Status_BLANK		0x40000000L
#define _Status_HSYNC		0x20000000L
#define _Status_VSYNC		0x10000000L

/* Configuration bits */

#define _Configuration_HSYNC	0x20000000L
#define _Configuration_VSYNC	0x10000000L
#define _Configuration_NOSYNC	0x00000000L

/* Read/write permission */

/* Control bits */										

#define _Control_Addr32		0x00000000L
#define _Control_Addr16		0x01000000L
#define _Control_Slave		0x00000000L
#define _Control_Master		0x02000000L
#define _Control_Data32		0x00000000L
#define _Control_Data16		0x04000000L
#define _Control_Reset0		0x00000000L
#define _Control_Reset1		0x80000000L

#define RR 1
#define WW 2
#define RW 3
#define DF 7

/* Pixel Modes */

#define _BYPASS		0
#define _422		1
#define _444		2
#define _4444		3
#define _RGB_422	4
#define _RGB_444	5
#define _444_422	6

/* Error codes */

#define NoError		0
#define YesError	1
#define BusError	2
#define FileError	3
#define LogicalError	4
#define MemError	5

/* Global variables */

extern short MODE_PIXEL,MODE_MSTR,MODE_INTL,MODE_FEND,MODE_DECOMPRESS;

extern short CL550HPeriod,CL550HDelay,CL550HActive,CL550HSync;
extern short CL550VPeriod,CL550VDelay,CL550VActive,CL550VSync;
extern short CL550ImageWidth,CL550ImageHeight;

extern unsigned long CL550BaseAddress;
extern CL550RegStruct CL550Register[];

extern short TraceRegisters,TraceHuffman,TraceQTables;
extern FILE *TraceFile;
extern short TraceMode,TraceCount;

extern short FileIO;
extern short Error;
extern short UseBoard;
extern short Comp;

/* Display mode control */

extern short DisplayRegisters,DisplayFlags,DisplayQTables,DisplayPixelStat;
extern short DisplayQLoad,DisplayImage,DisplayConfiguration,DisplayVertLineCnt,DisplayHuff;

extern short QFactor;

extern unsigned char *CompressedDataPtr,*ImageDataPtr;
extern unsigned long *ShadowPtr;
extern short Register[];
extern unsigned short compQTable[],decompQtable[];
extern unsigned long CL550CodeSize;

#define _NumInitReg	11
#define _NumPixelMode	7
#define _RegName	11

extern unsigned long C_InitRegValues[_NumInitReg][_NumPixelMode];
extern unsigned long D_InitRegValues[_NumInitReg][_NumPixelMode];
extern unsigned long ConfigRegValues[_RegName][_NumPixelMode];

#ifdef __cplusplus
}
#endif
