/*
 * (c) Copyright 1991-1995 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log: XPlxExt.h,v $
 * Revision 1.21  1996/01/19  00:12:46  drapeau
 * Fixed compilation errors for g++.
 *
 * Revision 1.20  1996/01/11  01:13:56  drapeau
 * Modified data structures to support a fully robust input model.  This
 * means adding "instance" information both to the plx_IO_list structure
 * (in the form of a "bundle" field) and instance information added to
 * the plx_format_list data structure (e.g., if an input bundle has
 * multiple composite cables).
 *
 * Revision 1.19  1995/12/23  02:49:03  drapeau
 * Moved the location of the input instance information: it's now part of
 * the same data structure that holds the channel info.
 *
 * Revision 1.18  1995/12/20  00:48:23  drapeau
 * Added some defined constants to support applications that work with
 * both video extensions (ParallaxVideo and XVideo, or "Xv").
 *
 * Revision 1.17  1995/10/17  19:10:32  drapeau
 * Modified plx_IO data structure to ease backward-compatibility.
 *
 * Revision 1.16  1995/09/28  04:55:54  drapeau
 * Added a platform-specific defined constant to help Solaris
 * applications use shared memory.
 *
 * Revision 1.15  1995/09/26  07:14:01  drapeau
 * Made the file friendly to C++ compilers.
 *
 * Revision 1.14  1995/09/21  00:21:15  drapeau
 * Fixed the XPlxHardwareInfo data structure to make it easier to convert
 * X protocol replies into the format needed by the data structure to
 * give easy access to application programmers.
 *
 * Revision 1.13  1995/08/16  18:48:23  drapeau
 * Minor change, fixed a typo in an explanatory comment.
 *
 * Revision 1.12  1995/08/03  18:48:41  drapeau
 * Added two new tag values defined by Andy Goodrich to improved the
 * graphics-over-video scheme for Solaris 2.4.
 *
 * Revision 1.11  1995/05/30  22:59:43  drapeau
 * Minor change: added defined constant used as key data when storing
 * Parallax-specific extension data in a Display structure.
 *
 * Revision 1.10  1994/10/22  04:30:35  drapeau
 * Added include files to support shared memory operations.
 *
 * Revision 1.9  1994/10/22  02:19:34  drapeau
 * Corrected the data structure definition for XPlxCImage.
 *
 * Revision 1.8  1994/10/03  16:10:28  drapeau
 * Added two new defined constants specifically as a short-term solution
 * to a problem for the HP platform.  The two new channel definitions,
 * "PLX_OUTPUT_JPEG" and "PLX_OUTPUT_JPEG_OFF", are to allow applications
 * such as MovieTool to send JPEG playback (decompression) to the output
 * channel.
 *
 * Revision 1.7  1994/08/27  05:13:46  drapeau
 * Added inclusion of a new file containing function prototypes for the
 * XPlx library.
 *
 * Revision 1.6  1993/10/26  17:54:27  drapeau
 * Added ifdefs around the header file to prevent multiple inclusion of
 * data structures and definitions.
 *
 * Revision 1.5  1993/10/26  17:53:43  drapeau
 * *** empty log message ***
 *
 * Revision 1.4  91/10/24  12:41:16  eric
 * use char * instead of caddr_t so we don't need sys/types.h
 * 
 * Revision 1.3  91/10/17  11:22:33  dave
 * added shared memory stuff to Image structure
 * 
 * Revision 1.2  91/08/19  12:40:40  eric
 * added rcs header
 * 
 *
 */
#ifndef _XPlxExtHeader_
#define _XPlxExtHeader_

#ifdef H_ID
#ifndef lint
static char *rcsid_XPlxExt_h = "@(#)$Id: XPlxExt.h,v 1.21 1996/01/19 00:12:46 drapeau Exp $ Parallax Graphics Inc";
#endif
#endif

#include	<X11/extensions/XShm.h>				    /* Used for XPlxCImage data structure shared memory field */

#ifdef __cplusplus
extern "C"	{
#endif


/***************************************************************************
 The following are the immediate defines for the Parallax video extension. 
****************************************************************************/

/*
 * The following is a key value used to store Plx-specific data in
 * the extension data part of a Display pointer.  The data are used
 * for accessing Parallax-specific information, such as which
 * hardware instance is currently in use (i.e., when multiple
 * Parallax boards are being used in a single system on a single
 * screen).
 */

#define XPlxHardwareInstanceCode	7272221

/*
 * The following defined constant is a key value used to mark that the
 * Display used for this connection is connected to a server that uses
 * the XPlx (i.e., "XVideo") X protocol extension.  This key will be
 * stored in the Display's "extension_data" list, and will be stored
 * in the "number" field.
 */

#define XPlxNativeExtension		7272222

/*
 * The next defined constant denotes that this server is using the Xv
 * extension.
 */

#define XPlxXvExtension			0x70615243

/*
 * The next defined constant denotes that no video extension was found
 * in the Display pointer being used for protocol requests.
 */

#define XPlxNoExtension			0

/*
The following are valid values of the plx_hardware variable updated as
a result of a XPlxQueryConfig call.  */

#define	PLX_UNKNOWN	0					    /* Unknown configuration */
#define	PLX_1280	2					    /* 1280 Series	*/	
#define	PLX_VIPER	3					    /* Viper Series	*/
#define	PLX_VIDEOVIEW	4					    /* VideoView	*/
#define	PLX_XVIDEO_8	8					    /* XVideo, eight bitplane version */
#define	PLX_XVIDEO_24	9					    /* XVideo, twenty four bitplane version	*/

/*
The following are valid nvselect Parameters for the XPlxVideoValuesSave and
XPlxVideoValuesRecall functions.
*/

#define USER_0		0					    /* user 0 */
#define USER_1		1					    /* user 1 */
#define FACTORY		2					    /* factory. XPlxVideoValuesRecall only */

/*
The following are valid values of the standard parameter of XPlxInputSelect and
XPlxOutputSelect and are vaild standard values returned by XPlxQueryVideo.
*/
#define	PLX_NTSC	2					    /* Video standards...	*/
#define	PLX_PAL		4
#define	PLX_SECAM	5

/*
The following are valid values of the channel parameter of
XPlxVideoInputSelect and XPlxVideoOutputSelect and are valid channel values
returned by XPlxQueryConfig.
*/
#define	PLX_IO_OFF		0				    /* Various input/output defs... */
#define	PLX_INPUT_0		1
#define	PLX_INPUT_1		2
#define PLX_OUTPUT_0		32	
#define PLX_OUTPUT_JPEG		33				    /* For HP platform only, used to produce output of JPEG... */
#define PLX_OUTPUT_JPEG_OFF	34				    /* ...decompressed images. */
/*
The following are valid values of the format parameter of
XPlxVideoInputSelect and XPlxVideoOutputSelect and are vaild
format values returned by XPlxQueryConfig.
*/
#define	PLX_COMP	2					    /* Composite	*/
#define	PLX_YC		3					    /* SuperVHS	*/
#define	PLX_YUV		4					    /* YUV Component */
#define	PLX_RGB		5					    /* RGB Compontent */

/*
The following are valid values of the type parameter of XPlxVideoInputSelect
and XPlxVideoOutputSelect and are vaild  type values returned by
XPlxQueryConfig.
*/
#define	PLX_YUV7	1					    /* Seven bit YUV format described in */
								    /* 1280 or VIPER Series hardware manual */
#define	PLX_RGB24	4					    /* Twenty four bit RGB (True Color)	*/
#define	PLX_RGB8	5					    /* Eight bit Pseudo color		*/
#define PLX_GRAY5	7					    /* Five bit Grayscale (bit <0>=0)	*/
								    /* see Viper Manual			*/
#define	PLX_GRAY7	8					    /* Seven bit grayscale (bit <0>=0)	*/
#define	PLX_GRAY8	9					    /* Eight bit grayscale			*/

/*
The following are valid values of the type parameter of XPlxVideoTag.
*/	
#define	PLX_VIDEO	2					    /* Video pixels	*/
#define	PLX_VIDEO_OVR	3					    /* Graphics overlay	*/
#define	PLX_GRAPHICS_1	4					    /* One bit graphics	*/
#define	PLX_GRAPHICS_8	5					    /* Eight bit graphics	*/
#define	PLX_GRAPHICS_24	6					    /* Twenty four bit graphics */
#define PLX_GRAPHICS    7					    /* just graphics, superseeds 1,8,24 above */
#define PLX_CLIENT_PREPARE 8					    /* Client prepares surface. */
#define PLX_SERVER_PREPARE 9					    /* Server prepares surface. */


/***************************************************************************
The following is the data structure maintained by the Parallax video
extension and made accessible to the programmer.
***************************************************************************/

/*
This data structure contains information about a particular video signal.
*/

typedef struct {
	unsigned	standard;				    /* PLX_NTSC, PLX_PAL, or PLX_SECAM */
	unsigned	w;					    /* Visible width of signal in pixels */
	unsigned 	h;					    /* Visible height of signal in lines */
	unsigned 	b;					    /* Digitizable vertical int. in lines */
	unsigned 	r;					    /* Frame rate in Hz	*/
	unsigned	interlaced;				    /* TRUE if signal is interlaced; */
	unsigned 	sync_ok;				    /* TRUE if good sync at input. */
								    /* Valid only as a	*/
								    /* result of a XPlxQueryVideo call. */
} plx_signal;

/*
The following is a list of signal definitions available for a particular channel
*/
typedef struct _plx_signal_list {
	plx_signal	*signal;				    /* Pointer to a plx_signal structure */ 
	struct _plx_signal_list *next_signal;			    /* Next plx_signal_list entry */
} plx_signal_list;

/* 
The following is a list of formats available for a particular channel
*/
typedef struct _plx_format_list {
  unsigned short		instance;			    /* For multiple cables of the same type on this input bundle */
  unsigned short		format;				    /* PLX_(COMP, YC, YUV, RGB)	*/
  struct _plx_format_list*	next_format;			    /* Next plx_format_list entry */
} plx_format_list;

/*
The following is a list of types available for a particular channel
*/
typedef struct _plx_type_list {
	unsigned type;						    /* PLX_(YUV7, RGB24, RGB8, GRAY7, GRAY8) */
	struct _plx_type_list *next_type;			    /* Next plx_type_list entry */
} plx_type_list;

/*
The following is a list of video input and output channels available for
this hardware configuration and the signal definitions and formats
available for each channel.
*/

typedef struct _plx_IO_list
{
  unsigned short	bundle;					    /* In case a digitizer is "split" into several bundles... */
								    /* ...of inputs (this will always be zero for output). */
  unsigned short	channel;				    /* INPUT_0, INPUT_1, OUTPUT_0... */
  plx_signal_list*	signal_list;				    /* List of signal definitions supported by this channel */
  plx_format_list*	format_list;				    /* List of formats supported by this channel */
  plx_type_list*	type_list;				    /* List of types supported by this channel */
  struct _plx_IO_list*	next_IO;				    /* Next plx_IO_list entry */
} plx_IO_list;


/*
 * The following structure holds detailed information about the
 * Parallax hardware being managed by the window server.
 */

typedef union _XPlxHardwareInfo
{
  struct
  {
    unsigned int	overlay			: 1;		    /* 1 = this is an overlay card, 0 = integrated framebuffer */
    unsigned int	zoom			: 1;		    /* 1 = hardware has zoom ability, 0 = not */
    unsigned int	numJPEGCompressors	: 2;		    /* Number of JPEG chips on this hardware (0-3) */
    unsigned int	numMPEGDecoders		: 2;		    /* Number of MPEG decode chips: if MPEG is supported at... */
								    /* ...all, both MPEG-1 and MPEG-2 will be available, so... */
								    /* ... this field stands for both types. */
    unsigned int	numH320Compressors	: 2;		    /* Number of H.320 chips: if H.320 is supported, then so... */
								    /* ...will H.261, so this field stands for both types. */
    unsigned int	reserved		: 16;		    /* Unused for now, will hold additional capabilities later */
    unsigned int	hardware		: 8;		    /* Holds original hw info returned by XPlxQueryConfig(),... */
								    /* ...kept for backward compatibility. */
  } info;
  unsigned int		allBits;				    /* Allows easy conversion b/w X reply and this... */
} XPlxHardwareInfo;						    /* ...structure (QueryConfig returns an unsigned int here). */


/*
The following is a structure that holds the hardware type and io lists.
A pointer to one of these is returned by XPlxQueryConfig.
*/

typedef struct _plx_IO
{
  unsigned int	overlay			: 1;			    /* 1 = this is an overlay card, 0 = integrated framebuffer */
  unsigned int	zoom			: 1;			    /* 1 = hardware has zoom ability, 0 = not */
  unsigned int	numJPEGCompressors	: 2;			    /* Number of JPEG chips on this hardware (0-3) */
  unsigned int	numMPEGDecoders		: 2;			    /* Number of MPEG decode chips: if MPEG is supported at... */
								    /* ...all, both MPEG-1 and MPEG-2 will be available, so... */
								    /* ... this field stands for both types. */
  unsigned int	numH320Compressors	: 2;			    /* Number of H.320 chips: if H.320 is supported, then so... */
								    /* ...will H.261, so this field stands for both types. */
  unsigned int	reserved		: 16;			    /* Unused for now, will hold additional capabilities later */
  unsigned int	hardware		: 8;			    /* Holds original hw info returned by XPlxQueryConfig(),... */
								    /* ...kept for backward compatibility. */
  plx_IO_list*	inputs;
  plx_IO_list*	outputs;
} plx_IO;

/*
 * Data structure for "compressed image" data, used by
 * compressed image manipulation routines.
 */

typedef struct _XPlxCImage
{
  int			width, height;				    /* Size of image */
  int			size;					    /* Size in bytes of image */
  char*			data;					    /* Pointer to image data */
  XShmSegmentInfo*	info;					    /* Shared memory segment info */
} XPlxCImage;


#if defined (__hpux) || defined (SVR4)
#define PlxSharedMemoryExtensionName "PLX-SHM"
#else
#define PlxSharedMemoryExtensionName "MIT-SHM"
#endif

#ifdef __cplusplus
}
#endif

#include <XPlxProtos.h>						    /* Include function prototypes */
#endif
