/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Header: /plx/source/Xclients/generic/RCS/XVideoInP.h,v 1.24 1996/01/22 06:20:29 drapeau Exp $ */
/* $Log: XVideoInP.h,v $
 * Revision 1.24  1996/01/22  06:20:29  drapeau
 * Added a new resource, "stillDelay", to allow control over the time
 * spent waiting between the time that the video digitizer is stopped and
 * the time a still image Pixmap or XImage is grabbed.
 *
 * Revision 1.23  1996/01/19  04:04:08  drapeau
 * Added support for both XImages and their faster Pixmap counterparts
 * for still images.  The widget detects which to use and uses that.
 *
 * Revision 1.22  1996/01/19  00:49:37  drapeau
 * Fixed compilation errors for g++.
 *
 * Revision 1.21  1996/01/10  08:21:36  drapeau
 * Added a new resource, "stillImage", to give the application programmer
 * a copy of the XImage created when stilling the widget.
 *
 * Revision 1.20  1995/12/20  00:51:52  drapeau
 * Added functionality to the autoSearch feature: now autoSearch can be
 * done on both channels (i.e., digitizers), or channel PLX_INPUT_0, or
 * channel PLX_INPUT_1, or not at all.  Before, autoSearch was either
 * both digitizers or none.
 *
 * Revision 1.19  1995/11/13  20:34:10  drapeau
 * Exported the widget's GC so that programmers doing graphics-over-video
 * operations can re-tag the GC.  This exporting of the GC is dangerous,
 * but perhaps a necessary evil to enable GOV.
 *
 * Revision 1.18  1995/09/26  07:13:43  drapeau
 * Made the file friendly to C++ compilers.
 *
 * Revision 1.17  1995/09/19  21:02:12  drapeau
 * Added include file for XPlx library function prototypes.
 *
 * Revision 1.16  1994/10/26  01:52:04  drapeau
 * Added a new resource, "drawMethod", to the widget, allowing
 * application programmers to draw graphics over the video without the
 * graphics being overwritten by video.
 *
 * Revision 1.15  1994/10/14  21:15:09  drapeau
 * Added structure fields to hold information event handlers that may be
 * installed on the widget.
 *
 * Revision 1.14  1994/08/27  06:14:07  drapeau
 * Added a new resource, autoSearch, that allows the widget to
 * automatically search for a valid video input on all channels if the
 * programmer's first choice has no valid video.  By default, this is set
 * to False.
 *
 * Revision 1.13  1994/07/20  21:52:48  drapeau
 * Added a resource "instance" to reflect the new model of the hardware.
 * For example, the PowerVideo/700 product has at least two composite
 * input cables available on either digitizer; the "instance" field is
 * used to specify which is to be used.
 *
 * Also made the "hardwareParams" field public so that applications have
 * better information about the hardware being used.
 *
 * Revision 1.12  1994/06/09  22:14:30  drapeau
 * Changed the implementation of still images in the VideoIn widget to
 * use XImages instead of Pixmaps.  This was done to support the HP
 * platform.
 *
 * Revision 1.11  1994/05/03  17:57:15  drapeau
 * Added a resource called "diagnostics"; when set, the widget will print
 * diagnostic messages about itself to aid in debugging.
 *
 * Revision 1.10  1993/09/10  12:14:37  drapeau
 * This is the first public release of the VideoIn widget header files.
 * */
#ifndef _XVideoInP_h
#define _XVideoInP_h
#ifdef __cplusplus
extern "C"	{
#endif

static char xVideoInPHeaderRcsId[] = "$Header: /plx/source/Xclients/generic/RCS/XVideoInP.h,v 1.24 1996/01/22 06:20:29 drapeau Exp $";

#include <XVideoIn.h>
#include <X11/CoreP.h>						    /* Include superclass private header file */
#include <XPlxExt.h>						    /* Include file for X extensions for Parallax video */
#include <XPlxProtos.h>						    /* Function prototypes for XPlx library functions  */

/* define unique representation types not found in <X11/StringDefs.h> */

#define XtRXVideoInResource "XVideoInResource"


/* Defined constants used internally by this widget */

#define DefaultBrightness	128
#define DefaultContrast		128
#define DefaultHue		128
#define DefaultSaturation	128
#define NoNVRAM			-1

typedef struct
{
  int empty;
} XVideoInClassPart;

typedef struct _XVideoInClassRec
{
  CoreClassPart		core_class;
  XVideoInClassPart	xVideoIn_class;
} XVideoInClassRec;

extern XVideoInClassRec xVideoInClassRec;

typedef struct
{
  int		brightness;					    /* Public resources */
  int		contrast;
  int		hue;
  int		saturation;
  Boolean	cropped;					    /* Indicates whether video be cropped or squeezed */
  Boolean	normalAspectRatio;				    /* Whether aspect ratio should be 4:3 (normal) or not */
  XRectangle*	sourceRect;					    /* Rectangle for the incoming video signal */
  XRectangle*	displayRect;					    /* Rectangle in which digitized video signal is displayed */
  Boolean	live;						    /* Whether video should be shown live or still-frame */
  Boolean	newFrame;					    /* Whether to grab a new frame of video for digitization */
  int		saveColorSettings;				    /* Specifies where to save current color settings */
  int		restoreColorSettings;				    /* Where to load color settings: from factory, user0, user1 */
  int		signalStandard;					    /* Which video standard to be used: NTSC, PAL, SECAM */
  int		signalFormat;					    /* Format of the video input: RGB, SVHS, etc. */
  int		hardwareType;					    /* Indicates board being used by the computer */
  Boolean	interlaced;					    /* Read-only: whether current signal is an interlaced signal */
  Boolean	sync;						    /* Read-only: whether valid sync signal is detected */
  Dimension	signalWidth;					    /* Number of rows and columns provided by the video... */
  Dimension	signalHeight;					    /* ...standard currently selected as input */
  Dimension	blankIntervalHeight;				    /* Number of rows for blanking interval for current input */
  Boolean	showBlankInterval;				    /* Whether to digitize blanking interval with rest of signal */
  int		channel;					    /* Indicates channel on which to look for video signal */
  String	videoParameterString;				    /* String specifies simple special effects for the board */
  Boolean	resizeParent;					    /* Whether to try and resize the widget's parent when... */
								    /* ...this widget is resized */
  int		instance;					    /* Which instance of an input to use on a digitizer */
  Boolean	diagnostics;					    /* Whether to print widget diagnostics */
  plx_IO*	hardwareParams;					    /* Holds all relevant info about the board being used. */
  XtExposeProc	drawMethod;					    /* Hook for app pgmr to draw his/her own graphics in widget */
  int		autoSearch;					    /* Whether to auto-search for a valid input signal */
  GC		gc;
  XImage**	stillImage;					    /* Used to return a still image to the app. programmer */
								    /* Private variables */
  Dimension	startingY;					    /* Y coordinate, where to begin digitizing video after... */
								    /* ....correcting for the blanking interval height. */
  XImage*	stillImagePixmap;				    /* Used to hold a still image when "live" resource is False */
  Boolean	pixmapExists;					    /* Is there a Pixmap to be used for displaying still video? */
  XRectangle	defaultSourceRect;
  XRectangle	defaultDisplayRect;
  Window	videoWindow;					    /* Points to window used for XPlx extension calls */
  Bool		eventHandlerInstalled;				    /* Keeps track of whether the widget looks for window... */
  Widget	eventHandlerWidget;				    /* ...move events, and on which widget. */
  Pixmap	stillPixmap;					    /* Used for stills on non-overlay boards; faster than XImage */
  int		stillDelay;					    /* Time in msecs to wait b/w stopping digitizer and... */
								    /* ...grabbing the still image. */
} XVideoInPart;

typedef struct _XVideoInRec
{
  CorePart	core;
  XVideoInPart	xVideoIn;
} XVideoInRec;

#ifdef __cplusplus
}
#endif
#endif /* _XVideoInP_h */
