/*
 *
 *   Copyright (c) 1994 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 */
/*
** File: 
**
**  Xvplx.h: Client accessable definitions for Xv video extension
**           DD level associated with Parallax adaptors.
**
** Author: 
**
**   Andy Goodrich (Parallax)
**
** Revisions:
**
**   20.01.94 Goodrich
**     - Initial coding.
*/
#ifndef Xvplx_h_INCLUDED
#define Xvplx_h_INCLUDED
#include <sys/types.h>
#include <Xvddx.h>						    /* Pick up generic DD level definitions. */

/*
 *  DEVICE-DEPENDENT ATTRIBUTES SUPPORTED:
 *
 *  Long attributes:
 *     1) "XV_PLX_QUERY_SIGNALS" - return parallax signal info on each encoding.
 *     2) "XV_PLX_TABLE_COMPRESSION" - access compression q table information.
 *     3) "XV_PLX_TABLE_DECOMPRESSION" - access decompress. q table information.
 *     4) "XV_PLX_VIDEO_PARAMETER_STRING" - send string to device support.
 *
 *  Integer attributes:
 *     1) "XV_BRIGHTNESS" - get or set brightness level (-1000..1000).
 *     2) "XV_CONTRAST" - get or set contrast level (-1000..1000).
 *     3) "XV_HUE" - get or set hue level (-1000..1000).
 *     4) "XV_PLX_CCUBE - return number of ccube chips on device.
 *     5) "XV_PLX_INIT_FROM_NVRAM" - read nvram and reinitialize
 *     6) "XV_PLX_CCUBE - return number of ccube chips on device.
 *     7) "XV_PLX_VALUE_RECALL" - load values from user0, user1, factory.
 *     8) "XV_PLX_VALUE_SAVE" - save values to user0, user1.
 *     9) "XV_SATURATION" - get or set saturation level (-1000..1000).
 *    10) "XV_STATUS" - get status of this port (see Xvddx.h for status values).
 *
 *  Control operations:
 *     1) "XV_PLX_GETCIMAGE" - get a compressed image
 *     2) "XV_PLX_FIRST_PUTCIMAGE" - put first of a set of compressed images
 *     3) "XV_PLX_PUTCIMAGE" - put subsequent compressed image
 */

typedef struct _XvAtomsRec
{
  Atom	encoding;
  Atom	crop;
  Atom	zoom;
  Atom	querySignals;
  Atom	tableCompression;
  Atom	tableDecompression;
  Atom	videoParameterString;
  Atom	brightness;
  Atom	contrast;
  Atom	hue;
  Atom	ccube;
  Atom	initFromNVRAM;
  Atom	valueRecall;
  Atom	valueSave;
  Atom	saturation;
  Atom	status;
  Atom	getCImage;
  Atom	firstPutCImage;
  Atom	putCImage;
  Atom	xImage;
} XvAtomsRec;



/*
 *  SIGNAL INFORMATION RECORD:
 */
typedef struct _XvplxSignalRec {
	unsigned int encoding_id;	/* Encoding id for this configuration. */
    unsigned int channel;   	/* Input or output channel no. */
    unsigned int type;      	/* = 0: input channel, = 1: output channel. */
    unsigned int standard;		/* Timing standard (e.g., PLX_NTSC). */
    unsigned int format;		/* Format (e.g., PLX_COMP). */
    unsigned int width;
    unsigned int height;
    unsigned int blank;
    unsigned int rate;
    unsigned int interlaced;
    unsigned int sync_ok;
} XvplxSignalRec;

/*enum {*/  /* Values for channel field of XvplxSignalRec: */
/*	PLX_INPUT_0,
	PLX_INPUT_1,
};
*/

/*
 *  ADDITIONAL ENCODING TYPES
 */
enum { /*            vv        */
    XV_ZOOM_4  = 0x00800000,
    XV_ZOOM_2  = 0x00400000,
	XV_NTSC_2X = XV_NTSC | XV_ZOOM_2,
	XV_NTSC_4X = XV_NTSC | XV_ZOOM_4,
	XV_PAL_2X = XV_PAL | XV_ZOOM_2,
	XV_PAL_4X = XV_PAL | XV_ZOOM_4,
	XV_SECAM_2X = XV_SECAM | XV_ZOOM_2,
	XV_SECAM_4X = XV_SECAM | XV_ZOOM_4,
};

#endif /* Xvplx_h_INCLUDED */

