/***************************************************************************
 *	movie.h - Private j_movie include file
 *
 *	(C) Copyright 1992 by Paradise Software, Inc.
 *
 *	Written by John Malleo-Roach
 *
 *	Version %G% - %I%
 *
 ***************************************************************************/
#ifndef _movieH_
#define _movieH_
#ifdef __cplusplus
extern "C"	{
#endif

static char movieHeaderRcsId[] = "$Id: movie.h,v 1.78 1996/01/19 00:53:31 drapeau Exp $";

/* CONSTANTS */

#ifndef NULL
#define NULL 			0
#endif
#ifndef TRUE
#define TRUE 			1
#endif
#ifndef FALSE
#define FALSE 			0
#endif
#ifndef BOOLEAN
#define BOOLEAN			int
#endif


#define JPEG_VERSION_1		  1				    /* Supported j_movie versions */
#define JPEG_VERSION_2		  2
#define JPEG_VERSION_3		  3
#define JPEG_VERSION_4		  4				    /* Adds support for new opcodes such as NEW_QFACTOR, etc. */

#define LOAD_AUDIO_3		226				    /* Bytes 200-255 are reserved as code words by... */
#define LOAD_AUDIO_2		227				    /* ...the compression/playback engine. */
#define LOAD_AUDIO_1		228
#define LOAD_AUDIO_0		229
#define NEW_QFACTOR		230				    /* Tells movie engine to load new Q tables; next 2 bytes... */
								    /* ...are an integer w/new Q factor. */
#define LOAD_JPEG		236
#define END_FRAME		237
#define CLEAR_FRAME		238
#define LOAD_UNIFLIX		239
#define LOAD_CMAP		249
#define APP_DATA_0		240				    /* Reserved for storing application-specific data in a... */
#define APP_DATA_1		241				    /* ...movie.  The opcode is followed by an integer... */
#define APP_DATA_2		242				    /* ...indicating the number of app-specific bytes to... */
#define APP_DATA_3		243				    /* ...follow, followed by the app-specific data. */
#define LOAD_COMPRESSED_AUDIO_0	244				    /* Reserved for future use in case compressed audio... */
#define LOAD_COMPRESSED_AUDIO_1	245				    /* ...standards are used; these opcodes behave as the... */
#define LOAD_COMPRESSED_AUDIO_2	246				    /* ...LOAD_JPEG opcodes do; that is, they are followed by... */
#define LOAD_COMPRESSED_AUDIO_3	247				    /* ...an integer indicating the number of audio bytes to...  */
								    /* ...follow. */

#define FILLER			(128 - sizeof(PlxAudioHeader) - 68)

/* TYPEDEFS */

typedef struct _PlxAudioHeader					    /* This data structure is stolen from the Sun Audio... */
{								    /* ...header format. */
  unsigned	sample_rate;					    /* samples per second */
  unsigned	samples_per_unit;				    /* samples per unit */
  unsigned	bytes_per_unit;					    /* bytes per sample unit */
  unsigned	channels;					    /* # of interleaved channels */
  unsigned	encoding;					    /* data encoding format */
  unsigned	endian;
  unsigned	data_size;					    /* length of data (optional) */
} PlxAudioHeader;



typedef struct _PlxMovieOptions
{
  unsigned	frameOrField	: 1;				    /* Indicates whether the movie is storing a series of... */
								    /* ...frames or fields; if 0 (default), the movie is... */
								    /* ...a series of full frames; if 1, the movie is a... */
								    /* ...series of fields, in which case two fields should... */
								    /* ...be stored for each frame.  In other words, for... */
								    /* ...field-wise movies, each frame will have two... */
								    /* ..."LOAD_JPEG" opcodes, one for each field in the frame. */
  unsigned	polarity	: 1;				    /* Useful only for field-wise compression: if 0, then... */
								    /* ...the 1st LOAD_JPEG opcode seen for a frame is meant... */
								    /* ...to start at line 0 during display; if 1, then the... */
								    /* ...1st LOAD_JPEG opcode seen for a frame is meant to... */
								    /* ...start at line 1 during display, and the 2nd... */
								    /* ... LOAD_JPEG opcode for the frame is meant to start... */
								    /* ...at line 0 during display. */
} PlxMovieOptions;



typedef struct							    /* JPEG header information	*/
{
  char			magic[8];				    /* defined as "j_movie"				*/
  int			version;				    /* Version number					*/
  int			fps;					    /* frames per second				*/
  int			frames;					    /* Number of frames in the sequence			*/
  int			width;					    /* Width of each frame */
  int			height;					    /* Height of each frame 				*/
  int			bandwidth;				    /* Bandwidth required for playback			*/
  int			qfactor;				    /* Quantization factor used in compression	*/
  int			mapsize;				    /* Number of colors used in colormap		*/
  int			indexbuf;				    /* Offset in file of frame indexes			*/
  int			tracks;					    /* Number of audio tracks used			*/
  int			volbase;				    /* Baseline for volume control			*/
  int			audioslice;				    /* Number of audio bytes per frame			*/
  PlxAudioHeader	audio;					    /* Audio information				*/
  int			firstValidFrame;			    /* First & last frames of the movie to be played, useful... */
  int			lastValidFrame;				    /* ...for trimming a few frames off the ends of movies...   */
								    /* ...w/o having to save the whole movie again. */
  PlxMovieOptions	options;				    /* Describes special options for this movie */
  char			filler[FILLER];				    /* Pad out this record to 128 bytes			*/
} jpheader;

#ifdef __cplusplus
}
#endif
#endif /* _movieH_ */
