/*
 * (c) Copyright 1991 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log: plxvideo.h,v $
 * Revision 1.13  1995/11/01  02:54:20  lovvik
 * Fixed the RTV support.
 *
 * Revision 1.12  1995/10/12  22:17:51  drapeau
 * Added initial, untested support for a PlxControl() protocol request.
 *
 * Revision 1.11  1995/10/11  02:22:53  drapeau
 * Added support for RTV integrated with the XPlx protocol extension.
 *
 * Revision 1.10  1993/12/15  18:50:29  schen
 * switch the hardware and the length field of the xVideoConfigReply.
 *
 * Revision 1.9  1991/10/21  07:59:41  dave
 * got rid of table request, not needed
 *
 * Revision 1.8  91/10/19  16:15:29  dave
 * fixed some stuff in table
 * 
 * Revision 1.7  91/10/19  15:58:24  dave
 * added a structure for load table
 * 
 * Revision 1.6  91/10/17  11:14:40  dave
 * changed some structure definitions for shared memory compression routines
 * 
 * Revision 1.5  91/09/17  10:25:55  dave
 * added string operations
 * 
 * Revision 1.4  91/08/19  12:40:45  eric
 * added rcs header
 * 
 *
 */

#ifdef H_ID
#ifndef lint
static char *rcsid_plxvideo_h = "@(#)$Id: plxvideo.h,v 1.13 1995/11/01 02:54:20 lovvik Exp $ Parallax Graphics Inc";
#endif
#endif

/*
 * Video request paramaters.
 */

#include "XPlxExt.h"	/* included here so anyone using this one gets it */

#define	X_Video			1
#define	X_PlxImage		1
#define	X_PlxRtvImage		1

#define	X_VideoSet		11
#define	X_VideoLive		12
#define	X_VideoStill		13
#define	X_VideoScale		14
#define	X_VideoStop		15
#define	X_VideoPriority		16
#define	X_VideoFrameMode	17
#define	X_VideoFrameStart	18
#define	X_VideoDitherControl	19
#define	X_VideoInput		20

#define	X_Plx_zoom_on		21
#define	X_Plx_zoom_off		22
#define	X_Plx_Debug		23

#define	X_VideoValueFactory	30
#define	X_VideoValueSet		31

#define X_VideoSqueezeLive	41
#define X_VideoSqueezeStill	42

#define X_VideoFlipLive		51
#define X_VideoFlipStill	52
#define X_VideoFlipScale	53
#define X_VideoFlipSqueezeLive	54
#define X_VideoFlipSqueezeStill	55

/* new functions defined in rev 4.0 */

#define X_VideoOutput		100
#define X_VideoScaleOutput	101
#define X_VideoTag		102
#define X_VideoInputSelect	103
#define X_VideoOutputSelect	104
#define X_VideoValueLoad	105
#define X_VideoValueQuery	106
#define X_VideoValueSave	107
#define X_VideoValueRecall	108
#define X_VideoDirection	109
#define X_VideoQueryConfig	110
#define X_VideoQueryVideo	111
#define X_Plx_Pan               112
#define X_Plx_ScaleCopyArea	113
#define X_Plx_Command		114
#define X_Plx_ImageLoad		115
#define X_PlxPutImage		116
#define X_PlxGetImage		117
#define X_PlxPutCImage		118
#define X_PlxGetCImage		119
#define X_PlxPutQTable		120
#define X_Plx_SPString		121
#define X_Plx_GPString		122
#define X_PlxShmPutCImage       123
#define X_PlxShmGetCImage       124
#define X_PlxControl		125
#define X_PlxRtvPutCImage       126
#define X_PlxRtvGetCImage       127
#define X_PlxSetRtvWindow	128
#define X_PlxUnsetRtvWindow	129

#define sz_xPlxImageReq		44
#define sz_xPlxGetImageReply	32

typedef struct _PlxImage {
    CARD8 reqType;
    CARD8 videoReqType;
    CARD16 length B16;
    Drawable drawable B32;
    GContext gc B32;
    INT16 src_x B16, src_y B16;
    CARD16 src_w B16, src_h B16;
    INT16 dst_x B16, dst_y B16;
    CARD16 dst_w B16, dst_h B16;
    CARD32 planeMask B32;
    CARD8 format;
    CARD8 leftPad;
    CARD8 depth;
    CARD8 sendEvent;
    CARD32 shmseg B32;
    CARD32 offset B32;
} xPlxImageReq;    

typedef struct {
    BYTE type;  /* X_Reply */
    CARD8 depth;
    CARD16 sequenceNumber B16;
    CARD32 length B32;
    VisualID visual B32;
    CARD32 size B32;
    CARD32 pad4 B32;
    CARD32 pad5 B32;
    CARD32 pad6 B32;
    CARD32 pad7 B32;
    } xPlxGetImageReply;

typedef struct _Video {
	CARD8 reqType;			/* always VideoReqCode */
	CARD8 videoReqType;		/* one of X_Video_XXXXX */
	CARD16 length B16;
	CARD32 drawable B32;
	CARD32 gc B32;
} xVideoReq;				/* followed by any request paramaters */
#define sz_xVideoReq 12

typedef struct _VideoArgs {
	INT16 vx B16, vy B16, vw B16, vh B16;	/* input video area */
	INT16 x B16, y B16, w B16, h B16;	/* windowing cord's */
} xVideoArgs;
#define sz_xVideoArgs 16

typedef struct _CVideoArgs {
	INT16 sx B16, sy B16, sw B16, sh B16;	/* source's geom */
	INT16 dx B16, dy B16, dw B16, dh B16;	/* dest's geom */
	CARD32 drawable B32;			/* sources drawable */
} xCVideoArgs;
#define sz_xCVideoArgs 20

#define	xva_arg1	vx
#define	xva_arg2	vy
#define x_pan_x		vx
#define x_pan_y		vy


typedef struct _VideoValueArgs {
	INT16 user0 B16, user1 B16;
	INT16 saturation B16;
	INT16 contrast B16;
	INT16 lgain B16;
	INT16 cgain B16;
	INT16 hue B16;
	INT16 brightness B16;
} xVideoValueArgs;
#define sz_xVideoValueArgs 16

#define	VIDEONAME	"ParallaxVideo"
#define	VIDEOPROCEDURES	"ParallaxVideoProcedures"

/* new structures added for rev. 4.0 */

typedef struct _PlxVideoSignal {
	unsigned channel;	/* input or output channel no. */
	unsigned type;		/* input or output 0 = input */
	unsigned standard;
	unsigned w;
	unsigned h;
	unsigned b;
	unsigned r;
	unsigned interlaced;
	unsigned sync_ok;
} PlxVideoSignal;

typedef struct _PlxVideoFormat {
	unsigned channel;	/* input or output channel no. */
	unsigned type;		/* input or output 0 = input */
	unsigned format;
} PlxVideoFormat;
	
typedef struct _PlxVideoType {
	unsigned channel;	/* input or output channel no. */
	unsigned type;		/* input or output 0 = input */
	unsigned format;
} PlxVideoType;
	
typedef struct _VideoConfigReply {
	CARD8 type;			/* always X_reply */
	CARD8 pad0;		/* filler */
	CARD16 sequenceNumber B16;  /* of last request received by server */
	CARD32 length B32;	/* of additional bytes */
	CARD32 hardware B32;	/* type of hardware */
	CARD32 nformats B32;	/* the number of formats returned */
	CARD32 ntypes B32; 	/* the number of types returned */
	CARD32 nsignals B32;	/* the number of signals returned */
	CARD32 pad1 B32;
	CARD32 pad2 B32;
} xVideoConfigReply;
#define sz_xVideoConfigReply 32

typedef struct _VidValueReply {
	CARD8 type;		/* always X_reply */
	CARD8 pad0;		/* filler */
	CARD16 sequenceNumber B16;  /* of last request received by server */
	CARD32 length B32; 	/* of additional bytes */
	CARD32 sat B32; 	/* saturation */
	CARD32 cont B32; 	/* contrast */
	CARD32 hue B32;		/* hue */
	CARD32 brit B32;	/* brightness */
	CARD32 pad1 B32;	/* brightness */
	CARD32 pad2 B32;	/* brightness */
} xVidValueReply;
#define  sz_xVidValueReply 32


typedef struct _VidValueArgs {
	INT16 select B16;
	INT16 sat B16;
	INT16 cont B16;
	INT16 hue B16;
	INT16 brit B16;
	INT16 pad0 B16;
	INT32 pad1 B32;
} xVidValueArgs;
#define sz_xVidValueArgs 16

typedef struct _VideoSelectArgs {
	INT16 channel B16;
	INT16 standard B16;
	INT16 format B16;
	INT16 type B16;
} xVideoSelectArgs;
#define sz_xVideoSelectArgs 8;


/*typedef struct _PlxRtvImage {
    CARD8 reqType;
    CARD8 videoReqType;
    CARD16 length B16;
    Drawable drawable B32;
    GContext gc B32;
    CARD32 pad0 B32;
    CARD32 pad1 B32;
    CARD32 pad2 B32;
    CARD32 pad3 B32;
    CARD32 pad4 B32;
} xPlxRtvImageReq;    
#define sz_xPlxRtvImageReq	32
#define sz_xPlxRtvImageReply	4
*/

typedef struct
{
  BYTE type;							    /* X_Reply */
  CARD8 depth;
  CARD16 sequenceNumber B16;
  CARD32 length B32;
  VisualID visual B32;
  CARD32 size B32;
  CARD32 pad4 B32;
  CARD32 pad5 B32;
  CARD32 pad6 B32;
  CARD32 pad7 B32;
} xPlxRtvGetImageReply;

#define sz_xPlxRtvGetImageReply	32

typedef struct _PlxControl
{
  CARD8		reqType;
  CARD8		videoReqType;
  CARD16	length B16;
  Drawable	drawable B32;
  CARD32	size B32;
  CARD32	pad0;
  CARD32	pad1;
  CARD32	pad2;
  CARD32	pad3;
  CARD32	pad4;
} xPlxControlReq;
#define sz_xPlxControlReq	32

typedef struct _PlxControlReply
{
  BYTE type;							    /* X_Reply */
  CARD8 pad0;							    /* Filler here */
  CARD16 sequenceNumber B16;
  CARD32 length B32;						    /* Number of data words in the reply */
  CARD32 size B32;						    /* Size in bytes of the usable data for the caller */
  CARD32 pad1 B32;
  CARD32 pad2 B32;
  CARD32 pad3 B32;
  CARD32 pad4 B32;
  CARD32 pad5 B32;
} xControlReply;

#define sz_xPlxControlReply	32

typedef struct _PlxRtvImage {
    CARD8 reqType;
    CARD8 videoReqType;
    CARD16 length B16;
    Drawable drawable B32;
    GContext gc B32;
    INT16 src_x B16, src_y B16;
    CARD16 src_w B16, src_h B16;
    INT16 dst_x B16, dst_y B16;
    CARD16 dst_w B16, dst_h B16;
    CARD32 planeMask B32;
    CARD8 format;
    CARD8 leftPad;
    CARD8 depth;
    CARD8 sendEvent;
    CARD32 shmseg B32;
    CARD32 offset B32;
} xPlxRtvImageReq;    

#define sz_xPlxRtvImageReq		44

typedef struct _PlxRtvImageReply {
	CARD8 type;		/* always X_reply */
	CARD8 pad0;		/* filler */
	CARD16 sequenceNumber B16;  /* of last request received by server */
	CARD32 length B32; 	/* of additional bytes */
	CARD32 reply B32; 	/* Reply */
	CARD32 pad1 B32; 	
	CARD32 pad2 B32;	
	CARD32 pad3 B32;	
	CARD32 pad4 B32;	
	CARD32 pad5 B32;	
} xPlxRtvImageReply;
#define sz_xPlxRtvImageReply	32
