#include "plxfb.h"

# define useTEGlyphBlt		pfbImageGlyphBlt8
# define useImageGlyphBlt	pfbImageGlyphBlt8
# define usePolyGlyphBlt	pfbPolyGlyphBlt8
# define usePushPixels		pfbPushPixels8
# define ZeroPolyArc		pfbZeroPolyArcSS8Copy

void pfbValidateGC();
void pfbChangeGC();
void pfbCopyGC();
void pfbDestroyGC();

#ifndef X11R6
extern void cfbChangeGC(), cfbCopyGC(), cfbDestroyGC(), cfbChangeClip();
extern void cfbDestroyClip(), cfbCopyClip(), cfb8FillRectStippledUnnatural();
#endif

static GCFuncs pfbGCFuncs = {
    pfbValidateGC,
    pfbChangeGC,
    pfbCopyGC,
    pfbDestroyGC,
#ifdef X11R6
    miChangeClip,
    miDestroyClip,
    miCopyClip,
#else
    cfbChangeClip,
    cfbDestroyClip,
    cfbCopyClip,
#endif
};

static GCOps	pfbOps = {
    pfbFillSpansSolid,		/* FillSpans */
    pfbSetSpans,		/* SetSpans */
    pfbPutImage,		/* PutImage */
    pfbCopyArea,		/* CopyArea */
    pfbCopyPlane,		/* CopyPlane */
    pfbPolyPoint,		/* PolyPoint */
    pfbLineSS,			/* Polylines */
    pfbSegmentSS,		/* PolySegment */
    pfbPolyRectangle,		/* PolyRectangle */
    pfbZeroPolyArc,		/* PolyArc */
    pfbFillPolygon,		/* FillPolygon */
    pfbPolyFillRect,		/* PolyFillRect */
    pfbPolyFillArc,		/* PolyFillArc */
    pfbPolyText8,		/* PolyText8  made it to here*/
    pfbPolyText16,		/* PolyText16 */
    pfbImageText8,		/* ImageText8 */
    pfbImageText16,		/* ImageText16 */
    pfbImageGlyphBlt8,		/* ImageGlyphBlt */
    pfbPolyGlyphBlt8,		/* PolyGlyphBlt */
    pfbPushPixels8,		/* PushPixels */
#ifdef NEED_LINEHELPER
    ,NULL
#endif
};

#ifndef X11R6

GCOps *miCreateGCOps(ops)
GCOps *ops;
{
    GCOps *ret;
    if((ret = (GCOps *)xalloc(sizeof(GCOps))) == NULL)
	return NULL;
    *ret = *ops;
    ret->devPrivate.val = 1;
}

void miDestroyGCOps(ops)
GCOps *ops;
{
    if(ops->devPrivate.val)
    {
	ops->devPrivate.val = 0;
	xfree(ops);
    }
}

#endif
Bool
pfbCreateGC(pGC)
    register GCPtr pGC;
{
    pfbPrivGC  *pPriv;
    Bool ret;
    int i;
    unsigned int *t;

    pPriv = (pfbPrivGC *)(pGC->devPrivates[pfbGCPrivateIndex].ptr);

    /*
     * Initialization for our private.
     */

    pPriv->tag = PLX_UNKNOWN;
    pPriv->channel = PLX_INPUT_0;
    pPriv->standard = PLX_NTSC;
    pPriv->format = PLX_COMP;
    pPriv->type = PLX_RGB24;
    pPriv->xdirection = 0;
    pPriv->ydirection = 0;
    pPriv->video_tag = 0;
    pPriv->qtab = NULL;
    pPriv->htab = NULL;
    pPriv->savedAlu = pGC->alu;
    pPriv->savedMask = pGC->planemask;

    switch (pGC->depth) {
    case 1:
    case 8:
	ret = cfbCreateGC(pGC);
	break;
    case 24:
    case 32:
	ret = cfb32CreateGC(pGC);
	break;
    default:
	ErrorF("pfbCreateGC: unsupported depth: %d\n", pGC->depth);
	return FALSE;
    }

    pfbNewMP(pGC);

    pPriv->pixOps = pGC->ops;	/* save c/m fb ops */
    pPriv->winOps = &pfbOps;
    pGC->funcs = &pfbGCFuncs;
    pGC->ops = pPriv->winOps;
    pGC->serialNumber |= 0x80000000;	/* force a validate */

    return ret;
}

#if 0
GCOps *
pfbMatchCommon (pGC, dP)
    GCPtr	    pGC;
    void *    dP;
{
    cfbPrivGCPtr    devPriv = (cfbPrivGCPtr) dP;
    if (pGC->lineWidth != 0)
	return 0;
    if (pGC->lineStyle != LineSolid)
	return 0;
    if (pGC->fillStyle != FillSolid)
	return 0;
    if (devPriv->rop != GXcopy)
	return 0;

    if (pGC->font &&
	FONTMAXBOUNDS(pGC->font,rightSideBearing) -
        FONTMINBOUNDS(pGC->font,leftSideBearing) <= 32 &&
	FONTMINBOUNDS(pGC->font,characterWidth) >= 0)
    {
	if (TERMINALFONT(pGC->font)
#ifdef FOUR_BIT_CODE
	    && FONTMAXBOUNDS(pGC->font,characterWidth) >= PGSZB
#endif
	)
#if 1
                return &pfbOps;
#else
#ifdef NO_ONE_RECT
                return &pfbTEOps1Rect;
#else
	    if (devPriv->oneRect)
		return &pfbTEOps1Rect;
	    else
		return &pfbTEOps;
#endif
	else
#ifdef NO_ONE_RECT
	    return &pfbNonTEOps1Rect;
#else
	    if (devPriv->oneRect)
		return &pfbNonTEOps1Rect;
	    else
		return &pfbNonTEOps;
#endif
#endif
    }
    return 0;
}
#endif

void
pfbValidateGC(pGC, changes, pDrawable)
    register GCPtr  pGC;
    unsigned long   changes;
    DrawablePtr	    pDrawable;
{
    int         index;		/* used for stepping through bitfields */
    int		new_rrop;
    int         new_line, new_text, new_fillspans, new_fillarea;
    int		new_rotate;
    int		xrot, yrot;
    cfbPrivGCPtr devPriv;
    pfbPrivGC   *pPriv;
    int		oneRect;
    unsigned int mask;

    pPriv = (pfbPrivGC *)(pGC->devPrivates[pfbGCPrivateIndex].ptr);
    devPriv = cfbGetGCPrivate(pGC);

    pGC->ops = pPriv->pixOps;	/* make sure pointing at pixmap ops */

    /* if a window, set alu to copy, and mask to -1 */
    /* save 'm first */

    if(pDrawable->type == DRAWABLE_WINDOW)
    {
	pPriv->savedAlu = pGC->alu;
	pPriv->savedMask = pGC->planemask;
	pGC->alu = GXcopy;
	pGC->planemask = ~0L;
    }

    switch (pGC->depth)
    {
	case 1:
	    mfbValidateGC(pGC, changes, pDrawable);
	    break;
	case 8:
	    cfbValidateGC(pGC, changes, pDrawable);
	    break;
	default:
	    cfb32ValidateGC(pGC, changes, pDrawable);
	    break;
    }

    pPriv->pixOps = pGC->ops;		/* save pixmap ops */

    if(pPriv->winOps == NULL)		/* what the hell ? */
	pPriv->winOps = &pfbOps;

    pGC->ops = pPriv->winOps;		/* point at our ops */

    if(pDrawable->type != DRAWABLE_WINDOW) /* if pixmap, we're done */
	return;

    /* restore alu, and mask */

    pGC->alu = pPriv->savedAlu;
    pGC->planemask = pPriv->savedMask;

    /* store new mask, pixel vals */

    pfbNewMP(pGC);

    new_rotate = pGC->lastWinOrg.x != pDrawable->x ||
		 pGC->lastWinOrg.y != pDrawable->y;

    pGC->lastWinOrg.x = pDrawable->x;
    pGC->lastWinOrg.y = pDrawable->y;

#if 0
    new_rrop = FALSE;
    new_line = FALSE;
    new_text = FALSE;
    new_fillspans = FALSE;
    new_fillarea = FALSE;
#else
    new_rrop = TRUE;
    new_line = TRUE;
    new_text = TRUE;
    new_fillspans = TRUE;
    new_fillarea = TRUE;
#endif

    /*
     * if the client clip is different or moved OR the subwindowMode has
     * changed OR the window's clip has changed since the last validation
     * we need to recompute the composite clip 
     */

    if ((changes & (GCClipXOrigin|GCClipYOrigin|GCClipMask|GCSubwindowMode)) ||
	(pDrawable->serialNumber != (pGC->serialNumber & DRAWABLE_SERIAL_BITS))
	)
    {
	miComputeCompositeClip (pGC, pDrawable);
#ifdef NO_ONE_RECT
#if 0
	devPriv->oneRect = FALSE;
#endif
#else
	oneRect = REGION_NUM_RECTS(devPriv->pCompositeClip) == 1;
	if (oneRect != devPriv->oneRect)
	    new_line = TRUE;
#if 0
	devPriv->oneRect = oneRect;
#endif
#endif
    }

#if 0
    mask = changes;
    while (mask) {
	index = lowbit (mask);
	mask &= ~index;

	/*
	 * this switch acculmulates a list of which procedures might have
	 * to change due to changes in the GC.  in some cases (e.g.
	 * changing one 16 bit tile for another) we might not really need
	 * a change, but the code is being paranoid. this sort of batching
	 * wins if, for example, the alu and the font have been changed,
	 * or any other pair of items that both change the same thing. 
	 */
	switch (index) {
	case GCFunction:
	case GCForeground:
	    new_rrop = TRUE;
	    break;
	case GCPlaneMask:
	    new_rrop = TRUE;
	    new_text = TRUE;
	    break;
	case GCBackground:
	    break;
	case GCLineStyle:
	case GCLineWidth:
	    new_line = TRUE;
	    break;
	case GCJoinStyle:
	case GCCapStyle:
	    break;
	case GCFillStyle:
	    new_text = TRUE;
	    new_fillspans = TRUE;
	    new_line = TRUE;
	    new_fillarea = TRUE;
	    break;
	case GCFillRule:
	    break;
	case GCTile:
	    new_fillspans = TRUE;
	    new_fillarea = TRUE;
	    break;

	case GCStipple:
	    new_fillspans = TRUE;
	    new_fillarea = TRUE;
	    break;

	case GCTileStipXOrigin:
	    new_rotate = TRUE;
	    break;

	case GCTileStipYOrigin:
	    new_rotate = TRUE;
	    break;

	case GCFont:
	    new_text = TRUE;
	    break;
	case GCSubwindowMode:
	    break;
	case GCGraphicsExposures:
	    break;
	case GCClipXOrigin:
	    break;
	case GCClipYOrigin:
	    break;
	case GCClipMask:
	    break;
	case GCDashOffset:
	    break;
	case GCDashList:
	    break;
	case GCArcMode:
	    break;
	default:
	    break;
	}
    }

    /*
     * If the drawable has changed,  ensure suitable
     * entries are in the proc vector. 
     */
    if (pDrawable->serialNumber != (pGC->serialNumber & (DRAWABLE_SERIAL_BITS))) {
	new_fillspans = TRUE;	/* deal with FillSpans later */
    }

    if (new_rotate || new_fillspans)
    {
	Bool new_pix = FALSE;

	xrot = pGC->patOrg.x + pDrawable->x;
	yrot = pGC->patOrg.y + pDrawable->y;

	switch (pGC->fillStyle)
	{
	case FillTiled:
	    break;
#ifdef FOUR_BIT_CODE
	case FillStippled:
	case FillOpaqueStippled:
	    break;
#endif
	}
    }

    if (new_rrop)
    {
	{
#ifdef PIXEL_ADDR
	    new_line = TRUE;
#endif
#ifdef WriteBitGroup
	    new_text = TRUE;
#endif
	    new_fillspans = TRUE;
	    new_fillarea = TRUE;
	}
    }

#endif /* 0 */

    if (new_rrop || new_fillspans || new_text || new_fillarea || new_line)
    {
	GCOps	*newops;

#if 0
	if (newops = pfbMatchCommon (pGC, devPriv))
 	{
	    if (pGC->ops->devPrivate.val)
		miDestroyGCOps (pGC->ops);
	    pGC->ops = newops;
	    new_rrop = new_line = new_fillspans = new_text = new_fillarea = 0;
	}
 	else
 	{
	    if (!pGC->ops->devPrivate.val)
	    {
		pGC->ops = miCreateGCOps (pGC->ops);
		pPriv->winOps = pGC->ops;
/*		pGC->ops->devPrivate.val = 1;	/* redundant */
	    }
	}
#else
	if (pGC->ops->devPrivate.val == 0)
	{
	    pGC->ops = miCreateGCOps(pGC->ops);
	    pPriv->winOps = pGC->ops;
	    pGC->ops->devPrivate.val = 1;	/* redundant but safe */
	}
#endif
    }

    /* deal with the changes we've collected */
    if (new_line)
    {
	pGC->ops->FillPolygon = pfbFillPolygon;
	if (pGC->lineWidth == 0)
	{
	    if ((pGC->lineStyle == LineSolid) && (pGC->fillStyle == FillSolid))
	    {
		    pGC->ops->PolyArc = pfbZeroPolyArc;
	    }
	    else
		pGC->ops->PolyArc = pfbPolyFillArc;
	}
	else
	    pGC->ops->PolyArc = pfbPolyFillArc;

	pGC->ops->PolySegment = pfbmiPolySegment;
	switch (pGC->lineStyle)
	{
	case LineSolid:
	    if(pGC->lineWidth == 0)
	    {
		if (pGC->fillStyle == FillSolid)
		{
		    	pGC->ops->Polylines = pfbLineSS;
		    	pGC->ops->PolySegment = pfbSegmentSS;
	    	}
 		else
		    pGC->ops->Polylines = pfbmiZeroLine;
	    }
	    else
		pGC->ops->Polylines = pfbmiWideLine;
	    break;
	case LineOnOffDash:
	case LineDoubleDash:
	    if (pGC->lineWidth == 0 && pGC->fillStyle == FillSolid)
	    {
		pGC->ops->Polylines = pfbLineSD;
		pGC->ops->PolySegment = pfbSegmentSD;
	    } else
		pGC->ops->Polylines = pfbmiWideDash;
	    break;
	}
    }


    if (new_text && (pGC->font))
    {
        if (FONTMAXBOUNDS(pGC->font,rightSideBearing) -
            FONTMINBOUNDS(pGC->font,leftSideBearing) > 32 ||
	    FONTMINBOUNDS(pGC->font,characterWidth) < 0)
        {
            pGC->ops->PolyGlyphBlt = pfbmiPolyGlyphBlt;
            pGC->ops->ImageGlyphBlt = pfbmiImageGlyphBlt;
        }
        else
        {
	    if (pGC->fillStyle == FillSolid)
	    {
		    pGC->ops->PolyGlyphBlt = pfbPolyGlyphBlt8;
	    }
	    else
		pGC->ops->PolyGlyphBlt = pfbmiPolyGlyphBlt;
#if 0
            /* special case ImageGlyphBlt for terminal emulator fonts */

	    if (TERMINALFONT(pGC->font))
	    {
		    pGC->ops->ImageGlyphBlt = pfbImageGlyphBlt8;
	    }
            else
#endif
	    {
		if ( (pGC->fillStyle == FillSolid))
		    pGC->ops->ImageGlyphBlt = pfbImageGlyphBlt8;
		else
		    pGC->ops->ImageGlyphBlt = pfbmiImageGlyphBlt;
	    }
        }
    }    


    if (new_fillspans) {
	switch (pGC->fillStyle) {
	case FillSolid:
	    pGC->ops->FillSpans = pfbFillSpansSolid;
	    break;
	case FillTiled:
	    if (devPriv->pRotatedPixmap)
		pGC->ops->FillSpans = pfbFillSpansTile;
	    else
		pGC->ops->FillSpans = pfbUnnaturalTileFS;
	    break;
	case FillStippled:
	    pGC->ops->FillSpans = pfbUnnaturalStippleFS;
	    break;
	case FillOpaqueStippled:
	    pGC->ops->FillSpans = pfbUnnaturalStippleFS;
	    break;
	default:
	    FatalError("pfbValidateGC: illegal fillStyle\n");
	}
    } /* end of new_fillspans */

    if (new_fillarea) {
	pGC->ops->PolyFillRect = pfbPolyFillRect;
	pGC->ops->PushPixels = pfbmiPushPixels;
	if (pGC->fillStyle == FillSolid && devPriv->rop == GXcopy)
	    pGC->ops->PushPixels = pfbPushPixels8;

	pGC->ops->PolyFillArc = pfbmiPolyFillArc;
	if (pGC->fillStyle == FillSolid)
	{
	    pGC->ops->PolyFillArc = pfbPolyFillArc;
	}
    }
}

void pfbChangeGC(pGC, mask)
GCPtr pGC;
unsigned long mask;
{
    pfbNewMP(pGC);
}

void pfbCopyGC(pGC, changes, pGCDst)
GCPtr pGC;
unsigned long changes;
GCPtr pGCDst;
{
    pfbPrivGCPtr pPriv = pfbGetGCPriv(pGC);
    XV_struct *xv = pfbGetScreenPriv(pGC->pScreen)->xv;
    if(pPriv->video_tag)
	O_Lut_video_tag(xv->p_lut, pPriv->video_tag); /* bump refcount */
}

void pfbDestroyGC(pGC)
GCPtr pGC;
{
#ifndef X11R6
    cfbPrivGC *pPriv;
#endif
    pfbPrivGC *pP;
    XV_struct *xv = pfbGetScreenPriv(pGC->pScreen)->xv;
    pP = pfbGetGCPriv(pGC);
    pGC->ops = pP->pixOps;
    if(pP->winOps->devPrivate.val)
	miDestroyGCOps(pP->winOps);

    if(pP->video_tag)
	O_Lut_free_video_tag(xv->p_lut, pP->video_tag);
	
#ifdef X11R6
    miDestroyGC(pGC);
#else

#if 1
    switch(pGC->depth)
    {
	case 1:
	    mfbDestroyGC(pGC);
	    break;
	case 8:
	    cfbDestroyGC(pGC);
	    break;
	default:
	    cfb32DestroyGC(pGC);
	    break;
    }
#else
    pPriv = (cfbPrivGC *)(pGC->devPrivates[cfbGCPrivateIndex].ptr);
    if (pPriv->pRotatedPixmap)
	pfbDestroyPixmap(pPriv->pRotatedPixmap);
    if (pPriv->freeCompClip)
	(*pGC->pScreen->RegionDestroy)(pPriv->pCompositeClip);
    cfbDestroyOps (pGC->ops);
#endif
#endif
}

