/*
 *   Copyright (c) 1987, 88 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 */

/************************************************************
Copyright 1987 by Sun Microsystems, Inc. Mountain View, CA.

                    All Rights Reserved

Permission  to  use,  copy,  modify,  and  distribute   this
software  and  its documentation for any purpose and without
fee is hereby granted, provided that the above copyright no-
tice  appear  in all copies and that both that copyright no-
tice and this permission notice appear in  supporting  docu-
mentation,  and  that the names of Sun or MIT not be used in
advertising or publicity pertaining to distribution  of  the
software  without specific prior written permission. Sun and
M.I.T. make no representations about the suitability of this
software for any purpose. It is provided "as is" without any
express or implied warranty.

SUN DISCLAIMS ALL WARRANTIES WITH REGARD TO  THIS  SOFTWARE,
INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FIT-
NESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL SUN BE  LI-
ABLE  FOR  ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,  DATA  OR
PROFITS,  WHETHER  IN  AN  ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION  WITH
THE USE OR PERFORMANCE OF THIS SOFTWARE.

********************************************************/
static char *rcsid = "@(#)$Id: plxscrinit.c,v 1.4 92/09/04 16:52:37 ruby Exp $";

#include "X.h"
#include "Xmd.h"
#include "servermd.h"
# include   "Xproto.h"
# include   "misc.h"
#include "regionstr.h"
#include "scrnintstr.h"
#include "gcstruct.h"
# include   "windowstr.h"
# include   "pixmapstr.h"
#include "mi.h"
#include "mipointrst.h"
#include "pixmapstr.h"
#include "resource.h"
#include "colormap.h"
#include "colormapst.h"
#include "mi.h"
#include "mistruct.h"
#include "dix.h"
#include "cfb.h"
#include "mibstore.h"
#include "plx.h"
#include "objects_internal.h"

extern Bool mfbCreateWindow();
extern Bool mfbPositionWindow();
extern Bool mfbChangeWindowAttributes();
extern Bool mfbMapWindow();
extern Bool mfbUnmapWindow();
extern Bool mfbDestroyWindow();

extern Bool mfbCreateGC();
extern PixmapPtr mfbCreatePixmap();
extern Bool mfbDestroyPixmap();
extern void mfbCopyWindow();
extern void mfbPaintWindow();
extern void mfbGetImage();

extern int mfbListInstalledColormaps();
extern void mfbInstallColormap();
extern void mfbUninstallColormap();
extern Bool mfbCreateColormap();
extern void mfbDestroyColormap();
extern void mfbResolveColor();

extern RegionPtr mfbPixmapToRegion();
extern Bool mfbAllocatePrivates();

Bool	(* pfbSavedCreateGC)();


extern RegionPtr pfbCopyPlane();

#ifdef X11R6
extern cfbCreateScreenResources();
#else
extern RegionPtr    (*cfbPuntCopyPlane)();
#endif


extern void plxSaveAreas();
extern void plxRestoreAreas();

static miBSFuncRec pfbBSFuncRec =
{ NULL, NULL, NULL, NULL, NULL };
extern miBSFuncRec cfbBSFuncRec;

extern Bool pfbPositionWindow();
extern Bool pfbChangeWindowAttributes();
extern Bool pfbMapWindow();
extern Bool pfbUnmapWindow();
extern void pfbGetImage();
extern void pfbGetSpans();
extern Bool pfbCreateGC();
extern PixmapPtr pfbCreatePixmap();
extern Bool pfbDestroyPixmap();
extern void pfbResolveColor();
extern void pfbPaintWindow();
extern void pfbCopyWindow();

extern int defaultColorVisualClass;

#define _BP 8
#define _RZ ((PSZ + 2) / 3)
#define _RS 0
#define _RM ((1 << _RZ) - 1)
#define _GZ ((PSZ - _RZ + 1) / 2)
#define _GS _RZ
#define _GM (((1 << _GZ) - 1) << _GS)
#define _BZ (PSZ - _RZ - _GZ)
#define _BS (_RZ + _GZ)
#define _BM (((1 << _BZ) - 1) << _BS)
#define _CE (1 << _RZ)

#define ONEBITVISUAL

static VisualRec visuals[] = {
/*    vid  class      bpRGB cmpE nplan rMask gMask  bMask oRed oGreen oBlue */
#ifdef ONEBITVISUAL
    { 0,   StaticGray,  1,     2,  1,    0,    0,     0,    0,   0,     0},
#endif
    { 0,   PseudoColor, 8,  1<<8,  8,    0,    0,     0,    0,   0,     0},
    { 0,   TrueColor,   8,  1<<8,  24,  255, 255<<8, 255<<16, 0,  8,    16},
    { 0,   DirectColor, 8,  1<<8,  24,  255, 255<<8, 255<<16, 0,  8,    16},
};


#define	NUMVISUALS	((sizeof visuals)/(sizeof visuals[0]))

#ifdef ONEBITVISUAL
#define VISUALINDEX_1BIT    0
#define VISUALINDEX_8BIT    1
#define VISUALINDEX_24BIT   2
#else
#define VISUALINDEX_8BIT    0
#define VISUALINDEX_24BIT   1
#endif

#if 0
#define VISUALINDEX_32BIT   8
#define VISUALINDEX_24BIT   1
#define VISUALINDEX_32BIT   2
#endif

static  VisualID VIDs[NUMVISUALS];

static DepthRec depths[] = {
/* depth	numVid		vids */
#ifdef ONEBITVISUAL
    1,		1,		VIDs,
#endif
    8,		1,		&VIDs[VISUALINDEX_8BIT],
   24,		2,		&VIDs[VISUALINDEX_24BIT],
};

#define NUMDEPTHS	((sizeof depths)/(sizeof depths[0]))

static PixmapFormatRec  formats[] = {
    1, 1, BITMAP_SCANLINE_PAD,  /* 1-bit deep */
    8, 8, BITMAP_SCANLINE_PAD,  /* 8-bit deep */
    24, 32, BITMAP_SCANLINE_PAD,  /* 24-bit deep */
};

#define NUMFORMATS      (sizeof formats)/(sizeof formats[0])

#ifdef CFB_NEED_SCREEN_PRIVATE
extern int cfbScreenPrivateIndex;
#ifdef hpux
extern int  cfbDualHeadBug;
extern int   cfbFirstIndex;
#else
#endif
#endif

static int   cfbFirstIndex = -1;
static int   cfb32FirstIndex = -1;
extern int cfb32ScreenPrivateIndex;

int pfbGCPrivateIndex;
int pfbScreenPrivateIndex;
int pfbWindowPrivateIndex;

static unsigned long pfbGeneration = 0;

#ifdef hpux
extern int plxhpscreen;
#endif

static Bool plxSaveScreen(pScreen, what)
ScreenPtr pScreen;
Bool what;
{
    XV_struct *xv = pfbGetScreenPriv(pScreen)->xv;
#ifdef hpux
   if(plxhpscreen != pScreen->myNum)
	return(TRUE);
#endif

#if 0
    plxv_disp_blank( xv, !what);
#else
    O_PVsun_visibility(xv, !what);
#endif
    return(TRUE);

}

static ColormapPtr InstalledMaps[MAXSCREENS];

static void plxStoreColors(pmap, ndef, pdefs)
ColormapPtr pmap;
int         ndef;
xColorItem  *pdefs;
{
    xColorItem  expanddefs[256];

#if 0
    XV_color xvcolor;
#else
    unsigned int color;
#endif

    int tag;

    XV_struct *xv = 
      ((plxPrivPtr)(pmap->pScreen->devPrivates[pfbScreenPrivateIndex].ptr))->xv;


    if(pmap != InstalledMaps[pmap->pScreen->myNum])
	return;


    tag = 0;
    if(pmap->pVisual->nplanes > 8)
    {
	if(pmap->pVisual->class == TrueColor)
	    return;

	if ((pmap->pVisual->class | DynamicClass) != DirectColor)
	    return;

	ndef = cfbExpandDirectColors(pmap, ndef, pdefs, expanddefs);
	pdefs = expanddefs;
	tag = 1;
    }

    while(ndef--)
    {
	unsigned int table;

	if(tag)
	{
	    table = O_Lut_Entry_Wid;
	    O_Lut_get_entries(xv->p_lut, table, pdefs->pixel, 1, &color);
	}
	else
	    table = O_Lut_Entry_Lut;

	if(!tag || (pdefs->flags & DoRed))
	{
	    color &= 0xffff00;
	    color |= (pdefs->red >> 8);
	}
	if(!tag || (pdefs->flags & DoGreen))
	{
	    color &= 0xff00ff;
	    color |= (pdefs->green & 0xff00);
	}
	if(!tag || (pdefs->flags & DoBlue))
	{
	    color &= 0xffff;
	    color |= ((pdefs->blue & 0xff00) << 8);
	}

	O_Lut_set_entries(xv->p_lut, table, pdefs->pixel, 1, &color);
	
	++pdefs;
    }
}

void plxInstallColormap(pmap)
ColormapPtr pmap;
{
    int index = pmap->pScreen->myNum;
    ColormapPtr oldpmap = InstalledMaps[index];

    if(pmap != oldpmap)
    {
	/* Uninstall pInstalledMap. No hardware changes required, just
	 * notify all interested parties. */
	if(oldpmap != (ColormapPtr)None)
	    WalkTree(pmap->pScreen, TellLostMap, (char *)&oldpmap->mid);
	/* Install pmap */
	InstalledMaps[index] = pmap;
	WalkTree(pmap->pScreen, TellGainedMap, (char *)&pmap->mid);

    }
}

static int
plxListInstalledColormaps(pScreen, pmaps)
    ScreenPtr   pScreen;
    Colormap    *pmaps;
{
    /* By the time we are processing requests, we can guarantee that there
     * is always a colormap installed */
    *pmaps = InstalledMaps[pScreen->myNum]->mid;
    return (1);
}

void
plxUninstallColormap(pmap)
    ColormapPtr	pmap;
{
    int index = pmap->pScreen->myNum;
    ColormapPtr curpmap = InstalledMaps[index];

    if(pmap == curpmap)
    {
	if (pmap->mid != pmap->pScreen->defColormap)
	{
	    curpmap = (ColormapPtr) LookupIDByType(pmap->pScreen->defColormap,
						   RT_COLORMAP);
	    (*pmap->pScreen->InstallColormap)(curpmap);
	}
    }
}


Bool plxCreateColormap(cmap)
ColormapPtr cmap;
{
    int i;
    xColorItem          localColor;
    cfbInitializeColormap(cmap);
    if(cmap->pVisual->class == PseudoColor)
    {
	for(i = 0; i < 256; i++) 
	{
	  localColor.pixel = i;
	  if(i == 1)
	  {
	    localColor.red = cmap->red[i].co.local.red = (short) -1;
	    localColor.green = cmap->red[i].co.local.green = (short) -1;
	    localColor.blue = cmap->red[i].co.local.blue = (short) -1;
	  }
	  else
	  {
	    localColor.red = cmap->red[i].co.local.red = (i & 0x7) << 13;
	    localColor.green = cmap->red[i].co.local.green = (i & 0x38) << 10;
	    localColor.blue = cmap->red[i].co.local.blue = (i & 0xc0) << 8;
	  }
	  plxStoreColors(cmap, 1, &localColor);
	}
    }
}

Bool
plxCreateDefColormap(pScreen)
    ScreenPtr pScreen;
{
    unsigned short zero = 0, ones = ~0;
    VisualPtr	pVisual;
    ColormapPtr	cmap;
    
    for (pVisual = pScreen->visuals;
	 pVisual->vid != pScreen->rootVisual;
	 pVisual++)
	;

    if (pVisual->nplanes == 1)
    {
	return (mfbCreateDefColormap(pScreen));
    }

    if (CreateColormap(pScreen->defColormap, pScreen, pVisual, &cmap,
		       (pVisual->class & DynamicClass) ? AllocNone : AllocAll,
		       0)
	!= Success)
	return FALSE;
    plxInstallColormap(cmap);
    if ((AllocColor(cmap, &zero, &zero, &zero, &(pScreen->blackPixel), 0) !=
       	   Success) ||
    	(AllocColor(cmap, &ones, &ones, &ones, &(pScreen->whitePixel), 0) !=
       	   Success))
    	return FALSE;
    return TRUE;
}


/*ARGSUSED*/
static Bool
plxCloseScreen (index, pScreen)
    int         index;
    ScreenPtr   pScreen;
{
    XV_struct *xv = 
      ((plxPrivPtr)(pScreen->devPrivates[pfbScreenPrivateIndex].ptr))->xv;
#if 0
    XV_color c;
#else
    unsigned int c;
#endif

#ifdef hpux
    plxSaveScreen(pScreen, FALSE);
#endif


#ifdef sun	/* not overlay FIX THIS */
    plxSaveScreen(pScreen, TRUE);
    O_Lut_set_entry(xv->p_lut, O_Lut_Entry_Lut, 0, 0xffffff);
    O_Lut_set_entry(xv->p_lut, O_Lut_Entry_Lut, 0xff, 0x0);
#endif

    plxv_tag(xv, PLXV_TAG_PIXEL_ATTR, 0);
    plxv_transp_off(xv);
    plxv_stencil(xv, 0, 0);
    plxv_rop(xv, PLXV_ROP_COPY);
    plxv_color_24(xv, 0x0);
    plxv_wmask(xv, PLXV_WMASK_ALL);

	/* clear the screen and overlays */
    plxv_box(xv, 0, 0, xv->main.linebytes, xv->main.ysize);

#if 0
    plxv_wmask(xv, PLXV_WMASK_PIXEL_ALL);
#endif
    plxv_wmask(xv, 0xffffff);
    plxv_ctl_1(xv, 0, 32);
    plxv_mode_8(xv);
    xfree (pScreen->devPrivates[pfbScreenPrivateIndex].ptr);
    (*((TV_PV *)xv->atv)->terminate)(xv);
    return TRUE;
}

/* dts * (inch/dot) * (25.4 mm / inch) = mm */

Bool
plxScreenInit(pScreen, dpix, dpiy, device, mode_flag)
    register ScreenPtr pScreen;
    int dpix, dpiy;
    char *device;
    int mode_flag;
{
    XV_struct *xv;
    register PixmapPtr pPixmap;
    register plxPrivPtr plxPriv;
    int	i, xsize, ysize, stride;
    pointer odp;
    int *m1base, *m8base, *m24base;
    O_Draw_Gc gc;

    caddr_t regbase, c3base;

    plxPriv = (plxPrivPtr) xalloc(sizeof(plxPrivRec));
    if (!plxPriv)
	return FALSE;

    plxPriv->pointer.pCursor = NULL;
    plxPriv->pointer.newCursor = NULL;
    plxPriv->pointer.blocked = FALSE;
    if(device == NULL)
	plxPriv->xv = O_HWL_O_HWL("full", O_HWL_device(), O_HWL_rsrc());
    else
	plxPriv->xv = O_HWL_O_HWL("full", device, O_HWL_rsrc());
    xv = plxPriv->xv;
    xsize = xv->main.xsize;
    ysize = xv->main.ysize;
    stride = xv->main.linebytes;

#if 0
    gc = pfb24bitgc;
    gc.color = 0xc0c0c0;
    gc.write_mask=0xffffffff;
    O_Draw_set_gc(xv->p_draw, &gc);
    O_Draw_fill_box(xv->p_draw, 0, 0, xsize, ysize);

#else
    plxv_init_draw(xv);
    plxv_28_now(xv, 0);

    if(mode_flag == 8)
    {
	plxv_mode_8(xv);
	plxv_tag(xv, PLXV_TAG_PIXEL_ATTR, 0);
    }
    else
    {
	plxv_mode_24(xv);
        plxv_tag(xv, PLXV_TAG_PIXEL_ATTR, O_Lut_true_tag(xv->p_lut));
    }

    plxv_transp_off(xv);
    plxv_stencil(xv, 0, 0);
    plxv_rop(xv, PLXV_ROP_COPY);
    plxv_color_24(xv, 0x0);
    plxv_wmask(xv, PLXV_WMASK_ALL);
    plxv_box(xv, 0, 0, stride, ysize);/* clear the screen and overlays */
    plxv_wmask(xv, PLXV_WMASK_PIXEL_ALL);
    plxv_colors_1(xv, 0, PLXV_WMASK_24, PLXV_WMASK_24);
    plxv_ctl_1(xv, 0, 32);
#endif

    plxVideoInit(plxPriv->xv);

    if (pfbGeneration != serverGeneration)
    {
	pfbScreenPrivateIndex = AllocateScreenPrivateIndex();
	pfbGCPrivateIndex = AllocateGCPrivateIndex();
	pfbWindowPrivateIndex = AllocateWindowPrivateIndex();
	for (i = 0; i < NUMVISUALS; i++) {
	    visuals[i].vid = FakeClientID(0);
	    VIDs[i] = visuals[i].vid;
	}
	pfbGeneration = serverGeneration;
    }
#ifdef CFB_NEED_SCREEN_PRIVATE
    else
    {
	cfbFirstIndex = cfbScreenPrivateIndex;
	cfb32FirstIndex = cfb32ScreenPrivateIndex;
    }
#endif

    if(!AllocateGCPrivate(pScreen, pfbGCPrivateIndex, sizeof(pfbPrivGC)))
	return FALSE;

#if 0
    cfbSetVisualTypes (8, 1<<PseudoColor, 8);	/* set pseudo for now */
    cfbScreenInit(pScreen, plxPriv->xv->m8base, xsize,
		  ysize, (xsize * 254) / (dpix * 10),
		  (ysize * 254) / (dpiy * 10), stride);
#else
    cfbSetupScreen(pScreen, plxPriv->xv->m8base, xsize, ysize,
		   (xsize * 254) / (dpix * 10),
		   (ysize * 254) / (dpiy * 10), stride);
#ifdef CFB_NEED_SCREEN_PRIVATE
    odp = pScreen->devPrivate;
#endif
    miScreenInit(pScreen, plxPriv->xv->m8base, xsize,
		  ysize, (xsize * 254) / (dpix * 10),
		  (ysize * 254) / (dpiy * 10), stride,
		  8, NUMDEPTHS, depths, visuals[VISUALINDEX_8BIT].vid,
		  NUMVISUALS, visuals, NULL);
#endif

    /*
     * This section of code is to correct a bug in the CFB 
     * implementation of dual head support.
     */

#ifdef CFB_NEED_SCREEN_PRIVATE
#ifdef hpux
barf
    if (cfbDualHeadBug != serverGeneration)
    {
	cfb32ScreenPrivateIndex = cfbFirstIndex = cfbScreenPrivateIndex;
	cfbDualHeadBug = serverGeneration;
    }
    else
    {
	pScreen->devPrivates[cfbFirstIndex].ptr = 
	    pScreen->devPrivates[cfbScreenPrivateIndex].ptr;
	cfbScreenPrivateIndex = cfb32ScreenPrivateIndex = cfbFirstIndex;
    }
#else
    if(cfbFirstIndex == -1)
	cfbFirstIndex = cfbScreenPrivateIndex;
    else if(cfbFirstIndex != cfbScreenPrivateIndex)
    {
	pScreen->devPrivates[cfbFirstIndex].ptr = 
	    pScreen->devPrivates[cfbScreenPrivateIndex].ptr;
	cfbScreenPrivateIndex = cfbFirstIndex;
    }
#endif
#endif

    /* override visuals, and depths */

    pScreen->numDepths = NUMDEPTHS;
    pScreen->allowedDepths = depths;
    pScreen->numVisuals = NUMVISUALS;
    pScreen->visuals = visuals;

    /* override the wrappers that we handle */
    /* anything that cfb doesn't know about is assumed to be done
       elsewhere.  (we put in no-op only for things that we KNOW
       are really no-op.
    */

    pScreen->CloseScreen = plxCloseScreen;
    pScreen->GetImage = pfbGetImage;
    pScreen->GetSpans = pfbGetSpans;
#if 0
    pScreen->ChangeWindowAttributes = pfbChangeWindowAttributes;
#endif

    pfbSavedCreateGC = pScreen->CreateGC;
    pScreen->CreateGC = pfbCreateGC;

#if 0
    pScreen->DestroyWindow = pfbDestroyWindow;
    pScreen->CreateWindow = pfbCreateWindow;
    pScreen->PositionWindow = pfbPositionWindow;
    pScreen->RealizeWindow = pfbMapWindow;
#endif

    pScreen->UnrealizeWindow = pfbUnmapWindow;
    pScreen->CreatePixmap = pfbCreatePixmap;
    pScreen->DestroyPixmap = pfbDestroyPixmap;

#ifdef	STATIC_COLOR
    pScreen->InstallColormap = pfbInstallColormap;
    pScreen->UninstallColormap = pfbUninstallColormap;
    pScreen->ListInstalledColormaps = pfbListInstalledColormaps;
    pScreen->StoreColors = NoopDDA;
#endif

    pScreen->ResolveColor = pfbResolveColor;
#if 1 /* find bug */
    pScreen->PaintWindowBackground = pfbPaintWindow;
    pScreen->PaintWindowBorder = pfbPaintWindow;
#endif
    pScreen->CopyWindow = pfbCopyWindow;

#if 0	/* leave 0 */
    pScreen->RealizeFont = mfbRealizeFont;
    pScreen->UnrealizeFont = mfbUnrealizeFont;
    pScreen->QueryBestSize = mfbQueryBestSize;
    pScreen->BitmapToRegion = mfbPixmapToRegion;
    pScreen->SourceValidate = (void (*)()) 0;
    pScreen->PostValidateTree = (void (*)()) 0;
#endif

    pScreen->InstallColormap = plxInstallColormap;
    pScreen->ListInstalledColormaps = plxListInstalledColormaps;
    pScreen->UninstallColormap = plxUninstallColormap;
    pScreen->StoreColors = plxStoreColors;
    pScreen->SaveScreen = plxSaveScreen;


#ifndef sun
    pScreen->CreateColormap = plxCreateColormap;
    pScreen->DestroyColormap = NoopDDA;
#endif

#if 1
    pfbBSFuncRec.SaveAreas = plxSaveAreas;
    pfbBSFuncRec.RestoreAreas = plxRestoreAreas;
    miInitializeBackingStore(pScreen, &pfbBSFuncRec);
#else
    miInitializeBackingStore(pScreen, &cfbBSFuncRec);
#endif

#ifdef X11R6
    pScreen->CreateScreenResources = cfbCreateScreenResources;
#endif

#ifdef CFB_NEED_SCREEN_PRIVATE    
    pScreen->devPrivates[cfbScreenPrivateIndex].ptr = pScreen->devPrivate;
    pScreen->devPrivate = odp;
#endif

    /* override misc other things */

    pScreen->backingStoreSupport = Always;
    pScreen->saveUnderSupport = NotUseful;

    pScreen->rootDepth = 8;

    pScreen->minInstalledCmaps = 1;
    pScreen->maxInstalledCmaps = 3;

#if 0
    /* let CreateDefColormap do whatever it wants */ 

    pScreen->blackPixel = pScreen->whitePixel = (Pixel) 0;
#endif

#ifndef hpux

 /* setup a 24 bit pixmap and poss a 1 bit pixmap */

#ifdef CFB_NEED_SCREEN_PRIVATE    

    mfbAllocatePrivates(pScreen, 0, 0);

    /* init 1 bit private */

    pPixmap = (PixmapPtr) xalloc(sizeof(PixmapRec));

    if (!pPixmap)
    {
	xfree(plxPriv);
	return FALSE;
    }
    pPixmap->drawable.type = DRAWABLE_PIXMAP;
    pPixmap->drawable.depth = 1;
    pPixmap->drawable.bitsPerPixel = 1;
    pPixmap->drawable.pScreen = pScreen;
    pPixmap->drawable.serialNumber = 0;
    pPixmap->drawable.x = 0;
    pPixmap->drawable.y = 0;
    pPixmap->drawable.width = xsize;
    pPixmap->drawable.height = ysize;
    pPixmap->refcnt = 1;
    pPixmap->devKind = stride >> 3;
    pPixmap->devPrivate.ptr = (pointer) xv->m1base;
    pScreen->devPrivate = (pointer)pPixmap;

#endif

    /* init 32 bit private */

    cfb32AllocatePrivates(pScreen, NULL, NULL);

    if(cfb32FirstIndex == -1)
	cfb32FirstIndex = cfb32ScreenPrivateIndex;
    else if(cfb32FirstIndex != cfb32ScreenPrivateIndex)
	cfb32ScreenPrivateIndex = cfb32FirstIndex;

    pPixmap = (PixmapPtr) xalloc(sizeof(PixmapRec));
    if (!pPixmap)
    {
	xfree(plxPriv);
	return FALSE;
    }
    pPixmap->drawable.type = DRAWABLE_PIXMAP;
    pPixmap->drawable.depth = 24;
    pPixmap->drawable.bitsPerPixel = 32;
    pPixmap->drawable.pScreen = pScreen;
    pPixmap->drawable.serialNumber = 0;
    pPixmap->drawable.x = 0;
    pPixmap->drawable.y = 0;
    pPixmap->drawable.width = xsize;
    pPixmap->drawable.height = ysize;
    pPixmap->refcnt = 1;
    pPixmap->devKind = stride << 2;
    pPixmap->devPrivate.ptr = (pointer) xv->m24base;
    pScreen->devPrivates[cfb32ScreenPrivateIndex].ptr = (pointer)pPixmap;

#else
fixhpuxport
#endif

    pScreen->devPrivates[pfbScreenPrivateIndex].ptr = (pointer) plxPriv;

    pScreen->defColormap = FakeClientID(0);
    if (defaultColorVisualClass < 0)
    {
	i = 1;
    }
    else
    {
	for (i = 0;
	     (i < NUMVISUALS) && (visuals[i].class != defaultColorVisualClass);
	     i++)
	    ;
	if (i >= NUMVISUALS)
	    i = 0;
    }

#ifdef ONEBITVISUAL
    if (mode_flag == 1)
    {
#if 0
	extern void plxGetImage();
	extern void plxGetSpans();
	extern void plxPaintWindow();

	pScreen->GetImage = plxGetImage;
	pScreen->GetSpans = plxGetSpans;
	pScreen->PaintWindowBackground = plxPaintWindow;
	pScreen->PaintWindowBorder = plxPaintWindow;
#endif
	pScreen->blackPixel = 0;
	pScreen->whitePixel = 1;
	pScreen->rootDepth = 1;
	pPixmap->drawable.depth = 1;
        i = VISUALINDEX_1BIT;
    }
    else if (mode_flag >= 24)
#else
    if (mode_flag >= 24)
#endif
    {
	pScreen->rootDepth = 24;
	pScreen->blackPixel = 0;
	pScreen->whitePixel = 0xffffff;
	pPixmap->drawable.depth = 24;
        i = VISUALINDEX_24BIT;
    }

    pScreen->rootVisual = visuals[i].vid;

#if 0
    mfbRegisterCopyPlaneProc (pScreen, pfbCopyPlane);
#endif


    PlxVideoExtensionInit();

#if 0
    XvplxScreenInit(pScreen);
#endif

    plxCreateDefColormap(pScreen);

#ifndef X11R6
    cfbPuntCopyPlane = miCopyPlane;
#endif

    return TRUE;
}


plxInit(index, pScreen, argc, argv)
long index;
ScreenPtr   pScreen;
long argc;
char *argv[];
{
    ColormapPtr cmap;
    VisualPtr   pVisual;
    int depth, i;
    char *device;
    static int devno = 0;
    int curdev;

    depth = 8;
    device = NULL;
    curdev = 0;

    for(i = 1; i < argc; ++i)
    {
	if(!strcmp(argv[i], "-defdepth"))
	{
	    if ((sscanf(argv[++i], "%d", &depth) < 1) ||
		((depth != 8) && (depth != 24)))
	    {
		fprintf(stderr, "-defdepth: bad depth\n");
		return FALSE;
	    }
	}
	else if((device == NULL) && !strcmp(argv[i], "-dev"))
	{
	    ++i;
	    ++curdev;
	    if(curdev > devno)
	    {
		device = argv[i];
		++devno;
	    }
	}
    }

    plxScreenInit(pScreen, 66, 66, device, depth);

    plxPointerInitialize (pScreen);

    return TRUE;
}


#ifdef sun

#include "sun.h"

extern sunFbDataRec sunFbData[];

Bool pfbTV2Probe(pScreenInfo, index, fbNum, argc, argv)
    ScreenInfo    *pScreenInfo; /* The screenInfo struct */
    int           index;        /* The index of pScreen in the ScreenInfo */
    int           fbNum;        /* Index into the sunFbData array */
    int           argc;         /* The number of the Server's arguments. */
    char          **argv;       /* The arguments themselves. Don't change! */
{
    int fd, i;
    char *name = sunFbData[fbNum].devName;

    if((fd = open(name, 2)) < 0)
	return FALSE;
    close(fd);
    return TRUE;
}

Bool pfbTV2Create(pScreenInfo, argc, argv)
    ScreenInfo	  *pScreenInfo;
    int	    	  argc;
    char    	  **argv;
{
    int i, fd;
    char *device;
    unsigned int no1, no8, no24;
    Bool retval;
    no1 = no8 = no24 = 1;

    /* add our formats nicely, find out what's there */
    for (i=0; i <  pScreenInfo->numPixmapFormats; ++i)
    {
	switch(pScreenInfo->formats[i].depth)
	{
	    case 1:
		no1 = 0;
		break;
	    case 8:
		no8 = 0;
		break;
	    case 24:
		no24 = 0;
		break;
	}
    }
	/* now only add the ones that aren't there */
    i = 0;
    if(no1)
	 pScreenInfo->formats[pScreenInfo->numPixmapFormats++] = formats[i];
    ++i;
    if(no8)
	 pScreenInfo->formats[pScreenInfo->numPixmapFormats++] = formats[i];
    ++i;
    if(no24)
	 pScreenInfo->formats[pScreenInfo->numPixmapFormats++] = formats[i];
    ++i;

    device = NULL;
    for(i = 1; i < argc; ++i)
    {
	if(!strcmp(argv[i], "-dev"))
	{
	    ++i;
	    if((fd = open(argv[i], O_RDWR)) >= 0)
	    {
		close(fd);
		device = argv[i];
		if (AddScreen(plxInit, argc, argv) >= 0)
		    retval = TRUE;
		else
		    retval = FALSE;
	    }
	}
    }

    if(device == NULL)
    {
	if((fd = open("/dev/tvtwo0", O_RDWR)) >= 0)
	{
	  close(fd);
	  if (AddScreen(plxInit, argc, argv) >= 0)
	    retval = TRUE;
	  else
	    retval = FALSE;
	}
	else
	    retval = FALSE;
    }

    return retval;
}

#endif
