/* $Header: /plx/source/Xclients/generic/RCS/callbacks.c,v 1.3 1995/12/12 18:57:49 lovvik Exp $ */

/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */

/* $Log: callbacks.c,v $
 * Revision 1.3  1995/12/12  18:57:49  lovvik
 * Fixed a problem with NewButtonPressed().  The problem was
 * that the Add dialog did not get cleared when File/New was
 * pressed.  This is a very minor change.
 *
 * Revision 1.2  1995/10/27  16:58:31  lovvik
 * Removed unused variables.
 *
 * Revision 1.1  1995/09/21  00:29:10  lovvik
 * Initial revision
 * */

static char movieCatRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/callbacks.c,v 1.3 1995/12/12 18:57:49 lovvik Exp $";

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>

#include <stdio.h>
#include <stdlib.h>

#include "interface.h"
#include "globals.h"
#include "structs.h"
#include "prototypes.h"

void XDmanage_link (Widget w, XtPointer client_data, XtPointer call_data)
{
  XtManageChild ( *(Widget *)client_data );
}

void XDunmanage_link (Widget w, XtPointer client_data, XtPointer call_data)
{
  XtUnmanageChild ( *(Widget *)client_data );
}

void XDpopup_link (Widget w, XtPointer client_data, XtPointer call_data)
{
  XtPopup ( *(Widget *)client_data, XtGrabNone );
}

void XDpopdown_link (Widget w, XtPointer client_data, XtPointer call_data)
{
  XtPopdown ( *(Widget *)client_data );
}

void XDmap_link (Widget w, XtPointer client_data, XtPointer call_data)
{
  XtMapWidget (*(Widget *)client_data);
}

void XDunmap_link (Widget w, XtPointer client_data, XtPointer call_data)
{
  XtUnmapWidget (*(Widget *)client_data);
}

void XDenable_link (Widget w, XtPointer client_data, XtPointer call_data)
{
  XtSetSensitive (*(Widget *)client_data, TRUE);
}

void XDdisable_link (Widget w, XtPointer client_data, XtPointer call_data)
{
  XtSetSensitive (*(Widget *)client_data, FALSE);
}

extern void XDhelp_link (Widget w, XtPointer client_data, XtPointer call_data);

void 
ShowHelpPage (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  /*
   * Client data is an integer that indicates the page number to display.
   */
  int pageNumber;
  
  pageNumber = (int)client_data;
  DisplayHelp(pageNumber);
}

void 
FileOkButtonPressed (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  char *filename;
  char errorText[100];
  XmString labelString;
  
  filename = XmTextFieldGetString(fileSelectionText);
  
  /*
   * If there are no other clips in the list, uninitialize the
   * movie module.
   */
  if(GetNumberOfClips() == 0)
  {
    JPEGUninitialize();
  }

  if(filename != NULL)
  {
    if(IsJPEGMovieFile(filename))
    {
      if(JPEGIsCompatibleMovie(filename))
      {
	XtUnmanageChild(fileListDialog);
	SelectMovieFileName(filename, True);
	XtFree(filename);
	XtSetSensitive(movieSelectionDoneButton, True);
	return;
      }
      else
      {
	sprintf(errorText, "%s is not compatible with the other movies.",
		filename);
      }
    }
    else
    {
      sprintf(errorText, "%s is not a valid jpeg movie.",
	      filename);
    }      
  }
  else
  {
    sprintf(errorText, "%s - bad filename.",
	    filename);
  }    
  XtSetSensitive(movieSelectionDoneButton, False);
  XmTextSetString(fromFrameText, "");
  XmTextSetString(toFrameText, "");
  labelString = XmStringCreateSimple("Movie Length:");
  XtVaSetValues(movieLengthLabel,
		XmNlabelString, labelString,
		NULL);
  XmStringFree(labelString);
  PrintError(errorText);
}

void 
ErrorShellOKButtonPressed (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  XtPopdown(errorShell);

  SetMovieCatStatus(GetStateBeforeError());
  
  /*
   * If we are currently creating a movie, we may want to give
   * the user a chance to correct the problem.  Here, we will
   * assume the problem is disk full.  If the user clears some
   * drive space and then presses OK, we would like to continue.
   */
  if(GetMovieCatStatus() == MC_CREATING)
  {
    XtUnmanageChild(errorShellCancelButton);
    JPEGSyncFiles();
    XtAppAddWorkProc(app_context, MakeMovieWorkProc, (XtPointer)NULL);  
  }
}

void 
ErrorShellCancelButtonPressed (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  XtPopdown(errorShell);

  SetMovieCatStatus(GetStateBeforeError());

  /*
   * If we are currently creating a movie, and the user chooses
   * to cancel when we run out of drive space, attempt to complete
   * the movie by backing up a couple of frames and writing the
   * index buffer.
   */
  if(GetMovieCatStatus() == MC_CREATING)
  {
    XtUnmanageChild(errorShellCancelButton);
    JPEGFinishMovieNow();

    /*
     * We are finished creating.  We should reset the current
     * state of MovieCat.
     */
    SetMovieCatStatus(MC_READY);
    
    /*
     * Also, do not leave the progress dialog up.
     */
    XtPopdown(movieProgressShell);
  }
}


void 
MovieProgressCancelButtonPressed (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  /*
   * Remove the popup.
   */
  XtPopdown(movieProgressShell);
  
  /*
   * Finish the movie.
   */
  JPEGCloseClipFile();
  JPEGCloseMovieFile();

  /*
   * Now set up the application state.
   */
  if(GetNumberOfClips() > 0)
  {
    SetMovieCatStatus(MC_READY);
  }
  
  else
  {
    SetMovieCatStatus(MC_NO_CLIPS);
  }    
}

void 
NewClipFileName (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  /*
   *  Here, we must check to make sure that it is a valid movie
   *  file.
   */
  char *filename;

  filename = XmTextFieldGetString(movieFileText);
  if(filename != NULL)
  {
    if(IsJPEGMovieFile(filename))
    {
      if(JPEGIsCompatibleMovie(filename))
      {
	SelectMovieFileName(filename, False);
	XtFree(filename);
	XtSetSensitive(movieSelectionDoneButton, True);
      }
      else
      {
	char errorText[100];
	char newFilenameString[2] = "\0";

	XmTextSetString(movieFileText, newFilenameString);      
	sprintf(errorText, "%s is not compatible with the other movies.", 
		filename);
	XtFree(filename);
	PrintError(errorText);
	XtSetSensitive(movieSelectionDoneButton, False);
      }
    }
    else
    {
      char errorText[100];
      char newFilenameString[2] = "\0";

      XmTextSetString(movieFileText, newFilenameString);      
      sprintf(errorText, "%s is not a valid jpeg movie.", 
	      filename);
      XtFree(filename);
      PrintError(errorText);
      XtSetSensitive(movieSelectionDoneButton, False);
    }
  }
}

void 
NewClipFileNameChanged (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  char *filename;
  XmString labelString;

  /*
   * If there are no other clips in the list, uninitialize the
   * movie module.
   */
  if(GetNumberOfClips() == 0)
  {
    JPEGUninitialize();
  }

  filename = XmTextFieldGetString(movieFileText);
  if(filename != NULL)
  {
    if(IsJPEGMovieFile(filename))
    {
      if(JPEGIsCompatibleMovie(filename))
      {
	SelectMovieFileName(filename, False);
	XtFree(filename);
	XtSetSensitive(movieSelectionDoneButton, True);
	return;
      }
    }
  }
  XtSetSensitive(movieSelectionDoneButton, False);
  XmTextSetString(fromFrameText, "");
  XmTextSetString(toFrameText, "");
  labelString = XmStringCreateSimple("Movie Length:");
  XtVaSetValues(movieLengthLabel,
		XmNlabelString, labelString,
		NULL);
  XmStringFree(labelString);
}


void 
CopyEntireMovie (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  SetCopyEntireMovie();
}

void 
CopySelectedFrames (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  SetCopySelectedFrames();
}

void 
FromFrameEntered (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  /*
   * Make sure that this frame is actually within the specified movie.
   */
  char *fromFrame;
  int frame;
  
  fromFrame = XmTextFieldGetString(fromFrameText);
  if(fromFrame)
  {
    frame = atoi(fromFrame);
    if(frame >= 0 && frame < GetClipLength())
    {
      UpdateFromFrame(frame);
    }
    else
    {
      UpdateFromFrame(0);
    }
    XtFree(fromFrame);
  }
  else
  {
    UpdateFromFrame(0);
  }
}

void 
ToFrameEntered (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  /*
   * Make sure that this frame is within the specified movie and comes
   * after the frame specified in the from field.
   */
  char *toFrame;
  int frame;

  /*
   * Start by reading the From text field.  (It may have
   * changed.
   */
  FromFrameEntered(fromFrameText, (XtPointer)NULL, (XtPointer)NULL);
  
  toFrame = XmTextFieldGetString(toFrameText);
  if(toFrame)
  {
    frame = atoi(toFrame);
    if(frame >= GetFromFrame() && frame < GetClipLength())
    {
      UpdateToFrame(frame);
    }
    else
    {
      UpdateToFrame(GetClipLength() - 1);
    }
    XtFree(toFrame);
  }
  else
  {
    UpdateToFrame(GetClipLength() - 1);
  }
}

void 
CopyAudio (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  SetCopyAudio();
}

void 
OmitAudio (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  SetOmitAudio();
}

void 
MovieSelectionDoneButtonPressed (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  switch(GetMovieCatStatus())
  {
    case MC_ADD:
    {
      AddMovieClip();
      if(GetNumberOfClips() > 0)
      {
	SetMovieCatStatus(MC_READY);
      }
      else
      {
	SetMovieCatStatus(MC_NO_CLIPS);
      }
      break;
    }
    case MC_INSERT:
    {
      InsertMovieClip();
      if(GetNumberOfClips() > 0)
      {
	SetMovieCatStatus(MC_READY);
      }
      else
      {
	SetMovieCatStatus(MC_NO_CLIPS);
      }
      break;
    }  
    case MC_MODIFY:
    {
      ModifyMovieClip();
      if(GetNumberOfClips() > 0)
      {
	SetMovieCatStatus(MC_READY);
      }
      else
      {
	SetMovieCatStatus(MC_NO_CLIPS);
      }
      break; 
    }
      
    default:
    {

    }
  }  
  
  /*
   * Make sure that the file dialog does not stay open.
   */
  if(XtIsManaged(fileListDialog))
  {
    XtUnmanageChild(fileListDialog);
  }
}

void 
MovieSelectionCancelButtonPressed (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  if(GetNumberOfClips() > 0)
  {
    SetMovieCatStatus(MC_READY);
  }
  
  else
  {
    SetMovieCatStatus(MC_NO_CLIPS);
  }    

  /*
   * Make sure that the file dialog does not stay open.
   */
  if(XtIsManaged(fileListDialog))
  {
    XtUnmanageChild(fileListDialog);
  }
}

void 
NewButtonPressed (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  DeleteClipList();
  InitializeMovieSelectionModule();
}

void 
QuitButtonPressed (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  exit(0);
}

void 
ClipListBrowse (Widget w, XtPointer client_data, XtPointer xt_call_data )
{

}

void 
ClipListSingle (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  int *positionList;
  int listCount;
  
  XmListGetSelectedPos(w, &positionList, &listCount);
  if(listCount == 1)
  {
    XtSetSensitive(insertClipButton, True);
    XtSetSensitive(modifyClipButton, True);
    XtSetSensitive(deleteClipButton, True);
    SetCurrentClip((positionList[0]) - 1);
  }
  else
  {
    XtSetSensitive(insertClipButton, False);
    XtSetSensitive(modifyClipButton, False);
    XtSetSensitive(deleteClipButton, False);
    /*
     * Select the last clip.
     */
    SetCurrentClip(GetNumberOfClips()-1);
  }
}

void 
AddClipButtonPressed (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  SetMovieCatStatus(MC_ADD);
  ChangeSelectionBoxTitle("MovieCat:  Add");
  XtSetSensitive(movieSelectionDoneButton, False);
}

void 
InsertClipButtonPressed (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  SetMovieCatStatus(MC_INSERT);
  ChangeSelectionBoxTitle("MovieCat:  Insert");
  XtSetSensitive(movieSelectionDoneButton, False);
}

void 
ModifyClipButtonPressed (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  SetMovieCatStatus(MC_MODIFY);
  StartToModifyClip();
  XtSetSensitive(movieSelectionDoneButton, True);
}

void 
DeleteClipButtonPressed (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  SetMovieCatStatus(MC_DELETE);
  
  DeleteClip();
  if(GetNumberOfClips() > 0)
  {
    SetMovieCatStatus(MC_READY);
  }
  else
  {
    SetMovieCatStatus(MC_NO_CLIPS);
  }
}

void 
DestinationFilenameEntered (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  char *fileName;
  
  fileName = XmTextFieldGetString(destinationFileText);
  SetDestinationFile(fileName);
  XtFree(fileName);
}

void 
CreateMovieButtonPressed (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  FILE *testFile;
  char *movieFileName;
  
  /*
   * Get the destination file name out of the text widget.
   */
  DestinationFilenameEntered(destinationFileText, (XtPointer)NULL, (XtPointer)NULL);
  
  /*
   * Check to make sure that it is a valid filename before creating a movie.
   */
  movieFileName = (char*)GetDestinationFile();
  if(*movieFileName != '\0')
  {
    testFile = fopen(movieFileName, "wb");
    if(testFile != NULL)
    {
      /*
       * The destination file name was good.
       */
      fclose(testFile);
      CreateMovie();
      return;
    }
  }
  /*
   * The destination file was not good.
   */
  PrintError("Bad destination filename.  Please enter another.");
}



