/* $Header: /plx/source/Xclients/generic/RCS/jpegMovie.c,v 1.3 1995/10/27 16:59:33 lovvik Exp $ */

/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */

/* $Log: jpegMovie.c,v $
 * Revision 1.3  1995/10/27  16:59:33  lovvik
 * Changed the return type of JPEGSetDestinationFile from int to
 * void.  Also, removed unused variables.
 *
 * Revision 1.2  1995/10/04  00:15:24  lovvik
 * Modified the function JPEGAudioIsCompatible().  The
 * previous version looked at movieHeader.tracks to see
 * if the audio was compatible.  Really, it should check
 * the number of audio channels.
 *
 * Revision 1.1  1995/09/21  00:32:31  lovvik
 * Initial revision
 * */

static char movieCatRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/jpegMovie.c,v 1.3 1995/10/27 16:59:33 lovvik Exp $";

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>

#include "interface.h"
#include "globals.h"
#include "structs.h"
#include "prototypes.h"

#include <stdio.h>
#include <stdlib.h>

#ifndef True
#define True 1
#endif

#ifndef False
#define False 0
#endif

#ifndef SEEK_SET
#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2
#endif


/*
 * This header structure is used in standardizing all of the
 * movies that are being put together.
 */
static jpheader standardMovieHeader;
static initialized = False;
static int totalFrames;
static int *destinationFrameTable = NULL;
static int destinationPosition;
static FILE *destinationFile;
static FILE *sourceFile;
static int destinationFrame;
static char* audioData = NULL;
static char* jpegData = NULL;
static int copyAudio;
static int fromFrame;
static int toFrame;
static jpheader movieHeader;
static int sourceFrame;
static int *frameTable = NULL;
static int *tempFramePointer;
static unsigned char opcode;
static int done;
static int jpegSize;
static int clipCount = 0;
static char *destinationFilename = NULL;


int IsJPEGMovieFile(char *fileName)
{
  /*
   * This function returns True if this is a valid movie file.
   */
  FILE *sourceFile;
  jpheader movieHeader;
  int numberOfRecords;
  
  sourceFile = fopen(fileName, "rb");
  if(!sourceFile)
  {
    return(False);
  }

  /*
   * Load the header of the source file.
   */
  numberOfRecords = fread(&movieHeader, sizeof(jpheader), 1, sourceFile);
  fclose(sourceFile);
  /*
   * Check to see if this is a valid movie file.
   */
  if(numberOfRecords == 1 &&
     strncmp(movieHeader.magic, "j_movie", 7) == 0)
  {
    return(True);
  }
  else
  {
    return(False);
  }
}


int ValidFirstAndLastFrames(jpheader *movieHeader)
{
  if(movieHeader->firstValidFrame >= 0 &&
     movieHeader->firstValidFrame < movieHeader->frames &&
     movieHeader->lastValidFrame > movieHeader->firstValidFrame &&
     movieHeader->lastValidFrame < movieHeader->frames)
  {
    return(True);
  }
  else
  {
    return(False);
  }  
}

int GetJPEGLastFrame(char *fileName)
{
  /*
   * This function returns the number of frames in the movie 
   * if this is a valid movie file.  Otherwise, it returns 0;
   * Note:  On version 3 movie files, this function returns
   * the difference between the first valid frame and the
   * last valid frame.
   */
  FILE *sourceFile;
  jpheader movieHeader;
  
  sourceFile = fopen(fileName, "rb");
  if(!sourceFile)
  {
    return(False);
  }

  /*
   * Load the header of the source file.
   */
  fread(&movieHeader, sizeof(jpheader), 1, sourceFile);
  fclose(sourceFile);
  /*
   * Check to see if this is a valid movie file.
   */
  if(strncmp(movieHeader.magic, "j_movie", 7) == 0)
  {
    if(movieHeader.version >= JPEG_VERSION_3)
    {
      /*
       * We must make sure that the firstValidFrame
       * and the lastValidFrame fields are reasonable.
       */
      if(ValidFirstAndLastFrames(&movieHeader))
      {
	return(movieHeader.lastValidFrame - movieHeader.firstValidFrame + 1);
      }
      else
      {
	return(movieHeader.frames);
      }
    }
    else
    {
      return(movieHeader.frames);
    }
  }
  else
  {
    return(False);
  }
}


void JPEGSetDestinationFile(char* destinationFileName)
{
  destinationFile = fopen(destinationFileName, "wb+");
  if(!destinationFile)
  {
    return;
  }
  if(destinationFilename != NULL)
  {
    free(destinationFilename);
  }
  destinationFilename = (char *)malloc(sizeof(char) * (strlen(destinationFileName) + 1));
  if(!destinationFilename)
  {
    PrintError("Error!  Could not allocate memory.");
    return;
  }    
  strcpy(destinationFilename, destinationFileName);
}

  
void JPEGInitializeMovieHeader(jpheader movieHeader)
{
  memcpy(&standardMovieHeader, &movieHeader, sizeof(jpheader));
  if(memcmp(&standardMovieHeader, &movieHeader, sizeof(jpheader)) != 0)
  {
    PrintError("Error!  Could not copy the movie header properly.");
    return;
  }
  totalFrames = movieHeader.frames;
  standardMovieHeader.frames = totalFrames;
  
  if(!destinationFrameTable)
  {
    destinationFrameTable = (int*)malloc(sizeof(int) * (totalFrames + 1));
    if(!destinationFrameTable)
    {
      PrintError("Error!  Could not allocate memory for the frame index.");
      return;
    }
  }
  
  if(!audioData)
  {
    audioData = (char*)malloc(sizeof(char) * 48000);
    if(!audioData)
    {
      PrintError("Error!  Could not allocate memory for audio data.");
      return;
    }    
  }

  if(!jpegData)
  {
    /*
     * Should not need more than 128 kilobytes for the jpeg data.
     */
    jpegData = (char*)malloc(sizeof(char) * 128 * 1024);
    if(!jpegData)
    {
      PrintError("Error!  Could not allocate memory for the jpeg image.");
      return;
    }
  }
  
  destinationFrame = 0;
  initialized = True;
}

int JPEGIsCompatibleMovie(char *filename)
{
  FILE *sourceFile;
  jpheader movieHeader;
  
  sourceFile = fopen(filename, "rb");
  if(!sourceFile)
  {
    return(False);
  }

  /*
   * Load the header of the source file.
   */
  fread(&movieHeader, sizeof(jpheader), 1, sourceFile);
  fclose(sourceFile);
  /*
   * Check to see if this is a valid movie file.
   */
  if(strncmp(movieHeader.magic, "j_movie", 7) == 0)
  {
    /*
     * This is a valid movie file.  Check to see if it
     * is compatible.
     */
    if(!initialized)
    {
      /*
       * If uninitialized, it is compatible.
       */
      JPEGInitializeMovieHeader(movieHeader);
      return(True);
    }
    else
    {
      /*
       * This module has an initialized standard movie
       * header.  Check to see if this clip is compatible
       * with it.  We must check qfactor, width, and height.
       */
      if(movieHeader.qfactor != standardMovieHeader.qfactor ||
	 movieHeader.width != standardMovieHeader.width ||
	 movieHeader.height != standardMovieHeader.height ||
	 movieHeader.fps != standardMovieHeader.fps)
      {
	return(False);
      }
      else
      {
	return(True);
      }
    }
  }
  else
  {
    /*
     * Not a valid movie file.
     */
    return(False);
  }
}



int JPEGAudioIsCompatible(char *filename)
{
  FILE *sourceFile;
  jpheader movieHeader;
  
  sourceFile = fopen(filename, "rb");
  if(!sourceFile)
  {
    return(False);
  }

  /*
   * Load the header of the source file.
   */
  fread(&movieHeader, sizeof(jpheader), 1, sourceFile);
  fclose(sourceFile);
  /*
   * Check to see if this is a valid movie file.
   */
  if(strncmp(movieHeader.magic, "j_movie", 7) == 0)
  {
    /*
     * This is a valid movie file.  Check to see if the audio
     * is compatible.
     */
    if(!initialized)
    {
      /*
       * If uninitialized, it is compatible.
       */
      JPEGInitializeMovieHeader(movieHeader);
      return(True);
    }
    else
    {
      /*
       * This module has an initialized standard movie
       * header.  Check to see if the audio is compatible.
       * Included in this test are sample rate, channels,
       * bits per sample, and encoding.
       */
      if(movieHeader.audio.channels != standardMovieHeader.audio.channels ||
	 movieHeader.audio.sample_rate != standardMovieHeader.audio.sample_rate ||
	 movieHeader.audio.bytes_per_unit != standardMovieHeader.audio.bytes_per_unit ||
	 movieHeader.audio.encoding != standardMovieHeader.audio.encoding ||
	 movieHeader.audioslice != standardMovieHeader.audioslice
	 )
      {
	return(False);
      }
      else
      {
	return(True);
      }
    }
  }
  else
  {
    /*
     * Not a valid movie file.
     */
    return(False);
  }
}

unsigned int EstimateClipSize(char *filename, int fromFrame, int toFrame, int copyAudio)
{
  FILE *sourceFile;
  jpheader movieHeader;
  int *frameTable;
  int estimatedSize;
  
  sourceFile = fopen(filename, "rb");
  if(!sourceFile)
  {
    return(0);
  }

  /*
   * Load the header of the source file.
   */
  fread(&movieHeader, sizeof(jpheader), 1, sourceFile);

  /*
   * Check to see if this is a valid movie file.
   */
  if(strncmp(movieHeader.magic, "j_movie", 7) == 0)
  {
    /*
     * This is a valid movie file.  Estimate clip size.
     */

    frameTable = (int*)malloc(sizeof(int) * (movieHeader.frames + 1));
    if(!frameTable)
    {
      PrintError("Error!  Could not allocate memory for the frame index.");
      return(0);
    }
    fseek(sourceFile, movieHeader.indexbuf, SEEK_SET);
    fread(frameTable, sizeof(int), movieHeader.frames, sourceFile);

    estimatedSize = frameTable[toFrame]-frameTable[fromFrame];
    
    /*
     * If we are not copying the audio track, we can delete (audioslice * frames)
     * from our estimate.
     */
    if(!copyAudio)
    {
      /*
       * This one: -------------------------------|
       * is for the opcode.                       v
       */
      estimatedSize -= ((movieHeader.audioslice + 1) * (toFrame - fromFrame + 1));
    }
    
    free(frameTable);
    fclose(sourceFile);
    return(estimatedSize);
  }
  else
  {
    /*
     * Not a valid movie file.
     */
    fclose(sourceFile);
    return(0);
  }
}

  

int JPEGIsDoneWithMovie(void)
{
  if(sourceFrame <= toFrame)
  {
    return(False);
  }
  else
  {
    return(True);
  }
}

  
int JPEGCopyFrame(void)
{
  int elementsWritten;
  
  /*
   * Find the next frame in the movie, read the frame,
   * copy the frame, and increment the frame counter.
   */

  /*
   * Make a note of the current position in the destination
   * file as the begining of the next frame.
   */
  fflush(destinationFile);
  destinationFrameTable[destinationFrame] = ftell(destinationFile);
  /*
   * Go to the next frame in the movie file.
   */
  fseek(sourceFile, frameTable[sourceFrame], SEEK_SET);
    
  done = False;
  while(!done)
  {
    /*
     * Read in the opcode.
     */
    
    fread(&opcode, sizeof(unsigned char), 1, sourceFile); 
    opcode &= 0x00ff;
    
    /*
     * Write the opcode to the destination file.
     */
    fflush(sourceFile);
    elementsWritten = fwrite(&opcode, sizeof(unsigned char), 1, destinationFile);
    if(elementsWritten != 1)
    {
      JPEGErrorInFrame(destinationFrame);
      return(JPEGPercentComplete());
    }
    
    fflush(destinationFile);
    
    switch((int)opcode)
    {
      case LOAD_AUDIO_0:
      case LOAD_AUDIO_1:
      case LOAD_AUDIO_2:
      case LOAD_AUDIO_3:
      {
	/*
	 * If we are copying audio and the audio in this movie
	 * matches the audio in the first movie, copy the audio
	 * to the destination file.
	 */
	if(movieHeader.audioslice == standardMovieHeader.audioslice &&
	   copyAudio == True)
	{
	  fread(audioData, standardMovieHeader.audioslice, 1, sourceFile);
	  fflush(sourceFile);
	  elementsWritten = fwrite(audioData, standardMovieHeader.audioslice, 1, destinationFile);
	  if(elementsWritten != 1)
	  {
	    JPEGErrorInFrame(destinationFrame);
	    return(JPEGPercentComplete());
	  }
	  
	  destinationPosition += standardMovieHeader.audioslice;
	  fflush(destinationFile);
	}
	else
	{
	  fseek(sourceFile, movieHeader.audioslice, SEEK_CUR);
	  fseek(destinationFile, -1, SEEK_CUR);
	}
      }
      break;
      
      case LOAD_JPEG:
      {
	/*
	 * Read in the size of the jpeg data.
	 */
	fread(&jpegSize, sizeof(int), 1, sourceFile);
	fflush(sourceFile);
	elementsWritten = fwrite(&jpegSize, sizeof(int), 1, destinationFile);
	if(elementsWritten != 1)
	{
	  JPEGErrorInFrame(destinationFrame);
	  return(JPEGPercentComplete());
	}
	
	destinationPosition += sizeof(int);
	fflush(destinationFile);
	
	fread(jpegData, jpegSize, 1, sourceFile);
	fflush(sourceFile);
	   
	elementsWritten = fwrite(jpegData, jpegSize, 1, destinationFile);
	if(elementsWritten != 1)
	{
	  JPEGErrorInFrame(destinationFrame);
	  return(JPEGPercentComplete());
	}
	
	destinationPosition += (jpegSize * sizeof(int));
	fflush(destinationFile);
      }
      break;
	
      case END_FRAME:
      {
	done = True;
      }
      break;

      default:
      {
	exit(1);
	break;
      }	   
    }
  }
  
  sourceFrame++;
  destinationFrame++;
  return(JPEGPercentComplete());
/* To here */  
}

void JPEGCopyFile(char* fileName, int FromFrame, int ToFrame, int CopyAudio)
{
  int elementsWritten;
  
  /*
   * Initialize module variables.
   */
  fromFrame = FromFrame;
  toFrame = ToFrame;
  copyAudio = CopyAudio;
  
  sourceFile = fopen(fileName, "rb");
  if(!sourceFile)
  {
    PrintError("Error!  Could not open file for reading.");
    return;
  }

  /*
   * Load the header of the source file.
   */
  fread(&movieHeader, sizeof(jpheader), 1, sourceFile);
  if(!clipCount)
  {
    JPEGInitializeMovieHeader(movieHeader);
    if(!initialized)
    {
      PrintError("Could not initialize header!");
      exit(1);
    }
    
    /*
     * If this is the first file, we must copy its header
     * into the destination file.  Remember to keep track of
     * the destination file position as you go.
     */
    fflush(sourceFile);
    elementsWritten = fwrite(&movieHeader, sizeof(jpheader), 1, destinationFile);
    if(elementsWritten != 1)
    {
      /*
       * Error occurred while writting the movie header.  Close the
       * source and destination files.  Re-initialize the jpeg movie
       * module.  Remove the file that we attempted to write.  Present
       * an error box to the user.
       */
      char errorString[100];
      
      fclose(sourceFile);
      sourceFile = NULL;
      fclose(destinationFile);
      destinationFile = NULL;

      /*
       * Since we cannot recover from this error, reset the MovieCat
       * state before printing the error.
       */
      SetMovieCatStatus(MC_READY);
      sprintf(errorString, "Error!  Could not write to file %s.", destinationFilename);
      PrintError(errorString);

      JPEGDeleteDestinationFile();
    }
    
    fflush(destinationFile);
  }
  else
  {
    /*
     * Verify that the movies can be put together.  Check things
     * like the movie width, height, qfactor, etc.
     */
    if(movieHeader.qfactor != standardMovieHeader.qfactor ||
       movieHeader.width != standardMovieHeader.width ||
       movieHeader.height != standardMovieHeader.height ||
       movieHeader.fps != standardMovieHeader.fps)
    {
      return;
    }

    if(movieHeader.version >= JPEG_VERSION_3)
    {
      if(ValidFirstAndLastFrames(&movieHeader))
      {
	fromFrame += movieHeader.firstValidFrame;
	toFrame += movieHeader.firstValidFrame;
      }
    }	
    
    /*
     * If this is not the first movie, we have to allocate more
     * memory for the frame index.
     */
    
    totalFrames += (ToFrame-FromFrame + 1);
    standardMovieHeader.frames = totalFrames;
  
    tempFramePointer = (int*)realloc(destinationFrameTable,
				     sizeof(int) * (totalFrames + 1));    
    if(!tempFramePointer)
    {
      PrintError("Error!  Could not re-allocate memory for the frame index table.");
      return;
    }
    destinationFrameTable = tempFramePointer;
  }
  
  /*
   * Load the index table from the source file.
   */
  frameTable = (int*)malloc(sizeof(int) * (movieHeader.frames + 1));
  if(!frameTable)
  {
    PrintError("Error!  Could not allocate memory for the frame index table.");
    return;
  }
  fseek(sourceFile, movieHeader.indexbuf, SEEK_SET);
  fread(frameTable, sizeof(int), movieHeader.frames, sourceFile);

  /*
   * For each frame in the movie, copy the data to the destination file.
   */
  sourceFrame = fromFrame;
  JPEGCopyFrame();
}

void JPEGSyncFiles(void)
{
  /*
   * This routine will only be called if an error has
   * occurred.  The purpose of this function is to point
   * to the beginning of the frame that the write error
   * occurred on.  The problem is most likely going to
   * be a hard-drive space error.  If that is the case,
   * this gives the user a chance to create some space
   * and get back to copying the movie.
   */
  if(sourceFile)
  {
    fflush(sourceFile);
    fflush(destinationFile);
    fseek(sourceFile, frameTable[sourceFrame], SEEK_SET);
    fseek(destinationFile, destinationFrameTable[destinationFrame], SEEK_SET);
  }
  else
  {
    /*
     * If there is no source file, then we got an error
     * while writing the index buffer.  In this case, don't
     * back up to the last frame.  Instead, position the file
     * pointer to the beginning of the index buffer.
     */
    fseek(destinationFile, standardMovieHeader.indexbuf, SEEK_SET);
  }
}

void JPEGFinishMovieNow(void)
{
  /*
   * This routine will only be called if an error has
   * occurred.  This function finishes the destination file
   * by backing up a couple of frames and writing the 
   * index buffer.
   */

  int framesToRemove;
  long freeSpace;
  
  /*
   * The distance we have to back up depends on the number of
   * frames recorded (stored in destinationFrame).  Since the
   * current frame caused the error, we need to back up one
   * frame.
   */
  if(destinationFrame > 0)
  {
    destinationFrame--;
  }
  
  /* 
   * The size of the last frame can be calculated by looking at
   * the index buffer.  Subtract the position of the last frame
   * (destinationFrameTable[destinationFrame]) from the current
   * file position.
   */

  framesToRemove = 0;
  freeSpace = ftell(destinationFile) - destinationFrameTable[destinationFrame];

  while (freeSpace <= (sizeof(int) * (destinationFrame + 1)))
  {
    /*
     * If this isn't enough, back up another frame.
     */
    framesToRemove++;
    if(framesToRemove > destinationFrame)
    {
      fflush(destinationFile);
      fclose(destinationFile);
      JPEGDeleteDestinationFile();
      return;
    }
    
    freeSpace = ftell(destinationFile) - destinationFrameTable[destinationFrame-framesToRemove];
  }

  /*
   * Position for the index table.
   */
  fseek(destinationFile, destinationFrameTable[destinationFrame - framesToRemove + 1], SEEK_SET);

  /*
   * Update the number of good frames.
   */
  destinationFrame -= framesToRemove + 1;
  
  /*
   * Close the current clip file.
   */
  JPEGCloseClipFile();

  /*
   * Write the index buffer and update the header.
   */
  JPEGCloseMovieFile();
}


void JPEGCloseClipFile(void)
{
  clipCount++;
  free(frameTable);
  frameTable = NULL;
  fclose(sourceFile);
  sourceFile = NULL;
}

void JPEGCloseMovieFile(void)
{
  int elementsWritten;
  
  if(destinationFrame == 0)
  {
    
  }
  
  /*
   * Close the file by first writing the frame index table.
   */
  fflush(destinationFile);
  standardMovieHeader.indexbuf = (int)ftell(destinationFile);
  elementsWritten = fwrite(destinationFrameTable, sizeof(int), destinationFrame, destinationFile);
  if(elementsWritten != destinationFrame)
  {
    JPEGErrorInIndexTable();
    return;
  }
  
  fflush(destinationFile);

  /*
   * Now, go back and update the header.
   */
  standardMovieHeader.frames = destinationFrame;
  standardMovieHeader.firstValidFrame = 0;
  standardMovieHeader.lastValidFrame = destinationFrame - 1;
  fseek(destinationFile, 0, SEEK_SET);

  /*
   * We will not be out of drive space while doing this
   * write.  Because of the way the error detection/correction
   * is designed in this application, we cannot simply go back
   * and try to write again after calling the error code.
   * I have chosen, then, to not detect errors on this write.
   */
  elementsWritten = fwrite(&standardMovieHeader, sizeof(jpheader), 1, destinationFile);
  fflush(destinationFile);
/*  fseek(destinationFile, 0, SEEK_END); */
  fclose(destinationFile);

  /*
   *  If the movie that was written has no frames in it, we should
   *  remove the file.
   */
  destinationFrame = 0;
  clipCount = 0;
}


void JPEGUninitialize(void)
{
  initialized = False;
}


void JPEGErrorInFrame(int frame)
{
  char errorString[100];
  
  sprintf(errorString, "Error writing frame %d.  Press OK to retry.", frame);
  PrintError(errorString);
}

void JPEGErrorInIndexTable(void)
{
  char errorString[100];

  if(destinationFrame > 0)
  {
    destinationFrame--;
  }
  
  sprintf(errorString, "Error writing movie index table.  Press OK to retry.");
  PrintError(errorString);
}

float JPEGPercentComplete(void)
{
  return(((float)(sourceFrame-fromFrame)/(float)(toFrame-fromFrame)) * 100.0);  
}


void JPEGDeleteDestinationFile(void)
{
  if(destinationFilename != NULL)
  {
    remove(destinationFilename);
    free(destinationFilename);
    destinationFilename = NULL;
  }
}
