/* $Header: /plx/source/Xclients/generic/RCS/mainDialog.c,v 1.3 1995/12/12 19:00:50 lovvik Exp $ */

/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */

/* $Log: mainDialog.c,v $
 * Revision 1.3  1995/12/12  19:00:50  lovvik
 * Fixed a minor problem with InsertClipInList().  This function
 * did not copy the clipSize during clip insertion.  The result
 * was an inaccurate estimate of drive space requirements.
 *
 * Revision 1.2  1995/10/27  17:02:36  lovvik
 * Added include files:  <stdlib.h>, <stdio.h>.  Also removed
 * unused variables.
 *
 * Revision 1.1  1995/09/21  00:33:03  lovvik
 * Initial revision
 * */

static char movieCatRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/mainDialog.c,v 1.3 1995/12/12 19:00:50 lovvik Exp $";

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>
#include <stdio.h>
#include <stdlib.h>

#include "interface.h"
#include "globals.h"
#include "structs.h"
#include "prototypes.h"

static ClipListEntry movieClipList[MAXIMUM_NUMBER_OF_CLIPS];
static MovieCatInfo  movieCat;


void Initialize(void)
{
  int index;
  
  /*
   * Start by initializing the clip list.  If the fileName is
   * NULL, then the entry is not initialized.  Here, we will
   * set all of the fileName fields to NULL.
   */
  for(index = 0; index < MAXIMUM_NUMBER_OF_CLIPS; index++)
  {
    movieClipList[index].fileName = NULL;
  }
  
  /*
   * Now we will set up the application state structure.
   */
  movieCat.destinationFileName = NULL;
  movieCat.numberOfClips = 0;
  movieCat.currentClip = 0;
  SetMovieCatStatus(MC_NO_CLIPS);
  
  /*
   * Here will call the initialization routines for the other
   * modules.
   */
  InitializeMovieSelectionModule();
  InitializeAboutBox();
  InitializeHelp();
}

char *GetDestinationFile(void)
{
  return(movieCat.destinationFileName);
}


void SetDestinationFile(char *fileName)
{
  if(movieCat.destinationFileName != NULL)
  {
    free(movieCat.destinationFileName);
  }
  
  movieCat.destinationFileName = (char *)malloc(sizeof(char) * (strlen(fileName) + 1));
  if(!movieCat.destinationFileName)
  {
    PrintError("Error allocating memory!");
    exit(0);
  }
  strcpy(movieCat.destinationFileName, fileName);
}


int GetNumberOfClips(void)
{
  return(movieCat.numberOfClips);
}


void SetNumberOfClips(int clipCount)
{
  if(clipCount >= 0)
  {
    movieCat.numberOfClips = clipCount;
  }
}

int GetCurrentClip(void)
{
  return(movieCat.currentClip);
}


void SetCurrentClip(int currentClip)
{
  if(currentClip >= 0 && currentClip <= movieCat.numberOfClips)
  {
    movieCat.currentClip = currentClip;
  }
}


int GetMovieCatStatus(void)
{
  return(movieCat.status);
}


void SetMovieCatStatus(int status)
{
  movieCat.status = status;
  
  switch(status)
  {
    case MC_NO_CLIPS:
    {
      XtSetSensitive(fileMenu, True);
      
      XtSetSensitive(clipList, False);
      XtSetSensitive(addClipButton, True);
      XtSetSensitive(insertClipButton, False);
      XtSetSensitive(modifyClipButton, False);
      XtSetSensitive(deleteClipButton, False);
      XtSetSensitive(destinationFileText, True);
      XtSetSensitive(createMovieButton, False);
      
      XtSetSensitive(movieSelectionForm, False);
      XtSetSensitive(movieSelectionDoneButton, False);
      XtSetSensitive(movieSelectionCancelButton, False);
      
      break;
    }

    case MC_MODIFY:
    {
      XtSetSensitive(fileMenu, True);
      
      XtSetSensitive(clipList, False);
      XtSetSensitive(addClipButton, False);
      XtSetSensitive(insertClipButton, False);
      XtSetSensitive(modifyClipButton, False);
      XtSetSensitive(deleteClipButton, False);
      XtSetSensitive(destinationFileText, False);
      XtSetSensitive(createMovieButton, False);
      
      XtSetSensitive(movieSelectionForm, True);
      XtSetSensitive(movieSelectionDoneButton, True);
      XtSetSensitive(movieSelectionCancelButton, True);
      
      break;
    }

    case MC_ADD:
    {
      XtSetSensitive(fileMenu, True);
      
      XtSetSensitive(clipList, False);
      XtSetSensitive(addClipButton, False);
      XtSetSensitive(insertClipButton, False);
      XtSetSensitive(modifyClipButton, False);
      XtSetSensitive(deleteClipButton, False);
      XtSetSensitive(destinationFileText, False);
      XtSetSensitive(createMovieButton, False);
      
      XtSetSensitive(movieSelectionForm, True);
      XtSetSensitive(movieSelectionDoneButton, True);
      XtSetSensitive(movieSelectionCancelButton, True);
      
      break;
    }

    case MC_INSERT:
    {
      XtSetSensitive(fileMenu, True);
      
      XtSetSensitive(clipList, False);
      XtSetSensitive(addClipButton, False);
      XtSetSensitive(insertClipButton, False);
      XtSetSensitive(modifyClipButton, False);
      XtSetSensitive(deleteClipButton, False);
      XtSetSensitive(destinationFileText, False);
      XtSetSensitive(createMovieButton, False);
      
      XtSetSensitive(movieSelectionForm, True);
      XtSetSensitive(movieSelectionDoneButton, True);
      XtSetSensitive(movieSelectionCancelButton, True);
      
      break;
    }

    case MC_DELETE:
    {
      XtSetSensitive(fileMenu, True);
      
      XtSetSensitive(clipList, False);
      XtSetSensitive(addClipButton, False);
      XtSetSensitive(insertClipButton, False);
      XtSetSensitive(modifyClipButton, False);
      XtSetSensitive(deleteClipButton, False);
      XtSetSensitive(destinationFileText, False);
      XtSetSensitive(createMovieButton, False);
      
      XtSetSensitive(movieSelectionForm, False);
      XtSetSensitive(movieSelectionDoneButton, False);
      XtSetSensitive(movieSelectionCancelButton, False);
      
      break;
    }

    case MC_CREATING:
    {
      XtSetSensitive(fileMenu, False);
      
      XtSetSensitive(clipList, False);
      XtSetSensitive(addClipButton, False);
      XtSetSensitive(insertClipButton, False);
      XtSetSensitive(modifyClipButton, False);
      XtSetSensitive(deleteClipButton, False);
      XtSetSensitive(destinationFileText, False);
      XtSetSensitive(createMovieButton, False);
      
      XtSetSensitive(movieSelectionForm, False);
      XtSetSensitive(movieSelectionDoneButton, False);
      XtSetSensitive(movieSelectionCancelButton, False);
      
      break;
    }

    case MC_READY:
    {
      int *positionList;
      int listCount;
      
      XtSetSensitive(fileMenu, True);
      
      XtSetSensitive(clipList, True);
      XtSetSensitive(addClipButton, True);

      XmListGetSelectedPos(clipList, &positionList, &listCount);
      if(listCount == 1)
      {
	XtSetSensitive(insertClipButton, True);
	XtSetSensitive(modifyClipButton, True);
	XtSetSensitive(deleteClipButton, True);
	SetCurrentClip((positionList[0]) - 1);
      }
      else
      {
	XtSetSensitive(insertClipButton, False);
	XtSetSensitive(modifyClipButton, False);
	XtSetSensitive(deleteClipButton, False);
      }

      XtSetSensitive(destinationFileText, True);
      XtSetSensitive(createMovieButton, True);
      
      XtSetSensitive(movieSelectionForm, False);
      XtSetSensitive(movieSelectionDoneButton, False);
      XtSetSensitive(movieSelectionCancelButton, False);
      
      break;
    }
  }
}


void ShutDown(void)
{
  /*
   * During shutdown, we should free memory.
   */
}


void AppendClipToList(ClipListEntry newEntry)
{
  int position;
  char entryString[100];
  XmString entry;

  position = GetNumberOfClips();

  /*
   * Add the clip to the list.
   */
  movieClipList[position].fileName = 
    (char *)malloc(sizeof(char) * (strlen(newEntry.fileName) + 1));
  if(!movieClipList[position].fileName)
  {
    PrintError("Error!  Could not allocate memory for filename!");
    return;
  }
  strcpy(movieClipList[position].fileName, newEntry.fileName);
  movieClipList[position].copyEntireMovie = newEntry.copyEntireMovie;
  movieClipList[position].fromFrame = newEntry.fromFrame;
  movieClipList[position].toFrame = newEntry.toFrame;
  movieClipList[position].copyAudio = newEntry.copyAudio;
  movieClipList[position].movieLength = newEntry.movieLength;
  movieClipList[position].clipSize = newEntry.clipSize;
  
  sprintf(entryString, "%s [%d -> %d]", movieClipList[position].fileName,
	  movieClipList[position].fromFrame, movieClipList[position].toFrame);
  entry = XmStringCreateSimple(entryString);

  XmListAddItem(clipList, entry, position + 1);
  XmListSelectPos(clipList, position + 1, True); 
  XmStringFree(entry);
  SetNumberOfClips(GetNumberOfClips() + 1);
  SetCurrentClip(GetNumberOfClips() -1);

  SetEstimatedMovieSize();
}


void InsertClipInList(ClipListEntry newEntry)
{
  int position;
  char entryString[100];
  XmString entry;
  int index;
  
  if(GetNumberOfClips() == 0)
  {
    position = 0;
  }
  else
  {
    position = GetCurrentClip();
  }
  
  /*
   * Move the other clips down in the list.
   */
  for (index = GetNumberOfClips()-1; index >= position; index--)
  {
    movieClipList[index + 1].fileName = movieClipList[index].fileName;
    movieClipList[index + 1].copyEntireMovie = movieClipList[index].copyEntireMovie;
    movieClipList[index + 1].fromFrame = movieClipList[index].fromFrame;
    movieClipList[index + 1].toFrame = movieClipList[index].toFrame;      
    movieClipList[index + 1].copyAudio = movieClipList[index].copyAudio;
    movieClipList[index + 1].movieLength = movieClipList[index].movieLength;
    movieClipList[index + 1].clipSize = movieClipList[index].clipSize;
  }
  
  /*
   * Insert the clip into the list.
   */
  movieClipList[position].fileName = 
    (char *)malloc(sizeof(char) * (strlen(newEntry.fileName) + 1));
  if(!movieClipList[position].fileName)
  {
    PrintError("Error!  Could not allocate memory for filename!");
    return;
  }
  strcpy(movieClipList[position].fileName, newEntry.fileName);
  movieClipList[position].copyEntireMovie = newEntry.copyEntireMovie;
  movieClipList[position].fromFrame = newEntry.fromFrame;
  movieClipList[position].toFrame = newEntry.toFrame;
  movieClipList[position].copyAudio = newEntry.copyAudio;
  movieClipList[position].movieLength = newEntry.movieLength;
  movieClipList[position].clipSize = newEntry.clipSize;

  sprintf(entryString, "%s [%d -> %d]", movieClipList[position].fileName,
	  movieClipList[position].fromFrame, movieClipList[position].toFrame);
  entry = XmStringCreateSimple(entryString);

  XmListAddItem(clipList, entry, position + 1);
  XmListSelectPos(clipList, position + 1, True); 
  XmStringFree(entry);
  SetNumberOfClips(GetNumberOfClips() + 1);
  SetCurrentClip(position);
  SetEstimatedMovieSize();
}


char *GetCurrentClipFileName(void)
{
  return(movieClipList[movieCat.currentClip].fileName);
}

int GetCurrentClipFromFrame(void)
{
  return(movieClipList[movieCat.currentClip].fromFrame);
}

int GetCurrentClipToFrame(void)
{
  return(movieClipList[movieCat.currentClip].toFrame);
}


int GetCurrentClipCopyAudio(void)
{
  return(movieClipList[movieCat.currentClip].copyAudio);
}



Boolean MakeMovieWorkProc(XtPointer unusedData)
{
  if(GetMovieCatStatus() != MC_CREATING)
  {
    return(True);
  }

  if(!JPEGIsDoneWithMovie())
  {
    /*
     * Not done with the current movie.  Must continue copying frames.
     */
    int percentDone;
    
    percentDone = JPEGCopyFrame();
    SetPercentDone(percentDone);
    return(False);
  }
  else
  {
    /*
     * Just finished with a movie.  Close the movie file, and start
     * another copy operation.
     */
    JPEGCloseClipFile();
    SetCurrentClip(GetCurrentClip() + 1);
    SetPercentDone(0);
    if(GetCurrentClip() < GetNumberOfClips())
    {
      char fileNameString[100];
      
      sprintf(fileNameString, "%s [%d -> %d]", GetCurrentClipFileName(),
	      GetCurrentClipFromFrame(), GetCurrentClipToFrame());
      SetCurrentMovieFilename(fileNameString);
      JPEGCopyFile(GetCurrentClipFileName(), GetCurrentClipFromFrame(), GetCurrentClipToFrame(),
		   GetCurrentClipCopyAudio());
      SelectClipNumber(GetCurrentClip() + 1);
      return(False);
    }
    else
    {

    }
  }

  /*
   * Done copying the clip list.  Now, close the movie file and exit.
   */
  JPEGCloseMovieFile();
  SetMovieCatStatus(MC_READY);
  XtPopdown(movieProgressShell);
  /* Initialize the movie list. */
  JPEGIsCompatibleMovie(movieClipList[0].fileName);
  return(True);
}


void CreateMovie(void)
{
  char fileNameString[100];
  /*
   * To create a movie, we must set up a work procedure (so that
   * the application does not "freeze".  This work procedure will
   * figure out the next frame to be copied from which file, and
   * call functions that can make that happen.
   */
  XtManageChild(movieProgressDialog);
  XtPopup(movieProgressShell, XtGrabExclusive);
  SetMovieCatStatus(MC_CREATING);
  SetCurrentClip(0);

  /*
   * Modify the label on the progress dialog.
   */
  sprintf(fileNameString, "%s [%d -> %d]", GetCurrentClipFileName(),
	  GetCurrentClipFromFrame(), GetCurrentClipToFrame());
  SetCurrentMovieFilename(fileNameString);

  JPEGSetDestinationFile(movieCat.destinationFileName);
  JPEGCopyFile(GetCurrentClipFileName(), GetCurrentClipFromFrame(), GetCurrentClipToFrame(),
	       GetCurrentClipCopyAudio());
  SelectClipNumber(GetCurrentClip() + 1);
  
  XtAppAddWorkProc(app_context, MakeMovieWorkProc, (XtPointer)NULL);
}


void StartToModifyClip(void)
{
  ChangeSelectionBoxTitle("MovieCat:  Modify");
  InitializeSelectionBox(movieClipList[GetCurrentClip()]);
}

void ModifyClip(ClipListEntry modifiedEntry)
{
  int position;
  char entryString[100];
  XmString entry;
  
  position = GetCurrentClip();
  
  if(movieClipList[position].fileName)
  {
    free(movieClipList[position].fileName);
  }
  
  movieClipList[position].fileName = 
    (char *)malloc(sizeof(char) * (strlen(modifiedEntry.fileName) + 1));
  if(!movieClipList[position].fileName)
  {
    PrintError("Error!  Could not allocate memory for filename!");
    return;
  }
  strcpy(movieClipList[position].fileName, modifiedEntry.fileName);
  movieClipList[position].copyEntireMovie = modifiedEntry.copyEntireMovie;
  movieClipList[position].fromFrame = modifiedEntry.fromFrame;
  movieClipList[position].toFrame = modifiedEntry.toFrame;
  movieClipList[position].copyAudio = modifiedEntry.copyAudio;
  movieClipList[position].movieLength = modifiedEntry.movieLength;
  movieClipList[position].clipSize = modifiedEntry.clipSize;

  sprintf(entryString, "%s [%d -> %d]", movieClipList[position].fileName,
	  movieClipList[position].fromFrame, movieClipList[position].toFrame);
  entry = XmStringCreateSimple(entryString);
  
  XmListDeleteItemsPos(clipList, 1, position + 1);
  XmListAddItem(clipList, entry, position + 1);
  XmListSelectPos(clipList, position + 1, False);
  XmStringFree(entry);  
  SetEstimatedMovieSize();
}

void DeleteClip(void)
{
  int position;
  int index;
  
  position = GetCurrentClip();
  
  /*
   * Delete this position by freeing the filename.
   */
  if(movieClipList[position].fileName)
  {
    free(movieClipList[position].fileName);
    movieClipList[position].fileName = NULL;
  }
  
  /*
   * For the rest of the clips in the list, copy the information.
   */

  for(index = position; index < (GetNumberOfClips() - 1); index++)
  {
    movieClipList[index].fileName = movieClipList[index + 1].fileName;
    movieClipList[index].copyEntireMovie = movieClipList[index + 1].copyEntireMovie;
    movieClipList[index].fromFrame = movieClipList[index + 1].fromFrame;
    movieClipList[index].toFrame = movieClipList[index + 1].toFrame;
    movieClipList[index].copyAudio = movieClipList[index + 1].copyAudio;
    movieClipList[index].movieLength = movieClipList[index + 1].movieLength;
    movieClipList[index].clipSize = movieClipList[index + 1].clipSize;
  }

  movieClipList[GetNumberOfClips()-1].fileName = NULL;
  SetNumberOfClips(GetNumberOfClips() - 1);
  XmListDeletePos(clipList, position + 1);
  SetEstimatedMovieSize();
}


void DeleteClipList(void)
{
  int index;
  int clipCount;
  
  clipCount = GetNumberOfClips();
  for(index = 0; index < clipCount; index++)
  {
    free(movieClipList[index].fileName);
    movieClipList[index].fileName = NULL;
  }
  SetNumberOfClips(0);
  SetMovieCatStatus(MC_NO_CLIPS);
  SetEstimatedMovieSize();
  XmListDeleteAllItems(clipList);
  JPEGUninitialize();
}


void SetEstimatedMovieSize(void)
{
  unsigned int estimatedSize;
  float estimatedMb;
  int totalFrames;
  int index;
  char string[100];
  XmString spaceLabelString;
  
  
  /*
   * Note:  This estimation is only good if the frames
   *        in each movie clip are sequential.  If the
   *        frames jump around, this estimation will
   *        not work.
   */

  /*
   * Start with the size of the JPEG movie header.
   */
  estimatedSize = 128;
  totalFrames = 0;

  /*
   * Add the estimated size of each movie clip.
   */
  for(index = 0; index < movieCat.numberOfClips; index++)
  {
    estimatedSize += movieClipList[index].clipSize;
    totalFrames += (movieClipList[index].toFrame -
		    movieClipList[index].fromFrame);
  }
  
  /*
   * Add the size of the index buffer.
   */
  estimatedSize += (sizeof(int) * (totalFrames + 1));

  /*
   * Convert to Mb.
   */
  estimatedMb = (float)estimatedSize / (float)(1024 * 1024);
  
  /*
   * Update spaceLabel
   */
  sprintf(string, "Space Required: %.2f MB", estimatedMb);
  spaceLabelString = XmStringCreateSimple(string);
  XtVaSetValues(spaceLabel,
		XmNlabelString, spaceLabelString,
		NULL);
  XmStringFree(spaceLabelString);
}

void SelectClipNumber(int clipListEntry)
{
  XmListSelectPos(clipList, clipListEntry, False);
}

