/* $Header: /plx/source/Xclients/generic/RCS/movieSelection.c,v 1.3 1995/12/12 18:59:05 lovvik Exp $ */

/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */

/* $Log: movieSelection.c,v $
 * Revision 1.3  1995/12/12  18:59:05  lovvik
 * Changed InitializeMovieSelectionModule() to completely
 * clear the Add dialog.  I added statements to clear the
 * 3 text fields.
 *
 * Revision 1.2  1995/10/27  17:01:51  lovvik
 * Added include files:  <Xm/ToggleB.h>, <Xm/Text.h>, <stdio.h>,
 * <stdlib.h>.
 *
 * Revision 1.1  1995/09/21  00:34:14  lovvik
 * Initial revision
 * */

static char movieCatRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/movieSelection.c,v 1.3 1995/12/12 18:59:05 lovvik Exp $";

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleB.h>
#include <Xm/ToggleBG.h>
#include <stdio.h>
#include <stdlib.h>

#include "interface.h"
#include "structs.h"
#include "prototypes.h"

static ClipListEntry currentClip;

void InitializeMovieSelectionModule(void)
{
  currentClip.fileName = NULL;
  XmToggleButtonSetState(entireMovieToggle, True, False);
  XmToggleButtonSetState(selectedFramesToggle, False, False);
  XmTextSetString(movieFileText, "");
  XmTextSetString(fromFrameText, "");
  XmTextSetString(toFrameText, "");
  XtSetSensitive(fromFrameText, False);
  XtSetSensitive(toFrameText, True);
  XmToggleButtonSetState(copyAudioToggle, True, False);
  XmToggleButtonSetState(omitAudioToggle, False, False);
}

void ChangeSelectionBoxTitle(char *title)
{
  XtVaSetValues(movieSelectionShell,
		XmNtitle, title,
		NULL);
}



int SelectMovieFileName(char *filename, int updateWidget)
{
  /*
   * This function looks at the file name that was entered, and
   * if it is a JPEG movie file, then it returns 1 and reads in
   * the movie length.  If it is not a movie file, this function
   * will return 0.
   */

  /*
   * Print the file name in the text widget.
   */
  if(updateWidget == True)
  {
    XmTextSetString(movieFileText, filename);
  }

  if(filename)
  {
    /*
     * Check to see if it is a valid movie file.
     */
    if(IsJPEGMovieFile(filename))
    {
      /*
       * Copy the file name into the clip entry.
       */
      if(currentClip.fileName)
      {
	free(currentClip.fileName);
      }
      currentClip.fileName = (char*)malloc(sizeof(char) * (strlen(filename) + 1));
      if(!currentClip.fileName)
      {
	PrintError("Error!  Could not allocate memory for a file name.");
	return(0);
      }
      strcpy(currentClip.fileName, filename);
      
      /*
       * Set up the first and last frame numbers;
       */
      currentClip.fromFrame = 0;
      UpdateFromFrame(currentClip.fromFrame);
      
      currentClip.movieLength = GetJPEGLastFrame(filename);
      UpdateClipLength(currentClip.movieLength);
      currentClip.toFrame = currentClip.movieLength - 1;
      UpdateToFrame(currentClip.toFrame);

      SetCopyEntireMovie();
      SetCopyAudio();

      currentClip.clipSize = EstimateClipSize(currentClip.fileName,
					      currentClip.fromFrame,
					      currentClip.toFrame,
					      currentClip.copyAudio);
      return(1);
    }
    /*
     * Movie was not a jpeg movie.
     */
    PrintError("Specified file name is not a valid movie file.");
    return(0);
  }
  else
  {
    return(0);
  }
}


void UpdateClipLength(int movieLength)
{
  char movieLengthString[100];
  XmString movieXmString;
  
  currentClip.movieLength = movieLength;
  sprintf(movieLengthString, "Movie Length:  %d", currentClip.movieLength);
  movieXmString = XmStringCreateSimple(movieLengthString);
  XtVaSetValues(movieLengthLabel,
		XmNlabelString, movieXmString,
		NULL);
  XmStringFree(movieXmString);
}

void SetCopyEntireMovie(void)
{
  if(!XmToggleButtonGetState(entireMovieToggle))
  {
    /*
     * This was called by the application.  We must modify
     * the user interface to reflect the current state of
     * the application.
     */
    XmToggleButtonSetState(entireMovieToggle, True, False);
    XmToggleButtonSetState(selectedFramesToggle, False, False);
  }
  
  currentClip.copyEntireMovie = True;
  currentClip.fromFrame = 0;
  currentClip.toFrame = currentClip.movieLength - 1;
  currentClip.clipSize = EstimateClipSize(currentClip.fileName,
					  currentClip.fromFrame,
					  currentClip.toFrame,
					  currentClip.copyAudio);
  
  XtSetSensitive(fromFrameText, False);
  XtSetSensitive(toFrameText, False);
}  

void SetCopySelectedFrames(void)
{
  if(!XmToggleButtonGetState(selectedFramesToggle))
  {
    /*
     * This was called by the application.  We must modify
     * the user interface to reflect the current state of
     * the application.
     */
    XmToggleButtonSetState(entireMovieToggle, False, False);
    XmToggleButtonSetState(selectedFramesToggle, True, False);
  }
  
  currentClip.copyEntireMovie = False;
  currentClip.fromFrame = 0;
  currentClip.toFrame = currentClip.movieLength - 1;
  currentClip.clipSize = EstimateClipSize(currentClip.fileName,
					  currentClip.fromFrame,
					  currentClip.toFrame,
					  currentClip.copyAudio);
  
  XtSetSensitive(fromFrameText, True);
  XtSetSensitive(toFrameText, True);
}  
    
void SetCopyAudio(void)
{
  /*
   * First, check to make sure that the audio is
   * compatible with the rest of the clips.
   */
  if(JPEGAudioIsCompatible(currentClip.fileName))
  {
    if(!XmToggleButtonGetState(copyAudioToggle))
    {
      /*
       * This function was called by the application.  We must
       * modify the user interface to reflect the current state
       * of the application.
       */
      XmToggleButtonSetState(copyAudioToggle, True, False);
      XmToggleButtonSetState(omitAudioToggle, False, False);
    }    
    XtSetSensitive(copyAudioToggle, True);
    XtSetSensitive(omitAudioToggle, True);
    currentClip.copyAudio = True;
  }
  else
  {
    XmToggleButtonSetState(copyAudioToggle, False, False);
    XmToggleButtonSetState(omitAudioToggle, True, False);
    XtSetSensitive(copyAudioToggle, False);
    XtSetSensitive(omitAudioToggle, False);
    currentClip.copyAudio = False;
  }
}

void SetOmitAudio(void)
{
  if(!XmToggleButtonGetState(omitAudioToggle))
  {
    /*
     * This function was called by the application.  We must
     * modify the user interface to reflect the current state
     * of the application.
     */
    XmToggleButtonSetState(copyAudioToggle, False, False);
    XmToggleButtonSetState(omitAudioToggle, True, False);
  }    
  currentClip.copyAudio = False;
}


void UpdateFromFrame(int frame)
{
  char fromFrame[10];

  currentClip.fromFrame = frame;
  sprintf(fromFrame, "%d", frame);
  XmTextSetString(fromFrameText, fromFrame);
  currentClip.clipSize = EstimateClipSize(currentClip.fileName,
					  currentClip.fromFrame,
					  currentClip.toFrame,
					  currentClip.copyAudio);
}

void UpdateToFrame(int frame)
{
  char toFrame[10];

  currentClip.toFrame = frame;
  sprintf(toFrame, "%d", frame);
  XmTextSetString(toFrameText, toFrame);
  currentClip.clipSize = EstimateClipSize(currentClip.fileName,
					  currentClip.fromFrame,
					  currentClip.toFrame,
					  currentClip.copyAudio);
}


void AddMovieClip(void)
{
  /*
   * First, scan the text areas to see if they have changed.
   */
  FromFrameEntered(fromFrameText, (XtPointer)NULL, (XtPointer)NULL);
  ToFrameEntered(toFrameText, (XtPointer)NULL, (XtPointer)NULL);

  /*
   * Now, add the clip to the clip list.
   */
  AppendClipToList(currentClip);
}

void InsertMovieClip(void)
{
  /*
   * First, scan the text areas to see if they have changed.
   */
  FromFrameEntered(fromFrameText, (XtPointer)NULL, (XtPointer)NULL);
  ToFrameEntered(toFrameText, (XtPointer)NULL, (XtPointer)NULL);

  /*
   * Now, add the clip to the clip list.
   */
  InsertClipInList(currentClip);
}

void ModifyMovieClip(void)
{
  /*
   * First, scan the text areas to see if they have changed.
   */
  FromFrameEntered(fromFrameText, (XtPointer)NULL, (XtPointer)NULL);
  ToFrameEntered(toFrameText, (XtPointer)NULL, (XtPointer)NULL);

  ModifyClip(currentClip);
}



int GetClipLength(void)
{
  return(currentClip.movieLength);
}


int GetFromFrame(void)
{
  return(currentClip.fromFrame);
}


int GetToFrame(void)
{
  return(currentClip.toFrame);
}

char *GetMovieFilename(void)
{
  return(currentClip.fileName);
}


void InitializeSelectionBox(ClipListEntry clip)
{
  SelectMovieFileName(clip.fileName, True);
  if(clip.copyEntireMovie)
  {
    SetCopyEntireMovie();
  }
  else
  {
    SetCopySelectedFrames();
    UpdateFromFrame(clip.fromFrame);
    UpdateToFrame(clip.toFrame);
  }
  if(clip.copyAudio)
  {
    SetCopyAudio();
  }
  else
  {
    SetOmitAudio();
  }
}
