/* $Header: /plx/source/Xclients/generic/RCS/MovieTool.h,v 1.56 1996/01/25 04:16:21 drapeau Exp $ */
/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Log: MovieTool.h,v $
 * Revision 1.56  1996/01/25  04:16:21  drapeau
 * Added support for the PlxOutputCascade to automatically create the
 * output menu hierarchy, for enhanced flexibility.
 *
 * Revision 1.55  1996/01/05  21:26:24  drapeau
 * Added a feature to save a still image from an open movie as a JFIF
 * file.
 *
 * Revision 1.54  1995/11/23  03:41:04  drapeau
 * Added function prototypes and include files to eliminate compiler
 * warnings.
 *
 * Revision 1.53  1995/10/12  22:15:53  drapeau
 * Added support for a second, black-and-white logo.
 *
 * Revision 1.52  1995/10/08  05:15:03  drapeau
 * Updated reference to include file for the Motif Input/Output Cascade
 * interface code.
 *
 * Revision 1.51  1995/10/06  22:52:05  drapeau
 * Added support for showing the current input signal type in use.
 *
 * Revision 1.50  1995/10/02  21:50:11  drapeau
 * Added support for keeping track of the user interface's visibility
 * state: whether some panels are mapped or not, etc.
 *
 * Revision 1.49  1995/09/27  19:42:49  drapeau
 * Added a variable that indicates whether JPEG hardware is available.
 *
 * Revision 1.48  1995/08/23  01:45:45  drapeau
 * Added two variables to aid in sync and diagnostics.
 *
 * Revision 1.47  1995/08/18  02:06:01  drapeau
 * Added code to put the company logo in the application's "About..."
 * panel.
 *
 * Revision 1.46  1995/08/17  21:52:32  drapeau
 * Added support for an "About..." panel.
 *
 * Revision 1.45  1995/08/17  19:26:27  drapeau
 * Minor changes to help compilation on the AIX platform and remove
 * compiler warning messages about duplicate defined variables.
 *
 * Revision 1.44  1995/05/16  02:52:31  drapeau
 * Added support for zoom, especially as it affects the
 * imageQualityWidget.
 *
 * Revision 1.43  1995/05/10  20:27:30  lovvik
 * Added currentAudioObject which points to the currently
 * selected audio object.  Also added audioDeviceWidget[]
 * which stores an array of toggle button widgets that
 * represent the available audio modules.
 *
 * Revision 1.42  1995/05/09  18:57:26  lovvik
 * Added displayName.  This is a character string that
 * stores the display hostname.  It is used in the connection
 * to the audio server.
 *
 * Revision 1.41  1995/05/02  17:49:09  drapeau
 * Added support for hardware zoom when available.
 *
 * Revision 1.40  1995/04/26  23:53:26  lovvik
 * Added external declarations for AudioObjects:  audioObject,
 *   defaultAudioObject, alternateAudioObject1, alternateAudioObject2,
 *   and noAudioObject.
 *
 * Revision 1.39  1995/04/07  17:16:48  lovvik
 * Added display to the list of external variables.  Also added a
 * prototype for WMDeleteWindowCallback().
 *
 * Revision 1.38  1995/02/13  18:16:32  lovvik
 * Made some changes for the new audio module.  We no
 * longer use audio file descriptors in the main application,
 * so these global variables were removed:  audioControlFD,
 * audioFD, audioPlayFD, audioRecordFD.  Also, audioStatus
 * was added to replace the error detection functionality of
 * the audio file descriptors.
 *
 * Revision 1.37  1994/10/15  01:46:18  drapeau
 * Added code to complete MovieTool's notion of what operations should be
 * allowed under what conditions.  For example, input selection should
 * not be allowed during recording and playback.
 *
 * Revision 1.36  1994/10/14  02:37:23  drapeau
 * Add variables to describe whether output is available and function
 * prototypes to handle interface changes.
 *
 * Revision 1.35  1994/10/10  23:04:55  drapeau
 * Added variables to reflect the input signal standard, format, and
 * instance.
 *
 * Revision 1.34  1994/10/06  02:14:05  drapeau
 * Minor change: added a function to remove files from the filesystem.
 *
 * Revision 1.33  1994/09/30  16:56:08  drapeau
 * Added support for output.
 *
 * Revision 1.32  1994/09/13  08:05:50  drapeau
 * Declared a new global variable to handle the size of the imageQuality
 * widget's source rectangle, which tracks the size of the video input
 * widget being viewed.
 *
 * Revision 1.31  1994/08/27  07:11:53  drapeau
 * Removed an extraneous function prototype since it is now in the
 * include file "XPlxProtos.h".
 *
 * Revision 1.30  1994/08/23  01:29:45  drapeau
 * Added code for an alert dialog box.
 *
 * Revision 1.29  1994/08/19  05:59:35  drapeau
 * Minor change: added function prototype for Stop() so that it can
 * cleanly be called from other callback functions.
 *
 * Revision 1.28  1994/08/11  03:13:43  drapeau
 * Added a few new variables for better diagnostics as well as
 * app-specific X resources.
 *
 * Revision 1.27  1994/06/10  01:20:48  drapeau
 * Included errno.h for better error reporting.
 *
 * Revision 1.26  1994/05/31  19:54:05  drapeau
 * Added a number of variables to support better diagnostics and
 * automatic playback and quit for movies.  Also, changed types of some
 * variables used in synchronization code (changed from integer data
 * types to floating point data types).
 *
 * Revision 1.25  1994/01/20  22:51:36  drapeau
 * Added two variables that had previously been removed.  They were added
 * to support a slight change in the PortableAudio interface which was
 * made to accommodate the Sun audio device driver implementation, which
 * is flawed, in my opinion.
 *
 * Revision 1.24  1994/01/20  18:05:59  drapeau
 * Modified code to support the PortableAudio interface, increasing
 * portability across platforms.
 *
 * Also added new variables to support new features and removed obsolete
 * variables.
 *
 * Revision 1.23  1994/01/10  20:12:56  drapeau
 * Added support for the MAEstro Multimedia Authoring Environment.  Also
 * added a few new variables for better statistics tracking and
 * synchronization.
 *
 * Revision 1.22  1993/12/04  01:03:29  drapeau
 * Modified the code to make the Pause feature more robust.  Before this
 * version, pausing a movie during playback would throw off the
 * synchronization mechanism; pause now works correctly.
 *
 * Revision 1.21  1993/12/03  02:06:11  drapeau
 * Minor change to clean up RCS id info; revision numbering change made, too.
 *
 * Revision 1.2  1993/12/03  01:30:13  drapeau
 * Added declarations for new functions and variables.  Also added
 * copyright notice to the code.
 * */
static char movieToolHeaderRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/MovieTool.h,v 1.56 1996/01/25 04:16:21 drapeau Exp $";

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <Xm/Xm.h>
#include <X11/StringDefs.h>
#include <Xm/CascadeB.h>
#include <Xm/DialogS.h>
#include <Xm/FileSB.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/List.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/Scale.h>
#include <Xm/ScrollBar.h>
#include <Xm/SelectioB.h>
#include <Xm/TextF.h>
#include <Xm/Text.h>
#include <Xm/ToggleB.h>
#include <Xm/ToggleBG.h>
#include <Xm/LabelG.h>
#include <Xm/PushBG.h>
#include <Xm/SeparatoG.h>
#include <Xm/AtomMgr.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/param.h>
#include <sys/time.h>
#include <signal.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <X11/Xlib.h>
#include <XPlxExt.h>
#include <XVideoIn.h>
#include <JPEG.h>
#include "PortableAudio.h"
#include <movie.h>						    /* For JPEG Movie file definitions */
#include "externs.h"
#include <Sender.h>
#include <Receiver.h>
#include <errno.h>
#include <math.h>
#include <IOCascade.h>
#include <xpm.h>
#include <icon/.plxlogo>
#include <icon/plxSmallBWLogo.xpm>
#include <malloc.h>
#include <stdlib.h>

#define	Stopped		0					    /* Defined constants to describe the status of the... */
#define	Playing		1					    /* ...application for recording and playback. */
#define	Recording	2
#define	Paused		3

#define MovieModified	0					    /* Defined constants to indicate whether a movie file has... */
#define MovieNew	1					    /* ...been modified and needs saving, or is empty (not... */
#define MovieOK		2					    /* ...yet been recorded), or has no changes made to it. */

#define MaxMovieDurationInFrames	(30 * 3600)		    /* Enough frames for one hour of NTSC video */
#define HighQualitySamplingRate		44100
#define OneImageSizeInBytes		(1024 * 256)		    /* Define maximum size to allocate for one JPEG image */

#define XtNaudioDelay	"audioDelay"
#define XtNaudioDevice	"audioDevice"
#define XtNboard	"board"
#define XtNchannels	"channels"
#define XtNsampleRate	"sampleRate"
#define XtNautoPlay	"autoPlay"
#define XtNautoQuit	"autoQuit"
#define XtNsilent	"silent"
#define XtNautoRepeat	"autoRepeat"
#define XtNhelp		"help"
#define XtNhostname	"hostname"

#define XtCAudioDelay	"AudioDelay"
#define XtCAudioDevice	"AudioDevice"
#define XtCBoard	"Board"
#define XtCChannels	"Channels"
#define XtCSampleRate	"SampleRate"
#define XtCAutoPlay	"AutoPlay"
#define XtCAutoQuit	"AutoQuit"
#define XtCSilent	"Silent"
#define XtCAutoRepeat	"AutoRepeat"
#define XtCHelp		"Help"
#define XtCHostname	"Hostname"

typedef struct _AppResources					    /* X resources specific to MovieTool */
{
  int		audioDelay;					    /* Number of frames to make audio lag behind video */
  char *	audioDevice;					    /* Name of audio object */
  char *	hostname;					    /* Hostname to register with for MAEstro messaging */
  int		board;						    /* Which board to use if multiple boards are available */
  int		channels;					    /* Number of audio channels to record (1 or 2) */
  int		sampleRate;					    /* Audio sampling rate, in Hertz */
  Boolean	diagnostics;					    /* Whether to print diagnostic info for record & playback */
  Boolean	autoPlay;					    /* Whether to play the movie on the cmd line immediately */
  Boolean	autoQuit;					    /* Whether to quit MovieTool after playing the movie */
  Boolean	silent;						    /* Whether to record audio along with video */
  Boolean	autoRepeat;					    /* Whether to play the movie over and over */
  Boolean	help;						    /* Whether to print a help message */
} AppResources;


typedef struct _MTPanelState					    /* Structure holds state about each shell, whether it is... */
{								    /* ...visible or not (mapped, managed). */
  Boolean	mainWindow;
  Boolean	audioShell;
  Boolean	inputSettingsShell;
  Boolean	input1Shell;
  Boolean	fileDialog;
  Boolean	smartMovieShell;
  Boolean	movieShell;
  Boolean	alertShell;
  Boolean	aboutShell;
} MTPanelState;


extern Widget		imageQualityWidget;			    /* Externs outside of XDesigner variables */
extern Widget		input1Widget;
extern Widget		playbackWidget;
extern jpheader		movieHeader;
extern int*		frameLocation;
extern int*		recordFrameIndex;
extern float*		skippedFrameIndex;
extern float*		recordSkippedFrameIndex;
extern char*		videoFrameIndex;
extern long*		audioReadTime;
extern long*		audioBetweenTime;
extern long*		audioWriteTime;
extern long*		compressTime;
extern long*		diskWriteTime;
extern long*		playInterval;
extern int		movieToolStatus;
extern XPlxCImage*	jpegImage;
extern XPlxCImage*	imageQualityImage;
extern char*		jpegData;
extern FILE*		movieFile;
extern int		currentFrame;
extern XtWorkProcId	playbackWorkProcID;
extern char*		audioData;
extern XtAppContext	app_context;
extern Display*         display;
extern int		signalFrameRate;
extern int		outputFile;
extern int		audioBytesPerFrame;
extern int		recordAudioFlag;
extern int		audioBandwidth;
extern PortableAudio	audioInfo;
extern AudioObject      audioObject;
extern AudioObject      defaultAudioObject;
extern AudioObject      alternateAudioObject1;
extern AudioObject      alternateAudioObject2;
extern AudioObject      noAudioObject;
extern AudioObject      *currentAudioObject;
extern Widget audioDeviceWidget[5];
extern AudioModuleMapEntry    audioMap[5];
extern jpheader		recordHeader;
extern XtWorkProcId	recordWorkProcID;
extern char		outputFileName[MAXPATHLEN];
extern char*		currentMovieFileName;
extern char		titleString[MAXPATHLEN + 64];  
extern int		totalBytesInRecording;
extern XRectangle	recordSourceRect;
extern Boolean		resetPlaybackClock;
extern struct timeval	playbackStartTime;
extern struct timeval	recordStartTime;
extern int		playbackStartFrame;
extern int		playbackEndFrame;
extern int		framesSkippedDuringPlayback;
extern int		framesSkippedDuringRecording;
extern int		recordFrameRate;
extern long		totalTimePaused;
extern Boolean		printDiagnostics;
extern int		audioDelayInFrames;
extern int		firstFrame;
extern int		lastFrame;
extern char		diagString[MAXPATHLEN + 64];
extern Sender*		sender;
extern Receiver*	receiver;
extern XtInputId	maestroFD;
extern int		movieStatus;
extern Audio_hdr	audioPlayHeader;
extern Audio_hdr	audioRecordHeader;
extern Boolean		silentMovie;
extern int		sampleRate;
extern int		numAudioChannels;
extern int		playbackSpeed;
extern Boolean		usingSharedMemory;
extern int              audioStatus;
extern char*		fileFormat;
extern Boolean		monitorStatus;
extern int		lastMonitorVolume;
extern Boolean		userTurnedOffMonitor;
extern Boolean		autoPlay;
extern Boolean		autoQuit;
extern Boolean		startupMovie;
extern Boolean		autoRepeat;
extern char*		startupMovieName;
extern char*		timecodeString;
extern char*		errorMessage;
extern XRectangle	imageQualitySourceRect;
extern XRectangle	imageQualityDisplaySourceRect;
extern XRectangle	imageQualityRect;
extern Widget		outputWidget;
extern Boolean		outputOn;
extern int		outputSignalStandard;
extern int		outputSignalWidth;
extern int		outputSignalHeight;
extern int		outputSignalFormat;
extern GC		outputGC;
extern int		inputSignalStandard;
extern int		inputSignalFormat;
extern int		inputInstance;
extern Boolean		outputAvailable;
extern int		zoomFactor;
extern XRectangle	playDestRect;
extern char*            displayName;
extern PlxInputCascade*	inputsCascade;			    /* Automatically generated for this hardware */
extern Widget		inputMenuCascade;
extern float		totalTimeLost;
extern Boolean		jpegAvailable;
extern MTPanelState	mtPanelState;
extern int		syncSlack;
extern PlxOutputCascade*	outputMenuCascade;			    /* Automatically generated for this hardware */

void		UpdateTimecodeDisplay(int);
void		OpenMovie(Widget, XtPointer, XtPointer);
void		CheckMAEstroMessages(XtPointer, int*, XtInputId*);
void		SetMovieStatus(int);
void		SaveMPEGFile(char*);
void		SaveMovie(Widget, XtPointer, XtPointer);
void		CompactMovie(Widget, XtPointer, XtPointer);
void		PrintDiagnostics(char*);
void		Stop(Widget, XtPointer, XtPointer);
long		ElapsedTime(struct timeval*, struct timeval*);
void		ResizeOutput(void);
void		RemoveMovie(void*, char*);
void		BuildInputMenu(void);
void		UpdateMovieLengthIndicators(void);
void		EnableJPEGCompressionControls(void);
void		DisableJPEGCompressionControls(void);
void            WMDeleteWindowCallback(Widget, XtPointer, XtPointer);
void		ShowInput(Widget, XtPointer, XtPointer);
void		SyncIODevice(void);
void		UpdateInputTypeString(signalSpec*, Boolean);
void		SetSyncSlack(Widget, XtPointer, XtPointer);
void		getDisplayName(int, char**);
void		InitMTPanelState(MTPanelState*);
void		SetMTPanelState(MTPanelState*, Widget, Boolean);
void		InitializeXVideo(void);
void		InitializeInterface(void);
void		InitMAEstroMessaging(void);
void		PrintErrorMessage(char*);
void		EnableAudioDeviceControls(void);
void		EnableTransportButtons(void);
void		DeSensitizePlaybackControls(void);
void		DisableAudioDeviceControls(void);
void		UpdateTotalTimePaused(Boolean);
void		PrintMovieDiagnostics(char*, jpheader*);
void		XDpopup_link(Widget, XtPointer, XtPointer);
void		StartOfMovie(Widget, XtPointer, XtPointer);
Boolean		PlayOneFrame(int, Boolean, Boolean);
void		Play(Widget, XtPointer, XtPointer);
void		Pause(Widget, XtPointer, XtPointer);
void		ChangeRecordSpeed(Widget, XtPointer, XtPointer);
void		SaveStill(Widget, XtPointer, XtPointer);
void		SetOutputSignalType(Widget, XtPointer, XtPointer);
Boolean		OutputWorkProc(XtPointer);
