/* $Header: /plx/source/Xclients/generic/RCS/PortableAudio.c,v 1.6 1995/11/23 03:42:09 drapeau Exp $ */
/*
 *   Copyright (c) 1994 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */
/* $Log: PortableAudio.c,v $
 * Revision 1.6  1995/11/23  03:42:09  drapeau
 * Minor changes to eliminate compiler warnings.
 *
 * Revision 1.5  1995/08/23  01:47:42  drapeau
 * Added two methods to the object: AudioOverrun(), which queries the
 * audio device as to whether audio overflow occurred (i.e., if the audio
 * device was filling up faster than the application could pull out audio
 * data); and AudioBytesInBuffer(), which asks the audio device how many
 * bytes are left in the audio buffer.
 *
 * Revision 1.4  1995/05/12  00:09:38  lovvik
 * Added a callback for each toggle buttons created in
 * createAudioDeviceButtons().
 *
 * Revision 1.3  1995/05/10  20:21:07  lovvik
 * Added functions createAudioDeviceButtons() and
 * setAudioDeviceButton().  createAudioDeviceButtons()
 * creates a toggle button for each supported audio module.
 * setAudioDeviceButton() sets the currently selected
 * audio device toggle button.
 *
 * Revision 1.2  1995/05/09  19:06:42  lovvik
 * Fixed a bug. getAudioObject() would do a string
 * compare between audioObjectName and the entries
 * in audioMap.  This would cause a comparison between
 * a NULL and a string if the audio device was not
 * specified on the command line.
 *
 * Revision 1.1  1995/04/27  00:00:05  lovvik
 * Initial revision
 * */
static char portableAudioRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/PortableAudio.c,v 1.6 1995/11/23 03:42:09 drapeau Exp $";

#include "MovieTool.h"
#include "PortableAudio.h"
#include <Xm/ToggleB.h>

void ChangeAudioModule(Widget, XtPointer, XtPointer);

void createAudioDeviceButtons(void)
{
  int index;
  XmString buttonLabel;

  index = 0;
  while(audioMap[index].audioObject != NULL)
  {
    buttonLabel = XmStringCreateLtoR(audioMap[index].audioDeviceLabelName, XmFONTLIST_DEFAULT_TAG);
    audioDeviceWidget[index] = 
      XtVaCreateManagedWidget(audioMap[index].audioDeviceName,
			      xmToggleButtonWidgetClass,
			      audioDeviceRadioBox,
			      XmNlabelString, buttonLabel,
			      XmNset, (index == 0),
			      NULL);
    XmStringFree(buttonLabel);
    XtAddCallback(audioDeviceWidget[index],
		  XmNvalueChangedCallback,
		  ChangeAudioModule,
		  (XtPointer)&index);
    index++;
  }
			    
}

void setAudioDeviceButton(void)
{
  int index;

  index = 0;
  while(audioMap[index].audioObject != NULL)
  {
    XmToggleButtonSetState(audioDeviceWidget[index],
			   (audioMap[index].audioObject == currentAudioObject), 
			   False);
    index++;
  }
}

AudioObject * getAudioObject(char *audioObjectName)
{
  int index = 0;

  while((audioMap[index].audioObject != NULL) && (audioObjectName != NULL))
  {
    if(strcmp(audioObjectName, audioMap[index].audioDeviceName) == 0)
    {
      return(audioMap[index].audioObject);
    }
    index ++;
  }
  /* audioObjectName not found in the map, use the default audio module */
  return(audioMap[0].audioObject);
}

void initializeAudioObject(AudioObject *selectedAudioObject)
{
  currentAudioObject = selectedAudioObject;
  
  if(selectedAudioObject->AudioWrite != NULL)
  {
    audioObject.AudioWrite = selectedAudioObject->AudioWrite;
  }
  if(selectedAudioObject->AudioRead != NULL)
  {
    audioObject.AudioRead = selectedAudioObject->AudioRead;
  }
  if(selectedAudioObject->AudioPlayConnect != NULL)
  {
    audioObject.AudioPlayConnect = selectedAudioObject->AudioPlayConnect;
  }
  if(selectedAudioObject->AudioPlayDisconnect != NULL)
  {
    audioObject.AudioPlayDisconnect = selectedAudioObject->AudioPlayDisconnect;
  }
  if(selectedAudioObject->AudioRecordConnect != NULL)
  {
    audioObject.AudioRecordConnect = selectedAudioObject->AudioRecordConnect;
  }
  if(selectedAudioObject->AudioRecordDisconnect != NULL)
  {
    audioObject.AudioRecordDisconnect = selectedAudioObject->AudioRecordDisconnect;
  }
  if(selectedAudioObject->AudioRecordPause != NULL)
  {
    audioObject.AudioRecordPause = selectedAudioObject->AudioRecordPause;
  }
  if(selectedAudioObject->AudioRecordResume != NULL)
  {
    audioObject.AudioRecordResume = selectedAudioObject->AudioRecordResume;
  }
  if(selectedAudioObject->AudioPlayPause != NULL)
  {
    audioObject.AudioPlayPause = selectedAudioObject->AudioPlayPause;
  }
  if(selectedAudioObject->AudioPlayResume != NULL)
  {
    audioObject.AudioPlayResume = selectedAudioObject->AudioPlayResume;
  }
  if(selectedAudioObject->PrintAudioInfo != NULL)
  {
    audioObject.PrintAudioInfo = selectedAudioObject->PrintAudioInfo;
  }
  if(selectedAudioObject->InitializeAudio != NULL)
  {
    audioObject.InitializeAudio = selectedAudioObject->InitializeAudio;
  }
  if(selectedAudioObject->SetAudioPlayVolume != NULL)
  {
    audioObject.SetAudioPlayVolume = selectedAudioObject->SetAudioPlayVolume;
  }
  if(selectedAudioObject->SetAudioRecordVolume != NULL)
  {
    audioObject.SetAudioRecordVolume = selectedAudioObject->SetAudioRecordVolume;
  }
  if(selectedAudioObject->SetAudioMonitorVolume != NULL)
  {
    audioObject.SetAudioMonitorVolume = selectedAudioObject->SetAudioMonitorVolume;
  }
  if(selectedAudioObject->SetPlaybackDevice != NULL)
  {
    audioObject.SetPlaybackDevice = selectedAudioObject->SetPlaybackDevice;
  }
  if(selectedAudioObject->SetRecordDevice != NULL)
  {
    audioObject.SetRecordDevice = selectedAudioObject->SetRecordDevice;
  }
  if(selectedAudioObject->SetAudioPlaybackConfiguration != NULL)
  {
    audioObject.SetAudioPlaybackConfiguration = selectedAudioObject->SetAudioPlaybackConfiguration;
  }
  if(selectedAudioObject->AudioClose != NULL)
  {
    audioObject.AudioClose = selectedAudioObject->AudioClose;
  }
  if(selectedAudioObject->SyncAudioPlaybackDevice != NULL)
  {
    audioObject.SyncAudioPlaybackDevice = selectedAudioObject->SyncAudioPlaybackDevice;
  }
  if(selectedAudioObject->SyncAudioRecordDevice != NULL)
  {
    audioObject.SyncAudioRecordDevice = selectedAudioObject->SyncAudioRecordDevice;
  }
  if(selectedAudioObject->AudioStopPlayDevice != NULL)
  {
    audioObject.AudioStopPlayDevice = selectedAudioObject->AudioStopPlayDevice;
  }
  if(selectedAudioObject->AudioStopRecordDevice != NULL)
  {
    audioObject.AudioStopRecordDevice = selectedAudioObject->AudioStopRecordDevice;
  }
  if(selectedAudioObject->AudioReadOverrun != NULL)
  {
    audioObject.AudioReadOverrun = selectedAudioObject->AudioReadOverrun;
  }
  if(selectedAudioObject->AudioBytesInBuffer != NULL)
  {
    audioObject.AudioBytesInBuffer = selectedAudioObject->AudioBytesInBuffer;
  }
} /* end function initializeAudioObject */





int AudioWrite(char *audioData, int audioslice)
{
  return(audioObject.AudioWrite(audioData, audioslice));
}

int AudioRead (char *audioData, int audioslice)
{
  return(audioObject.AudioRead(audioData, audioslice));
}

void AudioPlayConnect(void)
{
  audioObject.AudioPlayConnect();
}

void AudioPlayDisconnect(void)
{
  audioObject.AudioPlayDisconnect();
}

void AudioRecordConnect(void)
{
  audioObject.AudioRecordConnect();
}

void AudioRecordDisconnect(void)
{
  audioObject.AudioRecordDisconnect();
}

void AudioRecordPause(void)
{
  audioObject.AudioRecordPause();
}

void AudioRecordResume(void)
{
  audioObject.AudioRecordResume();
}

void AudioPlayPause(void)
{
  audioObject.AudioPlayPause();
}

void AudioPlayResume(void)
{
  audioObject.AudioPlayResume();
}

void PrintAudioInfo(PortableAudio *audioInfo)
{
  audioObject.PrintAudioInfo(audioInfo);
}

int InitializeAudio(void)
{
  return(audioObject.InitializeAudio());
}

void SetAudioPlayVolume(int percent)
{
  audioObject.SetAudioPlayVolume(percent);
}

void SetAudioRecordVolume(int percent)
{
  audioObject.SetAudioRecordVolume(percent);
}

void SetAudioMonitorVolume(int percent)
{
  audioObject.SetAudioMonitorVolume(percent);
}

void SetPlaybackDevice(int deviceType)
{
  audioObject.SetPlaybackDevice(deviceType);
}

void SetRecordDevice(int deviceType)
{
  audioObject.SetRecordDevice(deviceType);
}

int SetAudioPlaybackConfiguration(Audio_hdr * audioInfo)
{
  return(audioObject.SetAudioPlaybackConfiguration(audioInfo));
}

void AudioClose(void)
{
  audioObject.AudioClose();
}

void SyncAudioPlaybackDevice(void)
{
  audioObject.SyncAudioPlaybackDevice();
}

void SyncAudioRecordDevice(void)
{
  audioObject.SyncAudioRecordDevice();
}

void AudioStopPlayDevice(void)
{
  audioObject.AudioStopPlayDevice();
}

void AudioStopRecordDevice(void)
{
  audioObject.AudioStopRecordDevice();
}



Boolean AudioReadOverrun(void)
{
  return(audioObject.AudioReadOverrun());
}


long AudioBytesInBuffer(void)
{
  return(audioObject.AudioBytesInBuffer());
}
