/* $Header: /plx/source/Xclients/generic/RCS/PortableAudio.h,v 0.100 1995/11/23 03:42:34 drapeau Exp $ */
/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Log: PortableAudio.h,v $
 * Revision 0.100  1995/11/23  03:42:34  drapeau
 * Added function prototypes for this "object" to eliminate compiler
 * warnings.
 *
 * Revision 0.99  1995/08/23  01:48:17  drapeau
 * Added two methods to the object: AudioOverrun(), which queries the
 * audio device as to whether audio overflow occurred (i.e., if the audio
 * device was filling up faster than the application could pull out audio
 * data); and AudioBytesInBuffer(), which asks the audio device how many
 * bytes are left in the audio buffer.
 *
 * Revision 0.98  1995/06/16  19:57:26  drapeau
 * Added support for compilation under AIX.
 *
 * Revision 0.97  1995/05/12  00:12:18  lovvik
 * Added a new field in the structure AudioModuleMapEntry.
 * The field is called audioDeviceLabelName.  This field
 * is used to store a string that will be used as a label
 * for the audio object.
 *
 * Revision 0.96  1995/04/27  00:05:56  lovvik
 * Two additions:
 *   1)  Added an AudioObject structure.
 *   2)  Added an AudioMapEntry structure.
 * These structures are used to provide the application a set of possible
 * audio modules instead of only one.
 *
 * Revision 0.95  1995/02/13  18:18:39  lovvik
 * Modified the prototypes PrintAudioInfo() and
 * SetRecordDevice() to reflect the changes to the
 * audio module.  These functions no longer have
 * the audio file descriptor in the paramater list.
 *
 * Revision 0.94  1994/09/23  06:42:15  drapeau
 * Fixed the endif at the end of this file to eliminate annoying compiler
 * warnings.
 *
 * Revision 0.93  1994/06/15  03:26:10  drapeau
 * Added Sun-specific defined constants to enhance portability with older
 * programs that generate and read JPEG Movie files.
 *
 * Revision 0.92  1994/05/31  19:58:16  drapeau
 * Added new defined constants for the type of encoding used to digitize
 * audio.
 *
 * Revision 0.91  1994/01/20  22:52:26  drapeau
 * Added a couple of defined constants and a prototype to support
 * recording options.
 *
 * Revision 0.90  1994/01/20  18:02:24  drapeau
 * First working version.
 * */
#ifndef _PortableAudioHeader_
#define _PortableAudioHeader_
static char portableAudioHeaderRcsId[] = "$Header: /plx/source/Xclients/generic/RCS/PortableAudio.h,v 0.100 1995/11/23 03:42:34 drapeau Exp $";

#if defined(__hpux) || (defined(AIXV3) || defined(ibm))
#include <sys/audio.h>
#include "hpAudio.h"
#else
  #include <multimedia/libaudio.h>
  #include <multimedia/audio_device.h>
  #ifdef SVR4
    #include <multimedia/audio_encode.h>
  #else
    #include <multimedia/ulaw2linear.h>
  #endif
#endif

/* Here are some Sun-specific audio encoding defined constants. */
/* They are used for now in order to preserve compatibility... */
/* ...older versions of JPEG Movie recorders/players. */
/* This really should be eliminated and a cross-platform... */
/* ...audio standard should be defined once and for all. */

#ifndef AUDIO_ENCODING_ULAW
#define	AUDIO_ENCODING_ULAW	(1)	/* u-law encoding */
#endif
#ifndef AUDIO_ENCODING_ALAW
#define	AUDIO_ENCODING_ALAW	(2)	/* A-law encoding */
#endif
#ifndef AUDIO_ENCODING_LINEAR
#define	AUDIO_ENCODING_LINEAR	(3)	/* Linear PCM encoding */
#endif

#define AudioSpeaker	0
#define AudioHeadphone	1
#define AudioMicrophone 2
#define AudioLineIn	3
#define AudioLineOut	4

#define AudioULawEncoding	1
#define AudioALawEncoding	2
#define AudioLinearEncoding	3

typedef struct _PortableAudioPlayRecord
{
  int	sampleRate;
  int	channels;
  int	bitsPerSample;
  int	encoding;
  int	volume;
} PortableAudioPlayRecord;

typedef struct _PortableAudio
{
  PortableAudioPlayRecord	play;
  PortableAudioPlayRecord	record;
  int				monitorVolume;
} PortableAudio;

typedef struct _AudioObject
{
  int  (*AudioWrite)(char *, int);
  int  (*AudioRead)(char *, int);
  void (*AudioPlayConnect)(void);
  void (*AudioPlayDisconnect)(void);
  void (*AudioRecordConnect)(void);
  void (*AudioRecordDisconnect)(void);
  void (*AudioRecordPause)(void);
  void (*AudioRecordResume)(void);
  void (*AudioPlayPause)(void);
  void (*AudioPlayResume)(void);
  void (*PrintAudioInfo)(PortableAudio *);
  int  (*InitializeAudio)(void);
  void (*SetAudioPlayVolume)(int);
  void (*SetAudioRecordVolume)(int);
  void (*SetAudioMonitorVolume)(int);
  void (*SetPlaybackDevice)(int);
  void (*SetRecordDevice)(int);
  int  (*SetAudioPlaybackConfiguration)(Audio_hdr *);
  void (*AudioClose)(void);
  void (*SyncAudioPlaybackDevice)(void);
  void (*SyncAudioRecordDevice)(void);
  void (*AudioStopPlayDevice)(void);
  void (*AudioStopRecordDevice)(void);
  Boolean (*AudioReadOverrun)(void);
  long (*AudioBytesInBuffer)(void);
} AudioObject;


typedef struct _AudioModuleMapEntry
{
  char *audioDeviceName;
  char *audioDeviceLabelName;
  char *audioDeviceHelp;
  AudioObject *audioObject;
} AudioModuleMapEntry;

void		SetRecordDevice(int);
void		createAudioDeviceButtons(void);
void		initializeAudioObject(AudioObject*);
AudioObject*	getAudioObject(char*);
void		setAudioDeviceButton(void);
void		SetAudioMonitorVolume(int);
void		AudioClose(void);
int		AudioWrite(char*, int);
int		AudioRead(char*, int);
void		AudioPlayConnect(void);
void		AudioPlayDisconnect(void);
void		AudioRecordConnect(void);
void		AudioRecordDisconnect(void);
void		AudioRecordPause(void);
void		AudioRecordResume(void);
void		AudioPlayPause(void);
void		AudioPlayResume(void);
void		PrintAudioInfo(PortableAudio*);
int		InitializeAudio(void);
void		SetAudioPlayVolume(int);
void		SetAudioRecordVolume(int);
void		SetAudioMonitorVolume(int);
void		SetPlaybackDevice(int);
void		SetRecordDevice(int);
int		SetAudioPlaybackConfiguration(Audio_hdr*);
void		AudioClose(void);
void		SyncAudioPlaybackDevice(void);
void		SyncAudioRecordDevice(void);
void		AudioStopPlayDevice(void);
void		AudioStopRecordDevice(void);
Boolean		AudioReadOverrun(void);
long		AudioBytesInBuffer(void);

#endif /* _PortableAudioHeader_ */
