/* $Header: /plx/source/Xclients/generic/RCS/aixAudio.c,v 1.2 1995/08/23 01:51:29 drapeau Exp $ */
/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Dave Arnold
 */
/* $Log: aixAudio.c,v $
 * Revision 1.2  1995/08/23  01:51:29  drapeau
 * Fixed some errors in the way audio was being processed and configured.
 * Also, changed the behavior of the audio device to handle monitoring of
 * audio more immediately.
 *
 * Also, added methods to the device to determine whether audio overflow
 * occurred.
 *
 * Revision 1.1  1995/08/17  19:30:36  drapeau
 * Added a number of debugging messages that are, for now, commented out.
 *
 * Revision 1.0  1995/06/16  19:57:44  drapeau
 * Initial revision
 * */
static char aixAudioRcsId[] = "$Header: /plx/source/Xclients/generic/RCS/aixAudio.c,v 1.2 1995/08/23 01:51:29 drapeau Exp $";

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <time.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <errno.h>
#include "MovieTool.h"
#include "PortableAudio.h"
#include "aixAudio.h"

#include <UMSAudioDevice.h>

#ifndef _UMS_STRINGS_H
#include <UMSStrings.h>
#endif

static UMSAudioDeviceMClass audio_device_class;
static UMSAudioDeviceMClass paudio_device_class;
static UMSAudioDevice audio_device;
static UMSAudioDevice paudio_device;
static UMSAudioDevice_ReturnCode rc;
static int readBuffSize;

static Environment *ev;

static UMSAudioTypes_Buffer buffer;

#define DIVISOR 60

/* #define MUNGERATE(a) (((a) * 359) / 1000) */

/* #define MUNGERATE(a) 8000 */

static char *current_input;
static char *current_output;

AudioObject  defaultAudioObject =
{
  IBMAudioDeviceAudioWrite,
  IBMAudioDeviceAudioRead,
  IBMAudioDeviceAudioPlayConnect,
  IBMAudioDeviceAudioPlayDisconnect,
  IBMAudioDeviceAudioRecordConnect,
  IBMAudioDeviceAudioRecordDisconnect,
  IBMAudioDeviceAudioRecordPause,
  IBMAudioDeviceAudioRecordResume,
  IBMAudioDeviceAudioPlayPause,
  IBMAudioDeviceAudioPlayResume,
  IBMAudioDevicePrintAudioInfo,
  IBMAudioDeviceInitializeAudio,
  IBMAudioDeviceSetAudioPlayVolume,
  IBMAudioDeviceSetAudioRecordVolume,
  IBMAudioDeviceSetAudioMonitorVolume,
  IBMAudioDeviceSetPlaybackDevice,
  IBMAudioDeviceSetRecordDevice,
  IBMAudioDeviceSetAudioPlaybackConfiguration,
  IBMAudioDeviceAudioClose,
  IBMAudioDeviceSyncAudioPlaybackDevice,
  IBMAudioDeviceSyncAudioRecordDevice,
  IBMAudioDeviceAudioStopPlayDevice,
  IBMAudioDeviceAudioStopRecordDevice,
  IBMAudioDeviceAudioReadOverrun,
  IBMAudioDeviceAudioReadUsed
};

long AudioPosition()
{
    long pos;
    rc = UMSAudioDevice_get_position(audio_device, ev, &pos);
    return pos;
}

Boolean IBMAudioDeviceAudioReadOverrun()
{
    long ret;
    rc = UMSAudioDevice_read_buff_remain(audio_device, ev, &ret);
    if((ret == 0) || (rc == UMSAudioDevice_OverRun))
	return True;
    return False;
}

long IBMAudioDeviceAudioReadUsed()
{
    long ret;
    rc = UMSAudioDevice_read_buff_used(audio_device, ev, &ret);
    return (ret * audioRecordHeader.bytes_per_unit);
}

int IBMAudioDeviceAudioWrite(char *audioData, int audioslice)	    /* Write audio data to the audio device */
{
  long samples_written, ret;
  long cpose;
  int samples_to_write;
  int bytes_per_sample;
  int bytes_left;
  octet *p;
  
  bytes_per_sample = audioRecordHeader.bytes_per_unit;
  samples_to_write = audioslice / bytes_per_sample;
  p = audioData;
  samples_written = 0;
  bytes_left = audioslice;
  do
  {
    buffer._buffer = p;
    buffer._length = 0;
    buffer._maximum = bytes_left;
    rc = UMSAudioDevice_write(paudio_device, ev,&buffer, samples_to_write - samples_written, &ret);
    samples_written += ret;
    p += (ret * bytes_per_sample);
    bytes_left -= (ret * bytes_per_sample);
  } while (samples_written < samples_to_write);
  return(audioslice);
} /* end function IBMAudioDeviceAudioWrite */



int IBMAudioDeviceAudioRead(char*	audioData,		    /* Read the audio information from the audio device */
			    int		audioBytesPerFrame)
{
  int bytes_per_sample, samples_to_read;
  long ret;
  
  bytes_per_sample = audioRecordHeader.bytes_per_unit;
  samples_to_read = audioBytesPerFrame / bytes_per_sample;
  buffer._buffer = audioData;
  buffer._length = 0;
  buffer._maximum = audioBytesPerFrame;
  rc = UMSAudioDevice_read(audio_device, ev, &buffer, samples_to_read, &ret);
  return(ret * bytes_per_sample);
} /* end function IBMAudioDeviceAudioRead */

static int audio_is_recording = 0;
static int audio_is_playing = 0;
static int record_volume = 50;
static int monitor_volume = 50;
static int play_volume = 50;



void IBMAudioDeviceAudioRecordPause(void)
{
/*  printf("Debug: RecordPause().\n");*/
  rc = UMSAudioDevice_pause(audio_device, ev);
} /* end function IBMAudioDeviceAudioRecordPause */



void IBMAudioDeviceAudioRecordResume(void)
{
  long tval;
/*  printf("Debug: RecordResume().\n");*/
#if 0
/*  printf("0000000000000----Inside in if zero.\n");*/
  ioctl(audioFD, AUDIO_RESUME, AUDIO_RECEIVE);
#else
  if(audio_is_playing)
  {
/*    printf("Debug: RecordPause() --audio_is_playing.\n");*/
    audio_is_playing = 0;
    rc = UMSAudioDevice_stop(paudio_device, ev);
  }
  if(audio_is_recording)
  {
/*    printf("Debug: RecordPause() --audio_is_recording.\n");*/
    rc = UMSAudioDevice_stop(audio_device, ev);
#if 0
/*  printf("0000000000000----Inside in if zero.\n");*/
    rc = UMSAudioDevice_initialize(audio_device, ev);
    rc = UMSAudioDevice_set_audio_buffer_size(audio_device, ev, readBuffSize, &tval);
    rc = UMSAudioDevice_resume(audio_device, ev);
#endif
  }
#if 0
  else
#endif
  {
/*    printf("Debug: RecordResume() --audio_is_neither_playing_nor_recording.\n");*/
    audio_is_recording = 1;
    if(monitor_volume)
    {
      rc = UMSAudioDevice_set_volume(audio_device, ev, monitor_volume);
      rc = UMSAudioDevice_set_monitor(audio_device, ev, True);
    }
    else
      rc = UMSAudioDevice_set_monitor(audio_device, ev, False);
    rc = UMSAudioDevice_initialize(audio_device, ev);
    rc = UMSAudioDevice_set_audio_buffer_size(audio_device, ev, readBuffSize, &tval);
    rc = UMSAudioDevice_start(audio_device, ev);
  }
#endif
} /* end function IBMAudioDeviceAudioRecordResume */



void IBMAudioDeviceAudioPlayPause(void)
{
/*  printf("Debug: PlayPause().\n");*/
  rc = UMSAudioDevice_pause(paudio_device, ev);
} /* end function IBMAudioDeviceAudioPlayPause */



void IBMAudioDeviceAudioPlayResume(void)
{
  long tval;
/*  printf("Debug: PlayResume().\n");*/
#if 0
/*  printf("0000000000000----Inside in if zero.\n");*/
  ioctl(audioFD, AUDIO_RESUME, AUDIO_TRANSMIT);
#else
  if(audio_is_recording)
  {
/*    printf("Debug: PlayResume() -- audio_is_recording.\n");*/
    audio_is_recording = 0;
    rc = UMSAudioDevice_stop(audio_device, ev);
#if 1
    rc = UMSAudioDevice_set_monitor(audio_device, ev, False);
    rc = UMSAudioDevice_initialize(audio_device, ev);
    rc = UMSAudioDevice_set_audio_buffer_size(audio_device, ev, readBuffSize, &tval);
#endif
  }
  if(audio_is_playing)
  {
/*    printf("Debug: PlayResume() -- audio_is_playing.\n");*/
#if 0
    rc = UMSAudioDevice_initialize(paudio_device, ev);
    rc = UMSAudioDevice_set_audio_buffer_size(paudio_device, ev, readBuffSize, &tval);
#endif
    rc = UMSAudioDevice_set_volume(paudio_device, ev, play_volume);
    rc = UMSAudioDevice_resume(paudio_device, ev);
  }
  else
  {
/*    printf("Debug: PlayResume() -- audio_is_neither_playing_nor_recording.\n");*/
    audio_is_playing = 1;
    rc = UMSAudioDevice_set_volume(paudio_device, ev, play_volume);
    rc = UMSAudioDevice_initialize(paudio_device, ev);
    rc = UMSAudioDevice_set_audio_buffer_size(paudio_device, ev, readBuffSize, &tval);
    rc = UMSAudioDevice_start(paudio_device, ev);
  }
	
#endif
} /* end function IBMAudioDeviceAudioPlayResume */



void IBMAudioDevicePrintAudioInfo(PortableAudio*	audioInfo)
{
  long tVal;
  char *tString;
  PrintDiagnostics("Entered PrintAudioInfo.\n");
  
  rc = UMSAudioDevice_get_sample_rate(audio_device, ev, (long*)&sampleRate);
  audioInfo->play.sampleRate = sampleRate;
  audioInfo->record.sampleRate = sampleRate;
  sprintf(diagString, "PrintAudioInfo(): Highest alleged sample rate is %d.\n",
		  sampleRate);
  PrintDiagnostics(diagString);
  rc = UMSAudioDevice_get_number_of_channels(paudio_device, ev, &tVal);
  audioInfo->play.channels = tVal;
  rc = UMSAudioDevice_get_number_of_channels(audio_device, ev, &tVal);
  audioInfo->record.channels = tVal;
  rc = UMSAudioDevice_get_bits_per_sample(paudio_device, ev, &tVal);
  audioInfo->play.bitsPerSample = tVal;
  rc = UMSAudioDevice_get_bits_per_sample(audio_device, ev, &tVal);
  audioInfo->record.bitsPerSample = tVal;
  
  rc = UMSAudioDevice_get_audio_format_type(audio_device, ev, &tString);
  if(strcmp(tString, "MU_LAW") == 0)
  {
    PrintDiagnostics("Encoding: uLaw.\n");
    audioInfo->play.encoding = AudioULawEncoding;
    audioInfo->record.encoding = AudioULawEncoding;
  }
  else if(strcmp(tString, "A_LAW") == 0)
  {
    PrintDiagnostics("Encoding: aLaw.\n");
    audioInfo->play.encoding = AudioALawEncoding;
    audioInfo->record.encoding = AudioALawEncoding;
  }
  else if(strcmp(tString, "PCM") == 0)
  {
    PrintDiagnostics("Encoding: Linear.\n");
    audioInfo->play.encoding = AudioLinearEncoding;
    audioInfo->record.encoding = AudioLinearEncoding;
  }
  
  rc = UMSAudioDevice_get_volume(audio_device, ev, &tVal);
  audioInfo->monitorVolume = tVal;
  audioInfo->record.volume = record_volume;
  
  rc = UMSAudioDevice_get_volume(paudio_device, ev, &tVal);
  audioInfo->play.volume = tVal;
  
  XtVaSetValues(playbackScale, XmNvalue, audioInfo->play.volume, NULL);
  XtVaSetValues(recordScale, XmNvalue, audioInfo->record.volume, NULL);
  XtVaSetValues(monitorScale, XmNvalue, audioInfo->monitorVolume, NULL);
  
  PrintDiagnostics("Audio information:\n");
  PrintDiagnostics("Device type: IBM audio.\n");
  sprintf(diagString, "Sample rate: %d.\n", audioInfo->play.sampleRate);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Number of channels: %d.\n", audioInfo->play.channels);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Bits per sample: %d.\n", audioInfo->play.bitsPerSample);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Playback volume: %d.\n", audioInfo->play.volume);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Record volume: %d.\n", audioInfo->record.volume);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Monitor volume: %d.\n", audioInfo->monitorVolume);
  PrintDiagnostics(diagString);
  return;
} /* end function IBMAudioDevicePrintAudioInfo */



int IBMAudioDeviceInitializeAudio(void)				    /* returns status:  -1 = fail, 0 = ok */
{
  int	 audioFormat;
  int	returnedSampleRate;
  char *tempString, *alias, *error_string, *audio_formats_alias;
  char *audio_inputs_alias, *audio_outputs_alias, *master;
  UMSAudioDeviceMClass_ErrorCode audio_device_class_error;
  long		flags, tval;
  int left_gain = 50, right_gain = 50;
  
  current_input = LINE_IN;
  current_output = INTERNAL_SPEAKER;
  ev = somGetGlobalEnvironment();				    /* Set up the SOM enviroment */
  /*
   * Create the audio device object.  Create an AudioDeviceMClass
   * metaclass, and then use make_by_alias to create the particular
   * AudioDevice object that corresponds to the hardware.
   */
  audio_device_class = UMSAudioDeviceNewClass(UMSAudioDevice_MajorVersion,
					      UMSAudioDevice_MinorVersion);
  if (audio_device_class == NULL)
  {
    fprintf(stderr, "can't create AudioDeviceMClass metaclass\n");
    exit(1);
  }
  paudio_device_class = UMSAudioDeviceNewClass(UMSAudioDevice_MajorVersion,
					       UMSAudioDevice_MinorVersion);
  if (paudio_device_class == NULL)
  {
    fprintf(stderr, "can't create AudioDeviceMClass metaclass\n");
    exit(1);
  }
  alias = "Audio";
  flags = UMSAudioDevice_BlockingIO;				    /* */
  /*	flags = 0; */
  
  audio_device = UMSAudioDeviceMClass_make_by_alias(audio_device_class,
						    ev,
						    alias,
						    "RECORD",
						    flags,
						    &audio_device_class_error,
						    &error_string,
						    &audio_formats_alias,
						    &audio_inputs_alias,
						    &audio_outputs_alias);
  if (audio_device == NULL)
  {
    fprintf(stderr, "can't create record audio device object\n");
    return(-1);
  }
  rc = UMSAudioDevice_set_sample_rate(audio_device,		    /* Set up the record audio device for stereo 16-bit PCM. */
				      ev, (long)sampleRate,
				      (long*)&returnedSampleRate);
  if (sampleRate != returnedSampleRate)				    /* Was the requested sample rate used?  If not, return... */
  {								    /* ...an error code. */
    sprintf(errorMessage, "Could not record at the sample rate you requested (%d Hz).\n", sampleRate);
    PrintErrorMessage(errorMessage);
    return(-1);
  }

  readBuffSize = sampleRate * 5;
  rc = UMSAudioDevice_set_time_format(audio_device, ev, UMSAudioTypes_Samples);
  rc = UMSAudioDevice_set_bits_per_sample(audio_device, ev, 16);
  rc = UMSAudioDevice_set_number_of_channels(audio_device, ev, 2);
  rc = UMSAudioDevice_set_audio_format_type(audio_device, ev, "PCM");
  rc = UMSAudioDevice_set_byte_order(audio_device, ev, "MSB");
  rc = UMSAudioDevice_set_number_format(audio_device, ev, "TWOS_COMPLEMENT");
  rc = UMSAudioDevice_set_volume(audio_device, ev, 50);		    /* Set up the recording parameters */
  rc = UMSAudioDevice_set_balance(audio_device, ev, 0);
  rc = UMSAudioDevice_enable_input(audio_device, ev, current_input, (long*)&left_gain, (long*)&right_gain);
  rc = UMSAudioDevice_enable_output(audio_device, ev, current_output, (long*)&left_gain, (long*)&right_gain);
  rc = UMSAudioDevice_disable_output(audio_device, ev, LINE_OUT);
  rc = UMSAudioDevice_disable_output(audio_device, ev, AUX1);
  rc = UMSAudioDevice_set_monitor(audio_device, ev, TRUE);
  rc = UMSAudioDevice_initialize(audio_device, ev);
  
  rc = UMSAudioDevice_read_buff_size(audio_device, ev, &tval);
/*
  printf("+++Debugging: read buffer size before set is %d samples.\n", tval);
  printf("Debugging: About to request an audio buffer size of %d samples.\n", readBuffSize);
*/
  rc = UMSAudioDevice_set_audio_buffer_size(audio_device, ev, readBuffSize, &tval);

/*  printf("Debugging: set_audio_buffer_size returned an rc of %d, and a buffer size of %d samples.\n", rc, tval);*/

  rc = UMSAudioDevice_read_buff_size(audio_device, ev, &tval);
/*  printf("+++Debugging: read buffer size is %d samples.\n", tval);*/
  
  /*	flags = UMSAudioDevice_BlockingIO; */
  
  paudio_device = UMSAudioDeviceMClass_make_by_alias(paudio_device_class,
						     ev,
						     alias,
						     "PLAY",
						     flags,
						     &audio_device_class_error,
						     &error_string,
						     &audio_formats_alias,
						     &audio_inputs_alias,
						     &audio_outputs_alias);
  if (paudio_device == NULL)
  {
    fprintf(stderr, "can't create play audio device object\n");
    return(-1);
  }
  rc = UMSAudioDevice_set_sample_rate(paudio_device,		    /* Set up the play device for stereo 16-bit PCM. */
				      ev, sampleRate, (long*)&sampleRate);
  if (sampleRate != returnedSampleRate)				    /* Was the requested sample rate used?  If not, return... */
  {								    /* ...an error code. */
    sprintf(errorMessage, "Could not play at the sample rate you requested (%d Hz).\n", sampleRate);
    PrintErrorMessage(errorMessage);
    return(-1);
  }

  rc = UMSAudioDevice_set_bits_per_sample(paudio_device, ev, 16);
  rc = UMSAudioDevice_set_number_of_channels(paudio_device, ev, 2);
  rc = UMSAudioDevice_set_audio_format_type(paudio_device, ev, "PCM");
  rc = UMSAudioDevice_set_byte_order(paudio_device, ev, "MSB");
  rc = UMSAudioDevice_set_number_format(paudio_device, ev, "TWOS_COMPLEMENT");
  
  rc = UMSAudioDevice_set_volume(paudio_device, ev, 50);	    /* Set up the play parameters */
  rc = UMSAudioDevice_set_balance(paudio_device, ev, 0);
  rc = UMSAudioDevice_set_time_format(paudio_device, ev, UMSAudioTypes_Samples);
  rc = UMSAudioDevice_enable_output(paudio_device, ev, current_output, (long*)&left_gain, (long*)&right_gain);
  rc = UMSAudioDevice_disable_output(paudio_device, ev, LINE_OUT);
  rc = UMSAudioDevice_disable_output(paudio_device, ev, AUX1);
  rc = UMSAudioDevice_set_monitor(paudio_device, ev, FALSE);
  rc = UMSAudioDevice_initialize(paudio_device, ev);
  rc = UMSAudioDevice_set_audio_buffer_size(paudio_device, ev, readBuffSize, &tval);
  
  /* ok now read back stuff and set local databases */
  
  rc = UMSAudioDevice_get_audio_format_type(paudio_device, ev, &tempString);
  
  if(strcmp(tempString, "MU_LAW") == 0)
  {
    audioPlayHeader.encoding = AudioULawEncoding;
    audioRecordHeader.encoding = AudioULawEncoding;
  }
  else if(strcmp(tempString, "A_LAW") == 0)
  {
    audioPlayHeader.encoding = AudioALawEncoding;
    audioRecordHeader.encoding = AudioALawEncoding;
  }
  else if(strcmp(tempString, "PCM") == 0)
  {
    audioPlayHeader.encoding = AudioLinearEncoding;
    audioRecordHeader.encoding = AudioLinearEncoding;
  }
  
  audioRecordHeader.sample_rate = sampleRate;
  audioPlayHeader.sample_rate = sampleRate;
  audioRecordHeader.samples_per_unit = 1;
  audioPlayHeader.samples_per_unit = 1;
  
  rc = UMSAudioDevice_get_number_of_channels(audio_device, ev, &tval);
  audioRecordHeader.channels = tval;
  audioPlayHeader.channels = tval;
  
  rc = UMSAudioDevice_get_bits_per_sample(audio_device, ev, &tval);
  audioRecordHeader.bytes_per_unit = ((tval / 8) * audioRecordHeader.channels);
  audioPlayHeader.bytes_per_unit = ((tval / 8) * audioPlayHeader.channels);
  audioRecordHeader.data_size = sampleRate * 2;
  audioPlayHeader.data_size = sampleRate * 2;
  
  tval = audioRecordHeader.bytes_per_unit * (sampleRate / DIVISOR);
  rc = UMSAudioDevice_set_DMA_buffer_size(audio_device, ev, tval, &tval);
  tval = audioPlayHeader.bytes_per_unit * (sampleRate / DIVISOR);
  rc = UMSAudioDevice_set_DMA_buffer_size(paudio_device, ev, tval, &tval);
  
  /* ok, now close them since we need to change the sample rate with only one open */

#if 0
/*  printf("0000000000000----Inside in if zero.\n");*/
  rc = UMSAudioDevice_close(audio_device, ev);
  rc = UMSAudioDevice_close(paudio_device, ev);
#endif
  return(0);							    /* Return ok status */
} /* end function IBMAudioDeviceInitializeAudio */



void IBMAudioDeviceSetAudioPlayVolume(int volume)
{
/*  printf("Debug: SetPlayVolume().\n");*/
  if(!audio_is_recording)
    rc = UMSAudioDevice_set_volume(paudio_device, ev, volume);
  play_volume = volume;
} /* end function IBMAudioDeviceSetAudioPlayVolume */



void IBMAudioDeviceSetAudioRecordVolume(int volume)
{
  boolean enabled;
  long tval;
  
/*  printf("Debug: SetRecordVolume().\n");*/
  record_volume = volume;
  rc = UMSAudioDevice_enable_input(audio_device, ev,
				   current_input,
				   (long*)&record_volume, (long*)&record_volume);
  rc = UMSAudioDevice_initialize(audio_device, ev);
  rc = UMSAudioDevice_set_audio_buffer_size(audio_device, ev, readBuffSize, &tval);
} /* end function IBMAudioDeviceSetAudioRecordVolume */



void IBMAudioDeviceSetAudioMonitorVolume(int volume)
{
  long tval;
#if 0
  if(volume)
  {
    if(!monitor_volume && audio_is_recording)
    {
/*      printf("Debug: MonitorVolume()-monitor_volume 0 & audio_is_recording.\n");*/
      rc = UMSAudioDevice_set_monitor(audio_device, ev, True);
      rc = UMSAudioDevice_initialize(audio_device, ev);
      rc = UMSAudioDevice_set_audio_buffer_size(audio_device, ev, readBuffSize, &tval);
    }
  }
  else
  {
    if(monitor_volume && audio_is_recording)
    {
/*      printf("Debug: MonitorVolume()-monitor_volume non-zero & audio_is_recording.\n");*/
      rc = UMSAudioDevice_set_monitor(audio_device, ev, False);
      rc = UMSAudioDevice_initialize(audio_device, ev);
      rc = UMSAudioDevice_set_audio_buffer_size(audio_device, ev, readBuffSize, &tval);
    }
  }
  if(!audio_is_playing)
#endif
  {
/*    printf("Debug: MonitorVolume()-audio_is_not_playing.\n");*/
    if(volume)
      rc = UMSAudioDevice_set_monitor(audio_device, ev, True);
    else
      rc = UMSAudioDevice_set_monitor(audio_device, ev, False);
    rc = UMSAudioDevice_set_volume(audio_device, ev, volume);
    rc = UMSAudioDevice_initialize(audio_device, ev);
    rc = UMSAudioDevice_set_audio_buffer_size(audio_device, ev, readBuffSize, &tval);
  }
  monitor_volume = volume;
} /* end function IBMAudioDeviceSetAudioMonitorVolume */



void IBMAudioDeviceSetPlaybackDevice(int deviceType)
{
  long tval;
  char	*localDeviceType, *disable0, *disable1;
  int left_gain = 50, right_gain = 50;
  if(audio_is_recording)
  {
	audio_is_recording = 0;
	UMSAudioDevice_stop(audio_device, ev);
  }
  if(audio_is_playing)
  {
	audio_is_playing = 0;
	UMSAudioDevice_stop(paudio_device, ev);
  }
  if (deviceType == AudioSpeaker)
  {
    PrintDiagnostics("SetPlaybackDevice(): Speaker set.\n");
    localDeviceType = INTERNAL_SPEAKER;
    disable0 = LINE_OUT;
    disable1 = NULL;
  }
  else if (deviceType == AudioLineOut)
  {
    PrintDiagnostics("SetPlaybackDevice(): Line Out set.\n");
    localDeviceType = LINE_OUT;
    disable0 = INTERNAL_SPEAKER;
    disable1 = NULL;
  }
  else if (deviceType == AudioHeadphone)
  {
    localDeviceType = NULL;
    disable0 = INTERNAL_SPEAKER;
    disable1 = LINE_OUT;
    PrintDiagnostics("SetPlaybackDevice(): Headphone set.\n");
  }
  rc = UMSAudioDevice_disable_output(paudio_device, ev, disable0);
  rc = UMSAudioDevice_disable_output(audio_device, ev, disable0);
  if(disable1)
  {
    rc = UMSAudioDevice_disable_output(paudio_device, ev, disable1);
    rc = UMSAudioDevice_disable_output(audio_device, ev, disable1);
  }
  if(localDeviceType)
  {
    rc = UMSAudioDevice_enable_output(paudio_device, ev, localDeviceType,
				      (long*)&left_gain, (long*)&right_gain);
    rc = UMSAudioDevice_enable_output(audio_device, ev, localDeviceType,
				      (long*)&left_gain, (long*)&right_gain);
  }
  rc = UMSAudioDevice_initialize(audio_device, ev);
  rc = UMSAudioDevice_initialize(paudio_device, ev);
  rc = UMSAudioDevice_set_audio_buffer_size(audio_device, ev, readBuffSize, &tval);
  rc = UMSAudioDevice_set_audio_buffer_size(paudio_device, ev, readBuffSize, &tval);
  current_output = localDeviceType;
  return;
} /* end function IBMAudioDeviceSetPlaybackDevice */



void IBMAudioDeviceSetRecordDevice(int deviceType)
{
  char 	*localDeviceType, *disableDeviceType;
  long tval;
  
  if (deviceType == AudioMicrophone)
  {
/*    printf("Debug: SetRecordDevice()-Mic.\n");*/
    localDeviceType = LOW_GAIN_MIC;
    disableDeviceType = LINE_IN;
  }
  else
  {
/*    printf("Debug: SetRecordDevice()-Line In.\n");*/
    localDeviceType = LINE_IN;
    disableDeviceType = LOW_GAIN_MIC;
  }
  rc = UMSAudioDevice_disable_input(audio_device, ev, disableDeviceType);
  rc = UMSAudioDevice_enable_input(audio_device, ev, localDeviceType,
				   (long*)&record_volume, (long*)&record_volume);
  rc = UMSAudioDevice_initialize(audio_device, ev);
  rc = UMSAudioDevice_set_audio_buffer_size(audio_device, ev, readBuffSize, &tval);
  current_input = localDeviceType;
} /* end function IBMAudioDeviceSetRecordDevice */



int IBMAudioDeviceSetAudioPlaybackConfiguration(Audio_hdr*	audioInfo)
{
  int			audioResult;
  static Audio_hdr	currentAudioSettings;
  static Boolean	firstTime = True;
  int			result;
  int			returnedSampleRate;
  char			*tempString;
  char *device;
  
  if (firstTime == True)					    /* Note the initial audio playback device settings for... */
  {								    /* ...later use. */
    currentAudioSettings.channels = -1;
    currentAudioSettings.sample_rate = -1;
    currentAudioSettings.encoding = -1;
    firstTime = False;
  }
  if (currentAudioSettings.encoding != audioInfo->encoding)	    /* Only change the audio encoding if the requested... */
  {								    /* ...configuration differs from the current one. */
    switch (audioInfo->encoding)				    /* Translate the PortableAudio encoding constant into the... */
    {								    /* ...appropriate HP-specific constant. */
     case AudioULawEncoding:
      tempString = "MU_LAW";
      PrintDiagnostics("SetAudioPlaybackConfig(): Setting encoding to ulaw.\n");
      break;
     case AudioALawEncoding:
      tempString = "A_LAW";
      PrintDiagnostics("SetAudioPlaybackConfig(): Setting encoding to alaw.\n");
      break;
     case AudioLinearEncoding:
      tempString = "PCM";
      PrintDiagnostics("SetAudioPlaybackConfig(): Setting encoding to linear.\n");
      break;
    }
    rc = UMSAudioDevice_set_audio_format_type(paudio_device, ev, tempString);
  } /* end (if encodings differ...) */
  currentAudioSettings.encoding = audioInfo->encoding;
  if (currentAudioSettings.sample_rate != audioInfo->sample_rate)   /* Only change the sample rate if the requested... */
  {								    /* ...configuration differs from the current one. */
    rc = UMSAudioDevice_get_audio_device_pathname(paudio_device,    /* get path name so we can open later */
						  ev, &device);
    rc = UMSAudioDevice_close(audio_device, ev);		    /* close record so we can set sample rate */
    rc = UMSAudioDevice_set_sample_rate(paudio_device,
					ev, audioInfo->sample_rate,
					(long*)&returnedSampleRate);
    if (returnedSampleRate != audioInfo->sample_rate)		    /* Did the requested sample rate fail? */
    {								    /* ...an error code. */
      sprintf(errorMessage,
	      "Could not configure the audio device for playback at the requested sample rate (%d Hz).\n",
	      sampleRate);
      PrintErrorMessage(errorMessage);
      return(-1);
    }
    rc = UMSAudioDevice_set_sample_rate(audio_device,
					ev, audioInfo->sample_rate,
					(long*)&returnedSampleRate);
    if (returnedSampleRate != audioInfo->sample_rate)		    /* Did the requested sample rate fail? */
    {								    /* ...an error code. */
      sprintf(errorMessage,
	      "Could not configure the audio device for record at the requested sample rate (%d Hz).\n",
	      sampleRate);
      PrintErrorMessage(errorMessage);
      return(-1);
    }
    rc = UMSAudioDevice_open(audio_device,
			     ev, device, "RECORD",
			     UMSAudioDevice_BlockingIO);
  }  
  currentAudioSettings.sample_rate = audioInfo->sample_rate;
  if (currentAudioSettings.channels != audioInfo->channels)	    /* Only change the number of channels if the requested... */
  {								    /* ...configuration differs from the current one. */
    rc = UMSAudioDevice_set_number_of_channels(paudio_device, ev,
					       audioInfo->channels);
  }
  currentAudioSettings.channels = audioInfo->channels;
  return(0);							    /* Yes, return zero */
} /* end function IBMAudioDeviceSetAudioPlaybackConfiguration */



void IBMAudioDeviceAudioClose(void)
{
  rc = UMSAudioDevice_stop(audio_device, ev);
  rc = UMSAudioDevice_close(audio_device, ev);
  _somFree(audio_device);
  rc = UMSAudioDevice_stop(paudio_device, ev);
  rc = UMSAudioDevice_close(paudio_device, ev);
  _somFree(paudio_device);
  return;
} /* end function IBMAudioDeviceAudioClose */



void IBMAudioDeviceSyncAudioPlaybackDevice(void)
{
  struct timeval	after;
  struct timeval	before;
  int			bytesWritten;
  int			counter = 0;
  int			elapsedTime = 0;
  static char*		syncBuffer = NULL;
  
  IBMAudioDeviceAudioPlayResume();				    /* Make sure the audio device restarts correctly */
#if 0
/*  printf("0000000000000----Inside in if zero.\n");*/
  if (syncBuffer == NULL)					    /* Allocate some memory for holding audio to be pushed... */
  {								    /* ...through the audio device. */
    syncBuffer = malloc(48000);					    /* I'm guessing that 48K is the max. sample rate and will... */
    if (syncBuffer == NULL)					    /* ...be an upper-bound on sound size used. */
    {
      sprintf(errorMessage,
	      "Could not allocate enough audio memory to properly sync audio.\n");
      PrintErrorMessage(errorMessage);
      return;
    }
    memset(syncBuffer, '\0', 48000);				    /* Fill the audio buffer with zeros */
  }
  AudioWrite(syncBuffer, 32000);
  return;
  write(audioFD, syncBuffer, 32000);				    /* Fill up the audio device's "current page" */
  while (elapsedTime < (1000 / playbackSpeed))			    /* Keep pushing audio until time to play 1 frame is close... */
  {								    /* ...to real-time */
    if (printDiagnostics == True)
    {
      printf("Playing %d bytes of audio (iteration %d).\n",
	     movieHeader.audioslice, counter++);
    }
    gettimeofday(&before, (struct timezone*)NULL);
    bytesWritten = write(audioFD, syncBuffer,			    /* Playback 1 frame's worth of audio */
			 movieHeader.audioslice);
    gettimeofday(&after, (struct timezone*)NULL);
    if ((bytesWritten < 0) && (printDiagnostics == True))
    {
      printf("SyncAudioPlaybackDevice(): Error %d (errno is %d) writing audio.\n",
	     bytesWritten, errno);
    }
    elapsedTime = ElapsedTime(&before, &after);			    /* Count time in milliseconds to play 1 frame of audio */
  }
  return;
#endif
} /* end function IBMAudioDeviceSyncAudioPlaybackDevice */



void IBMAudioDeviceSyncAudioRecordDevice(void)
{
  return;
} /* end function IBMAudioDeviceSyncAudioRecordDevice */



void IBMAudioDeviceAudioPlayConnect(void)
{
  return;
}


void IBMAudioDeviceAudioPlayDisconnect(void)
{
  return;
}



void IBMAudioDeviceAudioRecordConnect(void)
{
  return;
}



void IBMAudioDeviceAudioRecordDisconnect(void)
{
  return;
}



void IBMAudioDeviceAudioStopPlayDevice(void)
{
  return;
}



void IBMAudioDeviceAudioStopRecordDevice(void)
{
  return;
}
