/* $Header: /plx/source/Xclients/generic/RCS/aixSpecific.c,v 1.6 1996/01/25 04:17:13 drapeau Exp $ */
/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Log: aixSpecific.c,v $
 * Revision 1.6  1996/01/25  04:17:13  drapeau
 * Added support for the PlxOutputCascade to automatically create the
 * output menu hierarchy, for enhanced flexibility.
 *
 * Revision 1.5  1996/01/05  21:30:13  drapeau
 * Minor addition to ShowCompactMovieDialog() to account for new
 * interface feature that should be greyed out when this function is
 * called.
 *
 * Revision 1.4  1995/10/06  23:00:56  drapeau
 * Added support for showing the current input signal type in use.
 *
 * Revision 1.3  1995/10/02  21:50:41  drapeau
 * Added support for keeping track of the user interface's visibility
 * state: whether some panels are mapped or not, etc.
 *
 * Revision 1.2  1995/08/23  01:53:35  drapeau
 * Fixed ShowInputWorkProc() so that it correctly resets the recording
 * frame rate based on the input video signal type it detects (30 fps for
 * NTSC, 25 FPS for PAL or SECAM).
 *
 * Revision 1.1  1995/08/17  19:32:14  drapeau
 * Added function SyncIODevice() as a test to read ahead for the movie
 * about to be played.  Currently it is not used.
 *
 * Revision 1.0  1995/06/16  19:59:28  drapeau
 * Initial revision
 * */
static char aixSpecificRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/aixSpecific.c,v 1.6 1996/01/25 04:17:13 drapeau Exp $";

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/Scale.h>
#include <Xm/PushBG.h>
#include "PortableAudio.h"
#include "MovieTool.h"


AudioModuleMapEntry audioMap[5] =
{
  {"IBMAudioDevice", "IBM Audio Device", "Uses Ultimedia Audio Services", &defaultAudioObject},
  {"None", "None", "Does not record or play audio", &noAudioObject},
  {"", "", "", (AudioObject *)NULL},
  {"", "", "", (AudioObject *)NULL},
  {"", "", "", (AudioObject *)NULL}
};



void SaveMPEGFile(char* fileName)
{
  sprintf(errorMessage,
	  "Saving MPEG files is not yet implemented on this platform.\n");
  PrintErrorMessage(errorMessage);
  return;
} /* end function SaveMPEGFile */



void ShowCompactMovieDialog(Widget	w,
			    XtPointer	client_data,
			    XtPointer	xt_call_data)
{
  XtVaSetValues(fileDialog,
		XmNtitle, "Compact This Movie",
		NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, SaveMovie, NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, OpenMovie, NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, CompactMovie, NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, SaveStill, NULL);
  XtAddCallback(fileSelectionBox, XmNokCallback, CompactMovie, NULL);
  XtVaSetValues(saveAsOptionButton, XmNsensitive, False, NULL);	    /* Don't allow author to choose final movie output format */
  XtManageChild(fileSelectionBox);
  return;
} /* end function ShowCompactMovieDialog */



void CreateJPEGWidget(void)
{
  int	oldImageQualityQFactor;
  int	oldPlaybackQFactor;
  
  if (playbackWidget != (Widget)NULL)				    /* Destroy old movie record & playback widget if it exists */
  {
    XtVaGetValues(playbackWidget,				    /* Save the Q factor currently being used by the playback... */
		  XtNqFactor, &oldPlaybackQFactor, NULL);	    /* ...widget. */
    
    XtDestroyWidget(playbackWidget);
  }
  playbackWidget =
    XtVaCreateManagedWidget("playbackWidget",			    /* Create a widget for showing JPEG movies */
			    jpegWidgetClass,
			    movieShell,				    /* Take zoom factor into account when doing the dimension... */
			    XtNwidth, (movieHeader.width * zoomFactor),	/* ...calculations. */
			    XtNheight, (movieHeader.height * zoomFactor),
			    XtNqFactor, movieHeader.qfactor,
			    NULL);
  XtVaGetValues(playbackWidget,					    /* Point to the shared memory segment used by this widget */
		XtNjpegData, &jpegImage,
		XtNuseSharedMemory, &usingSharedMemory,		    /* Determine whether the JPEG widget is using shared memory */
		NULL);
  if (usingSharedMemory == False)				    /* If the JPEG widget isn't using shared memory, allocate... */
  {								    /* ...enough memory to store images for playback. */
    jpegImage = XPlxCreateCImage(XtDisplay(playbackWidget),	    /* ...structure (the function will allocate the necessary... */
				 NULL,				    /* ...memory for storing a single JPEG image). */
				 OneImageSizeInBytes,
				 movieHeader.width,
				 movieHeader.height);
    XtVaSetValues(playbackWidget,				    /* Tell the JPEG widget to point to the newly-created... */
		  XtNjpegData, jpegImage,			    /* Parallax JPEG image structure. */
		  NULL);
    jpegImage->width = movieHeader.width;			    /* Set variables for the width and height of the JPEG... */
    jpegImage->height = movieHeader.height;			    /* ...images in the currently open movie. */
  }
  XtUnmanageChild(imageQualityFrame);				    /* Don't allow parent Frame to resize itself to 0x0 size */
  if (imageQualityWidget != (Widget)NULL)			    /* Destroy the old image quality widget if there is one */
  {
    XtVaGetValues(imageQualityWidget,				    /* Save the Q factor currently being used by the image... */
		  XtNqFactor, &oldImageQualityQFactor, NULL);	    /* ...quality widget. */
    XtDestroyWidget(imageQualityWidget);
  }
  imageQualityWidget =
    XtVaCreateManagedWidget("imageQualityWidget",		    /* Create a JPEG widget to show image quality for... */
			    jpegWidgetClass,			    /* ...varying Q factors. */
			    imageQualityFrame,
			    XtNwidth, 160,
			    XtNheight, 120,
			    XtNcompressSourceRect, &imageQualitySourceRect,
			    XtNcompressDestRect, &imageQualitySourceRect,
			    XtNdisplayDestRect, &imageQualityRect,
			    XtNqFactor, movieHeader.qfactor,
			    NULL);
  XtManageChild(imageQualityFrame);				    /* Okay to resize parent now that it has a child */
  return;
} /* end function CreateJPEGWidget */



Boolean OutputWorkProc(XtPointer	clientData)
{
  static Boolean	firstTime = True;
  Widget		w;
  
  w = clientData;
  if (movieStatus == MovieNew)					    /* Don't allow output until a movie has been played */
    return(True);
  outputOn = True;						    /* Note that output is about to be enabled */
  XtVaSetValues(recordButton, XmNsensitive, False, NULL);	    /* Don't allow recording while output is on */
  XtVaSetValues(saveAsMenuButton, XmNsensitive, False, NULL);	    /* Don't allow selection of a new movie file for recording */
  DisableJPEGCompressionControls();
  if (firstTime == True)
  {
    outputGC = XCreateGC(XtDisplay(w), XtWindow(playbackWidget), 0, 0);
    firstTime = False;
  }
  if (PLX_NTSC == outputSignalStandard)				    /* Was NTSC output selected? */
  {								    /* Yes, use NTSC values (This is a hack!  Please remove... */
    outputSignalWidth = 640;					    /* ...it and use the proper API calls to determine the... */
    outputSignalHeight = 480;					    /* ...width & height of the output signal.  Ideally, the... */
  }								    /* ...Output widget should be used for this. */
  else
  {
    outputSignalWidth = 768;
    outputSignalHeight = 575;
  }
  XtDestroyWidget(input1Widget);				    /* Make sure that the Port attached to the input... */
  input1Widget = XtVaCreateManagedWidget("input1Widget",	    /* ...widget is detached by destroying the widget's... */
					 xVideoInWidgetClass,	    /* ...window, then re-create the input widget but don't... */
					 input1Shell,		    /* ...immediately show video on it (showing video would... */
					 XtNchannel, PLX_IO_OFF,    /* ...cause a Port to be attached to the Window). */
					 XtNsignalStandard, inputSignalStandard,
					 XtNsignalFormat, inputSignalFormat,
					 XtNinstance, inputInstance,
					 NULL);
  CreateJPEGWidget();						    /* Create a new widget for playing and recording movies */
  XPlxVideoOutputSelect(XtDisplay(w),				    /* Force JPEG playback to use the 1st hardware port... */
			XtWindow(playbackWidget),		    /* ...(the "v" channel). */
			outputGC,
			PLX_OUTPUT_JPEG,
			outputSignalStandard,
			outputSignalFormat,
			0);
  XPlxVideoScaleOutput(XtDisplay(w),
		       XtWindow(playbackWidget),
		       outputGC,
		       0, 0,
		       movieHeader.width, movieHeader.height,
		       0, 0,
		       outputSignalWidth, outputSignalHeight);
  PlayOneFrame(currentFrame, False, True);			    /* Refresh the playback window after output is ready */
  return(True);
} /* end function OutputWorkProc */



void ResizeOutput(void)
{
  if (outputOn == False)
    return;
  
  XPlxVideoScaleOutput(XtDisplay(playbackWidget),
		       XtWindow(playbackWidget),
		       outputGC,
		       0, 0,
		       movieHeader.width, movieHeader.height,
		       0, 0,
		       outputSignalWidth, outputSignalHeight);
  return;
} /* end function ResizeOutput */



void OutputCallback(Widget	w,
		    XtPointer	client_data,
		    XtPointer	xt_call_data)
{
  XtWorkProcId	outputWorkProcID;
  
  outputWorkProcID = XtAppAddWorkProc(app_context,		    /* Start a work-proc to show output; Xt is... */
				      OutputWorkProc,		    /* ...poorly designed and this is the only way to assure... */
				      (XtPointer)w);		    /* ...that widgets needing to be destroyed right now... */
  return;							    /* ...will happen. */
} /* end function OutputCallback */



Boolean ShowInputWorkProc(XtPointer	clientData)
{
  static Boolean	firstTime = True;
  static GC		gc;
  int			channelToUse;
  signalSpec	theSignal;
  
  if ((XtPointer)0 == clientData)				    /* Was the "first available" special value passed in? */
  {								    /* Yes, do the appropriate thing */
    XtVaSetValues(input1Widget, XtNautoSearch, True, NULL);	    /* Let the widget find the 1st valid input source */
    theSignal.s.channel = PLX_INPUT_0;				    /* Begin the search with the 1st possible input for... */
    theSignal.s.standard = PLX_NTSC;				    /* ...XVideo hardware. */
    theSignal.s.format = PLX_COMP;
    theSignal.s.instance = 0;
  }
  else								    /* No, an explicit source was requested; use the value... */
  {
    theSignal.i = (unsigned int)clientData;			    /* ...passed in. */
  }
  if (firstTime == True)
  {
    gc = XCreateGC(XtDisplay(input1Widget),
		   XtWindow(playbackWidget), 0, 0);
    firstTime = False;
  }
  if (outputOn == True)						    /* Only disable output if it was on in the 1st place */
  {
    XPlxVideoOutputSelect(XtDisplay(input1Widget),		    /* Disable output of JPEG playback if it was previously on */
			  XtWindow(playbackWidget),
			  gc,
			  PLX_OUTPUT_JPEG_OFF,
			  outputSignalStandard,
			  PLX_COMP,
			  0);
    XPlxVideoOutputSelect(XtDisplay(input1Widget),
			  XtWindow(imageQualityWidget),
			  gc,
			  PLX_OUTPUT_JPEG_OFF,
			  outputSignalStandard,
			  PLX_COMP,
			  0);
    outputOn = False;						    /* Note that output is now disabled */
    XtVaSetValues(recordButton, XmNsensitive, True, NULL);	    /* Allow recording again now that output is off */
    XtVaSetValues(saveAsMenuButton, XmNsensitive, True, NULL);	    /* Don't allow selection of a new movie file for recording */
    EnableJPEGCompressionControls();
    CreateJPEGWidget();						    /* Create a new widget for playing and recording movies */
  } /* end if output is on */
  XtPopup(input1Shell, XtGrabNone);
  SetMTPanelState(&mtPanelState, input1Shell, True);		    /* Indicate that the live video input shell is now visible */
  XtVaSetValues(input1Widget, XtNchannel, PLX_IO_OFF, NULL);	    /* Disable video before changing channels */
  XtVaSetValues(input1Widget,					    /* Attempt to choose the new signal source */
		XtNchannel, theSignal.s.channel,
		XtNsignalStandard, theSignal.s.standard,
		XtNsignalFormat, theSignal.s.format,
		XtNinstance, theSignal.s.instance,
		NULL);
  XtVaGetValues(input1Widget,					    /* Determine the signal standard for this video input, so... */
		XtNsignalStandard, &inputSignalStandard,	    /* ...the frame rate for this standard can be determined. */
		XtNsignalFormat, &inputSignalFormat,
		XtNinstance, &inputInstance,
		NULL);
  if (inputSignalStandard == PLX_NTSC)				    /* Is it an NTSC signal? */
  {
    signalFrameRate = 30;					    /* Yep, frame rate is 30 Hz */
    recordFrameRate = 30;					    /* Set an initial rate for recording NTSC movies */
    XtVaSetValues(recordSpeedSlider, XmNvalue, 30, NULL);
  }
  else								    /* Nope, must be PAL or SECAM; frame rate is 25 Hz */
  {
    signalFrameRate = 25;
    recordFrameRate = 25;					    /* Set an initial rate for recording PAL or SECAM movies */
    XtVaSetValues(recordSpeedSlider, XmNvalue, 25, NULL);
  }
  UpdateInputTypeString(&theSignal, sync);			    /* Update the label showing the current signal type */
  return(True);
} /* end function ShowInputWorkProc */



void ShowInput(Widget		w,
	       XtPointer	clientData,
	       XtPointer	unusedCallData)
{
  XtWorkProcId	inputWorkProcID;
  inputWorkProcID = XtAppAddWorkProc(app_context,		    /* Start a work-proc to select a new input source; Xt is... */
				     ShowInputWorkProc,		    /* ...poorly designed and this is the only way to assure... */
				     (XtPointer)clientData);	    /* ...that widgets needing to be destroyed right now... */
  return;							    /* ...searching for alternate input sources. */
} /* end function ShowInput */



void ShowFirstAvailableInput(Widget	w,
			     XtPointer	unusedClientData,
			     XtPointer	unusedCallData)
{
  XtWorkProcId	firstAvailWorkProcID;
  
  XtVaSetValues(input1Widget, XtNautoSearch, True, NULL);	    /* Allow the widget to search for any valid input */
  firstAvailWorkProcID = XtAppAddWorkProc(app_context,		    /* Start a work-proc to select a new input source; Xt is... */
					  ShowInputWorkProc,	    /* ...poorly designed and this is the only way to assure... */
					  (XtPointer)w);	    /* ...that widgets needing to be destroyed right now... */
} /* end function ShowFirstAvailableInput */



void SyncIODevice(void)
{
  char		appName[MAXPATHLEN];
  char		cmdLine[MAXPATHLEN];
  int		startingFrame = 0;
  int		processID = 0;
  int		result = 0;
  int		counter;
  long		numFilesPerProcess;
  char		errorString[256];
  
  processID = fork();						    /* Try to fork and exec the new application */
  if (processID == -1)
  {
    return;
  }
  if (processID == 0)						    /* This is the child process part of the code */
  {
    numFilesPerProcess = sysconf(_SC_OPEN_MAX);			    /* Determine how many files could possibly be open */
    for (counter = 3; counter < numFilesPerProcess; counter++)	    /* Close off all open files except stdin, stdout, & ... */
    {								    /* ...stderr (file descriptors 0, 1, & 2). */
      close(counter);
    }
    setsid();							    /* Make sure this app can run on its own */
    sprintf(appName, "jpegMovieRead");				    /* Set up the name of the movie read-ahead program to... */
    startingFrame = currentFrame + (3 * movieHeader.fps);	    /* Set the read-ahead for three seconds' worth of movie */
    if (startingFrame >= playbackEndFrame)			    /* Would the read-ahead start after the end of the... */
    {								    /* ...segment to be played?  If so, it's worthless. */
      exit(0);							    /* Don't bother to call the read-ahead program */
    }
    sprintf(appName, "jpegMovieRead %d %d %d",			    /* ...launch, and its command-line arguments. */
	    currentFrame,
	    (3 * movieHeader.fps),				    /* Set the read-ahead for three seconds' worth of movie */
	    playbackSpeed);
    result = execlp(appName, appName, (char*)0);		    /* Try to launch the program and get it started */
    if (result == -1)						    /* Did the execlp fail? */
    {
      exit(0);							    /* Yes, exit this child process immediately */
    }
  }								    /* End child part of the fork */
  else								    /* This is the parent part of the fork  */
    return;
} /* end function SyncIODevice */
