/* $Header: /plx/source/Xclients/generic/2.17/src/client/motif/MovieTool/RCS/callbacks.c,v 1.65 1996/01/25 04:17:39 drapeau Exp $ */
/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Log: callbacks.c,v $
 * Revision 1.65  1996/01/25  04:17:39  drapeau
 * Added support for the PlxOutputCascade to automatically create the
 * output menu hierarchy, for enhanced flexibility.
 *
 * Revision 1.64  1995/12/09  00:14:59  lovvik
 * Fixed 3 problems:
 *   1.  Selecting full, quarter, or small size from the
 *       JPEG Compression preferences would change the
 *       size of the record AND playback windows.
 *   2.  Adjusting the Sync Slack slider would print
 *       diagnostic messages.
 *   3.  If the recording stopped due to an audio read
 *       error, the entire movie would be lost.
 *
 * Revision 1.63  1995/11/23  03:48:03  drapeau
 * Several changes:
 *
 * 1) Modified the synchronization algorithm used by the PlayWorkProc()
 * function to better handle times when the application gets far behind.
 * The new algorithm now counts long delays as pauses, the thinking being
 * that if the application is swapped out for a long period of time, it
 * might as well have been paused for that duration.  The application now
 * catches up, then adds to the total time paused for this playback.  As
 * a result, the problem in which audio lagged behind video by upwards of
 * a second has been eliminated.
 *
 * 2) Removed many extraneous local variables to eliminate compiler
 * warnings.
 *
 * 3) Fixed printf statements so that long integers are printed correctly
 * without the compiler complaining.
 *
 * Revision 1.62  1995/10/03  19:41:45  lovvik
 * Made minor changes to MonitorOn() and MonitorOff().  These
 * changes fix the following problem:  If the user turns the
 * audio monitor off and then plays a movie, MovieTool will
 * turn the audio monitor back on when the movie is finished.
 *
 * Revision 1.61  1995/10/03  18:49:53  lovvik
 * Added a function called FixMovie().  This function attempts
 * to save a movie that was stopped due to a write error.  This
 * sort of error is usually caused because there is no free
 * space on the file system.  FixMovie saves the movie by throwing
 * away the last few frames of the movie and writing the index
 * table.
 *
 * Revision 1.60  1995/10/02  21:52:55  drapeau
 * Two changes:
 *
 * 1) Enhanced the application to check if JPEG hardware is available.
 *    If not, the application is essentially an audio-only
 *    record/playback application.
 *
 * 2) Added support for keeping track of the user interface's visibility
 *    state: whether some panels are mapped or not, etc.
 *
 * Revision 1.59  1995/08/30  05:09:40  drapeau
 * Removed more IBM-specific code.
 *
 * Revision 1.58  1995/08/30  05:08:06  drapeau
 * Removed IBM-specific code for implementing synchronization "panic"
 * when the application gets too far behind.
 *
 * Revision 1.57  1995/08/23  01:56:02  drapeau
 * Several changes (all Dave Arnold changes):
 *
 * * Commented out the ChangeRecordSpeed() function for non-Sun
 * platforms, as the IBM audio device doesn't seem to have a problem with
 * frame rates that are not an integral divisor of the "sample unit".
 *
 * * Changed the synchronization scheme used by the RecordWorkProc()
 * function: if the application gets far, far behind, it will go into an
 * "immediate catchup" mode whereby it grabs all available audio
 * immediately, then writes as many empty movie frames as is necessary to
 * re-synchronize the movie's position with the real-time clock.
 *
 * Revision 1.56  1995/08/17  19:33:26  drapeau
 * Minor cosmetic change to the MonitorOn() function.
 *
 * Revision 1.55  1995/06/23  22:42:47  drapeau
 * Modified ZoomWorkProc() so that the JPEG image shell conforms exactly
 * to the size of the JPEG movie it is displaying.  The shell is not
 * allowed to resize.
 *
 * Revision 1.54  1995/06/23  19:15:03  drapeau
 * Modified PlayWorkProc() so that it correctly resets the total time
 * paused when it has reached the end of a movie and automatically stops.
 * This fixes a problem in which MovieTool would pause when starting
 * playback of a movie that had previously been paused during its last
 * playback, because the pause time had not correctly been reset.
 *
 * Also modified ZoomCallback(), moving most of the functionality into an
 * XtWorkProc so that widget destruction and creation happens immediately
 * as necessary for the proper functioning of the application.
 *
 * Revision 1.53  1995/06/14  13:49:14  lovvik
 * Fixed 2 problems:
 *   1)  After recording, the audio module remained insensitive.
 *   2)  The audio device would not get configured correctly when recording.
 * To fix the first problem, I placed a call to EnableAudioDeviceControls()
 * in Stop().  The second problem was fixed with a call to
 * SetAudioPlaybackConfiguration() in Play() and Record().
 *
 * Revision 1.52  1995/05/19  01:53:38  drapeau
 * Added interface code to indicate the current zoom ratio.
 *
 * Revision 1.51  1995/05/16  02:54:50  drapeau
 * Added full support for zoom, fixed errors in the way zoom sizes were
 * calculated for the playbackWidget.
 *
 * Revision 1.50  1995/05/12  00:13:19  lovvik
 * Added the callback ChangeAudioModule.  This callback handles
 * all of the audio module buttons.
 *
 * Revision 1.49  1995/05/10  20:07:34  drapeau
 * Merged changes from Paul Lovvik (specifically, to adjust color values
 * and reset to their defaults) with zoom changes from George Drapeau.
 *
 * Revision 1.48  1995/05/02  17:48:49  drapeau
 * Integrated changes and fixes made by Paul Lovvik that were lost by
 * George Drapeau's  latest modifications in support of zoom.
 *
 * Revision 1.47  1995/04/28  20:53:14  drapeau
 * *** empty log message ***
 *
 * Revision 1.46  1995/04/27  00:11:13  lovvik
 * Added instructions to disconnect from the audio device when not
 * playing or recording.  These instructions were added to PlayWorkProc()
 * and Stop().  When using the audio device directly, this is a no-op.
 * When using the Audio Server, this releases the server so other
 * audio applications can use the audio device.
 * Also added AutoLoadWorkProc().  This function takes care of
 * auto-loading a file and playing it.
 *
 * Revision 1.45  1995/04/07  17:15:34  lovvik
 * Added the function WMDeleteWindowCallback.
 *
 * Revision 1.44  1995/02/13  18:10:35  lovvik
 * Modified the calls to the audio module.  Some of the
 * parameter lists have changed to disinclude the audio
 * file descriptor.
 *
 * Revision 1.43  1994/10/22  02:28:08  drapeau
 * Fixed minor ANSI-C problems, conflicts with function prototypes.
 *
 * Revision 1.42  1994/10/15  01:45:55  drapeau
 * Added code to complete MovieTool's notion of what operations should be
 * allowed under what conditions.  For example, input selection should
 * not be allowed during recording and playback.
 *
 * Revision 1.41  1994/10/14  02:38:01  drapeau
 * Cleaned up errors having to do with changing the Q factor for both the
 * image quality and playback widgets.
 *
 * Revision 1.40  1994/10/10  23:00:05  drapeau
 * Added functions to allow input selection similar to that of VideoTool;
 * that is, any possible input (including YC inputs) can now be selected.
 *
 * Revision 1.39  1994/10/08  22:26:55  drapeau
 * Fixed two errors:
 *
 * 1) When running out of disk space during recording of a movie, the
 *    application did not properly clean up after itself by removing the
 *    bad movie from the filesystem.  It does so now.
 *
 * 2) When selecting a new start frame (from the "Edit" menu), sometimes
 *    the searchSlider would be set to an invalid value, causing the
 *    Motif toolkit to attempt to print a message to the terminal.  If
 *    the application is in the background (at least on the HP platform),
 *    this message cannot be printed until the app is placed back into
 *    the foreground.  This quirk causes the app to lock up.  The
 *    searchSlider error has been removed (it never gets set to an
 *    invalid value).
 *
 * Revision 1.38  1994/10/06  02:17:09  drapeau
 * Added a function to remove a file from the filesystem.
 *
 * Also, added code to disable and enable MovieTool transport interface
 * components (e.g., ffwd, rewind) and to enable and disable the output
 * feature according to the state of the application.
 *
 * Revision 1.37  1994/09/30  16:56:41  drapeau
 * Added support for output.
 *
 * Revision 1.36  1994/09/23  07:35:37  drapeau
 * Modified the Record() function: removed a hack that was put in to
 * support the OpenWindows server (not sure if the OpenWindows hack was
 * also necessary for Solaris 2.3).  The hack severely affected
 * performance for the first compression since it made the video window
 * still (causing the VideoIn widget to grab a frame as an XImage), then
 * switch back to live.  These state changes are time-consuming and
 * unnecessary (at least on the HP platform).  Removing the hack resulted
 * in a noticable performance improvement in the application.
 *
 * Revision 1.35  1994/09/23  06:41:14  drapeau
 * Fixed several errors:
 *
 * * Fixed an error in the PlayWorkProc() function: it had failed to play
 *   the last frame (off-by-one error).
 *
 * * Fixed an error in the Play() function: if the user had changed the Q
 *   factor via the Compression Preferences panel while an existing movie
 *   was open, this new Q factor would be used during playback of the
 *   existing movie, instead of the existing movie's Q factor.  Now,
 *   when the user presses the Play button, the Play function always
 *   restores the movie's Q factor, regardless of the slider's current
 *   value.
 *
 * * Minor changes throughout the playback functions to enhance audio
 *   performance (sprinkled pause and resume calls appropriately
 *   throughout).
 *
 * * Fixed an error in RecordOneFrame(), which used to use a global
 *   variable indicating which frame to store statistics into, instead of
 *   using the frame number passed in as argument.
 *
 * * Changed the printing of an error message to the terminal into a
 *   dialog box message, after which recording is stopped.
 *
 * * Removed an extraneous printf diagnostic in order to enhance record
 *   performance.
 *
 * Revision 1.34  1994/09/13  08:06:59  drapeau
 * Two changes:
 *
 * 1) Assure that the movie playback shell is popped up whenever playback
 *    is requested.
 *
 * 2) Track changes in the size of the live video widget so that the
 *    imageQuality widget grabs the whole window's contents correctly.
 *
 * Revision 1.33  1994/08/27  05:47:37  drapeau
 * Removed support for OS-specific code and placed it in OS-specific
 * modules.
 *
 * Also, slight modifications to code to disable the speed playback
 * slider during playback.
 *
 * Revision 1.32  1994/08/23  01:33:07  drapeau
 * * Removed extraneous messages to the terminal.
 *
 * * Modified ChangeRecordSpeed() to check if audio is really present; if
 *   it is not, there is no use in trying to make the record speed
 *   accommodate the audio device.
 *
 * * Modified the Stop() function for recording: if an error happened
 *   during recording then "currentFrame" would be set to zero, which
 *   could potentially cause a divide-by-zero error later in the
 *   function.  This case is now caught and prevented.
 *
 * Revision 1.31  1994/08/19  06:00:53  drapeau
 * Modified ChangeRecordSpeed() to account for frame rates that would
 * potentially cause problems with the Sun audio device (not yet sure
 * whether these same problems exist with HP audio).
 *
 * Revision 1.30  1994/08/11  03:16:05  drapeau
 * * Removed extraneous variable declarations.
 * * Added code to enhance diagnostic gathering.
 * * Added callback functions to support editable text fields in the
 *   timecode display.
 *
 * Revision 1.29  1994/06/24  01:16:02  drapeau
 * Two changes:
 *
 * 1) The audio device is now closed upon proper exit from the
 *    application.
 *
 * 2) The PlusButtonCallback() and MinusButtonCallback() functions were
 *    fixed; there were errors in the way the frame +/- buttons
 *    calculated which frame should be played next.  Behavior is now
 *    consistent with the author's expectations.
 *
 * Revision 1.28  1994/06/15  03:27:08  drapeau
 * Enhanced the code in the Record() function to maintain the ability to
 * read and write JPEG Movie files with audio that can be used with older
 * programs that also play JPEG Movies.
 *
 * Revision 1.27  1994/06/10  01:24:07  drapeau
 * Fixed errors in resizing video and JPEG shells (i.e., QuarterSize,
 * FullSize, SmallSize).
 *
 * Added code to handle the silentMovie command-line option more
 * completely, now avoiding both recording and playback of audio.
 *
 * Fixed errors in the calculation of timecode when advancing one second.
 *
 * Revision 1.26  1994/05/31  20:09:43  drapeau
 * A number of changes were made, generally falling into one of these
 * categories:
 *
 * 1) Synchronization code was enhanced.  It is now more precise, using
 *    floating point calculations instead of integer math.  Rounding
 *    errors are reduced by several orders of magnitude.
 *
 * 2) Diagnostic reporting is cleaner.
 *
 * 3) Audio monitoring and playback are cleaned up.
 *
 * Revision 1.25  1994/01/20  22:50:48  drapeau
 * Modified code to support minor change in the PortableAudio interface,
 * which was done to accommodate the Sun audio device driver
 * implementation.
 *
 * Revision 1.24  1994/01/20  17:50:16  drapeau
 * Made a number of changes:
 *
 * * Code now handles the case where the JPEG widget is not able to use
 *   shared memory.
 *
 * * Code now supports the new PortableAudio structure, making audio
 *   support more portable across platforms.
 *
 * * Removed extraneous error diagnostic messages.
 *
 * * When quitting, first checks if a valid MAEstro Sender was created;
 *   if not, don't bother trying to disconnect from the PortManager.
 *
 * Revision 1.23  1994/01/10  22:38:13  drapeau
 * Several changes to make the application more robust:
 *
 * 1) Added support for high-quality Sun audio.
 * 2) Added support for playback at speeds other than the
 *    originally-recorded speed.
 * 3) Added support for the MAEstro Multimedia Authoring Environment.
 * 4) Fixed minor errors in the gathering and reporting of diagnostic
 *    information.
 * 5) Fixed errors in synchronization code (previous versions didn't take
 *    into account the starting frame of a movie; they always assumed
 *    playback would always begin at frame 0).
 * 6) Cleaned up diagnostic reporting.
 * 7) Fixed timecode display; bad math problems.
 *
 * Revision 1.22  1993/12/04  01:02:15  drapeau
 * Modified the code to make the Pause feature more robust.  Before this
 * version, pausing a movie during playback would throw off the
 * synchronization mechanism; pause now works correctly.
 *
 * Revision 1.21  1993/12/03  02:10:24  drapeau
 * Minor change made in rcs info to allow me to change the RCS
 * numbering scheme for this file.
 * */
static char callbacksRcsid[] = "$Header: /plx/source/Xclients/generic/2.17/src/client/motif/MovieTool/RCS/callbacks.c,v 1.65 1996/01/25 04:17:39 drapeau Exp $";

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/Scale.h>
#include <Xm/PushBG.h>
#include "MovieTool.h"


void XDmanage_link (Widget w, XtPointer clientData, XtPointer call_data)
{
  XtManageChild ( *(Widget *)clientData );
  SetMTPanelState(&mtPanelState, *(Widget*)clientData, True);	    /* Inform the application that this panel is now visible */
} /* end function XDmanage_link */



void XDunmanage_link (Widget w, XtPointer clientData, XtPointer call_data)
{
  XtUnmanageChild ( *(Widget *)clientData );
  SetMTPanelState(&mtPanelState, *(Widget*)clientData, False);	    /* Inform the application that this panel is now invisible */
} /* end function XDunmanage_link */



void XDpopup_link (Widget w, XtPointer clientData, XtPointer call_data)
{
  XtPopup ( *(Widget *)clientData, XtGrabNone );
  SetMTPanelState(&mtPanelState, *(Widget*)clientData, True);	    /* Inform the application that this panel is now visible */
} /* end function XDpopup_link */



void XDpopdown_link (Widget w, XtPointer clientData, XtPointer call_data)
{
  XtPopdown ( *(Widget *)clientData );
  SetMTPanelState(&mtPanelState, *(Widget*)clientData, False);	    /* Inform the application that this panel is now invisible */
} /* end function XDpopdown_link */



void XDmap_link (Widget w, XtPointer clientData, XtPointer call_data)
{
  XtMapWidget (*(Widget *)clientData);
  SetMTPanelState(&mtPanelState, *(Widget*)clientData, True);	    /* Inform the application that this panel is now visible */
} /* end function XDmap_link */



void XDunmap_link (Widget w, XtPointer clientData, XtPointer call_data)
{
  XtUnmapWidget (*(Widget *)clientData);
  SetMTPanelState(&mtPanelState, *(Widget*)clientData, False);	    /* Inform the application that this panel is now invisible */
} /* end function XDunmap_link */



void XDenable_link (Widget w, XtPointer client_data, XtPointer call_data)
{
  XtSetSensitive (*(Widget *)client_data, TRUE);
}

void XDdisable_link (Widget w, XtPointer client_data, XtPointer call_data)
{
  XtSetSensitive (*(Widget *)client_data, FALSE);
}

extern void XDhelp_link (Widget w, XtPointer client_data, XtPointer call_data);



void MonitorOn(Widget		w,
	       XtPointer	client_data,
	       XtPointer	xt_call_data)
{
  if(w != (Widget)NULL)						    /* User turned monitor on again. */
  {
    userTurnedOffMonitor = False;
  }
  
  if(userTurnedOffMonitor == False)
  {
    SetAudioMonitorVolume(lastMonitorVolume);			    /* Restore monitor volume to last value */
    XmToggleButtonSetState(monitorOnToggle, True, False);		    /* Make the "Monitor On" toggle show and the "Monitor... */
    XmToggleButtonSetState(monitorOffToggle, False, False);	    /* ...Off" toggle not show. */
  }
}



void MonitorOff(Widget		w,
		XtPointer	client_data,
		XtPointer	xt_call_data)
{
  if (w != (Widget)NULL)					    /* Determine whether this function was called by the... */
    userTurnedOffMonitor = True;				    /* ...app itself or by the person using the app. */
  SetAudioMonitorVolume(0);				    /* Disable monitor output for now */
  XmToggleButtonSetState(monitorOffToggle, True, False);	    /* Make the "Monitor Off" toggle show and the "Monitor... */
  XmToggleButtonSetState(monitorOnToggle, False, False);	    /* ...On" toggle not show. */
}



void ChangePlaybackSpeed(Widget		w,
			 XtPointer	client_data,
			 XtPointer	xt_call_data)
{
  XmScaleCallbackStruct*	callData =
    (XmScaleCallbackStruct *) xt_call_data;
  
  playbackSpeed = callData->value;
  return;
}



void ChangeRecordSpeed(Widget		w,
		       XtPointer	client_data,
		       XtPointer	xt_call_data)
{
  int				audioBytesPerSecond;
  int				audioFrameSize;
  XmScaleCallbackStruct*	callData =
    (XmScaleCallbackStruct *) xt_call_data;
  int				frameSize;
  
  if (audioStatus == -1)					    /* No use trying to make the recording speed cooperate... */
    return;							    /* ...with the audio sample rate if audio isn't present. */
  recordFrameRate = callData->value;				    /* Note the change in recording frame rate */

  frameSize = audioRecordHeader.channels *			    /* Determine the "frame size" for sampling in Sun audio */
    audioRecordHeader.bytes_per_unit;				    /* (the Sun audio device only allows data in multiples... */
								    /* ...of the frame size to be written to it; any other... */
								    /* ...number of bytes is illegal and hoses the device). */
  audioBytesPerSecond = audioInfo.play.sampleRate *		    /* Determine how much bandwidth is necessary for one... */
    audioInfo.play.channels *					    /* ...second of audio. */
      audioInfo.play.bitsPerSample / 8;
  audioFrameSize = audioBytesPerSecond / recordFrameRate;	    /* Determine how much bandwidth for a single frame of audio */
  
  if ((audioFrameSize % frameSize) != 0)
  {
    for (recordFrameRate = callData->value;			    /* Try increasing the frame rate until a desired one... */
	 recordFrameRate < signalFrameRate;			    /* ...is found, if possible */
	 recordFrameRate++)
    {
      audioFrameSize = audioBytesPerSecond / recordFrameRate;	    /* Determine how much bandwidth for a single frame of audio */
      if ((audioFrameSize % frameSize) == 0)
	break;
    }
  }
  if ((audioFrameSize % frameSize) != 0)			    /* Was the new frame rate still unsatisfactory? */
  {
    for (recordFrameRate = signalFrameRate;			    /* Yes, try *decreasing* the frame rate until a desired... */
	 recordFrameRate > 1;					    /* ...one is found, if possible */
	 recordFrameRate--)
    {
      audioFrameSize = audioBytesPerSecond / recordFrameRate;	    /* Determine how much bandwidth for a single frame of audio */
      if ((audioFrameSize % frameSize) == 0)
	break;
    }
  }
  if (recordFrameRate != callData->value)			    /* Was it necessary to override the user's choice of... */
  {								    /* ...recording frame rate? */
    XtVaSetValues(recordSpeedSlider,				    /* Yes, forcibly update the Record Speed slider to... */
		XmNvalue, recordFrameRate, NULL);		    /* ...reflect the acceptable recording frame rate. */
  }
  return;
} /* end function ChangeRecordSpeed */



void ChangeImageQuality(Widget		w,
			XtPointer	client_data,
			XtPointer	xt_call_data)
{
  Dimension			width;
  Dimension			height;
  XmScaleCallbackStruct*	callData =
    (XmScaleCallbackStruct *) xt_call_data;
  
  XtVaGetValues(input1Widget,
		XtNwidth, &width,
		XtNheight, &height,
		NULL);
  imageQualitySourceRect.width = width;
  imageQualitySourceRect.height = height;
  imageQualityDisplaySourceRect.x = 0;
  imageQualityDisplaySourceRect.y = 0;
  imageQualityDisplaySourceRect.width = width / zoomFactor;
  imageQualityDisplaySourceRect.height = height / zoomFactor;
  
  XtVaSetValues(imageQualityWidget,				    /* Display the results of the quality adjustment */
		XtNsourceWin, XtWindow(input1Widget),
		XtNqFactor, callData->value,
		XtNdisplaySourceRect, &imageQualityDisplaySourceRect,
		XtNgetNewImage, True,
		XtNputNewImage, True,
		NULL);
} /* end function ChangeImageQuality */



void FullSize(Widget	w,
	      XtPointer	client_data,
	      XtPointer	xt_call_data)
{
  Dimension			signalWidth;
  Dimension			signalHeight;
  
  XtVaGetValues(input1Widget,					    /* Get the full size of this video widget's input signal */
		XtNsignalWidth, &signalWidth,
		XtNsignalHeight, &signalHeight,
		NULL);
  XtVaSetValues(input1Widget,					    /* Make the video widget full-size */
		XtNwidth, signalWidth,
		XtNheight, signalHeight,
		NULL);
  return;
} /* end function FullSize */



void QuarterSize(Widget w,
		 XtPointer client_data,
		 XtPointer xt_call_data)
{
  Dimension			signalWidth;
  Dimension			signalHeight;
  
  XtVaGetValues(input1Widget,					    /* Get the full size of this video widget's input signal */
		XtNsignalWidth, &signalWidth,
		XtNsignalHeight, &signalHeight,
		NULL);
  XtVaSetValues(input1Widget,					    /* Make the video widget quarter-size */
		XtNwidth, (signalWidth / 2),
		XtNheight, (signalHeight / 2),
		NULL);
  return;
} /* end function QuarterSize */



void SmallSize(Widget		w,
	       XtPointer	client_data,
	       XtPointer	xt_call_data)
{
  Dimension			signalWidth;
  Dimension			signalHeight;
  
  XtVaGetValues(input1Widget,					    /* Get the full size of this video widget's input signal */
		XtNsignalWidth, &signalWidth,
		XtNsignalHeight, &signalHeight,
		NULL);
  XtVaSetValues(input1Widget,					    /* Make the video widget sixteenth-size */
		XtNwidth, (signalWidth / 4),
		XtNheight, (signalHeight / 4),
		NULL);
  return;
} /* end function SmallSize */



void 
PerformRecordDiagnostics (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  return;
}



Boolean QuitWorkProc(XtPointer unusedClientData)
{
/*  printf("QuitWorkProc(): Should check if the movie needs saving before quitting.\n"); */

  SetAudioMonitorVolume(0);					    /* Disable monitor output for now */
  AudioClose();							    /* Close the audio device */

  if (sender != (Sender*)NULL)
    SenderDisconnectFromPortMgr(sender, &(receiver->receivePort));  /* Unregister MovieTool's services with the PortManager */

  XtDestroyWidget(mainWindow);

  exit(0);
} /* end function QuitWorkProc */



void Quit(Widget	w,
	  XtPointer	client_data,
	  XtPointer	xt_call_data)
{
  XtWorkProcId	quitProcID;

  quitProcID = XtAppAddWorkProc(app_context,			    /* Start a work-proc to quit the application; Xt is... */
				QuitWorkProc,			    /* ...poorly designed and this is the only way to assure... */
				(XtPointer)NULL);		    /* ...that shared memory will be freed from the JPEG... */
  return;							    /* ...widgets. */
} /* end function Quit */



Boolean PlayOneFrame(int	whichFrame,
		     Boolean	playAudio,
		     Boolean	playVideo)
{
  struct timeval	after;
  int			audioBytesWritten;
  struct timeval	before;
  Boolean		done = False;
  unsigned char		instruction;
  struct timeval	now;
  
  if (whichFrame <= playbackEndFrame)				    /* Was a valid frame specified? */
  {								    /* Yes, play one frame of the movie */
    gettimeofday(&now, (struct timezone*)NULL);			    /* Clock when this function was called */
    playInterval[whichFrame] = ElapsedTime(&playbackStartTime,	    /* Determine how long it has been since the start of the... */
					   &now);		    /* ...movie. */
    fseek(movieFile, frameLocation[whichFrame], 0);		    /* Go to the start of the desired frame */
    while (done == False)					    /* Keep reading until all the instructions in this frame... */
    {								    /* ...have been executed. */
      fread(&instruction, 1, 1, movieFile);			    /* Read the 1st opcode for this frame */
      instruction &= 0x00FF;
      switch ((int)instruction)					    /* Do the appropriate action for this opcode */
      {
       case END_FRAME:						    /* Was the end of this frame reached? */
	done = True;						    /* Yes, set a variable saying so */
	UpdateTimecodeDisplay(movieHeader.fps);
	break;
       case LOAD_JPEG:
	videoFrameIndex[whichFrame] = 0x01;			    /* Note that this frame had a video image in it */
	fread((char*)&(jpegImage->size), sizeof(int), 1, movieFile); /* Read the size of this JPEG image in bytes */
	if (playVideo == True)					    /* Is video supposed to be shown for this frame? */
	{							    /* Yes, do the right thing */
	  fread(jpegImage->data, jpegImage->size, 1, movieFile);    /* Read the JPEG image data themselves */
	  XtVaSetValues(playbackWidget,				    /* Tell the widget to render the new image to the screen now */
			XtNputNewImage, True,
			NULL);
	}
	else
	  fseek(movieFile, jpegImage->size, 1);			    /* Skip past the JPEG image information for this frame */
	break;
       case LOAD_AUDIO_0:
       case LOAD_AUDIO_1:
       case LOAD_AUDIO_2:
       case LOAD_AUDIO_3:
	if ((playAudio == True) && (silentMovie == False))	    /* Is audio supposed to be played in this frame? */
	{							    /* Yes, do the right thing */
	  fread(audioData, 1, movieHeader.audioslice, movieFile);   /* Read the audio data for this frame */
	  gettimeofday(&before, (struct timezone*)NULL);
          audioBytesWritten = AudioWrite(audioData,                 /* Playback the audio for this frame */
					 movieHeader.audioslice);   
	  if (audioBytesWritten < 0)				    /* Was there an error playing one frame of audio? */
	  {							    /* Yes, report the error and stop movie playback now */
	    sprintf(errorMessage,
		    "There was an error during playback trying to write %d bytes to the audio device.  Errno is %d.\n",
		    movieHeader.audioslice, errno);
	    PrintErrorMessage(errorMessage);
	    return(True);					    /* Tell the toolkit to halt playback right now */
	  }
	  gettimeofday(&after, (struct timezone*)NULL);
	  audioWriteTime[whichFrame] =				    /* Update stats on how long this frame's audio playback took */
	    ElapsedTime(&before, &after);
	}
	else
	{
	  fseek(movieFile, movieHeader.audioslice, 1);		    /* Skip past the audio data for this frame */
	}
	break;
       default:
/*
	printf("An unknown instruction (%d) was found in frame %d of the movie, skipping.\n",
	       instruction, whichFrame);
	printf("Should stop reporting this error after, say, 10 occurrances.\n");
*/
	break;
      }	/* end switch */
    } /* end while */
    return(False);						    /* Tell the toolkit to call this function again */
  }
  else								    /* No, there is no more movie left to play */
    return(True);						    /* Tell the toolkit not to bother calling again */
} /* end function PlayOneFrame */



void Search(Widget	w,
	    XtPointer	client_data,
	    XtPointer	xt_call_data)
{
  XmScaleCallbackStruct*	callData =
    (XmScaleCallbackStruct *) xt_call_data;
  
  if ((movieToolStatus != Paused) &&				    /* Search should not work unless a movie is either paused... */
      (movieToolStatus != Stopped))				    /* ...or stopped completely. */
    return;
  currentFrame = callData->value + firstFrame;			    /* Go to requested frame, taking into acct that movie...  */
								    /* ...may not begin at the 1st frame. */
  PlayOneFrame(currentFrame, False, True);			    /* Play a single frame from the movie, without audio */
} /* end function Search */




long	ElapsedTime(struct timeval*	before,
		    struct timeval*	after)
{
  return(((after->tv_sec - before->tv_sec) * 1000) +		    /* Convert the seconds part to milliseconds */
	 ((after->tv_usec - before->tv_usec) / 1000));		    /* Convert the microseconds part to milliseconds */
} /* end function ElapsedTime */



void PrintPlaybackStatistics(struct timeval*	stopTime,
			     jpheader*		movieHeaderInfo)
{
  int	counter;
  float	delta;
  float	timeBetweenFrames;
  float	timeForOneFrame;
  long	totalTimeTaken;
  long	totalNumberFramesPlayed;
  float	variance;
  
  if (printDiagnostics == False)				    /* Don't bother printing performance stats if not requested */
    return;
  totalTimeTaken = ElapsedTime(&playbackStartTime, stopTime);	    /* Calculate total milliseconds to play this clip */
  totalNumberFramesPlayed =					    /* Calculate total number of frames played in this clip */
    (currentFrame + 1) - playbackStartFrame;
  printf("\nPlayback Performance Information:\n");
  printf("Average frame rate: %.1f frames per second.\n",
	 (float)totalNumberFramesPlayed * 1000.0 / (float)totalTimeTaken);
  printf("Number of video frames skipped due to lagging: %d (%ld percent of total).\n",
	 framesSkippedDuringPlayback,
	 framesSkippedDuringPlayback * 100 / totalNumberFramesPlayed);
  for (counter = 0;						    /* Report on each frame skipped during playback */
       counter < MaxMovieDurationInFrames;
       counter++)
  {
    if (skippedFrameIndex[counter] != 0.0)
      printf("Frame %d was %.0f milliseconds (approx. %.1f frames) late.\n",
	     counter, skippedFrameIndex[counter],
	     skippedFrameIndex[counter] /
	     (float)(1000 / playbackSpeed));
  }
  printf("\n\nTime to play audio for each frame (only times greater than 1 msec are reported):\n");
  for (counter = 0;
       counter < MaxMovieDurationInFrames;
       counter++)
  {
    if (audioWriteTime[counter] > 1)
      printf("Frame %d: %ld milliseconds.\n", counter,
	     audioWriteTime[counter]);
  }
  printf("-------------\n\n\n");
  return;							    /* Don't bother printing the rest of the playback stats */
  
  timeForOneFrame = 1000.0 / playbackSpeed;			    /* Calculate # of millisecs to play one frame of this movie */
  delta = timeForOneFrame * .1;
  printf("\n\nTime passed between frames (only variances greater than 10%% (%.1f ms) are reported:\n",
	 delta);
  for (counter = 0;
       counter < MaxMovieDurationInFrames;
       counter++)
  {
    timeBetweenFrames = (counter == 0) ? playInterval[counter] :    /* Determine how many milliseconds passed b/w 2... */
      playInterval[counter] - playInterval[counter - 1];	    /* ...successive frames. */
    variance = fabs((double)(timeBetweenFrames) -		    /* Calculate how far off ideal frame rate this frame was... */
		    (double)timeForOneFrame);			    /* ...started. */
    if ((variance > delta) &&					    /* Was variance greater than allowed, and was the frame... */
	(playInterval[counter] !=0))				    /* ...actually played? (i.e., non-zero) If so, report it. */
    {
      if (counter == 0)						    /* Is this the 1st frame? */
	printf("1st Frame: %ld ms.\n", playInterval[counter]);	    /* Yes, report only the 1st frame's stats */
      else							    /* No, report each subsequent... */
	printf("Frame %d: %ld ms.\n",				    /* ...frame's start time and report that difference. */
	       counter,
	       playInterval[counter] - playInterval[counter-1]);
      continue;
    }
  }
/*
  printf("\nFrames in this movie with no video:\n");
  for (counter = firstFrame; counter < currentFrame; counter++)
  {
    if (videoFrameIndex[counter] == 0x00)
      printf("Frame %d.\n", counter);
  }
*/
  printf("-------------\n\n\n");
  return;
} /* end function PrintPlaybackStatistics */



void PrintRecordStatistics(struct timeval*	stopTime,
			   jpheader*		movieHeaderInfo)
{
  long	totalTimeTaken;
  long	totalNumberFramesRecorded;
  int	counter;
  
  if (printDiagnostics == False)				    /* Don't bother printing performance stats if not requested */
    return;
  totalTimeTaken = ElapsedTime(&recordStartTime, stopTime);	    /* Calculate total milliseconds to record this movie */
  totalNumberFramesRecorded = currentFrame;			    /* Calculate total number of frames recorded in this movie */
  printf("\nRecord Performance Information:\n");
  printf("Average frame rate: %.1f frames per second.\n",
	 (float)totalNumberFramesRecorded * 1000.0 / (float)totalTimeTaken);
  printf("Number of video frames skipped due to lagging: %d (%ld percent of total).\n",
	 framesSkippedDuringRecording,
	 framesSkippedDuringRecording * 100 / totalNumberFramesRecorded);
  for (counter = 0;						    /* Report on each frame skipped during recording */
       counter < MaxMovieDurationInFrames;
       counter++)
  {
    if (recordSkippedFrameIndex[counter] != 0.0)
      printf("Frame %d was %.0f milliseconds (approx. %.1f frames) late.\n",
	     counter, recordSkippedFrameIndex[counter],
	     recordSkippedFrameIndex[counter] /
	     (float)(1000.0 / recordFrameRate));
  }
  printf("\n\nTime to record audio for each frame:\n");
  for (counter = 0; counter < currentFrame; counter++)		    /* Print stats for audio recording time */
  {
    if (audioReadTime[counter] != 0)
      printf("Frame %d: %ld milliseconds.\n", counter,
	     audioReadTime[counter]);
  }
  printf("\n\nTime to compress each frame:\n");
  for (counter = 0; counter < currentFrame; counter++)		    /* Print stats for video compression time */
  {
    if (compressTime[counter] != 0)
      printf("Frame %d: %ld milliseconds.\n", counter,
	     compressTime[counter]);
  }
  printf("\n\nTime to write each frame to disk:\n");
  for (counter = 0; counter < currentFrame; counter++)		    /* Print stats for disk write time */
  {
    if (diskWriteTime[counter] != 0)
      printf("Frame %d: %ld milliseconds.\n", counter,
	     diskWriteTime[counter]);
  }
  return;
} /* end function PrintRecordStatistics */



Boolean PlayWorkProc(XtPointer clientData)
{
  struct timeval	now;
  struct timeval	playbackStopTime;
  long			playbackClock;
  float			scheduledTime;
  float			timeForOneFrame;
  Boolean		result;
   
  if (movieToolStatus == Paused)				    /* Is the app in pause mode? */
    return(False);						    /* Yes, don't do anything */
  if (movieToolStatus == Stopped)
    return(True);
  timeForOneFrame = 1000.0 / playbackSpeed;			    /* Calculate # of millisecs to play one frame of this movie */
  gettimeofday(&now, (struct timezone*)NULL);
  playbackClock = ElapsedTime(&playbackStartTime, &now);	    /* Determine the time relative to the start of the movie */
  scheduledTime = (currentFrame - playbackStartFrame) *		    /* Determine proper relative starting time for this frame */
    timeForOneFrame;
  if (((float)(playbackClock - totalTimePaused) >		    /* Is it past time to play this frame? (i.e., is the... */
      (scheduledTime + timeForOneFrame)) &&			    /* ...movie running late?  It can be up to 1 frame late) */
      (currentFrame < playbackEndFrame))			    /* And, is there still some movie left to play? */
  {
    framesSkippedDuringPlayback++;				    /* Yes, note that another video frame needed to be skipped */
    if (playbackSpeed == movieHeader.fps)			    /* Is the movie playing at its natural speed? */
      result = PlayOneFrame(currentFrame, True, False);		    /* Yes, play the audio for this frame but skip the video */
    else							    /* No, turn off audio during playback */
      result = PlayOneFrame(currentFrame, False, False);	    /* Skip both audio and video for this frame */
    skippedFrameIndex[currentFrame] =				    /* Note how late and for which frame this skip occurred */
      ((float)(playbackClock - totalTimePaused)) - scheduledTime;

    if (skippedFrameIndex[currentFrame] > (syncSlack * 1.0))	    /* Was the lost time greater than an acceptable number... */
    {								    /* ...of milliseconds? */
      totalTimePaused += skippedFrameIndex[currentFrame];	    /* Yes, write the time off as a loss and move on */
    }
    currentFrame++;						    /* Advance to the next frame in the movie */
    return(False);
  }
  if ((scheduledTime - (float)(playbackClock - totalTimePaused)) >  /* Is it too early to play this frame?  More than one... */
      timeForOneFrame)						    /* ...frame is too early. */
  {
    return(False);						    /* Yes, return without doing anything */
  }
  if (currentFrame <= playbackEndFrame)				    /* Is there still some movie left to play? */
  {								    /* Yes, play one frame of the movie if there's time */
    if (playbackSpeed == movieHeader.fps)			    /* Is the movie playing at its natural speed? */
      result = PlayOneFrame(currentFrame, True, jpegAvailable);	    /* Yes, play both audio and video for this frame, if JPEG... */
								    /* ...hardware is available. */
    else							    /* No, turn off audio during playback but go ahead and... */
      result = PlayOneFrame(currentFrame, False, jpegAvailable);    /* ...play the video for this frame. */
    currentFrame++;						    /* Advance to the next frame in the movie */
  }
  else								    /* No, there is no more movie left to play */
  {
    result = True;						    /* Done playing this movie */
    totalTimePaused = 0;					    /* Reset the total time paused */
    currentFrame = playbackEndFrame;				    /* Move the currentFrame one frame back so it's not past... */
								    /* ...the end of the movie. */
    if (autoQuit == True)					    /* Should this application quit right after playback is... */
    {								    /* ...done? */
      Quit((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);	    /* Yes, quit the app. immediately */
    }
    gettimeofday(&playbackStopTime, (struct timezone*)NULL);	    /* Record the time this movie playback was stopped */
    PrintPlaybackStatistics(&playbackStopTime, &movieHeader);	    /* Print info about playback performance */
    if (autoRepeat == True)					    /* Should the movie go back to the start and play again? */
    {								    /* Yes, reset the movie and get ready to play again */
      playbackStartFrame = firstFrame;
      currentFrame = firstFrame;
      result = False;
      gettimeofday(&playbackStartTime, (struct timezone*)NULL);	    /* Record the time this movie playback was started */
      framesSkippedDuringPlayback = 0;				    /* Reset counter of number of video frames that were omitted */
      memset(skippedFrameIndex, 0, MaxMovieDurationInFrames *	    /* Re-initialize the skipped frames index before playback */
	     sizeof(float));
      return(result);						    /* Return now, without shutting off movie playback */
    }
    if(movieToolStatus == Playing)
    {
      AudioStopPlayDevice();
      EnableAudioDeviceControls();
    }
    if(movieToolStatus == Recording)
    {
      AudioStopRecordDevice();
    }
    movieToolStatus = Stopped;					    /* Update the playback/record status of the app */
    EnableTransportButtons();
    if (userTurnedOffMonitor == False)				    /* Did the application itself disable the monitor during... */
      MonitorOn((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);    /* ...playback?  If so, turn it on again now that... */
  }								    /* ...playback is done. */
  return(result);
} /* end function PlayWorkProc */



void Play(Widget w,
	  XtPointer client_data,
	  XtPointer xt_call_data )
{
  if (movieToolStatus == Playing)				    /* Is a movie already playing?  If so, don't do anything */
    return;
  if (movieToolStatus == Stopped)				    /* Is a movie stopped?  If so, play from the start */
  {
    if (movieStatus == MovieNew)				    /* Does this movie have anything to play yet? */
      return;							    /* No, don't bother doing any work */
    MonitorOff((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);	    /* Make sure the monitor is turned off during playback. */
    movieToolStatus = Playing;					    /* Tell the application that it's playing a movie */
    if (True == jpegAvailable)					    /* Only bother showing the movie shell if JPEG hardware... */
      XDpopup_link(movieShell, &movieShell, movieShell);	    /* ...is available for decoding the JPEG data. */
    framesSkippedDuringPlayback = 0;				    /* Reset counter of number of video frames that were omitted */
    if (currentFrame >= playbackEndFrame)			    /* Is the app sitting at the end of a movie? */
    {								    /* Yes, assume that the viewer wishes to view the whole... */
      playbackStartFrame = firstFrame;				    /* ...movie; set the appropriate variables. */
      currentFrame = firstFrame;
    }
    else
      playbackStartFrame = currentFrame;			    /* Remember which frame began playback */
    memset(skippedFrameIndex, 0, MaxMovieDurationInFrames *	    /* Initialize the skipped frames index before playback */
	   sizeof(float));
    memset(audioWriteTime, 0, MaxMovieDurationInFrames *	    /* Initialize the audio write time array before playback */
	   sizeof(long));
    memset(playInterval, 0, MaxMovieDurationInFrames *		    /* Initialize time passed b/w frames array before playback */
	   sizeof(long));
    XtVaSetValues(speedSlider, XmNsensitive, False, NULL);	    /* Don't allow playback speed to change during playback */
    SetAudioPlaybackConfiguration(&audioPlayHeader);                /* Set the audio device for the current play configuration */
    playbackWorkProcID = XtAppAddWorkProc(app_context,		    /* Start a work-proc to play a movie; it's done this way... */
					  PlayWorkProc,		    /* ..so the user interface still works and is not blocked. */
					  (XtPointer)NULL);
    DeSensitizePlaybackControls();				    /* Disable controls for shuttling through the movie */
    DisableAudioDeviceControls();
    XtVaSetValues(inputMenuCascade, XmNsensitive, False, NULL);	    /* Don't allow input selection during playback */
    XtVaSetValues(searchSlider, XmNsensitive, True, NULL);	    /* Enable the search slider so it doesn't blink so much */
    SyncAudioPlaybackDevice();  				    /* Try to synchronize audio device */
    gettimeofday(&playbackStartTime, (struct timezone*)NULL);	    /* Record the time this movie playback was started */
  }
  else if (movieToolStatus == Paused)				    /* The app is in pause mode; resume playback */
  {
    movieToolStatus = Playing;
    XtVaSetValues(speedSlider, XmNsensitive, False, NULL);	    /* Don't allow playback speed to change during playback */
    UpdateTotalTimePaused(False);				    /* Tell the pause counter to update total time paused */
    AudioPlayResume();           				    /* Get audio device ready to play right now */
  }
  return;
} /* end function Play */



void Pause(Widget	w,
	   XtPointer	client_data,
	   XtPointer	xt_call_data )
{
  if (movieToolStatus == Paused)				    /* Is the app currently in pause mode? */
  {								    /* Yes, resume playback and get outtahere right now */
    movieToolStatus = Playing;
    UpdateTotalTimePaused(False);				    /* Tell the pause counter to update total time paused */
    AudioPlayResume();          				    /* Get audio device ready to play right now */
    return;
  }
  if (movieToolStatus == Stopped)				    /* Is the app stopped?  I.e., no movie is playing at all */
    return;							    /* Yes, Pause is not applicable; return now */
  if (movieToolStatus == Recording)				    /* is the app currently making a new movie?  If so, it... */
    return;							    /* ...cannot be interrupted so return now. */
  else								    /* The app is in a pausable state, like Playing */
  {
    movieToolStatus = Paused;					    /* Indicate that the app should pause */
    UpdateTotalTimePaused(True);				    /* Tell the paused counter to begin a new pause timer */
    AudioPlayPause();	                			    /* Pause the audio buffer to clear out unwanted audio data */
    return;
  }
} /* end function Pause */



Boolean RecordOneFrame(int	whichFrame,
		       Boolean	recordAudio,
		       Boolean	recordVideo)
{
  unsigned char		instruction;
  struct timeval	before;
  struct timeval	after;
  int			bytesWritten;
  
  recordFrameIndex[whichFrame] = totalBytesInRecording;		    /* Store the starting byte of this frame */
  if ((recordAudio == True) && (silentMovie == False))		    /* Should audio be recorded for this frame? */
  {
    instruction = LOAD_AUDIO_0;					    /* Prepare to write audio information for this frame to disk */
    bytesWritten = write(outputFile, &instruction, 1);		    /* Try to write the opcode to disk */
    if (bytesWritten < 0)					    /* Did the attempt fail? */
    {								    /* Yes, report the error and return a failed status */
      sprintf(errorMessage,
	      "Could not write the LOAD_AUDIO_0 instruction to disk.  Recording stopped.\n");
      PrintErrorMessage(errorMessage);
      return(True);						    /* Done recording */
    }
    totalBytesInRecording += bytesWritten;			    /* Update size of this movie in bytes */
    bytesWritten = write(outputFile, audioData, audioBytesPerFrame); /* Attempt to write 1 frame's worth of audio to disk */
    if (bytesWritten < 0)					    /* Did the attempt fail? */
    {								    /* Yes, report the error and return a failed status */
      sprintf(errorMessage,
	      "There was an error recording audio to disk.  Recording stopped.\n");
      PrintErrorMessage(errorMessage);
      return(True);						    /* Done recording */
    }
    totalBytesInRecording += bytesWritten;			    /* Update size of this movie in bytes */
  } /* end if recording audio should be done */
  if (recordVideo == True)					    /* Should video be recorded for this frame? */
  {
    instruction = LOAD_JPEG;					    /* Prepare to write JPEG info for this frame to disk */
    gettimeofday(&before, (struct timezone*)NULL);
    bytesWritten = write(outputFile, &instruction, 1);		    /* Try to write the opcode to disk */
    if (bytesWritten < 0)					    /* Did the attempt fail? */
    {								    /* Yes, report the error and return a failed status */
      sprintf(errorMessage,
	      "There was an error recording frame %d's JPEG image to disk.  Recording stopped.\n",
	      whichFrame);
      PrintErrorMessage(errorMessage);
      return(True);						    /* Done recording */
    }
    totalBytesInRecording += bytesWritten;			    /* Update size of this movie in bytes */
    bytesWritten = write(outputFile, &(jpegImage->size),	    /* Write the size in bytes of this frame's JPEG image data */
			 sizeof(int));
    if (bytesWritten < sizeof(int))				    /* Did the attempt fail? */
    {								    /* Yes, report the error and return a failed status */
      sprintf(errorMessage,
	      "There was an error recording this image's size information to disk.  Recording stopped.\n");
      PrintErrorMessage(errorMessage);
      return(True);						    /* Done recording */
    }
    totalBytesInRecording += bytesWritten;			    /* Update size of this movie in bytes */
    bytesWritten = write(outputFile, jpegImage->data, jpegImage->size); /* Write the JPEG image data for this frame */
    if (bytesWritten < jpegImage->size)				    /* Did the attempt fail? */
    {								    /* Yes, report the error and return a failed status */
      sprintf(errorMessage,
	      "There was an error recording the image for frame number %d to disk.  Recording stopped.\n",
	      whichFrame);
      PrintErrorMessage(errorMessage);
      return(True);						    /* Done recording */
    }
    totalBytesInRecording += bytesWritten;			    /* Update size of this movie in bytes */
    gettimeofday(&after, (struct timezone*)NULL);
    diskWriteTime[whichFrame] = ElapsedTime(&before, &after);	    /* Record time taken to write this video frame to disk */
  } /* end if recording video should be done */
  instruction = END_FRAME;					    /* Indicate that this is the end of this frame */
  bytesWritten = write(outputFile, &instruction, 1);		    /* Try to write the opcode to disk */
  if (bytesWritten < 0)						    /* Did the attempt fail? */
  {								    /* Yes, report the error and return a failed status */
    sprintf(errorMessage,
	    "There was an error writing the END_FRAME opcode to disk.  Recording stopped.\n");
    PrintErrorMessage(errorMessage);
    return(True);						    /* Done recording */
  }
  totalBytesInRecording += bytesWritten;			    /* Update size of this movie in bytes */
  UpdateTimecodeDisplay(recordFrameRate);
  return(False);						    /* Recording was fine, it's okay to continue recording */
} /* end function RecordOneFrame */



int FixMovie(void)
{

  /* 
   * This function is only called if the record operation failed.
   * FixMovie() will attempt to fix the failed movie by backing
   * up and writing the index table at the end of the file.  This
   * function returns True on success and False on failure.
   */

  int framesToRemove;
  long freeSpace;
  long result;
  
  /*
   * The distance we have to back up depends on the number
   * of frames recorded (stored in currentFrame).  Since the
   * current frame caused the error, we need to back up one
   * frame.
   */
  if(currentFrame > 0)
  {
    currentFrame --;
  }
  
  /*
   * The size of the last frame can be calculated by looking at
   * the index buffer.  Subtract the position of the last frame
   * recordFrameIndex[currentFrame]) from the current file
   * position.
   */

  framesToRemove = 0;
  freeSpace = totalBytesInRecording - recordFrameIndex[currentFrame];
  
  while(freeSpace <= (sizeof(int) * (currentFrame + 1)))
  {
    /*
     * If this isn't enough, back up another frame.
     */
    framesToRemove++;
    if(framesToRemove >= currentFrame)
    {
      /*
       * Not enough frames recorded to save this movie.
       */
      return(False);
    }
    freeSpace = totalBytesInRecording - recordFrameIndex[currentFrame-framesToRemove];
  }
  currentFrame -= (framesToRemove);
  totalBytesInRecording = recordFrameIndex[currentFrame + 1];
  /*
   * Back up to the position where the index table should be written.
   */
  result = lseek(outputFile, (long)recordFrameIndex[currentFrame + 1], SEEK_SET);
  if(result < 0)
  {
    return(False);
  }
  else
  {
    return(True);
  }
}



Boolean RecordWorkProc(XtPointer clientData)
{
  int			audioBytesRead;
  struct timeval	now;
  struct timeval	before;
  struct timeval	after;
  long			recordClock;
  float			scheduledTime;
  float			timeForOneFrame;
  Boolean		result = False;
  Boolean		hasVideo;
  
  timeForOneFrame = 1000.0 / recordFrameRate;			    /* Calculate # of millisecs to record a frame of a movie */
  gettimeofday(&now, (struct timezone*)NULL);
  recordClock = ElapsedTime(&recordStartTime, &now);		    /* Calculate time relative to the start of the recording */
  scheduledTime = currentFrame * timeForOneFrame;		    /* Calculate proper (relative) starting time for this frame */
  if ((scheduledTime - (float)recordClock) >			    /* Is recording ahead of schedule at all?  The assumption... */
      (-audioDelayInFrames * timeForOneFrame))			    /* ...is that recording shouldn't begin until after the... */
  {								    /* ...time for that frame has passed; otherwise, there's... */
    return(False);						    /* ...no audio ready to read. */
  }
  gettimeofday(&before, (struct timezone*)NULL);
  if (silentMovie == False)					    /* Is it okay to record audio? */
  {								    /* Yes, read audio from the audio device */
    audioBytesRead = AudioRead(audioData, audioBytesPerFrame);
    if (audioBytesRead < 0)
    {
      AudioRecordResume();					    /* Try to reset the audio device so record can resume */
      audioBytesRead = AudioRead(audioData, audioBytesPerFrame);    /* Read one frame's worth of audio from the audio device */
      sprintf(errorMessage,
	      "Recording was stopped due to a problem recording audio for frame %d.  Please try recording again.\n",
	      currentFrame);
      PrintErrorMessage(errorMessage);
								    /* The record process was interrupted by an audio error. */
      if(FixMovie() == True)					    /* Can the movie be fixed? */
      {
	Stop((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);	    /* Yes, stop recording right now */
      }
      else
      {
	Stop((Widget)NULL, (XtPointer)-1, (XtPointer)NULL);	    /* Yes, stop recording right now and remove the bad file. */
      }	
      return(True);						    /* Done recording */
    } /* end if audioBytesRead... */
  } /* end if silentMovie... */
  gettimeofday(&after, (struct timezone*)NULL);
  audioReadTime[currentFrame] = ElapsedTime(&before, &after);	    /* Update stats on how long this frame's audio read took */
  if (((float)recordClock) > (scheduledTime +			    /* Is recording late by more than two frames? (that is,... */
			      ((audioDelayInFrames + 1) * timeForOneFrame))) /* ...two frames plus the requested audio delay) */
  {								    /* Yes, record audio only */
    framesSkippedDuringRecording++;				    /* Note that a video frame needed to be skipped */
    recordSkippedFrameIndex[currentFrame] =			    /* Note how late and for which frame this skip occurred */
      ((float)recordClock) - scheduledTime -
	(timeForOneFrame * (audioDelayInFrames - 1));
    compressTime[currentFrame] = 0L;				    /* This frame was skipped so took no time to compress */
    result = RecordOneFrame(currentFrame++, recordAudioFlag,	    /* Record a single frame of audio (no video) to disk */
			    False);
    if (result == True)						    /* Did recording a single frame fail for some reason? */
    {
      Stop((Widget)NULL, (XtPointer)-1, (XtPointer)NULL);	    /* Yes, stop recording right now & remove the bad movie file */
    }
  }
  else								    /* No, there's still time to record both audio and video */
  {
    gettimeofday(&before, (struct timezone*)NULL);
    XtVaSetValues(playbackWidget, XtNgetNewImage, True, NULL);	    /* Tell the JPEG widget to compress another image right now */
    if (usingSharedMemory == False)				    /* Is the JPEG widget using shared memory?  If not, then... */
    {								    /* ...the app must point to a new JPEG image each time... */
      XtVaGetValues(playbackWidget,				    /* ...the app grabs another frame. */
		    XtNjpegData, &jpegImage,
		    NULL);
      if (zoomFactor != 1)					    /* Fix the actual size of the JPEG data if the zoom... */
      {								    /* ...factor isn't 1:1. */
	jpegImage->width = (jpegImage->width / zoomFactor);
	jpegImage->height = (jpegImage->height / zoomFactor);
      }
    }
    if ((True == jpegAvailable) &&				    /* Is JPEG hardware available for recording with?  AND... */
	(jpegImage->size > 0))					    /* ...was a valid JPEG frame retrieved? */
      hasVideo = True;						    /* Yes, it's okay to save the JPEG data to disk */
    else
      hasVideo = False;						    /* No, don't bother recording JPEG data to disk */
    gettimeofday(&after, (struct timezone*)NULL);
    compressTime[currentFrame] = ElapsedTime(&before, &after);	    /* Update stats on how long this frame's video compress took */
    result = RecordOneFrame(currentFrame++,			    /* Record a single frame to disk, having either audio,... */
			    recordAudioFlag, hasVideo);		    /* ...video, both, or neither, depending on conditions. */
    if (result == True)						    /* Did recording a single frame fail for some reason? */
    {
      currentFrame--;						    /* Back up to the bad frame. */
      if(FixMovie() == True)
      {
	Stop((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);	    /* Yes, stop recording right now */
      }
      else
      {
	Stop((Widget)NULL, (XtPointer)-1, (XtPointer)NULL);	    /* Yes, stop recording right now and remove the bad file. */
      }	
    }
  }
  return(result);
} /* end function RecordWorkProc */


void Record(Widget	w,
	    XtPointer	client_data,
	    XtPointer	xt_call_data)
{
  int				bytesWritten;
  Dimension			movieWidth;
  Dimension			movieHeight;
  struct timeval		tempBefore;
  struct timeval		tempAfter;
  Boolean			videoIsLive;
  
  if ((movieToolStatus == Playing) ||				    /* Is a movie already playing or recording?  If so,... */
      (movieToolStatus == Recording))				    /* ...don't do anything. */
    return;
  if (movieToolStatus == Stopped)				    /* Is it okay to begin recording a new movie? */
  {								    /* Yes it is, do the right thing */
    outputFile = open(outputFileName,				    /* Try to open a file for writing a movie to disk; create... */
		      (O_CREAT | O_WRONLY | O_TRUNC),		    /* ...it if it doesn't already exist; reset its length if... */
		      0644);					    /* ...it does exist. */
    if (outputFile == -1)					    /* Did the attempt fail? */
    {								    /* Yes, report the error and don't bother recording */
      sprintf(errorMessage,"Could not open the file %s for writing.\n",
	      outputFileName);
      PrintErrorMessage(errorMessage);
      return;
    }
    PrintAudioInfo(&audioInfo);         			    /* Get statistics about the audio device */
    switch (audioInfo.record.encoding)				    /* Make sure that the information written to the movie... */
    {								    /* ...file contains the old, device-specific audio... */
     case AudioULawEncoding:					    /* ...encoding information, in order to maintain... */
      recordHeader.audio.encoding = AUDIO_ENCODING_ULAW;	    /* ...compatibility with older JPEG Movie recorders and... */
      break;							    /* ...players. */
     case AudioALawEncoding:
      recordHeader.audio.encoding = AUDIO_ENCODING_ALAW;
      break;
     case AudioLinearEncoding:
      recordHeader.audio.encoding = AUDIO_ENCODING_LINEAR;
      break;
    }
    recordHeader.audio.sample_rate = audioInfo.record.sampleRate;
    recordHeader.audio.samples_per_unit = 1;
    recordHeader.audio.bytes_per_unit =
      audioInfo.record.bitsPerSample / 8;
    recordHeader.audio.channels = audioInfo.record.channels;
    audioBandwidth = audioInfo.play.sampleRate *		    /* Determine how much bandwidth is necessary for audio */
      audioInfo.play.channels *
	audioInfo.play.bitsPerSample / 8;
    recordHeader.audio.data_size = audioBandwidth;
    audioBytesPerFrame = audioBandwidth / recordFrameRate;	    /* Determine how much bandwidth for a single frame of audio */
    if (audioData != (char*)NULL)
      free(audioData);
    audioData = (char*)malloc(audioBytesPerFrame);		    /* Allocate enough memory to hold one frame of audio */
    if (audioData == (char*)NULL)				    /* Did the attempt fail? */
    {								    /* Yes, report the error and do nothing about it for now */
      sprintf(errorMessage,
	      "Could not allocate the memory necessary to record audio. Please try again, or quit MovieTool and restart it.\n");
      PrintErrorMessage(errorMessage);
      return;
    }
    SetAudioPlaybackConfiguration(&audioRecordHeader);              /* Set the audio device for the current record configuration */
    movieToolStatus = Recording;				    /* Tell the application it's recording a movie */
    currentFrame = 0;						    /* Start at the 1st frame of the new movie */
    strcpy(recordHeader.magic, "j_movie");
    recordHeader.version = JPEG_VERSION_2;
    recordHeader.fps = recordFrameRate;				    /* Record at the best chosen frame rate */
    recordHeader.frames = 0;					    /* When starting to record, there are no frames in the movie */
    recordHeader.bandwidth = 0;					    /* Before recording, the data rate of the movie is unknown */
    XtVaGetValues(input1Widget, XtNlive, &videoIsLive, NULL);	    /* Determine if video is live right now */
    if (videoIsLive == False)					    /* No, please turn on video before recording */
    {
      XtVaSetValues(input1Widget,
		    XtNlive, True,
		    NULL);
    }
    XtVaGetValues(input1Widget,					    /* Determine the dimensions of the window from which to... */
		  XtNwidth, &movieWidth,			    /* ...record. */
		  XtNheight, &movieHeight,
		  NULL);
    recordSourceRect.x = 0;
    recordSourceRect.y = 0;
    recordSourceRect.width = movieWidth;
    recordSourceRect.height = movieHeight;
    XtVaGetValues(imageQualitySlider,				    /* Set up width, height, and Q factor for each frame in... */
		  XmNvalue, &recordHeader.qfactor,		    /* ...this movie. */
		  NULL);
    XtVaSetValues(playbackWidget,
		  XtNsourceWin, XtWindow(input1Widget),		    /* Point to the video input widget's window */
		  XtNqFactor, recordHeader.qfactor,		    /* Get the new Q factor from the imageQuality slider */
		  XtNdestWin, 0,				    /* Don't draw compressed images; this saves time */
		  XtNwidth, movieWidth,				    /* Resize the widget to fit the size of the video input... */
		  XtNheight, movieHeight,			    /* ...widget's window. */
		  XtNcompressSourceRect, &recordSourceRect,	    /* Record a rectangle exactly the size of the video input... */
		  XtNcompressDestRect, &recordSourceRect,	    /* ...widget's window. */
		  XtNgetNewImage, True,				    /* Also force a single compression to incur any delays... */
		  NULL);					    /* ...before actual movie recording takes place. */
    if (usingSharedMemory == False)				    /* This code is written in the hope that the GetValues... */
    {								    /* ...call forces X events to be flushed, thereby... */
      XtVaGetValues(playbackWidget,				    /* ...avoiding the horrendously slow compression for the... */
		    XtNjpegData, &jpegImage,			    /* ...1st frame of video. */
		    NULL);
    }
    recordHeader.width = (int)movieWidth;
    recordHeader.height = (int)movieHeight;
    recordHeader.mapsize = 256 * 256 * 256;			    /* The movie doesn't use pseudo-color; all 24 bits are valid */
    if (silentMovie == True)					    /* Should audio be recorded along with the video? */
      recordHeader.tracks = 0;					    /* No, note this in the movie's header */
    else
      recordHeader.tracks = 1;					    /* Currently hard-coding a single track of audio */
    recordHeader.audioslice = audioBytesPerFrame;		    /* Set up the size in bytes of one frame of audio */
    bytesWritten = write(outputFile, &recordHeader, sizeof(jpheader)); /* Try to write the movie header to disk */
    if (bytesWritten < 0)					    /* Did the attempt fail? */
    {								    /* Yes, report the error and return immediately */
      sprintf(errorMessage,
	      "Could not write anything to the movie file.  Recording stopped.\n");
      PrintErrorMessage(errorMessage);
      close(outputFile);
      return;
    }
    recordFrameIndex[currentFrame] = sizeof(jpheader);		    /* The 1st frame of the movie begins after the header */
    if (maestroFD != (XtInputId)NULL)				    /* Is the application listening for MAEstro messages? */
    {								    /* Yes, continue */
      XtRemoveInput(maestroFD);					    /* Stop listening for MAEstro messages during recording */
      maestroFD = (XtInputId)NULL;				    /* Initialize the MAEstro file descriptor to be nothing */
    }
    totalBytesInRecording = sizeof(jpheader);			    /* Update size of this movie */
    framesSkippedDuringRecording = 0;				    /* Reset counter of number of video frames that were omitted */
    memset(recordSkippedFrameIndex, 0, MaxMovieDurationInFrames *   /* Initialize the skipped frames index before recording */
	   sizeof(float));
    memset(audioReadTime, 0, MaxMovieDurationInFrames *		    /* Initialize the audio read time array before recording */
	   sizeof(long));
    memset(compressTime, 0, MaxMovieDurationInFrames *		    /* Initialize the video compress time array before recording */
	   sizeof(long));
    memset(diskWriteTime, 0, MaxMovieDurationInFrames *		    /* Initialize the disk write time array before recording */
	   sizeof(long));
    DeSensitizePlaybackControls();				    /* Disable controls for shuttling through the movie */
    DisableAudioDeviceControls();
    XtVaSetValues(inputMenuCascade, XmNsensitive, False, NULL);	    /* Don't allow input selection during recording */
    XtPopup(input1Shell, XtGrabNone);				    /* Assure that the video input widget is visible */
    recordWorkProcID = XtAppAddWorkProc(app_context,		    /* Start a work-proc to record a movie; it's done this... */
					RecordWorkProc,		    /* ...way so as not to block the user interface. */
					(XtPointer)NULL);
    XtVaSetValues(playbackWidget, XtNgetNewImage, True, NULL);	    /* Tell the JPEG widget to compress another image right now */
    gettimeofday(&recordStartTime, (struct timezone*)NULL);	    /* Note the time that movie recording was started */
    gettimeofday(&tempBefore, (struct timezone*)NULL);		    /* Note the time that movie recording was started */
    AudioRecordResume();					    /* Try to reset the audio device so record can resume */
  } /* end (if movieToolStatus is Stopped) */
/*
  printf("Should really test if there's any disk space left to record.\n");
*/
  SyncAudioRecordDevice();       				    /* Fill up the audio device's buffer so recording goes okay */
  XtVaSetValues(searchSlider, XmNvalue, 0, NULL);		    /* Reset the search slider to point to start of new movie */
  gettimeofday(&tempAfter, (struct timezone*)NULL);		    /* Note the time that movie recording was started */
  AudioRecordResume();
  sprintf(diagString, "Time between opening audio device and exiting Record: %ld.\n",
	  ElapsedTime(&tempBefore, &tempAfter));
  PrintDiagnostics(diagString);
  return;
} /* end function Record */



void Stop(Widget	unusedWidget,
	  XtPointer	clientData,
	  XtPointer	unusedXtCallData)
{
  int				bytesWritten;
  struct timeval		playbackStopTime;
  struct timeval		recordStopTime;
  
  totalTimePaused = 0;						    /* No matter what, reset the total time paused */
  XtVaSetValues(speedSlider, XmNsensitive, True, NULL);		    /* Allow playback speed to change since movie is stopped */
  XtVaSetValues(inputMenuCascade, XmNsensitive, True, NULL);	    /* Allow input selection again */
  if (movieToolStatus == Playing)
  {
    gettimeofday(&playbackStopTime, (struct timezone*)NULL);	    /* Record the time this movie playback was stopped */
    XtRemoveWorkProc(playbackWorkProcID);			    /* Tell the toolkit to stop playback now */
    movieToolStatus = Stopped;					    /* Tell the app about its new state */
    EnableTransportButtons();
    EnableAudioDeviceControls();
    PrintPlaybackStatistics(&playbackStopTime, &movieHeader);	    /* Print info about playback performance */
    AudioStopPlayDevice();
    if (userTurnedOffMonitor == False)				    /* Did the application itself disable the monitor during... */
      MonitorOn((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);    /* ...playback?  If so, turn it on again now that... */
  }
  else if (movieToolStatus == Recording)
  {
    gettimeofday(&recordStopTime, (struct timezone*)NULL);	    /* Record the time this movie recording was stopped */
    movieToolStatus = Stopped;					    /* Tell the app about its new state */
    AudioRecordPause();                 			    /* Stop recording audio; don't fill up audio buffers anymore */ 
    XtRemoveWorkProc(recordWorkProcID);				    /* Tell the toolkit to stop recording now */
    AudioStopRecordDevice();
    if (receiver != (Receiver*)NULL)				    /* Is MAEstro messaging active? */
      maestroFD = XtAppAddInput(app_context,			    /* Yes, start listening for MAEstro messages again */
				receiver->receiveSocket,
				(XtPointer) XtInputReadMask,
				CheckMAEstroMessages,
				(XtPointer)NULL);
    if (clientData == (XtPointer)-1)				    /* Did the application call this function indicating... */
    {								    /* ...there was an egregious error in recording? */
      RemoveMovie((void*)outputFile, outputFileName);		    /* Yes, remove the movie completely and return MovieTool... */
      EnableTransportButtons();					    /* ...to its normal state. */
      EnableAudioDeviceControls();
      return;
    }
    if (currentFrame == 0)					    /* If this is zero, there must've been a problem... */
    {								    /* ...recording the movie; there *was* no movie. */
      sprintf(errorMessage, "This movie is empty.  Removing the file.\n");
      PrintErrorMessage(errorMessage);
      RemoveMovie((void*)outputFile, outputFileName);
      EnableTransportButtons();
      EnableAudioDeviceControls();
      return;
    }
    recordHeader.frames = currentFrame;				    /* Store the number of frames in this movie */
    recordHeader.firstValidFrame = 0;				    /* Mark the whole movie as valid for playback */
    recordHeader.lastValidFrame = currentFrame - 1;
    recordHeader.indexbuf = totalBytesInRecording;		    /* Mark the location of the frame index locations */
    bytesWritten = write(outputFile,				    /* Write the frame index location info to the movie file */
			 recordFrameIndex,
			 currentFrame * sizeof(int));
    if (bytesWritten < 0)					    /* Did the attempt fail? */
    {								    /* Yes, report the error and return immediately */
      sprintf(errorMessage,
	      "Could not write frame start location information to the movie file.\n");
      PrintErrorMessage(errorMessage);
      close(outputFile);
      EnableTransportButtons();
      EnableAudioDeviceControls();
      return;
    }
    recordHeader.bandwidth =					    /* Store the average data rate for this movie */
      ((totalBytesInRecording / currentFrame) *
       recordHeader.fps) / 1024;
    recordHeader.width = jpegImage->width;			    /* Use the size of the JPEG image dimensions instead of... */
    recordHeader.height = jpegImage->height;			    /* ...the window size, in case zoom is in effect. */
    lseek(outputFile, 0L, SEEK_SET);				    /* Go back to the start of the movie to write updated... */
    bytesWritten = write(outputFile, &recordHeader, sizeof(jpheader)); /* ...movie header information. */
    if (bytesWritten < 0)					    /* Did the attempt fail? */
    {								    /* Yes, report the error and return immediately */
      sprintf(errorMessage,
	      "Could not write an updated movie header to the movie file.\n");
      PrintErrorMessage(errorMessage);
      close(outputFile);
      EnableTransportButtons();
      EnableAudioDeviceControls();
      return;
    }
    close(outputFile);						    /* Done with the output file; close it now */
    PrintRecordStatistics(&recordStopTime, &recordHeader);	    /* Print info about record performance */
    EnableAudioDeviceControls();
    OpenMovie((Widget)NULL,					    /* Tell the app to make this movie available for playback... */
	      (XtPointer)outputFileName,			    /* ...so it can be reviewed. */
	      (XtPointer)NULL);
    currentFrame = firstFrame;					    /* Go to the beginning of the movie */

  } /* end else if Recording */
  return;
} /* end function Stop */



void AdvanceFrame(Widget	w,
		  XtPointer	client_data,
		  XtPointer	xt_call_data)
{
  if ((movieToolStatus != Paused) &&				    /* Single frame advance should not work unless a movie is... */
      (movieToolStatus != Stopped))				    /* ...either paused or stopped completely. */
    return;
  if (currentFrame < playbackEndFrame)				    /* Is it okay to go forward one frame? */
    PlayOneFrame(++currentFrame, False, True);			    /* Yes, play next frame in the movie, but don't play audio */
  return;
} /* end function AdvanceFrame */



void ReverseFrame(Widget	w,
		  XtPointer	client_data,
		  XtPointer	xt_call_data)
{
  if ((movieToolStatus != Paused) &&				    /* Single frame reverse should not work unless a movie is... */
      (movieToolStatus != Stopped))				    /* ...either paused or stopped completely. */
    return;
  if (currentFrame > playbackStartFrame)			    /* Is it okay to go back one frame? */
    PlayOneFrame(--currentFrame, False, True);			    /* Yes, play previous frame in the movie, don't play audio */
  return;
} /* end function ReverseFrame */



void EndOfMovie(Widget		w,
		XtPointer	client_data,
		XtPointer	xt_call_data )
{
  if ((movieToolStatus != Paused) &&				    /* End-of-movie should not work unless a movie is... */
      (movieToolStatus != Stopped))				    /* ...either paused or stopped completely. */
    return;
  currentFrame = lastFrame;
  playbackEndFrame = lastFrame;
  PlayOneFrame(playbackEndFrame, False, True);			    /* Play the last frame of the movie, don't play audio */
  return;
} /* end function EndOfMovie */



void StartOfMovie(Widget	w,
		  XtPointer	client_data,
		  XtPointer	xt_call_data )
{
  if (movieToolStatus == Playing)				    /* Did the viewer want to jump to the start of the movie... */
  {								    /* ...during playback?  If so, reset movie for playback: */
    framesSkippedDuringPlayback = 0;				    /* Reset counter of number of video frames that were omitted */
    memset(skippedFrameIndex, 0, MaxMovieDurationInFrames *	    /* Initialize the skipped frames index before playback */
	   sizeof(float));
    gettimeofday(&playbackStartTime, (struct timezone*)NULL);	    /* Record the time this movie playback was started */
    currentFrame = firstFrame;					    /* Go to the 1st valid frame of this movie */
    playbackStartFrame = firstFrame;				    /* Note that playback began at the 1st valid frame */
    return;
  }
  if ((movieToolStatus != Paused) &&				    /* Don't play a single frame unless a movie is... */
      (movieToolStatus != Stopped))				    /* ...either paused or stopped completely. */
    return;
  currentFrame = firstFrame;					    /* Go to the 1st valid frame of this movie */
  playbackStartFrame = firstFrame;				    /* Note that playback began at the 1st valid frame */
  PlayOneFrame(playbackStartFrame, False, True);		    /* Play the 1st frame of the movie, don't play audio */
  return;
} /* end function StartOfMovie */



void PrintMovieDiagnostics(char* 	fileName,
			   jpheader*	movieHeaderInfo)
{
  if (printDiagnostics == False)				    /* Don't bother printing performance stats if not requested */
    return;
  printf("Movie Information for the movie file %s:\n", fileName);
  printf("Frames per second: %d ", movieHeaderInfo->fps);
  printf("(%d milliseconds per frame.)\n", 1000 / movieHeaderInfo->fps);
  printf("Number of useful frames in the movie: %d.\n",
	 (movieHeaderInfo->lastValidFrame - movieHeaderInfo->firstValidFrame) + 1);
  printf("Total number of frames in the movie: %d.\n",
	 movieHeaderInfo->frames);
  return;
} /* end function PrintMovieDiagnostics */



void StartMovieHere(Widget	w,
		    XtPointer	client_data,
		    XtPointer	xt_call_data)
{
  if (firstFrame == currentFrame)				    /* Don't bother doing work if the movie already starts here */
    return;
  firstFrame = currentFrame;					    /* Set the new starting frame for this movie */
  if (playbackStartFrame < firstFrame)				    /* Is the 1st frame of playback for this movie now invalid? */
    playbackStartFrame = firstFrame;				    /* Yes, update it to reflect the new movie size. */
  SetMovieStatus(MovieModified);				    /* Update interface to reflect change in movie file status */
  UpdateTimecodeDisplay(movieHeader.fps);
  UpdateMovieLengthIndicators();				    /* Update interface parts for movie length, start, & end */
  return;
} /* end function StartMovieHere */



void EndMovieHere(Widget	w,
		  XtPointer	client_data,
		  XtPointer	xt_call_data)
{
  if (lastFrame == currentFrame)				    /* Don't bother doing work if the movie already ends here */
    return;
  lastFrame = currentFrame;					    /* Set the new ending frame for this movie */
  if (playbackEndFrame > lastFrame)				    /* Is the last frame for this movie now invalid? */
    playbackEndFrame = lastFrame;				    /* Yes, update it to reflect the new movie size */
  SetMovieStatus(MovieModified);				    /* Update interface to reflect change in movie file status */
  UpdateMovieLengthIndicators();				    /* Update interface parts for movie length, start, & end */
  UpdateTimecodeDisplay(movieHeader.fps);
  return;
} /* end function EndMovieHere */



void UseOriginalStartAndEndFrames(Widget	w,
				  XtPointer	client_data,
				  XtPointer	xt_call_data)
{
  firstFrame = (movieHeader.firstValidFrame < 0 ? 0 :		    /* Use the suggested start and end frames for this movie */
		movieHeader.firstValidFrame);
  lastFrame = (movieHeader.lastValidFrame > (movieHeader.frames -1) ?
	       movieHeader.frames -1 :
	       movieHeader.lastValidFrame);
  playbackStartFrame = firstFrame;
  playbackEndFrame = lastFrame;
  SetMovieStatus(MovieOK);
  UpdateMovieLengthIndicators();				    /* Update interface parts for movie length, start, & end */
  UpdateTimecodeDisplay(movieHeader.fps);
  return;
} /* end function UseOriginalStartAndEndFrames */



void PlusButtonCallback(Widget		w,
			XtPointer	client_data,
			XtPointer	xt_call_data)
{
  if ((movieToolStatus != Paused) &&				    /* Frame advance should not work unless a movie is... */
      (movieToolStatus != Stopped))				    /* ...either paused or stopped completely. */
    return;
  if ((w == hourPlusButton) &&					    /* Was the hours button pressed, and is there at least... */
      ((playbackEndFrame - currentFrame) >=			    /* ...one hour remaining in this movie? */
       (movieHeader.fps * 3600)))
  {
    currentFrame += movieHeader.fps * 3600;			    /* Yes, advance by one hour */
    PlayOneFrame(currentFrame, False, True);			    /* Play the frame just found */
  }
  if ((w == minutePlusButton) &&				    /* Was the minutes button pressed, and is there at least... */
      ((playbackEndFrame - currentFrame) >=			    /* ...one minute remaining in this movie? */
       (movieHeader.fps * 60)))
  {
    currentFrame += movieHeader.fps * 60;			    /* Yes, advance by one minute */
    PlayOneFrame(currentFrame, False, True);			    /* Play the frame just found */
  }
  if ((w == secondPlusButton) &&				    /* Was the seconds button pressed, and is there at least... */
      ((playbackEndFrame - currentFrame) >= movieHeader.fps))	    /* ...one second remaining in this movie? */
  {
    currentFrame += movieHeader.fps;				    /* Yes, advance by one second */
    PlayOneFrame(currentFrame, False, True);			    /* Play the frame just found */
  }
  if (w == framePlusButton)					    /* Was the frames button pressed, and is there at least... */
  {
    if (currentFrame < playbackEndFrame)			    /* ...one frame remaining in this movie? */
      PlayOneFrame(++currentFrame, False, True);		    /* Yes, advance by one frame and play it */
  }
  return;
} /* end function PlusButtonCallback */



void MinusButtonCallback(Widget		w,
			 XtPointer	client_data,
			 XtPointer	xt_call_data)
{
  if ((movieToolStatus != Paused) &&				    /* Frame reverse should not work unless a movie is... */
      (movieToolStatus != Stopped))				    /* ...either paused or stopped completely. */
    return;
  if ((w == hourMinusButton) &&
      ((currentFrame - firstFrame) >= (movieHeader.fps * 3600)))
  {
    currentFrame -= movieHeader.fps * 3600;			    /* Yes, rewind by one hour */
    PlayOneFrame(currentFrame, False, True);			    /* Play the frame just found */
  }
  if ((w == minuteMinusButton) &&
      ((currentFrame - firstFrame) >= (movieHeader.fps * 60)))
  {
    currentFrame -= movieHeader.fps * 60;			    /* Yes, rewind by one minute */
    PlayOneFrame(currentFrame, False, True);			    /* Play the frame just found */
  }
  if ((w == secondMinusButton) &&
      ((currentFrame - firstFrame) >= movieHeader.fps))
  {
    currentFrame -= movieHeader.fps;				    /* Yes, rewind by one second */
    PlayOneFrame(currentFrame, False, True);			    /* Play the frame just found */
  }
  if ((w == frameMinusButton) &&
      (currentFrame > firstFrame))
  {
    PlayOneFrame(--currentFrame, False, True);
  }
  return;
} /* end function MinusButtonCallback */



void ChangePlaybackVolume(Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  XmScaleCallbackStruct*	callData =
    (XmScaleCallbackStruct *) xt_call_data;
  
  SetAudioPlayVolume(callData->value);
  return;
} /* end function ChangePlaybackVolume */



void ChangeRecordVolume(Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  XmScaleCallbackStruct*	callData =
    (XmScaleCallbackStruct *) xt_call_data;
  
  SetAudioRecordVolume(callData->value);
  return;
} /* end function ChangeRecordVolume */



void ChangeMonitorVolume(Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  Boolean			isMonitorOn;
  XmScaleCallbackStruct*	callData =
    (XmScaleCallbackStruct *) xt_call_data;
  
  isMonitorOn = XmToggleButtonGetState(monitorOnToggle);	    /* Ask the "Monitor On" toggle if it is on */
  if (isMonitorOn == False)					    /* Nope, it's not; forcibly turn it on */
  {
    MonitorOn((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);
  }
  SetAudioMonitorVolume(callData->value);
  lastMonitorVolume = callData->value;				    /* Remember the new monitor volume in case monitor is... */
  return;							    /* ...turned off then on again later. */
} /* end function ChangeMonitorVolume */



void UseSpeakerForPlayback(Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  SetPlaybackDevice(AudioSpeaker);
  return;
} /* end function UseSpeakerForPlayback */



void UseHeadphonesForPlayback(Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  SetPlaybackDevice(AudioHeadphone);
  return;
} /* end function UseHeadphonesForPlayback */



void UseLineOutForPlayback(Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  SetPlaybackDevice(AudioLineOut);
} /* end function UseLineOutForPlayback */



void UseLineInForRecording(Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  SetRecordDevice(AudioLineIn);
} /* end function UseLineInForRecording */



void UseMicForRecording(Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  SetRecordDevice(AudioMicrophone);
} /* end function UseMicForRecording */



void ChangeAudioModule(Widget w, XtPointer client_data, XtPointer xt_call_data)
{
  int playVolume;
  int recordVolume;
  int monitorVolume;
  int index;
  
  if(!XmToggleButtonGetState(w))					/* If the button has not been pressed, */
  {
    return;								/* return. */
  }
  index = 0;
  while(audioDeviceWidget[index] != w)
  {
    index++;
  }
  XmScaleGetValue(playbackScale, &playVolume);			    /* Get the current state of the audio panel */
  XmScaleGetValue(recordScale, &recordVolume);
  XmScaleGetValue(monitorScale, &monitorVolume);
  AudioClose();								/* Close the audio device */
  sleep(1);
  initializeAudioObject(audioMap[index].audioObject);
  audioStatus = InitializeAudio();  
  if(audioStatus == -1)  					    /* If error occurred during initializeAudio, use noAudioObject */
  {
    initializeAudioObject(&noAudioObject);
    silentMovie = True;
  }
  else
  {
    silentMovie = False;
  }
  setAudioDeviceButton();
  SetAudioPlaybackConfiguration(&audioPlayHeader);
  
  XmScaleSetValue(playbackScale, playVolume);
  SetAudioPlayVolume(playVolume);
  
  XmScaleSetValue(recordScale, recordVolume);
  SetAudioRecordVolume(recordVolume);
  
  XmScaleSetValue(monitorScale, monitorVolume);
  if(XmToggleButtonGetState(monitorOnToggle))
  {
    SetAudioMonitorVolume(monitorVolume);    
  }
  if(XmToggleButtonGetState(playbackSpeakerToggle))
  {
    SetPlaybackDevice(AudioSpeaker);
  }
  if(XmToggleButtonGetState(playbackHeadphonesToggle))
  {
    SetPlaybackDevice(AudioHeadphone);
  }
  if(XmToggleButtonGetState(playbackLineOutToggle))
  {
    SetPlaybackDevice(AudioLineOut);
  }
  if(XmToggleButtonGetState(recordLineInToggle))
  {
    SetRecordDevice(AudioLineIn);
  }
  if(XmToggleButtonGetState(recordMicInToggle))
  {
    SetRecordDevice(AudioMicrophone);
  }
  PrintAudioInfo(&audioInfo);
  return;
} /* end function ChangeAudioModule */



void SetMovieFileFormat(Widget		w,
			XtPointer	clientData,
			XtPointer	xt_call_data )
{
  if (fileFormat != (char*)NULL)				    /* Free up any space previously pointed to by this variable */
    free(fileFormat);
  fileFormat = strdup(clientData);				    /* Set a variable determining file format for movies */
  return;
} /* end function SetMovieFileFormat */



void PrintDiagnostics(char* diagString)
{
  if (printDiagnostics == True)
    printf(diagString);
  return;
} /* end function PrintDiagnostics */



void HoursTextFieldCallback(Widget	w,
			    XtPointer	client_data,
			    XtPointer	xt_call_data)
{
  char*	textString;
  
  textString = XmTextFieldGetString(w);
  XtFree(textString);
  return;
} /* end function HoursTextFieldCallback */



void MinutesTextFieldCallback(Widget	w,
			    XtPointer	client_data,
			    XtPointer	xt_call_data)
{
  char*	textString;
  
  textString = XmTextFieldGetString(w);
  XtFree(textString);
  return;
} /* end function MinutesTextFieldCallback */



void SecondsTextFieldCallback(Widget	w,
			    XtPointer	client_data,
			    XtPointer	xt_call_data)
{
  char*	textString;
  
  textString = XmTextFieldGetString(w);
  XtFree(textString);
  return;
} /* end function SecondsTextFieldCallback */



void FramesTextFieldCallback(Widget	w,
			    XtPointer	client_data,
			    XtPointer	xt_call_data)
{
  char*	textString;
  
  textString = XmTextFieldGetString(w);
  XtFree(textString);
  return;
} /* end function FramesTextFieldCallback */



void SetCompositeOutput(Widget		w,
			XtPointer	client_data,
			XtPointer	xt_call_data)
{
  outputSignalFormat = PLX_COMP;
  return;
} /* end function SetCompositeOutput */




void SetYCOutput(Widget		w,
		 XtPointer	client_data,
		 XtPointer	xt_call_data)
{
  outputSignalFormat = PLX_YC;
  return;
} /* end function SetYCOutput */



void SetInputFormat(Widget	w,
		    XtPointer	clientData,
		    XtPointer	unusedCallData)
{
  inputSignalFormat = (int)clientData;
  return;
} /* end function SetCompositeInput */



void SetInputInstance(Widget	w,
		      XtPointer	clientData,
		      XtPointer	unusedCallData)
{
  inputInstance = (int)clientData;
  return;
} /* end function SetInputInstance */


void WMDeleteWindowCallback(Widget	widget,
			    XtPointer	clientData,
			    XtPointer	callData)
{
  if (widget == mainWindow)					    /* Was the window manager used to quit the app instead of... */
  {								    /* ...the app's File menu "Quit" item? */
    Quit(widget, (XtPointer)NULL, (XtPointer)NULL);		    /* Yes, call the normal Quit callback anyway */
  }
  if (widget == input1Shell)
  {
    SetMTPanelState(&mtPanelState, *(Widget*)widget, False);	    /* Inform the application that this shell is now invisible */
  }
  if (widget == movieShell)
  {
    SetMTPanelState(&mtPanelState, *(Widget*)widget, False);	    /* Inform the application that this shell is now invisible */
  }
  return;
} /* end function WMDeleteWindowCallback */


Boolean AutoLoadWorkProc(XtPointer unusedClientData)
{
  if(audioStatus == -1)
  {
    if(autoPlay == False)
    {
      sprintf(errorMessage,
	      "Could not open the audio device.  No audio will be available.\n");
      PrintErrorMessage(errorMessage);
      return(True);
    }
  }
  if (startupMovie == True)					    /* Was a movie specified on the command line? */
  {								    /* Yes, open the movie right away */
    OpenMovie((Widget)NULL, startupMovieName, (XtPointer)NULL);	    /* Yes, open the movie for playback */
    if ((movieFile != (FILE*)NULL) && (autoPlay == True))	    /* Did the file open successfully?  And, should the movie... */
    {								    /* ...be played immediately on playback? */
      Play((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);	    /* Yes, play the movie immediately. */
    }
  }
  return(True);
} /* end function AutoLoadWorkProc */



void BrightnessAdjust(Widget	w,
		      XtPointer client_data,
		      XtPointer	xt_call_data)
{
  int brightness;
  
  XmScaleGetValue(brightnessSlider, &brightness);
  XtVaSetValues(input1Widget, 
		XtNbrightness, brightness,
		NULL);
} /* end function BrightnessAdjust */



void ContrastAdjust(Widget	w,
		    XtPointer	client_data,
		    XtPointer	xt_call_data)
{
  int contrast;
  
  XmScaleGetValue(contrastSlider, &contrast);
  XtVaSetValues(input1Widget, 
		XtNcontrast, contrast,
		NULL);
} /* end function ContrastAdjust */



void HueAdjust(Widget		w,
	       XtPointer	client_data,
	       XtPointer	xt_call_data)
{
  int hue;
  
  XmScaleGetValue(hueSlider, &hue);
  XtVaSetValues(input1Widget, 
		XtNhue, hue,
		NULL);
} /* end function HueAdjust */



void SaturationAdjust(Widget	w,
		      XtPointer client_data,
		      XtPointer	xt_call_data)
{
  int saturation;
  
  XmScaleGetValue(saturationSlider, &saturation);
  XtVaSetValues(input1Widget, 
		XtNsaturation, saturation,
		NULL);
} /* end function SaturationAdjust */



void InputSettingsDefaultPressed(Widget		w,
				 XtPointer	client_data,
				 XtPointer	xt_call_data)
{
  XmScaleSetValue(brightnessSlider, 128);
  XmScaleSetValue(contrastSlider, 128);
  XmScaleSetValue(hueSlider, 128);
  XmScaleSetValue(saturationSlider, 128);
  XtVaSetValues(input1Widget,
		XtNbrightness, 128,
		XtNcontrast, 128,
		XtNhue, 128,
		XtNsaturation, 128,
		NULL);
} /* end function InputSettingsDefaultPressed */



Boolean ZoomWorkProc(XtPointer	clientData)
{
  Boolean	autoSearch;
  Dimension	height;
  Dimension	width;
  int		signalFormat;
  int		signalStandard;
  int		channel;
  int		instance;
  Boolean	zoomedLarger;
  int		magnification;
  
  XPlxSetZoomFactor(XtDisplay(input1Widget),			    /* Tell the hardware to zoom to the requested zoom ratio */
		    XtWindow(input1Widget),
		    (int)clientData);
  if (zoomFactor < (int)clientData)				    /* Was the old zoom factor smaller than the requested? */
  {
    zoomedLarger = True;					    /* Yes, note that the app is going to zoom to larger size */
    magnification = (int)clientData / zoomFactor;		    /* Note difference in magnification (e.g., from 2X to 1X) */
  }
  else								    /* No, note that the app is going to shrink size */
  {
    zoomedLarger = False;
    magnification = zoomFactor / (int)clientData;		    /* Note difference in magnification (e.g., from 1X to 2X) */
  }
  zoomFactor = (int)clientData;					    /* Tell the application the new zoom ratio */
  XtVaGetValues(input1Widget,					    /* Observe current widget values for later */
		XtNchannel, &channel,
		XtNinstance, &instance,
		XtNsignalFormat, &signalFormat,
		XtNsignalStandard, &signalStandard,
		XtNwidth, &width,
		XtNheight, &height,
		XtNautoSearch, &autoSearch,
		NULL);
  width = (zoomedLarger == True) ?				    /* Adjust width and height of live video widget to... */
    width * magnification :					    /* ...account for the zoom factor. */
      width / magnification;
  height = (zoomedLarger == True) ?
    height * magnification :
      height / magnification;
  XtDestroyWidget(input1Widget);				    /* Destroy the old input 1 widget */
  input1Widget = (Widget)NULL;
  input1Widget = XtVaCreateManagedWidget("input1Widget",	    /* Create a video input widget for the first digitizer */
					 xVideoInWidgetClass,
					 input1Shell,
					 XtNchannel, channel,
					 XtNinstance, instance,
					 XtNsignalFormat, signalFormat,
					 XtNsignalStandard, signalStandard,
					 XtNautoSearch, False,
					 XtNwidth, width,
					 XtNheight, height,
					 NULL);
  XtVaSetValues(input1Widget, XtNautoSearch, autoSearch, NULL);	    /* Restore current setting for "autoSearch" */
  XtVaSetValues(input1Shell,					    /* Make the enclosing shell fit the new input widget size */
		XtNwidth, width,
		XtNheight, height,
		NULL);
  if ((movieHeader.width != 0) && (movieHeader.height != 0))	    /* Don't bother zooming the JPEG playback unless a movie... */
  {								    /* ...has been opened. */
    playDestRect.x = 0;
    playDestRect.y = 0;
    playDestRect.width = movieHeader.width * zoomFactor;
    playDestRect.height = movieHeader.height * zoomFactor;
    XtVaSetValues(movieShell,					    /* Make the Movie Shell fit the movie size perfectly */
		  XmNwidth, (movieHeader.width * zoomFactor),	    /* Take zoom factor into account when doing the dimension... */
		  XmNheight, (movieHeader.height * zoomFactor),	    /* ...calculations. */
		  XmNmaxWidth, (movieHeader.width * zoomFactor),    /* Don't allow the movie shell to grow larger than the... */
		  XmNmaxHeight, (movieHeader.height * zoomFactor),  /* ...movie itself. */
		  XmNminWidth, (movieHeader.width * zoomFactor),    /* Don't allow the movie shell to grow smaller than the... */
		  XmNminHeight, (movieHeader.height * zoomFactor),  /* ...movie itself, either. */
		  NULL);
    XtVaSetValues(playbackWidget,				    /* Resize the movie playback widget to fit the movie */
		  XtNwidth, (movieHeader.width * zoomFactor),	    /* Take zoom factor into account when doing the dimension... */
		  XtNheight, (movieHeader.height * zoomFactor),	    /* ...calculations. */
		  XtNdisplayDestRect, &playDestRect,
		  NULL);
  }
  return(True);
} /* end function ZoomWorkProc */
 


void ZoomCallback(Widget	w,
		  XtPointer	clientData,
		  XtPointer	xt_call_data)
{
  Boolean	buttonState;
  XtWorkProcId	zoomProcID;
  
  buttonState = XmToggleButtonGetState(w);
  if (False == buttonState)					    /* Don't bother doing anything if this toggle was turned off */
    return;
  zoomProcID = XtAppAddWorkProc(app_context,			    /* Start a work-proc to zoom the windows; Xt is... */
				ZoomWorkProc,			    /* ...poorly designed and this is the only way to assure... */
				(XtPointer)clientData);		    /* ...that widget resources will be freed when needed. */
  return;
} /* end function ZoomCallback */



void AboutMTOKCallback(Widget	w,
		  XtPointer	clientData,
		  XtPointer	xt_call_data)
{
  mtPanelState.aboutShell = False;
  return;
} /* end function AboutMTOKCallback */




void SetSyncSlack(Widget	w,
		  XtPointer	client_data,
		  XtPointer	xt_call_data)
{
  XmScaleCallbackStruct*	callData =
    (XmScaleCallbackStruct *) xt_call_data;
  
  syncSlack = callData->value;
  return;
} /* end function SetSyncSlack */




void SetOutputSignalType(Widget		w,
			 XtPointer	clientData,
			 XtPointer	xt_call_data)
{
  signalSpec	theSignal;
  XtWorkProcId	outputWorkProcID;
  
  theSignal.i = (unsigned int)clientData;
  outputSignalStandard = theSignal.s.standard;
  outputSignalFormat = theSignal.s.format;
  outputWorkProcID = XtAppAddWorkProc(app_context,		    /* Start a work-proc to show output; Xt is... */
				      OutputWorkProc,		    /* ...poorly designed and this is the only way to assure... */
				      (XtPointer)w);		    /* ...that widgets needing to be destroyed right now... */
  return;
} /* end function SetOutputSignalType */
