/* $Header: /plx/source/Xclients/generic/RCS/globals.c,v 1.37 1996/01/25 04:18:04 drapeau Exp $ */
/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Log: globals.c,v $
 * Revision 1.37  1996/01/25  04:18:04  drapeau
 * Added support for the PlxOutputCascade to automatically create the
 * output menu hierarchy, for enhanced flexibility.
 *
 * Revision 1.36  1995/11/23  03:54:37  drapeau
 * Minor interface changes to add the "syncSlack" slider, allowing for
 * synchronization tolerance to help eliminate audio/video lag and
 * excessive pops/gaps in audio when playback conditions are less than
 * ideal.
 *
 * Revision 1.35  1995/10/08  05:33:14  drapeau
 * Modified include directive to include the new Input/Output Motif
 * Cascade definitions.
 *
 * Revision 1.34  1995/10/02  21:54:13  drapeau
 * Added support for the application's ability to deal with hardware that
 * doesn't have JPEG support.
 *
 * Revision 1.33  1995/08/23  01:56:50  drapeau
 * Added variables for enhancing sync and diagnostics.
 *
 * Revision 1.32  1995/08/17  21:52:50  drapeau
 * Added support for an "About..." panel.
 *
 * Revision 1.31  1995/08/17  19:37:24  drapeau
 * Removed an extraneous global variable.
 *
 * Revision 1.30  1995/05/23  19:36:22  lovvik
 * Added an initial value for the variable
 * audioPlayHeader and audioRecordHeader.
 *
 * Revision 1.29  1995/05/16  02:56:12  drapeau
 * Added support for the new input cascading menu code.  Also added a
 * variable that allows the imageQualityWidget to better display images
 * when zoom is on.
 *
 * Revision 1.28  1995/05/12  00:19:33  lovvik
 * Initialized the variable "lastMonitorVolume" to
 * 100.  This fixed a problem in which the monitor
 * would not be turned on again after a Play operation
 * unless the monitor slider was moved.
 *
 * Revision 1.27  1995/05/10  20:25:31  lovvik
 * Added a variable called currentAudioObject that
 * points to the currently selected audio object.
 * Also added a variable array called audioDeviceWidget
 * that contains the toggle button widgets that represent
 * each audio module.
 *
 * Revision 1.26  1995/05/09  18:59:08  lovvik
 * Added displayName.  This is a character string that contains the
 * name of the host that MovieTool is being displayed on.  This
 * string is used when communicating with the HPAudioServer.
 *
 * Revision 1.25  1995/05/02  17:49:21  drapeau
 * Added support for hardware zoom when available.
 *
 * Revision 1.24  1995/04/27  00:16:21  lovvik
 * Added an audio object that is used to call audio functions from
 * the appropriate audio module.
 *
 * Revision 1.23  1995/02/13  18:27:24  lovvik
 * Modified this file to reflect the changes to the
 * new audio module.  The audio module was changed to
 * disinclude the audio file descriptor in the function
 * parameter lists.  We no longer need global varibles
 * to store the audio file descriptors.  The following
 * variables were removed from this file:  audioControlFD,
 * audioFD, audioPlayFD, audioRecordFD.  audioStatus was
 * added to replace the error detection functionality that
 * was reflected in a NULL audio file descriptor.
 *
 * Revision 1.22  1994/10/14  02:49:45  drapeau
 * Added a global variable to keep track of whether output is available.
 *
 * Revision 1.21  1994/10/10  23:03:21  drapeau
 * Added variables for representing the current input signal standard,
 * format, and instance.
 *
 * Revision 1.20  1994/09/30  16:57:24  drapeau
 * Added support for output.
 *
 * Revision 1.19  1994/09/14  03:22:50  drapeau
 * Made the imageQualitySourceRect a global variable.
 *
 * Revision 1.18  1994/08/23  01:33:55  drapeau
 * Added code for an alert dialog box.
 *
 * Revision 1.17  1994/08/11  03:16:38  drapeau
 * Added a couple of variables for enhanced diagnostics.
 *
 * Revision 1.16  1994/05/31  20:10:59  drapeau
 * Added a number of variables to support better diagnostics and
 * automatic playback and quit for movies.  Also, changed types of some
 * variables used in synchronization code (changed from integer data
 * types to floating point data types).
 *
 * Revision 1.15  1994/01/20  22:51:57  drapeau
 * Added two variables that had previously been removed.  They were added
 * to support a slight change in the PortableAudio interface which was
 * made to accommodate the Sun audio device driver implementation, which
 * is flawed, in my opinion.
 *
 * Revision 1.14  1994/01/20  18:05:14  drapeau
 * Modified code to support the PortableAudio interface, increasing
 * portability across platforms.
 *
 * Also added new variables to support new features and removed obsolete
 * variables.
 *
 * Revision 1.13  1994/01/10  22:40:06  drapeau
 * Added support for high-quality audio and the MAEstro Multimedia
 * Authoring Environment.
 *
 * Revision 1.12  1993/12/04  01:03:22  drapeau
 * Modified the code to make the Pause feature more robust.  Before this
 * version, pausing a movie during playback would throw off the
 * synchronization mechanism; pause now works correctly.
 *
 * Revision 1.11  1993/12/03  01:35:33  drapeau
 * Added copyright message, RCS info, and a few variables for better diagnostics
 * and performance.
 * */
static char globalsRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/globals.c,v 1.37 1996/01/25 04:18:04 drapeau Exp $";

#include <stdio.h>
#include <sys/param.h>
#include <sys/time.h>
#include <signal.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <X11/Xlib.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <XPlxExt.h>
#include <XVideoIn.h>
#include <JPEG.h>
#include "PortableAudio.h"
#include <movie.h>
#include <Sender.h>
#include <Receiver.h>
#include <IOCascade.h>

Widget		imageQualityWidget;
Widget		input1Widget;
Widget		playbackWidget;
jpheader	movieHeader;
int*		frameLocation = (int*)NULL;
int*		recordFrameIndex = (int*)NULL;
float*		skippedFrameIndex = (float*)NULL;
float*		recordSkippedFrameIndex = (float*)NULL;
char*		videoFrameIndex = (char*)NULL;
long*		audioReadTime = (long*)NULL;
long*		audioBetweenTime = (long*)NULL;
long*		audioWriteTime = (long*)NULL;
long*		compressTime = (long*)NULL;
long*		diskWriteTime = (long*)NULL;
long*		playInterval = (long*)NULL;
int		movieToolStatus;
XPlxCImage*	jpegImage;
XPlxCImage*	imageQualityImage;
char*		jpegData;
FILE*		movieFile;
int		currentFrame;
XtWorkProcId	playbackWorkProcID;
char*		audioData;
int		signalFrameRate;
int		outputFile;
int		audioBytesPerFrame;
int		recordAudioFlag;
int		audioBandwidth;
PortableAudio	audioInfo;
AudioObject     audioObject;
AudioObject      *currentAudioObject;
Widget audioDeviceWidget[5] = {(Widget)NULL,
			       (Widget)NULL,
			       (Widget)NULL,
			       (Widget)NULL,
			       (Widget)NULL};
jpheader	recordHeader;
XtWorkProcId	recordWorkProcID;
char		outputFileName[MAXPATHLEN];
char*		currentMovieFileName;
char		titleString[MAXPATHLEN + 64];
int		totalBytesInRecording;
XRectangle	recordSourceRect;
Boolean		resetPlaybackClock;
struct timeval	playbackStartTime;
struct timeval	recordStartTime;
int		playbackStartFrame;
int		playbackEndFrame;
int		framesSkippedDuringPlayback;
int		framesSkippedDuringRecording;
int		recordFrameRate;
long		totalTimePaused;
Boolean		printDiagnostics;
int		audioDelayInFrames;
int		firstFrame;
int		lastFrame;
char		diagString[MAXPATHLEN + 64];
Sender*		sender;
Receiver*	receiver;
XtInputId	maestroFD;
int		movieStatus;
Audio_hdr	audioPlayHeader = {0, 0, 0, 0, 0, 0};
Audio_hdr	audioRecordHeader = {0, 0, 0, 0, 0, 0};
Boolean		silentMovie;
int		sampleRate;
int		numAudioChannels;
int		playbackSpeed;
Boolean		usingSharedMemory;
int             audioStatus;
char*		fileFormat;
Boolean		monitorStatus;
int		lastMonitorVolume = 100;
Boolean		userTurnedOffMonitor;
Boolean		autoPlay;
Boolean		autoQuit;
Boolean		startupMovie;
Boolean		autoRepeat;
char*		startupMovieName;
char*		timecodeString;
char*		errorMessage;
XRectangle	imageQualitySourceRect;
XRectangle	imageQualityDisplaySourceRect;
XRectangle	imageQualityRect;
Widget		outputWidget;
Boolean		outputOn;
int		outputSignalStandard;
int		outputSignalWidth;
int		outputSignalHeight;
int		outputSignalFormat;
GC		outputGC;
int		inputSignalStandard;
int		inputSignalFormat;
int		inputInstance;
Boolean		outputAvailable;
int		zoomFactor;
XRectangle	playDestRect;
char*           displayName = (char *)NULL;
PlxInputCascade*	inputsCascade;				    /* Automatically generated for this hardware */
PlxOutputCascade*	outputMenuCascade;			    /* Automatically generated for this hardware */
float		totalTimeLost;					    /* Time lost during recording of a movie */
Boolean		jpegAvailable;					    /* Whether JPEG hardware is available or not */
int		syncSlack;
