/* $Header: /plx/source/Xclients/generic/RCS/hpAudio.c,v 0.105 1995/06/13 22:26:22 drapeau Exp $ */
/*
 *   Copyright (c) 1994 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Log: hpAudio.c,v $
 * Revision 0.105  1995/06/13  22:26:22  drapeau
 * Put in some changes to make sure that the HP audio device is working
 * correctly when Play() is called (i.e., the audio device is flushed).
 *
 * Revision 0.104  1995/05/23  19:33:02  lovvik
 * Removed assignments to the global variable audioPlayHeader
 * within InitializeAudio().  This allows the application to
 * retain current audio settings while initializing the audio
 * device.
 *
 * Revision 0.103  1995/05/12  00:17:47  lovvik
 * Moved the static variable "firstTime" to the top of the
 * module.  This variable was located in the function
 * SetAudioPlaybackConfiguration().  Also, I set this variable
 * to True inside InitializeAudio().  This modification allows
 * the module to be initialized more than once.
 *
 * Revision 0.102  1995/05/05  16:15:02  lovvik
 * Fixed a bug in the InitializeAudio() function that set
 * up the play and monitor volumes incorrectly.  The problem
 * was that it was using the recordRange to scale the
 * play and monitor gains.  Also, I set it up so that the
 * default state uses the speaker and line in.
 *
 * Revision 0.101  1995/04/27  00:18:13  lovvik
 * Added an audio object called defaultAudioObject.  This audio object is
 * initialized with the HPAudioDevice audio functions.  Also, I prefixed
 * all of the function names with HPAudioDevice.
 *
 * Revision 0.100  1995/02/13  16:36:34  lovvik
 * Modified this module to separate calls to read and
 * write to the audio device from the main application.
 * Also, I modified the parameter lists of all funcions
 * to disinclude the audio file descriptor.
 *
 * Revision 0.99  1994/09/23  07:36:26  drapeau
 * Modified RecordPause(), restoring the original code to reset the audio
 * record buffer before pausing it.  The reset call should empty any
 * audio bytes that had been in the buffer before.
 *
 * Revision 0.98  1994/09/23  06:30:23  drapeau
 * Moved some operations into different functions to fix some errors in
 * the way playback settings were attempted (some combinations of
 * encoding, sample rate, and number of channels seem to cause the audio
 * device problems, and so the code to change these is all handled in a
 * single place).
 *
 * Revision 0.97  1994/08/23  05:09:34  drapeau
 * Modified InitializeAudio(), adding code to initialize the default
 * audioPlayHeader and audioRecordHeader data structures, thereby
 * removing a divide-by-zero error.
 *
 * Revision 0.96  1994/08/23  01:34:03  drapeau
 * Added code for an alert dialog box.
 *
 * Revision 0.95  1994/08/11  03:17:26  drapeau
 * Added an experimental function for synchronizing audio with video.  It
 * didn't really do any good thus far, so a return stmt was placed at the
 * beginning of the function.
 *
 * Revision 0.94  1994/06/24  01:16:24  drapeau
 * Added the function AudioClose() to properly close the audio device.
 *
 * Revision 0.93  1994/06/10  01:25:50  drapeau
 * Minor changes to the HP audio support for the first St. Filbert
 * release.  Also, added code to support command-line specification of a
 * sample rate.
 *
 * Revision 0.92  1994/05/31  20:11:50  drapeau
 * Enhanced support for audio on the HP platform, removing several errors
 * and making the software more complete.  Also, cleaned up diagnostic
 * reporting.
 *
 * Revision 0.91  1994/01/20  22:49:05  drapeau
 * Modified PortableAudio functions to accommodate a change in the Sun
 * implementation, which in my opinion is due to a flawed audio device
 * driver.
 *
 * Revision 0.90  1994/01/20  17:50:45  drapeau
 * This is the first working prototype, developed with the HP
 * prototype board.
 * */
static char audioFunctionsRcsId[] = "$Header: /plx/source/Xclients/generic/RCS/hpAudio.c,v 0.105 1995/06/13 22:26:22 drapeau Exp $";

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <time.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <errno.h>
#include <sys/audio.h>
#include "PortableAudio.h"
#include "MovieTool.h"
#include "hpAudioDevice.h"

static struct audio_gain	hpAudioVolumeInfo;
static struct audio_describe	hpGainInfo;
static int			playRange;
static int			recordRange;
static int			monitorRange;
static int                      audioFD;                           /* audio file descriptor */
static Boolean			firstTime = True;


AudioObject  defaultAudioObject = 
{
  HPAudioDeviceAudioWrite,
  HPAudioDeviceAudioRead,
  HPAudioDeviceAudioPlayConnect,
  HPAudioDeviceAudioPlayDisconnect,
  HPAudioDeviceAudioRecordConnect,
  HPAudioDeviceAudioRecordDisconnect,
  HPAudioDeviceAudioRecordPause,
  HPAudioDeviceAudioRecordResume,
  HPAudioDeviceAudioPlayPause,
  HPAudioDeviceAudioPlayResume,
  HPAudioDevicePrintAudioInfo,
  HPAudioDeviceInitializeAudio,
  HPAudioDeviceSetAudioPlayVolume,
  HPAudioDeviceSetAudioRecordVolume,
  HPAudioDeviceSetAudioMonitorVolume,
  HPAudioDeviceSetPlaybackDevice,
  HPAudioDeviceSetRecordDevice,
  HPAudioDeviceSetAudioPlaybackConfiguration,
  HPAudioDeviceAudioClose,
  HPAudioDeviceSyncAudioPlaybackDevice,
  HPAudioDeviceSyncAudioRecordDevice,
  HPAudioDeviceAudioStopPlayDevice,
  HPAudioDeviceAudioStopRecordDevice
};





int HPAudioDeviceAudioWrite(char *audioData, int audioslice)                    /* Write audio data to the audio device */
{
  return(write(audioFD, audioData, audioslice));                   /* Return the number of bytes written */
} /* end function AudioWrite */

int HPAudioDeviceAudioRead(char *audioData, int audioBytesPerFrame)             /* Read the audio information from the audio device */
{
  return(read(audioFD, audioData, audioBytesPerFrame));            /* Return the number of bytes read */
} /* end function AudioRead */

void HPAudioDeviceAudioPlayConnect(void)
{

}

void HPAudioDeviceAudioPlayDisconnect(void)
{

}

void HPAudioDeviceAudioRecordConnect(void)
{

}

void HPAudioDeviceAudioRecordDisconnect(void)
{

}

void HPAudioDeviceAudioRecordPause(void)
{
  ioctl(audioFD, AUDIO_RESET, RESET_RX_BUF);
  ioctl(audioFD, AUDIO_PAUSE, AUDIO_RECEIVE);

} /* end function AudioRecordPause */



void HPAudioDeviceAudioRecordResume(void)
{
  ioctl(audioFD, AUDIO_RESUME, AUDIO_RECEIVE);

} /* end function AudioRecordResume */



void HPAudioDeviceAudioPlayPause(void)
{
  ioctl(audioFD, AUDIO_PAUSE, AUDIO_TRANSMIT);

} /* end function AudioPlayPause */



void HPAudioDeviceAudioPlayResume(void)
{
  ioctl(audioFD, AUDIO_RESUME, AUDIO_TRANSMIT);

} /* end function AudioPlayResume */



void HPAudioDevicePrintAudioInfo(PortableAudio*	audioInfo)
{
  int			deviceType;
  int			result;
  int			audioFormat;
  struct audio_describe	hpAudioInfo;
  struct audio_gain	audioVolumeInfo;
  char			diagString[128];
  
  PrintDiagnostics("Entered PrintAudioInfo.\n");
  result = ioctl(audioFD, AUDIO_DESCRIBE, &hpAudioInfo);	    /* Try to get info about the audio device */
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    switch (errno)
    {
     case EBUSY:
      sprintf(errorMessage, "The audio device is busy.  Check if other applications are currently using the audio device.\n");
      PrintErrorMessage(errorMessage);
      break;
    }
    return;
  }
  audioInfo->play.sampleRate =					    /* What the hell, choose the highest-quality sample rate... */
    hpAudioInfo.sample_rate[hpAudioInfo.nrates - 1];		    /* ...available. */
  audioInfo->record.sampleRate =
    hpAudioInfo.sample_rate[hpAudioInfo.nrates - 1];
  audioInfo->play.sampleRate = sampleRate;
  audioInfo->record.sampleRate = sampleRate;
  sprintf(diagString, "PrintAudioInfo(): Highest alleged sample rate is %d.\n",
	 hpAudioInfo.sample_rate[hpAudioInfo.nrates - 1]);
  PrintDiagnostics(diagString);
  audioInfo->play.channels = hpAudioInfo.nchannels;
  audioInfo->record.channels = hpAudioInfo.nchannels;
  audioInfo->play.bitsPerSample = hpAudioInfo.max_bits_per_sample;
  audioInfo->record.bitsPerSample = hpAudioInfo.max_bits_per_sample;
  result = ioctl(audioFD, AUDIO_GET_DATA_FORMAT, &audioFormat);
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    switch (errno)
    {
     case EINVAL:
     case EBUSY:
      sprintf(errorMessage, "The audio device is busy.  Check if other applications are currently using the audio device.\n");
      PrintErrorMessage(errorMessage);
      break;
    }
    return;
  }
  switch (audioFormat)						    /* What format audio is currently being used? */
  {
   case AUDIO_FORMAT_ULAW:
    PrintDiagnostics("Encoding: uLaw.\n");
    audioInfo->play.encoding = AudioULawEncoding;
    audioInfo->record.encoding = AudioULawEncoding;
    break;
   case AUDIO_FORMAT_ALAW:
    PrintDiagnostics("Encoding: aLaw.\n");
    audioInfo->play.encoding = AudioALawEncoding;
    audioInfo->record.encoding = AudioALawEncoding;
    break;
   case AUDIO_FORMAT_LINEAR16BIT:
    PrintDiagnostics("Encoding: Linear.\n");
    audioInfo->play.encoding = AudioLinearEncoding;
    audioInfo->record.encoding = AudioLinearEncoding;
    break;
  } /* end switch */
  result = ioctl(audioFD, AUDIO_GET_GAINS, &audioVolumeInfo);
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    switch (errno)
    {
     case EBUSY:
      sprintf(errorMessage, "The audio device is busy.  Check if other applications are currently using the audio device.\n");
      PrintErrorMessage(errorMessage);
      break;
    }
    return;
  }
  result = ioctl(audioFD, AUDIO_GET_GAINS, &hpAudioVolumeInfo);
  audioInfo->play.volume = audioVolumeInfo.cgain[0].transmit_gain;
  audioInfo->record.volume = audioVolumeInfo.cgain[0].receive_gain;
  audioInfo->monitorVolume = audioVolumeInfo.cgain[0].monitor_gain;
  hpAudioVolumeInfo.channel_mask = (AUDIO_CHANNEL_LEFT | AUDIO_CHANNEL_RIGHT);
  audioVolumeInfo.channel_mask = (AUDIO_CHANNEL_LEFT | AUDIO_CHANNEL_RIGHT);
  XtVaSetValues(playbackScale,
		XmNvalue,
		((audioInfo->play.volume - hpGainInfo.min_transmit_gain)
		 * 100) / playRange,
		NULL);
  XtVaSetValues(recordScale,
		XmNvalue,
		((audioInfo->record.volume - hpGainInfo.min_receive_gain)
		 * 100) / recordRange,
		NULL);
  XtVaSetValues(monitorScale,
		XmNvalue,
		((audioInfo->monitorVolume - hpGainInfo.min_monitor_gain)
		 * 100) / monitorRange,
		NULL);
  PrintDiagnostics("Audio information:\n");
  PrintDiagnostics("Device type: HP audio.\n");
  if (hpAudioInfo.audio_id == AUDIO_ID_PSB2160)
    PrintDiagnostics("Low quality audio.\n");
  else
    PrintDiagnostics("High quality audio.\n");
  
  sprintf(diagString, "Sample rate: %d.\n", audioInfo->play.sampleRate);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Number of channels: %d.\n", audioInfo->play.channels);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Bits per sample: %d.\n", audioInfo->play.bitsPerSample);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Playback volume: %d.\n", audioInfo->play.volume);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Record volume: %d.\n", audioInfo->record.volume);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Monitor volume: %d.\n", audioInfo->monitorVolume);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Minimum transmit gain: %d; Max: %d.\n",
	 hpAudioInfo.min_transmit_gain,
	 hpAudioInfo.max_transmit_gain);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Minimum receive gain: %d; Max: %d.\n",
	 hpAudioInfo.min_receive_gain,
	 hpAudioInfo.max_receive_gain);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Minimum monitor gain: %d; Max: %d.\n",
	 hpAudioInfo.min_monitor_gain,
	 hpAudioInfo.max_monitor_gain);
  PrintDiagnostics(diagString);
  return;
} /* end function PrintAudioInfo */



int HPAudioDeviceInitializeAudio(void) /* returns status:  -1 = fail, 0 = ok */
{
  int			audioFormat;
  int			result;
  int			tempValue;
  struct audio_describe	hpAudioInfo;
  struct audio_gain	audioVolumeInfo;
  
  PrintDiagnostics("Entered InitializeAudio.\n");
  firstTime = True;
  audioFD = open("/dev/audio", O_RDWR);			    /* Try to open the audio device */
  if (audioFD == -1)						    /* Did the open fail? */
  {								    /* Yes, report the error and exit immediately */
    sprintf(errorMessage,
	    "Sorry, could not open the audio device.  Exiting.\n");
    PrintErrorMessage(errorMessage);
    return(-1);                                                     /* return error status */
  }
  result = ioctl(audioFD, AUDIO_DESCRIBE, &hpAudioInfo);	    /* Try to get info about the audio device */
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    switch (errno)
    {
     case EBUSY:
      sprintf(errorMessage, "The audio device is busy.  Check if other applications are currently using the audio device.\n");
      PrintErrorMessage(errorMessage);
      break;
    }
    return(-1);                                                     /* return error status */
  }
  result = ioctl(audioFD, AUDIO_GET_DATA_FORMAT, &audioFormat);
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    switch (errno)
    {
     case EINVAL:
     case EBUSY:
      sprintf(errorMessage, "The audio device is busy.  Check if other applications are currently using the audio device.\n");
      PrintErrorMessage(errorMessage);
      break;
    }
    return(-1);                                                     /* Return error status */
  }
  switch (audioFormat)						    /* What format audio is currently being used? */
  {
   case AUDIO_FORMAT_ULAW:
    audioRecordHeader.encoding = AudioULawEncoding;
    break;
   case AUDIO_FORMAT_ALAW:
    audioRecordHeader.encoding = AudioALawEncoding;
    break;
   case AUDIO_FORMAT_LINEAR16BIT:
    audioRecordHeader.encoding = AudioLinearEncoding;
    break;
  } /* end switch */

  audioRecordHeader.sample_rate = sampleRate;
  audioRecordHeader.samples_per_unit = 1;
  audioRecordHeader.bytes_per_unit =
    hpAudioInfo.max_bits_per_sample / 8;
  audioRecordHeader.channels = hpAudioInfo.nchannels;
  audioRecordHeader.data_size = sampleRate * 2;
  result = ioctl(audioFD, AUDIO_DESCRIBE, &hpGainInfo);		    /* Try to get info about the audio device again */
  playRange = hpGainInfo.max_transmit_gain - hpGainInfo.min_transmit_gain;
  recordRange = hpGainInfo.max_receive_gain - hpGainInfo.min_receive_gain;
  monitorRange = hpGainInfo.max_monitor_gain - hpGainInfo.min_monitor_gain;
  audioVolumeInfo.cgain[0].receive_gain =		    /* Set record to half volume */
    ((50 * recordRange) / 100) + hpGainInfo.min_receive_gain;
  audioVolumeInfo.cgain[1].receive_gain =		    /* Set record to half volume */
    ((50 * recordRange) / 100) + hpGainInfo.min_receive_gain;
  audioVolumeInfo.cgain[0].transmit_gain =		    /* Set playback to half volume */
    ((90 * playRange) / 100) + hpGainInfo.min_transmit_gain;
  audioVolumeInfo.cgain[1].transmit_gain =		    /* Set playback to half volume */
    ((90 * playRange) / 100) + hpGainInfo.min_transmit_gain;
  audioVolumeInfo.cgain[0].monitor_gain =		    /* Set monitor to half volume */
    ((100 * monitorRange) / 100) + hpGainInfo.min_monitor_gain;
  audioVolumeInfo.cgain[1].monitor_gain =		    /* Set monitor to half volume */
    ((100 * monitorRange) / 100) + hpGainInfo.min_monitor_gain;
  audioVolumeInfo.channel_mask = (AUDIO_CHANNEL_LEFT | AUDIO_CHANNEL_RIGHT);
  result = ioctl(audioFD, AUDIO_SET_GAINS, &audioVolumeInfo);
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    switch (errno)
    {
     case EBUSY:
      sprintf(errorMessage, "The audio device is busy.  Check if other applications are currently using the audio device.\n");
      PrintErrorMessage(errorMessage);
      break;
    }
    return(-1);
  }
  tempValue = AUDIO_OUT_SPEAKER;
  ioctl(audioFD, AUDIO_SET_OUTPUT, tempValue);
  tempValue = AUDIO_IN_MIKE;
  ioctl(audioFD, AUDIO_SET_INPUT, tempValue);
  if (hpAudioInfo.audio_id == AUDIO_ID_PSB2160)			    /* What type of audio device is this? */
  {
    PrintDiagnostics("Low quality audio.\n");
  }
  else
  {
    PrintDiagnostics("High quality audio.\n");
    audioRecordHeader.encoding = AudioLinearEncoding;
    result = ioctl(audioFD, AUDIO_SET_RXBUFSIZE, 192 * 1024);	    /* Allocate enough space to hold one second of highest...  */
    if (result == -1)						    /* ...quality audio for recording. */
    {
      PrintDiagnostics("Could not set audio record buffer size.\n");
    }
    result = ioctl(audioFD, AUDIO_SET_TXBUFSIZE, 192 * 1024);	    /* Allocate enough space to hold one second of highest... */
    if (result == -1)						    /* ...quality audio for playback. */
    {
      PrintDiagnostics("Could not set audio playback buffer size.\n");
    }
  }

  SetPlaybackDevice(AudioSpeaker);
  SetRecordDevice(AudioLineIn);

  AudioRecordPause();
  ioctl(audioFD, AUDIO_DRAIN, AUDIO_RECEIVE);

  return(0);                                                        /* Return ok status */
} /* end function InitializeAudio */



void HPAudioDeviceSetAudioPlayVolume(int volume)
{
  int	result;

  audioInfo.play.volume = volume;
  hpAudioVolumeInfo.cgain[0].transmit_gain =		    /* Scale playback volume to fit accepted HP range */
    ((volume * playRange) / 100) + hpGainInfo.min_transmit_gain;
  hpAudioVolumeInfo.cgain[1].transmit_gain =
    ((volume * playRange) / 100) + hpGainInfo.min_transmit_gain;
  result = ioctl(audioFD, AUDIO_SET_GAINS, &hpAudioVolumeInfo);
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    switch (errno)
    {
     case EBUSY:
      sprintf(errorMessage, "The audio device is busy and so cannot set playback volume.  Check if other applications are currently using the audio device.\n");
      PrintErrorMessage(errorMessage);
      break;
    }
  }
  return;
} /* end function SetAudioPlayVolume */



void HPAudioDeviceSetAudioRecordVolume(int volume)
{
  int	result;
  
  hpAudioVolumeInfo.cgain[0].receive_gain =		    /* Scale playback volume to fit accepted HP range */
    ((volume * recordRange) / 100) + hpGainInfo.min_receive_gain;
  hpAudioVolumeInfo.cgain[1].receive_gain =
    ((volume * recordRange) / 100) + hpGainInfo.min_receive_gain;
  result = ioctl(audioFD, AUDIO_SET_GAINS, &hpAudioVolumeInfo);
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    switch (errno)
    {
     case EBUSY:
      sprintf(errorMessage, "The audio device is busy and so cannot set record volume.  Check if other applications are currently using the audio device.\n");
      PrintErrorMessage(errorMessage);
      break;
    }
  }
} /* end function SetAudioRecordVolume */




void HPAudioDeviceSetAudioMonitorVolume(int volume)
{
  int	result;
  
  hpAudioVolumeInfo.cgain[0].monitor_gain =		    /* Scale playback volume to fit accepted HP range */
    ((volume * monitorRange) / 100) + hpGainInfo.min_monitor_gain;
  hpAudioVolumeInfo.cgain[1].monitor_gain =
    ((volume * monitorRange) / 100) + hpGainInfo.min_monitor_gain;
  result = ioctl(audioFD, AUDIO_SET_GAINS, &hpAudioVolumeInfo);
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    switch (errno)
    {
     case EBUSY:
      sprintf(errorMessage, "The audio device is busy and so cannot set monitor volume.  Check if other applications are currently using the audio device.\n");
      PrintErrorMessage(errorMessage);
      break;
    }
  }
} /* end function SetAudioMonitorVolume */



void HPAudioDeviceSetPlaybackDevice(int deviceType)
{
  int	localDeviceType;
  
  if (deviceType == AudioSpeaker)
  {
    PrintDiagnostics("SetPlaybackDevice(): Speaker set.\n");
    localDeviceType = AUDIO_OUT_SPEAKER;
  }
  if (deviceType == AudioLineOut)
  {
    PrintDiagnostics("SetPlaybackDevice(): Line Out set.\n");
    localDeviceType = AUDIO_OUT_LINE;
  }
  if (deviceType == AudioHeadphone)
  {
    PrintDiagnostics("SetPlaybackDevice(): Headphone set.\n");
    localDeviceType = AUDIO_OUT_HEADPHONE;
  }
  ioctl(audioFD, AUDIO_SET_OUTPUT, localDeviceType);
  return;
} /* end function SetPlaybackDevice */



void HPAudioDeviceSetRecordDevice(int deviceType)
{
  int	localDeviceType;
  
  if (deviceType == AudioMicrophone)
    localDeviceType = AUDIO_IN_MIKE;
  else
    localDeviceType = AUDIO_IN_LINE;
  ioctl(audioFD, AUDIO_SET_INPUT, localDeviceType);
} /* end function SetPlaybackDevice */



int HPAudioDeviceSetAudioPlaybackConfiguration(Audio_hdr*	audioInfo)
{
  int			audioResult;
  static Audio_hdr	currentAudioSettings;
  int			descriptorTableSize = 0;
  int			result;
  int			tempValue;
  struct timeval	waitTime;				    /* Used to tell select() how long to wait until it returns */

  if(audioInfo->encoding == 0 &&
     audioInfo->sample_rate == 0 &&
     audioInfo->channels == 0)
  {
    return(0);
  }
  descriptorTableSize = sysconf(_SC_OPEN_MAX);			    /* Get size of descriptor table for later use by select() */
  waitTime.tv_sec = 1;						    /* Set up a timeval to wait for 1000 msecs, giving the... */
  waitTime.tv_usec = 0;						    /* ...audio device time to empty its buffers. */
  if (firstTime == True)					    /* Note the initial audio playback device settings for... */
  {								    /* ...later use. */
    currentAudioSettings.channels = -1;
    currentAudioSettings.sample_rate = -1;
    currentAudioSettings.encoding = -1;
    firstTime = False;
  }
  if (currentAudioSettings.encoding != audioInfo->encoding)	    /* Only change the audio encoding if the requested... */
  {								    /* ...configuration differs from the current one. */
    switch (audioInfo->encoding)				    /* Translate the PortableAudio encoding constant into the... */
    {								    /* ...appropriate HP-specific constant. */
     case AudioULawEncoding:
      tempValue = AUDIO_FORMAT_ULAW;
      PrintDiagnostics("SetAudioPlaybackConfig(): Setting encoding to ulaw.\n");
      break;
     case AudioALawEncoding:
      tempValue = AUDIO_FORMAT_ALAW;
      PrintDiagnostics("SetAudioPlaybackConfig(): Setting encoding to alaw.\n");
      break;
     case AudioLinearEncoding:
      tempValue = AUDIO_FORMAT_LINEAR16BIT;
      PrintDiagnostics("SetAudioPlaybackConfig(): Setting encoding to linear.\n");
      break;
    }
    result = ioctl(audioFD, AUDIO_SET_DATA_FORMAT, tempValue);
    while (result == -1)
    {
      sprintf(diagString, "SetAudioPlaybackConfiguration(): Could not set audio data format, errno was %d.\n",
	      errno);
      PrintDiagnostics(diagString);
      select(descriptorTableSize,				    /* Perform the wait (not really listening for input on... */
	     0, 0, 0,						    /* ...any file descriptors). */
	     &waitTime);
      result = ioctl(audioFD, AUDIO_SET_DATA_FORMAT, tempValue);
    }
  } /* end (if encodings differ...) */
  currentAudioSettings.encoding = audioInfo->encoding;
  if (currentAudioSettings.sample_rate != audioInfo->sample_rate)   /* Only change the sample rate if the requested... */
  {								    /* ...configuration differs from the current one. */
    result = ioctl(audioFD, AUDIO_SET_SAMPLE_RATE, audioInfo->sample_rate);
    while (result == -1)
    {
      sprintf(diagString, "SetAudioPlaybackConfiguration(): Could not set sample rate, errno was %d.\n",
	      errno);
      PrintDiagnostics(diagString);
      select(descriptorTableSize,				    /* Perform the wait (not really listening for input on... */
	     0, 0, 0,						    /* ...any file descriptors). */
	     &waitTime);
      result = ioctl(audioFD, AUDIO_SET_SAMPLE_RATE, audioInfo->sample_rate);
    }
  }  
  currentAudioSettings.sample_rate = audioInfo->sample_rate;
  if (currentAudioSettings.channels != audioInfo->channels)	    /* Only change the number of channels if the requested... */
  {								    /* ...configuration differs from the current one. */
    result = ioctl(audioFD, AUDIO_SET_CHANNELS, audioInfo->channels);
    while (result == -1)
    {
      sprintf(diagString, "SetAudioPlaybackConfiguration(): Could not set number of channels, errno was %d.\n",
	      errno);
      PrintDiagnostics(diagString);
      select(descriptorTableSize,				    /* Perform the wait (not really listening for input on... */
	     0, 0, 0,						    /* ...any file descriptors). */
	     &waitTime);
      result = ioctl(audioFD, AUDIO_SET_CHANNELS, audioInfo->channels);
    }
  }
  currentAudioSettings.channels = audioInfo->channels;
  return(0);							    /* Yes, return zero */
} /* end function SetAudioPlaybackConfiguration */



void HPAudioDeviceAudioClose(void)
{
  ioctl(audioFD, AUDIO_RESET, (RESET_RX_BUF | RESET_TX_BUF));	    /* Clear out all audio in the audio device */
  close(audioFD);						    /* Close the audio device properly */
  return;
} /* end function AudioClose */



void HPAudioDeviceSyncAudioPlaybackDevice(void)
{
  struct timeval	after;
  struct timeval	before;
  int			bytesWritten;
  int			counter = 0;
  int			elapsedTime = 0;
  static char*		syncBuffer = NULL;
  
  HPAudioDeviceAudioPlayPause();				    /* Make sure the audio device play buffer is cleared out */
  HPAudioDeviceAudioPlayResume();
  if (syncBuffer == NULL)					    /* Allocate some memory for holding audio to be pushed... */
  {								    /* ...through the audio device. */
    syncBuffer = malloc(48000);					    /* I'm guessing that 48K is the max. sample rate and will... */
    if (syncBuffer == NULL)					    /* ...be an upper-bound on sound size used. */
    {
      sprintf(errorMessage,
	      "Could not allocate enough audio memory to properly sync audio.\n");
      PrintErrorMessage(errorMessage);
      return;
    }
    memset(syncBuffer, '\0', 48000);				    /* Fill the audio buffer with zeros */
  }
  write(audioFD, syncBuffer, 32000);				    /* Fill up the audio device's "current page" */
  return;
  while (elapsedTime < (1000 / playbackSpeed))			    /* Keep pushing audio until time to play 1 frame is close... */
  {								    /* ...to real-time */
    if (printDiagnostics == True)
    {
      printf("Playing %d bytes of audio (iteration %d).\n",
	     movieHeader.audioslice, counter++);
    }
    gettimeofday(&before, (struct timezone*)NULL);
    bytesWritten = write(audioFD, syncBuffer,		    /* Playback 1 frame's worth of audio */
			 movieHeader.audioslice);
    gettimeofday(&after, (struct timezone*)NULL);
    if ((bytesWritten < 0) && (printDiagnostics == True))
    {
      printf("SyncAudioPlaybackDevice(): Error %d (errno is %d) writing audio.\n",
	     bytesWritten, errno);
    }
    elapsedTime = ElapsedTime(&before, &after);			    /* Count time in milliseconds to play 1 frame of audio */
  }
  return;
} /* end function SyncAudioPlaybackDevice */



void HPAudioDeviceSyncAudioRecordDevice(void)
{
  struct timeval	after;
  struct timeval	before;
  int			bytesWritten;
  int			counter = 0;
  int			elapsedTime = 0;
  static char*		syncBuffer = NULL;
  
  return;
  if (syncBuffer == NULL)					    /* Allocate some memory for holding audio to be pushed... */
  {								    /* ...through the audio device. */
    syncBuffer = malloc(48000);					    /* I'm guessing that 48K is the max. sample rate and will... */
    if (syncBuffer == NULL)					    /* ...be an upper-bound on sound size used. */
    {
      sprintf(errorMessage,
	      "Could not allocate enough audio memory to properly sync audio.\n");
      PrintErrorMessage(errorMessage);
      return;
    }
    memset(syncBuffer, '\0', 48000);				    /* Fill the audio buffer with zeros */
  }
  read(audioFD, syncBuffer, 8000);				    /* Fill up the audio device's "current page" */
  return;
} /* end function SyncAudioRecordDevice */


void HPAudioDeviceAudioStopPlayDevice(void)
{

}

void HPAudioDeviceAudioStopRecordDevice(void)
{

}
