/* $Header: /plx/source/Xclients/generic/RCS/hpAudioSS.c,v 1.6 1995/05/23 19:31:57 lovvik Exp $ */

/*
 *   Copyright (c) 1994 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */

/* $Log: hpAudioSS.c,v $
 * Revision 1.6  1995/05/23  19:31:57  lovvik
 * Moved the static variable 'firstTime' to the top of the
 * module.  This variable used to be in the function
 * SetAudioPlaybackConfiguration().  I also removed assignments
 * to the global variable audioPlayHeader within InitializeAudio().
 * This allows the application to retain current audio settings
 * while initializing the audio device.
 *
 * Revision 1.5  95/05/09  19:03:41  19:03:41  lovvik ()
 * Modified the call to AOpenAudio so it tries to connect
 * with the audio server from the same host that
 * MovieTool is displaying on.
 * 
 * Revision 1.4  1995/05/05  16:16:00  lovvik
 * Added some code to correctly set up the play, record,
 * and monitor gains.  Also, I set it up so the default state
 * uses the speaker and line in.
 *
 * Revision 1.3  1995/05/01  16:29:53  lovvik
 * Fixed a problem in SetRecordDevice() and SetPlayDevice().  The problem
 * was that when you selected a device, the selection would not take place
 * until you started the record or play operation.  The fix was to do a
 * record or a play upon selecting a new device.
 *
 * Revision 1.2  1995/04/28  21:38:03  lovvik
 * Changed all occurances of AMinInputGain to AMinOutputGain.  Also changed
 * AMaxInputGain to AMaxOutputGain.  These functions return incorrect values,
 * and it was causing the record slider position to be set to a negative value.
 *
 * Revision 1.1  1995/04/27  00:23:23  lovvik
 * Initial revision
 * */

static char hpAudioSSRCSid[] = "$Header: /plx/source/Xclients/generic/RCS/hpAudioSS.c,v 1.6 1995/05/23 19:31:57 lovvik Exp $";

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/socket.h>
#include <unistd.h>
#include <netinet/in.h>
#include <errno.h>
#include <sys/audio.h> 
#include <audio/Alib.h>
#include <audio/CUlib.h>

#include "PortableAudio.h"
#include "MovieTool.h"
#include "hpAudioSS.h"

#define MAXIMUM_BITS_PER_SAMPLE 16

static struct audio_gain        hpAudioVolumeInfo;
static struct audio_describe    hpGainInfo;
static AGainDB                  playRange;
static int                      recordRange;
static int                      monitorRange;
static Audio*                   audio; 
static SBucket*                 playSBucket;
static SBucket*                 recordSBucket;
static SSPlayParams             playParameters;
static AGainEntry               playGainEntry[4];
static AGainMatrix              playGainMatrix;
static AudioAttributes          audioattr;
static ATime                    atime;
static int                      playStreamSocket;
static int                      recordStreamSocket;
static SStream                  playAudioStream;
static SStream                  recordAudioStream;
static ATransID                 playProcessID;
static ATransID                 recordProcessID;
static fd_set                   recordFDs;
static int                      playConnected = 0;
static int                      recordConnected = 0;
static Boolean			firstTime = True;

static SSRecordParams           recordParameters;
static AGainEntry               recordGainEntry[4];
static AGainMatrix              recordGainMatrix;

AudioObject  alternateAudioObject1 = 
{
  HPAudioServerAudioWrite,
  HPAudioServerAudioRead,
  HPAudioServerAudioPlayConnect,
  HPAudioServerAudioPlayDisconnect,
  HPAudioServerAudioRecordConnect,
  HPAudioServerAudioRecordDisconnect,
  HPAudioServerAudioRecordPause,
  HPAudioServerAudioRecordResume,
  HPAudioServerAudioPlayPause,
  HPAudioServerAudioPlayResume,
  HPAudioServerPrintAudioInfo,
  HPAudioServerInitializeAudio,
  HPAudioServerSetAudioPlayVolume,
  HPAudioServerSetAudioRecordVolume,
  HPAudioServerSetAudioMonitorVolume,
  HPAudioServerSetPlaybackDevice,
  HPAudioServerSetRecordDevice,
  HPAudioServerSetAudioPlaybackConfiguration,
  HPAudioServerAudioClose,
  HPAudioServerSyncAudioPlaybackDevice,
  HPAudioServerSyncAudioRecordDevice,
  HPAudioServerAudioStopPlayDevice,
  HPAudioServerAudioStopRecordDevice
};

int HPAudioServerAudioWrite(char *audioData, int audioslice)                    /* Write audio data to the audio device */
{

  long                  byteCount = 0;
  long                  bytesWritten = 0;
  int                   audioBufferPointer = 0;

  while(byteCount < audioslice)
  {
    bytesWritten = write(playStreamSocket, (audioData + audioBufferPointer), audioslice - audioBufferPointer);
    byteCount += bytesWritten;
    audioBufferPointer += (int)bytesWritten;
  }
  return(byteCount);                                               /* Return the number of bytes written */
}  /* end function AudioWrite */


int HPAudioServerAudioRead(char *audioData, int audioslice)                     /* Read the audio information from the audio device */
{
  long                  byteCount = 0;
  long                  bytesRead = 0;

  int audioBufferPointer = (int)NULL;

  if(select(recordStreamSocket + 1, (int *) &recordFDs, (int *)NULL,
	    (int *)NULL, (struct timeval *) 0) < 0)
  {
    return(-1);
  }
  if(FD_ISSET(recordStreamSocket, &recordFDs))
  {
    while(byteCount < audioslice)
    {
      bytesRead = read(recordStreamSocket, (audioData + audioBufferPointer), audioslice - audioBufferPointer);
      byteCount += bytesRead;
      audioBufferPointer += (int) bytesRead;
    }
  }

  return(byteCount);                                               /* Return the number of bytes read */
}  /* end function AudioRead */


void HPAudioServerAudioPlayConnect(void)
{
  long status;
  if(!playConnected)
  {
    playProcessID = APlaySStream(audio, ASDataFormatMask | ASBitsPerSampleMask | 
				 ASSamplingRateMask | ASChannelsMask | 
				 ASInterleaveMask | ASDurationMask, &audioattr, 
				 &playParameters, &playAudioStream, &status);
    playStreamSocket = socket(AF_INET, SOCK_STREAM, 0);
    status = connect(playStreamSocket, (struct sockaddr *)&playAudioStream.tcp_sockaddr,
		     sizeof(struct sockaddr_in));
    AGrabServer(audio, NULL);
    XFlush(XtDisplay(mainWindow));

    playConnected = 1;
  }
}


void HPAudioServerAudioPlayDisconnect(void)
{
  long status;

  if(playConnected)
  {
    AUngrabServer(audio, &status);
    if(status != NULL)
    {
    }
    AStopAudio(audio, playProcessID, ASMThisTrans, NULL, &status);
    if(status != NULL)
    {
    }
    close(playStreamSocket);
    playConnected = 0;
  }
}


void HPAudioServerAudioRecordConnect(void)
{
  long status;

  if(!recordConnected)
  {
    recordStreamSocket = socket(AF_INET, SOCK_STREAM, 0);
    recordProcessID = ARecordSStream(audio, ~0, &audioattr, 
				     &recordParameters, &recordAudioStream, &status);
    status = connect(recordStreamSocket, (struct sockaddr *)&recordAudioStream.tcp_sockaddr,
		     sizeof(struct sockaddr_in));
    FD_ZERO(&recordFDs);
    FD_SET(recordStreamSocket, &recordFDs);
    AGrabServer(audio, NULL);
    XFlush(XtDisplay(mainWindow));

    recordConnected = 1;
  }
}


void HPAudioServerAudioRecordDisconnect(void)
{
  if(recordConnected)
  {
    AUngrabServer(audio, NULL);
    AStopAudio(audio, recordProcessID, ASMThisTrans, NULL, NULL);
    FD_CLR(recordStreamSocket, &recordFDs);
    close(recordStreamSocket);
    recordConnected = 0;
  }
}



void HPAudioServerAudioRecordPause(void)
{
  if(recordConnected)
  {
    APauseAudio(audio, recordProcessID, NULL, NULL);
  }
}  /* end function AudioRecordPause */


void HPAudioServerAudioRecordResume(void)
{
  if(recordConnected)
  {
    AResumeAudio(audio, recordProcessID, NULL, NULL);
  }
}  /* end function AudioRecordResume */


void HPAudioServerAudioPlayPause(void)
{
  if(playConnected)
  {
    APauseAudio(audio, playProcessID, NULL, NULL);
  }
}  /* end function AudioPlayPause */


void HPAudioServerAudioPlayResume(void)
{
  if(playConnected)
  {
    AResumeAudio(audio, playProcessID, NULL, NULL);
  }
}  /* end function AudioPlayResume */


void HPAudioServerPrintAudioInfo(PortableAudio* audioInfo)
{
  unsigned long *sampleRatesPointer;
  long           numberOfSamplingRates;
  char           diagString[128];
  AInputChMask   inputChannels;
  AOutputChMask  outputChannels;
  AGainDB        gain;
  long           status;

  PrintDiagnostics("Entered PrintAudioInfo.\n");
/* get the sampling rate */
  sampleRatesPointer = ASamplingRates(audio);
  numberOfSamplingRates = ANumSamplingRates(audio);
  if(numberOfSamplingRates == 0)
  {
  }
  audioInfo->play.sampleRate = sampleRatesPointer[numberOfSamplingRates - 1];
  audioInfo->record.sampleRate = sampleRatesPointer[numberOfSamplingRates - 1];
  audioInfo->play.sampleRate = sampleRate;
  audioInfo->record.sampleRate = sampleRate;
  sprintf(diagString, "PrintAudioInfo(): Highest alleged sample rate is %d.\n",
	  sampleRatesPointer[numberOfSamplingRates - 1]);
  PrintDiagnostics(diagString);

/*************************/

/* get the number of channels */
  inputChannels = AInputChannels(audio);
  if(inputChannels & (ALeftInputChMask | ARightInputChMask))
  {
    audioInfo->play.channels = 2;
  }
  else
  {
    audioInfo->play.channels = 1;
  }

  outputChannels = AOutputChannels(audio);
  if(outputChannels & (ALeftOutputChMask | ARightOutputChMask))
  {
    audioInfo->record.channels = 2;
  }
  else
  {
    audioInfo->record.channels = 1;
  }
  
  audioInfo->play.bitsPerSample = MAXIMUM_BITS_PER_SAMPLE;
  audioInfo->record.bitsPerSample = MAXIMUM_BITS_PER_SAMPLE;

  PrintDiagnostics("Encoding: Linear.\n");
  audioInfo->play.encoding = AudioLinearEncoding;
  audioInfo->record.encoding = AudioLinearEncoding;

  AGetSystemChannelGain(audio, ASGTPlay, ACTMono, &gain, &status);
  audioInfo->play.volume = gain;

  AGetSystemChannelGain(audio, ASGTRecord, ACTLeft, &gain, &status);
  audioInfo->record.volume = gain;
  
  AGetSystemChannelGain(audio, ASGTMonitor, ACTMono, &gain, &status);
  audioInfo->monitorVolume = gain;

  XtVaSetValues(playbackScale,
		XmNvalue,
		((audioInfo->play.volume - AMinOutputGain(audio))
		 * 100) / playRange,
		NULL);

/* Note that I am using AMinOutputGain.  This is because AMinInputGain returns
   incorrect values.  This should be a good approximation of the legal gain
   limits. */

  XtVaSetValues(recordScale,
		XmNvalue,
		((audioInfo->record.volume - AMinOutputGain(audio))
		 * 100) / recordRange,
		NULL);
  XtVaSetValues(monitorScale,
		XmNvalue,
		((audioInfo->monitorVolume - AMinOutputGain(audio))
		 * 100) / monitorRange,
		NULL);

  PrintDiagnostics("Audio information:\n");
  PrintDiagnostics("Device type: HP audio.\n");
/*  if (hpAudioInfo.audio_id == AUDIO_ID_PSB2160)
    PrintDiagnostics("Low quality audio.\n");
  else
*/
    PrintDiagnostics("High quality audio.\n");
  
  sprintf(diagString, "Sample rate: %d.\n", audioInfo->play.sampleRate);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Number of channels: %d.\n", audioInfo->play.channels);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Bits per sample: %d.\n", audioInfo->play.bitsPerSample);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Playback volume: %d.\n", audioInfo->play.volume);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Record volume: %d.\n", audioInfo->record.volume);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Monitor volume: %d.\n", audioInfo->monitorVolume);
  PrintDiagnostics(diagString);

  sprintf(diagString, "Minimum transmit gain: %d; Max: %d.\n",
	  AMinOutputGain(audio),
	  AMaxOutputGain(audio));
  PrintDiagnostics(diagString);
  sprintf(diagString, "Minimum receive gain: %d; Max: %d.\n",
	  AMinInputGain(audio),
	  AMaxInputGain(audio));
  PrintDiagnostics(diagString);
  sprintf(diagString, "Minimum monitor gain: %d; Max: %d.\n",
	  AMinOutputGain(audio),
	  AMaxOutputGain(audio));
  PrintDiagnostics(diagString);
  return;
}  /* end function PrintAudioInfo */


int HPAudioServerInitializeAudio(void) /* returns status:  -1 = fail, 0 = ok */
{
  long status;


  PrintDiagnostics("Entered InitializeAudio.\n");
  audio = AOpenAudio(displayName, &status);                         /* Try to open the audio device */
  if (audio == NULL)                                                /* Did the open fail? */
  {                                                                 /* Yes, report the error and exit immediately */
    return(-1);  	                                            /* Return error status */
  } /* end if (audio == NULL) */



  playGainEntry[0].u.o.out_ch = AOCTLeft;
  playGainEntry[0].u.o.out_dst = AODTLeftJack;
  playGainEntry[0].gain = AUnityGain;
  
  playGainEntry[1].u.o.out_ch = AOCTRight;
  playGainEntry[1].u.o.out_dst = AODTRightJack;
  playGainEntry[1].gain = AUnityGain;

  playParameters.gain_matrix.type = AGMTOutput;
  playParameters.gain_matrix.num_entries = 2;
  playParameters.gain_matrix.gain_entries = playGainEntry;

  playParameters.play_volume = AUnityGain;

  playParameters.priority = APriorityNormal;
  playParameters.event_mask = (AEventMask) NULL;

  recordGainEntry[0].u.i.in_ch = AICTLeft;
  recordGainEntry[0].u.i.in_src = AISTLeftAuxiliary;
  recordGainEntry[0].gain = AUnityGain;
  
  recordGainEntry[1].u.i.in_ch = AICTRight;
  recordGainEntry[1].u.i.in_src = AISTRightAuxiliary;
  recordGainEntry[1].gain = AUnityGain;

  recordParameters.gain_matrix.type = AGMTInput;
  recordParameters.gain_matrix.num_entries = 2;
  recordParameters.gain_matrix.gain_entries = recordGainEntry;

  recordParameters.record_gain = AUnityGain;
  recordParameters.pause_first = False;

  recordParameters.event_mask = (AEventMask) NULL;

  atime.type = ATTSamples;
  atime.u.samples = 48000;
  audioattr.type = ATSampled;
  audioattr.attr.sampled_attr.data_format = ADFLin16;                    /* linear 16 bit format */
  audioattr.attr.sampled_attr.bits_per_sample = 16;
  audioattr.attr.sampled_attr.sampling_rate = 48000;                     /* 48000 samples per second */
  audioattr.attr.sampled_attr.channels = 2;     
  audioattr.attr.sampled_attr.interleave = 0; 
  audioattr.attr.sampled_attr.duration = atime; 


  playRange = AMaxOutputGain(audio) - AMinOutputGain(audio);
/*  AMinInputGain and AMaxInputGain return bad values.  I am using the values returned
    from AMinOutputGain and AMaxOutputGain instead. */

  recordRange = AMaxOutputGain(audio) - AMinOutputGain(audio);
  monitorRange = playRange;

  audioRecordHeader.encoding = AudioLinearEncoding;
  audioRecordHeader.sample_rate = sampleRate;
  audioRecordHeader.samples_per_unit = 1;
  audioRecordHeader.bytes_per_unit = 2;
  audioRecordHeader.channels = 2;
  audioRecordHeader.data_size = sampleRate * 2 ;

  SetPlaybackDevice(AudioSpeaker);
  SetRecordDevice(AudioLineIn);
  SetAudioPlayVolume(90);
  SetAudioRecordVolume(50);
  SetAudioMonitorVolume(100);

  firstTime = True;

  return(0);
}  /* end function InitializeAudio */


void HPAudioServerSetAudioPlayVolume(int percent)
{
  long status;
  AGainDB volume;

  volume = ((percent * playRange) / 100) + AMinOutputGain(audio);
  ASetSystemPlayGain(audio, volume, &status);
  if(status)
  {
  }
}  /* end function SetAudioPlayVolume */


void HPAudioServerSetAudioRecordVolume(int percent)
{
  long status;
  AGainDB volume;

  volume = ((percent * recordRange) / 100) + AMinOutputGain(audio);
  ASetSystemRecordGain(audio, volume, &status);
  if(status)
  {

  }
}  /* end function SetAudioRecordVolume */


void HPAudioServerSetAudioMonitorVolume(int percent)
{
  long status;
  AGainDB volume;

  volume = ((percent * playRange) / 100) + AMinOutputGain(audio);

  
  ASetSystemChannelGain(audio, ASGTMonitor, ACTMono, (AGainDB) volume, &status);
  if(status)
  {

  }
}  /* end function SetAudioMonitorVolume */


void HPAudioServerSetPlaybackDevice(int deviceType)
{
  int	localDeviceType;
  
  if (deviceType == AudioSpeaker)
  {
    PrintDiagnostics("SetPlaybackDevice(): Speaker set.\n");
    localDeviceType = AUDIO_OUT_SPEAKER;
    playGainEntry[0].u.o.out_dst = AODTMonoIntSpeaker;
    playParameters.gain_matrix.num_entries = 1;
  }
  if (deviceType == AudioLineOut)
  {
    PrintDiagnostics("SetPlaybackDevice(): Line Out set.\n");
    localDeviceType = AUDIO_OUT_LINE;
    playGainEntry[0].u.o.out_dst = AODTLeftJack;
    playParameters.gain_matrix.num_entries = 1;
  }
  if (deviceType == AudioHeadphone)
  {
    PrintDiagnostics("SetPlaybackDevice(): Headphone set.\n");
    localDeviceType = AUDIO_OUT_HEADPHONE;
    playGainEntry[0].u.o.out_dst = AODTLeftJack;
    playGainEntry[1].u.o.out_dst = AODTRightJack;
    playParameters.gain_matrix.num_entries = 2;
  }

  playParameters.gain_matrix.type = AGMTOutput;
  playParameters.gain_matrix.gain_entries = playGainEntry;

/* When making changes to the path of the audio, it is required that you initialize the
   path.  This means that if audio is currently playing, break the connection and re-establish
   it.  If audio is not currently playing, make the connection, play some silence, and then
   close the connection. */

  if(playConnected)
  {
    AudioPlayDisconnect();
    AudioPlayConnect();
  }
  else                         
  {
    char audioSilence[6400];
    long silentValue;
    long significantBytes;
    int index;

    silentValue = AGetSilenceValue(audio, 
				   audioattr.attr.sampled_attr.data_format,
				   &significantBytes,
				   NULL);
    for (index = 0; index < 6400; index++)
    {
      audioSilence[index] = (unsigned char) silentValue;
    }
    AudioPlayConnect();
    AudioWrite(audioSilence, 6400);
    AudioPlayDisconnect();
  }

  return;
}  /* end function SetPlaybackDevice */


void HPAudioServerSetRecordDevice(int deviceType)
{
  int	localDeviceType;
  
  if (deviceType == AudioMicrophone)
  {
    PrintDiagnostics("SetRecordDevice(): Microphone set.\n");
    localDeviceType = AUDIO_IN_MIKE;
    recordGainEntry[0].u.i.in_src = AISTLeftMicrophone;
    recordGainEntry[0].u.i.in_ch  = AICTLeft;
    recordGainEntry[1].u.i.in_src = AISTRightMicrophone;
    recordGainEntry[1].u.i.in_ch  = AICTRight;
    recordParameters.gain_matrix.num_entries = 2;
  }
  else
  {
    PrintDiagnostics("SetRecordDevice(): Line Out set.\n");
    localDeviceType = AUDIO_OUT_LINE;
    recordGainEntry[0].u.i.in_src = AISTLeftAuxiliary;
    recordGainEntry[0].u.i.in_ch  = AICTLeft;
    recordGainEntry[1].u.i.in_src = AISTRightAuxiliary;
    recordGainEntry[1].u.i.in_ch  = AICTRight;
    
    recordParameters.gain_matrix.num_entries = 2;
  }

  recordParameters.gain_matrix.type = AGMTInput;
  recordParameters.gain_matrix.gain_entries = recordGainEntry;

/* When making changes to the path of the audio, it is required that you initialize the
   path.  This means that if audio is currently recording, break the connection and re-establish
   it.  If audio is not currently recording, make the connection and then close the connection. 
   Note that when making a record connection, the record transaction starts immediately. */


  if(recordConnected)
  {
    AudioRecordDisconnect();
    AudioRecordConnect();
  }
  else
  {
    AudioRecordConnect();
    AudioRecordDisconnect();
  }
  return;
}  /* end function SetRecordDevice */


int HPAudioServerSetAudioPlaybackConfiguration(Audio_hdr* audioInfo)
{
  int			audioResult;
  static Audio_hdr	currentAudioSettings;
  int			descriptorTableSize = 0;
  int			result;
  int			tempValue;
  struct timeval	waitTime;				    /* Used to tell select() how long to wait until it returns */
  

  if(audioInfo->encoding == 0 &&
     audioInfo->sample_rate == 0 &&
     audioInfo->channels == 0)
  {
    return(0);							    /* No audio information in the structure */
  }
  descriptorTableSize = sysconf(_SC_OPEN_MAX);			    /* Get size of descriptor table for later use by select() */
  waitTime.tv_sec = 1;						    /* Set up a timeval to wait for 1000 msecs, giving the... */
  waitTime.tv_usec = 0;						    /* ...audio device time to empty its buffers. */
  if (firstTime == True)					    /* Note the initial audio playback device settings for... */
  {								    /* ...later use. */
    currentAudioSettings.channels = -1;
    currentAudioSettings.sample_rate = -1;
    currentAudioSettings.encoding = -1;
    firstTime = False;
  }
  if (currentAudioSettings.encoding != audioInfo->encoding)	    /* Only change the audio encoding if the requested... */
  {								    /* ...configuration differs from the current one. */
    switch (audioInfo->encoding)				    /* Translate the PortableAudio encoding constant into the... */
    {								    /* ...appropriate HP-specific constant. */
     case AudioULawEncoding:
      audioattr.attr.sampled_attr.data_format = ADFMuLaw;
      PrintDiagnostics("SetAudioPlaybackConfig(): Setting encoding to ulaw.\n");
      break;
     case AudioALawEncoding:
      audioattr.attr.sampled_attr.data_format = ADFALaw;
      PrintDiagnostics("SetAudioPlaybackConfig(): Setting encoding to alaw.\n");
      break;
     case AudioLinearEncoding:
      audioattr.attr.sampled_attr.data_format = ADFLin16;
      PrintDiagnostics("SetAudioPlaybackConfig(): Setting encoding to linear.\n");
      break;
    }
    currentAudioSettings.encoding = audioInfo->encoding;
  } /* end (if encodings differ...) */

  if (currentAudioSettings.sample_rate != audioInfo->sample_rate)   /* Only change the sample rate if the requested... */
  {								    /* ...configuration differs from the current one. */
    audioattr.attr.sampled_attr.sampling_rate = audioInfo->sample_rate;
    currentAudioSettings.sample_rate = audioInfo->sample_rate;
  }  
  if (currentAudioSettings.channels != audioInfo->channels)	    /* Only change the number of channels if the requested... */
  {								    /* ...configuration differs from the current one. */
    audioattr.attr.sampled_attr.channels = audioInfo->channels;;
    currentAudioSettings.channels = audioInfo->channels;
  }
  
  return(0);
}  /* end function SetAudioPlaybackConfiguration */


void HPAudioServerAudioClose(void)
{
  long status;

  if(playConnected)
  {
    AudioPlayDisconnect();
  }
  if(recordConnected)
  {
    AudioRecordDisconnect();
  }

/* The following connection/disconnection fixes a problem that may be in the audio server.  Without this
   code, the audio server does not release the audio device.
*/
  AudioRecordConnect();
  AudioRecordDisconnect();


  ASetCloseDownMode(audio, AKeepTransactions, &status);
  if(status != NULL)
  {
  }
  ACloseAudio(audio, &status);
  if(status != NULL)
  {
  }
}  /* end function AudioClose */


void HPAudioServerSyncAudioPlaybackDevice(void)
{
  if(playConnected)
  {
    AudioPlayDisconnect();
  }
  AudioPlayConnect();
  AFlushAudio(audio, playProcessID, NULL, NULL);
}  /* end function SyncAudioPlaybackDevice */

void HPAudioServerSyncAudioRecordDevice(void)
{
  long status;

  if(recordConnected)
  {
    AudioRecordDisconnect(); 
  }
  AudioRecordConnect();
}  /* end function SyncAudioRecordDevice */


void HPAudioServerAudioStopPlayDevice(void)
{
  if(playConnected)
  {
    AudioPlayDisconnect();
  }
}


void HPAudioServerAudioStopRecordDevice(void)
{
  if(recordConnected)
  {
    AudioRecordDisconnect();
  }
}







