/* $Header: /plx/source/Xclients/generic/RCS/hpSpecific.c,v 0.109 1996/01/25 04:18:18 drapeau Exp $ */
/*
 *   Copyright (c) 1994 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Log: hpSpecific.c,v $
 * Revision 0.109  1996/01/25  04:18:18  drapeau
 * Added support for the PlxOutputCascade to automatically create the
 * output menu hierarchy, for enhanced flexibility.
 *
 * Revision 0.108  1996/01/05  23:36:16  drapeau
 * Minor addition to ShowCompactMovieDialog() to account for new
 * interface feature that should be greyed out when this function is
 * called.
 *
 * Revision 0.107  1995/10/06  23:01:15  drapeau
 * Added support for showing the current input signal type in use.
 *
 * Revision 0.106  1995/10/02  21:54:27  drapeau
 * Added support for keeping track of the user interface's visibility
 * state: whether some panels are mapped or not, etc.
 *
 * Revision 0.105  1995/08/23  01:57:18  drapeau
 * Fixed ShowInputWorkProc() so that it correctly resets the recording
 * frame rate based on the input video signal type it detects (30 fps for
 * NTSC, 25 FPS for PAL or SECAM).
 *
 * Revision 0.104  1995/06/23  19:17:36  drapeau
 * Two fixes:
 *
 * 1) Modified CreateJPEGWidget() so that it correctly displays JPEG
 *    images after zooming from one zoom factor to another.  The previous
 *    version would not resize the JPEG image according to the new zoom
 *    factor.
 *
 * 2) Modified ShowInputWorkProc() so that input selection behaves as
 *    with VideoTool.  That is, when an input is explicitly selected,
 *    autoSearch is turned off (since the supposed intent is that the
 *    user wants to show whatever video was selected whether sync was
 *    available or not).
 *
 * Revision 0.103  1995/05/16  02:57:37  drapeau
 * Added support for the InputCascade code written by Dave Arnold, which
 * takes a plx_io data structure and creates a cascading menu-and-button
 * interface that reflects the hardware's input selections.
 *
 * Revision 0.102  1995/05/12  00:15:33  lovvik
 * Modified the initialization of audioMap[].  The structure
 * has been changed to add a string that will serve as a label
 * for each audio object.
 *
 * Revision 0.101  1995/05/02  17:50:12  drapeau
 * Added support for hardware zoom when available.
 *
 * Revision 0.100  1995/04/27  00:28:48  lovvik
 * Added audioMap.  This structure contains:
 *   1)  Command line parameter
 *   2)  Help message
 *   3)  Address of an audio object
 * for each audio object that is supported on HP.
 *
 * Revision 0.99  1994/10/17  21:09:31  drapeau
 * Two changes:
 *
 * 1) In CreateJPEGWidget(), store the previous values of Q factor for
 *    each JPEG widget so they can be restored when the new versions of
 *    the widgets are re-created.
 *
 * 2) In OutputWorkProc(), force the live video window to release its
 *    Port by destroying the VideoIn widget (thereby destroying its
 *    window).
 *
 * Revision 0.98  1994/10/15  01:46:08  drapeau
 * Added code to complete MovieTool's notion of what operations should be
 * allowed under what conditions.  For example, input selection should
 * not be allowed during recording and playback.
 *
 * Revision 0.97  1994/10/14  02:53:00  drapeau
 * Added code to insure that JPEG playback widgets have the correct Q
 * table loaded and that they refresh correctly when output is enabled.
 *
 * Revision 0.96  1994/10/10  23:01:42  drapeau
 * Added functions to allow any of the available input types to be
 * selected.  Also, removed some functions to reflect the interface
 * change (the new function ShowInput() replaces the two functions
 * ShowInput1() and ShowInput2()).
 *
 * Revision 0.95  1994/10/09  02:32:49  drapeau
 * Several changes, having to do with output:
 *
 * * Reorganized OutputCallback() to work in the style of Input1 and
 *   Input2 callbacks; that is, to call a XtWorkProc to do the actual
 *   work.  This is because the output callback now deletes and creates
 *   new windows, and this does not happen immediately in an Xt callback.
 *
 * * In the Output callback work proc, call OutputSelect with the special
 *   channel "PLX_OUTPUT_JPEG_OFF" for the imageQuality widget even
 *   though output is not applied to it.  This hack for the HP explictly
 *   removes XvPortID-to-Window mappings at the XPlx library layer, so
 *   that when the widgets are later deleted and re-created (via
 *   CreateJPEGWidget()), the widgets can allocate Xv Ports for
 *   themselves from scratch and no Ports will be left in the XPlx
 *   layer's internal list of Port-to-Window mappings.
 *
 * Revision 0.94  1994/10/06  02:20:55  drapeau
 * Removed extraneous code from Input1WorkProc() that reset the record
 * speed to the full frame rate for the video standard in effect (30 fps
 * for NTSC, 25 for PAL or SECAM).
 *
 * Revision 0.93  1994/10/03  17:25:50  drapeau
 * Two minor changes:
 *
 * 1) Don't allow output to be used until a JPEG movie is ready to be
 *    played (either one has just been recorded or one has just been
 *    opened for playback).
 *
 * 2) In CreateJPEGWidget(), managed the child of the imageQualityFrame
 *    widget after the new one has been created.
 *
 * Revision 0.92  1994/09/30  17:01:31  drapeau
 * Added support for output.  This version of the module is pretty badly
 * hacked; the implementation is not at all clean.  It would be better if
 * the JPEG widgets did not have to be destroyed and re-created in order
 * to get output working correctly.  Currently they have to be destroyed
 * because it is the only way to detach Xv Ports from their drawables.
 * It would be better if Xv provided a way to let you manually detach a
 * Port from a drawable.
 *
 * Revision 0.91  1994/09/13  08:09:58  drapeau
 * Removed extraneous variables.
 *
 * Revision 0.90  1994/08/27  05:49:02  drapeau
 * First working version.
 * */
static char hpSpecificRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/hpSpecific.c,v 0.109 1996/01/25 04:18:18 drapeau Exp $";

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/Scale.h>
#include <Xm/PushBG.h>
#include "PortableAudio.h"
#include "MovieTool.h"



AudioModuleMapEntry audioMap[5] =
{
  {"HPAudioDevice", "HP Audio Device", "Bypasses the HP Audio Server, uses a direct\n\t\t\t\tconnection to the audio hardware.  (Best Quality)", &defaultAudioObject},
  {"HPAudioServer", "HP Audio Server", "Uses the HP Audio Server.  (Allows MovieTool\n\t\t\t\tto run concurrently with other audio applications)", &alternateAudioObject1},
  {"None", "None", "Does not record or play audio", &noAudioObject},
  {"", "", "", (AudioObject *)NULL}
};

void SaveMPEGFile(char* fileName)
{
  char	theCommand[MAXPATHLEN + 64];
  
  sprintf(theCommand, "/usr/video/bin/Video_Convert -sf %s -df %s.mpg &", /* Set up a command to run HP's JPEGMovie-To-MPEG... */
	  fileName, fileName);					    /* ...file conversion program. */
  system(theCommand);						    /* Launch the new process */
  return;
} /* end function SaveMPEGFile */




void ShowCompactMovieDialog(Widget	w,
			    XtPointer	client_data,
			    XtPointer	xt_call_data)
{
  XtVaSetValues(fileDialog,
		XmNtitle, "Compact This Movie",
		NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, SaveMovie, NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, OpenMovie, NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, CompactMovie, NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, SaveStill, NULL);
  XtAddCallback(fileSelectionBox, XmNokCallback, CompactMovie, NULL);
  XtVaSetValues(saveAsOptionButton, XmNsensitive, True, NULL);	    /* Allow the author to choose final movie output format */
  XtManageChild(fileSelectionBox);
  return;
} /* end function ShowCompactMovieDialog */



void CreateJPEGWidget(void)
{
  int	oldImageQualityQFactor;
  int	oldPlaybackQFactor;
  
  if (playbackWidget != (Widget)NULL)				    /* Destroy old movie record & playback widget if it exists */
  {
    XtVaGetValues(playbackWidget,				    /* Save the Q factor currently being used by the playback... */
		  XtNqFactor, &oldPlaybackQFactor, NULL);	    /* ...widget. */
    
    XtDestroyWidget(playbackWidget);
  }
  playbackWidget =
    XtVaCreateManagedWidget("playbackWidget",			    /* Create a widget for showing JPEG movies */
			    jpegWidgetClass,
			    movieShell,				    /* Take zoom factor into account when doing the dimension... */
			    XtNwidth, (movieHeader.width * zoomFactor),	/* ...calculations. */
			    XtNheight, (movieHeader.height * zoomFactor),
			    XtNqFactor, movieHeader.qfactor,
			    XtNdisplayDestRect, &playDestRect,
			    NULL);
  XtVaGetValues(playbackWidget,					    /* Point to the shared memory segment used by this widget */
		XtNjpegData, &jpegImage,
		XtNuseSharedMemory, &usingSharedMemory,		    /* Determine whether the JPEG widget is using shared memory */
		NULL);
  if (usingSharedMemory == False)				    /* If the JPEG widget isn't using shared memory, allocate... */
  {								    /* ...enough memory to store images for playback. */
    jpegImage = XPlxCreateCImage(XtDisplay(playbackWidget),	    /* ...structure (the function will allocate the necessary... */
				 NULL,				    /* ...memory for storing a single JPEG image). */
				 OneImageSizeInBytes,
				 movieHeader.width,
				 movieHeader.height);
    XtVaSetValues(playbackWidget,				    /* Tell the JPEG widget to point to the newly-created... */
		  XtNjpegData, jpegImage,			    /* Parallax JPEG image structure. */
		  NULL);
    jpegImage->width = movieHeader.width;			    /* Set variables for the width and height of the JPEG... */
    jpegImage->height = movieHeader.height;			    /* ...images in the currently open movie. */
  }
  XtUnmanageChild(imageQualityFrame);				    /* Don't allow parent Frame to resize itself to 0x0 size */
  if (imageQualityWidget != (Widget)NULL)			    /* Destroy the old image quality widget if there is one */
  {
    XtVaGetValues(imageQualityWidget,				    /* Save the Q factor currently being used by the image... */
		  XtNqFactor, &oldImageQualityQFactor, NULL);	    /* ...quality widget. */
    XtDestroyWidget(imageQualityWidget);
  }
  imageQualityWidget =
    XtVaCreateManagedWidget("imageQualityWidget",		    /* Create a JPEG widget to show image quality for... */
			    jpegWidgetClass,			    /* ...varying Q factors. */
			    imageQualityFrame,
			    XtNwidth, 160,
			    XtNheight, 120,
			    XtNcompressSourceRect, &imageQualitySourceRect,
			    XtNcompressDestRect, &imageQualitySourceRect,
			    XtNdisplayDestRect, &imageQualityRect,
			    XtNqFactor, movieHeader.qfactor,
			    NULL);
  XtManageChild(imageQualityFrame);				    /* Okay to resize parent now that it has a child */
  return;
} /* end function CreateJPEGWidget */



Boolean OutputWorkProc(XtPointer	clientData)
{
  static Boolean	firstTime = True;
  Widget		w;
  
  w = clientData;
  if (movieStatus == MovieNew)					    /* Don't allow output until a movie has been played */
    return(True);
  outputOn = True;						    /* Note that output is about to be enabled */
  XtVaSetValues(recordButton, XmNsensitive, False, NULL);	    /* Don't allow recording while output is on */
  XtVaSetValues(saveAsMenuButton, XmNsensitive, False, NULL);	    /* Don't allow selection of a new movie file for recording */
  DisableJPEGCompressionControls();
  if (firstTime == True)
  {
    outputGC = XCreateGC(XtDisplay(w), XtWindow(playbackWidget), 0, 0);
    firstTime = False;
  }
  if (PLX_NTSC == outputSignalStandard)				    /* Was NTSC output selected? */
  {								    /* Yes, use NTSC values (This is a hack!  Please remove... */
    outputSignalWidth = 640;					    /* ...it and use the proper API calls to determine the... */
    outputSignalHeight = 480;					    /* ...width & height of the output signal.  Ideally, the... */
  }								    /* ...Output widget should be used for this. */
  else
  {
    outputSignalWidth = 768;
    outputSignalHeight = 575;
  }
  XtDestroyWidget(input1Widget);				    /* Make sure that the Port attached to the input... */
  input1Widget = XtVaCreateManagedWidget("input1Widget",	    /* ...widget is detached by destroying the widget's... */
					 xVideoInWidgetClass,	    /* ...window, then re-create the input widget but don't... */
					 input1Shell,		    /* ...immediately show video on it (showing video would... */
					 XtNchannel, PLX_IO_OFF,    /* ...cause a Port to be attached to the Window). */
					 XtNsignalStandard, inputSignalStandard,
					 XtNsignalFormat, inputSignalFormat,
					 XtNinstance, inputInstance,
					 NULL);
  CreateJPEGWidget();						    /* Create a new widget for playing and recording movies */
  XPlxVideoOutputSelect(XtDisplay(w),				    /* Force JPEG playback to use the 1st hardware port... */
			XtWindow(playbackWidget),		    /* ...(the "v" channel). */
			outputGC,
			PLX_OUTPUT_JPEG,
			outputSignalStandard,
			outputSignalFormat,
			0);
  XPlxVideoScaleOutput(XtDisplay(w),
		       XtWindow(playbackWidget),
		       outputGC,
		       0, 0,
		       movieHeader.width, movieHeader.height,
		       0, 0,
		       outputSignalWidth, outputSignalHeight);
  PlayOneFrame(currentFrame, False, True);			    /* Refresh the playback window after output is ready */
  return(True);
} /* end function OutputWorkProc */



void ResizeOutput(void)
{
  if (outputOn == False)
    return;
  
  XPlxVideoScaleOutput(XtDisplay(playbackWidget),
		       XtWindow(playbackWidget),
		       outputGC,
		       0, 0,
		       movieHeader.width, movieHeader.height,
		       0, 0,
		       outputSignalWidth, outputSignalHeight);
  return;
} /* end function ResizeOutput */



void OutputCallback(Widget	w,
		    XtPointer	client_data,
		    XtPointer	xt_call_data)
{
  XtWorkProcId	outputWorkProcID;
  
  outputWorkProcID = XtAppAddWorkProc(app_context,		    /* Start a work-proc to show output; Xt is... */
				      OutputWorkProc,		    /* ...poorly designed and this is the only way to assure... */
				      (XtPointer)w);		    /* ...that widgets needing to be destroyed right now... */
  return;							    /* ...will happen. */
} /* end function OutputCallback */



Boolean ShowInputWorkProc(XtPointer	clientData)
{
  static Boolean	firstTime = True;
  static GC		gc;
  int			channelToUse;
  signalSpec	theSignal;
  
  if ((XtPointer)0 == clientData)				    /* Was the "first available" special value passed in? */
  {								    /* Yes, do the appropriate thing */
    XtVaSetValues(input1Widget, XtNautoSearch, True, NULL);	    /* Let the widget find the 1st valid input source */
    theSignal.s.channel = PLX_INPUT_0;				    /* Begin the search with the 1st possible input for... */
    theSignal.s.standard = PLX_NTSC;				    /* ...XVideo hardware. */
    theSignal.s.format = PLX_COMP;
    theSignal.s.instance = 0;
  }
  else								    /* No, an explicit source was requested; use the value... */
  {
    theSignal.i = (unsigned int)clientData;			    /* ...passed in. */
    XtVaSetValues(input1Widget, XtNautoSearch, False, NULL);	    /* Force widget to show whatever was chosen, valid or not */
  }
  if (firstTime == True)
  {
    gc = XCreateGC(XtDisplay(input1Widget),
		   XtWindow(playbackWidget), 0, 0);
    firstTime = False;
  }
  if (outputOn == True)						    /* Only disable output if it was on in the 1st place */
  {
    XPlxVideoOutputSelect(XtDisplay(input1Widget),		    /* Disable output of JPEG playback if it was previously on */
			  XtWindow(playbackWidget),
			  gc,
			  PLX_OUTPUT_JPEG_OFF,
			  outputSignalStandard,
			  PLX_COMP,
			  0);
    XPlxVideoOutputSelect(XtDisplay(input1Widget),
			  XtWindow(imageQualityWidget),
			  gc,
			  PLX_OUTPUT_JPEG_OFF,
			  outputSignalStandard,
			  PLX_COMP,
			  0);
    outputOn = False;						    /* Note that output is now disabled */
    XtVaSetValues(recordButton, XmNsensitive, True, NULL);	    /* Allow recording again now that output is off */
    XtVaSetValues(saveAsMenuButton, XmNsensitive, True, NULL);	    /* Don't allow selection of a new movie file for recording */
    EnableJPEGCompressionControls();
    CreateJPEGWidget();						    /* Create a new widget for playing and recording movies */
  } /* end if output is on */
  XtPopup(input1Shell, XtGrabNone);
  SetMTPanelState(&mtPanelState, input1Shell, True);		    /* Indicate that the live video input shell is now visible */
  XtVaSetValues(input1Widget, XtNchannel, PLX_IO_OFF, NULL);	    /* Disable video before changing channels */
  XtVaSetValues(input1Widget,					    /* Attempt to choose the new signal source */
		XtNchannel, theSignal.s.channel,
		XtNsignalStandard, theSignal.s.standard,
		XtNsignalFormat, theSignal.s.format,
		XtNinstance, theSignal.s.instance,
		NULL);
  XtVaGetValues(input1Widget,					    /* Determine the signal standard for this video input, so... */
		XtNsignalStandard, &inputSignalStandard,	    /* ...the frame rate for this standard can be determined. */
		XtNsignalFormat, &inputSignalFormat,
		XtNinstance, &inputInstance,
		XtNsignalWidth, &signalWidth,
		XtNsignalHeight, &signalHeight,
		XtNhardwareParams, &hardwareInfo,
		NULL);
  if (inputSignalStandard == PLX_NTSC)				    /* Is it an NTSC signal? */
  {
    signalFrameRate = 30;					    /* Yep, frame rate is 30 Hz */
    recordFrameRate = 30;					    /* Set an initial rate for recording NTSC movies */
    XtVaSetValues(recordSpeedSlider, XmNvalue, 30, NULL);
  }
  else								    /* Nope, must be PAL or SECAM; frame rate is 25 Hz */
  {
    signalFrameRate = 25;
    recordFrameRate = 25;					    /* Set an initial rate for recording PAL or SECAM movies */
    XtVaSetValues(recordSpeedSlider, XmNvalue, 25, NULL);
  }
  UpdateInputTypeString(&theSignal, sync);			    /* Update the label showing the current signal type */
  return(True);
} /* end function ShowInputWorkProc */



void ShowInput(Widget		w,
	       XtPointer	clientData,
	       XtPointer	unusedCallData)
{
  XtWorkProcId	inputWorkProcID;
  inputWorkProcID = XtAppAddWorkProc(app_context,		    /* Start a work-proc to select a new input source; Xt is... */
				     ShowInputWorkProc,		    /* ...poorly designed and this is the only way to assure... */
				     (XtPointer)clientData);	    /* ...that widgets needing to be destroyed right now... */
  return;							    /* ...searching for alternate input sources. */
} /* end function ShowInput */



void ShowFirstAvailableInput(Widget	w,
			     XtPointer	unusedClientData,
			     XtPointer	unusedCallData)
{
  XtWorkProcId	firstAvailWorkProcID;
  
  XtVaSetValues(input1Widget, XtNautoSearch, True, NULL);	    /* Allow the widget to search for any valid input */
  firstAvailWorkProcID = XtAppAddWorkProc(app_context,		    /* Start a work-proc to select a new input source; Xt is... */
					  ShowInputWorkProc,	    /* ...poorly designed and this is the only way to assure... */
					  (XtPointer)w);	    /* ...that widgets needing to be destroyed right now... */
} /* end function ShowFirstAvailableInput */
