/* $Header: /plx/source/Xclients/generic/RCS/maestro.c,v 1.15 1995/11/23 03:55:26 drapeau Exp $ */
/* $Log: maestro.c,v $
 * Revision 1.15  1995/11/23  03:55:26  drapeau
 * Minor changes to eliminate compiler warnings.
 *
 * Revision 1.14  1995/10/04  17:27:34  lovvik
 * Corrected a potential problem with the speedSlider widget.
 * If the playback speed is zero, the slider widget will complain.
 * I check for this condition, and then set it to 1 (the minimum
 * value of the speedSlider widget).
 *
 * Revision 1.13  1995/10/02  21:55:39  drapeau
 * Implemented the HideApplication and ShowApplication methods correctly.
 * Also fixed minor errors in the SetSelection method.
 *
 * Revision 1.12  1994/08/23  01:34:34  drapeau
 * Added code for an alert dialog box.
 *
 * Revision 1.11  1994/05/31  20:13:12  drapeau
 * Cleaned up diagnostic reporting for this module.
 *
 * Revision 1.10  1994/01/10  22:40:21  drapeau
 * First working version of this module.
 * */
static char maestroRcsId[] = "$Header: /plx/source/Xclients/generic/RCS/maestro.c,v 1.15 1995/11/23 03:55:26 drapeau Exp $";

#include "MovieTool.h"
extern AppResources	mtResources;

void		OpenDoc(char**);
void		SetSelection(MAESelection*);
void		PerformSelection(void*);
char**		GetDoc(void*);
MAESelection*	GetSelection(void*);
void		HaltSelection(void*);
void		PauseSelection(void*);
void		ResumeSelection(void*);
void		HideApplication(void*);
void		ShowApplication(void*);
IconData*	GetAppIcon(void*);

void InitMAEstroMessaging(void)
{
  static Port		senderPort;
  static int		receiverPortNumber;
  static DispatchTable  DT = 
  {
    OpenDoc,
    GetDoc,
    GetSelection,
    SetSelection,
    PerformSelection,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    HaltSelection,
    PauseSelection,
    ResumeSelection,
    HideApplication,
    ShowApplication,
    GetAppIcon,
    };
  
  maestroFD = (XtInputId)NULL;					    /* Initialize the MAEstro file descriptor to be nothing */
  sender = (Sender*)NULL;
  receiver = (Receiver*)NULL;
  senderPort.hostName = malloc(MAXPATHLEN);
  strcpy(senderPort.hostName, mtResources.hostname);
/*  strcpy(senderPort.hostName, "localhost");*/			    /* Set up default values for the PortManager with which... */
  senderPort.portNumber = PortMgrPortNumber;			    /* ...to connect. */
  receiverPortNumber = AnyPort;
  BuildDispatchTable(&DT);					    /* Tell messaging library which MAEstro messages are valid */
  sender = NewSender(&senderPort);				    /* Attempt to create a network connection with the PMgr */
  if (sender == (Sender*)NULL)					    /* Did the attempt succeed? */
  {								    /* No, print an error message and return immediately */
/*
    sprintf(errorMessage, "MovieTool could not create a network connection with the Port Manager.\n");
    PrintErrorMessage(errorMessage);
*/
    return;
  }
  receiver = NewReceiver(sender, "MovieTool", receiverPortNumber);  /* Attempt to register MovieTool with the PortManager */
  if (receiver == (Receiver*)NULL)
  {
/*
    sprintf(errorMessage, "MovieTool could not register with the PortManager.\n");
    PrintErrorMessage(errorMessage);
*/
    return;
  }
  maestroFD = XtAppAddInput(app_context,			    /* Tell Xt to look for input on the Receiver's file... */
			    receiver->receiveSocket,		    /* ...descriptor. */
			    (XtPointer) XtInputReadMask,
			    CheckMAEstroMessages,
			    (XtPointer)NULL);
} /* end function InitMAEstroMessaging */



void	PrintMTDiagnostics(char* theString)
{
  if (printDiagnostics == True)
    printf("%s", theString);
  return;
}								    /* end function PrintMTDiagnostics */



void OpenDoc(char** filename)
{
  sprintf(diagString, "(Right now, MT doesn't handle being interrupted while playing a movie.)\n");
  PrintMTDiagnostics(diagString);
  if ((strcmp(currentMovieFileName, *filename) != 0) &&		    /* Is the movie being requested different from the... */
      (strcmp(*filename, "Untitled") != 0))			    /* ...currently-open movie, and was the filename passed... */
    OpenMovie((Widget)NULL,					    /* ...in valid?  If so, open the movie being requested. */
	      (XtPointer)*filename,
	      (XtPointer)NULL);
  movieToolStatus = Stopped;
  currentFrame = firstFrame;					    /* Go to the beginning of the movie */
  PlayOneFrame(firstFrame, False, True);			    /* Show the 1st frame of this movie but don't play audio */
  return;
} /* end function OpenDoc */



void SetSelection(MAESelection* selection)
{
  sprintf(diagString, "(For now, partial selections are not taken into account.)\n");
  PrintMTDiagnostics(diagString);
  if (selection->start > 0)					    /* Only use the new playback speed if it's valid */
    playbackSpeed = selection->start;
  if(playbackSpeed == 0)
  {
    playbackSpeed = 1;
  }
  XtVaSetValues(speedSlider, XmNvalue, playbackSpeed, NULL);	    /* Update the speed slider to indicate playback speed */

  playbackStartFrame = selection->offset * playbackSpeed / 1000;
  if (playbackStartFrame < firstFrame)				    /* Don't allow MovieTool to begin on an invalid frame */
    playbackStartFrame = firstFrame;
  currentFrame = playbackStartFrame;
  playbackEndFrame = (selection->duration * playbackSpeed / 1000) + playbackStartFrame;
  if (playbackEndFrame > lastFrame)				    /* Don't let MovieTool try to play past the end of the movie */
    playbackEndFrame = lastFrame;
/*
  printf("SetSelection(): start = %d, end = %d, offset = %d, duration = %d.\n",
	 selection->start, selection->end, selection->offset, selection->duration);
  printf("SetSelection(): startFrame is %d, endFrame = %d, # frames in movie = %d.\n",
	 playbackStartFrame, playbackEndFrame, lastFrame - firstFrame);
*/
  return;
} /* end function SetSelection */



void PerformSelection(void* unusedArg)
{
  Play((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);
}								    /* end function PerformSelection */



char** GetDoc(void* unusedArg)
{
  static Boolean	firstTime = True;
  static char*		tempFileName;
  
  if (firstTime == True)					    /* Is this the first time this functions is being called? */
  {
    firstTime = False;						    /* Yes, never let this happen again */
    tempFileName = malloc(MAXPATHLEN);				    /* Allocate space for a filename, assume it will succeed */
  }
  strcpy(tempFileName, currentMovieFileName);			    /* Copy the name of the current movie */
  return(&tempFileName);
} /* end function GetDoc */



MAESelection* GetSelection(void* unusedArg)
{
  static MAESelection	select;
  static int		firstTime = 0;
  
  if (firstTime == 0)						    /* Does space for a label need to be allocated? */
  {								    /* Yes, allocate and initialize that space */
    select.label = malloc(LabelLength);
    memset(select.label, 0, LabelLength);
    firstTime = 1;
  }
  if (strcmp(currentMovieFileName, "Untitled") == 0)		    /* Is a movie open? */
  {								    /* No, return a dummy MAESelection information packet */
    select.start = 0;
    select.end = 0;
    select.offset = 0;
    select.duration = 0;
    sprintf(select.label, "No Label -- No Movie");
    return(&select);
  }
  sprintf(diagString, "Number of frames in this movie: %d.\n",
	  lastFrame - firstFrame);
  PrintMTDiagnostics(diagString);
  select.start = playbackSpeed;
  select.end = 0;
  select.offset = 0;
  select.duration = (1000 / playbackSpeed) *			    /* Calculate time in msecs to play one frame, multiply by... */
    (playbackEndFrame - playbackStartFrame);			    /* ...number of frames to perform. */
  sprintf(diagString, "GetSelection(): select.duration is %d.\n",
	  select.duration);
  PrintMTDiagnostics(diagString);
  sprintf(select.label, "No Label");				    /* No label info is useful as of this version of MovieTool */
  return (&select);
} /* end function GetSelection */



void HaltSelection(void* unusedArg)
{
  sprintf(diagString, "\n\n--------Entering HaltSelection.\n");
  PrintMTDiagnostics(diagString);
  Stop((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);		    /* Stop playing this movie immediately */
  return;
} /* end function HaltSelection */



void PauseSelection(void* unusedArg)
{
  sprintf(diagString, "\n\n--------Entering PauseSelection.\n");
  PrintMTDiagnostics(diagString);
  Pause((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);
  return;
} /* end function PauseSelection */



void ResumeSelection(void* unusedArg)
{
  sprintf(diagString, "\n\n--------Entering ResumeSelection.\n");
  PrintMTDiagnostics(diagString);
  Play((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);
  return;
} /* end function ResumeSelection */



void HideApplication(void* unusedArg)
{
  sprintf(diagString, "\n\n--------Entering HideApplication.\n");
  PrintMTDiagnostics(diagString);
  sprintf(diagString, "(not yet implemented.)\n");
  PrintMTDiagnostics(diagString);
  XtUnmapWidget(mainWindow);
  XtUnmapWidget(input1Shell);
  if (movieToolStatus != Playing)				    /* Don't hide the movie shell if playback is in progress */
  {
    XtUnmapWidget(movieShell);
  }
  return;
} /* end function HideApplication */



void ShowApplication(void* unusedArg)
{
  sprintf(diagString, "\n\n--------Entering ShowApplication.\n");
  PrintMTDiagnostics(diagString);
  sprintf(diagString, "(not yet implemented.)\n");
  PrintMTDiagnostics(diagString);
  XtMapWidget(mainWindow);
  if (True == mtPanelState.audioShell)				    /* Show the appropriate shells and/or dialogs */
  {
    XtMapWidget(audioShell);
  }
  if (True == mtPanelState.inputSettingsShell)
  {
    XtMapWidget(inputSettingsShell);
  }
  if (True == mtPanelState.input1Shell)
  {
    XtMapWidget(input1Shell);
  }
  if (True == mtPanelState.fileDialog)
  {
    XtMapWidget(fileDialog);
  }
  if (True == mtPanelState.smartMovieShell)
  {
    XtMapWidget(smartMovieShell);
  }
  if (True == mtPanelState.movieShell)
  {
    XtMapWidget(movieShell);
  }
  if (True == mtPanelState.alertShell)
  {
    XtMapWidget(alertShell);
  }
  if (True == mtPanelState.aboutShell)
  {
    XtManageChild(aboutDialog);
    XtMapWidget(aboutShell);
  }
  return;
} /* end function ShowApplication */



IconData* GetAppIcon(void* unusedArg)
{
  static int		first = True;
  static IconData  	iconData;
  static unsigned short iconBits[] = {
#include "movietool.icon"
  };
  if (first)
    {
      iconData.iconData = (char *)malloc(sizeof(iconBits));
      memcpy(iconData.iconData, iconBits, sizeof(iconBits));
      if (iconData.iconData)
	iconData.dataLength = sizeof(iconBits);
      first = False;
    }
  return(&iconData);
}								    /* end function GetAppIcon */



void CheckMAEstroMessages(XtPointer	clientData,
			  int*		sourceFD,
			  XtInputId*	inputID)
{
  ReceiverListenForMessages();					    /* Check for incoming MAEstro messagesy */
} /* end function CheckMAEstroMessages */
