/* $Header: /plx/source/Xclients/sun4.x/Solaris1/src/client/motif/MovieTool/RCS/sunAudio.c,v 1.58 1996/01/26 00:31:31 drapeau Exp $ */
/* $Log: sunAudio.c,v $
 * Revision 1.58  1996/01/26  00:31:31  drapeau
 * Conditionally compile support for setting the Sun audio device's
 * record buffer size so that it is only tried on the Solaris 2.X
 * platform.  The SunOS 4.X platform does not support the "buffer_size"
 * field in the record buffer.
 *
 * Revision 1.57  1996/01/24  00:24:20  lovvik
 *   Added code to SunAudioDeviceInitializeAudio() that explicitly
 *   sets the buffer size for recording.  This fixes a problem when
 *   recording at low frame rates (< 10).
 *
 * Revision 1.56  1995/12/09  00:39:56  lovvik
 * Fixed 3 problems:
 *   1.  Recording at lower audio sample rates caused
 *       audio read problems.  I believe that this is
 *       a bug on the Sun side.  As a workaround, I
 *       perform the read up to 5 times (waiting 1
 *       ms between tries).  If this fails, the read
 *       returns an error.
 *   2.  When MovieTool would start, it would sometimes
 *       indicate that Line In was the input source
 *       selected, but the microphone would be active.
 *       I removed a line of code that set up the
 *       microphone as the default input source that
 *       occured after the audio device query.
 *   3.  When the audio device is initialized to a
 *       sample rate that is not evenly divisible by the
 *       record frame rate, the record frame rate wouldn't
 *       change.  This was fixed by a call to
 *       ChangeRecordSpeed() after the audio device was
 *       initialized.
 *
 * Revision 1.55  1995/11/23  03:55:59  drapeau
 * Minor changes to eliminate unused variables and compiler warnings.
 *
 * Revision 1.54  1995/11/17  21:43:53  drapeau
 * Fixed SunAudioDeviceInitializeAudio(): the function now correctly
 * reports the input and output ports in use.
 *
 * Revision 1.53  1995/08/31  07:34:06  drapeau
 * Added not-implemented functions for two new methods for the audio
 * object.  These two methods are currently used only for the AIX UMS
 * audio driver.
 *
 * Revision 1.52  1995/06/09  18:35:08  drapeau
 * Added minor change to the SunAudioDeviceSyncPlaybackDevice() function
 * to compensate for a change in the Play() function in the callbacks.c
 * module.  The Play() function no longer pauses and resumes the audio
 * device; it now only calls the Sync() method for the audio "object",
 * thus the Sync() object now needs to call its Pause() and Resume()
 * methods itself to assure that audio is flushed from the device's
 * buffer.
 *
 * Revision 1.51  1995/05/25  23:40:00  drapeau
 * Fixed minor typographical error in
 * SunAudioDeviceSetAudioPlaybackConfiguration().
 *
 * Revision 1.50  1995/05/23  19:34:22  lovvik
 * Removed assignments to the global variable audioPlayHeader
 * within InitializeAudio().  This allows the application to
 * retain current audio settings while initializing the audio
 * device.
 *
 * Revision 1.49  1995/04/28  21:35:51  lovvik
 * Changed SunAudioDeviceAudioRead so it will return a -1 if an error
 * occurs during the read operation.  This allows the application
 * to try to recover.
 *
 * Revision 1.48  1995/04/27  00:37:49  lovvik
 * Added an audio object called defaultAudioObject that contains pointers
 * to all of the audio functions in this module.  I prefixed each
 * function name with SunAudioDevice.  Added support for non-blocking
 * I/O.  This fixes an audio locking problem on the Sparc-5.
 *
 * Revision 1.47  1995/02/13  19:48:32  lovvik
 * Modified this file to take care of the audio file
 * descriptors.  Global audio file descriptors are no
 * longer needed.  Also added two functions (AudioWrite()
 * and AudioRead()) so the main application doesn't need
 * to talk directly to the audio device.
 *
 * Revision 1.46  1995/02/13  19:44:29  drapeau
 * Made more changes to support SPARC-5 audio, especially for Solaris
 * 2.X.
 *
 * Revision 1.45  1995/02/07  22:47:33  lovvik
 * Added support for Sparc 5 Audio.  This was a simple change to
 * PrintAudioInfo() and InitializeAudio().
 *
 * Revision 1.44  1994/09/27  20:56:27  drapeau
 * Added a function, SyncAudioRecordDevice(), which for the Sun platform
 * does nothing at present.
 *
 * Revision 1.43  1994/09/14  03:23:19  drapeau
 * Enabled audio output on line out and headphones as well as speakers.
 *
 * Revision 1.42  1994/09/13  08:10:27  drapeau
 * Check for internal high-quality audio as well as speakerbox
 * high-quality audio.
 *
 * Revision 1.41  1994/08/24  01:53:56  drapeau
 * Slight modification to PrintAudioInfo(), to get information about the
 * current monitor volume.
 *
 * Revision 1.40  1994/08/23  05:07:35  drapeau
 * Made changes to the record side of the device controller, trying to
 * obey SPARC audio device rules better and therefore eliminate popping
 * noises during initial recording.
 *
 * Revision 1.39  1994/08/23  01:35:08  drapeau
 * Added code for an alert dialog box.
 *
 * Revision 1.38  1994/08/19  06:05:28  drapeau
 * Removed extraneous printf stmts, and made a small modification to
 * account for Sun audio on Solaris 2.3 (which includes an extra data
 * field for byte ordering).
 *
 * Revision 1.37  1994/08/08  01:04:05  drapeau
 * Two changes:
 *
 * 1) Added a function, SyncAudioPlaybackDevice(), to test
 *    synchronization b/w audio and video.  So far, it is inoperative.
 *
 * 2) Added proper support for high-quality audio on the Solaris 2.X
 *    platform.
 *
 * Revision 1.36  1994/06/27  22:57:27  drapeau
 * Removed an extraneous diagnostic message that was used for debugging.
 *
 * Revision 1.35  1994/06/24  01:16:47  drapeau
 * Added the function AudioClose() to properly close the audio device.
 *
 * Revision 1.34  1994/01/20  22:50:10  drapeau
 * Made implementation more portable, in support of the PortableAudio
 * interface.  Also had to modify the PortableAudio interface to support
 * what I consider to be flaws in the Sun audio device driver: it appears
 * that you cannot have a single file descriptor to be used for both
 * recording and playback of audio.
 *
 * Revision 1.33  1994/01/20  18:02:00  drapeau
 * Several major changes, to support audio under both SunOS 4.X and
 * Solaris 2.X, and to support the new PortableAudio structure in order
 * that MovieTool is more portable across platforms.
 *
 * Revision 1.32  1994/01/10  22:34:31  drapeau
 * Added support for the Sun speakerbox (high-quality audio).
 * Also, cleaned up diagnostics reporting.
 *
 * Revision 1.31  1993/12/03  02:09:52  drapeau
 * Minor change made; mostly it was done to allow me to change the RCS
 * numbering scheme for this file.
 *
 * Revision 1.3  1993/12/03  01:31:44  drapeau
 * Added functions to control audio volume;  also made the code more
 * robust in general.
 *
 * Revision 1.2  1993/11/01  17:08:07  drapeau
 * No functional changes; just using a newer RCS version.
 *
 * Revision 1.1  93/09/09  16:09:56  drapeau
 * Initial revision
 *  */
static char	audioFunctionsRcsId[] = "$Header: /plx/source/Xclients/sun4.x/Solaris1/src/client/motif/MovieTool/RCS/sunAudio.c,v 1.58 1996/01/26 00:31:31 drapeau Exp $";

#include <stdio.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <stropts.h>
#include "MovieTool.h"
#include "PortableAudio.h"
#include "sunAudio.h"

#define MAX_AUDIO_ERROR_COUNT 50

static int   audioFD;

AudioObject  defaultAudioObject =
{
  SunAudioDeviceAudioWrite,
  SunAudioDeviceAudioRead,
  SunAudioDeviceAudioPlayConnect,
  SunAudioDeviceAudioPlayDisconnect,
  SunAudioDeviceAudioRecordConnect,
  SunAudioDeviceAudioRecordDisconnect,
  SunAudioDeviceAudioRecordPause,
  SunAudioDeviceAudioRecordResume,
  SunAudioDeviceAudioPlayPause,
  SunAudioDeviceAudioPlayResume,
  SunAudioDevicePrintAudioInfo,
  SunAudioDeviceInitializeAudio,
  SunAudioDeviceSetAudioPlayVolume,
  SunAudioDeviceSetAudioRecordVolume,
  SunAudioDeviceSetAudioMonitorVolume,
  SunAudioDeviceSetPlaybackDevice,
  SunAudioDeviceSetRecordDevice,
  SunAudioDeviceSetAudioPlaybackConfiguration,
  SunAudioDeviceAudioClose,
  SunAudioDeviceSyncAudioPlaybackDevice,
  SunAudioDeviceSyncAudioRecordDevice,
  SunAudioDeviceAudioStopPlayDevice,
  SunAudioDeviceAudioStopRecordDevice,
  SunAudioDeviceAudioReadOverrun,
  SunAudioDeviceAudioBytesInBuffer
};



int SunAudioDeviceAudioWrite(char *audioData, int audioslice)
{
  long                  byteCount = 0;
  long                  bytesWritten = 0;
  int                   audioBufferPointer = 0;

  while(byteCount < audioslice)
  {
    bytesWritten = write(audioFD, (audioData + audioBufferPointer), audioslice - audioBufferPointer);
    if(bytesWritten == -1)
    { 						              /* write error - maybe audio device is not draining */
      ioctl(audioFD, I_FLUSH, FLUSHRW);		       	      /* Clear out all audio data */	
    }
    else
    {
      byteCount += bytesWritten;
      audioBufferPointer += (int)bytesWritten;
    }
  }
  return(byteCount);                                               /* Return the number of bytes written */
}  /* end function AudioWrite */

int SunAudioDeviceAudioRead(char *audioData, int audioslice)
{
  long                  byteCount = 0;
  long                  bytesRead = 0;
  int			errorCount = 0;

  int audioBufferPointer = (int)NULL;

  /*
   * Starting with Solaris 2.5, we encountered a problem with low
   * quality audio.  The problem is that an audio read would return
   * a negative number, and errno would be set to EAGAIN (Resource
   * temporarily unavailable).  This seems to be a problem on the
   * Sun side.  The fix for the client is to keep asking for the
   * audio data until it returns successfully.  I have a definition
   * called MAX_AUDIO_ERROR_COUNT (currently set to 5) that indicates
   * the maximum number of errors to ignore.  If the audio read
   * call fails more than MAX_AUDIO_ERROR_COUNT times, this function
   * will return a failure.
   */
  while(byteCount < audioslice && errorCount < MAX_AUDIO_ERROR_COUNT)
  {
    bytesRead = read(audioFD, (audioData + audioBufferPointer), audioslice - audioBufferPointer);
    if(bytesRead >= 0)                    				/* Did an error occur? */
    {									/* No */
      byteCount += bytesRead;
      audioBufferPointer += (int) bytesRead;
    }
    else
    {
      /*
       * Sleep for 1 ms.
       */
      XPlxSleep(1000);
      errorCount++;
    }
  }
  if(errorCount >= MAX_AUDIO_ERROR_COUNT)
  {
    return(-1);
  }
  else
  {
    return(byteCount);
  }
}  /* end function AudioRead */

void  SunAudioDeviceAudioPlayConnect(void)
{

}

void SunAudioDeviceAudioPlayDisconnect(void)
{

}

void SunAudioDeviceAudioRecordConnect(void)
{

}

void SunAudioDeviceAudioRecordDisconnect(void)
{

}

void SunAudioDeviceAudioRecordPause(void)
{
  audio_pause_record(audioFD);					    /* Stop recording data now */
  ioctl(audioFD, I_FLUSH, FLUSHRW);				    /* Clear out all audio data */
} /* end function AudioRecordPause */



void SunAudioDeviceAudioRecordResume(void)
{
  audio_resume_record(audioFD);

} /* end function AudioRecordResume */


void SunAudioDeviceAudioPlayPause(void)
{
  audio_pause_play(audioFD);
  ioctl(audioFD, I_FLUSH, FLUSHRW);				    /* Clear out all audio data */
} /* end function AudioPlayPause */



void SunAudioDeviceAudioPlayResume(void)
{
  audio_resume_play(audioFD);
} /* end function AudioPlayResume */



void SunAudioDevicePrintAudioInfo(PortableAudio*	audioInfo)
{
  int			result;
  Audio_info		tempAudioInfo;
#ifdef SVR4
  audio_device_t	deviceType;
#else  
  int			deviceType;
#endif
  
  result = ioctl(audioFD, AUDIO_GETINFO, &tempAudioInfo);	    /* Try to get info about the audio device */
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    switch (errno)
    {
     case EBUSY:
      sprintf(errorMessage, "The audio device is busy.  Check if other applications are currently using the audio device.\n");
      PrintErrorMessage(errorMessage);
      break;
    }
    return;
  }
  result = ioctl(audioFD, AUDIO_GETDEV, &deviceType);
  audioInfo->play.sampleRate = tempAudioInfo.play.sample_rate;	    /* Fill in the PortableAudio data structure containing... */
  audioInfo->play.channels = tempAudioInfo.play.channels;	    /* ...device-specific information about this audio device. */
  audioInfo->play.bitsPerSample = tempAudioInfo.play.precision;
  audioInfo->play.volume = tempAudioInfo.play.gain;
  switch (tempAudioInfo.play.encoding)
  {
   case AUDIO_ENCODING_ULAW:
    audioInfo->play.encoding = AudioULawEncoding;
    PrintDiagnostics("Playback Encoding: uLaw.\n");
    break;
   case AUDIO_ENCODING_ALAW:
    audioInfo->play.encoding = AudioALawEncoding;
    PrintDiagnostics("Playback Encoding: aLaw.\n");
    break;
   case AUDIO_ENCODING_LINEAR:
    PrintDiagnostics("Playback Encoding: Linear.\n");
    audioInfo->play.encoding = AudioLinearEncoding;
    break;
  }
  audioInfo->record.sampleRate = tempAudioInfo.record.sample_rate;
  audioInfo->record.channels = tempAudioInfo.record.channels;
  audioInfo->record.bitsPerSample = tempAudioInfo.record.precision;
  audioInfo->record.volume = tempAudioInfo.record.gain;
  audioInfo->monitorVolume = tempAudioInfo.monitor_gain;
  switch (tempAudioInfo.record.encoding)
  {
   case AUDIO_ENCODING_ULAW:
    audioInfo->record.encoding = AudioULawEncoding;
    PrintDiagnostics("Record Encoding: uLaw.\n");
    break;
   case AUDIO_ENCODING_ALAW:
    audioInfo->record.encoding = AudioALawEncoding;
    PrintDiagnostics("Record Encoding: aLaw.\n");
    break;
   case AUDIO_ENCODING_LINEAR:
    audioInfo->record.encoding = AudioLinearEncoding;
    PrintDiagnostics("Record Encoding: Linear.\n");
    break;
  }
  if (printDiagnostics == True)					    /* Should diagnostic information actually be printed? */
  {								    /* Yes, tell the author about the device characteristics */
    printf("Audio information:\n");
    printf("Sample rate: %d.\n", audioInfo->play.sampleRate);
    printf("Number of channels: %d.\n", audioInfo->play.channels);
    printf("Bits per sample: %d.\n", audioInfo->play.bitsPerSample);
    printf("Playback volume: %d.\n", audioInfo->play.volume);
    printf("Record volume: %d.\n", audioInfo->record.volume);
    printf("Monitor volume: %d.\n", audioInfo->monitorVolume);
#ifdef SVR4
    printf("Solaris audio: device name is %s, version is %s, config is %s.\n",
	   deviceType.name, deviceType.version, deviceType.config);
    if ((strcmp(deviceType.name, "SUNW,dbri") == 0) ||		    /* Solaris 2.X way of telling me it's high-quality audio */
	(strcmp(deviceType.name, "SUNW,CS4231") == 0))
#else
    if ((deviceType == AUDIO_DEV_SPEAKERBOX) ||			    /* SunOS 4.X way of telling me it's high-quality audio */
	(deviceType == 5) ||                                        /* Sparc 5 support */
	(deviceType == AUDIO_DEV_CODEC))
#endif
      printf("Device type: High quality audio.\n");
    else
      printf("Device type: Low quality audio.\n");
  } /* end if printDiagnostics... */
  return;
} /* end function PrintAudioInfo */



int SunAudioDeviceInitializeAudio(void)  /* returns status:  -1 = fail, 0 = ok */
{
  Audio_info		tempAudioInfo;
  int			result;
#ifdef SVR4
  audio_device_t	deviceType;
#else  
  int			deviceType;
#endif
  XmScaleCallbackStruct scaleCallbackStruct;
  
  
  audioFD = open("/dev/audio", (O_RDWR | O_NONBLOCK));			    /* Try to open the audio control device for record & play */
  if (audioFD == -1)     					    /* Did the open fail? */
  {								    /* Yes, report the error and exit immediately */
    sprintf(errorMessage, "Sorry, could not open the audio device for record and play.\n");
    PrintErrorMessage(errorMessage);
    return(-1);					                    /* Return fail status */
  }
  result = ioctl(audioFD, AUDIO_GETDEV, &deviceType);
#ifdef SVR4
  if ((strcmp(deviceType.name, "SUNW,dbri") == 0) ||		    /* Solaris 2.X way of telling me it's high-quality audio */
      (strcmp(deviceType.name, "SUNW,CS4231") == 0))
#else
  if ((deviceType == AUDIO_DEV_SPEAKERBOX) ||			    /* SunOS 4.X way of telling me it's high-quality audio */
      (deviceType == AUDIO_DEV_CODEC) ||
      (deviceType == 5))
#endif
  {
    audioRecordHeader.sample_rate = sampleRate;
    audioRecordHeader.samples_per_unit = 1;
    audioRecordHeader.bytes_per_unit = 2;
    audioRecordHeader.channels = numAudioChannels;
    audioRecordHeader.encoding = AUDIO_ENCODING_LINEAR;
    audioRecordHeader.data_size = sampleRate * 2;
    AudioRecordPause();					    /* Clear out the record buffer and pause audio recording... */
    result = audio_set_record_config(audioFD, &audioRecordHeader);  /* ...before changing record parameters. */
  }
#ifdef SVR4
  if (strcmp(deviceType.name, "SUNW,am79c30") == 0)		    /* Solaris 2.X way of telling me it's low-quality audio */
#else
  if (deviceType == AUDIO_DEV_AMD)				    /* SunOS 4.X way of telling me it's low-quality audio */
#endif
  {
    audioRecordHeader.sample_rate = 8000;
    audioRecordHeader.samples_per_unit = 1;
    audioRecordHeader.bytes_per_unit = 1;
    audioRecordHeader.channels = 1;
    audioRecordHeader.encoding = AUDIO_ENCODING_ULAW;
    audioRecordHeader.data_size = 8000;
    result = audio_set_record_config(audioFD, &audioRecordHeader);
  }
  result = ioctl(audioFD, AUDIO_GETINFO, &tempAudioInfo);	    /* Try to get info about the audio device */
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    switch (errno)
    {
     case EBUSY:
      sprintf(errorMessage, "The audio device is busy.  Check if other applications are currently using the audio device.\n");
      PrintErrorMessage(errorMessage);
      break;
    }
    return(-1);
  }
#ifdef SVR4
  tempAudioInfo.record.buffer_size = audioRecordHeader.sample_rate *
    audioRecordHeader.bytes_per_unit * audioRecordHeader.channels * 2;
  result = ioctl(audioFD, AUDIO_SETINFO, &tempAudioInfo);
#endif
  audioInfo.play.sampleRate =	 	tempAudioInfo.play.sample_rate;	/* Fill in PortableAudio data structure containing... */
  audioInfo.play.channels = 		tempAudioInfo.play.channels; /* ...device-specific information about this audio device. */
  audioInfo.play.bitsPerSample =	tempAudioInfo.play.precision;
  audioInfo.play.encoding = 		tempAudioInfo.play.encoding;
  audioInfo.play.volume = 		tempAudioInfo.play.gain;
  audioInfo.record.sampleRate = 	tempAudioInfo.record.sample_rate;
  audioInfo.record.channels = 		tempAudioInfo.record.channels;
  audioInfo.record.bitsPerSample = 	tempAudioInfo.record.precision;
  audioInfo.record.encoding = 		tempAudioInfo.record.encoding;
  audioInfo.record.volume = 		tempAudioInfo.record.gain;
  PrintAudioInfo(&audioInfo);				    /* Fill in information about the audio device */
  XtVaSetValues(playbackScale,
		XmNvalue,
		(audioInfo.play.volume * 100) / AUDIO_MAX_GAIN,
		NULL);
  XtVaSetValues(recordScale,
		XmNvalue,
		(audioInfo.record.volume * 100) / AUDIO_MAX_GAIN,
		NULL);
  XtVaSetValues(monitorScale,
		XmNvalue,
		(audioInfo.monitorVolume * 100) / AUDIO_MAX_GAIN,
		NULL);
  if ((tempAudioInfo.play.port & AUDIO_SPEAKER) != 0)		    /* Is the speaker enabled? */
  {								    /* Yes, set the correct toggles */
    XmToggleButtonSetState(playbackSpeakerToggle, True, False);
    XmToggleButtonSetState(playbackHeadphonesToggle, False, False);
    XmToggleButtonSetState(playbackLineOutToggle, False, False);
  }
  if ((tempAudioInfo.play.port & AUDIO_HEADPHONE) != 0)		    /* Is the Headphone jack enabled? */
  {								    /* Yes, set the correct toggles */
    XmToggleButtonSetState(playbackSpeakerToggle, False, False);
    XmToggleButtonSetState(playbackHeadphonesToggle, True, False);
    XmToggleButtonSetState(playbackLineOutToggle, False, False);
  }
  if ((tempAudioInfo.play.port & AUDIO_LINE_OUT) != 0)		    /* Is the Line Out enabled? */
  {								    /* Yes, set the correct toggles */
    XmToggleButtonSetState(playbackSpeakerToggle, False, False);
    XmToggleButtonSetState(playbackHeadphonesToggle, False, False);
    XmToggleButtonSetState(playbackLineOutToggle, True, False);
  }

  if ((tempAudioInfo.record.port & AUDIO_MICROPHONE) != 0)	    /* Is the mic input enabled? */
  {								    /* Yes, set the correct toggles */
    XmToggleButtonSetState(recordMicInToggle, True, False);
    XmToggleButtonSetState(recordLineInToggle, False, False);
  }
  if ((tempAudioInfo.record.port & AUDIO_LINE_IN) != 0)		    /* Is the Line In input enabled? */
  {								    /* Yes, set the correct toggles */
    XmToggleButtonSetState(recordMicInToggle, False, False);
    XmToggleButtonSetState(recordLineInToggle, True, False);
  }

  scaleCallbackStruct.value = recordFrameRate;
  ChangeRecordSpeed(recordSpeedSlider, (XtPointer)NULL, (XtPointer)&scaleCallbackStruct);
  return(0);
} /* end function SunAudioDeviceInitializeAudio */



void SunAudioDeviceSetAudioPlayVolume(int	volume)
{
  double	deviceVolume;
  
  deviceVolume = volume / 100.0;
  audio_set_play_gain(audioFD, &deviceVolume);
  return;
} /* end function SetAudioPlayVolume */



void SunAudioDeviceSetAudioRecordVolume(int	volume)
{
  double	deviceVolume;
  
  deviceVolume = volume / 100.0;
  audio_set_record_gain(audioFD, &deviceVolume);
  return;
} /* end function SetAudioRecordVolume */



void SunAudioDeviceSetAudioMonitorVolume(int	volume)
{
  double	deviceVolume;
  
  deviceVolume = volume / 100.0;
  audio_set_monitor_gain(audioFD, &deviceVolume);
  return;
} /* end function SetAudioMonitorVolume */



void SunAudioDeviceSetPlaybackDevice(int deviceType)
{
  int	localDeviceType;
  
  if (deviceType == AudioSpeaker)
    localDeviceType = AUDIO_SPEAKER;
  if (deviceType == AudioLineOut)
    localDeviceType = AUDIO_LINE_OUT;
  if (deviceType == AudioHeadphone)
    localDeviceType = AUDIO_HEADPHONE;
  audio_set_play_port(audioFD, &localDeviceType);
} /* end function SetPlaybackDevice */




void SunAudioDeviceSetRecordDevice(int deviceType)
{
  int	localDeviceType;
  
  if (deviceType == AudioMicrophone)
    localDeviceType = AUDIO_MICROPHONE;
  else
    localDeviceType = AUDIO_LINE_IN;
  AudioRecordPause();					    /* Pause the record device & clear out old data before... */
  audio_set_record_port(audioFD, &localDeviceType);		    /* ...changing the audio device parameters. */
} /* end function SetRecordDevice */



int SunAudioDeviceSetAudioPlaybackConfiguration(Audio_hdr*	audioInfo)
{
  int		audioResult;
  Audio_hdr	tempInfo;
  
  if(audioInfo->encoding == 0 &&
     audioInfo->sample_rate == 0 &&
     audioInfo->channels == 0)
  {
    return(0);
  }
  tempInfo.sample_rate = audioInfo->sample_rate;
  tempInfo.samples_per_unit = audioInfo->samples_per_unit;
  tempInfo.bytes_per_unit = audioInfo->bytes_per_unit;
  tempInfo.channels = audioInfo->channels;
  tempInfo.data_size = audioInfo->data_size;

#ifdef SVR4
  tempInfo.endian = audioInfo->endian;
#endif  
  switch (audioInfo->encoding)					    /* Translate the PortableAudio encoding constant into the... */
  {								    /* ...appropriate Sun-specific constant. */
   case AudioULawEncoding:
    tempInfo.encoding = AUDIO_ENCODING_ULAW;
    break;
   case AudioALawEncoding:
    tempInfo.encoding = AUDIO_ENCODING_ALAW;
    break;
   case AudioLinearEncoding:
    tempInfo.encoding = AUDIO_ENCODING_LINEAR;
    break;
  }
  audioResult = audio_set_play_config(audioFD,  		    /* Try to correctly configure the audio device */
				      &tempInfo);
  if (audioResult == AUDIO_SUCCESS)				    /* Did the attempt succeed? */
    return(0);							    /* Yes, return zero */
  else								    /* Nope, return an error code */
    return(-1);
} /* end function SetAudioPlaybackConfiguration */



void SunAudioDeviceAudioClose(void)
{
  ioctl(audioFD, I_FLUSH, FLUSHRW);				    /* Clear out all audio data */
  close(audioFD);						    /* Close the audio device properly */
  return;
} /* end function AudioClose */



void SunAudioDeviceSyncAudioPlaybackDevice(void)
{
  struct timeval	after;
  struct timeval	before;
  int			bytesWritten;
  int			counter = 0;
  int			elapsedTime = 0;
  static char*		syncBuffer = NULL;
  
  SunAudioDeviceAudioPlayPause();				    /* Make sure the audio device play buffer is cleared out,... */
  SunAudioDeviceAudioPlayResume();				    /* ...and tell the audio device to prepare for playback. */
  return;
  if (syncBuffer == NULL)					    /* Allocate some memory for holding audio to be pushed... */
  {								    /* ...through the audio device. */
    syncBuffer = malloc(48000);					    /* I'm guessing that 48K is the max. sample rate and will... */
    if (syncBuffer == NULL)					    /* ...be an upper-bound on sound size used. */
    {
      sprintf(errorMessage, "Could not allocate enough audio memory to properly sync audio.\n");
      PrintErrorMessage(errorMessage);
      return;
    }
    memset(syncBuffer, '\0', 48000);				    /* Fill the audio buffer with zeros */
  }
  while (elapsedTime < (1000 / playbackSpeed))			    /* Keep pushing audio until time to play 1 frame is close... */
  {								    /* ...to real-time */
    if (printDiagnostics == True)
    {
      printf("Playing %d bytes of audio (iteration %d).\n",
	     movieHeader.audioslice, counter++);
    }
    gettimeofday(&before, (struct timezone*)NULL);
    bytesWritten = write(audioFD, syncBuffer,			    /* Playback 1 frame's worth of audio */
			 movieHeader.audioslice);
    gettimeofday(&after, (struct timezone*)NULL);
    if ((bytesWritten < 0) && (printDiagnostics == True))
    {
      printf("SyncAudioPlaybackDevice(): Error %d (errno is %d) writing audio.\n",
	     bytesWritten, errno);
    }
    elapsedTime = ElapsedTime(&before, &after);			    /* Count time in milliseconds to play 1 frame of audio */
  }
  return;
} /* end function SyncAudioPlaybackDevice */



void SunAudioDeviceSyncAudioRecordDevice(void)
{
  return;							    /* Currently no need to sync the Sun audio device for record */
} /* end function SyncAudioRecordDevice */


void  SunAudioDeviceAudioStopPlayDevice(void)
{

}

void SunAudioDeviceAudioStopRecordDevice(void)
{

}


int SunAudioDeviceAudioRecordOpen(int audioFD)
{
  int			result;
  Audio_info		audioInfo;
#ifdef SVR4
  audio_device_t	deviceType;
#else  
  int			deviceType;
#endif
  
  result = ioctl(audioFD, AUDIO_GETDEV, &deviceType);		    /* Determine which type of audio device is connected */
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return an error code */
    sprintf(errorMessage,
	    "Could not get information about the audio device for recording.\n");
    PrintErrorMessage(errorMessage);
    return(-1);
  }
#ifdef SVR4
  if (strcmp(deviceType.name, "SUNW,am79c30") == 0)		    /* Solaris 2.X way of telling me it's low-quality audio */
#else
  if (deviceType == AUDIO_DEV_AMD)				    /* SunOS 4.X way of telling me it's low-quality audio */
#endif
    audioInfo.record.port = AUDIO_MICROPHONE;			    /* Yes, set up for low-quality audio */
  else								    /* No, use the highest available quality audio */
    audioInfo.record.port = AUDIO_LINE_IN;
  result = ioctl(audioFD, AUDIO_GETINFO, &audioInfo);		    /* Try to determine current state of the audio device */
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    sprintf(errorMessage,
	    "AudioRecordOpen could not determine state of the the audio device for recording.\n");
    PrintErrorMessage(errorMessage);
    return(-1);
  }
#ifdef SVR4
  audioInfo.record.buffer_size = audioInfo.record.sample_rate * (audioInfo.record.precision/8) * 
      audioInfo.record.channels * 2;
#endif
  result = ioctl(audioFD, AUDIO_SETINFO, &audioInfo);		    /* Try to reconfigure audio device to the app's preferences */
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    sprintf(errorMessage, "AudioRecordOpen: could not reconfigure the audio device for recording.\n");
    PrintErrorMessage(errorMessage);
    return(-1);
  }
  return(audioFD);						    /* Return a valid file descriptor for the audio device */
} /* end function AudioRecordOpen */





int SunAudioDeviceAudioPlaybackOpen(int audioFD)
{
  return(audioFD);
} /* end function AudioPlaybackOpen */



Boolean SunAudioDeviceAudioReadOverrun(void)
{
  return(False);
} /* end function AudioReadOverrun */



long SunAudioDeviceAudioBytesInBuffer(void)
{
  return(0L);
} /* end function SunAudioDeviceAudioBytesInBuffer */
