/* $Header: /plx/source/Xclients/generic/RCS/sunSpecific.c,v 0.103 1996/01/25 04:18:51 drapeau Exp $ */
/*
 *   Copyright (c) 1994 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Log: sunSpecific.c,v $
 * Revision 0.103  1996/01/25  04:18:51  drapeau
 * Added support for the PlxOutputCascade to automatically create the
 * output menu hierarchy, for enhanced flexibility.
 *
 * Revision 0.102  1996/01/05  23:37:11  drapeau
 * Minor addition to ShowCompactMovieDialog() to account for new
 * interface feature that should be greyed out when this function is
 * called.
 *
 * Revision 0.101  1995/12/09  00:29:43  lovvik
 * Fixed 3 problems:
 *   1.  Fixed the label on the main MovieTool panel
 *       that indicates the source and standard of
 *       the video signal.  The structure called
 *       "theSignal" in ShowInputWorkProc() did
 *       not contain the current values returned
 *       from the input1Widget query.
 *   2.  Output did not turn off when video input
 *       was selected.  I added a code section
 *       in ShowInputWorkProc() that turns of
 *       output if it is on.
 *   3.  After turning on output, the user can no
 *       longer adjust the image quality or record
 *       speed.  I modified the output code segment
 *       in ShowInputWorkProc() to enable those
 *       controls.
 *
 * Revision 0.100  1995/11/23  03:56:24  drapeau
 * Minor changes to eliminate compiler warnings.
 *
 * Revision 0.99  1995/10/27  20:49:14  drapeau
 * Check if output has been disabled; if so, allow recording and saving
 * to be done again.
 *
 * Revision 0.98  1995/10/06  23:01:41  drapeau
 * Added support for showing the current input signal type in use.
 *
 * Revision 0.97  1995/10/03  18:46:20  lovvik
 * Fixed a problem in ShowInput1() and ShowInput2().  The
 * record frame rate was not being set when the signal
 * standard changed.  I simply added an assignment to
 * set the recordFrame rate every time the signalFrameRate
 * is set and the recordSpeedSlider is set.
 *
 * Revision 0.96  1995/10/02  21:56:10  drapeau
 * Added support for keeping track of the user interface's visibility
 * state: whether some panels are mapped or not, etc.
 *
 * Also, updated the input selection functions.
 *
 * Revision 0.95  1995/05/12  00:21:11  lovvik
 * Added label names in the initialization of audioMap[].
 * This was required because of a change in the
 * AudioModuleMapEntry structure.
 *
 * Revision 0.94  1995/04/27  00:42:01  lovvik
 * Added audioMap which contains:
 *   1)  Command line parameter string
 *   2)  AudioObject-specific help
 *   3)  The address of the audio object
 * for each audio object that is supported on sun.
 *
 * Revision 0.93  1995/02/13  18:37:07  lovvik
 * Added some functions to add output functionality to
 * this version of MovieTool.  This was originally
 * implemented on the HP platform.  I added CreateJPEGWidget(),
 * OutputWorkProc(), and OutputCallback() to this file.
 * These functions are pretty much just a copy of the hp
 * version of the functions.  The output option does not work
 * correctly because of an error in the JPEG widget.  When
 * the JPEG widget does its redisplay method, it turns output
 * off.
 *
 * Revision 0.92  1994/10/26  17:33:56  drapeau
 * Added stubs for output; the output feature is not currently
 * implemented on the Sun platform.  Also, added support for a new input
 * model, including the addition of the "first available input" function.
 *
 * Revision 0.91  1994/09/13  08:10:41  drapeau
 * Removed extraneous variables.
 *
 * Revision 0.90  1994/08/27  05:49:38  drapeau
 * First working version.
 * */
static char sunSpecificRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/sunSpecific.c,v 0.103 1996/01/25 04:18:51 drapeau Exp $";


#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/Scale.h>
#include <Xm/PushBG.h>
#include "PortableAudio.h"
#include "MovieTool.h"


AudioModuleMapEntry audioMap[5] =
{
  {"SunAudioDevice", "Sun Audio Device", "Uses a direct connection to the audio hardware.", &defaultAudioObject},
  {"NoAudioDevice", "None", "Does not record or play audio", &noAudioObject},
  {"", "", "", (AudioObject *)NULL}
};



void SaveMPEGFile(char* fileName)
{
  sprintf(errorMessage,
	  "Saving MPEG files is not yet implemented on this platform.\n");
  PrintErrorMessage(errorMessage);
  return;
} /* end function SaveMPEGFile */



void ShowCompactMovieDialog(Widget	w,
			    XtPointer	client_data,
			    XtPointer	xt_call_data)
{
  XtVaSetValues(fileDialog,
		XmNtitle, "Compact This Movie",
		NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, SaveMovie, NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, OpenMovie, NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, CompactMovie, NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, SaveStill, NULL);
  XtAddCallback(fileSelectionBox, XmNokCallback, CompactMovie, NULL);
  XtVaSetValues(saveAsOptionButton, XmNsensitive, False, NULL);	    /* Don't allow author to choose final movie output format */
  XtManageChild(fileSelectionBox);
  return;
} /* end function ShowCompactMovieDialog */



void ShowInput1(Widget		w,
		XtPointer	client_data,
		XtPointer	xt_call_data )
{
  Dimension		signalWidth;
  Dimension		signalHeight;
  int			signalStandard;
  XmScaleCallbackStruct scaleCallbackStruct;
  
  XtPopup(input1Shell, XtGrabNone);
  SetMTPanelState(&mtPanelState, input1Shell, True);		    /* Indicate that the live video input shell is now visible */
  XtVaSetValues(input1Widget, XtNchannel, PLX_IO_OFF, NULL);
  XtVaSetValues(input1Widget, XtNchannel, PLX_INPUT_0, NULL);
  XtVaGetValues(input1Widget,					    /* Determine the signal standard for this video input, so... */
		XtNsignalStandard, &signalStandard,		    /* ...the frame rate for this standard can be determined. */
		XtNsignalWidth, &signalWidth,
		XtNsignalHeight, &signalHeight,
		NULL);
/*
  XtVaSetValues(input1Widget,
		XtNwidth, (signalWidth / 2),
		XtNheight, (signalHeight / 2),
		NULL);
*/
  if (signalStandard == PLX_NTSC)				    /* Is it an NTSC signal? */
  {
    signalFrameRate = 30;					    /* Yep, frame rate is 30 Hz */
    recordFrameRate = 30;
    XtVaSetValues(recordSpeedSlider, XmNvalue, 30, NULL);
  }
  else								    /* Nope, must be PAL or SECAM; frame rate is 25 Hz */
  {
    signalFrameRate = 25;
    recordFrameRate = 25;
    XtVaSetValues(recordSpeedSlider, XmNvalue, 25, NULL);
  }
  
  scaleCallbackStruct.value = recordFrameRate;
  ChangeRecordSpeed(recordSpeedSlider, (XtPointer)NULL, (XtPointer)&scaleCallbackStruct);
  return;
} /* end function ShowInput1 */



void ShowInput2(Widget		w,
		XtPointer	client_data,
		XtPointer	xt_call_data )
{
  Dimension		signalWidth;
  Dimension		signalHeight;
  int			signalStandard;
  XmScaleCallbackStruct scaleCallbackStruct;
  
  XtPopup(input1Shell, XtGrabNone);
  SetMTPanelState(&mtPanelState, input1Shell, True);		    /* Indicate that the live video input shell is now visible */
  XtVaSetValues(input1Widget, XtNchannel, PLX_IO_OFF, NULL);
  XtVaSetValues(input1Widget, XtNchannel, PLX_INPUT_1, NULL);
  XtVaGetValues(input1Widget,					    /* Determine the signal standard for this video input, so... */
		XtNsignalStandard, &signalStandard,		    /* ...the frame rate for this standard can be determined. */
		XtNsignalWidth, &signalWidth,
		XtNsignalHeight, &signalHeight,
		NULL);
/*
  XtVaSetValues(input1Widget,
		XtNwidth, (signalWidth / 2),
		XtNheight, (signalHeight / 2),
		NULL);
*/
  if (signalStandard == PLX_NTSC)				    /* Is it an NTSC signal? */
  {
    signalFrameRate = 30;					    /* Yep, frame rate is 30 Hz */
    recordFrameRate = 30;
    XtVaSetValues(recordSpeedSlider, XmNvalue, 30, NULL);
  }
  else								    /* Nope, must be PAL or SECAM; frame rate is 25 Hz */
  {
    signalFrameRate = 25;
    recordFrameRate = 25;
    XtVaSetValues(recordSpeedSlider, XmNvalue, 25, NULL);
  }

  scaleCallbackStruct.value = recordFrameRate;
  ChangeRecordSpeed(recordSpeedSlider, (XtPointer)NULL, (XtPointer)&scaleCallbackStruct);
  return;
}



void ResizeOutput(void)
{
  return;							    /* This function is not necessary for the Sun at present */
} /* end function ResizeOutput */



Boolean ShowInputWorkProc(XtPointer	clientData)
{
  Boolean		sync;
  signalSpec		theSignal;
  XmScaleCallbackStruct scaleCallbackStruct;
  
  if ((XtPointer)0 == clientData)				    /* Was the "first available" special value passed in? */
  {								    /* Yes, do the appropriate thing */
    XtVaSetValues(input1Widget, XtNautoSearch, True, NULL);	    /* Let the widget find the 1st valid input source */
    theSignal.s.channel = PLX_INPUT_0;				    /* Begin the search with the 1st possible input for... */
    theSignal.s.standard = PLX_NTSC;				    /* ...XVideo hardware. */
    theSignal.s.format = PLX_COMP;
    theSignal.s.instance = 0;
  }
  else								    /* No, an explicit source was requested; use the value... */
  {
    theSignal.i = (unsigned int)clientData;			    /* ...passed in. */
  }
  if (outputOn == True)						    /* Only disable output if it was on in the 1st place */
  {
    XPlxVideoOutputSelect(XtDisplay(playbackWidget),
			  XtWindow(playbackWidget),
			  outputGC,
			  PLX_IO_OFF,
			  outputSignalStandard,
			  outputSignalFormat,
			  PLX_RGB24);
    outputOn = False;
    XtVaSetValues(recordButton, XmNsensitive, True, NULL);	    /* Allow recording again now that output is off */
    XtVaSetValues(saveAsMenuButton, XmNsensitive, True, NULL);	    /* Don't allow selection of a new movie file for recording */
    EnableJPEGCompressionControls();
  }
  XtPopup(input1Shell, XtGrabNone);
  SetMTPanelState(&mtPanelState, input1Shell, True);		    /* Indicate that the live video input shell is now visible */
  XtVaSetValues(input1Widget, XtNchannel, PLX_IO_OFF, NULL);	    /* Disable video before changing channels */
  XtVaSetValues(input1Widget,					    /* Attempt to choose the new signal source */
		XtNchannel, theSignal.s.channel,
		XtNsignalStandard, theSignal.s.standard,
		XtNsignalFormat, theSignal.s.format,
		XtNinstance, theSignal.s.instance,
		NULL);
  XtVaGetValues(input1Widget,					    /* Determine the signal standard for this video input, so... */
		XtNsignalStandard, &inputSignalStandard,	    /* ...the frame rate for this standard can be determined. */
		XtNsignalFormat, &inputSignalFormat,
		XtNinstance, &inputInstance,
		XtNsync, &sync,
		NULL);
  if (inputSignalStandard == PLX_NTSC)				    /* Is it an NTSC signal? */
  {
    signalFrameRate = 30;					    /* Yep, frame rate is 30 Hz */
    recordFrameRate = 30;					    /* Set an initial rate for recording NTSC movies */
    XtVaSetValues(recordSpeedSlider, XmNvalue, 30, NULL);
  }
  else								    /* Nope, must be PAL or SECAM; frame rate is 25 Hz */
  {
    signalFrameRate = 25;
    recordFrameRate = 25;					    /* Set an initial rate for recording PAL or SECAM movies */
    XtVaSetValues(recordSpeedSlider, XmNvalue, 25, NULL);
  }

  theSignal.s.standard = inputSignalStandard;			    /* Reload theSignal struct with current values. */
  theSignal.s.format = inputSignalFormat;
  theSignal.s.instance = inputInstance;
  
  UpdateInputTypeString(&theSignal, sync);			    /* Update the label showing the current signal type */

  scaleCallbackStruct.value = recordFrameRate;
  ChangeRecordSpeed(recordSpeedSlider, (XtPointer)NULL, (XtPointer)&scaleCallbackStruct);
  return(True);
} /* end function ShowInputWorkProc */



void ShowInput(Widget		w,
	       XtPointer	clientData,
	       XtPointer	unusedCallData)
{
  XtWorkProcId	inputWorkProcID;
  inputWorkProcID = XtAppAddWorkProc(app_context,		    /* Start a work-proc to select a new input source; Xt is... */
				     ShowInputWorkProc,		    /* ...poorly designed and this is the only way to assure... */
				     (XtPointer)clientData);	    /* ...that widgets needing to be destroyed right now... */
  return;							    /* ...searching for alternate input sources. */
} /* end function ShowInput */



void ShowFirstAvailableInput(Widget	w,
			     XtPointer	unusedClientData,
			     XtPointer	unusedCallData)
{
  XtVaSetValues(input1Widget, XtNautoSearch, True, NULL);	    /* Allow the widget to search for any valid input */
  ShowInput(w, (XtPointer)0, (XtPointer)NULL);			    /* Tell the application to perform the actual search */
  return;
} /* end function ShowFirstAvailableInput */


void CreateJPEGWidget(void)
{

}


Boolean OutputWorkProc(XtPointer	clientData)
{
  static Boolean	firstTime = True;
  Widget		w;
  
  w = clientData;
  if (movieStatus == MovieNew)					    /* Don't allow output until a movie has been played */
    return(True);
  outputOn = True;						    /* Note that output is about to be enabled */
  XtVaSetValues(recordButton, XmNsensitive, False, NULL);	    /* Don't allow recording while output is on */
  XtVaSetValues(saveAsMenuButton, XmNsensitive, False, NULL);	    /* Don't allow selection of a new movie file for recording */
  DisableJPEGCompressionControls();
  if (firstTime == True)
  {
    outputGC = XCreateGC(XtDisplay(w), XtWindow(playbackWidget), 0, 0);
    firstTime = False;
  }
  if (PLX_NTSC == outputSignalStandard)				    /* Was NTSC output selected? */
  {								    /* Yes, use NTSC values (This is a hack!  Please remove... */
    outputSignalWidth = 640;					    /* ...it and use the proper API calls to determine the... */
    outputSignalHeight = 480;					    /* ...width & height of the output signal.  Ideally, the... */
  }								    /* ...Output widget should be used for this. */
  else
  {
    outputSignalWidth = 768;
    outputSignalHeight = 575;
  }
  XtDestroyWidget(input1Widget);				    /* Make sure that the Port attached to the input... */
  input1Widget = XtVaCreateManagedWidget("input1Widget",	    /* ...widget is detached by destroying the widget's... */
					 xVideoInWidgetClass,	    /* ...window, then re-create the input widget but don't... */
					 input1Shell,		    /* ...immediately show video on it (showing video would... */
					 XtNchannel, PLX_IO_OFF,    /* ...cause a Port to be attached to the Window). */
					 XtNsignalStandard, inputSignalStandard,
					 XtNsignalFormat, inputSignalFormat,
					 XtNinstance, inputInstance,
					 NULL);
  XPlxVideoOutputSelect(XtDisplay(w),				    /* Force JPEG playback to use the 1st hardware port... */
			XtWindow(playbackWidget),		    /* ...(the "v" channel). */
			outputGC,
			PLX_OUTPUT_0,
			outputSignalStandard,
			outputSignalFormat,
			0);
  XPlxVideoScaleOutput(XtDisplay(w),
		       XtWindow(playbackWidget),
		       outputGC,
		       0, 0,
		       movieHeader.width, movieHeader.height,
		       0, 0,
		       outputSignalWidth, outputSignalHeight);
  PlayOneFrame(currentFrame, False, True);			    /* Refresh the playback window after output is ready */
  return(True);
} /* end function OutputWorkProc */



void OutputCallback(Widget	w,
		    XtPointer	client_data,
		    XtPointer	xt_call_data)
{
  XtWorkProcId	outputWorkProcID;
  
  outputWorkProcID = XtAppAddWorkProc(app_context,		    /* Start a work-proc to show output; Xt is... */
				      OutputWorkProc,		    /* ...poorly designed and this is the only way to assure... */
				      (XtPointer)w);		    /* ...that widgets needing to be destroyed right now... */
  return;							    /* ...will happen. */
} /* end function OutputCallback */


