/* $Header: /plx/source/Xclients/generic/RCS/utils.c,v 1.51 1996/01/25 04:19:05 drapeau Exp $ */
/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Log: utils.c,v $
 * Revision 1.51  1996/01/25  04:19:05  drapeau
 * Added support for the PlxOutputCascade to automatically create the
 * output menu hierarchy, for enhanced flexibility.
 *
 * Revision 1.50  1996/01/05  23:38:21  drapeau
 * Two changes:
 *
 * 1) Added functionality to SetMovieStatus() to account for the new
 * still-image-save feature.
 *
 * 2) Fixed an error in UpdateInputTypeString() to account for YUV
 * inputs.
 *
 * Revision 1.49  1995/11/23  03:57:11  drapeau
 * Minor changes:
 *
 * 1) Added support for the "syncSlack" slider, allowing for
 * synchronization tolerance to help eliminate audio/video lag and
 * excessive pops/gaps in audio when playback conditions are less than
 * ideal.
 *
 * 2) Minor changes to eliminate compiler warnings.
 *
 * Revision 1.48  1995/11/09  20:40:48  drapeau
 * Removed minor errors with memory leaks.
 *
 * Revision 1.47  1995/10/12  22:17:05  drapeau
 * Modified CreateLogoPixmap() to add support for a second,
 * black-and-white logo.
 *
 * Revision 1.46  1995/10/06  23:01:48  drapeau
 * Added support for showing the current input signal type in use.
 *
 * Revision 1.45  1995/10/04  17:23:34  lovvik
 * The searchSlider widget would complain if the last
 * valid frame in the movie is the same as the first
 * valid frame.  I corrected this by detecting this
 * situation, and if it occurs, I set the maximum slider
 * value to 1.
 *
 * Revision 1.44  1995/10/02  21:56:25  drapeau
 * Added support for keeping track of the user interface's visibility
 * state: whether some panels are mapped or not, etc.
 *
 * Revision 1.43  1995/08/18  02:06:25  drapeau
 * Added code to put the company logo in the application's "About..."
 * panel.
 *
 * Revision 1.42  1995/08/17  21:51:46  drapeau
 * Added support for an "About..." panel.
 *
 * Revision 1.41  1995/05/16  02:58:44  drapeau
 * Removed extraneous code no longer needed since adding the cascading
 * menu code written by Dave Arnold.
 *
 * Revision 1.40  1995/05/10  20:28:33  lovvik
 * Added two functions:
 *   1)  DisableAudioDeviceControls()
 *   2)  EnableAudioDeviceControls()
 *
 * Revision 1.39  1995/05/09  19:01:04  lovvik
 * Added a function called getDisplayName().  This function
 * looks through the command line parameters for '-display'.
 * If '-display' is not found, getDisplayName() gets the display
 * name from the DISPLAY environment variable.
 *
 * Revision 1.38  1995/04/27  00:43:37  lovvik
 * Modified PrintErrorMessage() to NOT put up a message if MovieTool
 * is trying to autoPlay a movie.
 *
 * Revision 1.37  1995/04/18  01:07:02  drapeau
 * Added support for the 2nd YC input for the HP; this should also apply
 * to the IBM platform.
 *
 * Revision 1.36  1995/04/07  17:18:12  lovvik
 * Added an include statement for Xm/Protocols.h.  This header is
 * required for XmAddWMProtocolCallback.
 *
 * Revision 1.35  1995/04/01  02:36:58  drapeau
 * Added code to InitializeInterface() to support quitting the
 * application cleanly from the window manager's title bar menu.
 *
 * Revision 1.34  1994/10/17  21:20:09  drapeau
 * Fixed a minor error in EnableTransportButtons(): now checks if output
 * is enabled; if so, the JPEG compression preferences are not enabled.
 *
 * Revision 1.33  1994/10/15  01:45:41  drapeau
 * Added code to complete MovieTool's notion of what operations should be
 * allowed under what conditions.  For example, input selection should
 * not be allowed during recording and playback.
 *
 * Revision 1.32  1994/10/14  03:02:01  drapeau
 * A number of errors were fixed:
 *
 * * Added the function EnableJPEGCompressionControls() to allow
 *   compression preferences to be set even when movies cannot yet be
 *   played.
 *
 * * Updated SetMovieStatus() to better reflect the differences in
 *   states; for example, it is possible to save a movie as MPEG when it
 *   is opened but not modified, but the previous version of the code did
 *   not allow for this possibility.
 *   Also reset a number of interface components and internal variables
 *   when the MovieNew state was set.
 *
 * * Added a test to UpdateMovieLengthIndicators(): if called before a
 *   movie has been recorded, the movieHeader data structure will be
 *   initialized to zero and so its frames-per-second field will be zero,
 *   causing a divide-by-zero error in the ConvertToTime() function when
 *   trying to determine the duration of a new, empty movie.  The new
 *   code now detects this and uses the default frame rate (which is
 *   based on the video signal standard in effect at launch time).
 *
 * * Added interface code to handle the new output option and richer
 *   input selection model.  This includes, but is not limited to, the
 *   new function BuildInputMenu().
 *
 * Revision 1.31  1994/10/10  23:04:14  drapeau
 * Updated functions in InitializeInterface() to reflect the addition of
 * the new input selection menu items and removal of obsolete items.
 *
 * Revision 1.30  1994/10/06  02:25:36  drapeau
 * Enhanced functions that enable and disable interface components to
 * eliminate possible errors during record and playback.
 *
 * Revision 1.29  1994/10/03  17:26:56  drapeau
 * Added code to disallow output before a movie is ready to play, and to
 * allow output when the movie is ready.
 *
 * Revision 1.28  1994/09/30  16:58:42  drapeau
 * Added support for output.
 *
 * Revision 1.27  1994/09/23  06:35:16  drapeau
 * Fixed an error in UpdateTimecodeDisplay() during the recording
 * process: when a previous edit had changed the start frame or end frame
 * so that the resulting movie was shorter than the original, a new
 * movie's recording would cause the timecode display to show negative
 * numbers until the old movie's new starting frame was reached.  This
 * math error has been corrected.
 *
 * Revision 1.26  1994/09/13  08:11:45  drapeau
 * Fixed potential dangling memory pointer problem (declared character
 * space as static to keep it around).  Also dimmed the "Compact..." menu
 * item when the movie is unmodified.
 *
 * Revision 1.25  1994/08/23  01:35:42  drapeau
 * Added code for an alert dialog box.
 *
 * Revision 1.24  1994/05/31  20:14:40  drapeau
 * Fixed tools for converting and displaying timecode for a movie.
 *
 * Revision 1.23  1994/01/10  22:41:44  drapeau
 * Added functionality to properly display movie status, timecode, etc.
 * Also, fixed minor errors in the translation between timecode and frame
 * number values.
 *
 * Revision 1.22  1993/12/04  01:02:54  drapeau
 * Added the function UpdateTotalTimePaused() to make the pause feature
 * more robust.  Before this version, pausing a movie during playback
 * would throw off the synchronization mechanism; pause now works
 * correctly.
 *
 * Revision 1.21  1993/12/03  02:12:55  drapeau
 * Minor change in RCS info; mostly it was done to allow me to change the
 * RCS numbering scheme for this file.
 *
 * Revision 1.2  1993/12/03  01:37:58  drapeau
 * Added a function to update the timecode display on the main panel;
 * also added a prototype function to disable certain controls during
 * playback and record.
 * */
static char utilsRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/utils.c,v 1.51 1996/01/25 04:19:05 drapeau Exp $";

#include <Xm/Protocols.h>
#include <stdlib.h>
#include <string.h>
#include "MovieTool.h"

/* Convert time to address in frames */
int
ConvertToAddress(int hr, int min, int sec, int frame)
{
  int address;
  
  address = signalFrameRate * (hr * 3600 + min * 60 + sec) + frame;
  return(address);
}								    /* end function ConvertToAddress */



void ConvertToTime(int* hr,					    /* Convert address in frames to time */
		   int* min,
		   int* sec,
		   int*	frame,
		   int	frameNumber,
		   int	framesPerSecond)
{
  int	framesPerHour;
  int	framesPerMinute;

  if(framesPerSecond == 0)
  {
    *hr = 0;
    *min = 0;
    *sec = 0;
    *frame = 0;
  }
  else
  {
    framesPerHour = 3600 * framesPerSecond;			    /* 3600 = seconds per hour */
    framesPerMinute = 60 * framesPerSecond;
    
    *hr = frameNumber / framesPerHour;
    *min = (frameNumber % framesPerHour) / framesPerMinute;
    *sec = ((frameNumber % framesPerHour) % framesPerMinute) / framesPerSecond;
    *frame = ((frameNumber % framesPerHour) % framesPerMinute) % framesPerSecond;
  }
}								    /* end function ConvertToTime */



void UpdateTimecodeDisplay(int	frameRate)
{
  int		hour;
  int		minute;
  int		second;
  int		frame;
  static int	lastHour = 0;
  static int	lastMinute = 0;
  static int	lastSecond = 0;
  static int	lastFrame = 0;
  int		whichFrame;
  
  if (movieToolStatus == Recording)				    /* Determine how to calculate the timecode to be used;... */
    whichFrame = currentFrame;					    /* ...it depends on whether MovieTool is recording or... */
  else								    /* ...playing back. */
    whichFrame = currentFrame - firstFrame;
  ConvertToTime(&hour, &minute, &second, &frame,		    /* Determine the correct timecode value based on the... */
		whichFrame, frameRate);				    /* ...current frame and the frame rate. */
  if (hour != lastHour)						    /* Did the hour value change?  Avoid unnecessary drawing... */
  {								    /* ...and updating of the text field */
    lastHour = hour;
    sprintf(timecodeString, "%02d", hour);
    XmTextSetString(hoursTextField, timecodeString);
  }
  if (minute != lastMinute)					    /* Did the minute value change?  Avoid unnecessary... */
  {								    /* ... drawing and updating of the text field */
    lastMinute = minute;
    sprintf(timecodeString, "%02d", minute);
    XmTextSetString(minutesTextField, timecodeString);
  }
  if (second != lastSecond)					    /* Did the second value change?  Avoid unnecessary... */
  {								    /* ...drawing and updating of the text field */
    lastSecond = second;
    sprintf(timecodeString, "%02d", second);
    XmTextSetString(secondsTextField, timecodeString);
  }
  if (frame != lastFrame)					    /* Did the frame value change?  Avoid unnecessary drawing... */
  {								    /* ...and updating of the text field */
    lastFrame = frame;
    sprintf(timecodeString, "%02d", frame);
    XmTextSetString(framesTextField, timecodeString);
  }
  if (movieToolStatus != Recording)				    /* If MovieTool is recording, no use updating the slider */
    XtVaSetValues(searchSlider,					    /* Else, update the search slider to show the current... */
		  XmNvalue, (currentFrame - firstFrame),	    /* ...position within the movie. */
		  NULL);
  return;
} /* end function UpdateTimecodeDisplay */



void DeSensitizePlaybackControls(void)
{
  XtVaSetValues(hourPlusButton, XmNsensitive, False, NULL);
  XtVaSetValues(minutePlusButton, XmNsensitive, False, NULL);
  XtVaSetValues(secondPlusButton, XmNsensitive, False, NULL);
  XtVaSetValues(framePlusButton, XmNsensitive, False, NULL);
  
  XtVaSetValues(hourMinusButton, XmNsensitive, False, NULL);
  XtVaSetValues(minuteMinusButton, XmNsensitive, False, NULL);
  XtVaSetValues(secondMinusButton, XmNsensitive, False, NULL);
  XtVaSetValues(frameMinusButton, XmNsensitive, False, NULL);
  
  XtVaSetValues(searchSlider, XmNsensitive, False, NULL);
  XtVaSetValues(frameAdvanceButton, XmNsensitive, False, NULL);
  XtVaSetValues(frameReverseButton, XmNsensitive, False, NULL);
  XtVaSetValues(endOfMovieButton, XmNsensitive, False, NULL);
  XtVaSetValues(startOfMovieButton, XmNsensitive, False, NULL);
  
  DisableJPEGCompressionControls();
  return;
} /* end function DeSensitizePlaybackControls */



void DisableJPEGCompressionControls(void)
{
  XtVaSetValues(imageQualitySlider, XmNsensitive, False, NULL);
  XtVaSetValues(recordSpeedSlider, XmNsensitive, False, NULL);
  XtVaSetValues(fullSizeButton, XmNsensitive, False, NULL);
  XtVaSetValues(quarterSizeButton, XmNsensitive, False, NULL);
  XtVaSetValues(smallSizeButton, XmNsensitive, False, NULL);
  XtVaSetValues(outputOptionCascade, XmNsensitive, False, NULL);
} /* end function DisableJPEGCompressionControls */



void EnableJPEGCompressionControls(void)
{
  if (outputOn == False)
  {
    XtVaSetValues(imageQualitySlider, XmNsensitive, True, NULL);
    XtVaSetValues(recordSpeedSlider, XmNsensitive, True, NULL);
    XtVaSetValues(fullSizeButton, XmNsensitive, True, NULL);
    XtVaSetValues(quarterSizeButton, XmNsensitive, True, NULL);
    XtVaSetValues(smallSizeButton, XmNsensitive, True, NULL);
  }
  XtVaSetValues(outputOptionCascade, XmNsensitive, True, NULL);
} /* end function EnableJPEGCompressionControls */


void DisableAudioDeviceControls(void)
{
  XtSetSensitive(audioDeviceFrame, False);
}

void EnableAudioDeviceControls(void)
{
  XtSetSensitive(audioDeviceFrame, True);
}



void UpdateTotalTimePaused(Boolean startPausing)
{
  static struct timeval		pauseStartTime;
  struct timeval		pauseStopTime;
  
  if (startPausing == True)
  {
    gettimeofday(&pauseStartTime, (struct timezone*)NULL);	    /* Record the time that the pause was turned on*/
  }
  else
  {
    gettimeofday(&pauseStopTime, (struct timezone*)NULL);	    /* Record the time that the pause was turned off */
    totalTimePaused += ElapsedTime(&pauseStartTime, &pauseStopTime); /* Calculate the total time paused and update the variable */
  }
  return;
} /* end function UpdateTotalTimePaused */



void UpdateTitleBar()
{
  static char	titleBarString[MAXPATHLEN + 64];
  
  if (movieStatus == MovieModified)
  {
    sprintf(titleBarString, "Parallax Graphics MovieTool: %s (modified)", currentMovieFileName);
  }
  if (movieStatus == MovieNew)
  {
    sprintf(titleBarString, "Parallax Graphics MovieTool: %s (new)", currentMovieFileName);
  }
  if (movieStatus == MovieOK)
  {
    sprintf(titleBarString, "Parallax Graphics MovieTool: %s", currentMovieFileName);
  }
  XtVaSetValues(mainWindow, XmNtitle, titleBarString, NULL);	    /* Reset the title string for MovieTool's main window */
  return;
} /* end function UpdateTitleBar */



void SetMovieStatus(int	status)
{
  movieStatus = status;
  if (movieStatus == MovieModified)				    /* Should the "Save" button be enabled? */
  {
    XtVaSetValues(saveMenuButton, XmNsensitive, True, NULL);	    /* Yes, this movie needs to be saved */
  }
  else
  {
    XtVaSetValues(saveMenuButton, XmNsensitive, False, NULL);
  }
  if (movieStatus == MovieNew)					    /* Only new movies cannot be saved as MPEG or compacted */
  {
    XtVaSetValues(compactMovieButton, XmNsensitive, False, NULL);
    currentFrame = 0;
    recordHeader.frames = 0;
    recordHeader.firstValidFrame = 0;
    recordHeader.lastValidFrame = 0;
    firstFrame = 0;
    lastFrame = 2 * recordFrameRate;				    /* Set an arbitrary new lastFrame to 2 seconds long */
    UpdateTimecodeDisplay(recordFrameRate);
    UpdateMovieLengthIndicators();				    /* Update interface parts for movie length, start, & end */
    DeSensitizePlaybackControls();				    /* Don't allow the author to play any existing movie */
    EnableJPEGCompressionControls();				    /* But allow the JPEG preferences to be manipulated */
  }
  else
  {
    XtVaSetValues(compactMovieButton, XmNsensitive, True, NULL);
    XtVaSetValues(saveOneStillButton, XmNsensitive, True, NULL);    /* Allow still images to be saved now that they're available */
  }
  if ((movieStatus != MovieNew) && (outputAvailable == True))	    /* Allow output only if a valid movie has just been... */
  {								    /* ...recorded, or if one has been opened for playback. */
    XtVaSetValues(outputOptionCascade, XmNsensitive, True, NULL);   /* Allow output now that a movie is there */
  }
  UpdateTitleBar();
  return;
} /* end function SetMovieStatus */



void UpdateMovieLengthIndicators(void)
{
  int					hour;
  int					minute;
  int					second;
  int					frame;
  int					frameRate;
  char					tempString[32];
  XmString				labelString;
  int					sliderLength;
  
  if (movieHeader.fps == 0)					    /* When the app launches, the movieHeader structure is... */
    frameRate = recordFrameRate;				    /* ...initialized to zero; in this case, set the frame... */
  else								    /* ...rate to use to the initial record speed in frames. */
    frameRate = movieHeader.fps;
  
  sliderLength = lastFrame-firstFrame;
  if(sliderLength == 0)
  {
    sliderLength = 1;
  }
  XtVaSetValues(searchSlider,					    /* Update the search slider to constrain itself to the... */
		XmNminimum, 0,					    /* ...number of frames in this movie. */
		XmNmaximum, sliderLength,
		NULL);
  ConvertToTime(&hour, &minute, &second, &frame,		    /* Determine duration of this movie in timecode terms */
		lastFrame - firstFrame,
		frameRate);
  sprintf(tempString, "%02d:%02d:%02d:%02d", hour, minute, second, frame);
  labelString = XmStringCreateSimple(tempString);		    /* Create a string to put on the end-of-movie timecode label */
  XtVaSetValues(endTimeLabel, XmNlabelString, labelString, NULL);   /* Update the label showing the end timecode of this movie */
  XmStringFree(labelString);
  return;
} /* end function UpdateMovieLengthIndicators */



void CreateLogoPixmap(Widget	parentWidget)			    /* Creates a masked Pixmap in shape of the company logo... */
{								    /* ...to display in the application's "About..." panel. */
  unsigned long	background;
  Display*	display = XtDisplay(parentWidget);
  int		depth;
  XGCValues	gcValues;
  Pixmap	logoPixmap;
  Pixmap	logoShapeMask;
  XpmAttributes	logoAttrs;
  GC		logoGC;
  Pixmap	bwLogoPixmap;
  Pixmap	bwLogoShapeMask;
  XpmAttributes	bwLogoAttrs;
  int		result;
  Pixmap	thePixmap;
  Pixmap	bwPixmap;
  Window	window = XtWindow(parentWidget);
  
  logoAttrs.valuemask = 0;
  result = XpmCreatePixmapFromData(display,			    /* Attempt to create a Pixmap from XPM data */
				   window,			    /* The function, if successful, will return both a Pixmap... */
				   plxLogoData,			    /* ...in the shape of the company logo, in color, plus... */
				   &logoPixmap,			    /* ...a mask that can be used to show only the Pixmap... */
				   &logoShapeMask,		    /* ...data (as if it were transparent). */
				   &logoAttrs);
  if (result != 0)						    /* Did the Pixmap creation fail? */
    return;							    /* Yes, don't use any Pixmap for the logo panel */
  bwLogoAttrs.valuemask = 0;
  result = XpmCreatePixmapFromData(display,			    /* Attempt to create a black-and-white version of the... */
				   window,			    /* ...corporate logo. */
				   bwPlxLogoData,
				   &bwLogoPixmap,
				   &bwLogoShapeMask,
				   &bwLogoAttrs);
  if (result != 0)						    /* Did the Pixmap creation fail? */
    return;							    /* Yes, don't use any Pixmap for the logo panel */
  XtVaGetValues(parentWidget,					    /* Get the depth and background color of the parent widget */
		XmNdepth, &depth,
		XmNbackground, &background,
		NULL);
  thePixmap = XCreatePixmap(display,				    /* Create a new Pixmap to be used as the application's logo */
			    window,
			    logoAttrs.width, logoAttrs.height,
			    depth);
  bwPixmap = XCreatePixmap(display,				    /* Create a new Pixmap to be used as the application's logo */
			   window,
			   bwLogoAttrs.width, bwLogoAttrs.height,
			   depth);
  gcValues.background = background;
  gcValues.foreground = background;
  logoGC = XCreateGC(display,					    /* Create a GC that paints the application's background... */
		     window,					    /* ...color into the final logo Pixmap */
		     (GCBackground | GCForeground),
		     &gcValues);
  XFillRectangle(display, thePixmap, logoGC, 0, 0,		    /* Paint the bkgnd color into the final logo Pixmap */
		 logoAttrs.width, logoAttrs.height);
  XFillRectangle(display, bwPixmap, logoGC, 0, 0,		    /* Paint the bkgnd color into the black-and-white Pixmap */
		 bwLogoAttrs.width, bwLogoAttrs.height);
  XSetClipMask(display, logoGC, logoShapeMask);			    /* Make sure that painting the new, color logo data only... */
  XCopyArea(display, logoPixmap, thePixmap, logoGC,		    /* ...paints into the areas it's supposed to, making... */
	    0, 0, logoAttrs.width, logoAttrs.height,		    /* ...a "transparent" effect. */
	    0, 0);
  XSetClipMask(display, logoGC, bwLogoShapeMask);		    /* Do the same for the black-and-white Pixmap */
  XCopyArea(display, bwLogoPixmap, bwPixmap, logoGC,
	    0, 0, bwLogoAttrs.width, bwLogoAttrs.height,
	    0, 0);
  XtVaSetValues(aboutDialog, XmNsymbolPixmap, thePixmap, NULL);	    /* Use the newly-created icon as the About panel's symbol */
  XtVaSetValues(logoLabel, XmNlabelPixmap, bwPixmap, NULL);	    /* Use the newly-created icon as the logo's symbol, too */
  return;
} /* end function CreateLogoPixmap */



void InitializeInterface(void)
{
  XmScaleCallbackStruct	callData;
  Atom			deleteWindowAtom;
  
  XtVaSetValues(practiceRunButton, XmNsensitive, False, NULL);
  XtVaSetValues(hourMinusButton, XmNsensitive, False, NULL);
  XtVaSetValues(minuteMinusButton, XmNsensitive, False, NULL);
  XtVaSetValues(secondMinusButton, XmNsensitive, False, NULL);
  XtVaSetValues(frameMinusButton, XmNsensitive, False, NULL);
  XtVaSetValues(hourPlusButton, XmNsensitive, False, NULL);
  XtVaSetValues(minutePlusButton, XmNsensitive, False, NULL);
  XtVaSetValues(secondPlusButton, XmNsensitive, False, NULL);
  XtVaSetValues(framePlusButton, XmNsensitive, False, NULL);
  
  XtVaSetValues(searchSlider, XmNsensitive, False, NULL);
  XtVaSetValues(speedSlider, XmNsensitive, False, NULL);
  XtVaSetValues(playButton, XmNsensitive, False, NULL);
  XtVaSetValues(pauseButton, XmNsensitive, False, NULL);
  XtVaSetValues(frameAdvanceButton, XmNsensitive, False, NULL);
  XtVaSetValues(frameReverseButton, XmNsensitive, False, NULL);
  XtVaSetValues(endOfMovieButton, XmNsensitive, False, NULL);
  XtVaSetValues(startOfMovieButton, XmNsensitive, False, NULL);
/*  XtVaSetValues(helpMenuCascade, XmNsensitive, False, NULL);*/
  XtVaSetValues(outputOptionCascade, XmNsensitive, False, NULL);    /* Don't allow output until a movie is ready to play */
  XtVaSetValues(saveOneStillButton, XmNsensitive, False, NULL);	    /* Don't allow still images to be saved until movie's opened */
  callData.value = recordFrameRate;
  ChangeRecordSpeed((Widget)NULL,				    /* Make sure the recording speed is okay for the audio... */
		    (XtPointer)NULL,				    /* record rate being used. */
		    (XtPointer)&callData);
  XmScaleGetValue(syncSlackScale, &syncSlack);			    /* Initialize sync tolerance variable to that shown */
  deleteWindowAtom = XmInternAtom(display,			    /* Create an Atom for communicating with the window mgr... */
				  "WM_DELETE_WINDOW",		    /* ...that the app. needs to be informed about shells... */
				  False);			    /* ...being deleted. */
  XmAddWMProtocolCallback(mainWindow, deleteWindowAtom,		    /* Set up a quit callback for the top-level shell */
			  (XtCallbackProc)WMDeleteWindowCallback,
			  (XtPointer)NULL);
  XmAddWMProtocolCallback(input1Shell, deleteWindowAtom,	    /* Set up a callback for the live input shell to act... */
			  (XtCallbackProc)WMDeleteWindowCallback,   /* ...properly when the window manager deletes the window. */
			  (XtPointer)NULL);
  XmAddWMProtocolCallback(movieShell, deleteWindowAtom,	    /* Set up a callback for the movie shell to act... */
			  (XtCallbackProc)WMDeleteWindowCallback,   /* ...properly when the window manager deletes the window. */
			  (XtPointer)NULL);
  CreateLogoPixmap(aboutShell);					    /* Create a graphical logo for the About panel */
  return;
} /* end function InitializeInterface */



void EnableTransportButtons(void)
{
  XtVaSetValues(hourMinusButton, XmNsensitive, True, NULL);
  XtVaSetValues(minuteMinusButton, XmNsensitive, True, NULL);
  XtVaSetValues(secondMinusButton, XmNsensitive, True, NULL);
  XtVaSetValues(frameMinusButton, XmNsensitive, True, NULL);
  XtVaSetValues(hourPlusButton, XmNsensitive, True, NULL);
  XtVaSetValues(minutePlusButton, XmNsensitive, True, NULL);
  XtVaSetValues(secondPlusButton, XmNsensitive, True, NULL);
  XtVaSetValues(framePlusButton, XmNsensitive, True, NULL);
  
  XtVaSetValues(searchSlider, XmNsensitive, True, NULL);
  XtVaSetValues(speedSlider, XmNsensitive, True, NULL);
  XtVaSetValues(playButton, XmNsensitive, True, NULL);
  XtVaSetValues(pauseButton, XmNsensitive, True, NULL);
  XtVaSetValues(frameAdvanceButton, XmNsensitive, True, NULL);
  XtVaSetValues(frameReverseButton, XmNsensitive, True, NULL);
  XtVaSetValues(endOfMovieButton, XmNsensitive, True, NULL);
  XtVaSetValues(startOfMovieButton, XmNsensitive, True, NULL);
  
  if (outputOn == False)
  {
    XtVaSetValues(recordSpeedSlider, XmNsensitive, True, NULL);
    XtVaSetValues(fullSizeButton, XmNsensitive, True, NULL);
    XtVaSetValues(quarterSizeButton, XmNsensitive, True, NULL);
    XtVaSetValues(smallSizeButton, XmNsensitive, True, NULL);
    XtVaSetValues(imageQualitySlider, XmNsensitive, True, NULL);
  }
  if (outputAvailable == True)					    /* Allow output to be generated if the hardware is there */
    XtVaSetValues(outputOptionCascade, XmNsensitive, True, NULL);
  XtVaSetValues(inputMenuCascade, XmNsensitive, True, NULL);
  return;
} /* end function EnableTransportButtons */



void PrintErrorMessage(char*	message)
{
  XmString		messageString;
  
  if(!(autoPlay == True))				    	    /* Is this supposed to play a movie automatically? */
  {
    messageString = XmStringCreateLocalized(message);		    /* No, go ahead and display the message */
    XtVaSetValues(alertLabel, XmNlabelString, messageString, NULL);
    XmStringFree(messageString);
    if (XtIsManaged(alertDialog) == False)
    {
      XtManageChild(alertDialog);
    }
  }
} /* end function PrintErrorMessage */



void RemoveMovie(void*	theMovie,
		 char*	movieFileName)
{
  close((int)theMovie);						    /* Close the file being recorded */
  unlink(movieFileName);					    /* Remove the file from the filesystem since it's no good */
  SetMovieStatus(MovieNew);					    /* Update interface to reflect change in movie file status */
  return;
} /* end function RemoveMovie */



void BuildInputMenu(void)
{
  plx_IO*		hardwareInfo;				    /* Info about the Parallax board being used */
  plx_IO_list*		outputList = (plx_IO_list*) NULL;
  
  XtVaGetValues(input1Widget,					    /* Retrieve information about the hardware being used */
		XtNhardwareParams, &hardwareInfo,
		NULL);
  inputsCascade = (PlxInputCascade*)NULL;
  inputsCascade = PlxCreateInputCascade(inputMenu,		    /* Create a cascading menu that creates only those input... */
					(Widget)NULL,		    /* ...selections that are appropriate for the hardware... */
					(Widget)NULL,		    /* ...being used. */
					(Widget)NULL,
					hardwareInfo,
					ShowInput);		    /* Call "ShowInput()" to change input settings */
  outputList = hardwareInfo->outputs;				    /* Point to the list of output information for this board */
  if (outputList != (plx_IO_list*)NULL)				    /* Does output exist for this hardware? */
  {								    /* Yes, do the appropriate thing */
    outputAvailable = True;
    outputMenuCascade = (PlxOutputCascade*)NULL;
    outputMenuCascade = PlxCreateOutputCascade(outputMenu,	    /* Create a cascading menu that creates only those output... */
					       (Widget)NULL,	    /* ...selections that are appropriate for the hardware... */
					       (Widget)NULL,	    /* ...being used. */
					       (Widget)NULL,
					       hardwareInfo,
					       SetOutputSignalType);
  }
  return;
} /* end function BuildInputMenu */



void getDisplayName(int argc, char **argv)
{
  int index;
  
  index = 1;
  while (index < argc)
  {
    if(strcmp(argv[index], "-display") == 0)
    {
      displayName = malloc(strlen(argv[index + 1]));
      strcpy(displayName, argv[index + 1]);
      return;
    }
    index++;
  }
  if(displayName == (char *)NULL)
  {
    if(getenv("DISPLAY"))
    {
      displayName = malloc((int)strlen(getenv("DISPLAY")) + 1);
      strcpy(displayName, getenv("DISPLAY"));
    }
  }
}



void InitMTPanelState(MTPanelState*	panelState)		    /* Initializes an instance of an MTPanelState structure:... */
{								    /* ...by default, no interface panels are visible. */
  panelState->mainWindow = False;
  panelState->audioShell = False;
  panelState->inputSettingsShell = False;
  panelState->input1Shell = False;
  panelState->fileDialog = False;
  panelState->smartMovieShell = False;
  panelState->movieShell = False;
  panelState->alertShell = False;
  panelState->aboutShell = False;
  return;
} /* end function InitMTPanelState */



void SetMTPanelState(MTPanelState*	panelState,		    /* Sets the visible state of a specified panel within... */
		     Widget		whichPanel,		    /* ...the panelState variable passed in as argument. */
		     Boolean		visibleState)		    /* For example, to indicate that the "audioShell" panel... */
{								    /* ...is no longer visible, call as follows:... */
  if (whichPanel == mainWindow)					    /* ..."SetMTPanelState(&mtPanelState, audioShell, False)". */
  {
    panelState->mainWindow = visibleState;
    return;
  }
  if (whichPanel == audioDialog)
  {
    panelState->audioShell = visibleState;
    return;
  }
  if (whichPanel == inputSettingsDialog)
  {
    panelState->inputSettingsShell = visibleState;
    return;
  }
  if (whichPanel == input1Shell)
  {
    panelState->input1Shell = visibleState;
    return;
  }
  if (whichPanel == fileDialog)
  {
    panelState->fileDialog = visibleState;
    return;
  }
  if (whichPanel == smartMovieDialog)
  {
    panelState->smartMovieShell = visibleState;
    return;
  }
  if (whichPanel == movieShell)
  {
    panelState->movieShell = visibleState;
    return;
  }
  if (whichPanel == alertDialog)
  {
    panelState->alertShell = visibleState;
    return;
  }
  if (whichPanel == aboutDialog)
  {
    panelState->aboutShell = visibleState;
    return;
  }
  return;
} /* end function SetMTPanelState */



void UpdateInputTypeString(signalSpec*	signalInfoPtr,
			   Boolean	sync)
{
  char		channelString[16];
  char		formatString[16];
  char		inputStatusString[64];
  char		instanceString[3];
  XmString	label;
  char		standardString[8];
  char		syncString[16];
  
  sprintf(instanceString, "%d", signalInfoPtr->s.instance + 1);	    /* Print the instance number */
  switch (signalInfoPtr->s.channel)
  {
   case PLX_INPUT_0:
    sprintf(channelString, "Input 1");
    break;
   case PLX_INPUT_1:
    sprintf(channelString, "Input 2");
    break;
   case PLX_IO_OFF:
    sprintf(channelString, "(Off)");
    break;
  }
  switch (signalInfoPtr->s.format)				    /* Print the appropriate label on the popup menu cascade... */
  {								    /* ...button, according to signal format in effect. */
   case PLX_COMP:						    /* One wire: composite video */
    sprintf(formatString, "Composite");
    break;
   case PLX_YC:							    /* Two wires: S-Video */
    sprintf(formatString, "S-VHS (YC)");
    break;
   case PLX_RGB:						    /* Three wires: Red-Green-Blue video */
    sprintf(formatString, "RGB");
    break;
   case PLX_YUV:						    /* Three wires: YUV video */
    sprintf(formatString, "YUV");
    break;
  }
  switch (signalInfoPtr->s.standard)				    /* Print the appropriate label on the popup menu cascade... */
  {								    /* ...button, according to the actual signal standard... */
   case PLX_NTSC:						    /* ...in effect. */
    sprintf(standardString, "NTSC");
    break;
   case PLX_PAL:
    sprintf(standardString, "PAL");
    break;
   case PLX_SECAM:
    sprintf(standardString, "SECAM");
    break;
  }
  if (True == sync)
    sprintf(syncString, " ");
  else
    sprintf(syncString, " (No Sync)");
  sprintf(inputStatusString, "%s : %s : %s : %s%s",
	  channelString,
	  formatString,
	  instanceString,
	  standardString,
	  syncString);
  label = XmStringCreateLocalized(inputStatusString);		    /* Create the string that will be used as a label for the... */
  XtVaSetValues(inputSignalTypeLabel, XmNlabelString, label, NULL); /* ...Label widget showing the new input information. */
  XmStringFree(label);
  return;
} /* end function UpdateInputTypeString */
