/* $Header: /plx/source/Xclients/generic/RCS/video.c,v 1.41 1995/11/23 03:57:37 drapeau Exp $ */
/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Log: video.c,v $
 * Revision 1.41  1995/11/23  03:57:37  drapeau
 * Minor changes to eliminate compiler warnings.
 *
 * Revision 1.40  1995/10/27  17:56:31  drapeau
 * Updated the imageQualitySlider to reflect the initial Q factor of the
 * imageQuality widget.
 *
 * Revision 1.39  1995/10/25  00:38:15  drapeau
 * Check for hardware zoom capability and disable the Zoom cascade menu
 * if not available.
 *
 * Revision 1.38  1995/10/11  02:26:51  drapeau
 * (From Paul Lovvik) Fixed an error in the way video widgets resize
 * under the OPEN LOOK Window Manager.  The code assures that a 4-to-3
 * aspect ratio is maintained, which has the side effect of assuring that
 * live video windows always remain the full height they should be, as
 * opposed to part of the full height because the window manager is not
 * honoring resize requests.
 *
 * Revision 1.37  1995/10/02  21:56:46  drapeau
 * Added support for detecting whether JPEG hardware is available.
 *
 * Revision 1.36  1995/08/23  01:58:48  drapeau
 * Added support for an additional diagnostic: counting the time passed
 * between successive audio reads.
 *
 * Revision 1.35  1995/06/23  19:18:50  drapeau
 * Fixed an error in the way zoom factor was selected and displayed on
 * the user interface.  When the application launches, in now correctly
 * senses the current zoom factor and sets the correct toggle in the user
 * interface.
 *
 * Revision 1.34  1995/05/16  02:59:07  drapeau
 * Fixed the way the number of digitizers are calculated, both for input
 * and output.
 *
 * Revision 1.33  1995/05/02  17:48:09  drapeau
 * Added support for hardware zoom when available.
 *
 * Revision 1.32  1994/10/10  23:03:02  drapeau
 * Minor changes, updating global variables for input signal type when
 * appropriate and desensitizing YC and RGB input menu items if only a
 * single digitizer is available (i.e., the VIO option or PowerVideoPlus
 * option is not present).
 *
 * Revision 1.31  1994/09/30  16:58:29  drapeau
 * Added support for output (queried the hardware to determine whether
 * output is available).
 *
 * Also, made the "imageQualityImage" jpegData resource a global variable
 * so it can be seen by other modules in this application.
 *
 * Revision 1.30  1994/09/13  08:12:00  drapeau
 * Removed extraneous variables.
 *
 * Revision 1.29  1994/08/30  00:13:47  drapeau
 * Fixed the way the Image Quality Widget was created, using the width
 * and height of the source widget's window instead of the video input's
 * signal width and height (they will normally be slightly different, and
 * in the future when it's possible to record any window it will be more
 * correct to use the window's dimensions rather thna the video signal's
 * dimensions).
 *
 * Revision 1.28  1994/08/23  01:35:53  drapeau
 * Added code for an alert dialog box.
 *
 * Revision 1.27  1994/08/11  03:17:54  drapeau
 * Added code to support enhanced diagnostics.
 *
 * Revision 1.26  1994/06/10  01:27:32  drapeau
 * Minor updates to reflect a new signal recording rate when switching
 * signal standards (i.e., from PAL to NTSC).
 *
 * Revision 1.25  1994/05/31  20:15:44  drapeau
 * Changed allocation of memory for diagnostic and synchronization
 * reporting to reflect new floating point math being used.  The old
 * scheme used integer math, which caused rounding errors several orders
 * of magnitude greater than the new scheme.
 *
 * Revision 1.24  1994/01/20  18:06:22  drapeau
 * Added code to support the case where the JPEG widget cannot use shared
 * memory.
 *
 * Revision 1.23  1994/01/10  22:42:07  drapeau
 * Minor cosmetic changes only; nothing significant here.
 *
 * Revision 1.22  1993/12/04  01:03:10  drapeau
 * Modified the code to make the Pause feature more robust.  Before this
 * version, pausing a movie during playback would throw off the
 * synchronization mechanism; pause now works correctly.
 *
 * Revision 1.21  1993/12/03  02:13:52  drapeau
 * Minor cosmetic change, mostly to allow me to change the RCS numbering
 * scheme for this file.
 *
 * Revision 1.2  1993/12/03  01:39:49  drapeau
 * Fixed the initialization of the image quality JPEG widget;
 * also added additional init code for more diagnostics.
 * */
static char videoRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/video.c,v 1.41 1995/11/23 03:57:37 drapeau Exp $";

#include "MovieTool.h"

void InitializeXVideo(void)
{
  Dimension		signalWidth;
  Dimension		signalHeight;
  Dimension		videoWidgetWidth;
  Dimension		videoWidgetHeight;
  int			qFactor;
  plx_IO*		hardwareInfo;				    /* Info about the Parallax board being used */
  plx_IO_list*		outputList = (plx_IO_list*) NULL;
  
  zoomFactor = XPlxGetZoomFactor(XtDisplay(mainWindow),		    /* Inquire the current zoom factor for this connection */
				 XtWindow(mainWindow));
  if (2 == zoomFactor)						    /* Is zoom on? */
  {
    XmToggleButtonSetState(zoom2XToggle, True, False);		    /* Yes, reflect that in the interface */
    XmToggleButtonSetState(zoom1XToggle, False, False);		    /* Explicitly turn off the zoom 1 toggle */
  }
  input1Widget = XtVaCreateManagedWidget("input1Widget",
					 xVideoInWidgetClass,
					 input1Shell,
					 NULL);
  XtVaSetValues(input1Shell,					    /* Make sure that the input shell conforms to a 4-to-3... */
		XtNminAspectX, 4,				    /* ...aspect ratio.  This is necessary for dealing with... */
		XtNminAspectY, 3,				    /* ...the OPEN LOOK window manager (the Motif window... */
		XtNmaxAspectX, 4,				    /* ...manager doesn't require this restriction since it... */
		XtNmaxAspectY, 3,				    /* ...listens to the XVideoIn widget's resize requests). */
		NULL);
  XtVaGetValues(input1Widget,					    /* Determine the signal standard for this video input, so... */
		XtNsignalStandard, &inputSignalStandard,	    /* ...the frame rate for this standard can be determined. */
		XtNsignalFormat, &inputSignalFormat,
		XtNinstance, &inputInstance,
		XtNsignalWidth, &signalWidth,
		XtNsignalHeight, &signalHeight,
		XtNhardwareParams, &hardwareInfo,
		NULL);
  outputList = hardwareInfo->outputs;				    /* Point to the list of output information for this board */
  if (outputList != (plx_IO_list*)NULL)				    /* Does output exist for this hardware? */
  {								    /* Yes, do the appropriate thing */
    outputAvailable = True;
  }
  if (True == outputAvailable)					    /* Is the output option available? */
  {
    XtVaSetValues(outputOptionCascade, XmNsensitive, True, NULL);   /* Yes, allow it to be selected */
  }
  else								    /* No, don't allow output to be selected */
  {
    XtVaSetValues(outputOptionCascade, XmNsensitive, False, NULL);
  }
  if (0 == hardwareInfo->zoom)					    /* Does this product lack the ability to zoom? */
  {
    XtVaSetValues(zoomCascade, XmNsensitive, False, NULL);	    /* Yes, no zoom, disable the appropriate interface component */
  }
  XtVaSetValues(input1Widget,					    /* Make the video widget quarter-size by default */
		XtNwidth, (signalWidth / 2),
		XtNheight, (signalHeight / 2),
		NULL);
  if (inputSignalStandard == PLX_NTSC)				    /* Is it an NTSC signal? */
  {
    signalFrameRate = 30;					    /* Yep, frame rate is 30 Hz */
    recordFrameRate = 30;					    /* Set an initial rate for recording NTSC movies */
    XtVaSetValues(recordSpeedSlider, XmNvalue, 30, NULL);
  }
  else								    /* Nope, must be PAL or SECAM; frame rate is 25 Hz */
  {
    signalFrameRate = 25;
    recordFrameRate = 25;					    /* Set an initial rate for recording PAL or SECAM movies */
    XtVaSetValues(recordSpeedSlider, XmNvalue, 25, NULL);
  }
  XtVaGetValues(input1Widget,
		XtNwidth, &videoWidgetWidth,
		XtNheight, &videoWidgetHeight,
		NULL);
  imageQualityRect.x = 0;
  imageQualityRect.y = 0;
  imageQualityRect.width = 160;
  imageQualityRect.height = 120;
  imageQualitySourceRect.x = 0;
  imageQualitySourceRect.y = 0;
  imageQualitySourceRect.width = videoWidgetWidth;
  imageQualitySourceRect.height = videoWidgetHeight;
  imageQualityWidget = XtVaCreateManagedWidget("imageQualityWidget", /* Create a JPEG widget to show image quality for... */
					       jpegWidgetClass,	    /* ...varying Q factors. */
					       imageQualityFrame,
					       XtNwidth, 160,
					       XtNheight, 120,
					       XtNcompressSourceRect, &imageQualitySourceRect,
					       XtNcompressDestRect, &imageQualitySourceRect,
					       XtNdisplayDestRect, &imageQualityRect,
					       NULL);
  playbackWidget = XtVaCreateManagedWidget("playbackWidget",	    /* Create a widget for showing JPEG movies */
					   jpegWidgetClass,
					   movieShell,
					   XtNwidth,		    /* Make it fit a quarter-size video widget initially */
					   ((signalWidth / 2) * zoomFactor),
					   XtNheight,
					   ((signalHeight / 2) * zoomFactor),
					   NULL);
  XtVaGetValues(playbackWidget,					    /* Point to the shared memory segment used by this widget */
		XtNjpegData, &jpegImage,
		XtNuseSharedMemory, &usingSharedMemory,		    /* Determine whether the JPEG widget is using shared memory */
		XtNjpegAvailable, &jpegAvailable,		    /* Determine whether this connection has JPEG hardware */
		NULL);
  if (usingSharedMemory == False)				    /* If the JPEG widget isn't using shared memory, allocate... */
  {								    /* ...enough memory to store images for playback. */
    jpegData = malloc(OneImageSizeInBytes);			    /* Try to allocate the necessary memory */
    if (jpegData == NULL)					    /* Did the allocation fail? */
    {								    /* Yes, report the error and quit this application now */
      PrintErrorMessage("Could not allocate enough memory to store JPEG images.  Exiting.\n");
      exit(-1);
    }
  }
  XtVaGetValues(imageQualityWidget,				    /* Point to the shared memory segment used by the... */
		XtNjpegData, &imageQualityImage,		    /* ...image quality JPEG widget. */
		XtNqFactor, &qFactor,
		NULL);
  XtVaSetValues(imageQualitySlider, XmNvalue, qFactor, NULL);	    /* Initialize the slider's value to that of the JPEG widget */
  movieToolStatus = Stopped;					    /* When the app starts, no movie is playing */
  audioData = NULL;						    /* When the app starts, there is no audio buffer */
  recordFrameIndex = (int*)malloc(MaxMovieDurationInFrames *	    /* Allocate space to hold frame start points for... */
				  sizeof(int));			    /* ...recording movies. */
  skippedFrameIndex = (float*)malloc(MaxMovieDurationInFrames *	    /* Allocate space for performance stats: which frames in... */
				     sizeof(float));		    /* ...this movie were skipped during playback. */
  memset(skippedFrameIndex, 0, MaxMovieDurationInFrames *	    /* Initialize the skipped frames index before playback */
	 sizeof(float));
  recordSkippedFrameIndex = (float*)malloc(MaxMovieDurationInFrames /* Allocate space for performance stats: which frames in... */
				   * sizeof(float));		    /* ...this movie were skipped during recording. */
  memset(recordSkippedFrameIndex, 0, MaxMovieDurationInFrames *	    /* Initialize the skipped frames index before recording */
	 sizeof(float));
  audioReadTime = (long*)malloc(MaxMovieDurationInFrames *	    /* Allocate space for performance stats: how long each... */
				   sizeof(long));		    /* ...audio frame in this movie took to read. */
  memset(audioReadTime, 0, MaxMovieDurationInFrames *		    /* Initialize the audio read time array before recording */
	 sizeof(long));
  audioBetweenTime = (long*)malloc(MaxMovieDurationInFrames *	    /* Allocate space for performance stats: how much time... */
				   sizeof(long));		    /* ...passed between each frame of audio is read. */
  memset(audioBetweenTime, 0, MaxMovieDurationInFrames *	    /* Initialize the audio between time array before recording */
	 sizeof(long));
  audioWriteTime = (long*)malloc(MaxMovieDurationInFrames *	    /* Allocate space for performance stats: how long each... */
				   sizeof(long));		    /* ...audio frame in this movie took to write (playback). */
  memset(audioWriteTime, 0, MaxMovieDurationInFrames *		    /* Initialize the audio write time array before playback */
	 sizeof(long));
  compressTime = (long*)malloc(MaxMovieDurationInFrames *	    /* Allocate space for performance stats: how long each... */
				   sizeof(long));		    /* ...video frame in this movie took to compress. */
  memset(compressTime, 0, MaxMovieDurationInFrames *		    /* Initialize the video compress time array before recording */
	 sizeof(long));
  diskWriteTime = (long*)malloc(MaxMovieDurationInFrames *	    /* Allocate space for performance stats: how long each... */
				   sizeof(long));		    /* ...video frame in this movie took to write to disk. */
  memset(diskWriteTime, 0, MaxMovieDurationInFrames *		    /* Initialize the disk write time array before recording */
	 sizeof(long));
  playInterval = (long*)malloc(MaxMovieDurationInFrames *	    /* Allocate space for performance stats: how long b/w... */
				   sizeof(long));		    /* ...each frame in the movie was played. */
  memset(playInterval, 0, MaxMovieDurationInFrames *		    /* Initialize the playback interval array before playback */
	 sizeof(long));
  sprintf(outputFileName, "%s/jpegMovie", getenv("HOME"));	    /* Choose a better default file name for recording movies */
  currentMovieFileName = malloc(MAXPATHLEN);			    /* Allocate space to store a filename for playing movies */
  strcpy(currentMovieFileName, "Untitled");			    /* Tell MovieTool it's currently not playing any movie */
  SetMovieStatus(MovieNew);					    /* Update interface to reflect change in movie file status */
  resetPlaybackClock = True;					    /* Tell the playback timer to reset itself */
  recordAudioFlag = True;					    /* By default, assume authors want sound in their movies */
  totalTimePaused = 0;						    /* No pause time has elapsed before anything has been done */
  outputOn = False;						    /* The output option is currently disabled */
}								    /* end function InitializeXVideo */
