/* $Header: /plx/source/Xclients/generic/RCS/aboutBox.c,v 1.5 1995/12/14 17:27:10 lovvik Exp $ */

/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */

/* $Log: aboutBox.c,v $
 * Revision 1.5  1995/12/14  17:27:10  lovvik
 * I added a function called CreateBWLogoPixmap() that creates
 * a black&white version of the Parallax logo and puts that on
 * the main panel.
 *
 * Revision 1.4  1995/11/20  20:38:23  lovvik
 * Removed the small Parallax logo from the main window.
 *
 * Revision 1.3  1995/11/01  02:57:11  lovvik
 * Changed the logo on the main RTMovieTool panel from the full-size color
 * logo to the small black and white one.
 *
 * Revision 1.2  1995/10/12  17:48:07  lovvik
 * Added a line of code to place the Parallax logo in a label
 * widget on the front panel of RTMovieTool.
 *
 * Revision 1.1  1995/10/02  23:36:55  lovvik
 * Initial revision
 * */

static char movieToolRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/aboutBox.c,v 1.5 1995/12/14 17:27:10 lovvik Exp $";

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>

#include <xpm.h>

#include "interface.h"
#include "globals.h"
#include "structs.h"
#include "prototypes.h"
#include <icon/.plxlogo>
#include <icon/plxSmallBWLogo.xpm>

void InitializeAboutBox(void)
{
  /*
   * This function places a parallax logo pixmap in
   * the about box.  Note that this must be done
   * after the main dialog box is realized.  To
   * accomplish this, we will set up a work proceedure.
   */
  XtAppAddWorkProc(app_context, CreateLogoWorkProc, NULL);
}

Boolean CreateLogoWorkProc(XtPointer unusedData)
{
  if(XtIsRealized(mainWindow))
  {
    CreateLogoPixmap(aboutDialog);
    CreateBWLogoPixmap(logoLabel);
    return(True);
  }
  else
  {
    return(False);
  }
}


void CreateLogoPixmap(Widget	parentWidget)			    /* Creates a masked Pixmap in shape of the company logo... */
{								    /* ...to display in the application's "About..." panel. */

  unsigned long	background;
  Display*	display = XtDisplay(parentWidget);
  int		depth;
  XGCValues	gcValues;
  Pixmap	logoPixmap;
  Pixmap	logoShapeMask;
  XpmAttributes	logoAttrs;
  GC		logoGC;
  int		result;
  Pixmap	thePixmap;
  Window	window = XtWindow(aboutShell);
  
  logoAttrs.valuemask = 0;
  result = XpmCreatePixmapFromData(display,			    /* Attempt to create a Pixmap from XPM data */
				   window,			    /* The function, if successful, will return both a Pixmap... */
				   plxLogoData,			    /* ...in the shape of the company logo, in color, plus... */
				   &logoPixmap,			    /* ...a mask that can be used to show only the Pixmap... */
				   &logoShapeMask,		    /* ...data (as if it were transparent). */
				   &logoAttrs);
  if (result != 0)						    /* Did the Pixmap creation fail? */
    return;							    /* Yes, don't use any Pixmap for the logo panel */
  XtVaGetValues(parentWidget,					    /* Get the depth and background color of the parent widget */
		XmNdepth, &depth,
		XmNbackground, &background,
		NULL);
  thePixmap = XCreatePixmap(display,				    /* Create a new Pixmap to be used as the application's logo */
			    window,
			    logoAttrs.width, logoAttrs.height,
			    depth);
  gcValues.background = background;
  gcValues.foreground = background;
  logoGC = XCreateGC(display,					    /* Create a GC that paints the application's background... */
		     window,					    /* ...color into the final logo Pixmap */
		     (GCBackground | GCForeground),
		     &gcValues);
  XFillRectangle(display, thePixmap, logoGC, 0, 0,		    /* Paint the bkgnd color into the final logo Pixmap */
		 logoAttrs.width, logoAttrs.height);
  XSetClipMask(display, logoGC, logoShapeMask);			    /* Make sure that painting the new, color logo data only... */
  XCopyArea(display, logoPixmap, thePixmap, logoGC,		    /* ...paints into the areas it's supposed to, making... */
	    0, 0, logoAttrs.width, logoAttrs.height,		    /* ...a "transparent" effect. */
	    0, 0);
  XtVaSetValues(parentWidget, XmNsymbolPixmap, thePixmap, NULL);    /* Use the newly-created icon as the About panel's symbol */
  XFreeGC(display, logoGC);
  return;

} /* end function CreateLogoPixmap */


void CreateBWLogoPixmap(Widget	parentWidget)			    /* Creates a masked Pixmap in shape of the company logo... */
{								    /* ...to display in the application's main panel. */

  unsigned long	background;
  Display*	display = XtDisplay(parentWidget);
  int		depth;
  XGCValues	gcValues;
  Pixmap	bwLogoPixmap;
  Pixmap	bwLogoShapeMask;
  XpmAttributes	bwLogoAttrs;
  GC		logoGC;
  int		result;
  Pixmap	bwPixmap;
  Window	window = XtWindow(parentWidget);
  
  bwLogoAttrs.valuemask = 0;
  result = XpmCreatePixmapFromData(display,			    /* Attempt to create a Pixmap from XPM data */
				   window,			    /* The function, if successful, will return both a Pixmap... */
				   bwPlxLogoData,		    /* ...in the shape of the company logo, in black and white, plus... */
				   &bwLogoPixmap,		    /* ...a mask that can be used to show only the Pixmap... */
				   &bwLogoShapeMask,		    /* ...data (as if it were transparent). */
				   &bwLogoAttrs);
  if (result != 0)						    /* Did the Pixmap creation fail? */
    return;							    /* Yes, don't use any Pixmap for the logo panel */
  XtVaGetValues(parentWidget,					    /* Get the depth and background color of the parent widget */
		XmNdepth, &depth,
		XmNbackground, &background,
		NULL);
  bwPixmap = XCreatePixmap(display,				    /* Create a new Pixmap to be used as the application's logo */
			   window,
			   bwLogoAttrs.width, bwLogoAttrs.height,
			   depth);
  gcValues.background = background;
  gcValues.foreground = background;
  logoGC = XCreateGC(display,					    /* Create a GC that paints the application's background... */
		     window,					    /* ...color into the final logo Pixmap */
		     (GCBackground | GCForeground),
		     &gcValues);
  XFillRectangle(display, bwPixmap, logoGC, 0, 0,		    /* Paint the bkgnd color into the final logo Pixmap */
		 bwLogoAttrs.width, bwLogoAttrs.height);
  XSetClipMask(display, logoGC, bwLogoShapeMask);		    /* Make sure that painting the new, color logo data only... */
  XCopyArea(display, bwLogoPixmap, bwPixmap, logoGC,		    /* ...paints into the areas it's supposed to, making... */
	    0, 0, bwLogoAttrs.width, bwLogoAttrs.height,	    /* ...a "transparent" effect. */
	    0, 0);
  XtVaSetValues(logoLabel, XmNlabelPixmap, bwPixmap, NULL);	    /* Use the newly-created icon as the About panel's symbol */
  XFreeGC(display, logoGC);
  return;

} /* end function CreateLogoPixmap */
