/* $Header: /plx/source/Xclients/generic/RCS/audio.c,v 1.3 1995/11/15 02:57:21 lovvik Exp $ */

/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */

/* $Log: audio.c,v $
 * Revision 1.3  1995/11/15  02:57:21  lovvik
 * Added error code.
 *
 * Revision 1.2  1995/11/01  03:11:10  lovvik
 * Fixed compiler warnings.
 *
 * Revision 1.1  1995/10/02  23:37:21  lovvik
 * Initial revision
 * */

static char movieToolRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/audio.c,v 1.3 1995/11/15 02:57:21 lovvik Exp $";

/* Module for the Audio dialog */
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <X11/StringDefs.h>
#include <Xm/CascadeB.h>
#include <Xm/DialogS.h>
#include <Xm/FileSB.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/List.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/Scale.h>
#include <Xm/ScrollBar.h>
#include <Xm/SelectioB.h>
#include <Xm/Separator.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
#include <Xm/CascadeBG.h>
#include <Xm/LabelG.h>
#include <Xm/PushBG.h>
#include <Xm/SeparatoG.h>
#include <Xm/ToggleBG.h>
#include <stdio.h>

#include "interface.h"
#include "structs.h"
#include "globals.h"
#include "prototypes.h"


static AudioStruct audio;

void InitializeAudioDialog(void)
{
  int audioAvailable;
  
  audio.outputDevice = 0;
  audio.inputDevice =  0;
  audio.monitorUserSetting = ON;
  audio.monitorAppSetting = ON;
  audio.audioVolume.playback = 0;
  audio.audioVolume.record = 0;
  audio.audioVolume.monitor = 0;

  /*
   * If high quality audio is available, then audio on is
   * the default.
   */
  if(!RTVAudioAvailable(&audioAvailable))
  {
    PrintDiagnostics("Could not request audio status from the rtv server.");
    audioAvailable = False;
  }
  if(!audioAvailable)
  {
    PrintDiagnostics("Audio not available.");
    SetAudioNotAvailable();
    audio.audioOn = False;
  }
  else
  {
    PrintDiagnostics("Audio is available.");
    SunAudioInitializeAudio();
    audio.audioOn = True;
  }
}


int GetOutputDevice(void)
{
  return(audio.outputDevice);
}


void SetOutputDevice(int newOutputDevice)
{
  newOutputDevice = newOutputDevice &				    /* Make sure output device is correct. */
                    (SPEAKER | HEADPHONES | LINE_OUT);

  audio.outputDevice = newOutputDevice;				    /* . . . update the structure . . . */
  SunAudioSetOutput(newOutputDevice);
}


int GetInputDevice(void)
{
  return(audio.inputDevice);
}


void SetInputDevice(int newInputDevice)
{
  newInputDevice = newInputDevice &				    /* Make sure input device is correct. */
    (LINE_IN | MICROPHONE);

  if(newInputDevice != 0)					    /* If the input device was valid . . . */
  {
    audio.inputDevice = newInputDevice;			    	    /* . . . update the structure . . . */
  }
  else								    /* . . . otherwise, print an error message. */
  {
    PrintDiagnostics("Warning!  Selected input device is not valid!");
    return;
  }
  SunAudioSetInput(newInputDevice);
}


/*
 * The monitor user setting is the state that the user puts the
 * monitor function in.  If the user turns off the monitor, this
 * value should be False.  If the user turns on the monitor, this
 * value should be True.  The application will modify the monitor
 * output state also.  The application should turn it on only if
 * the user setting is True.
 */

int GetMonitorUserSetting(void)
{
  return(audio.monitorUserSetting);
}


void SetMonitorUserSetting(int newSetting)
{
  if(newSetting == True)
  {
    audio.monitorUserSetting = True;
    SunAudioMonitorOn();
    XtSetSensitive(monitorScale, True);
  }
  else
  {
    audio.monitorUserSetting = False;
    SunAudioMonitorOff();
    XtSetSensitive(monitorScale, False);
  }
  return;
}


int GetMonitorAppSetting(void)
{
  return(audio.monitorAppSetting);
}


void SetMonitorAppSetting(int newSetting)
{
  if(newSetting == True)
  {
    audio.monitorAppSetting = True;
    if(GetMonitorUserSetting() == True)
    {
      SunAudioMonitorOn();
      XmToggleButtonSetState(monitorOnToggle, True, False);
      XmToggleButtonSetState(monitorOffToggle, False, False);
    }
  }
  else
  {
    audio.monitorAppSetting = False;
    SunAudioMonitorOff();
    XmToggleButtonSetState(monitorOnToggle, False, False);
    XmToggleButtonSetState(monitorOffToggle, True, False);
  }
  return;
}    


int GetAudioOnStatus(void)
{
  return(audio.audioOn);
}


void TurnAudioOn(void)
{
  if(audio.audioOn == False)
  {
    audio.audioOn = True;
  }
  RTVAudioOn();
}


void TurnAudioOff(void)
{
  if(audio.audioOn == True)
  {
    audio.audioOn = False;
  }
  
  /*
   * If the state is being set by the application (not the
   * user), we need to set the toggle buttons to reflect this
   * state.
   */
     
  if(XmToggleButtonGetState(audioOnToggle))
  {
    XmToggleButtonSetState(audioOnToggle, False, False);
    XmToggleButtonSetState(audioOffToggle, True, False);
  }

  /*
   * Tell the rtv server not to record or play audio.
   */
  RTVAudioOff();
}


int GetPlaybackVolume(void)
{
  return(audio.audioVolume.playback);
}


void SetPlaybackVolume(int percentVolume)
{
  audio.audioVolume.playback = percentVolume;
  SunAudioSetPlayVolume(percentVolume);
}


int GetRecordVolume(void)
{
  return(audio.audioVolume.record);
}


void SetRecordVolume(int percentVolume)
{
  audio.audioVolume.record = percentVolume;
  SunAudioSetRecordVolume(percentVolume);
}


int GetMonitorVolume(void)
{
  return(audio.audioVolume.monitor);
}


void SetMonitorVolume(int percentVolume)
{
  audio.audioVolume.monitor = percentVolume;
  if(GetMonitorAppSetting() == True)				    /* If Monitor is on, change the volume. */
  {
    SunAudioSetMonitorVolume(percentVolume);
  }
}

void DisplayPlayVolume(int percentVolume)
{
  if(audio.audioVolume.playback != percentVolume)
  {
    audio.audioVolume.playback = percentVolume;
    XmScaleSetValue(playbackScale, percentVolume);
  }
}


void DisplayRecordVolume(int percentVolume)
{
  if(audio.audioVolume.record != percentVolume)
  {
    audio.audioVolume.record = percentVolume;
    XmScaleSetValue(recordScale, percentVolume);
  }
}


void DisplayMonitorVolume(int percentVolume)
{
  if(audio.audioVolume.monitor != percentVolume)
  {
    audio.audioVolume.monitor = percentVolume;
    XmScaleSetValue(monitorScale, percentVolume);
  }
}


void DisplayOutputDevice(int newOutputDevice)
{
  newOutputDevice = newOutputDevice &				    /* Make sure output device is correct. */
                    (SPEAKER | HEADPHONES | LINE_OUT);

  if(newOutputDevice != 0)					    /* If the output device was valid . . . */
  {

    audio.outputDevice = newOutputDevice;			    /* . . . update the structure . . . */
    if(audio.outputDevice & SPEAKER)
    {
      XmToggleButtonSetState(playbackSpeakerToggle, True, False);
    }
    else
    {
      XmToggleButtonSetState(playbackSpeakerToggle, False, False);
    }
      
    if(audio.outputDevice & HEADPHONES)
    {
      XmToggleButtonSetState(playbackHeadphonesToggle, True, False);
    }
    else
    {
      XmToggleButtonSetState(playbackHeadphonesToggle, False, False);
    }	
      
    if(audio.outputDevice & LINE_OUT)
    {
      XmToggleButtonSetState(playbackLineOutToggle, True, False);
    }
    else
    {
      XmToggleButtonSetState(playbackLineOutToggle, False, False);
    }
  }
  else								    /* . . . otherwise, print an error message. */
  {
    PrintDiagnostics("Warning!  Selected output device is not valid!");
    return;
  }
}


void DisplayInputDevice(int newInputDevice)
{
  newInputDevice = newInputDevice &				    /* Make sure input device is correct. */
                    (LINE_IN | MICROPHONE);

  if(newInputDevice != 0)					    /* If the input device was valid . . . */
  {
    if(audio.inputDevice != newInputDevice) 
    {
      audio.inputDevice = newInputDevice;			    /* . . . update the structure . . . */
      if(audio.inputDevice & LINE_IN)
      {
	XmToggleButtonSetState(recordLineInToggle, True, False);
      }
      else
      {
	XmToggleButtonSetState(recordLineInToggle, False, False);
      }
      
      if(audio.inputDevice & MICROPHONE)
      {
	XmToggleButtonSetState(recordMicInToggle, True, False);
      }
      else
      {
	XmToggleButtonSetState(recordMicInToggle, False, False);
      }	
    }
  }
  else								    /* . . . otherwise, print an error message. */
  {
    PrintDiagnostics("Warning!  Selected input device is not valid!");
    return;
  }
}


void UnmapSpeakerToggle(void)
{
  XtUnmanageChild(playbackSpeakerToggle);
}


void UnmapHeadphonesToggle(void)
{
  XtUnmanageChild(playbackHeadphonesToggle);
}


void UnmapLineOutToggle(void)
{
  XtUnmanageChild(playbackLineOutToggle);
}

void UnmapMicrophoneToggle(void)
{
  XtUnmanageChild(recordMicInToggle);
}


void UnmapLineInToggle(void)
{
  XtUnmanageChild(recordLineInToggle);
}

void SetAudioOutputOneOnly(void)
{
  XtVaSetValues(playbackRadioBox,
		XmNradioBehavior, True,
		NULL);
  XtVaSetValues(playbackSpeakerToggle,
		XmNindicatorType, XmONE_OF_MANY,
		NULL);

  XtVaSetValues(playbackHeadphonesToggle,
		XmNindicatorType, XmONE_OF_MANY,
		NULL);

  XtVaSetValues(playbackLineOutToggle,
		XmNindicatorType, XmONE_OF_MANY,
		NULL);
}


void UpdateAudioControls(void)
{
  /*
   * Update the controls right now by flushing all requests that
   * have been buffered.
   */
  switch(GetMovieToolStatus())
  {
    case MT_RECORD:
    case MT_PLAY:
    {
      /*
       * The app will hang if the volumes are modified during
       * play or record.
       */
      break;
    }

    default:
    {
      XFlush(XtDisplay(mainWindow));      
    }
  }
}


void SetAudioNotAvailable(void)
{
  /*
   * First, indicate that audio is not active by setting the audio
   * on/off toggle buttons.  Look at the current value to see if
   * it needs to be changed.
   */
  if(XmToggleButtonGetState(audioOnToggle))
  {
    XmToggleButtonSetState(audioOnToggle, False, False);
    XmToggleButtonSetState(audioOffToggle, True, False);
  }
  
  /*
   * Prevent the user from trying to manipulate the audio device by
   * disabling the entire audio dialog box.
   */
  XtSetSensitive(audioForm, False);
}
