/* $Header: /plx/source/Xclients/sun4.ow3/lovvik/src/client/motif/RTMovieTool/RCS/callbacks.c,v 1.9 1996/01/30 20:38:18 lovvik Exp $ */

/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */

/* $Log: callbacks.c,v $
 * Revision 1.9  1996/01/30  20:38:18  lovvik
 *   Cleaned the ToggleAudioOnStatus() code.  Added
 *   some synchronization code to PlayWorkProc() and
 *   Pause().  Also added some callbacks for the new
 *   movie editing feature.
 *
 * Revision 1.8  1996/01/11  18:06:05  lovvik
 *   Removed functions specific to output.  The output
 *   callback is now in video.c.
 *
 * Revision 1.7  1996/01/04  18:53:09  lovvik
 * Added code to OpenMovieWorkProc() that correctly sets up
 * the video to support the movie that is to be opened.  That
 * is, if you want to play a movie recorded from a PAL source,
 * this code will set the video input to PAL so the movie can
 * be played.  Also, added code to OpenMovie() that pops down
 * the video window, and added code to SaveMovie() that pops
 * down the playback window.
 *
 * Revision 1.6  1995/11/15  02:56:15  lovvik
 * Added WMDeleteWindowCallback().  This function is called when the
 * quit function is called from the window manager decoration.
 *
 * Revision 1.5  1995/11/01  02:40:07  lovvik
 * Added error code to OpenMovie() and SaveMovie().
 *
 * Revision 1.4  1995/10/12  20:07:03  lovvik
 * Added record syncronization code.  Added a call to RTVWindowPlayPrepare()
 * and RTVWindowRecordPrepare().  Also, added support for field mode.
 *
 * Revision 1.3  1995/10/11  17:03:07  lovvik
 * Added a conditional around a call to StopVideo() in the
 * Play() function.  This is to provide support for Rtv in
 * a server that does not yet have Rtv functionality.
 *
 * Revision 1.2  1995/10/06  17:11:00  lovvik
 * Removed comments around StopVideo() in the Play() function.
 * This was a Kludge that was used until server support for
 * Rtv existed.
 *
 * Revision 1.1  1995/10/02  23:37:49  lovvik
 * Initial revision
 * */

static char movieToolRcsid[] = "$Header: /plx/source/Xclients/sun4.ow3/lovvik/src/client/motif/RTMovieTool/RCS/callbacks.c,v 1.9 1996/01/30 20:38:18 lovvik Exp $";

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/CascadeB.h>
#include <Xm/FileSB.h>
#include <Xm/PushB.h>
#include <Xm/Scale.h>
#include <Xm/SelectioB.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
#include <Xm/ToggleBG.h>
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>

#include "globals.h"
#include "interface.h"
#include "structs.h"
#include "prototypes.h"

static void SynchronizationTimeout(XtPointer, XtIntervalId *);
Bool XPlxIsRTVFile(char *);

void XDmanage_link (Widget w, XtPointer client_data, XtPointer call_data)
{
  XtManageChild ( *(Widget *)client_data );
}

void XDunmanage_link (Widget w, XtPointer client_data, XtPointer call_data)
{
  XtUnmanageChild ( *(Widget *)client_data );
}

void XDpopup_link (Widget w, XtPointer client_data, XtPointer call_data)
{
  XtPopup ( *(Widget *)client_data, XtGrabNone );
}

void XDpopdown_link (Widget w, XtPointer client_data, XtPointer call_data)
{
  XtPopdown ( *(Widget *)client_data );
}

void XDmap_link (Widget w, XtPointer client_data, XtPointer call_data)
{
  XtMapWidget (*(Widget *)client_data);
}

void XDunmap_link (Widget w, XtPointer client_data, XtPointer call_data)
{
  XtUnmapWidget (*(Widget *)client_data);
}

void XDenable_link (Widget w, XtPointer client_data, XtPointer call_data)
{
  XtSetSensitive (*(Widget *)client_data, TRUE);
}

void XDdisable_link (Widget w, XtPointer client_data, XtPointer call_data)
{
  XtSetSensitive (*(Widget *)client_data, FALSE);
}

extern void XDhelp_link (Widget w, XtPointer client_data, XtPointer call_data);


void 
ShowOpenFileDialog (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  /*
   * Re-title the file dialog.  Since we are using only one file dialog for this
   * application, we must modify the title to make it clear to the user exactly
   * what is being chosen in the file dialog.
   */
  XtVaSetValues(fileDialog,
		XmNtitle, "Open",
		NULL);

  XtRemoveCallback(fileSelectionBox, XmNokCallback, SaveMovie, NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, OpenMovie, NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, SaveStill, NULL);
  
  XtAddCallback(fileSelectionBox, XmNokCallback, OpenMovie, NULL);

  /*
   * Do not allow the user to choose the movie output format.
   */
  XtUnmanageChild(saveAsOptionMenu); 
  XtVaSetValues(saveAsOptionButton, XmNsensitive, False, NULL);
  XtManageChild(fileSelectionBox);
}

void SaveMovie(Widget		w,
	       XtPointer	client_data,
	       XtPointer	xt_call_data)
{
  char*					fileName = NULL;
  char					message[100];
  Boolean				success;
  XmFileSelectionBoxCallbackStruct*	callData =
    (XmFileSelectionBoxCallbackStruct *) xt_call_data;
  
  success = XmStringGetLtoR(callData->value,
			   XmSTRING_DEFAULT_CHARSET,
			   &fileName);
  if (!success)
  {
    XtUnmanageChild(fileSelectionBox);
    if(fileName != NULL)
    {
      free(fileName);
    }
    return;
  }

  if(!CanRecordToFile(fileName))
  {
    sprintf(message, "Cannot record to %s.  Check permissions.", fileName);
    PrintError(message, ERROR_NONFATAL);
    free(fileName);
    return;
  }

  SetMovieFileName(fileName);
  if(fileName != NULL)
  {
    free(fileName);
  }

  XtUnmanageChild(fileSelectionBox);
  SetMovieLoaded(False);

  if(GetOutputStatus() == True)
  {
    StopOutput();
  }
  XtPopdown(movieShell);
  XtPopup(input1Shell, XtGrabNone);
  if(!GetLiveVideoStatus())
  {
    StartVideo();
  }
  UnlockVideoWindowSize();

  SetMovieToolTitle();
  SetMovieToolStatus(MT_IDLE);
  sprintf(message, "%s opened for recording.", GetMovieFileName());
  PrintDiagnostics(message);
  SetMovieLoaded(False);

  /*
   * Initialize RTV for recording to the new file.
   */
  RTVRecordInit(); 
  return;
} /* end function SaveMovie */


Boolean SaveStillWorkProc(XtPointer filename)
{
  Dimension				sourceWindowWidth;
  Dimension				sourceWindowHeight;

  if(!IsPlayWindowMapped())
  {
    /*
     * Window is not up yet.
     */
    return(False);
  }
    
  /*
   * Get the dimensions of the window.
   */
  XtVaGetValues(playbackWidget,
		XtNwidth, &sourceWindowWidth,
		XtNheight, &sourceWindowHeight,
		NULL);

  SaveImageFileInFormat(display,
			XtWindow(playbackWidget),
			sourceWindowWidth,
			sourceWindowHeight,
			GetImageFormat(),
			(char*)filename);
  return(True);
}


void SaveStill(Widget		w,
	       XtPointer	client_data,
	       XtPointer	xt_call_data)
{
  char*					fileName = NULL;
  char					message[100];
  Boolean				success;
  FILE *				imageFile;
  XmFileSelectionBoxCallbackStruct*	callData =
    (XmFileSelectionBoxCallbackStruct *) xt_call_data;
  
  success = XmStringGetLtoR(callData->value,
			   XmSTRING_DEFAULT_CHARSET,
			   &fileName);

  XtUnmanageChild(fileSelectionBox);
  if (!success)
  {
    if(fileName != NULL)
    {
      free(fileName);
    }
    return;
  }

  imageFile = fopen(fileName, "a+");
  if((FILE*)NULL == imageFile)
  {
    /*
     * The path and filename are not valid.
     */
    sprintf(message, "RTMovieTool cannot write to %s.  Check permissions.", fileName);
    PrintError(message, ERROR_NONFATAL);
    return;
  }
  fclose(imageFile);

  /*
   * Be sure that the window is on top and not iconified.
   */
  if(!IsPlayWindowMapped())
  {
    XtMapWidget(movieShell);
  }
  XtPopup(movieShell, XtGrabNone);
  XtAppAddWorkProc(app_context, SaveStillWorkProc, (XtPointer)fileName);
}



void 
ShowSaveFileDialog (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  /*
   * Re-title the file dialog.  Since we are using only one file dialog for this
   * application, we must modify the title to make it clear to the user exactly
   * what is being chosen in the file dialog.
   */
  XtVaSetValues(fileDialog,
		XmNtitle, "Save",
		NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, SaveMovie, NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, OpenMovie, NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, SaveStill, NULL); 
  XtAddCallback(fileSelectionBox, XmNokCallback, SaveMovie, NULL);

  /*
   * Do not allow the user to choose the output format.
   */
  XtUnmanageChild(saveAsOptionMenu); 
  XtVaSetValues(saveAsOptionButton, XmNsensitive, False, NULL);
  XtManageChild(fileSelectionBox);
  return;
}

void 
ShowStillFileDialog (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  /*
   * Re-title the file dialog.  Since we are using only one file dialog for this
   * application, we must modify the title to make it clear to the user exactly
   * what is being chosen in the file dialog.
   */
  XtVaSetValues(fileDialog,
		XmNtitle, "Save Still",
		NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, SaveMovie, NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, OpenMovie, NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, SaveStill, NULL); 
  XtAddCallback(fileSelectionBox, XmNokCallback, SaveStill, NULL);

  /*
   * Allow the user to choose the output format.
   */
  XtManageChild(saveAsOptionMenu); 
  XtVaSetValues(saveAsOptionButton, XmNsensitive, True, NULL);
  XtManageChild(fileSelectionBox);
  return;
}

Boolean QuitWorkProc(XtPointer unusedClientData)
{
  XtDestroyWidget(mainWindow);

  exit(0);
} /* end function QuitWorkProc */

void 
Quit (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  XtWorkProcId quitProcID;

  /*
   * Close the stream.
   */
  DisconnectRTV();
  
  /*
   * Start a work-proc to quit the application.  Xt is poorly designed and this is the
   * only way to assure that shared memory will be freed.
   */
  quitProcID = XtAppAddWorkProc(app_context,
				QuitWorkProc,
				(XtPointer)NULL);
  return;
}

void 
PlusButtonCallback (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  int currentFrame;
  int mode;
  int unitsPerSecond;
  
  unitsPerSecond = GetFramesPerSecond();
  mode = GetMode();
  if(mode == FIELD_MODE)
  {
    unitsPerSecond *= 2;
  }
  
  currentFrame = GetCurrentFrame();
  if((w == hourPlusButton) &&
     (GetMovieLengthInFrames() - currentFrame >= (unitsPerSecond * 3600)))
   {
     currentFrame += (unitsPerSecond * 3600);
   }

  if((w == minutePlusButton) &&
     (GetMovieLengthInFrames() - currentFrame >= (unitsPerSecond * 60)))
   {
     currentFrame += (unitsPerSecond * 60);
   }

  if((w == secondPlusButton) &&
     (GetMovieLengthInFrames() - currentFrame >= (unitsPerSecond)))
   {
     currentFrame += unitsPerSecond;
   }
     
  if((w == framePlusButton) &&
     (GetMovieLengthInFrames() - currentFrame >= (1)))
   {
     currentFrame += 1;
   }
  
  if(currentFrame != GetCurrentFrame())
  {
    SetCurrentFrame(currentFrame);
  }
}

void 
HoursTextFieldCallback (Widget w, XtPointer client_data, XtPointer xt_call_data )
{

}

void 
MinusButtonCallback (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  int currentFrame;
  int mode;
  int unitsPerSecond;
  
  unitsPerSecond = GetFramesPerSecond();
  mode = GetMode();
  if(mode == FIELD_MODE)
  {
    unitsPerSecond *= 2;
  }
  
  currentFrame = GetCurrentFrame();
  if((w == hourMinusButton) &&
     (currentFrame - 0 >= (unitsPerSecond * 3600)))
   {
     currentFrame -= (unitsPerSecond * 3600);
   }

  if((w == minuteMinusButton) &&
     (currentFrame - 0 >= (unitsPerSecond * 60)))
   {
     currentFrame -= (unitsPerSecond * 60);
   }

  if((w == secondMinusButton) &&
     (currentFrame - 0 >= (unitsPerSecond)))
   {
     currentFrame -= unitsPerSecond;
   }
     
  if((w == frameMinusButton) &&
     (currentFrame - 0 >= (1)))
   {
     currentFrame -= 1;
   }
  
  if(currentFrame != GetCurrentFrame())
  {
    SetCurrentFrame(currentFrame);
  }
}

void 
MinutesTextFieldCallback (Widget w, XtPointer client_data, XtPointer xt_call_data )
{

}

void 
SecondsTextFieldCallback (Widget w, XtPointer client_data, XtPointer xt_call_data )
{

}

void 
FramesTextFieldCallback (Widget w, XtPointer client_data, XtPointer xt_call_data )
{

}


void 
Search (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  int position;
  int frameNumber;
  
  if(GetFramesPerSecond() == 0)
  {
    return;
  }

  if(GetMovieToolStatus() == MT_PLAY)
  {
    /*
     * Do not listen to the search slider during play operations.
     */
    return;
  }
  
  XmScaleGetValue(searchSlider, &position);
  frameNumber = (int)((float)GetMovieLengthInFrames() * ((float)position / (float)100));
  /*
   * Note:  Displaying a frame with the search slider is different in that
   * you could actually end up displaying more than 30 frames per second.
   * To make sure that the driver can handle the number of requests that
   * can possibly come out of a slider movement, another call was developed
   * in the RTV api.  Be sure to use this call instead, or the rtv driver
   * could crash.
   */
  SetCurrentFrameWithSlider(frameNumber);
}

Boolean RecordWorkProc(XtPointer clientData)
{

  switch(GetMovieToolStatus())
  {
    case MT_RECORD_PAUSE:
    {
      return(False);
    }
      
    case MT_RECORD:
    {
      if(TimeForNewFrame(GetFramesPerSecond()))
      {
	IncrementTimer(GetFramesPerSecond());
      }
      return(False);
    }
      
     
    default:
    {  
      SetCurrentFrame(0);
      return(True);
    }
  }
}


void 
Record (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  switch(GetMovieToolStatus())
  {
    case MT_RECORD_PAUSE:
    {
      if(!IsRecordWindowMapped())
      {
	XtMapWidget(input1Shell);
      }
      XtPopup(input1Shell, XtGrabNone);

      SetMovieToolStatus(MT_RECORD);
      if(!RTVRecordResume())
      {
	PrintDiagnostics("Record-resume operation failed.");
	return;
      }
      SetMovieToolTitle();
      PrintDiagnostics("Record operation resumed.");
      break;
    }
      
    default:
    {
      char message[100];

      sprintf(message, "Now recording %s.", GetMovieFileName());
      PrintDiagnostics(message);

      if(!IsRecordWindowMapped())
      {
	XtMapWidget(input1Shell);
      }
      XtPopup(input1Shell, XtGrabNone);
      XtPopup(input1Shell, XtGrabNone);
      LockVideoWindowSize();

      SetMovieToolStatus(MT_RECORD);
      SetMovieToolTitle();
      SetCurrentFrame(0);
      ResetClock();
      if(!RTVRecord()) return;
      XtAppAddWorkProc(app_context, RecordWorkProc, (XtPointer)NULL);
#ifndef DEBUG
      XtAppAddTimeOut(app_context, SynchDelay, SynchronizationTimeout, (XtPointer)NULL);
#endif
      break;
    }
  }
}

void 
Pause (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  int status;
  
  status = GetMovieToolStatus();
  
  switch(status)
  {
    case MT_PLAY_PAUSE:
    {
      if(!IsPlayWindowMapped())
      {
	XtMapWidget(movieShell);
      }
      XtPopup(movieShell, XtGrabNone);

      SetMovieToolStatus(MT_PLAY);
      if(!RTVResume())
      {
	PrintDiagnostics("Play-resume operation failed.");
	return;
      }
      PrintDiagnostics("Play operation resumed.");
      break;
    }
    
    case MT_RECORD_PAUSE:
    {  
      if(!IsRecordWindowMapped())
      {
	XtMapWidget(input1Shell);
      }
      XtPopup(input1Shell, XtGrabNone);

      SetMovieToolStatus(MT_RECORD);
      if(!RTVRecordResume())
      {
	PrintDiagnostics("Record-resume operation failed.");
	return;
      }
      ResumeClock();
      PrintDiagnostics("Record operation resumed.");
      break;
    }
      
    case MT_PLAY:
    {
      SetMovieToolStatus(MT_PLAY_PAUSE);
      if(!RTVPause())
      {
	PrintDiagnostics("Pause operation failed.");
	return;
      }	
      SynchronizeFrames();
      PrintDiagnostics("Play operation paused.");
      break;
    }
      
    case MT_RECORD:
    {
      if(!IsRecordWindowMapped())
      {
	XtMapWidget(input1Shell);
      }
      XtPopup(input1Shell, XtGrabNone);

      SetMovieToolStatus(MT_RECORD_PAUSE);
      if(!RTVRecordPause())
      {
	PrintDiagnostics("Pause operation failed.");
	return;
      }
      PauseClock();
      PrintDiagnostics("Record operation paused.");
      break;
    }
  }
  SetMovieToolTitle();
}


static void SynchronizationTimeout(XtPointer clientData, XtIntervalId *id)
{
  int currentFrame;
  int framesPerSecond;
  int rtvCurrentFrame;
  
  switch(GetMovieToolStatus())
  {
    case MT_IDLE:
    case NO_MOVIE:
    {
      /*
       * We should not be in this timeout, and we should not
       * set up a new timeout.
       */
      return;
    }

    case MT_RECORD: 
    {
      /*
       * Synchronize the current frame display.
       */
      framesPerSecond = GetFramesPerSecond();
      currentFrame = TimeToFrames(framesPerSecond);
      if(!RTVGetServerStatus(&rtvCurrentFrame))
      {
	/*
	 * Error condition!  Rtv stopped recording, but we did
	 * not!  Stop recording now.
	 */
	Stop((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);
	PrintError("Error!  RTV stopped during record!\n", ERROR_NONFATAL);
	return;
      }
      else
      {
	int mode;
	
	mode = GetMode();
	if((mode == FIELD_MODE) && 
	   ((currentFrame != rtvCurrentFrame * 2) ||
	    (currentFrame != rtvCurrentFrame * 2 + 1)))
	{
	  SetTimerCurrentFrame(rtvCurrentFrame * 2, framesPerSecond);
	}
	   
	 if(mode == FRAME_MODE && currentFrame != rtvCurrentFrame)
	{
	  SetTimerCurrentFrame(rtvCurrentFrame, framesPerSecond);
	}
      }
      break;
    }  

    case MT_PLAY:
    {
      /*
       * Synchronize the current frame display.
       */
      framesPerSecond = GetFramesPerSecond();
      currentFrame = TimeToFrames(framesPerSecond);
      if(!RTVGetServerStatus(&rtvCurrentFrame))
      {
	/*
	 * Error condition!  Rtv stopped playing, but we did
	 * not!  Stop playing now.
	 */
	Stop((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);
	PrintError("Error!  RTV stopped during play!\n", ERROR_NONFATAL);
	return;
      }
      else
      {
	int mode;
	
	mode = GetMode();
	if(mode == FIELD_MODE &&
	   ((currentFrame != rtvCurrentFrame * 2) ||
	    (currentFrame != rtvCurrentFrame * 2 + 1)))
	{
	  SetCurrentFrame(rtvCurrentFrame * 2);
	}
	
	if(mode == FRAME_MODE && currentFrame != rtvCurrentFrame)
	{
	  SetCurrentFrame(rtvCurrentFrame);
	}
      }
      break;
    }  /* End of case MT_PLAY */
  }      
  XtAppAddTimeOut(app_context, SynchDelay, SynchronizationTimeout, (XtPointer)NULL);
}



Boolean PlayWorkProc(XtPointer clientData)
{
  int framesPerSecond;
  int currentFrame;

  switch(GetMovieToolStatus())
  {
    case MT_PLAY_PAUSE:
    {
      return(False);
    }
      
    case MT_PLAY:
    {
      framesPerSecond = GetFramesPerSecond();
      currentFrame = TimeToFrames(framesPerSecond);

      /*
       * If the movie is finished playing, reset the application state variable, 
       * update the main title, and exit the work procedure.
       *
       * If the movie is not finished playing, show another frame, and enter
       * the work procedure again.
       */
      if(currentFrame > GetMovieLengthInFrames())
      {
	SetMonitorAppSetting(True);
	SetMovieToolStatus(MT_IDLE);
	RTVStop();
	SynchronizeFrames();
	SetMovieToolTitle();
	return(True);
      }
      else

      {
	if(TimeForNewFrame(framesPerSecond))
	{
	  currentFrame++;
	  SetCurrentFrame(currentFrame);
	}
	return(False);
      }
    }

    default:
    {  
      /* 
       * The movie is finished playing.  Turn the audio monitor back on, and
       * finish the work-proc.
       */
      SetMonitorAppSetting(True);
      return(True);
    }
  }
}


void 
Play (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  int currentFrame;

  /*
   * Make sure that the application and the rtv process agree
   * on the current movie position.
   */
  SynchronizeFrames();
  currentFrame = GetCurrentFrame();

  switch(GetMovieToolStatus())
  {
    case MT_PLAY_PAUSE:
    {
      if(!IsPlayWindowMapped())
      {
	XtMapWidget(movieShell);
      }
      XtPopup(movieShell, XtGrabNone);

      SetMovieToolStatus(MT_PLAY);
      if(!RTVResume())
      {
	PrintDiagnostics("Play-resume operation failed.\n");
	return;
      }
      SetMovieToolTitle();
      PrintDiagnostics("Play operation resumed.");
      break;
    }
      
    case MT_IDLE:
    {
      char message[100];

      sprintf(message, "Now playing %s.", GetMovieFileName());
      PrintDiagnostics(message);

      if(!IsPlayWindowMapped())
      {
	XtMapWidget(movieShell);
      }
      XtPopup(movieShell, XtGrabNone);
      LockVideoWindowSize();

      /*
       * If the movie is at the end, go back to the beginning before playing the movie.
       */
      if(currentFrame >= GetMovieLengthInFrames())
      {
	SetCurrentFrame(0);
      }

      SetMonitorAppSetting(False);
      if(!RTVPlay()) return;
      SetMovieToolStatus(MT_PLAY);
      SetMovieToolTitle();

      ResetClock();
      XtAppAddWorkProc(app_context, PlayWorkProc, (XtPointer)NULL);
#ifndef DEBUG
      XtAppAddTimeOut(app_context, SynchDelay, SynchronizationTimeout, (XtPointer)NULL);
#endif
      break;
    }
  }
}


void 
Stop (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  switch(GetMovieToolStatus())
  {
    case MT_PLAY_PAUSE:
    case MT_PLAY:
    {
      SetMonitorAppSetting(True);
      if(!RTVStop())
      {
	PrintDiagnostics("Play-stop operation failed.");
	return;
      }
      /*
       * Make sure that the application and the rtv process
       * agree on the current frame number.
       */
      SynchronizeFrames();
      PrintDiagnostics("Stopped play operation.");
      break;
    }
     
    case MT_RECORD_PAUSE:
    {
      /*
       * Make sure that the application and the rtv process
       * agree on the current movie position.
       */
      SynchronizeFrames();
      
      /*
       * Note:  There is no break here on purpose!
       */
    }
    case MT_RECORD:
    {     
      int currentFrame;

      if(!RTVStop())
      {
	PrintDiagnostics("Record-stop operation failed.\n");
	return;
      }
      if(!RTVGetCurrentFrame(&currentFrame))
      {
	PrintDiagnostics("Current frame could not be determined.");
	return;
      }
      PrintDiagnostics("Stopped record operation.");

      StopVideo();
      OpenMovie((Widget)NULL, (XtPointer)GetMovieFileName(), NULL); 
      break;
    }
  }
  SetMovieToolStatus(MT_IDLE);
  SetMovieToolTitle();
}

void 
StartOfMovie (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  if(GetCurrentFrame() != 0)
  {
    SetCurrentFrame(0);
  }
}

void 
ReverseFrame (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  int currentFrame;
  
  currentFrame = GetCurrentFrame();
  if(currentFrame > 0)
  {
    currentFrame --;
    SetCurrentFrame(currentFrame);
  }
}

void 
AdvanceFrame (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  int currentFrame;
  
  currentFrame = GetCurrentFrame();
  if(currentFrame < GetMovieLengthInFrames())
  {
    currentFrame++;
    SetCurrentFrame(currentFrame);
  }
}

void 
EndOfMovie (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  if(GetCurrentFrame() != GetMovieLengthInFrames())
  {
    SetCurrentFrame(GetMovieLengthInFrames());
  }
}

void 
BrightnessAdjust (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  int brightness;
  
  XmScaleGetValue(w, &brightness);
  SetBrightness(brightness);
}

void 
ContrastAdjust (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  int contrast;
  
  XmScaleGetValue(w, &contrast);
  SetContrast(contrast);
}

void 
HueAdjust (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  int hue;
  
  XmScaleGetValue(w, &hue);
  SetHue(hue);
}

void 
SaturationAdjust (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  int saturation;
  
  XmScaleGetValue(w, &saturation);
  SetSaturation(saturation);
}

void 
InputSettingsDefaultPressed (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  DefaultColorSettings();
}

void 
ChangePlaybackVolume (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  int percentVolume;
  
  XmScaleGetValue(playbackScale, &percentVolume);
  
  SetPlaybackVolume(percentVolume);
}

void 
ChangeRecordVolume (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  int percentVolume;
  
  XmScaleGetValue(recordScale, &percentVolume);
  SetRecordVolume(percentVolume);
}

void 
ChangeMonitorVolume (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  int percentVolume;
  
  XmScaleGetValue(monitorScale, &percentVolume);
  SetMonitorVolume(percentVolume);
}

void 
ChangeAudioOutputDevice (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  int outputDevice = 0;

  if(XmToggleButtonGetState(playbackSpeakerToggle))
  {
    outputDevice += SPEAKER;
  }
  
  if(XmToggleButtonGetState(playbackHeadphonesToggle))
  {
    outputDevice += HEADPHONES;
  }
  
  if(XmToggleButtonGetState(playbackLineOutToggle))
  {
    outputDevice += LINE_OUT;
  }

  if(outputDevice == 0)
  {
    DisplayOutputDevice(GetOutputDevice());
    UpdateAudioControls();
  }
  else
  {
    SetOutputDevice(outputDevice);
  }
}

void 
UseLineInForRecording (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  SetInputDevice(LINE_IN);
}

void 
UseMicForRecording (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  SetInputDevice(MICROPHONE);
}

void 
MonitorOn (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  SetMonitorUserSetting(True);
}

void 
MonitorOff (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  SetMonitorUserSetting(False);
}

void 
ToggleAudioOnStatus (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  XmToggleButtonCallbackStruct *callbackStruct = xt_call_data;
  
  if(w == audioOnToggle && callbackStruct->set == True)
  {
    TurnAudioOn();
  }

  if(w == audioOffToggle && callbackStruct->set == True)
  {
    TurnAudioOff();
  }
}

Boolean OpenMovieWorkProc(XtPointer data)
{
  char* filename;
  char message[100];
  int framesPerSecond;
  int movieWidth;
  int movieHeight;
  int movieLength;
  int mode;

  /*
   * Be sure that the playback window is mapped
   * before preparing the window for playing.
   */
  if(!IsPlayWindowMapped())
  {
    return(False);
  }
  
  filename = (char*)malloc(sizeof(char) * (strlen(GetMovieFileName()) + 1));
  if(filename == NULL)
  {
    sprintf(message, "Could not allocate memory.");
    PrintError(message, ERROR_NONFATAL);
  }
  strcpy(filename, GetMovieFileName());

  if(!RTVOpenMovie(filename, &framesPerSecond, &movieWidth, &movieHeight))
  {
    sprintf(message, "Error!  Could not open movie %s.", filename);
    PrintError(message, ERROR_NONFATAL);
    free(filename);
    return(True);
  }
  if(framesPerSecond != GetFramesPerSecond())
  {
    /*
     * The movie fps has to match the video signal that is connected
     * to the video board.  (You can't play a movie recorded from a 
     * PAL signal with NTSC connected).  This code will select the
     * appropriate signal so the movie can be played.
     */
    SetMovieToolStatus(MT_IDLE);
    ConfigureInputForMovie(framesPerSecond);
  }
  
  sprintf(message, "Opened movie %s.", filename);
  PrintDiagnostics(message);
  SetMovieLoaded(True);
  
  SetFramesPerSecond(framesPerSecond);

  SetMovieToolStatus(NO_MOVIE);
  SetCurrentMovieSize(movieWidth, movieHeight);
  LockPlaybackWindowSize();
  SetMovieToolStatus(MT_IDLE);

  if(!GetRTVMovieLength(&movieLength))
  {
    PrintDiagnostics("Movie length request failed.");
    return(True);
  }
  
  mode = GetMode();
  if(mode == FIELD_MODE)
  {
    movieLength *= 2;
  }
  SetMovieLengthInFrames(movieLength);
  /*
   * Initialize Jpeg for RTV via server protocol request.
   */
  RTVWindowPlayPrepare(); 

  SetMovieToolTitle();
  SetCurrentFrame(0);
  free(filename);

  if(ShowOutputOnPlayback())
  {
    /*
     * Output was on - turn it back on to play this movie.
     */
    StartOutput();
  }
  return(True);
}


void 
OpenMovie (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  XmFileSelectionBoxCallbackStruct*	call_data =
    (XmFileSelectionBoxCallbackStruct *) xt_call_data;
  char * fileName = NULL;
  Boolean success = False;
  char message[100];

  if(w != (Widget)NULL)
  {
    success = XmStringGetLtoR(call_data->value,
			     XmSTRING_DEFAULT_CHARSET,
			     &fileName);
    if(!success)
    {
      XtUnmanageChild(fileSelectionBox);
      PrintError("Error - filename included unprintable characters", ERROR_NONFATAL);
      if(fileName != NULL)
      {
	free(fileName);
      }
      return;
    }
  }

  else
  {
    fileName = (char*)malloc(sizeof(char) * (strlen(client_data) + 1));
    if(fileName == NULL)
    {
      PrintDiagnostics("Error!  Could not allocate memory for a filename!");
      return;
    }
    strcpy(fileName, client_data);
  }

  if(!FileExists(fileName))
  {
    sprintf(message, "Error - %s does not exist.", fileName);
    PrintError(message, ERROR_NONFATAL);
    free(fileName);
    return;
  }
    
  if(!XPlxIsRTVFile(fileName))
  {
    sprintf(message, "Error - %s is not a valid Rtv movie file.", fileName);
    PrintError(message, ERROR_NONFATAL);
    free(fileName);
    return;
  }

  XtUnmanageChild(fileSelectionBox);

  SetMovieFileName(fileName);

  if(fileName != NULL)
  {
    free(fileName);
  }

  XtPopdown(input1Shell);
  XtPopup(movieShell, XtGrabNone);
  XtAppAddWorkProc(app_context,
		   OpenMovieWorkProc,
		   (XtPointer)NULL);
}

void 
SetMovieFileFormat (Widget w, XtPointer client_data, XtPointer xt_call_data )
{

}

void 
ChangeImageQuality (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  int bandwidth;

  XmScaleGetValue(imageQualitySlider, &bandwidth);
  SetBandwidth(bandwidth, True);
}

void 
FullSize (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  ChangeInputSize(FULL_SIZE);
}

void 
QuarterSize (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  ChangeInputSize(QUARTER_SIZE);
}

void 
ChangeMinimumQfactor (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  int qfactor;
  
  XmScaleGetValue(qfactorSlider, &qfactor);
  SetQfactor(qfactor);
}


void ErrorShellCancelButtonPressed (Widget w, XtPointer client_data, XtPointer xt_call_data )
{

}

void ErrorShellOKButtonPressed (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  int errorCode = GetErrorCode();
  
  switch(errorCode)
  {
    case ERROR_NONFATAL:
    {
      ClearErrorCode();
      break;
    }
      
    case ERROR_FATAL:
    {
      Quit((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);
      break;
    }
  }
}


void 
ShowHelpPage (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  /*
   * Client data is an integer that indicates the page number to display.
   */
  int pageNumber;
  
  pageNumber = (int)client_data;
  DisplayHelp(pageNumber);
}

void 
ClearDiagnosticMessages (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  ClearAllDiagnosticMessages();
}


void 
SetFieldMode (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  XmToggleButtonCallbackStruct *call_data = (XmToggleButtonCallbackStruct *) xt_call_data;
  int currentFrame;
  int movieLength;
  int mode;
  
  mode = GetMode();
  if(!(call_data->set))
  {
    if(!XmToggleButtonGetState(frameToggleButton))
    {
      /*
       * The other toggle button is not set either.  Keep this
       * one set.
       */
      XmToggleButtonSetState(fieldToggleButton, True, False);
    }
    return;
  }
  
  XmToggleButtonSetState(fieldToggleButton, True, False);
  XmToggleButtonSetState(frameToggleButton, False, False);
  if(mode != FIELD_MODE)
  {
    currentFrame = GetCurrentFrame();
    movieLength = GetMovieLengthInFrames();
    SetMode(FIELD_MODE);
    SetMovieLengthInFrames(movieLength * 2);
    SetCurrentFrame(currentFrame * 2);
  }
}

void 
SetFrameMode (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  XmToggleButtonCallbackStruct *call_data = (XmToggleButtonCallbackStruct *) xt_call_data;
  int currentFrame;
  int movieLength;
  int mode;
  
  mode = GetMode();
  if(!(call_data->set))
  {
    if(!XmToggleButtonGetState(fieldToggleButton))
    {
      /*
       * The other toggle button is not set either.  Keep this
       * one set.
       */
      XmToggleButtonSetState(frameToggleButton, True, False);
    }
    return;
  }
  
  XmToggleButtonSetState(frameToggleButton, True, False);
  XmToggleButtonSetState(fieldToggleButton, False, False);
  if(mode != FRAME_MODE)
  {
    currentFrame = GetCurrentFrame();
    movieLength = GetMovieLengthInFrames();
    SetMode(FRAME_MODE);
    SetMovieLengthInFrames(movieLength / 2);
    SetCurrentFrame(currentFrame / 2);
  }
}


void 
SetImageSaveFormat (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  SetImageFormat(client_data);
}

void 
StartMovieHere (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  int currentFrame;
  int movieLength;
  int mode;
  
  currentFrame = GetCurrentFrame();
  mode = GetMode();
  if(mode != FRAME_MODE)
  {
    currentFrame /= 2;
  }
  RTVSetFirstValidFrame(currentFrame);

  if(!GetRTVMovieLength(&movieLength))
  {
    PrintDiagnostics("Movie length request failed.");
    return;
  }
  
  if(mode == FIELD_MODE)
  {
    movieLength *= 2;
  }
  SetMovieLengthInFrames(movieLength);
  SetCurrentFrame(0);
}

void 
EndMovieHere (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  int currentFrame;
  int mode;
  int movieLength;
  
  currentFrame = GetCurrentFrame();
  mode = GetMode();
  if(mode != FRAME_MODE)
  {
    currentFrame /= 2;
  }
  RTVSetLastValidFrame(currentFrame);

  if(!GetRTVMovieLength(&movieLength))
  {
    PrintDiagnostics("Movie length request failed.");
    return;
  }
  
  if(mode == FIELD_MODE)
  {
    movieLength *= 2;
  }
  SetMovieLengthInFrames(movieLength);
  SetCurrentFrame(movieLength);
  XmScaleSetValue(searchSlider, 100); 
}


void 
UseOriginalMovieLength (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  int movieLength;
  int currentFrame;
  int mode;
  
  currentFrame = RTVUseOriginalMovieLength();

  if(!GetRTVMovieLength(&movieLength))
  {
    PrintDiagnostics("Movie length request failed.");
    return;
  }
  
  mode = GetMode();
  if(mode == FIELD_MODE)
  {
    movieLength *= 2;
  }
  SetMovieLengthInFrames(movieLength);
  SetCurrentFrame(currentFrame);

}


void SignalHandlerTimeOut(XtPointer clientData, XtIntervalId *id)
{
  Quit((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);
  return;
}

void SignalHandler(int theSignal)
{
  signal(SIGHUP, SignalHandler);
  signal(SIGINT, SignalHandler);
  signal(SIGQUIT, SignalHandler);
  signal(SIGTERM, SignalHandler);

  XtAppAddTimeOut(app_context, 0, SignalHandlerTimeOut, (XtPointer)theSignal);
  return;
} /* end function SignalHandler */

void CatchSignals(void)
{
  signal(SIGHUP, SignalHandler);
  signal(SIGINT, SignalHandler);
  signal(SIGQUIT, SignalHandler);
  signal(SIGTERM, SignalHandler);
  return;
} /* end function CatchSignals */


