/* $Header: /plx/source/Xclients/generic/RCS/sunAudio.c,v 1.4 1995/11/15 02:47:47 lovvik Exp $ */

/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */

/* $Log: sunAudio.c,v $
 * Revision 1.4  1995/11/15  02:47:47  lovvik
 * I put the signal handling guts in a timeout.  This fixed a lockup
 * problem that occured during play/record operations.
 *
 * Revision 1.3  1995/11/01  02:16:13  lovvik
 * Added 2 new functions:  PauseAudioSignalHandler() and
 * ResumeAudioSignalHandler().  These functions allow the
 * audio module to restart the audio signal handler only
 * if the client is currently using it.  (If Rtv is playing
 * or recording, the signal handler routine breaks occasionally).
 *
 * Revision 1.1  1995/10/02  23:48:10  lovvik
 * Initial revision
 * */

static char movieToolRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/sunAudio.c,v 1.4 1995/11/15 02:47:47 lovvik Exp $";

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/CascadeB.h>
#include <Xm/FileSB.h>
#include <Xm/PushB.h>
#include <Xm/Scale.h>
#include <Xm/SelectioB.h>
#include <Xm/TextF.h>
#include <Xm/ToggleBG.h>

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/types.h>
#include <stropts.h>
#include <sys/conf.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <multimedia/libaudio.h>
#include <multimedia/audio_device.h>
#include <multimedia/audio_encode.h>

#include "structs.h"
#include "globals.h"
#include "prototypes.h"



static int audioFile;
static int currentPlayGain;
static int currentRecordGain;
static int currentMonitorGain;
static int playPort;
static int recordPort;
static int monitorVolumeSave;
static int audioSignalHandlerInstalled = True;
static int audioSignalHandlerCalled = False;

#define InstallAudioSignalHandler() signal(SIGPOLL, SunAudioSignalHandler);\
				    audioSignalHandlerInstalled = True;


#define RemoveAudioSignalHandler() signal(SIGPOLL, SIG_IGN);\
				   audioSignalHandlerInstalled = False;

#define PauseAudioSignalHandler()  signal(SIGPOLL, SIG_IGN);




void SunAudioInitializeAudio(void)
{
  audio_info_t audioInfo;
#ifdef SVR4
  audio_device_t deviceType;
#else  
  int		 deviceType;
#endif
  int outputPort = 0;
  int inputPort = 0;

  audioFile = open("/dev/audioctl", O_RDWR);
  if(!audioFile)
  {
    /* 
     * If the audio controller could not be opened, it is probably because:
     * 1.  The file permissions for /dev/audioctl are incorrect.
     * 2.  Too many file descriptors open at once.
     * 3.  The system does not have an audio device.
     */
    PrintDiagnostics("/dev/audioctl could not be opened.  Audio is not available.");
    SetAudioNotAvailable();
    return;
  }

  ioctl(audioFile, AUDIO_GETDEV, &deviceType);
#ifdef SVR4
  /*
   * If this is Solaris 2.x, we can look at the audio device name to
   * determine if it can handle 44.1khz audio or not.
   */
  if ((strcmp(deviceType.name, "SUNW,dbri") == 0) ||
      (strcmp(deviceType.name, "SUNW,CS4231") == 0))
#else
  /*
   * If this is SunOS 4.x, we can look at the device type to determine if
   * the audio configuration can handle 44.1 khz audio or not.
   */
  if ((deviceType == AUDIO_DEV_SPEAKERBOX) ||
      (deviceType == AUDIO_DEV_CODEC) ||
      (deviceType == 5))					    /* For Sparc-5. */
#endif
  {
    /*
     * The audio configuration can handle 44.1 khz audio.
     */
    PrintDiagnostics("High quality audio is available.");
  }
  else
  {
    /*
     * The audio configuration cannot handle 44.1 khz audio.  We deal with this
     * by turning off the audio flag and disabling the audio panel.  Also, to
     * make this complete, we need to close the /dev/audioctl file and set the
     * file descriptor to zero so no other audio function will respond to requests.
     */

    PrintDiagnostics("High quality audio is not supported.");
#ifndef DEBUG
    TurnAudioOff();
    SetAudioNotAvailable();
    close(audioFile);
    audioFile = 0;
    return;
#endif
  }
  
  ioctl(audioFile, AUDIO_GETINFO, &audioInfo);

  currentPlayGain = audioInfo.play.gain;
  DisplayPlayVolume(VolumeToPercent(audioInfo.play.gain));
  currentRecordGain = audioInfo.record.gain;
  DisplayRecordVolume(VolumeToPercent(audioInfo.record.gain));
  currentMonitorGain = audioInfo.monitor_gain;
  monitorVolumeSave = currentMonitorGain;
  DisplayMonitorVolume(VolumeToPercent(audioInfo.monitor_gain));

  CheckAvailableOutputPorts();
  CheckAvailableInputPorts();
  
  /*
   * Find the currently selected output device(s).
   */

  playPort = audioInfo.play.port;
  if((playPort & AUDIO_SPEAKER) != 0)
  {
    outputPort |= SPEAKER;
  }
  
  if((playPort & AUDIO_HEADPHONE) != 0)
  {
    outputPort |= HEADPHONES;
  }
  
  if((playPort & AUDIO_LINE_OUT) != 0)
  {
    outputPort |= LINE_OUT;
  }

  /* 
   * Set the appropriate toggle button in the user interface.
   */

  DisplayOutputDevice(outputPort);

  /*
   * Find the currently selected input device.
   */

  recordPort = audioInfo.record.port;
  if((recordPort & AUDIO_MICROPHONE) != 0)
  {
    inputPort = MICROPHONE;
  }
  
  if((recordPort & AUDIO_LINE_IN) != 0)
  {
    inputPort = LINE_IN;
  }

  /*
   * Set the appropriate toggle button in the user interface.
   */

  DisplayInputDevice(inputPort);

  /*
   * Request to receive the next signal.
   */

  ioctl(audioFile, I_SETSIG, S_MSG);  
  signal(SIGPOLL, SunAudioSignalHandler);
}

void CheckAvailableOutputPorts(void)
{
  audio_info_t audioInfo;
  int currentOutputPort;

  if(!audioFile)
  {
    return;
  }

  ioctl(audioFile, AUDIO_GETINFO, &audioInfo);
  if(!(audioInfo.play.avail_ports & AUDIO_SPEAKER))
  {
    PrintDiagnostics("Audio device reported no speaker is available.");
    UnmapSpeakerToggle();
  }
  
  if(!(audioInfo.play.avail_ports & AUDIO_HEADPHONE))
  {
    PrintDiagnostics("Audio device reported no headphones are available.");
    UnmapHeadphonesToggle();
  }
  
  if(!(audioInfo.play.avail_ports & LINE_OUT))
  {
    PrintDiagnostics("Audio device reported no line out is available.");
    UnmapLineOutToggle();
  }

  /*
   * Check to see if more than 1 output is available at a time.  I do this
   * by selecting all available outputs at the same time.  Then, I load the
   * audio information to see if the current output port is set to all available
   * ports.  If it is set to all available ports, you can output to more than
   * one device at a time.  Otherwise, the audio device can handle only one
   * destination at a time.
   */

  currentOutputPort = audioInfo.play.port;			    /* Start by saving the currently selected output port. */
  audioInfo.play.port = audioInfo.play.avail_ports;
  ioctl(audioFile, AUDIO_SETINFO, &audioInfo);
  ioctl(audioFile, AUDIO_GETINFO, &audioInfo);
  if(audioInfo.play.port != audioInfo.play.avail_ports)
  {
    PrintDiagnostics("Only one audio output at a time is supported.");
    SetAudioOutputOneOnly();					    /* Change the user-interface to accept one at a time. */
  }
  audioInfo.play.port = currentOutputPort;			    /* Restore the output port that was originally selected. */
  ioctl(audioFile, AUDIO_SETINFO, &audioInfo);
}

void CheckAvailableInputPorts(void)
{
  audio_info_t audioInfo;

  if(!audioFile)
  {
    return;
  }

  ioctl(audioFile, AUDIO_GETINFO, &audioInfo);
  if(!(audioInfo.record.avail_ports & AUDIO_MICROPHONE))
  {
    PrintDiagnostics("Audio device reported no microphone is available.");
    UnmapMicrophoneToggle();
  }
  
  if(!(audioInfo.record.avail_ports & AUDIO_LINE_IN))
  {
    PrintDiagnostics("Audio device reported no line in is available.");
    UnmapLineInToggle();
  }
}


static void SunAudioSignalHandlerTimeOut(XtPointer unusedClientData, XtIntervalId *id)
{
  RemoveAudioSignalHandler();
  {  
    audio_info_t audioInfo;
    int outputPort = 0;
      
    ioctl(audioFile, AUDIO_GETINFO, &audioInfo);

    if(currentPlayGain != audioInfo.play.gain)
    {
      currentPlayGain = audioInfo.play.gain;
      DisplayPlayVolume(VolumeToPercent(audioInfo.play.gain));
      UpdateAudioControls();
    }
  
    if(currentRecordGain != audioInfo.record.gain)
    {
      currentRecordGain = audioInfo.record.gain;
      DisplayRecordVolume(VolumeToPercent(audioInfo.record.gain));
      UpdateAudioControls();
    }
  
    if(currentMonitorGain != audioInfo.monitor_gain)
    {
      if(GetMonitorUserSetting())
      {
	currentMonitorGain = audioInfo.monitor_gain;
	DisplayMonitorVolume(VolumeToPercent(audioInfo.monitor_gain));
	UpdateAudioControls();
      }
      else
      {
	audioInfo.monitor_gain = 0;
	ioctl(audioFile, AUDIO_SETINFO, &audioInfo);
      }
    }

    if(playPort != audioInfo.play.port)
    {
      playPort = audioInfo.play.port;
      if(playPort & AUDIO_SPEAKER)
      {
	outputPort = outputPort | SPEAKER;
      }
    
      if(playPort & AUDIO_HEADPHONE)
      {
	outputPort = outputPort | HEADPHONES;
      }
  
      if(playPort & AUDIO_LINE_OUT)
      {
	outputPort = outputPort | LINE_OUT;
      }

      DisplayOutputDevice(outputPort);
      UpdateAudioControls();
    }
  }
  /*
   * Turn signal handling back on.
   */
  audioSignalHandlerCalled = False;
  InstallAudioSignalHandler();
}



void SunAudioSignalHandler(int Signal)
{
  /*
   * Ignore signals while in this function.  This is done because
   * sometimes when another application is changing the system
   * audio parameters, signals will arrive while this function is
   * being executed.  If this happens, then this application will
   * exit with a Pollable Event message.
   */

  PauseAudioSignalHandler();

  if(!audioFile)
  {
    return;
  }
  if(audioSignalHandlerCalled == False)
  {
    XtAppAddTimeOut(app_context,
		    500,
		    SunAudioSignalHandlerTimeOut,
		    (XtPointer)NULL);
    audioSignalHandlerCalled = True;
  }
}


int VolumeToPercent(int volume)
{
  float normalizedVolume;
  
  normalizedVolume = ((float)(volume - AUDIO_MIN_GAIN) / (float)AUDIO_MAX_GAIN);
  return((int)(normalizedVolume * 100));
}


int PercentToVolume(int percent)
{
  return((int)(((float)(percent * AUDIO_MAX_GAIN) / (float) 100) + AUDIO_MIN_GAIN));
}



void SunAudioSetPlayVolume(int percentVolume)
{
  PauseAudioSignalHandler();
  {  
    int newVolume;
    audio_info_t audioInfo;

    if(!audioFile)
    {
      return;
    }

    newVolume = PercentToVolume(percentVolume);
    if(currentPlayGain != newVolume)
    {
      AUDIO_INITINFO(&audioInfo);
      audioInfo.play.gain = newVolume;
      ioctl(audioFile, AUDIO_SETINFO, &audioInfo);
      currentPlayGain = newVolume;
    }    
  }
  ResumeAudioSignalHandler();
}


void SunAudioSetRecordVolume(int percentVolume)
{
  PauseAudioSignalHandler();
  {
    int newVolume;
    audio_info_t audioInfo;
  
    if(!audioFile)
    {
      return;
    }

    newVolume = PercentToVolume(percentVolume);
    if(currentRecordGain != newVolume)
    {
      AUDIO_INITINFO(&audioInfo);
      audioInfo.record.gain = newVolume;
      ioctl(audioFile, AUDIO_SETINFO, &audioInfo);
      currentRecordGain = newVolume;
    }    
  }
  ResumeAudioSignalHandler();
}


void SunAudioSetMonitorVolume(int percentVolume)
{
  PauseAudioSignalHandler();
  {
    int newVolume;
    audio_info_t audioInfo;
  
    if(!audioFile)
    {
      return;
    }
  
    newVolume = PercentToVolume(percentVolume);
    if(currentMonitorGain != newVolume)
    {
      AUDIO_INITINFO(&audioInfo);
      audioInfo.monitor_gain = newVolume;
      ioctl(audioFile, AUDIO_SETINFO, &audioInfo);
      currentMonitorGain = newVolume;
    }    
  }
  ResumeAudioSignalHandler();
}


void SunAudioSetOutput(int output)
{
  PauseAudioSignalHandler();
  {
    int newOutputDevice = 0;
    audio_info_t audioInfo;
    
    if(!audioFile)
    {
      return;
    }
  
    if(output & SPEAKER)
    {
      newOutputDevice += AUDIO_SPEAKER;
    }
    
    if(output & HEADPHONES)
    {
      newOutputDevice += AUDIO_HEADPHONE;
    }
  
    if(output & LINE_OUT)
    {
      newOutputDevice += AUDIO_LINE_OUT;
    }

    AUDIO_INITINFO(&audioInfo);
    audioInfo.play.port = newOutputDevice;
    ioctl(audioFile, AUDIO_SETINFO, &audioInfo);  
  }
  ResumeAudioSignalHandler();
}


void SunAudioSetInput(int input)
{
  PauseAudioSignalHandler();
  {
    int newInputDevice = 0;
    audio_info_t audioInfo;
  
    if(!audioFile)
    {
      return;
    }

    if(input & LINE_IN)
    {
      newInputDevice = AUDIO_LINE_IN;
    }
    if(input == MICROPHONE)
    {
      newInputDevice = AUDIO_MICROPHONE;
    }
    AUDIO_INITINFO(&audioInfo);
    audioInfo.record.port = newInputDevice;
    ioctl(audioFile, AUDIO_SETINFO, &audioInfo);  
  }
  ResumeAudioSignalHandler();
}


void SunAudioMonitorOn(void)
{
  PauseAudioSignalHandler();
  {
    audio_info_t audioInfo;

    if(!audioFile)
    {
      return;
    }
    currentMonitorGain = monitorVolumeSave;
    AUDIO_INITINFO(&audioInfo);
    audioInfo.monitor_gain = monitorVolumeSave;
    ioctl(audioFile, AUDIO_SETINFO, &audioInfo);  
  }
  ResumeAudioSignalHandler();
}


void SunAudioMonitorOff(void)
{
  PauseAudioSignalHandler();
  {
    audio_info_t audioInfo;
  
    if(!audioFile)
    {
      return;
    }
    /*
     * Must save the current monitor volume for later.  If you set the
     * monitor volume to zero, the signal handler will reset the variable
     * 'currentMonitorGain' to reflect the change.
     */
    
    monitorVolumeSave = currentMonitorGain;
    AUDIO_INITINFO(&audioInfo);
    audioInfo.monitor_gain = 0;
    ioctl(audioFile, AUDIO_SETINFO, &audioInfo);
    DisplayMonitorVolume(VolumeToPercent(monitorVolumeSave));
  }
  ResumeAudioSignalHandler();
}


void ResumeAudioSignalHandler(void)
{
  if(audioSignalHandlerInstalled == True &&
     audioSignalHandlerCalled == False)
  {
    signal(SIGPOLL, SunAudioSignalHandler);  
  }
}

