/* $Header: /plx/source/Xclients/generic/RCS/timer.c,v 1.4 1995/11/27 23:56:48 lovvik Exp $ */

/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */

/* $Log: timer.c,v $
 * Revision 1.4  1995/11/27  23:56:48  lovvik
 * Removed prototype for gettimeofday().  The prototype can be found
 * in <sys/time.h> on 2.5 systems.  This prototype was put in because
 * 2.3 systems do not have a prototype for the function.
 *
 * Revision 1.3  1995/11/01  03:01:50  lovvik
 * Changed the logic in DisplayFrames(), DisplaySeconds(), DisplayMinutes(),
 * and DisplayHours().  This makes the time code display much more efficient.
 *
 * Revision 1.2  1995/10/12  20:08:25  lovvik
 * Added field support to the timer code.
 *
 * Revision 1.1  1995/10/02  23:48:32  lovvik
 * Initial revision
 * */

static char movieToolRcisd[] = "$Header: /plx/source/Xclients/generic/RCS/timer.c,v 1.4 1995/11/27 23:56:48 lovvik Exp $";

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <X11/StringDefs.h>
#include <Xm/CascadeB.h>
#include <Xm/DialogS.h>
#include <Xm/FileSB.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/List.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/Scale.h>
#include <Xm/ScrollBar.h>
#include <Xm/SelectioB.h>
#include <Xm/Separator.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
#include <Xm/CascadeBG.h>
#include <Xm/LabelG.h>
#include <Xm/PushBG.h>
#include <Xm/SeparatoG.h>
#include <Xm/ToggleBG.h>
#include <stdio.h>

#include <sys/time.h>						    /* Required for gettimeofday */

#include "interface.h"
#include "structs.h"
#include "prototypes.h"

static TimerStruct timer;
static struct timeval startTime;
static struct timeval startPauseTime;
static struct timeval currentTime;
static int currentFrame;
static long totalPausedTime;

long ElapsedTime(struct timeval*, struct timeval*);

void InitializeTimer(void)
{
  timer.hours = 0;
  timer.minutes = 0;
  timer.seconds = 0;
  timer.frames = 0;

  DisplayFrames();
  DisplaySeconds();
  DisplayMinutes();
  DisplayHours();
}


void DisplayFrames(void)
{
  char frames[3];
  
  sprintf(frames, "%02d", timer.frames);
  XmTextSetString(framesTextField, frames);
}


void DisplaySeconds(void)
{
  char seconds[3];
  
  sprintf(seconds, "%02d", timer.seconds);
  XmTextSetString(secondsTextField, seconds);
}


void DisplayMinutes(void)
{
  char minutes[3];
  
  sprintf(minutes, "%02d", timer.minutes);
  XmTextSetString(minutesTextField, minutes);
}


void DisplayHours(void)
{
  char hours[3];
  
  sprintf(hours, "%02d", timer.hours);
  XmTextSetString(hoursTextField, hours);
}



int TimeToFrames(int framesPerSecond)
{
  int mode;
  int unitsPerSecond;
  
  unitsPerSecond = framesPerSecond;
  mode = GetMode();
  if(mode == FIELD_MODE)
  {
    unitsPerSecond *= 2;
  }
  
  return((((((timer.hours)*60) + timer.minutes)*60) +
	  timer.seconds) * unitsPerSecond + timer.frames);
}

int FrameHours(int totalFrames, int framesPerSecond)
{
  int mode;
  int unitsPerSecond;
  
  unitsPerSecond = framesPerSecond;
  mode = GetMode();
  if(mode == FIELD_MODE)
  {
    unitsPerSecond *= 2;
  }
  
  if(framesPerSecond == 0)
  {
    return(0);
  }
  
  return(timer.hours/(3600 * unitsPerSecond));
}


int FrameMinutes(int totalFrames, int framesPerSecond)
{
  int hours;
  int mode;
  int unitsPerSecond;
  
  unitsPerSecond = framesPerSecond;
  mode = GetMode();
  if(mode == FIELD_MODE)
  {
    unitsPerSecond *= 2;
  }
  
  if(framesPerSecond == 0)
  {
    return(0);
  }
  
  hours = FrameHours(totalFrames, framesPerSecond);
  totalFrames -= (hours * 3600 * unitsPerSecond);
  return(totalFrames / (60 * unitsPerSecond));
}


int FrameSeconds(int totalFrames, int framesPerSecond)
{
  int hours;
  int minutes;
  int mode;
  int unitsPerSecond;
  
  unitsPerSecond = framesPerSecond;
  mode = GetMode();
  if(mode == FIELD_MODE)
  {
    unitsPerSecond *= 2;
  }
  
  if(framesPerSecond == 0)
  {
    return(0);
  }
  
  hours = FrameHours(totalFrames, framesPerSecond);
  totalFrames -= (hours * 3600 * unitsPerSecond);
  
  minutes = FrameMinutes(totalFrames, framesPerSecond);
  totalFrames -= (minutes * 60 * unitsPerSecond);
  
  return(totalFrames / unitsPerSecond);
}


int FrameFrames(int totalFrames, int framesPerSecond)
{
  int hours;
  int minutes;
  int seconds;
  int mode;
  int unitsPerSecond;
  
  unitsPerSecond = framesPerSecond;
  mode = GetMode();
  if(mode == FIELD_MODE)
  {
    unitsPerSecond *= 2;
  }
  
  if(framesPerSecond == 0)
  {
    return(0);
  }
  
  hours = FrameHours(totalFrames, framesPerSecond);
  totalFrames -= (hours * 3600 * unitsPerSecond);
  
  minutes = FrameMinutes(totalFrames, framesPerSecond);
  totalFrames -= (minutes * 60 * unitsPerSecond);

  seconds = FrameSeconds(totalFrames, framesPerSecond);
  totalFrames -= (seconds * unitsPerSecond);
  
  return(totalFrames);
}



void IncrementTimer(int framesPerSecond)
{
  int mode;
  int unitsPerSecond;
  
  unitsPerSecond = framesPerSecond;
  mode = GetMode();
  if(mode == FIELD_MODE)
  {
    unitsPerSecond *= 2;
  }
  
  timer.frames++;
  if(timer.frames >= unitsPerSecond)
  {
    timer.frames = 0;
    
    timer.seconds++;
    if(timer.seconds >= 60)
    {
      timer.seconds = 0;
      
      timer.minutes++;
      if(timer.minutes >= 60)
      {
	timer.minutes = 0;
	
	timer.hours++;
	DisplayHours();
      }
      DisplayMinutes();
    }
    DisplaySeconds();
  }
  DisplayFrames();
}


void DecrementTimer(int framesPerSecond)
{
  int mode;
  int unitsPerSecond;
  
  unitsPerSecond = framesPerSecond;
  mode = GetMode();
  if(mode == FIELD_MODE)
  {
    unitsPerSecond *= 2;
  }
  
  timer.frames--;
  if(timer.frames < 0)
  {
    timer.frames = unitsPerSecond - 1;
    
    timer.seconds--;
    if(timer.seconds < 0)
    {
      timer.seconds = unitsPerSecond - 1;
      
      timer.minutes--;
      if(timer.minutes < 0)
      {
	timer.minutes = unitsPerSecond - 1;
	
	timer.hours--;
	if(timer.hours < 0)
	{
	  timer.hours = 0;
	}
	DisplayHours();
      }
      DisplayMinutes();
    }
    DisplaySeconds();
  }
  DisplayFrames();
}


void SetTimerCurrentFrame(int currentFrameNumber, int framesPerSecond)
{
  int hours;
  int minutes;
  int seconds;
  int frames;
  int mode;
  int unitsPerSecond;

  unitsPerSecond = framesPerSecond;
  mode = GetMode();
  if(mode == FIELD_MODE)
  {
    unitsPerSecond *= 2;
  }

  hours = currentFrameNumber / (3600 * unitsPerSecond);
  currentFrameNumber -= hours * 3600 * unitsPerSecond;
  
  if(hours != timer.hours)
  {
    timer.hours = hours;
    DisplayHours();
  }
  
  minutes = currentFrameNumber / (60 * unitsPerSecond);
  currentFrameNumber -= minutes * 60 * unitsPerSecond;
  
  if(minutes != timer.minutes)
  {
    timer.minutes = minutes;
    DisplayMinutes();
  }
  
  seconds = currentFrameNumber / unitsPerSecond;
  currentFrameNumber -= seconds * unitsPerSecond;
  
  if(seconds != timer.seconds)
  {
    timer.seconds = seconds;
    DisplaySeconds();
  }
  
  frames = currentFrameNumber;

  if(frames != timer.frames)
  {
    timer.frames = frames;
    DisplayFrames();
  }
}


void ResetClock(void)
{
  gettimeofday(&startTime, NULL);
  totalPausedTime = 0;
  currentFrame = 0;
}

void PauseClock(void)
{
  gettimeofday(&startPauseTime, NULL);
}


void ResumeClock(void)
{
  gettimeofday(&currentTime, NULL);
  totalPausedTime += ElapsedTime(&startPauseTime, &currentTime);
}



int TimeForNewFrame(int framesPerSecond)
{
  int mode;
  int unitsPerSecond;

  unitsPerSecond = framesPerSecond;
  mode = GetMode();
  if(mode == FIELD_MODE)
  {
    unitsPerSecond *= 2;
  }
  
  gettimeofday(&currentTime, NULL);
  if(ElapsedTime(&startTime, &currentTime) - totalPausedTime >=
     ((long)(currentFrame * 1000)/(long)unitsPerSecond))
  {
    currentFrame++;
    return(True);
  }
  return(False);
}


long ElapsedTime(before, after)
     struct timeval* before;
     struct timeval* after;
{
  return(((after->tv_sec - before->tv_sec) * 1000) +        /* Convert the seconds part to milliseconds */
         ((after->tv_usec - before->tv_usec) /1000));       /* Convert the microseconds part to milliseconds */
} /* end function ElapsedTime */
