/* $Header: /plx/source/Xclients/sun4.ow3/lovvik/src/client/motif/RTMovieTool/RCS/video.c,v 1.10 1996/01/30 20:28:18 lovvik Exp $ */

/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */

/* $Log: video.c,v $
 * Revision 1.10  1996/01/30  20:28:18  lovvik
 *   Fixed a widget resize problem.  Under mwm,
 *   video widgets should NOT set the XtNresizeParent
 *   resource to True.  This can cause a flood of
 *   resize events, and the application will hang
 *   until they are all processed.
 *
 *   Also I changed the behavior of this application
 *   to record full size PAL if the hardware is a
 *   PowerVideo board.
 *
 * Revision 1.9  1996/01/11  18:18:58  lovvik
 *   Modified the output code to work with the
 *   XVideoRGB card.  Though this software can
 *   turn on RGB output, RGB format does not
 *   work correctly when the window server is
 *   in Rtv mode.
 *
 * Revision 1.8  1996/01/04  18:39:59  lovvik
 * Added two functions for output:  StartOutput() and
 * StopOutput().  Also changed the PAL size to 8 pixels
 * less (in X and Y) than the PAL signal size.  Some configurations
 * cannot handle PAL at full size, so this is done to make
 * the movies generated with RTMovieTool compatible with
 * all hardware configurations supported by RTV.
 *
 * Also, for XVideoVIO cards, the following changes were
 * made:  Remove the "Input 1" button from the Inputs menu.
 * Set the first digitizer to PAL mode.  Perform all
 * auto-searches on the second digitizer only.  The problem
 * that these changes address is that the first digitizer on
 * this card runs slower than the second.  The work-around is
 * to do all Rtv operations on the second digitizer.
 *
 * Revision 1.7  1995/12/14  17:21:55  lovvik
 * I made two changes:
 *   1.  Fixed a bug with the following description:  If RTMovieTool is
 *       launched with a PAL input on Input 1 and then an NTSC source
 *       is selected on Input 2, the video window does not resize.  The
 *       actual problem was that InitializeVideoWidget() did not initialize
 *       the input parameters with a call to SetInputParameters().
 *   2.  Added a function called UpdateInputTypeString() that prints the
 *       video signal information in a label on the main panel.
 *
 * Revision 1.6  1995/11/21  01:22:03  lovvik
 * Made a change to ShowInput which allowed RTMovieTool to display video from
 * the second input with an XVideo board.
 *
 * Revision 1.5  1995/11/20  20:43:49  lovvik
 * Fixed a problem in ShowInput.  When the signal standard changed, the
 * video window did not resize.
 *
 * Revision 1.4  1995/11/15  02:44:18  lovvik
 * Forced the record window width and height to be a multiple of 8.  Also
 * worked on output code.
 *
 * Revision 1.3  1995/11/01  02:45:07  lovvik
 * Added functions ResizePlaybackWindow(), LockPlaybackWindow(), and
 * UnlockPlaybackWindow().
 *
 * Revision 1.2  1995/10/11  18:05:57  lovvik
 * Changed #include "InputCascade.h" to #include "IOCascade.h".
 *
 * Revision 1.1  1995/10/02  23:48:50  lovvik
 * Initial revision
 * */

static char movieToolRcsid[] = "$Header: /plx/source/Xclients/sun4.ow3/lovvik/src/client/motif/RTMovieTool/RCS/video.c,v 1.10 1996/01/30 20:28:18 lovvik Exp $";

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <display.h>
#include <image.h>
#include <alien.h>
#include <X.h>

#include <Xm/Xm.h>
#include <Xm/CascadeB.h>
#include <Xm/FileSB.h>
#include <Xm/PushB.h>
#include <Xm/Scale.h>
#include <Xm/SelectioB.h>
#include <Xm/TextF.h>
#include <Xm/ToggleBG.h>

#include <XPlxExt.h>
#include <XVideoIn.h>
#include <TCWindow.h>

#include <IOCascade.h>
#include <stdio.h>
#include <stdlib.h>

#include "globals.h"
#include "interface.h"
#include "structs.h"
#include "prototypes.h"

#ifndef True
#define True 1
#define False 0
#endif

Widget recordWidget;
Widget playbackWidget;

Widget   	outputCascadeButton = NULL;
static GC 	outputGC;
static int     	autoSearchValue = True;
static int	autoSearchChannel = PLX_INPUT_0;
char 		client_name[] = "RTMovieTool";

void UpdateInputTypeString(signalSpec*, Boolean);

void InitializeVideoWidget(void)
{
  Dimension		signalWidth;
  Dimension		signalHeight;
  InputParameters	inputParameters;
  int			signalStandard;
  int			recordRate;
  int			channel;
  int			signalFormat;
  int			inputInstance;
  Boolean		sync;
  signalSpec		signal;

  recordWidget = XtVaCreateManagedWidget("recordWidget",
					 xVideoInWidgetClass,
					 input1Shell,
					 NULL);
  /*
   * Determine the signal standard for this video input so the frame rate
   * can be determined.  NTSC video has a frame rate of 30 fps.  PAL video
   * has a frame rate of 25 fps.
   */
  XtVaGetValues(recordWidget,
		XtNchannel, &channel,
		XtNsignalFormat, &signalFormat,
		XtNinstance, &inputInstance,
		XtNsignalStandard, &signalStandard,
		XtNsignalWidth, &signalWidth,
		XtNsignalHeight, &signalHeight,
		XtNsync, &sync,
		NULL);

  inputParameters.channelToUse = channel;
  inputParameters.inputSignalStandard = signalStandard;
  inputParameters.inputSignalFormat = signalFormat;
  inputParameters.inputInstance = inputInstance;
  SetInputParameters(&inputParameters);
  signal.s.channel = channel;
  signal.s.standard = signalStandard;
  signal.s.format = signalFormat;
  signal.s.instance = inputInstance;
  UpdateInputTypeString(&signal, sync);
  
  /*
   * JPEG compression requires that the width of the image be a multiple of
   * 8.  This can be enforced by setting the width increment of the window
   * to 8 and the baseWidth to the maximum size.
   */

  XtVaSetValues(input1Shell,
		XtNbaseWidth, 16,
		XtNwidthInc, 16,
		NULL);


  /*
   * Calculate the frame rate based on the signal standard.
   */
  if (signalStandard == PLX_NTSC)
  {
    recordRate = 30;
  }
  else
  {
    recordRate = 25;
    /*
     * PAL signal size is slightly larger than what some
     * configurations can handle (i.e. Sparc 5 w/XVideoVIO).
     * To deal with this problem, we will reduce the signal
     * size by 8 pixels in each direction on XVideo cards.
     */

    if(True == XVideoCard())
    {
      signalWidth -= 8;
      signalWidth -= signalWidth % 8;
      signalHeight -= 8;
      signalHeight -= signalHeight % 8;
    }
  }

  /*
   * Make the video widget full-size by default.
   */
  SetLiveVideoSize(signalWidth, signalHeight, True);
  SetInputWidth(signalWidth);
  SetInputHeight(signalHeight);

  SetFramesPerSecond(recordRate);
  UnlockVideoWindowSize();
  SetLiveVideoStatus(True);

  /*
   * Set up the playback widget.
   */

  playbackWidget = XtVaCreateManagedWidget("playbackWidget",
					   tcWindowWidgetClass,
					   movieShell,
					   XtNwidth, signalWidth,
					   XtNheight, signalHeight,
					   NULL);

  SetCurrentMovieSize(signalWidth, signalHeight);
}


void BuildInputMenu(void)
{
  PlxInputCascade	*inputsCascade;
  PlxOutputCascade	*outputsCascade;
  plx_IO*		hardwareInfo;
  plx_IO_list*		outputList = (plx_IO_list*) NULL;
  int			channel;
  XmString		buttonString;
  Widget		button;
  Widget		outputFormatMenu;
  
  /*
   * Retrieve information about the hardware being used.
   */
  XtVaGetValues(recordWidget,
		XtNhardwareParams, &hardwareInfo,
		NULL);

  /*
   * Create a cascading menu that creates only those selections that
   * are appropriate for the hardware being used.  Also set ShowInput()
   * as the function to call when changing input settings.
   */
  inputsCascade = (PlxInputCascade*)NULL; 
  inputsCascade = PlxCreateInputCascade(inputMenu,
					(Widget)NULL,
					(Widget)NULL,
					(Widget)NULL,
					hardwareInfo,
					ShowInput);
  /*
   * Point to the list of output information for this board.  Use this
   * information to modify the user interface to include or disinclude
   * an output option.
   */
  outputList = hardwareInfo->outputs;
  if (outputList != (plx_IO_list*)NULL)
  {
    Widget input1Button;
    
    SetOutputAvailable(True);
    outputsCascade = PlxCreateOutputCascade(optionsMenu,
					    (Widget)NULL,
					    (Widget)NULL,
					    (Widget)NULL,
					    hardwareInfo,
					    NULL);
    /*
     * This is an XVideoVIO card.  Since the first digitizer is too slow
     * to run Rtv, we must ensure that the second digitizer is used instead.
     * Do this by removing the Input 1 button.
     */
    input1Button = PlxInputCascadeGetChild(inputMenu, IOC_Input1);
    if(input1Button != (Widget)NULL)
    {
      XtUnmanageChild(input1Button);
      /*
       * Set the auto search value so it only looks at the
       * second input.
       */
      autoSearchValue = AutoSearchInput1;
      autoSearchChannel = PLX_INPUT_1;

      /*
       * The first digitizer of the XVideoVIO card should be set to
       * PAL mode.  For some reason, this allows the second digitizer
       * to run fast enough to handle Rtv operations.
       */
      SetInput1ToPAL();

      /*
       * Auto search on the second digitizer.
       */
      ShowInput((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);
      XtVaGetValues(recordWidget,
		    XtNchannel, &channel,
		    NULL);

      if(channel != PLX_INPUT_1)
      {
	/*
	 * The auto search did not find anything.  Set the input to NTSC.
	 * If no signal is found, the label that describes the video
	 * source indicates Input 1 (which is illegal).
	 */
	ConfigureInputForMovie(30);
      }
      UnlockVideoWindowSize();
    }

    /*
     * Unmap components of the output menu.
     */
    outputCascadeButton = (Widget)PlxOutputCascadeGetChild(optionsMenu, IOC_Output1);
    if(outputCascadeButton != (Widget)NULL)
    {
      buttonString = XmStringCreateSimple("Output 1 Off");
      XtVaSetValues(outputCascadeButton,
		    XmNlabelString, buttonString,
		    NULL);
      XmStringFree(buttonString);

      /*
       * Destroy the submenu for each output format.
       */
      button = (Widget)PlxOutputCascadeGetChild(optionsMenu, IOC_Output1 | IOC_FormatComp);
      if(button != (Widget)NULL)
      {
	XtVaSetValues(button,
		      XmNsubMenuId, NULL,
		      NULL);
	XtAddCallback(button, XmNactivateCallback, OutputButtonPressed, (XtPointer)PLX_COMP);
      }

      button = (Widget)PlxOutputCascadeGetChild(optionsMenu, IOC_Output1 | IOC_FormatYC);
      if(button != (Widget)NULL)
      {
	XtVaSetValues(button,
		      XmNsubMenuId, NULL,
		      NULL);
	XtAddCallback(button, XmNactivateCallback, OutputButtonPressed, (XtPointer)PLX_YC);
      }

      button = (Widget)PlxOutputCascadeGetChild(optionsMenu, IOC_Output1 | IOC_FormatYUV);
      if(button != (Widget)NULL)
      {
	XtVaSetValues(button,
		      XmNsubMenuId, NULL,
		      NULL);
	XtAddCallback(button, XmNactivateCallback, OutputButtonPressed, (XtPointer)PLX_YUV);
      }

      button = (Widget)PlxOutputCascadeGetChild(optionsMenu, IOC_Output1 | IOC_FormatRGB);
      if(button != (Widget)NULL)
      {
	XtVaSetValues(button,
		      XmNsubMenuId, NULL,
		      NULL);
	XtAddCallback(button, XmNactivateCallback, OutputButtonPressed, (XtPointer)PLX_RGB);
      }
      
      /*
       * Add an 'Off' button to the format menu.
       */
      outputFormatMenu = (Widget)PlxOutputCascadeGetChild(optionsMenu,
							  IOC_Output1 | IOC_FormatMenu);
      if(outputFormatMenu != (Widget)NULL)
      {
	XmString 	buttonString;
	Widget		outputOffButton;
	
	buttonString = XmStringCreateSimple("Off");
	outputOffButton = XtVaCreateManagedWidget("outputOffButton",
						  xmPushButtonWidgetClass,
						  outputFormatMenu,
						  XmNlabelString, buttonString,
						  NULL);
	XmStringFree(buttonString);
	XtAddCallback(outputOffButton,
		      XmNactivateCallback,
		      OutputButtonPressed,
		      (XtPointer)PLX_IO_OFF);
      }
    }
  }
  else
  {
    SetOutputAvailable(False);
  }
  free(inputsCascade);
  return;
} /* end function BuildInputMenu */


void ShowInput(Widget		w,
	       XtPointer	clientData,
	       XtPointer	unusedCallData)
{
  /*
   * If the clientData is zero, the user hit the First Available button.  In
   * this case, we should turn auto-search on. 
   */

  signalSpec		selectedSignal;
  InputParameters	inputParameters;
  int width;
  int height;
  int			recordRate;
  int			previousStandard;
  Dimension		signalWidth;
  Dimension		signalHeight;
  Boolean		sync;
  signalSpec		signal;

  if(GetOutputStatus() == True)
  {
    /*
     * Stop the digitization of output.
     */
    XPlxVideoStop(XtDisplay(recordWidget),
		  XtWindow(recordWidget),
		  outputGC);
    
    XPlxVideoOutputSelect(XtDisplay(playbackWidget),
			  XtWindow(playbackWidget),
			  outputGC,
			  PLX_IO_OFF,
			  PLX_NTSC,
			  PLX_COMP,
			  0);

    SetOutputStatus(False);
  }
  
  GetInputParameters(&inputParameters);
  previousStandard = inputParameters.inputSignalStandard;
  
  width = GetInputWidth();
  height = GetInputHeight();
  
  if(clientData == (XtPointer)0)
  {
    XtVaSetValues(recordWidget,
		  XtNchannel, PLX_IO_OFF,
		  NULL);
    
    XtVaSetValues(recordWidget,
		  XtNautoSearch, autoSearchValue,
		  XtNchannel, autoSearchChannel,
		  XtNsignalStandard, PLX_NTSC,
		  XtNsignalFormat, PLX_COMP,
		  XtNinstance, 0,
		  XtNwidth, width,
		  XtNheight, height,
		  NULL);

  }
  else
  {
    /*
     * Get the signal information from the client data.
     */
    selectedSignal.i = (unsigned int)clientData;
    inputParameters.inputSignalStandard = selectedSignal.s.standard;
    inputParameters.channelToUse = selectedSignal.s.channel;
    inputParameters.inputSignalFormat = selectedSignal.s.format;
    inputParameters.inputInstance = selectedSignal.s.instance;
    
    /*
     * The user specified the exact input source.  Do not change
     * that source even if there is no synchronization on it.
     */
    XtVaSetValues(recordWidget, XtNautoSearch, False, NULL);

    /*
     * Disable video before changing channels.  This is a cosmetic
     * addition.
     */
    XtVaSetValues(recordWidget, XtNchannel, PLX_IO_OFF, NULL);

    /*
     * Set the new signal source in the widget.
     */
    XtVaSetValues(recordWidget,
		  XtNchannel, inputParameters.channelToUse,
		  XtNsignalStandard, inputParameters.inputSignalStandard,
		  XtNsignalFormat, inputParameters.inputSignalFormat,
		  XtNinstance, inputParameters.inputInstance,
		  XtNwidth, width,
		  XtNheight, height,
		  NULL);
  }

  XtVaGetValues(recordWidget,
		XtNchannel, &inputParameters.channelToUse,
		XtNsignalStandard, &inputParameters.inputSignalStandard,
		XtNsignalFormat, &inputParameters.inputSignalFormat,
		XtNinstance, &inputParameters.inputInstance,
		XtNsync, &sync,
		NULL);

  SetInputParameters(&inputParameters);
  signal.s.channel = inputParameters.channelToUse;
  signal.s.standard = inputParameters.inputSignalStandard;
  signal.s.format = inputParameters.inputSignalFormat;
  signal.s.instance = inputParameters.inputInstance;
  UpdateInputTypeString(&signal, sync);

  /*
   * Determine the signal standard for this video input so the frame
   * rate can be determined.
   */
  if (inputParameters.inputSignalStandard == PLX_NTSC)
  {
    recordRate = 30;
  }
  else
  {
    recordRate = 25;
  }
  SetFramesPerSecond(recordRate);

  if(previousStandard != inputParameters.inputSignalStandard)
  {
    /*
     * The video signal was switched NTSC->PAL or PAL->NTSC.
     * resize the video window.
     */
    XtVaGetValues(recordWidget,
		  XtNsignalWidth, &signalWidth,
		  XtNsignalHeight, &signalHeight,
		  NULL);

    if(inputParameters.inputSignalStandard != PLX_NTSC)
    {
      /*
       * PAL signal size is slightly larger than what some
       * configurations can handle (i.e. Sparc 5 w/XVideoVIO).
       * To deal with this problem, we will reduce the signal
       * size by 8 pixels in each direction on XVideo boards.
       */
      if(True == XVideoCard())
      {
	signalWidth -= 8;
	signalWidth -= signalWidth % 8;
	signalHeight -= 8;
	signalHeight -= signalHeight % 8;
      }
    }
    SetInputWidth(signalWidth);
    SetInputHeight(signalHeight);
  
    /*
     * Make the video widget full-size by default.
     */
    SetLiveVideoSize(signalWidth, signalHeight, True);
  }

  if(GetMovieToolStatus() == NO_MOVIE)
  {
    XtPopup(input1Shell, XtGrabNone);
  }
  
  SetLiveVideoStatus(True);
  SetMovieToolTitle();
  return;
} /* end function ShowInput */


void ChangeInputSize(int size)
{
  Dimension			signalWidth;
  Dimension			signalHeight;
  int				signalStandard;
  int				divisor;

  switch(size)
  {
    case FULL_SIZE:
    {
      divisor = 1;
      break;
    }
      
    case QUARTER_SIZE:
    {
      divisor = 2;
      break;
    }
      
    default:
    {
      return;
    }
  }

  /*
   * Get the full size of the video widget's input signal.
   */
  XtVaGetValues(recordWidget,
		XtNsignalWidth, &signalWidth,
		XtNsignalHeight, &signalHeight,
		XtNsignalStandard, &signalStandard,
		NULL);

    if(signalStandard != PLX_NTSC)
    {
      /*
       * PAL signal size is slightly larger than what some
       * configurations can handle (i.e. Sparc 5 w/XVideoVIO).
       * To deal with this problem, we will reduce the signal
       * size by 8 pixels in each direction on XVideo boards.
       */
      if(True == XVideoCard())
      {
	signalWidth -= 8;
	signalWidth -= signalWidth % 8;
	signalHeight -= 8;
	signalHeight -= signalHeight % 8;
      }
    }
  
  /*
   * Change the size of the video widget based on the signal size and
   * the requested window size (Full or Quarter).
   */
  SetLiveVideoSize(signalWidth/divisor, signalHeight/divisor, True); 
  SetInputWidth(signalWidth);
  SetInputHeight(signalHeight);
  
  return;
  
}


void StartOutput(void)
{
  InputParameters 	inputParameters;
  OutputParameters 	outputParameters;
  int			width;
  int			height;
    

  GetInputParameters(&inputParameters);
  GetOutputParameters(&outputParameters);
  outputParameters.outputSignalStandard = inputParameters.inputSignalStandard;

  if(outputGC == (GC)NULL)
  {
    outputGC = XCreateGC(XtDisplay(playbackWidget), XtWindow(playbackWidget), 0, 0);
  }

  /*
   * Output Composite NTSC
   */
  if(outputParameters.outputSignalStandard == PLX_NTSC) 
  {
    outputParameters.outputSignalWidth = 640;
    outputParameters.outputSignalHeight = 480;
  }
  
  /*
   * Output Composite PAL
   */
  if(outputParameters.outputSignalStandard == PLX_PAL)
  {
    outputParameters.outputSignalWidth = 768;
    outputParameters.outputSignalHeight = 575;
  }
  
  /*
   * Output Composite SECAM
   */
  if(outputParameters.outputSignalStandard == PLX_SECAM)
  {
    outputParameters.outputSignalWidth = 768;
    outputParameters.outputSignalHeight = 575;
  }

  SetOutputParameters(&outputParameters);
  XPlxVideoOutputSelect(XtDisplay(playbackWidget),
			XtWindow(playbackWidget),
			outputGC,
			PLX_OUTPUT_0,
			outputParameters.outputSignalStandard,
			outputParameters.outputSignalFormat,
			0);
  GetCurrentMovieSize(&width, &height);
  XPlxVideoScaleOutput(XtDisplay(playbackWidget),
		       XtWindow(playbackWidget),
		       outputGC,
		       0, 0,
		       width, height,
		       0, 43,
		       outputParameters.outputSignalWidth,
		       outputParameters.outputSignalHeight);
  SetOutputStatus(True);
}

void StopOutput(void)
{
  if(GetOutputStatus() == True)
  {
    /*
     * Stop the digitization of output.
     */
    XPlxVideoOutputSelect(XtDisplay(playbackWidget),
			  XtWindow(playbackWidget),
			  outputGC,
			  PLX_IO_OFF,
			  PLX_NTSC,
			  PLX_COMP,
			  0);

    SetOutputStatus(False);
  }
}

void StopVideo(void)
{
  if(GetLiveVideoStatus() == True)
  {
    XtVaSetValues(recordWidget,
		  XtNchannel, PLX_IO_OFF,
		  NULL);
    SetLiveVideoStatus(False);
  }
}


void StartVideo(void)
{
  InputParameters inputParameters;
  int brightness;
  int contrast;
  int hue;
  int saturation;

  if(GetOutputStatus() == True)
  {
    /*
     * Stop the digitization of output.
     */
    XPlxVideoOutputSelect(XtDisplay(playbackWidget),
			  XtWindow(playbackWidget),
			  outputGC,
			  PLX_IO_OFF,
			  PLX_NTSC,
			  PLX_COMP,
			  0);

    SetOutputStatus(False);
  }
  
  GetInputParameters(&inputParameters);

  brightness = GetBrightness();
  contrast = GetContrast();
  hue = GetHue();
  saturation = GetSaturation();
  
  XtVaSetValues(recordWidget, PLX_IO_OFF, NULL);
  XtVaSetValues(recordWidget,
		XtNchannel, inputParameters.channelToUse,
		XtNsignalStandard, &inputParameters.inputSignalStandard,
		XtNsignalFormat, &inputParameters.inputSignalFormat,
		XtNinstance, &inputParameters.inputInstance,
		XtNbrightness, brightness,
		XtNcontrast, contrast,
		XtNhue, hue,
		XtNsaturation, saturation,
		NULL);
  SetLiveVideoStatus(True);
}

       
void SetColorSettings(int brightness, int contrast, int hue, int saturation)
{
  if (brightness < 0 || brightness > 255 ||
      contrast < 0 || contrast > 255 ||
      hue < 0 || hue > 255 ||
      saturation < 0 || saturation > 255)
  {
    PrintDiagnostics("Error!  SetColorSettings called with bad value.");
    return;
  }
  
  XtVaSetValues(recordWidget,
		XtNbrightness, brightness,
		XtNcontrast, contrast,
		XtNhue, hue,
		XtNsaturation, saturation,
		NULL);
}


void SetVideoBrightness(int brightness)
{
  if(brightness < 0 || brightness > 255)
  {
    PrintDiagnostics("SetVideoBrightness called with bad value!");
    return;
  }
  XtVaSetValues(recordWidget,
		XtNbrightness, brightness,
		NULL);
}


void SetVideoContrast(int contrast)
{
  if(contrast < 0 || contrast > 255)
  {
    PrintDiagnostics("SetVideoContrast called with bad value!");
    return;
  }
  
  XtVaSetValues(recordWidget,
		XtNcontrast, contrast,
		NULL);
}


void SetVideoHue(int hue)
{
  if(hue < 0 || hue > 255)
  {
    PrintDiagnostics("SetVideoHue called with bad value!");
    return;
  }
  
  XtVaSetValues(recordWidget,
		XtNhue, hue,
		NULL);
}



void SetVideoSaturation(int saturation)
{
  if(saturation < 0 || saturation > 255)
  {
    PrintDiagnostics("SetVideoSaturation called with bad value!");
    return;
  }
  
  XtVaSetValues(recordWidget,
		XtNsaturation, saturation,
		NULL);
}


void ResizeVideoWindow(void)
{
  int width;
  int height;
  
  GetLiveVideoSize(&width, &height);
  
  if(width > 0 && height > 0)
  {
    XtVaSetValues(recordWidget,
		  XtNwidth, width,
		  XtNheight, height,
		  NULL);
  }
  else
  {
    char message [100];
    
    sprintf(message, "Error!  New width = %d, new height = %d", width, height);
    PrintDiagnostics(message);
  }
}


void ResizePlaybackWindow(void)
{
  int width=0;
  int height=0;
  
  GetCurrentMovieSize(&width, &height);
  
  
  if(width > 0 && height > 0)
  {
    UnlockPlaybackWindowSize();
    XtVaSetValues(movieShell,
		  XtNwidth, width,
		  XtNheight, height,
		  NULL);
  }
  else
  {
    char message [100];
    
    sprintf(message, "Error!  New width = %d, new height = %d", width, height);
    PrintDiagnostics(message);
  }
}


void LockPlaybackWindowSize(void)
{
  Dimension currentWidth;
  Dimension currentHeight;

  XtVaGetValues(movieShell,
		XtNwidth, &currentWidth,
		XtNheight, &currentHeight,
		NULL);
  
  XtVaSetValues(movieShell,
		XtNminWidth, currentWidth,
		XtNminHeight, currentHeight,
		XtNmaxWidth, currentWidth,
		XtNmaxHeight, currentHeight,
		NULL);
}


void UnlockPlaybackWindowSize(void)
{
  XtVaSetValues(movieShell,
		XtNminWidth, 320,
		XtNminHeight, 240,
		XtNmaxWidth, 1000,
		XtNmaxHeight, 1000,
		NULL);
}



void LockVideoWindowSize(void)
{
  Dimension currentWidth;
  Dimension currentHeight;

  XtVaGetValues(input1Shell,
		XtNwidth, &currentWidth,
		XtNheight, &currentHeight,
		NULL);
  
  XtVaSetValues(input1Shell,
		XtNminWidth, currentWidth,
		XtNminHeight, currentHeight,
		XtNmaxWidth, currentWidth,
		XtNmaxHeight, currentHeight,
		NULL);
}


void UnlockVideoWindowSize(void)
{
  int 		minWidth;
  int 		minHeight;
  int 		maxWidth;
  int 		maxHeight;
  Dimension 	signalWidth;
  Dimension 	signalHeight;
  int		signalStandard;
  InputParameters inputParameters;
  

  XtVaGetValues(recordWidget,
		XtNsignalWidth, &signalWidth,
		XtNsignalHeight, &signalHeight,
  		XtNsignalStandard, &signalStandard, 
		NULL);

  /*
   * The videoIn widget is reporting the wrong signal standard
   * in the following scenario:  Record, playback with output,
   * Record again.  If you look at the signal standard, it will
   * not correspond to anything (-268440156).  The fix is to see
   * if the standard is valid. If the standard is not valid,
   * use the current standard.
   */
  switch(signalStandard)
  {
    case PLX_NTSC:
    case PLX_PAL:
    case PLX_SECAM:
    {
      /*
       * Do nothing - the standard is valid.
       */
      break;
    }
    default:
    {
      /*
       * The standard is not valid.  Use the current standard.
       */
      GetInputParameters(&inputParameters);
      signalStandard = inputParameters.inputSignalStandard;
    }
  }

  if(signalStandard != PLX_NTSC)
  {
    /*
     * PAL signal size is slightly larger than what some
     * configurations can handle (i.e. Sparc 5 w/XVideoVIO).
     * To deal with this problem, we will reduce the signal
     * size by 8 pixels in each direction on XVideo boards.
     */
    if(True == XVideoCard())
    {
      signalWidth -= 8;
      signalWidth -= signalWidth % 8;
      signalHeight -= 8;
      signalHeight -= signalHeight % 8;
    }
  }

  minWidth = ((int)signalWidth/2);
  minHeight = ((int)signalHeight/2);
  maxWidth = ((int)signalWidth);
  maxHeight = ((int)signalHeight);
  
  XtVaSetValues(input1Shell,
		XtNminWidth, minWidth,
		XtNminHeight, minHeight,
		XtNmaxWidth, maxWidth,
		XtNmaxHeight, maxHeight,
		NULL);
}


void UpdateInputTypeString(signalSpec*	signalInfoPtr,
			   Boolean	sync)
{
  char		channelString[16];
  char		formatString[16];
  char		inputStatusString[64];
  char		instanceString[3];
  XmString	label;
  char		standardString[8];
  char		syncString[16];
  
  sprintf(instanceString, "%d", signalInfoPtr->s.instance + 1);	    /* Print the instance number */
  switch (signalInfoPtr->s.channel)
  {
   case PLX_INPUT_0:
    sprintf(channelString, "Input 1");
    break;
   case PLX_INPUT_1:
    sprintf(channelString, "Input 2");
    break;
   case PLX_IO_OFF:
    sprintf(channelString, "(Off)");
    break;
  }
  switch (signalInfoPtr->s.format)				    /* Print the appropriate label on the popup menu cascade... */
  {								    /* ...button, according to signal format in effect. */
   case PLX_COMP:						    /* One wire: composite video */
    sprintf(formatString, "Composite");
    break;
   case PLX_YC:							    /* Two wires: S-Video */
    sprintf(formatString, "S-VHS (YC)");
    break;
   case PLX_YUV:						    /* Three wires */
    sprintf(formatString, "YUV");
    break;
   case PLX_RGB:						    /* Three wires: Red-Green-Blue video */
    sprintf(formatString, "RGB");
    break;
  }
  switch (signalInfoPtr->s.standard)				    /* Print the appropriate label on the popup menu cascade... */
  {								    /* ...button, according to the actual signal standard... */
   case PLX_NTSC:						    /* ...in effect. */
    sprintf(standardString, "NTSC");
    break;
   case PLX_PAL:
    sprintf(standardString, "PAL");
    break;
   case PLX_SECAM:
    sprintf(standardString, "SECAM");
    break;
  }
  if (True == sync)
    sprintf(syncString, " ");
  else
    sprintf(syncString, " (No Sync)");
  sprintf(inputStatusString, "%s : %s : %s : %s%s",
	  channelString,
	  formatString,
	  instanceString,
	  standardString,
	  syncString);
  label = XmStringCreateLocalized(inputStatusString);		    /* Create the string that will be used as a label for the... */
  XtVaSetValues(inputSignalTypeLabel, XmNlabelString, label, NULL); /* ...Label widget showing the new input information. */
  XmStringFree(label);
  return;
} /* end function UpdateInputTypeString */


void SetInput1ToPAL(void)
{
  signalSpec		signal;

  signal.s.channel = PLX_INPUT_0;
  signal.s.format = PLX_COMP;
  signal.s.instance = 0;
  signal.s.standard = PLX_PAL;
  
  ShowInput(recordWidget, (XtPointer)signal.i, (XtPointer)NULL);
}

void ConfigureInputForMovie(int fps)
{
  signalSpec		signal;

  signal.s.channel = PLX_INPUT_0;
  signal.s.format = PLX_COMP;
  signal.s.instance = 0;

  if(GetOutputAvailable() == True)
  {
    signal.s.channel = PLX_INPUT_1;
  }

  if(fps == 30)
  {
    signal.s.standard = PLX_NTSC;
  }
  else
  {
    signal.s.standard = PLX_PAL;
  }
  
  ShowInput(recordWidget, (XtPointer)signal.i, (XtPointer)NULL);
}

int ShowOutputOnPlayback(void)
{
  OutputParameters outputParameters;

  GetOutputParameters(&outputParameters);
  return(!(outputParameters.outputSignalFormat == PLX_IO_OFF));
}


void OutputButtonPressed (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  OutputParameters 	outputParameters;
  char outputButtonString[100];
  char formatString[100];
  XmString labelString;
  

  GetOutputParameters(&outputParameters);
  outputParameters.outputSignalFormat = (int)client_data;
  SetOutputParameters(&outputParameters);
  switch(outputParameters.outputSignalFormat)
  {
    case PLX_IO_OFF:
    {
      /* 
       * Turn off output.
       */
      sprintf(formatString, "Off");
      break;
    }
    case PLX_COMP:
    {
      /*
       * Composite output.
       */
      sprintf(formatString, "Composite");
      break;
    }
    case PLX_YC:
    {
      /*
       * YC output.
       */
      sprintf(formatString, "YC");
      break;
    }
    case PLX_YUV:
    {
      /*
       * YUV output.
       */
      sprintf(formatString, "YUV");
      break;
    }
    case PLX_RGB:
    {
      /*
       * RGB output.
       */
      sprintf(formatString, "RGB");
      break;
    }
  }
  
  sprintf(outputButtonString, "Output 1: %s", formatString);
  labelString = XmStringCreateSimple(outputButtonString);
  XtVaSetValues(outputCascadeButton,
		XmNlabelString, labelString,
		NULL);
  XmStringFree(labelString);
  if(outputParameters.outputSignalFormat != PLX_IO_OFF)
  {
    StartOutput();
    SetLiveVideoStatus(False);
  }
  else
  {
    StopOutput();
  }
}


Boolean XVideoCard(void)
{
  Window	window;
  plx_IO 	*hardwareInfo;
  plx_IO_list 	*inputPtr;
  plx_IO_list 	*outputPtr;
  int 		channel;
  GC		videoGC;
  int    	hardwareCode = 0;

  /*
   * This function determines if you are using an XVideo
   * VIO card based on the number of input and output
   * channels.  Two input channels and one output channel
   * indicates that you are using an XVideoVIO card.
   */
  window = XDefaultRootWindow(display);
  videoGC = XCreateGC(display, window, 0, NULL);
  hardwareInfo = XPlxQueryConfig(display, window, videoGC);
  if(hardwareInfo == NULL)
  {
    /*
     * This is not a Parallax video card!
     */
    return(False);
  }
  
  /*
   * Count the input channels:
   */
  inputPtr = hardwareInfo->inputs;
  while(inputPtr != NULL)
  {
    channel = inputPtr->channel;
    /*
     * Get rid of the instance information.
     */
    channel &= 0x00FF;

    hardwareCode |= (1 << (channel - 1));
    inputPtr = inputPtr->next_IO;
  }

  /*
   * Count the output channels:
   */
  outputPtr = hardwareInfo->outputs;
  while(outputPtr != NULL)
  {
    channel = outputPtr->channel;
    /*
     * Get rid of the instance information.
     */
    channel &= 0x00FF;

    hardwareCode |= ((1 << channel) << 8);
    outputPtr = outputPtr->next_IO;
  }

  XFree(videoGC);
  
  /*
   * hardwareCode:
   *                       0xFFFF
   *                          ^ ^
   *   Indicate the outputs  /   \ Indicate the inputs.
   */

  return(hardwareCode == 0x0103);
}


void SaveImageFileInFormat(Display*	display,
			   Window	imageWindow,
			   Dimension	width,
			   Dimension	height,
			   char*	fileFormat,
			   char*	fileName)
{
  char		windowName[256];
  Image*	theImage = (Image*)NULL;
  
  sprintf(windowName, "%dx", (int)imageWindow);
  theImage = ReadXImage(windowName,				    /* Use ImageMagick routine to digitize the contents of an... */
			XDisplayName(NULL),			    /* ...X window, the one that will be written to disk. */
			False, False, False);
  if (theImage == (Image*)NULL)					    /* Did the image grab fail? */
    return;							    /* Yes, return without doing anything more */
  sprintf(theImage->filename, "%s:%s", fileFormat, fileName);
  WriteAlienImage(theImage);					    /* Write the Sun Raster file to disk */
  DestroyImage(theImage);					    /* Done with the space needed to store the image */
  return;
}								    /* end function SaveImageFileInFormat */
