/* $Header: /plx/source/Xclients/generic/RCS/VideoTool.c,v 1.42 1996/01/22 20:45:22 drapeau Exp $ */
/* $Log: VideoTool.c,v $
 * Revision 1.42  1996/01/22  20:45:22  drapeau
 * Added support for the "Capture Preferences" popup dialog shell.
 *
 * Revision 1.41  1996/01/22  07:56:39  drapeau
 * Added support for a new XVideoIn widget resource, "stillDelay".  The
 * new resource allows the programmer to set the delay time between the
 * time the widget freezes the digitizer and the time the widget grabs a
 * still image from its window.
 *
 * Revision 1.40  1995/10/11  02:28:11  drapeau
 * (From Paul Lovvik) Fixed an error in the way video widgets resize
 * under the OPEN LOOK Window Manager.  The code assures that a 4-to-3
 * aspect ratio is maintained, which has the side effect of assuring that
 * live video windows always remain the full height they should be, as
 * opposed to part of the full height because the window manager is not
 * honoring resize requests.
 *
 * Revision 1.39  1995/10/05  04:36:24  drapeau
 * Allowed the "Select Window" button to be used now, since the ability
 * to select and track a specific window for the purpose of output is now
 * implemented.
 *
 * Revision 1.38  1995/08/18  01:55:50  drapeau
 * Added code to put the company logo in the application's "About..."
 * panel.
 *
 * Revision 1.37  1995/08/17  19:49:55  drapeau
 * Add support for an "About..." panel.
 *
 * Revision 1.36  1995/06/23  19:20:30  drapeau
 * Fixed an error in the user interface: the user interface code now
 * correctly shows the current zoom factor.
 *
 * Revision 1.35  1995/05/15  16:53:06  drapeau
 * Minor changes to remove variables that are no longer necessary (i.e.,
 * for input selection, which has been superceded by the new cascading
 * menu code written by Dave Arnold).
 *
 * Also, added a WorkProc that takes care of opening a JPEG file when
 * specified on the command line.  This fixes an earlier problem in which
 * the size of the JPEG image shell was incorrect, seemingly because the
 * callback for opening the image was called before XtAppMainLoop() was
 * entered.  Calling the image-opening function from a WorkProc seems to
 * fix the problem by waiting for XtAppMainLoop() to be entered first.
 * Unknown why.
 *
 * Revision 1.34  95/05/02  20:37:23  20:37:23  drapeau ()
 * Added support for saving and displaying zoomed JPEG images.
 * 
 * Revision 1.33  1995/04/21  22:24:22  drapeau
 * Cosmetic change only: fixed the formatting of the help message.
 *
 * Revision 1.32  1995/04/21  20:35:41  drapeau
 * Removed extraneous debugging message now that multiple board support
 * works.
 *
 * Revision 1.31  1995/04/21  20:11:17  drapeau
 * Added support for multiple Plx boards on a single screen.
 *
 * Revision 1.30  1995/04/01  01:36:51  drapeau
 * Added support for quitting the application directly from the window
 * manager's title bar menu.
 *
 * Revision 1.29  1994/10/14  21:13:55  drapeau
 * Added variables to keep track of whether optional signal types are
 * available.
 *
 * Revision 1.28  1994/10/14  03:04:11  drapeau
 * Added the infoShell, a currently unused Shell widget for displaying
 * informational messages (such as when saving a TIFF image, a process
 * that could take a while).
 *
 * Revision 1.27  1994/09/30  22:29:04  drapeau
 * Two minor changes:
 *
 * 1) Used the proper Motif 1.2 way of creating simple strings.
 *
 * 2) Accounted for the case that no valid video signal is available on
 *    Input 1; if not, the program should launch with the Input 1 toggle
 *    disabled (previous versions automatically enabled it whether or not
 *    a valid video signal was found).
 *
 * Revision 1.26  1994/09/30  16:54:46  drapeau
 * Disabled help for this version, since the help text has not yet been
 * implemented for either the Sun or HP platform.
 *
 * Revision 1.25  1994/09/23  06:45:04  drapeau
 * Added code to handle detection of input signals.  If inputs are not
 * enabled, the input shells are not popped up and saving of stills is
 * not allowed.
 *
 * Revision 1.24  1994/09/14  06:02:02  drapeau
 * Significant changes to the implementation of the rubberband code.  It
 * works cleanly now.
 *
 * Revision 1.23  1994/09/13  08:03:24  drapeau
 * Minor change, in width of rubberband line to be drawn when changing
 * the video output rectangle.
 *
 * Revision 1.22  1994/09/04  07:12:09  drapeau
 * Two changes:
 *
 * 1) Video input shells now respond correctly when the window manager
 *    tries to delete them; they unmap and then turn off the
 *    corresponding toggle on the main panel.
 *
 * 2) Removed the force of shared memory off on the JPEG widget, since
 *    the JPEG shared memory error has now been fixed.
 *
 * Revision 1.21  1994/09/03  01:57:21  drapeau
 * Added a variable "autoSearch" telling the application whether to
 * enable or disable tho "autoSearch" feature of the VideoIn widgets.  If
 * autoSearch is off, the widget will look for the requested video signal
 * but fail if not immediately found.  If autoSearch is on, the widget
 * will look at all possible video input configurations until a valid
 * sync signal is found or until exhausting all possible input types.
 *
 * Revision 1.20  1994/08/27  05:13:19  drapeau
 * Simple cosmetic changes, and disabled output by default (the user must
 * explicitly select the output toggle to turn on output).
 *
 * Revision 1.19  1994/08/24  02:02:33  drapeau
 * Added support for a command-line argument, for the HP VUE environment.
 * If an argument is passed in, VideoTool will interpret it as the name
 * of a file to display (a JFIF or JPEGMovie file).
 *
 * Revision 1.18  1994/08/19  02:36:57  andyg
 * Added a stmt to disable the "Select Window" feature of video
 * output generation.
 *
 * Revision 1.17  1994/08/14  01:42:23  drapeau
 * Three minor changes:
 *
 * 1) Moved the dimming of the "Save As TIFF" button to the
 *    OsSpecificInit() function.
 *
 * 2) Disabled shared memory in the JPEG widget.
 *
 * 3) Added a function call for OS-specific initialization.
 *
 * Revision 1.16  1994/08/10  03:12:52  drapeau
 * Removed the creation of the output widget and placed it in a module
 * that can detect whether actually to create the widget.  The widget
 * should not be created if no output function is present on the board.
 *
 * Revision 1.15  1994/08/01  18:33:23  drapeau
 * Moved the PlaceWindows() function into a different module, one that
 * contains general utility functions.
 *
 * Also made minor modifications to the code to facilitate better
 * selection of inputs.
 *
 * Revision 1.14  1994/06/09  21:27:34  drapeau
 * Disabled several features to support the first release of the HP
 * software product.
 *
 * Revision 1.13  1994/04/23  03:51:54  drapeau
 * Added an alert panel to display error messages.
 *
 * Revision 1.12  1994/04/16  00:36:28  drapeau
 * Restored the "input2Widget" video input widget to display channel
 * "INPUT_1".  It had previously been changed to display channel
 * "INPUT_0" to accommodate the HP hardware, but this change was always
 * meant to be short-term.
 *
 * Revision 1.11  1994/03/22  04:50:30  drapeau
 * Removed lots of code as part of a restructuring of the application.
 * The interface-only portion of the code moved to a new module called
 * "interface.c".
 *
 * Revision 1.1  1993/12/01  03:02:40  drapeau
 * Initial revision
 * */
static char videoToolRcsId[] = "$Header: /plx/source/Xclients/generic/RCS/VideoTool.c,v 1.42 1996/01/22 20:45:22 drapeau Exp $";
/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/*
** Generated by X-Designer 
*/
/*
**LIBS: -lXm -lXt -lX11
*/
#include "VideoTool.h"

Widget		input1Widget = (Widget)NULL;
Widget		input2Widget = (Widget)NULL;
Widget		outputWidget = (Widget)NULL;
Widget		jpegWidget = (Widget)NULL;
Widget		selectedSettingsWidget = (Widget)NULL;
Widget		selectedVideoWidget = (Widget)NULL;
Widget		rootWindowWidget = (Widget)NULL;
char*		fileFormat = (char*)NULL;
XRectangle	outputSourceRect;     
Cursor		handCursor;
GC		rubberBandGC;
int		rubberBandMode;					    /* Tells whether resizing or moving the output widget */
XtAppContext	app_context;
Display*	display;					    /*  Display             */
char*		errorMessage;
Boolean		autoSearch;
Atom		deleteWindowAtom;
int		lastInstanceSelected;
Boolean		input1Sync;
Boolean		input2Sync;
Boolean		input1ShellPoppedUp;
Boolean		input2ShellPoppedUp;
int		inputSignalFormat;
Cursor		watchCursor;
AppResources	vtResources;					    /* VideoTool-specific X Resources */
XmFileSelectionBoxCallbackStruct	imageName;		    /* Name of default JPEG file to try to open on app. launch */
int		zoomFactor;					    /* Zoom ratio for this connection */
XRectangle	displayDestRect;				    /* Rectangle for showing (zoomed) JPEG images */
PlxInputCascade*	inputMenuCascade;			    /* Automatically generated for this hardware */
PlxOutputCascade*	outputMenuCascade;			    /* Automatically generated for this hardware */
Boolean			outputAvailable;			    /* Whether output is available on this hardware */


XtResource	videoToolResources[] =				    /* VideoTool-specific X Resources: default values */
{
{
  XtNboard,
  XtCBoard,
  XtRInt,
  sizeof(int),
  XtOffsetOf(AppResources, board),
  XtRImmediate,
  (XtPointer) 0,						    /* By default, use the 1st available board */
},
{
  XtNhelp,
  XtCHelp,
  XtRBoolean,
  sizeof(Boolean),
  XtOffsetOf(AppResources, help),
  XtRImmediate,
  (XtPointer) False,
},
};

XrmOptionDescRec	vtOptions[] =				    /* VideoTool command-line options */
{
{"-board",	"*board",	XrmoptionSepArg,	NULL},
{"-help",	"*help",	XrmoptionNoArg,		"True"},
};



void Usage(void)
{
  printf("VideoTool may be launched like this:\n");
  printf("VideoTool <options> <jpegFileName>\n");
  printf("Note: The following options are also X resources; for example,\n");
  printf("-board:\tSpecifies which board to use if you have installed multiple\n");
  printf("\tParallax Graphics boards in the same computer.\n");
  printf("\tFor VideoTool to use the first board, type \"-board 0\";\n");
  printf("\tto use the second board, type \"-board 1\".\n");
  printf("-----------\n\n");
  return;
} /* end function Usage */



void CheckOptions(int argc, char** argv)
{
  int	counter;
  
  XtVaGetApplicationResources(mainWindow,			    /* Get user-specified X resources or command-line options */
			      &vtResources,
			      videoToolResources,
			      XtNumber(videoToolResources),
			      NULL);
  if (vtResources.help == True)					    /* Was help requested? */
  {
    Usage();							    /* Yes, print the help and exit immediately */
    exit(-1);
  }
  imageName.value = (XmString)NULL;				    /* By default, no JPEG filename was specified */
  if (argc > 1)							    /* Was an image name specified on the command line? */
  {								    /* Yes, try to open it as a compressed image (JFIF or... */
    imageName.value = XmStringCreateSimple(argv[1]);		    /* ...JPEG Movie still). */
  }
  return;
} /* end function CheckOptions */



Boolean AutoLoadWorkProc(XtPointer fileName)
{
  XmFileSelectionBoxCallbackStruct* imageName =
    (XmFileSelectionBoxCallbackStruct *)fileName;
  
  OpenImageFile((Widget)NULL,
		(XtPointer)NULL,
		(XtPointer)imageName);
  XmStringFree(imageName->value);				    /* Return memory no longer needed */
  return(True);							    /* Done with this work proc */
} /* end function AutoLoadWorkProc */



int main (int argc, char **argv)
{
  int					hardwareInstance;
  char*					hardwareInstanceEnvVar;
  XSetWindowAttributes			attributes;
  XmString				onString = XmStringCreateLocalized("On");
  
  XtSetLanguageProc ( (XtAppContext) NULL, (XtLanguageProc) NULL, (XtPointer) NULL );
  XtToolkitInitialize ();
  app_context = XtCreateApplicationContext ();
  display = XtOpenDisplay (app_context,
			   NULL,
			   argv[0],
			   "VideoTool",
			   (XrmOptionDescRec*)&vtOptions,
			   XtNumber(vtOptions),
			   &argc, argv);
  if (!display)
  {
    printf("VideoTool cannot open a connection to the X server; exiting.\n");
    exit (-1);
  }
  create_mainWindow ( display, argv[0], argc, argv );
  create_outputConfigShell ( mainWindow );
  create_imageShell ( mainWindow );
  create_fileDialog ( mainWindow );
  create_outputShell ( mainWindow );
  create_input2Shell ( mainWindow );
  create_input1Shell ( mainWindow );
  create_configShell ( mainWindow );
  create_alertShell ( mainWindow );
  create_infoShell ( mainWindow );
  create_aboutShell ( mainWindow );
  create_capturePrefsShell ( mainWindow );
  errorMessage = malloc(1024);					    /* Allocate some memory to store error messages */
  input1ShellPoppedUp = False;					    /* Note that neither input shell is popped up */
  input2ShellPoppedUp = False;
  XtVaSetValues(input1Shell,					    /* Make sure that the input shell conforms to a 4-to-3... */
		XtNminAspectX, 4,				    /* ...aspect ratio.  This is necessary for dealing with... */
		XtNminAspectY, 3,				    /* ...the OPEN LOOK window manager (the Motif window... */
		XtNmaxAspectX, 4,				    /* ...manager doesn't require this restriction since it... */
		XtNmaxAspectY, 3,				    /* ...listens to the XVideoIn widget's resize requests). */
		NULL);
  XtVaSetValues(input2Shell,
		XtNminAspectX, 4,
		XtNminAspectY, 3,
		XtNmaxAspectX, 4,
		XtNmaxAspectY, 3,
		NULL);
  CheckOptions(argc, argv);					    /* Check for command-line options */
/*  XtVaSetValues(helpMenuCascade, XmNsensitive, False, NULL);*/	    /* Don't allow the Help menu to be selected */
  deleteWindowAtom = XmInternAtom(display,			    /* Create an Atom for communicating with the window mgr... */
				  "WM_DELETE_WINDOW",		    /* ...that the app. needs to be informed about shells... */
				  False);			    /* ...being deleted. */
  XmAddWMProtocolCallback(input1Shell, deleteWindowAtom,	    /* Intercept window manager Delete Window calls in order... */
			  (XtCallbackProc)WMDeleteWindowCallback,   /* ...to update the interface components correctly. */
			  (XtPointer)NULL);
  XmAddWMProtocolCallback(input2Shell, deleteWindowAtom,	    /* Do this for the 2nd video input shell, too */
			  (XtCallbackProc)WMDeleteWindowCallback,
			  (XtPointer)NULL);
  XmAddWMProtocolCallback(mainWindow, deleteWindowAtom,		    /* Set up a quit callback for the top-level shell */
			  (XtCallbackProc)WMDeleteWindowCallback,
			  (XtPointer)NULL);
  autoSearch = True;
  XPlxSelectHardwareInstance(display, vtResources.board);	    /* Use the board specified by the user, if any */
  input1Widget = XtVaCreateManagedWidget("input1Widget",	    /* Create a video input widget for the first digitizer */
					 xVideoInWidgetClass,
					 input1Shell,
					 XtNchannel, PLX_INPUT_0,
					 XtNautoSearch, autoSearch,
					 NULL);
  XtVaGetValues(input1Widget, XtNsync, &input1Sync, NULL);	    /* Determine if an active signal is on input 1 */
  if (input1Sync == True)					    /* Sync was found, it's okay to save still images */
  {
    XtVaSetValues(saveButton, XmNsensitive, True, NULL);
    XtVaSetValues(saveAsButton, XmNsensitive, True, NULL);
  }
  else
  {
    XtVaSetValues(saveButton, XmNsensitive, False, NULL);
    XtVaSetValues(saveAsButton, XmNsensitive, False, NULL);
  }
  BuildInputMenu();						    /* Determine how many digitizers & other h/w configuration */
  zoomFactor = 1;						    /* Initialize the zoom factor to 1:1 */
  zoomFactor = XPlxGetZoomFactor(XtDisplay(input1Widget),	    /* Inquire the current zoom factor for this connection */
				 XtWindow(input1Widget));
  if (2 == zoomFactor)						    /* Is zoom on? */
  {
    XmToggleButtonSetState(zoom2XToggle, True, False);		    /* Yes, reflect that in the interface */
    XmToggleButtonSetState(zoom1XToggle, False, False);		    /* Explicitly turn off the zoom 1 toggle */
  }
  rootWindowWidget = XtCreateWidget("root",
				    rootWindowWidgetClass,
				    mainWindow,
				    NULL, 0);
  jpegWidget = XtVaCreateManagedWidget("jpegWidget",
				       jpegWidgetClass,
				       imageShell,
				       NULL);
  selectedSettingsWidget = input1ChannelButton;
  selectedVideoWidget = input1Widget;
  ShowInputSettings(selectedSettingsWidget, (XtPointer)NULL, (XtPointer)NULL);
  SetImageSaveFormat(saveAsJFIFButton, "JFIF", (XtPointer)NULL);
  XtRealizeWidget (mainWindow);
  CreateLogoPixmap(aboutShell);					    /* Create a graphical logo for the About panel */
  OsSpecificInit();						    /* Do platform-specific initialization */
  PlaceWindows();
  if (input1Sync == True)					    /* Show the 1st input if there's a valid signal on it */
  {
    XtPopup(input1Shell, XtGrabNone);				    /* Bring up the 1st input window on application launch */
    XtVaSetValues(videoIn1Toggle, XmNlabelString, onString, NULL);  /* Make the Input1 toggle reflect apperance of Input 1 */
    XtVaSetValues(stillInput1Button, XmNsensitive, True, NULL);	    /* Allow capture on the 1st video input window */
    XmStringFree(onString);					    /* Return memory no longer needed */
    XmToggleButtonSetState(videoIn1Toggle, True, False);	    /* Manually turn on the videoIn1 toggle */
    XtVaSetValues(input1Widget, XtNautoSearch, False, NULL);	    /* Disable auto-search after application launch */
  }
  if (input2Widget != NULL)					    /* Was the 2nd video input widget created? */
  {								    /* Yes, disable auto-search for it */
    XtVaSetValues(input2Widget, XtNautoSearch, False, NULL);	    /* Disable auto-search after application launch */
  }
  rubberBandGC = XCreateGC(XtDisplay(mainWindow),		    /* Initialize resources for drawing rubberBandGC boxes */
			   XtWindow(mainWindow), 0, NULL);
  XSetSubwindowMode(XtDisplay(mainWindow),
		    rubberBandGC,
		    IncludeInferiors); 
  XSetForeground(XtDisplay(mainWindow),
		 rubberBandGC,
		 BlackPixel(XtDisplay(mainWindow),
			    XScreenNumberOfScreen(XtScreen(mainWindow))));
  
  XSetFunction(XtDisplay(mainWindow), rubberBandGC, GXxor);
  XSetLineAttributes(XtDisplay(mainWindow),
		     rubberBandGC,
		     3,
		     LineSolid, CapButt, JoinMiter);
  handCursor = XCreateFontCursor(XtDisplay(mainWindow), XC_hand2);
  watchCursor = XCreateFontCursor(XtDisplay(mainWindow), XC_watch);
  XtVaSetValues(outputShell, XtNborderWidth, 0, NULL);
  if (imageName.value != (XmString)NULL)			    /* Was an image name specified on the command line? */
  {								    /* Yes, try to open it as a compressed image (JFIF or... */
    imageName.value = XmStringCreateSimple(argv[1]);		    /* ...JPEG Movie still). */
    XtAppAddWorkProc(app_context,
		     AutoLoadWorkProc,
		     (XtPointer)&imageName);
  }
  XtManageChild(infoAlert);
  XtAppMainLoop (app_context);
  exit (0);
}
