/* $Header: /plx/source/Xclients/generic/RCS/VideoTool.h,v 1.31 1996/01/05 23:42:20 drapeau Exp $ */
/*
 *   Copyright (c) 1994 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Log: VideoTool.h,v $
 * Revision 1.31  1996/01/05  23:42:20  drapeau
 * Added a feature to save a still image from an open movie as a JFIF
 * file.
 *
 * Revision 1.30  1995/10/05  04:36:53  drapeau
 * Added support for the new OutputCascade interface code for presenting
 * and selecting output signal types.
 *
 * Revision 1.29  1995/09/28  05:47:29  drapeau
 * Added support for signal handlers.
 * Also reformatted code to make it easier to read and edit.
 *
 * Revision 1.28  1995/08/18  01:57:50  drapeau
 * Added code to put the company logo in the application's "About..."
 * panel.
 *
 * Revision 1.27  1995/08/17  19:50:52  drapeau
 * Two changes:
 *
 * 1) Add better support for the latest JPEG Movie File Format
 *    specification.
 *
 * 2) Add support for an "About..." panel.
 *
 * Revision 1.26  1995/05/31  18:55:11  drapeau
 * Added function prototype for ShowInputSettings(), which is used
 * throughout.
 *
 * Revision 1.25  1995/05/15  18:29:24  lovvik
 * Changed 'pvic.h' to InputCascade.h to reflect the
 * filename change.
 *
 * Revision 1.24  1995/05/15  16:53:33  drapeau
 * Added support for new modules that automatically create cascading
 * menus to reflect the input choices available on the Parallax hardware
 * being used.
 *
 * Revision 1.23  1995/05/02  20:37:12  drapeau
 * Added support for saving and displaying zoomed JPEG images.
 *
 * Revision 1.22  1995/04/21  20:11:28  drapeau
 * Added support for multiple Plx boards on a single screen.
 *
 * Revision 1.21  1994/10/14  21:13:49  drapeau
 * Added variables to keep track of whether optional signal types are
 * available.
 *
 * Revision 1.20  1994/10/14  03:04:42  drapeau
 * Added a declaration for a watch cursor to be displayed during
 * time-consuming operations.
 *
 * Revision 1.19  1994/10/07  02:17:42  drapeau
 * Added include file for the constant MAXPATHLEN, used to indicate the
 * maximum size of a filename.
 *
 * Revision 1.18  1994/10/06  21:26:36  drapeau
 * Added a function prototype for an HP hack function,
 * RestoreInput2IfNeeded().
 *
 * Revision 1.17  1994/10/01  01:32:54  drapeau
 * Added a global variable to keep track of the input format (composite
 * or YC).
 *
 * Revision 1.16  1994/09/23  06:45:55  drapeau
 * Added variable declarations and function prototypes for new code to
 * handle whether inputs are enabled.  If they're not enabled, saving of
 * stills is not allowed and the input shells will not pop up.
 *
 * Revision 1.15  1994/09/14  05:58:34  drapeau
 * Significant changes to the implementation of the rubberband code.  It
 * works cleanly now.
 *
 * Revision 1.14  1994/09/13  08:04:04  drapeau
 * Added function prototypes to accommodate new functionality.
 *
 * Revision 1.13  1994/09/04  07:12:59  drapeau
 * Added support for correctly handling deletion of video input shells
 * via the XmProtocols mechanism.  Also added the "autoSearch" variable,
 * which is set on a platform-specific basis (autoSearch is turned off
 * for HP and on for Sun).
 *
 * Revision 1.12  1994/08/27  05:51:11  drapeau
 * Minor changes to add global variable declarations and function
 * prototypes.
 *
 * Revision 1.11  1994/08/14  01:43:32  drapeau
 * Added a function prototype for os-specific initialization code.
 *
 * Revision 1.1  1994/03/22  04:51:20  drapeau
 * Initial revision
 * */
#ifndef _VideoToolHeader_
#define _VideoToolHeader_
static char videoToolHeaderRcsId[] = "$Header: /plx/source/Xclients/generic/RCS/VideoTool.h,v 1.31 1996/01/05 23:42:20 drapeau Exp $";
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <Xm/Xm.h>
#include <Xm/CascadeB.h>
#include <Xm/DialogS.h>
#include <Xm/FileSB.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/List.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/Scale.h>
#include <Xm/ScrollBar.h>
#include <Xm/SelectioB.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
#include <Xm/CascadeBG.h>
#include <Xm/LabelG.h>
#include <Xm/PushBG.h>
#include <Xm/SeparatoG.h>
#include <Xm/ToggleBG.h>
#include <stdio.h>
#include <XPlxExt.h>
#include <XVideoIn.h>
#include <XVideoOut.h>
#include <JPEG.h>
#include "RootWindow.h"
#include <X11/cursorfont.h>
#include "externs.h"
#include <stdlib.h>
#include <Xm/Protocols.h>
#include <sys/param.h>
#include "IOCascade.h"
#include <xpm.h>
#include <icon/.plxlogo>
#include <signal.h>

extern XtAppContext	app_context;
extern Widget 		selectedVideoWidget;
extern Widget 		input1Widget;
extern Widget 		input2Widget;
extern Widget 		outputWidget;
extern Widget 		jpegWidget;
extern Widget		selectedSettingsWidget;
extern Widget 		rootWindowWidget;
extern char*		fileFormat;
extern XRectangle	outputSourceRect;     
extern Cursor		handCursor;
extern GC		rubberBandGC;
extern int		rubberBandMode;
extern char*		errorMessage;
extern Boolean		autoSearch;
extern Atom		deleteWindowAtom;
extern int		lastInstanceSelected;
extern Boolean		input1Sync;
extern Boolean		input2Sync;
extern Boolean		input1ShellPoppedUp;
extern Boolean		input2ShellPoppedUp;
extern int		inputSignalFormat;
extern Cursor		watchCursor;
extern int		zoomFactor;
extern XRectangle	displayDestRect;
extern PlxInputCascade*		inputMenuCascade;
extern PlxOutputCascade*	outputMenuCascade;
extern Boolean			outputAvailable;


void	SelectVideoWidget(Widget);
void	MoveResize(Widget, XtPointer, XEvent*, Boolean*);
void	StartRubberBandAction(Widget, XtPointer, XEvent*, Boolean*);
void	EndRubberBandAction(Widget, XtPointer, XEvent*, Boolean*);
void	ResizeVideoOutputWidget(Widget, XtPointer, XEvent*, Boolean*);
void	OsSpecificInit(void);
void	WMDeleteWindowCallback(Widget, XtPointer, XtPointer);
void	ResizeVideoWidgetIfNecessary(Widget widget);
void	DisableOutputControls(void);
void	EnableOutputControls(void);
void	RestoreInput2IfNeeded(void);
void	SetSignalStandard(Widget, XtPointer, XtPointer);
void	ShowInputSettings(Widget, XtPointer, XtPointer);
void	CatchSignals(void);
Window	SelectWindow(Widget);
void	SetOutputSignalType(Widget, XtPointer, XtPointer);
void	SaveStill(Widget, XEvent*, String*, Cardinal*);


#define ResizeMode	0
#define MoveMode	1
#define AudioFillerSize	(28)
#define FILLER		(128 - AudioFillerSize - 64)
#define JPEG_VERSION_1 	1
#define JPEG_VERSION_2 	2
#define JPEG_VERSION_3  3

#define LOAD_JPEG	236
#define END_FRAME	237
#define LOAD_AUDIO_0	229
#define LOAD_AUDIO_1	228
#define LOAD_AUDIO_2	227
#define LOAD_AUDIO_3	226


typedef struct {
	char magic[8];						    /* defined as "j_movie"					*/
	int version;						    /* Version number						*/
	int fps;						    /* frames per second					*/
	int frames;						    /* Number of frames in the sequence			        */
	int width;						    /* Width of each frame 					*/
	int height;						    /* Height of each frame 					*/
	int bandwidth;						    /* Bandwidth required for playback			        */
	int qfactor;						    /* Quanization factor used in compression		        */
	int mapsize;						    /* Number of colors used in colormap		        */
	int indexbuf;						    /* Offset in file of frame indexes			        */
	int tracks;						    /* Number of audio tracks used				*/
	int volbase;						    /* Baseline for volume control				*/
	int audioslice;						    /* Number of audio bytes per frame			        */
	char unusedAudioField[AudioFillerSize];			    /* Sun-specific audio field not used by this app. */
	int firstValidFrame;					    /* First & last frames of the movie to be played, useful... */
	int lastValidFrame;					    /* ...for trimming a few frames off the ends of movies... */
								    /* ...w/o having to save the whole movie again. */
	char filler[FILLER];	/* Pad out this record to 128 bytes			*/
} jpheader;

#define XtNboard	"board"
#define XtNhelp		"help"

#define XtCBoard	"Board"
#define XtCHelp		"Help"


typedef struct _AppResources					    /* X resources specific to VideoTool */
{
  int		board;						    /* Which hardware instance to use if multiples are available */
  Boolean	help;						    /* Whether to print a help message */
} AppResources;

#endif /* _VideoToolHeader_ */
