/* $Header: /plx/source/Xclients/generic/RCS/alien.c,v 1.2 1995/12/14 23:38:43 lovvik Exp $ */
/* $Log: alien.c,v $
 * Revision 1.2  1995/12/14  23:38:43  lovvik
 * Modified the call to WriteTIFFImage so it does not
 * print diagnostic messages.
 *
 * Revision 1.1  1993/12/01  03:03:08  drapeau
 * Initial revision
 * */
static char alienRcsId[] = "$Header: /plx/source/Xclients/generic/RCS/alien.c,v 1.2 1995/12/14 23:38:43 lovvik Exp $";
/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */

/*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                                                             %
%                                                                             %
%                        AAA   L      IIIII EEEEE  N   N                      %
%                       A   A  L        I   E      NN  N                      %
%                       AAAAA  L        I   EEE    N N N                      %
%                       A   A  L        I   E      N  NN                      %
%                       A   A  LLLLL  IIIII EEEEE  N   N                      %
%                                                                             %
%                                                                             %
%                  Utility Routines to Read Alien Image Formats               %
%                                                                             %
%                                                                             %
%                                                                             %
%                             Software Design                                 %
%                               John Cristy                                   %
%                              January 1992                                   %
%                                                                             %
%                                                                             %
%  Copyright 1992 E. I. du Pont de Nemours & Company                          %
%                                                                             %
%  Permission to use, copy, modify, distribute, and sell this software and    %
%  its documentation for any purpose is hereby granted without fee,           %
%  provided that the above Copyright notice appear in all copies and that     %
%  both that Copyright notice and this permission notice appear in            %
%  supporting documentation, and that the name of E. I. du Pont de Nemours    %
%  & Company not be used in advertising or publicity pertaining to            %
%  distribution of the software without specific, written prior               %
%  permission.  E. I. du Pont de Nemours & Company makes no representations   %
%  about the suitability of this software for any purpose.  It is provided    %
%  "as is" without express or implied warranty.                               %
%                                                                             %
%  E. I. du Pont de Nemours & Company disclaims all warranties with regard    %
%  to this software, including all implied warranties of merchantability      %
%  and fitness, in no event shall E. I. du Pont de Nemours & Company be       %
%  liable for any special, indirect or consequential damages or any           %
%  damages whatsoever resulting from loss of use, data or profits, whether    %
%  in an action of contract, negligence or other tortious action, arising     %
%  out of or in connection with the use or performance of this software.      %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Functions in this library convert to and from `alien' image formats to the
%  MIFF image format.
%
%
*/

/*
  Include declarations.
*/
#include "display.h"
#include "image.h"
#include "alien.h"
#include "X.h"

/*
  Global declarations.
*/
static char
  *AlienTypes[]=
  {
    "AVS",
    "CMYK",
    "FAX",
    "GIF",
    "GRAY",
    "HISTOGRAM",
    "JPEG",
    "JPG",
    "MIFF",
    "MTV",
    "PNM",
    "PS",
    "RGB",
    "RLE",
    "SUN",
    "TEXT",
    "TIFF",
    "VICAR",
    "YUV",
    "X",
    "XC",
    "XBM",
    "XWD",
    (char *) NULL,
  };

/*
  External declarations.
*/
extern char
  *client_name;
/*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                                                             %
%                                                                             %
%  M S B F i r s t O r d e r L o n g                                          %
%                                                                             %
%                                                                             %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Function MSBFirstOrderLong converts a least-significant byte first buffer
%  of long integers to most-significant byte first.
%
%  The format of the MSBFirstOrderLong routine is:
%
%       MSBFirstOrderLong(p,length);
%
%  A description of each parameter follows.
%
%   o  p:  Specifies a pointer to a buffer of integers.
%
%   o  length:  Specifies the length of the buffer.
%
%
*/
static void MSBFirstOrderLong(p,length)
register char
  *p;

register unsigned
  length;
{
  register char
    c,
    *q,
    *sp;

  q=p+length;
  while (p < q)
  {
    sp=p+3;
    c=(*sp);
    *sp=(*p);
    *p++=c;
    sp=p+1;
    c=(*sp);
    *sp=(*p);
    *p++=c;
    p+=2;
  }
}

/*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                                                             %
%                                                                             %
%  M S B F i r s t O r d e r S h o r t                                        %
%                                                                             %
%                                                                             %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Function MSBFirstOrderShort converts a least-significant byte first buffer
%  of short integers to most-significant byte first.
%
%  The format of the MSBFirstOrderShort routine is:
%
%       MSBFirstOrderLongShort(p,length);
%
%  A description of each parameter follows.
%
%   o  p:  Specifies a pointer to a buffer of integers.
%
%   o  length:  Specifies the length of the buffer.
%
%
*/
static void MSBFirstOrderShort(p,length)
register char
  *p;

register unsigned
  length;
{
  register char
    c,
    *q;

  q=p+length;
  while (p < q)
  {
    c=(*p);
    *p=(*(p+1));
    p++;
    *p++=c;
  }
}

/*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                                                             %
%                                                                             %
%  R e a d S U N I m a g e                                                    %
%                                                                             %
%                                                                             %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Function ReadSUNImage reads an image file and returns it.  It allocates
%  the memory necessary for the new Image structure and returns a pointer to
%  the new image.
%
%  The format of the ReadSUNImage routine is:
%
%      image=ReadSUNImage(alien_info)
%
%  A description of each parameter follows:
%
%    o image:  Function ReadSUNImage returns a pointer to the image after
%      reading.  A null image is returned if there is a a memory shortage or
%      if the image cannot be read.
%
%    o alien_info: Specifies a pointer to an AlienInfo structure.
%
%
*/
static Image *ReadSUNImage(alien_info)
AlienInfo
  *alien_info;
{
#define RMT_EQUAL_RGB  1
#define RMT_NONE  0
#define RMT_RAW  2
#define RT_STANDARD  1
#define RT_ENCODED  2
#define RT_FORMAT_RGB  3

  typedef struct _Rasterfile
  {
    int
      magic,
      width,
      height,
      depth,
      length,
      type,
      maptype,
      maplength;
  } Rasterfile;

  Image
    *image;

  Rasterfile
    sun_header;

  register int
    bit,
    i,
    x,
    y;

  register RunlengthPacket
    *q;

  register unsigned char
    *p;

  unsigned char
    *sun_data,
    *sun_pixels;

  unsigned long
    lsb_first;

  unsigned short
    index;

  unsigned int
    status;

  /*
    Allocate image structure.
  */
  image=AllocateImage("SUN");
  if (image == (Image *) NULL)
    return((Image *) NULL);
  /*
    Open image file.
  */
  (void) strcpy(image->filename,alien_info->filename);
  OpenImage(image,"r");
  if (image->file == (FILE *) NULL)
    {
      Warning("unable to open file",image->filename);
      DestroyImage(image);
      return((Image *) NULL);
    }
  /*
    Read raster image.
  */
  status=ReadData((char *) &sun_header,1,sizeof(Rasterfile),image->file);
  if (status == False)
    {
      Warning("not a SUN image file",(char *) NULL);
      DestroyImage(image);
      return((Image *) NULL);
    }
  do
  {
    /*
      Ensure the header byte-order is most-significant byte first.
    */
    lsb_first=1;
    if (*(char *) &lsb_first)
      MSBFirstOrderLong((char *) &sun_header,sizeof(sun_header));
    if (sun_header.magic != 0x59a66a95)
      {
        Warning("not a SUN raster,",image->filename);
        DestroyImages(image);
        return((Image *) NULL);
      }
    switch (sun_header.maptype)
    {
      case RMT_NONE:
      {
        if (sun_header.depth < 24)
          {
            /*
              Create linear color ramp.
            */
            image->colors=1 << sun_header.depth;
            image->colormap=(ColorPacket *)
              malloc(image->colors*sizeof(ColorPacket));
            if (image->colormap == (ColorPacket *) NULL)
              {
                Warning("memory allocation error",(char *) NULL);
                return((Image *) NULL);
              }
            for (i=0; i < image->colors; i++)
            {
              image->colormap[i].red=(255*i)/(image->colors-1);
              image->colormap[i].green=(255*i)/(image->colors-1);
              image->colormap[i].blue=(255*i)/(image->colors-1);
            }
          }
        break;
      }
      case RMT_EQUAL_RGB:
      {
        unsigned char
          *sun_colormap;

        /*
          Read Sun raster colormap.
        */
        image->colors=sun_header.maplength/3;
        image->colormap=(ColorPacket *)
          malloc(image->colors*sizeof(ColorPacket));
        sun_colormap=(unsigned char *)
          malloc(image->colors*sizeof(unsigned char));
        if ((image->colormap == (ColorPacket *) NULL) ||
            (sun_colormap == (unsigned char *) NULL))
          {
            Warning("memory allocation error",(char *) NULL);
            DestroyImages(image);
            return((Image *) NULL);
          }
        (void) ReadData((char *) sun_colormap,1,(int) image->colors,
          image->file);
        for (i=0; i < image->colors; i++)
          image->colormap[i].red=sun_colormap[i];
        (void) ReadData((char *) sun_colormap,1,(int) image->colors,
          image->file);
        for (i=0; i < image->colors; i++)
          image->colormap[i].green=sun_colormap[i];
        (void) ReadData((char *) sun_colormap,1,(int) image->colors,
          image->file);
        for (i=0; i < image->colors; i++)
          image->colormap[i].blue=sun_colormap[i];
        (void) free((char *) sun_colormap);
        break;
      }
      case RMT_RAW:
      {
        unsigned char
          *sun_colormap;

        /*
          Read Sun raster colormap.
        */
        sun_colormap=(unsigned char *)
          malloc(sun_header.maplength*sizeof(unsigned char));
        if (sun_colormap == (unsigned char *) NULL)
          {
            Warning("memory allocation error",(char *) NULL);
            DestroyImages(image);
            return((Image *) NULL);
          }
        (void) ReadData((char *) sun_colormap,1,sun_header.maplength,
          image->file);
        (void) free((char *) sun_colormap);
        break;
      }
      default:
      {
        Warning("colormap type is not supported",image->filename);
        DestroyImages(image);
        return((Image *) NULL);
      }
    }
    sun_data=(unsigned char *) malloc(sun_header.length*sizeof(unsigned char));
    if (sun_data == (unsigned char *) NULL)
      {
        Warning("memory allocation error",(char *) NULL);
        DestroyImages(image);
        return((Image *) NULL);
      }
    (void) ReadData((char *) sun_data,1,sun_header.length,image->file);
    sun_pixels=sun_data;
    if (sun_header.type == RT_ENCODED)
      {
        register int
          count,
          number_pixels;

        register unsigned char
          byte,
          *q;

        /*
          Read run-length encoded raster pixels.
        */
        number_pixels=(sun_header.width+(sun_header.width % 2))*
          sun_header.height*(((sun_header.depth-1) >> 3)+1);
        sun_pixels=(unsigned char *)
          malloc(number_pixels*sizeof(unsigned char));
        if (sun_pixels == (unsigned char *) NULL)
          {
            Warning("memory allocation error",(char *) NULL);
            DestroyImages(image);
            return((Image *) NULL);
          }
        p=sun_data;
        q=sun_pixels;
        while ((q-sun_pixels) <= number_pixels)
        {
          byte=(*p++);
          if (byte != 128)
            *q++=byte;
          else
            {
              /*
                Runlength-encoded packet: <count><byte>
              */
              count=(*p++);
              if (count > 0)
                byte=(*p++);
              while (count >= 0)
              {
                *q++=byte;
                count--;
              }
           }
        }
      (void) free((char *) sun_data);
    }
    /*
      Create image.
    */
    image->alpha=(sun_header.depth == 32);
    image->class=(sun_header.depth < 24 ? PseudoClass : DirectClass);
    image->columns=sun_header.width;
    image->rows=sun_header.height;
    image->packets=image->columns*image->rows;
    image->pixels=(RunlengthPacket *)
      malloc((unsigned int) image->packets*sizeof(RunlengthPacket));
    image->comments=(char *)
      malloc((strlen(image->filename)+2048)*sizeof(char));
    (void) sprintf(image->comments,
      "\n  Imported from Sun raster image:  %s\n\0",image->filename);
    if ((image->pixels == (RunlengthPacket *) NULL) ||
        (image->comments == (char *) NULL))
      {
        Warning("memory allocation error",(char *) NULL);
        DestroyImages(image);
        return((Image *) NULL);
      }
    /*
      Convert Sun raster image to runlength-encoded packets.
    */
    p=sun_pixels;
    q=image->pixels;
    if (sun_header.depth == 1)
      for (y=0; y < image->rows; y++)
      {
        /*
          Convert bitmap scanline to runlength-encoded color packets.
        */
        for (x=0; x < (image->columns >> 3); x++)
        {
          for (bit=7; bit >= 0; bit--)
          {
            index=((*p) & (0x01 << bit) ? 0x00 : 0x01);
            q->red=image->colormap[index].red;
            q->green=image->colormap[index].green;
            q->blue=image->colormap[index].blue;
            q->index=index;
            q->length=0;
            q++;
          }
          p++;
        }
        if ((image->columns % 8) != 0)
          {
            for (bit=7; bit >= (8-(image->columns % 8)); bit--)
            {
              index=((*p) & (0x01 << bit) ? 0x00 : 0x01);
              q->red=image->colormap[index].red;
              q->green=image->colormap[index].green;
              q->blue=image->colormap[index].blue;
              q->index=index;
              q->length=0;
              q++;
            }
            p++;
          }
        if ((((image->columns/8)+(image->columns % 8 ? 1 : 0)) % 2) != 0)
          p++;
      }
    else
      if (image->class == PseudoClass)
        for (y=0; y < image->rows; y++)
        {
          /*
            Convert PseudoColor scanline to runlength-encoded color packets.
          */
          for (x=0; x < image->columns; x++)
          {
            index=(*p++);
            q->red=image->colormap[index].red;
            q->green=image->colormap[index].green;
            q->blue=image->colormap[index].blue;
            q->index=index;
            q->length=0;
            q++;
          }
          if ((image->columns % 2) != 0)
            p++;
        }
      else
        for (y=0; y < image->rows; y++)
        {
          /*
            Convert DirectColor scanline to runlength-encoded color packets.
          */
          for (x=0; x < image->columns; x++)
          {
            q->index=(unsigned short) (image->alpha ? (*p++) : 0);
            if (sun_header.type == RT_STANDARD)
              {
                q->blue=(*p++);
                q->green=(*p++);
                q->red=(*p++);
              }
            else
              {
                q->red=(*p++);
                q->green=(*p++);
                q->blue=(*p++);
              }
            if (image->colors > 0)
              {
                q->red=image->colormap[q->red].red;
                q->green=image->colormap[q->green].green;
                q->blue=image->colormap[q->blue].blue;
              }
            q->length=0;
            q++;
          }
          if (((image->columns % 2) != 0) && (image->alpha == False))
            p++;
        }
    (void) free((char *) sun_pixels);
    if (image->class == PseudoClass)
      CompressColormap(image);
    /*
      Proceed to next image.
    */
    status=ReadData((char *) &sun_header,1,sizeof(Rasterfile),image->file);
    if (status == True)
      {
        /*
          Allocate image structure.
        */
        image->next=AllocateImage("SUN");
        if (image->next == (Image *) NULL)
          {
            DestroyImages(image);
            return((Image *) NULL);
          }
        image->next->file=image->file;
        (void) sprintf(image->next->filename,"%s.%u\0",alien_info->filename,
          image->scene+1);
        image->next->scene=image->scene+1;
        image->next->last=image;
        image=image->next;
      }
  } while (status == True);
  while (image->last != (Image *) NULL)
    image=image->last;
  CloseImage(image);
  return(image);
}

#ifdef AlienTIFF
#include "tiff.h"
#include "tiffio.h"
/*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                                                             %
%                                                                             %
%   W r i t e T I F F I m a g e                                               %
%                                                                             %
%                                                                             %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Function WriteTIFFImage writes a TIFF image to a file.
%
%  The format of the WriteTIFFImage routine is:
%
%      status=WriteTIFFImage(image,verbose)
%
%  A description of each parameter follows:
%
%    o status:  Function WriteTIFFImage return True if the image is written.
%      False is returned is there is of a memory shortage or if the image
%      file cannot be opened for writing.
%
%    o image:  A pointer to a Image structure.
%
%    o  verbose:  A value greater than zero prints detailed information about
%       the image.
%
%
*/
static unsigned int WriteTIFFImage(image,verbose)
Image
  *image;

unsigned int
  verbose;
{
  register RunlengthPacket
    *p;

  register int
    i,
    j,
    x,
    y;

  register unsigned char
    *q;

  TIFF
    *file;

  unsigned char
    *scanline;

  unsigned short
    photometric,
    rows_per_strip;

  /*
    Open TIFF file.
  */
  file=TIFFOpen(image->filename,"w");
  if (file == (TIFF *) NULL)
    return(False);
  /*
    Initialize TIFF fields.
  */
  TIFFSetField(file,TIFFTAG_DOCUMENTNAME,image->filename);
  TIFFSetField(file,TIFFTAG_SOFTWARE,"ImageMagick");
  if (image->comments != (char *) NULL)
    TIFFSetField(file,TIFFTAG_IMAGEDESCRIPTION,image->comments);
  if ((image->class == DirectClass) || (image->colors > 256))
    {
      photometric=PHOTOMETRIC_RGB;
      TIFFSetField(file,TIFFTAG_BITSPERSAMPLE,8);
      TIFFSetField(file,TIFFTAG_SAMPLESPERPIXEL,(image->alpha ? 4 : 3));
    }
  else
    {
      /*
        Determine if image is gray scale.
      */
      photometric=PHOTOMETRIC_MINISBLACK;
      for (i=0; i < image->colors; i++)
        if ((image->colormap[i].red != image->colormap[i].green) ||
            (image->colormap[i].green != image->colormap[i].blue))
          {
            photometric=PHOTOMETRIC_PALETTE;
            break;
          }
      if ((image->colors == 2) && (photometric == PHOTOMETRIC_MINISBLACK))
        TIFFSetField(file,TIFFTAG_BITSPERSAMPLE,1);
      else
        TIFFSetField(file,TIFFTAG_BITSPERSAMPLE,8);
      TIFFSetField(file,TIFFTAG_SAMPLESPERPIXEL,1);
    }
  TIFFSetField(file,TIFFTAG_PHOTOMETRIC,photometric);
  TIFFSetField(file,TIFFTAG_IMAGELENGTH,image->rows);
  TIFFSetField(file,TIFFTAG_IMAGEWIDTH,image->columns);
  TIFFSetField(file,TIFFTAG_FILLORDER,FILLORDER_MSB2LSB);
  TIFFSetField(file,TIFFTAG_ORIENTATION,ORIENTATION_TOPLEFT);
  TIFFSetField(file,TIFFTAG_PLANARCONFIG,PLANARCONFIG_CONTIG);
  TIFFSetField(file,TIFFTAG_COMPRESSION,COMPRESSION_LZW);
  if (image->compression == NoCompression)
    TIFFSetField(file,TIFFTAG_COMPRESSION,COMPRESSION_NONE);
  rows_per_strip=8192/TIFFScanlineSize(file);
  if (rows_per_strip == 0)
    rows_per_strip=1;
  TIFFSetField(file,TIFFTAG_ROWSPERSTRIP,rows_per_strip);
  scanline=(unsigned char *) malloc(TIFFScanlineSize(file));
  if (scanline == (unsigned char *) NULL)
    {
      Warning("memory allocation error",(char *) NULL);
      return(False);
    }
  p=image->pixels;
  q=scanline;
  x=0;
  y=0;
  if (photometric == PHOTOMETRIC_RGB)
    for (i=0; i < image->packets; i++)
    {
      for (j=0; j <= (int) p->length; j++)
      {
        /*
          Convert DirectClass packets to contiguous RGB scanlines.
        */
        *q++=p->red;
        *q++=p->green;
        *q++=p->blue;
        if (image->alpha)
          *q++=(unsigned char) p->index;
        x++;
        if (x == image->columns)
          {
            if (TIFFWriteScanline(file,scanline,y,0) < 0)
              break;
            q=scanline;
            x=0;
            y++;
          }
      }
      p++;
    }
  else
    if (photometric == PHOTOMETRIC_PALETTE)
      {
        unsigned short
          blue[256],
          green[256],
          red[256];

        /*
          Initialize TIFF colormap.
        */
        for (i=0; i < image->colors; i++)
        {
          red[i]=(unsigned short)
            ((image->colormap[i].red*65535)/(unsigned int) MaxRGB);
          green[i]=(unsigned short)
            ((image->colormap[i].green*65535)/(unsigned int) MaxRGB);
          blue[i]=(unsigned short)
            ((image->colormap[i].blue*65535)/(unsigned int) MaxRGB);
        }
        TIFFSetField(file,TIFFTAG_COLORMAP,red,green,blue);
        /*
          Convert PseudoClass packets to contiguous colormap indexed scanlines.
        */
        for (i=0; i < image->packets; i++)
        {
          for (j=0; j <= (int) p->length; j++)
          {
            *q++=(unsigned char) p->index;
            x++;
            if (x == image->columns)
              {
                if (TIFFWriteScanline(file,scanline,y,0) < 0)
                  break;
                q=scanline;
                x=0;
                y++;
              }
          }
          p++;
        }
      }
    else
      if (image->colors > 2)
        for (i=0; i < image->packets; i++)
        {
          for (j=0; j <= (int) p->length; j++)
          {
            /*
              Convert PseudoClass packets to contiguous grayscale scanlines.
            */
            *q++=(unsigned char) image->colormap[p->index].red;
            x++;
            if (x == image->columns)
              {
                if (TIFFWriteScanline(file,scanline,y,0) < 0)
                  break;
                q=scanline;
                x=0;
                y++;
              }
          }
          p++;
        }
      else
        {
          register unsigned char
            bit,
            byte,
            polarity;

          /*
            Convert PseudoClass packets to contiguous monochrome scanlines.
          */
          polarity=(Intensity(image->colormap[0]) >
            Intensity(image->colormap[1]) ? 0 : 1);
          bit=0;
          byte=0;
          x=0;
          for (i=0; i < image->packets; i++)
          {
            for (j=0; j <= (int) p->length; j++)
            {
              byte<<=1;
              if (p->index == polarity)
                byte|=0x01;
              bit++;
              if (bit == 8)
                {
                  *q++=byte;
                  bit=0;
                  byte=0;
                }
              x++;
              if (x == image->columns)
                {
                  /*
                    Advance to the next scanline.
                  */
                  if (bit != 0)
                    *q++=byte << (8-bit);
                  if (TIFFWriteScanline(file,scanline,y,0) < 0)
                    break;
                  q=scanline;
                  bit=0;
                  byte=0;
                  x=0;
                  y++;
               }
            }
            p++;
          }
        }
  (void) free((char *) scanline);
  (void) TIFFFlushData(file);
  if (verbose == True)
    TIFFPrintDirectory(file,stderr,False);
  (void) TIFFClose(file);
  return(True);
}
#else
static unsigned int WriteTIFFImage(image,verbose)
Image
  *image;

unsigned int
  verbose;
{
  Warning("TIFF library is not available",image->filename);
  return(False);
}
#endif

/*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                                                             %
%                                                                             %
%   W r i t e S U N I m a g e                                                 %
%                                                                             %
%                                                                             %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Function WriteSUNImage writes an image to a file on disk in SUN
%  rasterfile format.
%
%  The format of the WriteSUNImage routine is:
%
%      status=WriteSUNImage(image)
%
%  A description of each parameter follows.
%
%    o status: Function WriteSUNImage return True if the image is written.
%      False is returned is there is a memory shortage or if the image file
%      fails to write.
%
%    o image:  A pointer to a Image structure.
%
%
*/
static unsigned int WriteSUNImage(image)
Image
  *image;
{
#define RMT_EQUAL_RGB  1
#define RMT_NONE  0
#define RMT_RAW  2
#define RT_STANDARD  1
#define RT_FORMAT_RGB  3

  typedef struct SUN
  {
    int
      magic,
      width,
      height,
      depth,
      length,
      type,
      maptype,
      maplength;
  } SUN;

  SUN
    sun_header;

  register int
    i,
    j,
    x;

  register RunlengthPacket
    *p;

  register unsigned char
    *q;

  unsigned char
    *sun_pixels;

  unsigned int
    grayscale;

  unsigned long
    lsb_first;

  /*
    Open output image file.
  */
  OpenImage(image,"w");
  if (image->file == (FILE *) NULL)
    {
      Warning("unable to open file",image->filename);
      return(False);
    }
  /*
    Initialize SUN raster file header.
  */
  sun_header.magic=0x59a66a95;
  sun_header.width=image->columns;
  sun_header.height=image->rows;
  sun_header.type=(image->class == DirectClass ? RT_FORMAT_RGB : RT_STANDARD);
  sun_header.maptype=RMT_NONE;
  sun_header.maplength=0;
  if ((image->class == DirectClass) || (image->colors > 256))
    {
      /*
        Full color sun raster.
      */
      sun_header.depth=(image->alpha ? 32 : 24);
      sun_header.length=image->columns*image->rows*(image->alpha ? 4 : 3);
      sun_header.length+=image->columns % 2 ? image->rows : 0;
    }
  else
    {
      /*
        Determine if image is grayscale.
      */
      grayscale=True;
      for (i=0; i < image->colors; i++)
        if ((image->colormap[i].red != image->colormap[i].green) ||
            (image->colormap[i].green != image->colormap[i].blue))
          {
            grayscale=False;
            break;
          }
      if ((image->colors > 2) || !grayscale)
        {
          /*
            Colormapped sun raster.
          */
          sun_header.depth=8;
          sun_header.length=image->columns*image->rows;
          sun_header.maptype=RMT_EQUAL_RGB;
          sun_header.maplength=image->colors*3;
          sun_header.length+=image->columns % 2 ? image->rows : 0;
        }
      else
        {
          /*
            Monochrome sun raster.
          */
          sun_header.depth=1;
          sun_header.length=((image->columns+7) >> 3)*image->rows;
          sun_header.length+=((image->columns/8)+(image->columns % 8 ? 1 : 0)) %
            2 ? image->rows : 0;
        }
    }
  /*
    Ensure the SUN raster header byte-order is most-significant byte first.
  */
  lsb_first=1;
  if ((*(char *) &lsb_first) == 0)
    (void) fwrite((char *) &sun_header,sizeof(sun_header),1,image->file);
  else
    {
      SUN
        reverse_header;

      reverse_header=sun_header;
      MSBFirstOrderLong((char *) &reverse_header,sizeof(reverse_header));
      (void) fwrite((char *) &reverse_header,sizeof(reverse_header),1,
        image->file);
    }
  /*
    Convert MIFF to SUN raster pixels.
  */
  sun_pixels=(unsigned char *) malloc(sun_header.length*sizeof(unsigned char));
  if (sun_pixels == (unsigned char *) NULL)
    {
      Warning("unable to allocate memory",(char *) NULL);
      return(False);
    }
  p=image->pixels;
  q=sun_pixels;
  x=0;
  if ((image->class == DirectClass) || (image->colors > 256))
    {
      /*
        Convert DirectClass packet to SUN RGB pixel.
      */
      for (i=0; i < image->packets; i++)
      {
        for (j=0; j <= (int) p->length; j++)
        {
          if (image->alpha)
            *q++=(unsigned char) p->index;
          *q++=p->red;
          *q++=p->green;
          *q++=p->blue;
          x++;
          if (x == image->columns)
            {
              if ((image->columns % 2) != 0)
                q++;  /* pad scanline */
              x=0;
            }
        }
        p++;
      }
    }
  else
    if ((image->colors > 2) || !grayscale)
      {
        unsigned char
          *sun_colormap;

        /*
           Dump colormap to file.
        */
        sun_colormap=(unsigned char *)
          malloc(sun_header.maplength*sizeof(unsigned char));
        if (sun_colormap == (unsigned char *) NULL)
          {
            Warning("unable to allocate memory",(char *) NULL);
            return(False);
          }
        q=sun_colormap;
        for (i=0; i < image->colors; i++)
          *q++=image->colormap[i].red;
        for (i=0; i < image->colors; i++)
          *q++=image->colormap[i].green;
        for (i=0; i < image->colors; i++)
          *q++=image->colormap[i].blue;
        (void) fwrite((char *) sun_colormap,sizeof(char),sun_header.maplength,
          image->file);
        (void) free((char *) sun_colormap);
        /*
          Convert PseudoClass packet to SUN colormapped pixel.
        */
        q=sun_pixels;
        for (i=0; i < image->packets; i++)
        {
          for (j=0; j <= (int) p->length; j++)
          {
            *q++=p->index;
            x++;
            if (x == image->columns)
              {
                if ((image->columns % 2) != 0)
                  q++;  /* pad scanline */
                x=0;
              }
          }
          p++;
        }
      }
    else
      {
        register unsigned char
          bit,
          byte,
          polarity;

        /*
          Convert PseudoClass image to a SUN monochrome image.
        */
        polarity=(Intensity(image->colormap[0]) <
          Intensity(image->colormap[1]) ? 0 : 1);
        bit=0;
        byte=0;
        for (i=0; i < image->packets; i++)
        {
          for (j=0; j <= (int) p->length; j++)
          {
            byte<<=1;
            if (p->index == polarity)
              byte|=0x01;
            bit++;
            if (bit == 8)
              {
                *q++=byte;
                bit=0;
                byte=0;
              }
            x++;
            if (x == image->columns)
              {
                /*
                  Advance to the next scanline.
                */
                if (bit != 0)
                  *q++=byte << (8-bit);
                if ((((image->columns/8)+
                    (image->columns % 8 ? 1 : 0)) % 2) != 0)
                  q++;  /* pad scanline */
                bit=0;
                byte=0;
                x=0;
             }
          }
          p++;
        }
      }
  (void) fwrite((char *) sun_pixels,sizeof(char),sun_header.length,image->file);
  (void) free((char *) sun_pixels);
  CloseImage(image);
  return(True);
}

/*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                                                             %
%                                                                             %
%   G e t A l i e n I n f o                                                   %
%                                                                             %
%                                                                             %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Function GetAlienInfo initializes the AlienInfo structure.
%
%  The format of the GetAlienInfo routine is:
%
%      GetAlienInfo(alien_info)
%
%  A description of each parameter follows:
%
%    o alien_info: Specifies a pointer to a AlienInfo structure.
%
%
*/
void GetAlienInfo(alien_info)
AlienInfo
  *alien_info;
{
  alien_info->server_name=(char *) NULL;
  alien_info->font=(char *) NULL;
  alien_info->geometry=(char *) NULL;
  alien_info->density=(char *) NULL;
  alien_info->verbose=False;
}

/*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                                                             %
%                                                                             %
%   W r i t e A l i e n I m a g e                                             %
%                                                                             %
%                                                                             %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Function WriteAlienImage writes an image to a file.  You can specify a
%  particular image format by prefixing the file with the image type and a
%  colon (i.e. mtv:image) or specify the image type as the filename suffix
%  (i.e. image.mtv).
%
%  The format of the WriteAlienImage routine is:
%
%      status=WriteAlienImage(image)
%
%  A description of each parameter follows:
%
%    o status: Function WriteAlienImage return True if the image is written.
%      False is returned is there is a memory shortage or if the image file
%      fails to write.
%
%    o image: A pointer to a Image structure.
%
%
*/
unsigned int WriteAlienImage(image)
Image
  *image;
{
  char
    magick[12],
    *p;

  Image
    alien_image;

  register char
    c,
    *q;

  register int
    i;

  unsigned int
    status;

  /*
    Look for 'image.format' in filename.
  */
  alien_image=(*image);
  p=alien_image.filename+strlen(alien_image.filename)-1;
  while ((*p != '.') && (p > alien_image.filename))
    p--;
  if ((*p == '.') && (strlen(p) < sizeof(magick)))
    {
      /*
        User specified alien image format.
      */
      (void) strcpy(magick,p+1);
      for (q=magick; *q != '\0'; q++)
      {
        c=(*q);
        if (isascii(c) && islower(c))
          *q=toupper(c);
      }
      for (i=0; AlienTypes[i] != (char *) NULL; i++)
        if (strcmp(magick,AlienTypes[i]) == 0)
          {
            (void) strcpy(alien_image.magick,magick);
            break;
          }
    }
  /*
    Look for explicit 'format:image' in filename.
  */
  p=alien_image.filename;
  while ((*p != ':') && (*p != '\0'))
    p++;
  if ((*p == ':') && ((p-alien_image.filename) < sizeof(magick)))
    {
      /*
        User specified alien image format.
      */
      (void) strncpy(magick,alien_image.filename,p-alien_image.filename);
      magick[p-alien_image.filename]='\0';
      for (q=magick; *q != '\0'; q++)
      {
        c=(*q);
        if (isascii(c) && islower(c))
          *q=toupper(c);
      }
      for (i=0; AlienTypes[i] != (char *) NULL; i++)
        if (strcmp(magick,AlienTypes[i]) == 0)
          {
            /*
              Strip off image format prefix.
            */
            p++;
            (void) strcpy(alien_image.filename,p);
            (void) strcpy(alien_image.magick,magick);
            break;
          }
    }
  /*
    Call appropriate image writer based on image type.
  */
  switch (*alien_image.magick)
  {
    case 'S':
    {
      status=WriteSUNImage(&alien_image);
      break;
    }
    case 'T':
    {
      if (strcmp(alien_image.magick,"TIFF") == 0)
        status=WriteTIFFImage(&alien_image,False);
      else
        {
	  status = False;
        }
      break;
    }
    default:
      status = False;
      break;
  }
  return(status);
}
