/*
 *   Copyright (c) 1994 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 *   Some material writte by John Malleo-Roach of Paradise Software
 */
/* $Header: /plx/source/Xclients/generic/RCS/hpImage.c,v 0.71 1994/09/25 07:27:53 drapeau Exp $ */
/* $Log: hpImage.c,v $
 * Revision 0.71  1994/09/25  07:27:53  drapeau
 * Fixed errors in the way raster images were saved to disk, mostly in
 * the encoding and depth of the image.
 *
 * Revision 0.70  1994/06/09  01:40:08  drapeau
 * First working version.
 * */
static char hpImageRcsId[] = "$Header: /plx/source/Xclients/generic/RCS/hpImage.c,v 0.71 1994/09/25 07:27:53 drapeau Exp $";

/* INCLUDE FILES */

#include <stdio.h>
#include "rasterfile.h"
#include <sys/types.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

/* CONSTANTS */

/* Number of bytes in an XImage */

#define	bytes_in_ximage(x)	((x)->bytes_per_line*(x)->height)

#define	ESCAPE	128


/* FUNCTIONS */

/*
 * Dump the specified ximage and colour map to a file
 */

pr_dump_ximage(file, ximage, map_length, r, g, b)
FILE	*file;
XImage	*ximage;
int	     map_length;
u_char	*r, *g, *b;
{
    struct rasterfile	rh;
    int					rc;
    u_char	       		*data_ptr, *out_data;
    register int       	i, ras_bytes, bytesPerScanline;
    
    if (!ximage) return(-1);
    
    rh.ras_magic     = RAS_MAGIC;
    rh.ras_width     = ximage->width;
    rh.ras_height    = ximage->height;
/* This needs more work here
    rh.ras_depth     = ximage->depth;
*/
    rh.ras_depth     = 32;
    rh.ras_type      = RT_STANDARD;
	if (map_length == 0) {
		rh.ras_maptype   = RMT_NONE;
		rh.ras_maplength = 0;
    }
	else {
		rh.ras_maptype   = RMT_EQUAL_RGB;
		rh.ras_maplength = 3*map_length;
    }
	    
    /* 
     * Dump everything to the file 
     * 1. Calculate the amount of image data to be dumped
     * 2. The rasterfile header
     * 3. the colour map if there is one
     * 4. the image data
     */
    
    ras_bytes = ximage->width * rh.ras_depth / 8;		    /* Bytes per scan line = width in pixels * bytes per pixel */
    if ((out_data = (u_char *) malloc(ras_bytes)) == NULL) return(-1);
    memset(out_data, 0, ras_bytes);
    
    rh.ras_length = ras_bytes * ximage->height;
    rc = fwrite(&rh, sizeof(struct rasterfile), 1, file);
    if(rh.ras_maptype == RMT_EQUAL_RGB) {
		rc = fwrite(r, sizeof(u_char), map_length, file);
		rc = fwrite(g, sizeof(u_char), map_length, file);
		rc = fwrite(b, sizeof(u_char), map_length, file);
    }
    data_ptr = (u_char *) ximage->data;				    /* Writing image data line by line ensures good raster file */
    if (rh.ras_depth == 1)
		 bytesPerScanline = ximage->width/8 + (ximage->width%8 ? 1 : 0);
    else bytesPerScanline = ximage->width * rh.ras_depth/8;
    
    for (i=0; i<ximage->height; i++) {
		memcpy(out_data, data_ptr, bytesPerScanline);
		rc = fwrite(out_data, sizeof(u_char), ras_bytes, file);
		memset(out_data, 0, ras_bytes);
		data_ptr += ras_bytes;
    }
    free(out_data);
    return(0);
}
