/*
 *   Copyright (c) 1994 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Header: /plx/source/Xclients/generic/RCS/hpSpecific.c,v 0.112 1996/01/22 07:57:37 drapeau Exp $ */
/* $Log: hpSpecific.c,v $
 * Revision 0.112  1996/01/22  07:57:37  drapeau
 * Added support for a new XVideoIn widget resource, "stillDelay".  The
 * new resource allows the programmer to set the delay time between the
 * time the widget freezes the digitizer and the time the widget grabs a
 * still image from its window.
 *
 * Revision 0.111  1996/01/19  04:11:10  drapeau
 * Fixed the module to support the new OutputCascade scheme.
 *
 * Revision 0.110  1995/10/05  04:38:16  drapeau
 * Deleted functionality that is now replaced by the OutputCascade
 * functions.
 *
 * Revision 0.109  1995/08/16  19:44:59  drapeau
 * Minor modification to SaveTIFFImageWorkProc(): added a comment in the
 * image stating that the Parallax VideoTool application created it.
 *
 * Revision 0.108  1995/05/31  18:58:31  drapeau
 * Modified Input2Callback() to call ShowInputSettings() so that the
 * interface always shows the most current input settings.
 *
 * Revision 0.107  1995/05/15  16:56:07  drapeau
 * Removed calls to dim buttons and other interface components that are
 * no longer in the interface due to the new cascading menu scheme for
 * setting inputs.
 *
 * Revision 0.106  95/04/18  01:07:23  01:07:23  drapeau ()
 * Added support for the 2nd YC input for the HP; this should also apply
 * to the IBM platform.
 * 
 * Revision 0.105  1995/03/03  02:14:43  drapeau
 * Moved FirstAvailableCallback() from this module into a
 * platform-neutral module since it is no longer HP-specific.
 *
 * Revision 0.104  1994/10/21  18:56:19  drapeau
 * Minor change in the way TIFF images are saved, to accommodate the HP
 * ImageTool MPower application.  It could not accept images with matte
 * channels in them, as would be saved by default with the ImageMagick
 * "convert" program.
 *
 * Revision 0.103  1994/10/14  03:10:20  drapeau
 * Two changes:
 *
 * 1) Major reorganization in the way TIFF files are saved, to try and
 *    allow for better feedback from the interface as to the progress of
 *    the saving process.
 *
 * 2) Added a function to allow the user to show the "first available"
 *    input source (i.e., enabling auto-search).
 *
 * Revision 0.102  1994/10/11  01:20:10  drapeau
 * Enabled the saving of TIFF files for the HP platform.
 *
 * Revision 0.101  1994/10/07  02:20:28  drapeau
 * Added support for saving TIFF files.  The support is a hack, but might
 * do for the short term.  It depends on an external ImageMagick program
 * called "convert" to be in the user's path.  Parallax will distribute
 * it in the same directory as the rest of the Parallax programs.
 *
 * Revision 0.100  1994/10/06  21:29:14  drapeau
 * Two changes:
 *
 * 1) Disabled output controls when Input2Callback() activates the 2nd
 *    input.  Recall that activating the 2nd input on the HP platform
 *    must deactivate the output channel.
 *
 * 2) Added a call to a new HP hack function, RestoreInput2IfNeeded(), that
 *    restores the 2nd video input to a working state if it has been stilled
 *    and a compressed image was taken from it.
 *
 * Revision 0.99  1994/10/05  01:50:38  drapeau
 * Added a (temporary) hack to Input2Callback(): force the 2nd video
 * input to be live whenever it is popped up, since it might have been
 * still before.  When the 2nd input is still and output had been turned
 * on before the 2nd input window is mapped again, the 2nd input's
 * Drawable will have lost its Port.  If this happens, XPutImage() calls
 * will fail and currently there is no clean way to get back the Port.
 *
 * Revision 0.98  1994/09/30  16:54:04  drapeau
 * Added two parameters to SaveImageFileInFormat(), to accommodate saving
 * images in other than full-size (width & height).
 *
 * Revision 0.97  1994/09/25  07:29:07  drapeau
 * Two fixes:
 *
 * 1) Dimmed the loading and saving of NVRAM values since they are not
 * yet implemented.
 *
 * 2) Fixed an error in the way raster images are saved to disk.  The
 * function calling pr_dump_image() had not closed the file after
 * completion, and so sometimes the image would not be completely saved.
 * This error has been eliminated.
 *
 * Revision 0.96  1994/09/23  06:28:03  drapeau
 * New code to detect whether inputs and outputs are currently active.
 * If no inputs are active, menu items allowing saving of still images
 * are disabled.  If no output is active, output settings controls are
 * disabled.
 *
 * Revision 0.95  1994/09/04  07:16:11  drapeau
 * Disabled auto-search of video inputs.
 *
 * Revision 0.94  1994/08/27  05:22:39  drapeau
 * Modified Input2Callback(), explicitly setting the input channel on
 * when the toggle is set.  This allows the user to re-check for a sync
 * signal if one isn't found the first time VideoTool is started.
 *
 * Also added support for displaying error messages in a dialog box
 * instead of on stdout.
 *
 * Revision 0.93  1994/08/19  07:43:02  drapeau
 * Removed code that does not work well on the HP platform.
 *
 * Revision 0.92  1994/08/19  02:33:13  andyg
 * Added HP-specific code for producing Sun raster files.
 *
 * Revision 0.91  1994/08/14  01:43:37  drapeau
 * Added the function OsSpecificInit() for platform-specific
 * initialization code.
 *
 * Revision 0.90  1994/08/10  03:09:00  drapeau
 * First working version.
 * */
static char hpSpecificRcsId[] = "$Header: /plx/source/Xclients/generic/RCS/hpSpecific.c,v 0.112 1996/01/22 07:57:37 drapeau Exp $";

#include "VideoTool.h"


static Dimension	tiffImageWidth;				    /* Variables local to this module, for saving TIFF images */
static Dimension	tiffImageHeight;
static Window		tiffImageWindow;
static char		tiffImageFileName[MAXPATHLEN];



void Input2Callback(Widget	w,
		    XtPointer	client_data,
		    XtPointer	xt_call_data )
{
  static int			lastStandard = PLX_NTSC;
  int				signalStandard;
  XmString			onString = XmStringCreateSimple("On");
  XmString			offString = XmStringCreateSimple("Off");
  XmToggleButtonCallbackStruct*	callData =
    (XmToggleButtonCallbackStruct *) xt_call_data;
  
  if (callData->set == True)					    /* Should input 2 be activated? */
  {								    /* Yes, do the appropriate book work */
    XtVaSetValues(outputWidget, XtNchannel, PLX_IO_OFF, NULL);	    /* Turn off generation of a video signal */
    XtVaSetValues(input2Widget, XtNchannel, PLX_INPUT_1,	    /* Restart video on the 2nd input */
		  XtNlive, True,				    /* Hack: force video to be live even if it was still... */
		  NULL);					    /* ...since still video allows output to steal the Port. */
    XtVaGetValues(input2Widget,					    /* Determine if an active signal is on input 2 and... */
		  XtNsync, &input2Sync,				    /* ...the new signal standard for the widget, if any. */
		  XtNsignalStandard, &signalStandard,
		  NULL);
    ShowInputSettings(input2ChannelButton, (XtPointer)NULL,	    /* Update the current input settings now that a new... */
		      (XtPointer)NULL);				    /* ...input signal has been selected. */
    if (input2Sync == False)					    /* Sync wasn't found, don't bother popping up the shell */
    {
      XmStringFree(onString);
      XmStringFree(offString);
      return;
    }
    if (lastStandard != signalStandard)				    /* Did the signal standard for this widget change? */
      ResizeVideoWidgetIfNecessary(input2Widget);		    /* Yes, widget might need resizing */
    lastStandard = signalStandard;
    XtVaSetValues(videoOutToggle, XmNlabelString, offString, NULL);
    XmToggleButtonSetState(videoOutToggle, False, False);	    /* Turn off the video output's toggle */
    XtPopup(input2Shell, XtGrabNone);				    /* Popup the 2nd video input's shell */
    XtVaSetValues(videoIn2Toggle, XmNlabelString, onString, NULL);
    XtVaSetValues(stillInput2Button, XmNsensitive, True, NULL);	    /* Allow capture on the 2nd video input window */
    DisableOutputControls();
  }
  else								    /* Input 2 should not be activated, disable input 2 for now */
  {
    XtPopdown(input2Shell);
    XtVaSetValues(videoIn2Toggle, XmNlabelString, offString, NULL);
    XtVaSetValues(stillInput2Button, XmNsensitive, False, NULL);    /* Don't allow capture on the 2nd video input window */
  }
  XmStringFree(onString);
  XmStringFree(offString);
  return;
}								    /* end function Input2Callback */



void 
  OutputCallback(Widget		w,
		 XtPointer	client_data,
		 XtPointer	xt_call_data)
{
  XmString			onString = XmStringCreateSimple("On");
  XmString			offString = XmStringCreateSimple("Off");
  XmToggleButtonCallbackStruct*	callData =
    (XmToggleButtonCallbackStruct *) xt_call_data;
  int				channel;
  int				instance;
  XmString			label;
  int				signalFormat;
  int				signalStandard;
  signalSpec			theSignal;
  
  if (callData->set == True)					    /* Was output requested to be turned on? */
  {								    /* Yes, do the necessary bookkeeping */
    XtPopdown(input2Shell);					    /* Remove the 2nd input from the screen for now */
    XmToggleButtonSetState(videoIn2Toggle, False, False);	    /* Turn off the 2nd video input's toggle */
    XtVaSetValues(videoIn2Toggle, XmNlabelString, offString, NULL);
    XtVaSetValues(stillInput2Button, XmNsensitive, False, NULL);    /* Don't allow capture on the 2nd video input window */
    XtVaSetValues(input2Widget, XtNchannel, PLX_IO_OFF, NULL);	    /* Turn off the 2nd input so as not to conflict with output */
    XtVaSetValues(outputWidget, XtNchannel, PLX_OUTPUT_0, NULL);    /* Turn on generation of a video signal, if not already on */
    XtVaSetValues(videoOutToggle, XmNlabelString, onString, NULL);
    EnableOutputControls();
    XtVaGetValues(outputWidget,					    /* Try to get the currently-selected output values */
		  XtNchannel, &channel,
		  XtNsignalStandard, &signalStandard,
		  XtNsignalFormat, &signalFormat,
		  XtNinstance, &instance,
		  NULL);
    theSignal.s.format = signalFormat;
    theSignal.s.instance = instance;
    theSignal.s.standard = signalStandard;
    theSignal.s.channel = channel;
    PlxShowOutputSelections(outputMenuCascade, theSignal);	    /* Update the output settings cascade menu */
  }
  else								    /* No, output was requested to be turned off */
  {
    XtVaSetValues(outputWidget, XtNchannel, PLX_IO_OFF, NULL);	    /* Turn off generation of a video signal */
    XtVaSetValues(videoOutToggle, XmNlabelString, offString, NULL);
    DisableOutputControls();
  }
  XmStringFree(onString);
  XmStringFree(offString);
  return;
}								    /* end function OutputCallback */



void OsSpecificInit(void)
{
  autoSearch = True;
  XtVaSetValues(effectsOptionButton, XmNsensitive, False, NULL);
  XtVaSetValues(loadSettingsCascade, XmNsensitive, False, NULL);    /* Until NVRAM works, don't let users save or load... */
  XtVaSetValues(saveSettingsCascade, XmNsensitive, False, NULL);    /* ...color settings. */
  return;
} /* end function OsSpecificInit */



Boolean SaveTIFFImageWorkProc(XtPointer	clientData)
{
  char		convertCommand[32 + MAXPATHLEN];
  FILE*		imageFile;
  XImage*	localImage;
  int		result;
  char*		tempFileName = "/tmp/.VideoToolTempImage";
  
  localImage = XGetImage(XtDisplay(selectedVideoWidget),
			 tiffImageWindow, 0, 0,
			 tiffImageWidth, tiffImageHeight,
			 AllPlanes, ZPixmap);
  imageFile = fopen(tempFileName, "w");
  if (imageFile == (FILE*)NULL)
  {
    sprintf(errorMessage, "Could not create a temporary file %s for saving your image.\n");
    PrintErrorMessage(errorMessage);
    XDestroyImage(localImage);
    return(True);
  }
  pr_dump_ximage(imageFile, localImage, 0, NULL, NULL, NULL);
  fclose(imageFile);						    /* Make sure all data are written out, then close the file */
  XDestroyImage(localImage);
  sprintf(convertCommand,					    /* Yes, create a command to convert from the Sun raster... */
	  "convert -comment 'Created by Parallax Graphics, Inc. VideoTool' +matte %s tiff:%s",
	  tempFileName,						    /* ...image into a TIFF using ImageMagick tools. */
	  tiffImageFileName);
  result = system(convertCommand);
  if (result == -1)						    /* Did the conversion fail? */
  {								    /* Yes, print an error message and return */
    sprintf(errorMessage, "Could not create the TIFF image %s.\n", tiffImageFileName);
    PrintErrorMessage(errorMessage);
  }
  unlink(tempFileName);						    /* Remove the temporary Sun Raster file */
  return(True);
} /* end function SaveTIFFImageWorkProc */




void SaveImageFileInFormat(Display*	display,
			   Window	imageWindow,
			   Dimension	width,
			   Dimension	height,
			   char*	fileFormat,
			   char*	fileName)
{
  FILE*		imageFile;
  XImage*	localImage;
  int		result;
  XtWorkProcId	saveTiffProcID;
  char		windowName[256];
  
  if (strcmp(fileFormat, "SUN") == 0)				    /* Hack!!!!!: if the image type to be saved is Sunraster,... */
  {								    /* ...go around normal means on the HP platform to save... */
    XtUnmanageChild(fileDialogSelectionBox);
    localImage = XGetImage(display, imageWindow, 0, 0,
			   width, height,
			   AllPlanes, ZPixmap);
    imageFile = fopen(fileName, "w");
    if (imageFile == (FILE*)NULL)
    {
      sprintf(errorMessage, "Could not open the file %s for writing.\n", fileName);
      PrintErrorMessage(errorMessage);
      return;
    }
    pr_dump_ximage(imageFile, localImage, 0, NULL, NULL, NULL);
    fclose(imageFile);						    /* Make sure all data are written out, then close the file */
    XDestroyImage(localImage);
  }
  if (strcmp(fileFormat, "TIFF") == 0)				    /* Is the user requesting to save as a TIFF image? */
  {
    tiffImageWindow = imageWindow;
    tiffImageHeight = height;
    tiffImageWidth = width;
    strcpy(tiffImageFileName, fileName);
    XtUnmanageChild(fileDialogSelectionBox);
    XFlush(display);
    XSync(display, False);
    saveTiffProcID = XtAppAddWorkProc(app_context,		    /* Start a work-proc to save the TIFF image; Xt is... */
				      SaveTIFFImageWorkProc,	    /* ...poorly designed and this is the only way to assure... */
				      (XtPointer)NULL);		    /* ...that window actions will be taken immediately. */
    return;
  } /* end if TIFF is the requested file format */
  return;
}								    /* end function SaveImageFileInFormat */



void RestoreInput2IfNeeded(void)				    /* An HP hack: if Input 2 is stilled, then a compression... */
{								    /* ...operation is done, then Input2 is made live again,... */
  if ((input2ShellPoppedUp == True) &&				    /* ...video would not restore.  The hack is to force... */
      (selectedVideoWidget == input2Widget))			    /* ...Input2 to re-select a video channel.  That works fine. */
  {
    XtVaSetValues(selectedVideoWidget, XtNchannel, PLX_IO_OFF, NULL);
    XtVaSetValues(selectedVideoWidget, XtNchannel, PLX_INPUT_1, NULL);
  }
  return;
} /* end function RestoreInput2IfNeeded */
