/*
 *   Copyright (c) 1994 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Header: /plx/source/Xclients/generic/RCS/sunSpecific.c,v 0.102 1996/01/22 07:58:05 drapeau Exp $ */
/* $Log: sunSpecific.c,v $
 * Revision 0.102  1996/01/22  07:58:05  drapeau
 * Added support for a new XVideoIn widget resource, "stillDelay".  The
 * new resource allows the programmer to set the delay time between the
 * time the widget freezes the digitizer and the time the widget grabs a
 * still image from its window.
 *
 * Revision 0.101  1995/10/05  04:39:59  drapeau
 * Removed functionality made obsolete by the new OutputCascade code for
 * representing and selecting output signal types.
 *
 * Revision 0.100  1995/05/31  18:58:55  drapeau
 * Modified Input2Callback() to call ShowInputSettings() so that the
 * interface always shows the most current input settings.
 *
 * Revision 0.99  1995/05/26  00:41:26  drapeau
 * Fixed an error in OutputCallback(): the output interface controls were
 * not enabled when output was turned on, nor were they disabled when
 * output was turned off.  They are now correctly enabled and disabled at
 * the appropriate times.
 *
 * Revision 0.98  1994/10/06  21:27:10  drapeau
 * Added a stub for a function written as an HP hack,
 * RestoreInput2IfNeeded().  Since it is called by OS-neutral code, a
 * stub had to be placed in this Sun-specific module.
 *
 * Revision 0.97  1994/09/30  16:53:22  drapeau
 * Added two parameters to SaveImageFileInFormat(), to accommodate saving
 * images in other than full-size (width & height).
 *
 * Revision 0.96  1994/09/23  06:46:33  drapeau
 * New code to handle detection of input signals.  If inputs are not
 * enabled, saving of stills is not allowed.
 *
 * Revision 0.95  1994/09/13  08:04:59  drapeau
 * Fixed an error in Input2Callback(), which was incorrectly setting the
 * 2nd input's input to the 1st digitizer.  Also, added a call to
 * ResizeVideoWidgetIfNecessary() to handle widget adjustments when
 * finding a new signal standard.
 *
 * Revision 0.94  1994/09/04  07:15:45  drapeau
 * Enabled auto-search on video input widgets.
 *
 * Revision 0.93  1994/08/24  22:32:02  drapeau
 * Removed a memory leak in SaveImageFileInFormat(); now properly
 * destroying image data after saving to disk and no longer needed.
 *
 * Revision 0.92  1994/08/19  02:36:06  andyg
 * Moved function SaveImageFileInFormat() into this platform-
 * specific file.
 *
 * Revision 0.91  1994/08/14  01:43:45  drapeau
 * Added the function OsSpecificInit() for platform-specific
 * initialization code.
 *
 * Revision 0.90  1994/08/10  03:09:00  drapeau
 * First working version.
 * */
static char sunSpecificRcsId[] = "$Header: /plx/source/Xclients/generic/RCS/sunSpecific.c,v 0.102 1996/01/22 07:58:05 drapeau Exp $";

#include "VideoTool.h"

#include <display.h>						    /* Include files from the ImageMagick package */
#include <image.h>
#include <alien.h>
#include <X.h>

#define True 1
#define False 0


void Input2Callback(Widget	w,
		    XtPointer	client_data,
		    XtPointer	xt_call_data )
{
  static int			lastStandard = PLX_NTSC;
  int				signalStandard;
  XmString			onString = XmStringCreateSimple("On");
  XmString			offString = XmStringCreateSimple("Off");
  XmToggleButtonCallbackStruct*	callData =
    (XmToggleButtonCallbackStruct *) xt_call_data;
  
  if (callData->set == True)
  {
    XtPopup(input2Shell, XtGrabNone);
    XtVaSetValues(videoIn2Toggle, XmNlabelString, onString, NULL);
    XtVaSetValues(input2Widget, XtNchannel, PLX_INPUT_1, NULL);	    /* Restart video input if possible */
    XtVaGetValues(input2Widget,					    /* Determine if an active signal is on input 2 and... */
		  XtNsync, &input2Sync,				    /* ...the new signal standard for the widget, if any. */
		  XtNsignalStandard, &signalStandard,
		  NULL);
    ShowInputSettings(input2ChannelButton, (XtPointer)NULL,	    /* Update the current input settings now that a new... */
		      (XtPointer)NULL);				    /* ...input signal has been selected. */
    if (input2Sync == False)					    /* Sync wasn't found, don't bother popping up the shell */
    {
      XmStringFree(onString);
      XmStringFree(offString);
      return;
    }
    XtVaSetValues(stillInput2Button, XmNsensitive, True, NULL);	    /* Allow capture on the 2nd video input window */
    if (lastStandard != signalStandard)				    /* Did the signal standard for this widget change? */
      ResizeVideoWidgetIfNecessary(input2Widget);		    /* Yes, widget might need resizing */
    lastStandard = signalStandard;
  }
  else
  {
    XtPopdown(input2Shell);
    XtVaSetValues(videoIn2Toggle, XmNlabelString, offString, NULL);
    XtVaSetValues(input2Widget, XtNchannel, PLX_IO_OFF, NULL);	    /* Turn off video input 2 */
    XtVaSetValues(stillInput2Button, XmNsensitive, False, NULL);    /* Don't allow capture on the 2nd video input window */
  }
  XmStringFree(onString);
  XmStringFree(offString);
  return;
}								    /* end function Input2Callback */



void OutputCallback(Widget	w,
		    XtPointer	client_data,
		    XtPointer	xt_call_data)
{
  XmToggleButtonCallbackStruct*	callData =
    (XmToggleButtonCallbackStruct *) xt_call_data;
  XmString			onString = XmStringCreateSimple("On");
  XmString			offString = XmStringCreateSimple("Off");
  int				channel;
  int				instance;
  XmString			label;
  int				signalFormat;
  int				signalStandard;
  signalSpec			theSignal;
  
  if (callData->set == True)
  {
    XtVaSetValues(outputWidget, XtNchannel, PLX_OUTPUT_0, NULL);    /* Turn on generation of a video signal, if not already on */
    XtVaSetValues(videoOutToggle, XmNlabelString, onString, NULL);
    EnableOutputControls();
    XtVaGetValues(outputWidget,					    /* Try to get the currently-selected output values */
		  XtNchannel, &channel,
		  XtNsignalStandard, &signalStandard,
		  XtNsignalFormat, &signalFormat,
		  XtNinstance, &instance,
		  NULL);
    theSignal.s.format = signalFormat;
    theSignal.s.instance = instance;
    theSignal.s.standard = signalStandard;
    theSignal.s.channel = channel;
    PlxShowOutputSelections(outputMenuCascade, theSignal);	    /* Update the output settings cascade menu */
  }
  else
  {
    XtVaSetValues(outputWidget,					    /* Turn off generation of a video signal */
		  XtNchannel, PLX_IO_OFF,
		  XtNnormalAspectRatio, True,			    /* Also, reset to 4-to-3 aspect ratio in case window...*/
		  NULL);					    /* ...tracking had been on. */
    XtVaSetValues(videoOutToggle, XmNlabelString, offString, NULL);
    DisableOutputControls();
  }
  XmStringFree(onString);
  XmStringFree(offString);
  return;
}								    /* end function OutputCallback */



void OsSpecificInit(void)
{
  autoSearch = True;						    /* Allow auto-searching for a valid video input signal */
  return;							    /* For now, don't do anything special for the Sun platform */
} /* end function OsSpecificInit */



void SaveImageFileInFormat(Display*	display,
			   Window	imageWindow,
			   Dimension	width,
			   Dimension	height,
			   char*	fileFormat,
			   char*	fileName)
{
  char		windowName[256];
  Image*	theImage = (Image*)NULL;
  
  sprintf(windowName, "%dx", imageWindow);
  theImage = ReadXImage(windowName,				    /* Use ImageMagick routine to digitize the contents of an... */
			XDisplayName(NULL),			    /* ...X window, the one that will be written to disk. */
			False, False, False);
  if (theImage == (Image*)NULL)					    /* Did the image grab fail? */
    return;							    /* Yes, return without doing anything more */
  sprintf(theImage->filename, "%s:%s", fileFormat, fileName);
  WriteAlienImage(theImage);					    /* Write the Sun Raster file to disk */
  DestroyImage(theImage);					    /* Done with the space needed to store the image */
  return;
}								    /* end function SaveImageFileInFormat */



void RestoreInput2IfNeeded(void)
{
  return;							    /* This is a no-op on the Sun */
} /* end function RestoreInput2IfNeeded */
