/*
 *   Copyright (c) 1994 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Header: /plx/source/Xclients/generic/RCS/utils.c,v 1.24 1996/01/22 07:31:21 drapeau Exp $ */
/* $Log: utils.c,v $
 * Revision 1.24  1996/01/22  07:31:21  drapeau
 * Added support for a new XVideoIn widget resource, "stillDelay".  The
 * new resource allows the programmer to set the delay time between the
 * time the widget freezes the digitizer and the time the widget grabs a
 * still image from its window.
 *
 * Revision 1.23  1996/01/18  19:18:24  drapeau
 * Minor change to BuildInputMenu() to make it harder to accidentally
 * still a live video window.
 *
 * Revision 1.22  1996/01/09  02:38:34  drapeau
 * Removed the calls to PlxCompress{In,Out}putCascade() since they do not
 * currently work correctly on PowerVideo products.
 *
 * Revision 1.21  1996/01/08  21:48:04  drapeau
 * Two changes:
 *
 * 1) Added code to check whether the 2nd input widget is created before
 * trying to install a translation table on it.
 *
 * 2) Added the functions PlxCompress{In,Out}putCascade() that eliminates
 * extraneous menu interface components.
 *
 * Revision 1.20  1996/01/05  21:04:06  drapeau
 * 1) Added a translation table and actions table to the XVideoIn widgets
 * in the function BuildInputMenu() to allow for quicker and easier
 * saving of still images.  The new functionality allows you to click on
 * either of the Video Input windows and instantly grab a still image.
 *
 * Revision 1.19  1995/12/20  02:27:28  drapeau
 * Enhanced the autoSearch method.  The new autoSearch method now allows
 * the application programmer to specify a specific digitizer on which to
 * do autoSearch.  The previous version of the autoSearch feature in the
 * XVideoIn widget only allowed autoSearch to be performed on all
 * digitizers.
 *
 * Revision 1.18  1995/10/27  17:25:01  drapeau
 * Modified BuildInputMenu() so that it checks whether hardware zoom is
 * available; if not, the appropriate interface component is greyed out.
 *
 * Revision 1.17  1995/10/25  00:32:55  drapeau
 * Fixed minor error in BuildInputMenu(): correctly initialized the
 * "numDigitizers" variable.
 *
 * Revision 1.16  1995/10/06  21:07:27  drapeau
 * Minor change to CreateLogoPixmap(), adding the logo to the main panel.
 *
 * Revision 1.15  1995/10/05  04:30:34  drapeau
 * Added support for selecting a single window for output.  The previous
 * version of the application supported only selecting a region of the
 * display, not tracking an individual window.
 *
 * Revision 1.14  1995/08/18  01:55:07  drapeau
 * Added code to put the company logo in the application's "About..."
 * panel.
 *
 * Revision 1.13  1995/08/17  20:03:02  drapeau
 * Added support for putting the Parallax logo in the "About..." panel.
 * Not yet used.
 *
 * Revision 1.12  1995/06/23  19:23:37  drapeau
 * Two changes to BuildInputMenu():
 *
 * 1) Fixed an error in the user interface code: if the 2nd digitizer is
 *    not available, don't allow the "Window 2" popup menu to be chosen.
 *
 * 2) Minor cosmetic fix: move a comment around to make the code more
 *    readable.
 *
 * Revision 1.11  1995/05/16  00:34:17  lovvik
 * Modified BuildInputMenu to count the
 * number of digitizers and use that number
 * to decide whether or not to create
 * input2Widget.
 *
 * Revision 1.10  1995/05/15  16:57:41  drapeau
 * Added support for new modules that automatically create cascading
 * menus to reflect the input choices available on the Parallax hardware
 * being used.
 *
 * Revision 1.9  1995/02/16  00:27:52  drapeau
 * Two changes:
 *
 * - Moved FirstAvailableInput() into this module from the hpSpecific.c
 *   module since the code appears to be platform-neutral.
 *
 * - Modified BuildInputMenu() to add stmts to explicitly assign the
 *   result of calls to XPlxFindMatchingSignal() to temporary variables
 *   that are later used in if...then tests.  This technique aides in
 *   debugging, since it allows the programmer to stop at the point just
 *   after the call to XPlxFindMatchingSignal() was made.
 *
 * Revision 1.8  1994/10/14  21:13:28  drapeau
 * Added code to keep track of whether optional signal types (e.g., YC)
 * are available.
 *
 * Revision 1.7  1994/10/10  18:19:57  drapeau
 * Fixed minor error in DisableOutputControls(): the aspect ratio toggle
 * buttons had not had their sensitivity set to False.
 *
 * Revision 1.6  1994/09/25  07:31:04  drapeau
 * Minor code clean-up, to correctly reflect which inputs are currently
 * shown.
 *
 * Revision 1.5  1994/09/23  06:26:38  drapeau
 * Several changes made to support output more completely.  The
 * functionality disables output settings controls when output is turned
 * off.
 *
 * Revision 1.4  1994/09/03  01:58:32  drapeau
 * Minor modifications to PlaceWindows(), using XmN strings instead of
 * XtN strings in the hope that Motif widgets would do a better job of
 * suggesting their preferred positions to the HP VUE window manager.  So
 * far the attempt has failed.
 *
 * Revision 1.3  1994/08/27  05:12:11  drapeau
 * Fixed the application so that it does not enable output by default
 * (the user must select the toggle in order to enable output).  Also
 * added the function PrintErrorMessage(), a convenience function to put
 * up a simple error dialog box with a message.
 *
 * Revision 1.2  1994/08/10  03:12:08  drapeau
 * Made the application's sensitivity to various board configurations
 * more complete.  The application should now be fully aware of whether
 * the underlying hardware has two digitizers or only one.
 *
 * Revision 1.1  1994/08/01  18:55:57  drapeau
 * Initial revision
 * */
static char utilsRcsId[] = "$Header: /plx/source/Xclients/generic/RCS/utils.c,v 1.24 1996/01/22 07:31:21 drapeau Exp $";

#include "VideoTool.h"


void PlaceWindows()						    /* Try to place the main windows in a nice position */
{
  int		screenWidth;
  Dimension	width;
  Dimension	height;
  
  screenWidth = XWidthOfScreen(XtScreen(mainWindow));		    /* How wide is the screen?  Use for centering windows */
  XtVaGetValues(mainWindow,					    /* How large is the main window? */
		XtNwidth, &width,
		XtNheight, &height,
		NULL);
  XtVaSetValues(mainWindow,					    /* Place the main window at the top of the screen, centered */
		XmNx, (Position)((screenWidth / 2) - (width / 2)),
		XmNy, 0,
		NULL);
  XtVaSetValues(input1Shell,					    /* Place the 1st video input window at the left side of... */
		XmNx, 0,					    /* ...the screen, just below the main window. */
		XmNy, (Position)height,
		NULL);
  XtVaGetValues(input2Shell,					    /* How wide is video input window 2?  Use for centering */
		XtNwidth, &width,
		NULL);
  XtVaSetValues(input2Shell,					    /* Place the 2nd video input window against the right... */
		XmNx, (Position)(screenWidth - width),		    /* ...side of the screen, also just below the main window. */
		XmNy, (Position)height,
		NULL);
  XtVaGetValues(imageShell,					    /* How wide is the still image shell?  Use for centering */
		XtNwidth, &width,
		NULL);
  XtVaSetValues(imageShell,					    /* Place the still image shell window centered, just... */
		XmNx, (Position)((screenWidth / 2) - (width / 2)),  /* ...below the main window. */
		XmNy, (Position)height,
		NULL);
  return;
} /* end function PlaceWindows */



void BuildInputMenu(void)
{
  plx_IO*		hardwareInfo;				    /* Info about the Parallax board being used */
  plx_IO_list*		inputList = (plx_IO_list*) NULL;
  plx_IO_list*		outputList = (plx_IO_list*) NULL;
  int			autoSearchOn = False;
  int			channel;
  int			numDigitizers = 0;
  Bool			signalFound;
  Bool			signal2Found;
  int			stillDelay;
  Boolean		sync;
  Boolean		outputAvailable;
  static String		vtTranslations = "#override Shift<BtnUp>: SaveStill()";
  XtTranslations	transTable;
  static XtActionsRec	vtActions[] = 
  {
	{"SaveStill", SaveStill},
  };
  
  XtVaSetValues(input2ChannelButton, XmNsensitive, False, NULL);    /* By default, don't let 2nd input settings to be used */
  XtVaGetValues(input1Widget,					    /* Retrieve information about the hardware being used */
		XtNhardwareParams, &hardwareInfo,
		NULL);
  inputMenuCascade = (PlxInputCascade*)NULL;
  inputMenuCascade = PlxCreateInputCascade(inputTypeRowColumn,	    /* Create a cascading menu that creates only those input... */
					   (Widget)NULL,	    /* ...selections that are appropriate for the hardware... */
					   (Widget)NULL,	    /* ...being used. */
					   (Widget)NULL,
					   hardwareInfo,
					   SetSignalStandard);	    /* Call "SetSignalStandard()" to change input settings */
  outputMenuCascade = (PlxOutputCascade*)NULL;
  outputMenuCascade = PlxCreateOutputCascade(outputTypeRowColumn,   /* Create a cascading menu that creates only those output... */
					     (Widget)NULL,	    /* ...selections that are appropriate for the hardware... */
					     (Widget)NULL,	    /* ...being used. */
					     (Widget)NULL,
					     hardwareInfo,
					     SetOutputSignalType);
  XtVaSetValues(videoIn2Toggle, XmNsensitive, False, NULL);	    /* Dim the second input, assuming only 1 digitizer on board */
  XtVaSetValues(videoOutToggle, XmNsensitive, False, NULL);	    /* Dim the output, too, assuming only 1 digitizer on board */
  XtVaSetValues(outputConfigButton, XmNsensitive, False, NULL);	    /* Don't let the output configuration panel be selected */
  for (inputList = hardwareInfo->inputs;			    /* Search through all available inputs for this hardware,... */
       inputList != (plx_IO_list*) NULL;			    /* ...to determine if 2 digitizers are on board. */
       inputList = inputList->next_IO)
  {
    numDigitizers++;
  }
  if(numDigitizers == 2)
  {
    XtVaSetValues(videoIn2Toggle, XmNsensitive, True, NULL);	    /* Allow the 2nd input to be used */
    XtVaGetValues(input1Widget,					    /* See what channel the 1st video widget is using... */
		  XtNsync, &sync,				    /* ...and whether it has a valid sync signal detected. */
		  XtNchannel, &channel,
		  NULL);
    if ((True == sync) && (PLX_INPUT_0 == channel))		    /* If the other widget sees a valid sync signal on the... */
    {								    /* ...1st digitizer, then set autoSearch to look for... */
      autoSearchOn = AutoSearchInput1;				    /* ...a signal on only the 2nd digitizer. */
    }
    if ((True == sync) && (PLX_INPUT_1 == channel))		    /* If the other widget sees a valid sync signal on the... */
    {								    /* ...2nd digitizer, then set autoSearch to look for... */
      autoSearchOn = AutoSearchInput0;				    /* ...a signal on only the 1st digitizer. */
    }
    if (False == sync)						    /* If the other widget sees no valid sync, then allow... */
    {								    /* ...autoSearch to look for a signal on any digitizer. */
      autoSearchOn = autoSearch;
    }
    input2Widget = XtVaCreateManagedWidget("input2Widget",
					   xVideoInWidgetClass,
					   input2Shell,
					   XtNchannel, PLX_IO_OFF,
					   XtNchannel, PLX_INPUT_1,
					   XtNinstance, 1,	    /* Choose the 2nd video cable for this digitizer by default */
					   XtNautoSearch, autoSearchOn,
					   NULL);
    XtVaGetValues(input2Widget, XtNsync, &input2Sync, NULL);	    /* Determine if an active signal is on input 2 */
  }
  outputList = hardwareInfo->outputs;				    /* Point to the list of output information for this board */
  if (outputList != (plx_IO_list*)NULL)				    /* Does output exist for this hardware? */
  {								    /* Yes, do the appropriate thing */
    outputAvailable = True;
    outputWidget = XtVaCreateManagedWidget("outputWidget",	    /* Enable video output (as a side effect, video... */
					   xVideoOutWidgetClass,    /* ...output will be turned on immediately). */
					   outputShell,
					   NULL);
    XtVaSetValues(outputWidget, XtNchannel, PLX_IO_OFF, NULL);	    /* Turn off generation of a video signal */
    XtVaSetValues(videoOutToggle, XmNsensitive, True, NULL);	    /* Allow output to be used */
    DisableOutputControls();					    /* Don't allow output settings to be changed just yet */
  }
  else								    /* No output on this hardware */
  {
    outputAvailable = False;
  }
  if (0 == hardwareInfo->zoom)					    /* Does this product lack the ability to zoom? */
  {
    XtVaSetValues(zoomCascade, XmNsensitive, False, NULL);	    /* Yes, no zoom, disable the appropriate interface component */
  }
  XtAppAddActions(app_context, vtActions, XtNumber(vtActions));	    /* Make app-specific actions available for extra neat stuff */
  transTable = XtParseTranslationTable(vtTranslations);		    /* Build a translation table to be used for XVideoIn widgets */
  XtVaSetValues(input1Widget,					    /* Install the new translations for each of the video... */
		XtNtranslations, transTable,			    /* ...input widgets. */
		NULL);
  if (input2Widget != (Widget)NULL)				    /* Was a 2nd input widget created? */
    XtVaSetValues(input2Widget,					    /* Yes, go ahead and create a translation table for it, too */
		  XtNtranslations, transTable,
		  NULL);
  XtVaGetValues(input1Widget, XtNstillDelay, &stillDelay, NULL);    /* Retrieve 1st input widget's current "stillDelay" value */
  XmScaleSetValue(captureDelayScale, stillDelay);		    /* Display the current "stillDelay" value in the interface */
  return;
} /* end function BuildInputMenu */



void PrintErrorMessage(char*	message)
{
  XmString		messageString;
  
  messageString = XmStringCreateLocalized(message);
  XtVaSetValues(alertShellMessageLabel, XmNlabelString, messageString, NULL);
  XmStringFree(messageString);
  if (XtIsManaged(alertWidget) == False)
  {
    XtManageChild(alertWidget);
  }
} /* end function PrintErrorMessage */



void DisableOutputControls(void)
{
  XtVaSetValues(outputAnyAspectToggle, XmNsensitive, False, NULL);
  XtVaSetValues(output4To3AspectToggle, XmNsensitive, False, NULL);
  XtVaSetValues(outputTypeRowColumn, XmNsensitive, False, NULL);
  XtVaSetValues(maxSizeButton, XmNsensitive, False, NULL);
  XtVaSetValues(showOutputRectButton, XmNsensitive, False, NULL);
  XtVaSetValues(outputConfigButton, XmNsensitive, False, NULL);
  return;
} /* end function DisableOutputControls */



void EnableOutputControls(void)
{
  XtVaSetValues(outputAnyAspectToggle, XmNsensitive, True, NULL);
  XtVaSetValues(output4To3AspectToggle, XmNsensitive, True, NULL);
  XtVaSetValues(outputTypeRowColumn, XmNsensitive, True, NULL);
  XtVaSetValues(maxSizeButton, XmNsensitive, True, NULL);
  XtVaSetValues(showOutputRectButton, XmNsensitive, True, NULL);
  XtVaSetValues(outputConfigButton, XmNsensitive, True, NULL);
  return;
} /* end function EnableOutputControls */



void CreateLogoPixmap(Widget	parentWidget)			    /* Creates a masked Pixmap in shape of the company logo... */
{								    /* ...to display in the application's "About..." panel. */
  unsigned long	background;
  Display*	display = XtDisplay(parentWidget);
  int		depth;
  XGCValues	gcValues;
  Pixmap	logoPixmap;
  Pixmap	logoShapeMask;
  XpmAttributes	logoAttrs;
  GC		logoGC;
  int		result;
  Pixmap	thePixmap;
  Window	window = XtWindow(parentWidget);
  
  logoAttrs.valuemask = 0;
  result = XpmCreatePixmapFromData(display,			    /* Attempt to create a Pixmap from XPM data */
				   window,			    /* The function, if successful, will return both a Pixmap... */
				   plxLogoData,			    /* ...in the shape of the company logo, in color, plus... */
				   &logoPixmap,			    /* ...a mask that can be used to show only the Pixmap... */
				   &logoShapeMask,		    /* ...data (as if it were transparent). */
				   &logoAttrs);
  if (result != 0)						    /* Did the Pixmap creation fail? */
    return;							    /* Yes, don't use any Pixmap for the logo panel */
  XtVaGetValues(parentWidget,					    /* Get the depth and background color of the parent widget */
		XmNdepth, &depth,
		XmNbackground, &background,
		NULL);
  thePixmap = XCreatePixmap(display,				    /* Create a new Pixmap to be used as the application's logo */
			    window,
			    logoAttrs.width, logoAttrs.height,
			    depth);
  gcValues.background = background;
  gcValues.foreground = background;
  logoGC = XCreateGC(display,					    /* Create a GC that paints the application's background... */
		     window,					    /* ...color into the final logo Pixmap */
		     (GCBackground | GCForeground),
		     &gcValues);
  XFillRectangle(display, thePixmap, logoGC, 0, 0,		    /* Paint the bkgnd color into the final logo Pixmap */
		 logoAttrs.width, logoAttrs.height);
  XSetClipMask(display, logoGC, logoShapeMask);			    /* Make sure that painting the new, color logo data only... */
  XCopyArea(display, logoPixmap, thePixmap, logoGC,		    /* ...paints into the areas it's supposed to, making... */
	    0, 0, logoAttrs.width, logoAttrs.height,		    /* ...a "transparent" effect. */
	    0, 0);
  XtVaSetValues(aboutDialog, XmNsymbolPixmap, thePixmap, NULL);	    /* Use the newly-created icon as the About panel's symbol */
  XtVaSetValues(logoLabel, XmNlabelPixmap, thePixmap, NULL);	    /* Use the newly-created icon as the logo label's label */
  return;
} /* end function CreateLogoPixmap */



Boolean SignalHandlerWorkProc(XtPointer clientData)
{
  int	theSignal = (int)clientData;
  
  switch(theSignal)
  {
   case SIGHUP:
    printf("Caught a SIGHUP signal; Exiting the application.\n");
    Quit((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);
    return(True);
    break;
   case SIGINT:
    printf("Caught a SIGINT signal; Exiting the application.\n");
    Quit((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);
    return(True);
    break;
   case SIGQUIT:
    printf("Caught a SIGQUIT signal; Exiting the application.\n");
    Quit((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);
    return(True);
    break;
   case SIGTERM:
    printf("Caught a SIGTERM signal; Exiting the application.\n");
    Quit((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);
    return(True);
    break;
    
  } /* end switch */
  return(True);
} /* end function SignalHandlerWorkProc */



void SignalHandler(int theSignal)
{
  XtAppAddWorkProc(app_context,					    /* Pass in the signal being handled as client data */
		   SignalHandlerWorkProc,
		   (XtPointer)theSignal);
  XSync(XtDisplay(mainWindow), False);
  printf("Just called XSync.\n");
  return;
} /* end function SignalHandler */



void CatchSignals(void)
{
  signal(SIGHUP, SignalHandler);
  signal(SIGINT, SignalHandler);
  signal(SIGQUIT, SignalHandler);
  signal(SIGTERM, SignalHandler);
  return;
} /* end function CatchSignals */



Window SelectWindow(Widget widget)
{
  int		status;
  Cursor	cursor;
  XEvent	event;
  Window	target_win = None;
  Window	root;
  Display*	dpy;
  int		buttons = 0;
  
  dpy = XtDisplay(widget);
  root = RootWindow(dpy, XScreenNumberOfScreen(XtScreen(widget)));
  cursor = XCreateFontCursor(dpy, XC_crosshair);		    /* Make the target cursor */
  status = XGrabPointer(dpy, root, False,			    /* Grab the pointer using target cursor, letting it... */
			ButtonPressMask|ButtonReleaseMask,	    /* ...roam all over */
			GrabModeSync,
			GrabModeAsync, root, cursor, CurrentTime);
  if (status != GrabSuccess)
    return(root);
  while ((target_win == None) || (buttons != 0))		    /* Let the user select a window... */
  {
    XAllowEvents(dpy, SyncPointer, CurrentTime);		    /* allow one more event */
    XWindowEvent(dpy, root, ButtonPressMask|ButtonReleaseMask, &event);
    switch (event.type)
    {
     case ButtonPress:
      if (target_win == None)
      {
	target_win = event.xbutton.subwindow;			    /* window selected */
	if (target_win == None) target_win = root;
      }
      buttons++;
      break;
     case ButtonRelease:
      if (buttons > 0)						    /* There may have been some down before we started */
	buttons--;
       break;
    }
  } 
  XUngrabPointer(dpy, CurrentTime);				    /* Done with pointer */
  return(target_win);
} /* end function SelectWindow */
