/* $Header: /plx/source/Xclients/generic/RCS/videoIcon.c,v 1.3 1995/10/13 19:20:10 lovvik Exp $ */

/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */

/* $Log: videoIcon.c,v $
 * Revision 1.3  1995/10/13  19:20:10  lovvik
 * Modified GetIconSize() to return values <= 60 pixels.  The
 * problem with choosing the largest possible icon is that olwm
 * supports icons up to 160x160 pixels.  This is too large to be
 * an icon.
 *
 * Revision 1.2  1995/10/13  16:58:11  lovvik
 * Added a function called GetIconSize() that returns the largest
 * icon size supported by the window manager.
 *
 * Revision 1.1  1995/10/13  00:37:50  lovvik
 * Initial revision
 * */

static char videoIconRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/videoIcon.c,v 1.3 1995/10/13 19:20:10 lovvik Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <Xm/Xm.h>
#include <XPlxExt.h>
#include <XVideoIn.h>

#define BEST_ICON_SIZE 60

/* Global data */
Widget video, videoicon;

Widget CreateIconWidget(Widget parent, int width,
			int height)
{
  Widget iconwidget;
  iconwidget = XtVaCreatePopupShell("iconwindow",
				    transientShellWidgetClass,
				    parent,
				    XmNwidth, (Dimension)width,
				    XmNheight, (Dimension)height,
				    NULL);
  XtRealizeWidget(iconwidget);

  XtVaSetValues(parent,
		XtNiconWindow, XtWindow(iconwidget),
		XtNinitialState, IconicState,
		NULL);

  return(iconwidget);
} /* end of CreateIconWidget */

static track_icon_status(Widget widget,
			 XtPointer client_data,
			 XEvent* event,
			 Boolean* continue_to_dispatch)
{

  /*
   * This function manages the appropriate widget.
   */

  if(continue_to_dispatch)
  {
  }

  switch(event->xany.type)
  {
  case MapNotify:
                {
		  XtUnmanageChild(videoicon);
		  XtManageChild(video);
		  break;
		}
  case UnmapNotify:
		{
		  XtUnmanageChild(video);
		  XtManageChild(videoicon);
		  break;
		}
  } /* end of switch */

} /* end of track_icon_status */

void TrackIconStatus(Widget shell_widget,
		     XtPointer client_data)
{
  XtAddEventHandler(shell_widget,
		    StructureNotifyMask,
		    False,
		    (XtEventHandler) track_icon_status,
		    client_data);
} /* end of TrackIconStatus */


void GetIconSize(int *width, int *height)
{
  Display *display;
  Window window;
  int screenNumber;
  XIconSize *sizeList;
  int sizeCount;
  int widthIncrement;
  int heightIncrement;

  /*
   * This function returns an icon size supported by the window manager.
   */
  
  display = XtDisplay(video);
  screenNumber = XScreenNumberOfScreen(XtScreen(video));
  window = XRootWindow(display, screenNumber);
  
  if(0 != XGetIconSizes(display,
			XRootWindow(display, screenNumber),
			&sizeList,
			&sizeCount))
  {
    /*
     * 
     */
    widthIncrement = 0;
    heightIncrement = 0;

    while((sizeList->max_width - widthIncrement)  > BEST_ICON_SIZE)
    {
      widthIncrement += sizeList->width_inc;
    }
	
    while((sizeList->max_height - heightIncrement) > BEST_ICON_SIZE)
    {
      heightIncrement += sizeList->height_inc;
    }

    *width = sizeList->max_height - heightIncrement;
    *height = sizeList->max_height - heightIncrement;
    XFree(sizeList);
  }
  else
  {
    /*
     * XGetIconSizes() failed.  Use a generic value.
     */
    *width = 60;
    *height = 60;
  }
}


main(argc, argv)
int argc;
char** argv;
{
  Widget shell, icon;
  XtAppContext app;
  Arg args[20];
  int n = 0;
  int iconWidth;
  int iconHeight;

  /*
   * This program creates 2 video widgets (one full size, the
   * other icon size).  We simply catch all mapping events and
   * determine which widget to manage.
   */

  XtSetArg(args[n], XmNmappedWhenManaged, False); n++;
  XtSetArg(args[n], XmNwidth, 640); n++;
  XtSetArg(args[n], XmNheight, 480); n++;

  shell = XtAppInitialize(&app, "Videoicon", NULL, 0,
                          &argc, argv, NULL, args, n);
  XtRealizeWidget(shell);

  /*
   * Since this application will start in the iconic state, do
   * not manage the video widget.  Instead, we will manage the
   * videoicon widget.
   */
  video = XtVaCreateWidget("Video", xVideoInWidgetClass,
			   shell,
			   XmNwidth, 640,
			   XmNheight, 480,
			   NULL);

  GetIconSize(&iconWidth, &iconHeight);
  icon = CreateIconWidget(shell, iconWidth, iconHeight);
  videoicon = XtVaCreateManagedWidget("VideoIcon", xVideoInWidgetClass,
				      icon,
				      XmNx, iconWidth,
				      XmNy, iconHeight,
				      XtNnormalAspectRatio, False,
				      NULL);

  TrackIconStatus(shell, (XtPointer) NULL);

  XtMapWidget(shell);
  XtAppMainLoop(app);
}
