/* $Header: /plx/source/Xclients/generic/RCS/scomp.c,v 1.78 1995/12/20 17:46:36 lovvik Exp $ */
/*
 *   Copyright (c) 1990 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Murali. V. Srinivasan
 */

/* $Log: scomp.c,v $
 * Revision 1.78  1995/12/20  17:46:36  lovvik
 * Changed the way shared memory is allocated.  This fixed a
 * lock-up problem.  Also, I changed the way the command-line
 * arguments were parsed.  This is the correct way to do parse
 * command-line arguments in Xt.
 * */
static char scompRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/scomp.c,v 1.78 1995/12/20 17:46:36 lovvik Exp $";

#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <X11/Xlib.h>
#include <X11/extensions/XShm.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/PanedW.h>
#include <Xm/BulletinB.h>
#include <Xm/RowColumn.h>
#include <Xm/PushBG.h>
#include <Xm/Scale.h>
#include <Xm/Protocols.h>
#include <XPlxExt.h>
#include <XPlxProtos.h>
#include <TCWindow.h>

#define XtNuseSharedMemory "useSharedMemory"
#define XtNdiagnostics "diagnostics"
#define XtNhelp "help"
#define XtCUseSharedMemory "UseSharedMemory"
#define XtCDiagnostics "Diagnostics"
#define XtCHelp "Help"

static Atom 			deleteWindowAtom;
static Bool			xShmAttachErrorOccurred = False;
static Bool			xShmAttachOccurred = False;
static Display *		display;
static GC 			videoGC;
static GC			decompressionGC;
static int 			videowidth = 320;
static int 			videoheight = 240;
static int 			sourcewidth;
static int			sourceheight;
static int			sourcestart;
static int 			initialVideoWindowExpose = True;
static int 			waitingForButtonPress = True;
static int 			requestWorkerExit = False;
static int			shmMajorOpcode;
static short 			QFactor;
static unsigned char *		qTable;
static void *			currentErrorHandler;
static Widget 			sourcePopup;
static Widget 			swidget;
static Widget			dwidget;
static Widget			slwidget;
static Widget			form;
static Widget 			decompressionPopup;
static Widget			sliderPopup;
static Window 			sourceWindow;
static Window			destinationWindow;
static XPlxCImage *		shmimage = NULL;
static XShmSegmentInfo *	shminfo;
static XtAppContext 		app_context;
static XtWorkProcId		workerId = (XtWorkProcId)NULL;

typedef struct
{
  Boolean useSharedMemory;
  Boolean diagnostics;
  Boolean help;
} ScompData;

ScompData scompData;

XtResource resources[] = 
{
  {
    XtNuseSharedMemory,
    XtCUseSharedMemory,
    XtRBoolean,
    sizeof(Boolean),
    XtOffsetOf(ScompData, useSharedMemory),
    XtRImmediate,
    (XtPointer)True,
  },
  {
    XtNdiagnostics,
    XtCDiagnostics,
    XtRBoolean,
    sizeof(Boolean),
    XtOffsetOf(ScompData, diagnostics),
    XtRImmediate,
    (XtPointer)False,
  },
  {
    XtNhelp,
    XtCHelp,
    XtRBoolean,
    sizeof(Boolean),
    XtOffsetOf(ScompData, help),
    XtRImmediate,
    (XtPointer)False,
  },
};

  
static XrmOptionDescRec options[] = 
{
  {"-noSharedMemory",	"*useSharedMemory",	XrmoptionNoArg,	"False"},
  {"-diagnostics",	"*diagnostics",		XrmoptionNoArg, "True"},
  {"-help",		"*help",		XrmoptionNoArg, "True"},
};    

static String fallbackResources[] =
{
  "scomp*background: white",
  "scomp*foreground: black",
  NULL
};


int ShmErrorHandler(Display* display, XErrorEvent* errorEvent)
{
  /*
   * Was it the shared memory attach that failed?
   */
  if ((errorEvent->request_code == shmMajorOpcode) &&
      (errorEvent->minor_code == X_ShmAttach))
  {
    /*
     * Yes, return an error code.
     */
    xShmAttachErrorOccurred = True;
    return(0);
  }
  else
  {
    /*
     * No, something else failed, return success.
     */
    return(1);
  }
}


Boolean QuitWorkProc(XtPointer unusedClientData)
{
  if (scompData.useSharedMemory && xShmAttachOccurred)
  {
    XPlxShmDetach(display, shminfo);
    XPlxFreeShmInfo(shmimage->info);
    if(scompData.diagnostics == True)
    {
      printf("Shared memory detached.\n");
    }
  }
  exit(0);
}


/*
 * To clean up when program exit.
 */
static void
exit_func(sig, code, scp)
     int         code;
     int         sig;
     struct sigcontext *scp;
{
  signal (SIGINT, exit_func);
  signal (SIGSEGV, exit_func);
  signal (SIGBUS, exit_func);

  /*
   * If the worker work proceedure is being called,
   * request that it terminate the application.
   */
  if(workerId != (XtWorkProcId)NULL)
  {
    requestWorkerExit = True;
  }
  else
  {
    XtAppAddWorkProc(app_context,
		     QuitWorkProc,
		     (XtPointer)NULL);
  }
}

void WMDeleteWindowCallback(Widget widget,
			    XtPointer clientData,
			    XtPointer callData)
{
  /*
   * If the worker work proceedure is being called,
   * request that it terminate the application.
   */
  if(workerId != (XtWorkProcId)NULL)
  {
    requestWorkerExit = True;
  }
  else
  {
    XtAppAddWorkProc(app_context,
		     QuitWorkProc,
		     (XtPointer)NULL);
  }
}


static void 
dragcall(w, data, call_data)
     Widget w;
     caddr_t data, call_data;
{
  int size;
  
  XmScaleCallbackStruct *p = (XmScaleCallbackStruct *)call_data;
  
  QFactor = p->value;
  size = MakeQTables(QFactor, &qTable);
  XPlxPutTable(display, sourceWindow, videoGC, qTable, size, 0);
  XPlxPutTable(display, destinationWindow, decompressionGC, qTable, size, 1);
}



/*
 * this function figures the size of a window since sometimes the
 * the xt functions don't work properly.
 */
static void get_size(display, win, w, h, bw)
Display *display;
Window win;
int *w, *h, *bw;
{
    XWindowAttributes attrib;
    
    XGetWindowAttributes(display, win, &attrib);
    *w = attrib.width;
    *h = attrib.height;
    *bw = attrib.border_width;
}



/*
 *    function to make the destination window the same size as the source
 *    when the source size changes.
 */
static int flag = 0;
static void configit(w, data, event)
Widget w;
caddr_t data;
XEvent *event;
{
  if ((initialVideoWindowExpose == 0) || flag)
  {
    int bw;
    flag = 1;
    get_size(display, sourceWindow, &videowidth, &videoheight, &bw);
    if (((videowidth * 3) / 4) > ((videoheight * 4) / 3))
      videoheight = (videowidth * 3) / 4;
    else
      videowidth = (videoheight * 4) / 3;
    
    XtResizeWidget(sourcePopup, videowidth, videoheight, bw);
    XtResizeWidget(decompressionPopup, videowidth, videoheight, bw);
    XtResizeWidget(swidget, videowidth, videoheight, bw);
    XtResizeWidget(dwidget, videowidth, videoheight, bw);
  }
}


/*
 * Since we tag the windows such that the tag is not drawn
 * we need to tag as video so the next video/decompress
 * will set the tags properly
 */
static void exposit(w, data, event)
Widget w;
caddr_t data;
XExposeEvent *event;
{
  if (w == swidget)
    initialVideoWindowExpose = True;
}



/*
 *    Start things going on first button press
 */
static void buttonp(w, data, event)
Widget w;
caddr_t data;
XButtonEvent *event;
{
  waitingForButtonPress = False;
  if(scompData.diagnostics == True)
  {
    printf("Compression/decompression started.\n");
  }
}



/*
 *    Main work process.
 *    capture a video image, compress it, then decompress it
 */
Boolean worker_shm(XtPointer data)
{
  if(requestWorkerExit == True)
  {
    /*
     * The application is about to exit.
     */
    XtAppAddWorkProc(app_context,
		     QuitWorkProc,
		     (XtPointer)NULL);
    return(True);
  }

  /*
   * Wait until a button is pressed.
   */
  if (waitingForButtonPress) return False;

  if (initialVideoWindowExpose)
    XPlxVideoTag(display, sourceWindow, videoGC, PLX_VIDEO);

  /*
   * Capture a video frame into the source.
   */
  XPlxVideoSqueezeStill(display, sourceWindow, videoGC,
			0, sourcestart, sourcewidth, sourceheight,
			0, 0, videowidth, videoheight);

  /*
   * Get a compressed image.
   */
  shmimage->width = videowidth;	
  shmimage->height = videoheight;
  XPlxShmGetCImage(display, sourceWindow, videoGC, shmimage, 0, 0,
		   videowidth, videoheight);

  /*
   * Decompress the image into the destination window.
   */
  XPlxShmPutCImage(display, destinationWindow, decompressionGC,
		   shmimage, 0, 0, videowidth, videoheight,
		   0, 0, videowidth, videoheight, 0, 0);

  /*
   * Set the tag such that subsequent video/decompress will not
   * set the tag bits, saves time.
   */
  if (initialVideoWindowExpose)
  {
    XPlxVideoTag(display, sourceWindow, videoGC, PLX_VIDEO_OVR);
    initialVideoWindowExpose = 0;
  }

  return(FALSE); /* work processes must return FALSE */
} /* end function worker_shm */



Boolean worker_nonshm(XtPointer data)
{
  XPlxCImage *image;

  if(requestWorkerExit == True)
  {
    /*
     * The application is about to exit.
     */
    XtAppAddWorkProc(app_context,
		     QuitWorkProc,
		     (XtPointer)NULL);
    return(True);
  }

  /*
   * Wait until button is pressed.
   */
  if (waitingForButtonPress) return False;

  if (initialVideoWindowExpose)
    XPlxVideoTag(display, sourceWindow, videoGC, PLX_VIDEO);

  /*
   * Capture a video frame into the source
   */
  XPlxVideoSqueezeStill(display, sourceWindow, videoGC, 0, sourcestart, sourcewidth,
			sourceheight, 0, 0, videowidth, videoheight);

  /*
   * Get a compressed image
   */
  image = (XPlxCImage *) XPlxGetCImage(display, sourceWindow, videoGC, 0, 0,
				       videowidth, videoheight, videowidth, videoheight);

  /*
   * Decompress the image into the destination window
   */
  XPlxPutCImage(display, destinationWindow, decompressionGC, image, 0, 0, videowidth, videoheight,
		0, 0,  videowidth, videoheight, 1);

  /*
   * Free up storage.
   */
  XPlxDestroyCImage(image);

  /*
   * Set the tag such that subsequent video/decompress will not
   * set the tag bits, saves time.
   */
  if (initialVideoWindowExpose)
  {
    XPlxVideoTag(display, sourceWindow, videoGC, PLX_VIDEO_OVR);
    initialVideoWindowExpose = 0;
  }
  return(FALSE); /* work processes must return FALSE */
}

static void printUsage(c)
char *c;
{
  printf("\n");
  printf("usage: %s [-noSharedMemory] [-diagnostics] [-help]\n", c);
  printf("\n");
  exit(1);
}


Boolean PostRealizeWorkProc(XtPointer unusedClientData)
{
  Display *		display;
  int			shmFirstEvent;
  int			shmFirstError;
  Bool			sharedPixmaps;
  Bool			shmExtensionExists;
  plx_IO *		plxio;
  plx_IO_list *		iolist;
  plx_signal_list *	siglist;
  plx_signal *		sig;
  int			tmp;
  int			size;
  int			badsync;
  Bool			okayToUseSharedMemory;
  Status		status;
  int			majorVersion;
  int			minorVersion;
  XtWorkProc		worker = worker_nonshm;

  display = XtDisplay(swidget);
  XtPopup(decompressionPopup, XtGrabNone);
  XtPopup(sliderPopup, XtGrabNone);
  
  sourceWindow = XtWindow(swidget);
  if(scompData.diagnostics == True)
  {
    printf("Source window id = %ld\n", sourceWindow);
  }
  videoGC = XCreateGC(display, sourceWindow, 0, 0);
  XSetForeground(display, videoGC, 0);

  destinationWindow = XtWindow(dwidget);
  if(scompData.diagnostics == True)
  {
    printf("Destination window id = %ld\n", destinationWindow);
  }
  decompressionGC = XCreateGC(display, destinationWindow, 0, 0);
  XSetForeground(display, decompressionGC, 0);
  size = MakeQTables(QFactor, &qTable);	

  /*
   * Store qtables for compress and decompress in the GCs.
   */
  XPlxPutTable(display, sourceWindow, videoGC, qTable, size, 0);
  XPlxPutTable(display, destinationWindow, decompressionGC, qTable, size, 1);

  if (scompData.useSharedMemory)
  {
    shmExtensionExists = XQueryExtension(display,
					 PlxSharedMemoryExtensionName,
					 &shmMajorOpcode,
					 &shmFirstEvent,
					 &shmFirstError);
    if(shmExtensionExists == False)
    {
      if(scompData.diagnostics == True)
      {
	printf("Shared memory extension does not exist.\n");
      }
      scompData.useSharedMemory = False;
    }
    else
    {
      okayToUseSharedMemory = XPlxShmQueryVersion(display,
						  &majorVersion,
						  &minorVersion,
						  &sharedPixmaps);
      if(okayToUseSharedMemory == True)
      {
	shminfo = XPlxGetShmInfo();
	if(shminfo != (XShmSegmentInfo*)NULL)
	{
	  shmimage = (XPlxCImage *)XPlxShmCreateCImage(display, shminfo->shmaddr,
						       shminfo, 0, 0, 0);
	  currentErrorHandler = XSetErrorHandler(ShmErrorHandler);
	  XSync(display, False);

	  shminfo->readOnly = False;
	  status = XPlxShmAttach(display, shminfo);
	  XSync(display, False);
	  XSetErrorHandler(currentErrorHandler);
	  XSync(display, False);

	  if((status == 0) || xShmAttachErrorOccurred == True)
	  {
	    XPlxFreeShmInfo(shmimage->info);
	    scompData.useSharedMemory = False;
	    shmimage = NULL;
	    if(scompData.diagnostics == True)
	    {
	      printf("Shared memory attach error occurred.  Shared memory will not be used.\n");
	    }
	  }
	  else
	  {
	    xShmAttachOccurred = True;
	    worker = worker_shm;
	    if(scompData.diagnostics == True)
	    {
	      printf("Shared memory attached.\n");
	    }
	  }
	}
      }
    }
  }
  if(scompData.diagnostics == True)
  {
    printf("Shared memory %s being used.\n", 
	   (scompData.useSharedMemory && xShmAttachOccurred) ? "is" : "is not");
  }    

  /*
   * Query the server to get hardware/signal information.
   */
  plxio = (plx_IO *) XPlxQueryConfig(display, sourceWindow, videoGC);
  
  /*
   * If the hardware type is not XVideo, it can't work.
   */
  if (plxio->hardware != PLX_XVIDEO_24)
  {
    printf("scomp can only run on Parallax XVideo hardware\n");
    exit(1);
  }
    
  /*
   * Find a pointer to channel one input.
   */
  for (iolist = plxio->inputs; iolist && (iolist->channel != PLX_INPUT_0);
       iolist = iolist->next_IO);
    
  /*
   * Go through the list of signals and find an active signal.
   */
  badsync = 1;
    
  for (siglist = iolist->signal_list; siglist;
       siglist = siglist->next_signal)
  {
    /*
     * Do an input select.
     * We are only using input 0 so it, and PLX_COMP are hard coded.
     * We also know that all the XVideo hardware can do is PLX_RGB24.
     */
    XPlxVideoInputSelect(display, sourceWindow, videoGC, PLX_INPUT_0,
			 siglist->signal->standard,
			 PLX_COMP, PLX_RGB24);
      
    /*
     * Do a query video to find out if there is sync.
     */
    sig = (plx_signal *) XPlxQueryVideo(display, sourceWindow, videoGC);
    if (sig->sync_ok)
    {
      badsync = 0;
      break;
    }
  }
    
  if (badsync)
  {
    printf("no video connected to input 0\n");
    exit(1);
  }
    
  sourcewidth = sig->w;
  tmp = (sig->w * 3) / 4;
  sourceheight = tmp;
  tmp = sig->h - tmp;
  sourcestart = tmp + 1;

  workerId = XtAppAddWorkProc(app_context, worker, NULL);
  return(True);
}


void main(argc, argv)
     int argc;
     char **argv;
{
  scompData.useSharedMemory = True;
  QFactor = 100;
  /*
   * Install signal handler for kill and crahses.
   */
  signal (SIGINT, exit_func);
  signal (SIGSEGV, exit_func);
  signal (SIGBUS, exit_func);

  XtSetLanguageProc((XtAppContext)NULL, (XtLanguageProc)NULL, (XtPointer)NULL);
  sourcePopup = XtVaAppInitialize(&app_context,
				  "scomp",
				  options, XtNumber(options),
				  &argc, argv,
				  fallbackResources,
				  NULL);

  XtGetApplicationResources(sourcePopup, &scompData,
			    resources, XtNumber(resources),
			    NULL, 0);


  if(scompData.help == True)
  {
    /*
     * Print a help screen.
     */
    printUsage(argv[0]);
  }

  decompressionPopup = XtCreateWidget("DecompressShell",
				      topLevelShellWidgetClass, 
				      sourcePopup,
				      NULL, 0);
    
  sliderPopup = XtVaCreateWidget("QFactor",
				 topLevelShellWidgetClass, 
				 sourcePopup,
				 XtNwidth, videowidth,
				 XtNheight, 50,
				 NULL);

  form = XtVaCreateManagedWidget("QFactorAdjust", 
				 xmFormWidgetClass,
				 sliderPopup,
				 XtNborderWidth, 0,
				 XtNwidth, videowidth,
				 NULL);
    
  swidget = XtVaCreateManagedWidget("src", 
				    tcWindowWidgetClass,
				    sourcePopup,
				    XtNborderWidth, 0,
				    XtNwidth, videowidth,
				    XtNheight, videoheight,
				    NULL);

  dwidget = XtVaCreateManagedWidget("dest",
				    tcWindowWidgetClass,
				    decompressionPopup,
				    XtNborderWidth, 0,
				    XtNwidth, videowidth,
				    XtNheight, videoheight,
				    NULL);

  slwidget = XtVaCreateManagedWidget("qfactor", 
				     xmScaleWidgetClass,
				     form, 
				     XmNx, 20,
				     XmNwidth, 255,
				     XmNorientation, XmHORIZONTAL,
				     XmNprocessingDirection, XmMAX_ON_RIGHT,
				     XmNhighlightOnEnter, True,
				     XmNscaleWidth, 250,
				     XmNmaximum, 1000,
				     XmNminimum, 25,
				     XmNvalue, 100,
				     XmNshowValue, True,
				     XmNborderWidth, 0,
				     XmNtitleString, XmStringCreate("Qfactor", 
								    XmSTRING_DEFAULT_CHARSET),
				     NULL);

  XtAddCallback(slwidget, XmNdragCallback, dragcall, 0);
  XtAddEventHandler(swidget, ButtonPressMask, False, buttonp, NULL);
  XtAddEventHandler(dwidget, ButtonPressMask, False, buttonp, NULL);
  XtAddEventHandler(swidget, StructureNotifyMask, False, configit, NULL);
  XtAddEventHandler(swidget, ExposureMask, False, exposit, NULL);

  display = XtDisplay(sourcePopup);
  deleteWindowAtom = XmInternAtom(display, "WM_DELETE_WINDOW", False);

  XmAddWMProtocols(sourcePopup, &deleteWindowAtom, 1);
  XmAddWMProtocols(decompressionPopup, &deleteWindowAtom, 1);
  XmAddWMProtocols(sliderPopup, &deleteWindowAtom, 1);
  
  XmAddWMProtocolCallback(sourcePopup, deleteWindowAtom,
			  (XtCallbackProc)WMDeleteWindowCallback,
			  (XtPointer)NULL);
  XmAddWMProtocolCallback(decompressionPopup, deleteWindowAtom,
			  (XtCallbackProc)WMDeleteWindowCallback,
			  (XtPointer)NULL);
  XmAddWMProtocolCallback(sliderPopup, deleteWindowAtom,
			  (XtCallbackProc)WMDeleteWindowCallback,
			  (XtPointer)NULL);

  /*
   * The rest of the setup will occur after the shell widgets have been
   * realized.
   */
  XtAppAddWorkProc(app_context,
		   PostRealizeWorkProc,
		   (XtPointer)NULL);
  XtRealizeWidget(sourcePopup);
  XtAppMainLoop(app_context);
} /* end program */
