/* $Header: /plx/source/Xclients/generic/RCS/govLogo2.c,v 1.4 1995/11/01 07:59:28 drapeau Exp $ */

/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */

/* $Log: govLogo2.c,v $
 * Revision 1.4  1995/11/01  07:59:28  drapeau
 * Fixed initialization code to work with Sun CC.
 *
 * Revision 1.3  1995/10/14  01:59:19  lovvik
 * Removed a definition that includes code to paint tags in an
 * area before painting the pixmap.
 *
 * Revision 1.2  1995/10/14  01:48:16  lovvik
 * Added code to handle the MapNotify event.
 *
 * Revision 1.1  1995/10/13  00:47:54  lovvik
 * Initial revision
 * */

static char govLogo2Rcsid[] = "$Header: /plx/source/Xclients/generic/RCS/govLogo2.c,v 1.4 1995/11/01 07:59:28 drapeau Exp $";



#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <xpm.h>
#include <XPlxExt.h>
#include <XPlxProtos.h>
#include <icon/.plxlogo>

#define VIDEO 1
#define GRAPHICS 2
#define MASK_OVERSPRAY 10


typedef struct Pict
{
  int X;
  int Y;
  int Width;
  int Height;
  int PrevX;
  int PrevY;
} Pict;

typedef struct VideoWindow
{
  int Width;
  int Height;
} VideoWindow;



plx_IO *XPlxQueryConfig();
plx_signal *XPlxQueryVideo();
void GetNextPictPosition(VideoWindow, Pict *, int);

static    int firstTime = 1;
static firstExpose = 1;
static  Pixmap pixmap;
static  Pixmap positivePixmapMask;
static  Pixmap negativePixmapMask;
static  Pixmap eraseMask;
static  XpmAttributes pixmapAttributes;

plx_IO *plxio = NULL;
plx_signal *plxsignal = NULL;

static Arg args[25];
static unsigned int n;

int videowidth = 640;
int videoheight = 480;

GetVideoSignal(Display *display, Window win, GC gc, int channel)
{
  /*
   * Query the hardware for configuration parameters.
   */
  plxio = XPlxQueryConfig(display, win, gc);

  /*
   * Ask for input on channel 0, with the following characteristics:
   *   Standard = NTSC
   *   Format   = Composite
   *   Instance = 0
   */
  XPlxVideoInputSelect(display, win, gc,
		       channel,		
		       PLX_NTSC,		
		       PLX_COMP,		
		       0);		
    
  /*
   * Wait until we get sync. 
   */
  plxsignal = XPlxQueryVideo(display, win, gc);
  while ( !plxsignal->sync_ok )			
  {
    printf(" Sync Absent - Hook up an input source\n");
    XPlxSleep(500000);
    plxsignal = XPlxQueryVideo(display, win, gc);
  }
  videowidth = plxsignal->w;
  videoheight = plxsignal->h - plxsignal->b;
}


DrawPict(Display *display, Window win, GC gcForeground, Pict *pict)
{
  /* Draw the pixmap. */
  XSetClipOrigin(display, gcForeground,
		 pict->X, pict->Y);

#ifdef PAINT_TAGS_FIRST_HACK
  /*
   * This hack is necessary on servers that do not use the GC tag
   * passed to XCopyArea() to paint the pixmap.  This FillRectangle()
   * simply paints tags BEFORE the pixmap is painted, so the pixmap
   * will not be erased by the video.
   */
  XFillRectangle(display, win, gcForeground, 
		 pict->X, pict->Y,
		 pict->Width, pict->Height);
#endif
    
  XCopyArea(display, pixmap, win, gcForeground,
	    0, 0, pict->Width, pict->Height,
	    pict->X, pict->Y);
}

CreateEraseClipmask(Display *display, Window win, GC gcBackground,
		    Pict *pict)
{
  GC maskGC;
  XGCValues values;
  
  values.foreground = 0x00;

  maskGC = XCreateGC(display, eraseMask, GCForeground, &values);

  /* Create a mask that will delete the previous image.  Do this by using the erase
   * clip mask to create a mask that represents only the parts of the pixmap that need
   * to be erased.
   */

  /*
   * First, fill the mask with 0 to clear it.
   */
  XFillRectangle(display, eraseMask, maskGC,
		 0, 0,
		 pixmapAttributes.width + MASK_OVERSPRAY,
		 pixmapAttributes.height + MASK_OVERSPRAY);

  /*
   * Now, put a copy of the erase mask into the new mask.
   */
  XSetFunction(display, maskGC, GXcopy);
  
  XCopyArea(display, positivePixmapMask, eraseMask, maskGC,
	    0, 0, pixmapAttributes.width, pixmapAttributes.height,
	    (MASK_OVERSPRAY/2)-(pict->X-pict->PrevX), 
	    (MASK_OVERSPRAY/2)-(pict->Y-pict->PrevY));

  /*
   * Now, and the new mask with with the negative of the pixmap mask.
   */
  XSetFunction(display, maskGC, GXand);

  XCopyArea(display, negativePixmapMask, eraseMask, maskGC,
	    0, 0, pixmapAttributes.width, pixmapAttributes.height,
	    (MASK_OVERSPRAY/2), 
	    (MASK_OVERSPRAY/2));
}


MovePict(Display *display, Window win, GC gcForeground, GC gcBackground, int borderWidth, Pict *pict)
{
  DrawPict(display, win, gcForeground, pict);
  
  /* Draw the border in the background color.  This effectively erases the previous Pict */
  /* while avoiding the flashing that you would get if you simply drew the Pict in the   */
  /* background color, and then redrew the Pict in the foreground color.                 */

  CreateEraseClipmask(display, win, gcBackground, pict);
  XSetClipMask(display, gcBackground, eraseMask);   
  XSetClipOrigin(display, gcBackground,             /* position the clipping mask */
		 pict->X - (MASK_OVERSPRAY/2), pict->Y - (MASK_OVERSPRAY/2));
  XFillRectangle(display, win, gcBackground,        /* draw a rectangle */
		 pict->X-(MASK_OVERSPRAY/2), pict->Y-(MASK_OVERSPRAY/2),
		 pict->Width + MASK_OVERSPRAY, pict->Height + MASK_OVERSPRAY);
}


int PictInExposeArea(Pict *pict, XExposeEvent *expose)
{
  /*
   * This function determines if the pixmap was in the exposed region.  This
   * lets the expose event handler know whether or not to re-draw the pixmap.
   */
  Region exposeRegion;
  XPoint exposeRectangle[4];

  exposeRectangle[0].x = expose->x;
  exposeRectangle[0].y = expose->y;
  exposeRectangle[1].x = expose->x + expose->width;
  exposeRectangle[1].y = expose->y;
  exposeRectangle[2].x = expose->x + expose->width;
  exposeRectangle[2].y = expose->y + expose->height;
  exposeRectangle[3].x = expose->x;
  exposeRectangle[3].y = expose->y + expose->height;
  
  exposeRegion = XPolygonRegion(exposeRectangle,
				4,
				WindingRule);
  
  /*
   * If the pixmap is not in the expose region, return False.  Otherwise,
   * return True.
   */
  if(XRectInRegion(exposeRegion,
		   pict->X, pict->Y,
		   pict->Width, pict->Height) == RectangleOut)
  {
    return(False);
  }
  else
  {
    return(True);
  }
}

void main(argc, argv)
int argc;
char **argv;
{
  Window win;
  Display *display;
  int scr, mask;
  XGCValues values;
  GC gcBackground;
  GC gcForeground;
  GC gcErase;
  GC negativePixmapMaskGC;

  int status;
  
  XEvent event;
  XSetWindowAttributes attr;
  XVisualInfo vinfo;
  int depth = 32;
  XVisualInfo*		visualInfo;
  Pict pict;
  VideoWindow videowindow;
  unsigned long ColorValue = 0x0000ff;
  int borderWidth = 5;
  
  videowindow.Width = videowidth;
  videowindow.Height = videoheight;
  pict.X = (videowidth / 2);					    /* Set up the pict structure */
  pict.Y = (videoheight / 2);
  pict.PrevX = pict.X;
  pict.PrevY = pict.Y;

  
  
  display = XOpenDisplay(getenv("DISPLAY"));
  scr = (int) XDefaultScreen(display);

  /*
   * Get a proper video for using 24-bit video in a window.
   */
  visualInfo = XPlxGetVideoVisual(scr, display);

  depth = visualInfo->depth;

  /*
   * WARNING!!! Must set up the colormap, the background_pixel, and
   * the border_pixel to make it work when the window depth is different
   * from the root depth.
   */

  mask = CWBackPixel | CWColormap | CWBorderPixel;	
  attr.colormap = XCreateColormap(display, XRootWindow(display, scr),
				  visualInfo->visual, AllocNone);
  attr.background_pixel = BlackPixel(display, DefaultScreen(display));
  attr.border_pixel = WhitePixel(display, DefaultScreen(display));

  /*
   * Create the drawable for the XPlx calls.
   */
  win = XCreateWindow(display, RootWindow(display, scr),
		      0, 0, videowidth, videoheight, 1, depth,
		      InputOutput, visualInfo->visual, mask, &attr);

  /*
   * Catch exposure events, button press events, and window parameter change events.
   */
  XSelectInput(display, win, ExposureMask |
	       ButtonPressMask |           
	       StructureNotifyMask);       

  values.line_width = borderWidth;

  gcBackground = XCreateGC(display, win, GCLineWidth, &values);

  /*
   * The background gc will be used for video.
   */
  XPlxVideoTag(display, win, gcBackground, PLX_VIDEO);

  /*
   * The foreground gc will be used for 24-bit graphics.
   */
  gcForeground = XCreateGC(display, win, 0, NULL);
  XPlxVideoTag(display, win, gcForeground, PLX_GRAPHICS_24);

  /*
   * Set up the video and verify that there is synch.
   */
  GetVideoSignal(display, win, gcBackground, PLX_INPUT_0);

  /*
   * Set up the gc that will be used in erasing the pixmap.
   */

  /*
   * NOTE:  Currently, there is no way to get 2 gc's tagged
   * with the same video tag.  If you create a separate gc
   * and tag it for PLX_VIDEO, it will get a different tag
   * value than the one assigned to the background gc.  The
   * result is that the erase gc does not erase - it paints
   * black instead of video.  The hack used here is to use
   * the same gc for painting the background and erasing.  This
   * may not always be desirable because the erase gc is going
   * to use a clip mask while the background gc does not need
   * one.
   */
  gcErase = gcBackground;

  /* 
   * NOTE:  XSetPlaneMask does not work correctly.  The idea behind using it here
   * is that when the erase gc is used, we do not want to show the color - we
   * just want to paint the tags.
   */
  XSetPlaneMask(display, gcErase, 0); 

  /* set up the pixmap attributes */
  pixmapAttributes.depth = visualInfo->depth;
  pixmapAttributes.colormap = attr.colormap;
  pixmapAttributes.visual = visualInfo->visual;
  pixmapAttributes.valuemask = XpmDepth | XpmColormap | XpmVisual;

  status = XpmCreatePixmapFromData(display,
				   win,
				   plxLogoData, &pixmap, &positivePixmapMask,
				   &pixmapAttributes);
  if(status != XpmSuccess)
  {
    printf("Could not create the pixmap.  Exiting.\n");
    exit(0);
  }

  pict.Width = pixmapAttributes.width;
  pict.Height = pixmapAttributes.height;

  XSetClipMask(display, gcForeground, positivePixmapMask); 

  negativePixmapMask = XCreatePixmap(display, win,
				     pixmapAttributes.width, pixmapAttributes.height,
				     1);					    /* Depth of 1. */

  eraseMask = XCreatePixmap(display, win,
			    pixmapAttributes.width + MASK_OVERSPRAY, pixmapAttributes.height + MASK_OVERSPRAY,
			    1);					    /* Depth of 1. */
  values.foreground = 0x01;
  values.function = GXcopyInverted;
  negativePixmapMaskGC = XCreateGC(display, negativePixmapMask, GCForeground, &values);
  XFillRectangle(display, negativePixmapMask, negativePixmapMaskGC,
		 0, 0,
		 pixmapAttributes.width,
		 pixmapAttributes.height);
  XSetFunction(display, negativePixmapMaskGC, GXcopyInverted);
  
  XCopyArea(display, positivePixmapMask, negativePixmapMask, negativePixmapMaskGC,
	    0, 0, pixmapAttributes.width, pixmapAttributes.height,
	    0, 0);
  
  XResizeWindow(display, win, videowidth, videoheight);
  XMapWindow(display, win); 

  /*
   * Event loop.
   */
  while (1)
  {
    static int firstExposeEvent = True;

    if(XPending(display))
    {
      XNextEvent(display, &event);
      switch(event.type)
      {
	case MapNotify:
        {
	  /*
	   * When the expose event gets processed, we want the digitizer to
	   * be restarted.
	   */
	  firstExposeEvent = True;
	  break;
	}
	  
        case Expose:
        {
	  XExposeEvent *expose = &event.xexpose;

	  /*
	   * If this is the first expose event, or the window has been resized, we
	   * must start the video in the window and draw the pixmap.
	   */
	  if(firstExposeEvent)
	  {
	    XPlxVideoSqueezeLive(display, win, gcBackground, 0, plxsignal->b, 
				 videowidth, videoheight,
				 0, 0,
				 videowindow.Width, videowindow.Height);

	    DrawPict(display, win, gcForeground, &pict);
	    firstExposeEvent = False;
	  }

	  /*
	   * Generally, an expose event means that you only need to repaint the
	   * affected area.  Use a gc tagged for video to repaint the video
	   * areas of the window.  If the pixmap was occluded, redraw the
	   * pixmap with a gc tagged for graphics.
	   */
	  else
	  {
	    XSetClipMask(display, gcErase, None);
	    XFillRectangle(display, win, gcBackground,
			   expose->x, expose->y,
			   expose->width, expose->height);
	    XSetClipMask(display, gcErase, eraseMask);
	    if(PictInExposeArea(&pict, expose) == True)
	    {
	      DrawPict(display, win, gcForeground, &pict);
	    }
	  }
	  break;
	}


	case ButtonPress:
	{
	  switch(event.xbutton.button)
	  {
	    case 1:
	    {
	      /*
	       * Stop video if the first mouse button was pressed.
	       */
	      XPlxVideoStop(display, win, gcBackground);
	      break;
	    }

	    case 2:
	    {
	      /*
	       * Restart video if the second mouse button was pressed.
	       */
	      XPlxVideoSqueezeLive(display, win, gcBackground, 0, plxsignal->b, 
				   videowidth, videoheight,
				   0, 0,
				   videowindow.Width, videowindow.Height);
	      DrawPict(display, win, gcForeground, &pict);
	      break;
	    }  

	    case 3:
	    {
	      /*
	       * Stop video and exit if the third mouse button was pressed.
	       */
	      XPlxVideoStop(display, win, gcBackground);
	      XFreeGC(display, gcForeground);
	      XFreeGC(display, gcBackground);
	      XCloseDisplay(display);
	      exit(1);
	    }
	  }
	  break;
	}
	/*
	 * On ConfigureNotify events, we must determine if the window was resized or
	 * just moved.  If the window was moved, we don't worry about it (the server
	 * will copy the video and graphics).  If the window was resized, we must resize
	 * our video and redraw our graphics.  We accomplish this by setting the
	 * firstExposeEvent variable to True.  (An expose event will automatically be
	 * generated by the resize request.
	 */
	  case ConfigureNotify:
	{
	  int resize = False;

	  if(event.xconfigure.width != videowindow.Width)
	  {
	    firstExposeEvent = True;
	    if(event.xconfigure.width > videowidth)
	    {
	      videowindow.Width = videowidth;
	      resize = True;
	    }
	    else
	    {
	      videowindow.Width = event.xconfigure.width;
	    }
	  }
	  if(event.xconfigure.height != videowindow.Height)
	  {
	    firstExposeEvent = True;
	    if(event.xconfigure.height > videoheight)
	    {
	      videowindow.Height = videoheight;
	      resize = True;
	    }
	    else
	    {
	      videowindow.Height = event.xconfigure.height;
	    }
	  }
	  if(resize == True)
	  {
	    XResizeWindow(display, win, videowindow.Width, videowindow.Height);
	  }
	  break;
	} /* end of case ConfigureNotify */
      }  /* end of switch */
    } /* end of if */
    else
    {
      /*
       * Since no events are waiting to be processed, we can move the pixmap.
       */
      GetNextPictPosition(videowindow, &pict, 4);
      MovePict(display, win, gcForeground, gcErase, borderWidth, &pict);
      XFlush(display); 
    
      XPlxSleep(300000); 
    }
  }   /* end of while */
} /* end main */


void GetNextPictPosition(VideoWindow videowindow, Pict *pict, int delta)
{
  static dX = 0;
  static dY = 0;

  if(!dX && !dY)  /* if the pict has no defined direction */
  {
    dX = delta;
    dY = delta;       /* start the pict in some direction */
  }
  pict->PrevX = pict->X;
  pict->PrevY = pict->Y;

  pict->X = pict->X + dX;  /* move the pict in the x direction */
  pict->Y = pict->Y + dY;  /* move the pict in the y direction */

  if(0 > (pict->X)) /* pict went too far left? */
  {
    dX = delta;  /* yes, make it bounce */
    pict->X = pict->X + dX; 
  }

  if(videowindow.Width < (pict->X + pict->Width)) /* pict went too far right? */
  {
    dX = -delta; /* yes, make it bounce */
    pict->X = pict->X + dX;
  }

  if(0 > (pict->Y))  /* pict went too far up? */
  {
    dY = delta; /* yes, make it bounce */
    pict->Y = pict->Y + dY;
  }

  if(videowindow.Height < (pict->Y + pict->Height))  /* pict went too far down? */
  {
    dY = -delta; /* yes, make it bounce */
    pict->Y = pict->Y + dY;
  }
}  /* end of GetNextPictPosition */


