/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */
/* $Log: govTest.c,v $
 * Revision 1.4  1995/11/01  08:00:12  drapeau
 * Fixed initialization code to work with Sun CC.
 *
 * Revision 1.3  1995/10/14  02:00:05  drapeau
 * Cleaned up the code for better readability.  Also, removed extraneous
 * calls to XPlxVideoTag(), using the call more sparingly to make it
 * easier for application programmers to understand.
 *
 * Revision 1.2  1995/10/13  22:55:37  drapeau
 * Cleaned up the code for better readability.
 *
 * Revision 1.1  1995/10/13  00:46:33  lovvik
 * Initial revision
 * */
/* $Header: /plx/source/Xclients/generic/RCS/govTest.c,v 1.4 1995/11/01 08:00:12 drapeau Exp $ */
static char govTestRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/govTest.c,v 1.4 1995/11/01 08:00:12 drapeau Exp $";

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <XPlxExt.h>
#include <XPlxProtos.h>

static int	firstTime = 1;
static int	firstExpose = 1;
static int	outputOn = False;
plx_IO*		plxio = NULL;
plx_signal*	plxsignal = NULL;
static Arg	args[25];
static unsigned int n;
int		videowidth = 640;
int		videoheight = 480;

typedef struct Ball
{
  int Size;
  int X;
  int Y;
} Ball;

typedef struct VideoWindow
{
  int Width;
  int Height;
} VideoWindow;

void printhelpscreen(void)
{
  printf("===================================================\n");
  printf("|             govTest - help screen               |\n");
  printf("|              Parallax Graphics                  |\n");
  printf("---------------------------------------------------\n");
  printf("|                                                 |\n");
  printf("| usage:  videoball [<-option> <-option> . . .]   |\n");
  printf("|                                                 |\n");
  printf("| options:                                        |\n");
  printf("|   h - help (this message)                       |\n");
  printf("|   b - set the background color (use a color     |\n");
  printf("|         name or 'video')                        |\n");
  printf("|   f - set the foreground color (use a color     |\n");
  printf("|         name or 'video')                        |\n");
  printf("|   s - set the size of the ball (in pixels)      |\n");
  printf("|   d - set the delta (amount of movement in      |\n");
  printf("|         pixels)                                 |\n");
  printf("|   t - place text over video in foreground       |\n");
  printf("|         color                                   |\n");
  printf("|                                                 |\n");
  printf("| example:                                        |\n");
  printf("|   videoball -bvideo -fcyan -s100 -t             |\n");
  printf("|                                                 |\n");
  printf("|                                                 |\n");
  printf("===================================================\n");
  return;
} /* end of printhelpscreen */



GetVideoSignal(Display *display, Window win, GC gc)
{
  XPlxVideoTag(display, win, gc, PLX_VIDEO);			    /* "PLX_VIDEO" means to explicitly paint video tags into... */
								    /* ...each pixel affected by an X operation. */
								    /* This has the effect of forcing video to show through,... */
								    /* ...even if graphics had previously been drawn in this... */
								    /* ...area of the window. */
  plxio = XPlxQueryConfig(display, win, gc);			    /* Query the hardware for configuration parameters */
  XPlxVideoInputSelect(display, win, gc,			    /* Ask for input on channel 0, with the following... */
		       PLX_INPUT_0,				    /* ...characteristics: */
		       PLX_NTSC,				    /* Standard = NTSC */
		       PLX_COMP,				    /* Format = Composite */
		       PLX_RGB24);				    /* Type = RGB24 */
    
  plxsignal = XPlxQueryVideo(display, win, gc);			    /* Query the Video to see if sync is there		*/
  while ( !plxsignal->sync_ok )					    /* Wait until a signal is input into the board	*/
  {
    printf(" Sync Absent - Hook up an input source\n");
    XPlxSleep(500000);
    plxsignal = XPlxQueryVideo(display, win, gc);		    /*  Query the Video again to check sync             */
  }
  videowidth = plxsignal->w;
  videoheight = plxsignal->h - plxsignal->b;
}



DrawBall(Display *display, Window win, GC gcForeground, Ball *ball)
{
  /* Draw the ball */
  XFillArc(display, win, gcForeground,                                  /* use the foreground color */
	   ball->X, ball->Y,						/* upperleft x, upperleft y */
	   ball->Size, ball->Size,                                      /* width, height            */
	   0,                                                           /* start angle              */
	   360*64);                                                     /* fill angle (degrees*64)  */
}



int BallInExposeArea(Ball *ball, XExposeEvent *expose)
{
  Region exposeRegion;
  XPoint exposeRectangle[5];

  exposeRectangle[0].x = expose->x;
  exposeRectangle[0].y = expose->y;
  exposeRectangle[1].x = expose->x + expose->width;
  exposeRectangle[1].y = expose->y;
  exposeRectangle[2].x = expose->x + expose->width;
  exposeRectangle[2].y = expose->y + expose->height;
  exposeRectangle[3].x = expose->x;
  exposeRectangle[3].y = expose->y + expose->height;
  exposeRectangle[4].x = expose->x;
  exposeRectangle[4].y = expose->y;
  
  exposeRegion = XPolygonRegion(exposeRectangle,
				4,
				WindingRule);
  
  printf("XRectInRegion returned %d\n", XRectInRegion(exposeRegion,
						      ball->X, ball->Y,
						      ball->Size, ball->Size));
  if(XRectInRegion(exposeRegion, 					/* If the ball is not in the expose region . . . */
		   ball->X, ball->Y,
		   ball->Size, ball->Size) == RectangleOut)
  {
    return(False);							/* . . . return false. */
  }
  else									/* Otherwise . . . */
  {
    return(True);							/* . . . return true */
  }
} /* end function BallInExposeArea */



void main(int argc, char** argv)
{
  Window		childWindow;
  Window		win;
  Display*		display;
  int			scr;
  int			mask;
  int			index;
  XGCValues		values;
  GC			gcBackground;
  GC			gcForeground;
  GC			outputGC;
  XEvent		event;
  XSetWindowAttributes	attr;
  XVisualInfo		vinfo;
  int			depth;
  XVisualInfo*		visualInfo;
  XExposeEvent*		expose;
  Ball			ball;
  VideoWindow		videowindow;
  unsigned long		ColorValue = 0x0000ff;			    /* Set 24-bit color value to Red (pixel order is BGR) */
  int			resize;
  int			windowXPosition;
  int			windowYPosition;
  
  ball.Size = 80;						    /* Ball, size:  80, centered in video window */
  ball.X = videowidth / 2;
  ball.Y = videoheight / 2;
  videowindow.Width = videowidth;
  videowindow.Height = videoheight;
  if(argc > 1)
  {
    if(strcmp(argv[1], "-output") == 0)
    {
      outputOn = True;
    }
  }
  display = XOpenDisplay(getenv("DISPLAY"));			    /* Connect your client program with the X server	*/
  scr = (int) XDefaultScreen(display);				    /* Get a default screen				*/
  visualInfo = XPlxGetVideoVisual(scr, display);		    /* Get a proper Visual for using video in a window */
  depth = visualInfo->depth;
  mask = CWBackPixel |						    /* WARNING!!! Must set up the colormap, the... */
    CWColormap |						    /* ...background_pixel and the border_pixel to make it... */
      CWBorderPixel;						    /* ...work when the window depth is diff. from root depth.  */
  attr.colormap = XCreateColormap(display,
				  XRootWindow(display, scr),
				  visualInfo->visual,
				  AllocNone);
  attr.background_pixel = BlackPixel(display,
				     DefaultScreen(display));
  attr.border_pixel = WhitePixel(display, DefaultScreen(display));
  win = XCreateWindow(display, RootWindow(display, scr),	    /* Create the drawable for the XPlx calls */
		      0, 0, videowidth, videoheight,
		      1, depth, InputOutput,
		      visualInfo->visual, mask, &attr);
  XSelectInput(display, win,					    /* Catch exposure events . . .*/
	       ExposureMask |
	       ButtonPressMask |				    /* . . . and button press events . . . */
	       StructureNotifyMask);				    /* . . . and window parameter change events */
  if(outputOn)
  {
    outputGC = XCreateGC(display, win, 0, NULL); 
    XPlxVideoOutputSelect(display, win, outputGC,
			  PLX_OUTPUT_0,
			  PLX_NTSC,
			  PLX_COMP,
			  PLX_RGB24);
  }
  gcBackground = XCreateGC(display, win, 0, NULL);		    /* Create a GC for drawing video in the background */
  values.foreground = ColorValue;
  GetVideoSignal(display, win, gcBackground);
  gcForeground = XCreateGC(display, win, GCForeground, &values);    /* Create a GC for drawing 24-bit graphics */
  XPlxVideoTag(display, win, gcForeground, PLX_GRAPHICS_24);	    /* Note: when doing graphics and video in the same... */
								    /* ...window, any GC's used for drawing graphics *must*... */
								    /* ...be explicitly tagged for graphics, as done here. */
  XResizeWindow(display, win, videowidth, videoheight);
  XMapWindow(display, win); 
  while (1)							    /* Continuously show live video with the standard... */
  {								    /* ...signal width and height */
    static int firstExposeEvent = True;
    
    XNextEvent(display, &event);
    switch(event.type)
    {
     case MapNotify:						    /* This flag tells the Expose case handler to restart... */
      firstExposeEvent = True;					    /* ...the digitizer, since unmapping the window caused... */
      break;							    /* ...the digitizer to stop. */
     case Expose:
      printf("Expose event\n");
      expose = &event.xexpose;
      if(True == firstExposeEvent)				    /* If this is the 1st expose event, or window has been... */
      {								    /* ...resized, start video and draw graphics. */
	printf("This is the first Expose event.\n");
	XPlxVideoSqueezeLive(display, win, gcBackground,
			     0, plxsignal->b,
			     videowidth, videoheight,
			     0, 0,
			     videowindow.Width, videowindow.Height);
	if(outputOn)
	{
	  XPlxVideoScaleOutput(display, win, outputGC,
			       0, 0,
			       videowidth, videoheight, 
			       0, 43,
			       videowidth, videoheight);
	}
	  
	DrawBall(display, win, gcForeground, &ball);
	firstExposeEvent = False;
      }
      else							    /* Generally, an expose event means that you only need... */
								    /* ...to repaint the affected area.  Use a gc tagged... */
								    /* ...for video for the video areas of the window, and... */
								    /* ...if the graphic image was occluded, redraw that... */
								    /* ...graphic with a gc tagged for graphics. */
      {
	XFillRectangle(display, win, gcBackground,		    /* Draw only the exposed area with the video gc; each... */
		       expose->x, expose->y,			    /* ...pixel affected by the XFillRectangle() will be... */
		       expose->width, expose->height);		    /* ...painted with a video tag, causing video to show. */
	if(BallInExposeArea(&ball, expose) == True)		    /* If the ball was in the exposed area, draw it too. */
	{
	  printf("drawing the ball\n");
	  DrawBall(display, win, gcForeground, &ball);
	}
      }
      break;
     case ButtonPress:
      switch(event.xbutton.button)
      {
       case 1:
	XPlxVideoStop(display, win, gcBackground);
	break;
       case 2:
	XPlxVideoTag(display, win, gcBackground, PLX_VIDEO_OVR);    /* Reset tag so that video ops aren't destructive */
	XPlxVideoSqueezeLive(display, win, gcBackground,	    /* Turn on video (the tag for this GC should be set to... */
			     0, plxsignal->b,			    /* ..."PLX_VIDEO_OVR", which causes the video digitizer... */
			     videowidth, videoheight,		    /* ...to turn on but not overwrite any existing graphics... */
			     0, 0,				    /* ...pixels.  This allows graphics-over-video to be... */
			     videowindow.Width, videowindow.Height); /* ...persistent. */
	XPlxVideoTag(display, win, gcBackground, PLX_VIDEO);	    /* Set GC's tag to forcibly draw video into affected pixels */
	break;
       case 3:
	XPlxVideoStop(display, win, gcBackground);		    /* Stop video in the window and digitization of the video */
	if(outputOn)
	{
	  XPlxVideoOutputSelect(display, win, outputGC,		    /* Stop the output. */
				PLX_IO_OFF,
				PLX_NTSC,
				PLX_COMP,
				PLX_RGB24); 
	}
	XFreeGC(display, gcForeground);
	XFreeGC(display, gcBackground);
	if(outputOn)
	{
	  XFreeGC(display, outputGC); 
	}
	XCloseDisplay(display);
	exit(1);
      }	/* end switch event.xbutton.button */
      break;
     case ConfigureNotify:
	
      printf("ConfigureNotify event\n");
      resize = False;
      windowXPosition = event.xconfigure.x;
      windowYPosition = event.xconfigure.y;
      if(event.xconfigure.width != videowindow.Width)		    /* Is the width of the window larger than the video signal? */
      {
	firstExposeEvent = True;				    /* If window resized, do a XPlxVideoSqueezeLive */
	if(event.xconfigure.width > videowidth)
	{
	  videowindow.Width = videowidth;
	  resize = True;					    /* Yes, we must resize the window */
	}
	else							    /* No, no need to resize the window */
	{
	  videowindow.Width = event.xconfigure.width;
	}
      }
      if(event.xconfigure.height != videowindow.Height)		    /* Is height of the window larger than the video signal? */
      {
	firstExposeEvent = True;				    /* If window resized, do a XPlxVideoSqueezeLive */
	if(event.xconfigure.height > videoheight)
	{
	  videowindow.Height = videoheight;
	  resize = True;					    /* Yes, we must resize the window */
	}
	else							    /* No, no need to resize the window */
	{
	  videowindow.Height = event.xconfigure.height;
	}
      }
      if(resize == True)
      {
	XResizeWindow(display, win, videowindow.Width, videowindow.Height);
      }
      else
      {
	if(outputOn)
	{
	  XPlxVideoScaleOutput(display, win, outputGC,
			       0, 0,
			       event.xconfigure.width, event.xconfigure.height, 
			       0, 43,
			       event.xconfigure.width, event.xconfigure.height);	
	}
      }
      break;							    /* end case ConfigureNotify */
    } /* end of switch */
  } /* end of while */
} /* end main */
