/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */

/* $Log: rtvRecord.c,v $
 * Revision 1.5  1996/01/31  15:54:22  lovvik
 *   Changed the error detection to look for negative
 *   values only.
 *
 * Revision 1.4  1996/01/30  21:18:11  lovvik
 *   Changed the behavior so that a PAL movie will be full
 *   size if recorded on a PowerVideo board.  If an XVideo-VIO
 *   board is used, the movie will be 8 pixels less than
 *   PAL size in the x and y direction.
 *
 * Revision 1.3  1996/01/18  18:42:33  lovvik
 *   Fixed a frame-rate problem that occurred when recording
 *   PAL movies.
 *
 * Revision 1.2  1996/01/11  16:58:05  lovvik
 *   Added support for the XVideoVIO card.  This includes
 *   the following changes:
 *     1.  If the video card is an XVideoVIO, set the first
 * 	input to PAL.
 *     2.  If the input signal format is PAL or SECAM,
 * 	decrease the width and height by 8 pixels.
 *
 * Revision 1.1  1995/11/29  00:45:39  lovvik
 *   Initial revision
 * */

/* $Header: /plx/source/Xclients/sun4.ow3/lovvik/src/client/xlib/example/RCS/rtvRecord.c,v 1.5 1996/01/31 15:54:22 lovvik Exp $ */

static char rtvRecordRcsid[] = "$Header: /plx/source/Xclients/sun4.ow3/lovvik/src/client/xlib/example/RCS/rtvRecord.c,v 1.5 1996/01/31 15:54:22 lovvik Exp $";

#include <stdio.h>
#include <signal.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Xatom.h>
#include <XPlxExt.h>
#include <XPlxProtos.h>
#include <rtv_ops.h>

#ifndef MAXHOSTNAMELEN
#define MAXHOSTNAMELEN 	256
#endif

void Control_C_Handler(int sig);
Boolean XVideoCard(Display *display, Window window, GC videoGC);

Widget			top, video_widget;
plx_IO*			plxio = NULL;
plx_signal*		plxsignal = NULL;
int			videoWidth = 640;
int			videoHeight = 512;
int			frameRate = 30;
int			done;
int			inputFormat = PLX_COMP;
int			signalStandard;
static 			Atom deleteWindowAtom;
int			inputChannel = PLX_INPUT_0;

void main(argc, argv)
     int argc;
     char **argv;
{
  Window		window;
  Display*		display;
  int			scr;
  int			mask;
  GC			videoGC; 
  GC			rtvGC;
  XEvent		event;
  XSetWindowAttributes	attr;
  int			depth;
  XVisualInfo*		visualInfo;
  int			firstExpose = True;
  RTV_OBJECT		*handle;
  RTV_ATTRIBUTES	attributes;
  char                  hostname[MAXHOSTNAMELEN];

  done = False;

  if(argc != 2)
  {
    fprintf(stderr, "Must supply path and filename on the command line.\n");
    fprintf(stderr, "ex:  rtvRecord /home/movie\n");
    exit(0);
  }

  /*
   * Catch Control-C.  Note:  Do NOT leave the Rtv
   * server in a record state when the client exits.
   */
  signal(SIGINT, Control_C_Handler);

  /*
   * Connect your client program with the X server.
   */
  display = XOpenDisplay(getenv("DISPLAY"));

  /*
   * Get a default screen.
   */
  scr = (int) XDefaultScreen(display);

  /*
   * Get a proper visual for using 24-bit video in a window.
   */
  visualInfo = XPlxGetVideoVisual(scr, display);
  depth = visualInfo->depth;

  /*
   * If the video visual is a true color window, we must create
   * a color map for it.
   */
  if (TrueColor == visualInfo->class)
  {
    attr.colormap = XCreateColormap(display, XRootWindow(display, scr),
				    visualInfo->visual, AllocNone);
  }
  else
  {
    attr.colormap = XDefaultColormap(display, scr);
  }

  /*
   * WARNING!!! Must set up the colormap, the background pixel, and the
   * border pixel to make it work when the window depth is different from
   * the root depth.
   */
  mask = CWBackPixel | CWColormap | CWBorderPixel;
  attr.background_pixel = BlackPixel(display, DefaultScreen(display));
  attr.border_pixel = WhitePixel(display, DefaultScreen(display));

  /*
   * Create the drawable for the XPlx calls.
   */
  window = XCreateWindow(display, RootWindow(display, scr),
			 0, 0, videoWidth, videoHeight, 1, depth,
			 InputOutput, visualInfo->visual, mask, &attr);

  /*
   * Catch exposure events.
   */
  XSelectInput(display, window, ExposureMask);

  videoGC = XCreateGC(display, window, 0, 0);
  rtvGC = XCreateGC(display, window, 0, 0);
  XFlush(display);

  XPlxVideoTag(display, window, videoGC, PLX_VIDEO);

  /*
   * Connect to the Rtv server.
   */
  gethostname(hostname, MAXHOSTNAMELEN);
  handle = (RTV_OBJECT*)RtvConnect(hostname);
  if(handle == NULL)
  {
    fprintf(stderr, "Error!  Could not connect to the Rtv server.\n");
    exit(0);
  }

  if(XVideoCard(display, window, videoGC) == True)
  {
    /*
     * This is an XVideoVIO or an XVideoRGB card.  We must set the first
     * input to PAL mode.
     */
    XPlxVideoInputSelect(display, window, videoGC,
			 PLX_INPUT_0,
			 PLX_PAL,
			 PLX_COMP,
			 0);
    inputChannel = PLX_INPUT_1;
  }

  /*
   * Find a video signal connected to inputChannel.
   */
  plxio = XPlxQueryConfig(display, window, videoGC);
  do
  {
    XPlxVideoInputSelect(display, window, videoGC,
			 inputChannel,
			 PLX_NTSC,
			 inputFormat,
			 0);
    frameRate = 30;
    signalStandard = PLX_NTSC;
    XPlxSleep(500);  
    plxsignal = XPlxQueryVideo(display, window, videoGC);
    
    if(!plxsignal->sync_ok)
    {
      XPlxVideoInputSelect(display, window, videoGC,
			   inputChannel,
			   PLX_PAL,
			   inputFormat,
			   0);
      frameRate = 25;
      signalStandard = PLX_PAL;
      XPlxSleep(500);
      plxsignal = XPlxQueryVideo(display, window, videoGC);
    }

    if(!plxsignal->sync_ok)
    {
      XPlxVideoInputSelect(display, window, videoGC,
			   inputChannel,
			   PLX_SECAM,
			   inputFormat,
			   0);
      frameRate = 25;
      signalStandard = PLX_SECAM;
      XPlxSleep(500);
      plxsignal = XPlxQueryVideo(display, window, videoGC);
    }

    if(!plxsignal->sync_ok)
    {
      inputFormat++;
      if(inputFormat > PLX_RGB)
      {
	inputFormat = PLX_COMP;
      }
    }
  } while(!plxsignal->sync_ok);

  videoWidth = plxsignal->w;
  videoHeight = plxsignal->h - plxsignal->b;

  if(signalStandard != PLX_NTSC)
  {
    /*
     * Some configurations cannot record SECAM or PAL
     * standard inputs with complete reliability.  For
     * these configurations, we will reduce the width
     * and height of the video window by 8 pixels.
     */
    if(True == XVideoCard(display, window, videoGC))
    {
      videoWidth -= 8;
      videoHeight -= 8;
    }
  }
  
  /*
   * Indicate to the window server that this is an Rtv window.
   */
  XPlxSetRtvWindow(display, window, rtvGC);

  /*
   * Prepare the window for recording the movie.
   */
  XResizeWindow(display, window, videoWidth, videoHeight);

  /*
   * Set up the attributes for recording.
   */
  attributes.interlace = 1;
  attributes.qfactor = 60;
  attributes.fps = frameRate;
  attributes.rate = 16;
  RtvSetup(handle, SET_RTV_ATTRIBUTES, &attributes);
  
  /*
   * Open a record session.
   */
  RtvOpenStream(handle, argv[1]);
  
  RtvRequest(handle, INIT_RTV_RECORD);
  
  XMapWindow(display, window); 

  /*
   * Catch the Delete Window call from the window manager.
   */
  deleteWindowAtom = XInternAtom(display,
				 "WM_DELETE_WINDOW",
				 False);
  XSetWMProtocols(display, window, &deleteWindowAtom, 1);
  while (1)
  {
    if(XPending(display))
    {
      XNextEvent(display, &event);
      switch(event.type)
      {
	case Expose:
        {
	  /*
	   * Start the video on all expose events.
	   */
	  XPlxVideoSqueezeLive(display, window, videoGC,
			       0, plxsignal->b, plxsignal->w, plxsignal->h,
			       0, 0, videoWidth, videoHeight);
	  if(firstExpose == True)
	  {
	    /*
	     * Indicate to the window server that this is an Rtv window
	     * to be used for recording a movie.  XPlxRtvGetCImage() must
	     * be called AFTER the first expose event.
	     */
	    XPlxRtvGetCImage(display, window, rtvGC, 0, 0,
			     videoWidth, videoHeight, 
			     videoWidth, videoHeight);

	    /*
	     * In this simple example, we will have the
	     * first expose event trigger the record operation.
	     */
	    RtvRequest(handle, START_RTV_RECORD);
	    firstExpose = False;
	  }
	  break;
	}

	case ClientMessage:
	{
	  /*
	   * Catch the delete window message from the window
	   * manager.
	   */
	  if(event.xclient.data.l[0] == deleteWindowAtom)
	  {
	    done = True;
	  }
	  break;
	}
      }	/* end of switch */
    }

    /*
     * If Rtv encounters an error during recording, Rtv server sets handle->errno
     * to a non-zero value.  We can use that value here to terminate the
     * movie and exit the client.
     */
    if(handle->errno < 0 || done == True)
    {
      /*
       * The movie is finished recording.  Disconnect from the Rtv server and
       * indicate to the window server that it does not need to forward
       * window configuration changes to the Rtv server.
       */
      RtvRequest(handle, STOP_RTV_RECORD);
      handle->errno = 0;
      RtvCloseStream(handle);
      RtvRequest(handle, CLIENT_EXIT, 0);
      RtvDisconnect(handle);
      XPlxUnsetRtvWindow(display, window, rtvGC);

      /*
       * Stop the display of live video, and free the GCs.
       */

      XPlxVideoStop(display, window, videoGC);
      XFreeGC(display, rtvGC);
      XFreeGC(display, videoGC);
      exit(0);
    }
  }
} /* end program */


void Control_C_Handler(int sig)
{
  signal(SIGINT, Control_C_Handler);
  done = True;
}

Boolean XVideoCard(Display *display, Window window, GC videoGC)
{
  plx_IO 	*hardwareInfo;
  plx_IO_list 	*inputPtr;
  plx_IO_list 	*outputPtr;
  int 		channel;
  
  int    hardwareCode = 0;

  /*
   * This function determines if you are using an XVideo
   * VIO card based on the number of input and output
   * channels.  Two input channels and one output channel
   * indicates that you are using an XVideoVIO card.
   */
  hardwareInfo = XPlxQueryConfig(display, window, videoGC);
  if(hardwareInfo == NULL)
  {
    /*
     * This is not a Parallax video card!
     */
    return(False);
  }
  
  /*
   * Count the input channels:
   */
  inputPtr = hardwareInfo->inputs;
  while(inputPtr != NULL)
  {
    channel = inputPtr->channel;
    /*
     * Get rid of the instance information.
     */
    channel &= 0x00FF;

    hardwareCode |= (1 << (channel - 1));
    inputPtr = inputPtr->next_IO;
  }

  /*
   * Count the output channels:
   */
  outputPtr = hardwareInfo->outputs;
  while(outputPtr != NULL)
  {
    channel = outputPtr->channel;
    /*
     * Get rid of the instance information.
     */
    channel &= 0x00FF;

    hardwareCode |= ((1 << channel) << 8);
    outputPtr = outputPtr->next_IO;
  }

  /*
   * hardwareCode:
   *                       0xFFFF
   *                          ^ ^
   *   Indicate the outputs  /   \ Indicate the inputs.
   */

  return(hardwareCode == 0x0103);
}

