/*
 *   Copyright (c) 1993, 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* This program is used to show the ability to read Parallax JPEG
 * Movie files from disk, and to perform some simple performance tests
 * on the JPEG widget.  The program takes one argument, the name of
 * the disk file from which to read a JPEG Movie.  The program calls a
 * function which opens the file, then parses the JPEG Movie and
 * displaying the video part in the widget's window as quickly as
 * possible.  When the program is finished, it displays some simple
 * statistics to indicate how quickly it was able to display images.
 *
 * Usage: jpegPlaybackTest fileName */
/* $Header: /plx/source/Xclients/generic/RCS/jpegPlaybackTest.c,v 1.17 1995/10/06 23:04:58 drapeau Exp $ */
/* $Log: jpegPlaybackTest.c,v $
 * Revision 1.17  1995/10/06  23:04:58  drapeau
 * Minor fix to eliminate compiler warnings.
 *
 * Revision 1.16  1995/08/17  20:15:23  drapeau
 * Removed explicit inclusion of "malloc.h" to allow compilation on AIX
 * with GCC (inclusion of this file had caused the linker to show
 * unresolved symbols "kernel_heap" and "xmalloc".
 *
 * Revision 1.15  1995/07/28  15:51:21  drapeau
 * Enhanced the program to accept Parallax JPEG Movie files as opposed to
 * a file of raw JPEG images (the corresponnding program "jpegTimingTest"
 * has also been enhanced to produce JPEG Movie files).
 *
 * Revision 1.14  1995/06/27  20:53:05  drapeau
 * Minor fix: removed extraneous diagnostic messages.
 *
 * Revision 1.13  1994/10/22  04:41:12  drapeau
 * Made the code more ANSI-compatible.
 *
 * Revision 1.12  1993/11/17  02:32:04  drapeau
 * No real changes; only small cosmetic adjustments here.
 *
 * Revision 1.11  93/09/10  18:04:12  drapeau
 * Added support for JPEG widgets that don't use shared memory.
 * 
 * Revision 1.10  93/09/10  12:37:31  drapeau
 * First public version of this test program.
 *  */
static char jpegPlaybackTestRcsId[] = "$Header: /plx/source/Xclients/generic/RCS/jpegPlaybackTest.c,v 1.17 1995/10/06 23:04:58 drapeau Exp $";

typedef struct {
	unsigned	sample_rate;		/* samples per second */
	unsigned	samples_per_unit;	/* samples per unit */
	unsigned	bytes_per_unit;		/* bytes per sample unit */
	unsigned	channels;		/* # of interleaved channels */
	unsigned	encoding;		/* data encoding format */
	unsigned	data_size;		/* length of data (optional) */
} Audio_hdr;

#include <stdio.h>
#include <sys/param.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <X11/Xlib.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <XPlxExt.h>
#include <XVideoIn.h>
#include <JPEG.h>
#include <movie.h>

Widget top, jpegWidget;
Boolean	WorkProc(XtPointer);

char*		audioBuffer;
int		inputFile;
char*		fileName;
int		totalNumberOfGrabs = 0;
struct timeval	before;
XPlxCImage*	jpegDataPtr;
XPlxCImage*	allocatedImage;
char*		imageData;
jpheader	movieHeader;
int*		frameIndex;
int		bytesRead;

 
void main(int argc, char** argv)
{
  XtAppContext	appContext;
  int		result;
  XtWorkProcId	timerID;
  Boolean	useSharedMemory;
  
  top = XtAppInitialize(&appContext, "JPEGTest", NULL, 0,
			&argc, argv,
			NULL, (ArgList)NULL, 0);
  if (argc < 2)							    /* Was an input file specified? */
  {								    /* No, report the error and exit this program */
    printf("No input file was specified.  Exiting.\n");
    exit(-1);
  }
  fileName = argv[1];						    /* Yes, point to the file name specified on the command line */
  inputFile = open(fileName, O_RDONLY);				    /* Try to open the specified file for reading JPEG images */
  if (inputFile == -1)
  {
    printf("Could not open the file %s for reading.  Exiting.\n",
	   fileName);
    exit(inputFile);
  }
  bytesRead = read(inputFile, &movieHeader, sizeof(jpheader));	    /* Try to read the movie header from the newly-opened file */
  if (bytesRead < sizeof(jpheader))				    /* Did the read fail for some reason? */
  {								    /* Yes, print an error message and exit now */
    printf("Could not read the Movie header from the file.  Exiting.\n");
    exit(-1);
  }
  result = strncmp(movieHeader.magic, "j_movie", strlen("j_movie")); /* Determine if this is really a Parallax JPEG Movie */
  if (result != 0)						    /* Is this a Parallax JPEG Movie file? */
  {								    /* No, report the error and exit immediately */
    printf("The file '%s' is not a Parallax JPEG Movie.  Exiting.\n", fileName);
    exit(-1);
  }
  frameIndex = (int*)malloc(movieHeader.frames * sizeof(int));	    /* Attempt to allocate space for the movie's index ptr's */
  if ((int*)NULL == frameIndex)					    /* Did the memory allocation fail? */
  {								    /* Yes, report the error and exit immediately */
    printf("Could not allocate enough memory to hold %d frame's worth of movie offset information.  Exiting.\n",
	   movieHeader.frames);
    exit(-1);
  }
  if (movieHeader.audioslice > 0)				    /* Is audio in this movie?  */
    audioBuffer = (char*)malloc(movieHeader.audioslice);	    /* Yes, try to allocate space enough for audio data */
  else								    /* Apparently not; allocate enough space for 1 second's... */
    audioBuffer = (char*)malloc(192 * 1024);			    /* ...worth, just to be on the safe side. */
  if ((char*)NULL == audioBuffer)
  {
    printf("Could not allocate enough memory to hold 1 frame of audio data.  Exiting.\n");
    exit(-1);
  }
  lseek(inputFile, (long)movieHeader.indexbuf, SEEK_SET);	    /* Go to the point in the file where index info is stored */
  bytesRead = read(inputFile,					    /* Try to read the movie's frame offsets information  */
		   frameIndex,
		   (movieHeader.frames * sizeof(int)));
  if (bytesRead < (movieHeader.frames * sizeof(int)))
  {
    printf("Could not read the movie offset information.  Exiting.\n");
    exit(-1);
  }
  jpegWidget = XtVaCreateManagedWidget("jpeg", jpegWidgetClass,	    /* Create a JPEG widget used for displaying the raw JPEG... */
				       top,			    /* ...images that will be read in from disk. */
				       NULL);
  XtVaGetValues(jpegWidget,					    /* Determine if the JPEG widget is using shared memory... */
		XtNuseSharedMemory, &useSharedMemory,		    /* ...to store JPEG data. */
		NULL);
  if (useSharedMemory == False)
  {
    imageData = (char*)malloc(1024 * 256);			    /* Allocate enough data to hold a big image */
    allocatedImage = XPlxCreateCImage(XtDisplay(jpegWidget),	    /* Create a XPlxCImage structure to hold JPEG images */
				      imageData, 1024 * 256,
				      640, 480);
    if (allocatedImage == NULL)
    {
      printf("Could not create a CImage for storing JPEG images.  Exiting.\n");
      exit(-1);
    }
    XtVaSetValues(jpegWidget, XtNjpegData, allocatedImage, NULL);   /* Point the JPEG widget at the newly-created JPEG image */
  }
  XtVaGetValues(jpegWidget,					    /* Point to the JPEG data structure used by this widget;... */
		XtNjpegData,					    /* ...if the widget is using shared memory, then it's... */
		&jpegDataPtr,					    /* ...better to read data directly into the shared memory... */
		NULL);						    /* ...memory segment used by the widget. */
  jpegDataPtr->width = movieHeader.width;
  jpegDataPtr->height = movieHeader.height;
  XtVaSetValues(top,						    /* Resize the top-level widget to correspond to the... */
		XtNwidth, (Dimension)movieHeader.width,		    /* ...dimensions of the movie. */
		XtNheight, (Dimension)movieHeader.height,
		NULL);
  XtVaSetValues(jpegWidget,					    /* Force the widget to re-examine its JPEG image and... */
		XtNputNewImage, True,				    /* ...determine whether to redraw itself. */
		NULL);
  XtRealizeWidget(top);
  result = gettimeofday(&before, (struct timezone*)NULL);	    /* Store the current time of day for later stats reporting */
  timerID = XtAppAddWorkProc(appContext, WorkProc,		    /* Set up to read JPEG images from disk and display them... */
			     (XtPointer)NULL);			    /* ...whenever there's free time. */
  XtAppMainLoop(appContext);
  exit(0);
}								    /* end function main */



Boolean WorkProc(XtPointer clientData)
{
  int			counter;
  static int		currentFrame = 0;
  unsigned char		instruction;
  int			result;
  int			grabsPerSecond;
  Boolean		moreDataToRead = True;
  struct timeval	after;
  struct timeval	totalTime;
  struct timeval	timeForOneGrab;
  
  if (currentFrame <= (movieHeader.frames - 1))
  {
    lseek(inputFile, frameIndex[currentFrame], SEEK_SET);	    /* Go to the current frame to be played */
    while (True == moreDataToRead)
    {
      result = read(inputFile, &instruction, 1);		    /* Read an opcode from this frame */
      if (result < 1)						    /* Did the read fail? */
      {								    /* Yes, report the error, cleanup as best as can be done,... */
	printf("An error occurred reading the start of frame %d.  Exiting.\n",
	       currentFrame);
	XtDestroyWidget(jpegWidget);
	exit(-1);						    /* ...and exit immediately. */
      }
      instruction &= 0x00FF;
      switch ((int)instruction)					    /* Examine the movie opcode and do the right thing */
      {
       case LOAD_JPEG:						    /* Some JPEG image data is to be read */
	result = read(inputFile,				    /* Read in the size of the following JPEG image data */
		      &(jpegDataPtr->size),
		      sizeof(int));
	if (result < sizeof(int))				    /* Did the read fail? */
	{							    /* Yes, report the error, cleanup, and exit now */
	  printf("Could not read the size of the JPEG image data for frame %d.  Exiting.\n",
		 currentFrame);
	  XtDestroyWidget(jpegWidget);
	  exit(-1);
	}
	result = read(inputFile,				    /* Try to read one JPEG image */
		      jpegDataPtr->data,
		      jpegDataPtr->size);
	if (result < jpegDataPtr->size)				    /* Did the read fail? */
	{							    /* Yes, report the error, cleanup, and exit now */
	  printf("Could not read the JPEG image data for frame %d.  Exiting.\n",
		 currentFrame);
	  XtDestroyWidget(jpegWidget);
	  exit(-1);
	}
	XtVaSetValues(jpegWidget,				    /* Force the widget to re-examine its JPEG image and... */
		      XtNputNewImage, True,			    /* ...determine whether to redraw itself. */
		      NULL);
	break;							    /* Done reading the JPEG image for this frame */
       case LOAD_AUDIO_0:					    /* This program does not play audio, so simply consume... */
	result = read(inputFile,				    /* ...the audio data. */
		      audioBuffer,
		      movieHeader.audioslice);
	if (result < movieHeader.audioslice)
	{
	  printf("An error occurred reading audio data from frame %d.  Exiting.\n",
		 currentFrame);
	  XtDestroyWidget(jpegWidget);
	  exit(-1);
	}
	break;							    /* Done reading the JPEG image for this frame */
       case END_FRAME:
	moreDataToRead = False;					    /* Indicate that this frame is done */
	currentFrame++;
	return(False);
	break;
       default:
	printf("An unknown opcode (%d) was found on frame %d of the movie.  Exiting.\n",
	       instruction, currentFrame);
	XtDestroyWidget(jpegWidget);
	exit(-1);
      } /* end switch */
    } /* end while more data are to be read */
  }
  close(inputFile);						    /* Done reading JPEG data from disk */
  totalNumberOfGrabs = movieHeader.frames + 1;
  result = gettimeofday(&after, (struct timezone*)NULL);
  before.tv_usec += (before.tv_sec * 1000000);			    /* Convert this timeval to all microseconds */
  after.tv_usec += (after.tv_sec * 1000000);			    /* Do the same for this timeval */
  totalTime.tv_usec = after.tv_usec - before.tv_usec;		    /* Determine the total time taken for the above compressions */
  timeForOneGrab.tv_usec = (totalTime.tv_usec / totalNumberOfGrabs);
  grabsPerSecond = (1000000 / timeForOneGrab.tv_usec);
  timeForOneGrab.tv_sec = (timeForOneGrab.tv_usec / 1000000);	    /* Convert this timeval back into seconds and microseconds */
  timeForOneGrab.tv_usec = (timeForOneGrab.tv_usec % 1000000);
  totalTime.tv_sec = (totalTime.tv_usec / 1000000);		    /* Convert this timeval back into seconds and microseconds */
  totalTime.tv_usec = (totalTime.tv_usec % 1000000);
  printf("Total time taken for %d frames is %d.%d seconds.\n",
	 totalNumberOfGrabs,
	 totalTime.tv_sec,
	 totalTime.tv_usec);
  printf("Time taken for 1 frame is %d.%d seconds.\n",
	 timeForOneGrab.tv_sec,
	 timeForOneGrab.tv_usec);
  printf("Number of frames per second: %d.\n", grabsPerSecond);
  XtDestroyWidget(top);						    /* Destroy all widgets, especially the JPEG widget, to... */
  exit(0);							    /* ...free up shared memory resources taken by it. */
} /* end function WorkProc */
