/*
 * ttest.c
 * to compile : cc ttest.c -o ttest -I$PARALLAX_HOME/openwin/include
 *				    -I$OPENWINHOME/include -I/usr/demo/SOUND
 *				    -L$PARALLAX_HOME/openwin/lib
 *				    -L$OPENWINHOME/lib -L/usr/demo/SOUND
 *				    -lPlxApi -laudio -lXVvid -lXvid -lpixrect
 *				    -lxview -lolgx -lX11 -lXext -ltt -lce -lm
 */
#include <desktop/tt_c.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/panel.h>

static Xv_Window frame;
static fname, xname, yname, wname, hname;
static Canvas  video;
static Display *dpy;
static Window xid;
static int wid, hgt, off, fmt, bpd, dep, bpl, len;

#define VIDEO_WID 320
#define VIDEO_HGT 241


Done()
{
    xv_destroy_safe(frame);
}


Tt_callback_action PutAbstract(msg_in, pat)
Tt_message msg_in;
Tt_pattern pat;
{
    int mark;
    XImage *ximage, *hdr;
 
    if (strcmp(tt_message_op(msg_in), "Abstract") != 0)
    /*
     * do MORE for this
     */
	return;

    mark = tt_mark();
    tt_message_arg_bval(msg_in, 1, &hdr, &len);
    if (!hdr || !len) return;

    ximage = XCreateImage(dpy,
			xv_get(frame, XV_VISUAL),
			hdr->depth, hdr->format, hdr->xoffset, hdr + 1,
			hdr->width, hdr->height, hdr->bitmap_pad,
			hdr->bytes_per_line);
    XPutImage(dpy, xid,
		DefaultGC(dpy, DefaultScreen(dpy)),
		ximage, 0, 0, 0, 0, hdr->width, hdr->height);
 
    XFlush(dpy);
    XDestroyImage(ximage);

    tt_release(mark);
    tt_message_destroy(msg_in);
    return TT_CALLBACK_PROCESSED; 
}


Send_TT_Abstract(item, event)
Panel_item item;
Event *event;
{
    Tt_message msg_out;

    /*
     * Create and send the Abstract message.
     */
    msg_out = tt_prequest_create(TT_SESSION, "Abstract");
    tt_message_callback_add(msg_out, PutAbstract);
    tt_message_file_set(msg_out, xv_get(fname, PANEL_VALUE));
    tt_message_arg_add(msg_out, TT_IN, "JPEG_Video", NULL);
    tt_message_arg_add(msg_out, TT_OUT, "X_Bitmap", NULL);

    tt_message_send(msg_out);
}


Send_TT_Play(item, event)
Panel_item item;
Event *event;
{
    Tt_message msg_out;

    /*
     * Create and send the Display message.
     */
    msg_out = tt_prequest_create(TT_SESSION, "Display");
    tt_message_file_set(msg_out, xv_get(fname, PANEL_VALUE));
    tt_message_arg_add(msg_out, TT_IN, "JPEG_Video", 0 );

    tt_message_send(msg_out);
}


static void receive_tt_message()
{
    Tt_message msg_in;
    int mark;
 
    msg_in = tt_message_receive();
    if (msg_in != NULL && (tt_pointer_error(msg_in) == TT_ERR_NOMP))
        fprintf(stderr,"ToolTalk server down.\n");
    return;
}
 
 
Init_TT()
{
    char *procid;
    int ttfd;
    Tt_pattern pat;

    /*
     * Initialize ToolTalk, using the initial default session, and
     * obtain the file descriptor that will become active whenever
     * ToolTalk has a message for this process.
     */

    procid = tt_open();
    if (tt_pointer_error(procid) != TT_OK)
	printf("Error: tt_pointer_error failed\n");

    ttfd = tt_fd();

    /*
     * Create and register a pattern so ToolTalk knows we are
     * interested in "XBM_info" messages within the session we
     * join.
     */
    pat = tt_pattern_create();
    tt_pattern_op_add(pat, "XBM_info");
    tt_pattern_category_set(pat, TT_OBSERVE);
    tt_pattern_scope_add(pat, TT_SESSION);

    if (tt_pattern_register(pat) != TT_OK)
	printf("Error: tt_pattern_register failed\n");

    notify_set_input_func(frame, (Notify_func)receive_tt_message, ttfd);
    tt_session_join(tt_default_session());
    tt_free(procid);
}

CreatePanel()
{
    Panel panel;
    Panel_item abutton, sbutton, dbutton;

    frame = xv_create(NULL, FRAME,
		FRAME_SHOW_LABEL, TRUE,
		FRAME_DONE_PROC, Done,
		FRAME_SHOW_RESIZE_CORNER, FALSE,
		XV_LABEL, "Tooltalk Test Program",
		XV_X, 300,
		XV_Y, 300,
		XV_WIDTH, VIDEO_WID + 2,
		XV_HEIGHT, VIDEO_HGT + 70 + 2,
		NULL);
 

    panel = xv_create(frame, PANEL, NULL);
    abutton = xv_create(panel, PANEL_BUTTON,
		PANEL_NOTIFY_PROC, Send_TT_Abstract,
		PANEL_LABEL_STRING, "Abstract",
		XV_X, xv_col(panel, 0),
		XV_Y, xv_row(panel, 0),
		NULL);
    sbutton = xv_create(panel, PANEL_BUTTON,
                PANEL_NOTIFY_PROC, Send_TT_Play,
                PANEL_LABEL_STRING, "PLAY",
		XV_X, xv_col(panel, 10),
		XV_Y, xv_row(panel, 0),
                NULL);
    dbutton = xv_create(panel, PANEL_BUTTON,
                PANEL_NOTIFY_PROC, Done,
                PANEL_LABEL_STRING, "DONE",
		XV_X, xv_col(panel, 17),
		XV_Y, xv_row(panel, 0),
                NULL);

    fname = xv_create(panel, PANEL_TEXT,
		PANEL_VALUE_DISPLAY_LENGTH, 36,
		PANEL_VALUE_STORED_LENGTH, 135,
		PANEL_LABEL_STRING, "File Name : ",
		PANEL_VALUE, "./image",
		PANEL_LABEL_X, xv_col(panel, 0),
		PANEL_LABEL_Y, xv_row(panel, 1),
                NULL);

    video = (Canvas) xv_create(frame, CANVAS,
		XV_X, xv_col(panel, 0),
		XV_Y, xv_row(panel, 2),
		XV_WIDTH, VIDEO_WID,
		XV_HEIGHT, VIDEO_HGT,
		NULL);

    dpy = (Display *)xv_get(frame, XV_DISPLAY);
    xid = xv_get(canvas_paint_window(video), XV_XID);

    window_fit(video);
    window_fit(frame);
    xv_set(frame, WIN_SHOW, TRUE, NULL);
}


main(argc,argv)
int argc;
char *argv[];
{
    xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);
    CreatePanel();
    Init_TT();
    xv_main_loop(frame);
    tt_close();
}
