/***************************************************************************
 *	client.c : Network video client display
 *
 *	(C) Copyright 1990 by Paradise Software, Inc.
 *
 *	Written by John Malleo-Roach
 *
 *	Version %G% - %I%
 *
 ***************************************************************************/

/* INCLUDE FILES */

#include	<stdio.h> 
#include	<errno.h>
#include	<fcntl.h>
#include	<signal.h>
#include	<sys/types.h>
#include	<sys/ioctl.h>
#include	<sys/file.h>
#include	<sys/time.h>
#include	<sys/param.h>
#include	<sys/mman.h>
#ifndef SVR4
#include	<sun/fbio.h>
#endif
#include	<sys/stat.h>
#include	<sys/socket.h>
#include	<netinet/in.h>
#include	<arpa/inet.h>
#include	<netdb.h>
#include	<unistd.h>

#include <multimedia/libaudio.h>
#include <multimedia/audio_device.h>

/* CONSTANTS */

#define MAP			64
#define MAP2		(MAP * 2)
#define MAP3		(MAP * 3)
#define MAP4		(MAP * 4)

/* #define DEBUG            /* Define this symbol for generating debuging code */
/* #define TRACE            /* Define this symbol for generating tracing output */

#define BOOLEAN     int
#ifndef FALSE
#define FALSE       0
#endif
#ifndef TRUE
#define TRUE        1
#endif

#define AUDIO_DATA				0xffffffff

/* STATIC VARIABLES */

static char version[] = "%Z% Copyright (c) 1991 by Paradise Software - %M%\t%I%\t%G%";

static int FrameBuffer;

static struct fbcmap CMap;

static unsigned char *One;
static unsigned char *Two;

static unsigned char   red[MAP4];
static unsigned char green[MAP4];
static unsigned char  blue[MAP4];

static unsigned int  fbadd;

static unsigned int  pagesize;

static unsigned int  Debug=FALSE;			/* Display debugging information				*/
#ifdef HIRES
static unsigned int  HiRes=FALSE;			/* Display frame using interpolation			*/
#endif
static unsigned int  Speed=0;				/* User requested playback speed				*/
static unsigned char Buffer[16384];
static unsigned int  FrameNumber=0;
static unsigned int  Volume=6;

static unsigned char *aval[8];
static unsigned char *bval[8];

static BOOLEAN VideoRunning;				/* Toggle for video state						*/
static BOOLEAN VideoExit;					/* Toggle for client exit						*/
static BOOLEAN VideoStep;					/* Toggle for single frame step					*/

static int VideoSocket;
static struct sockaddr_in Destination;

static int Audio=NULL;						/* File descriptor for audio device				*/
static unsigned char audiobuf[16384];		/* Buffer for loading audio data				*/
static BOOLEAN Speaker=TRUE;				/* Use internal speaker/jack					*/

static Audio_hdr AudioHeader;				/* Audio record header for device				*/

/* EXTERNAL DEFINITIONS */

extern int optind;
extern char *optarg;

/* FUNCTIONS */

/* This is the SIGUSR1 handler.  It toggles the VideoRunning variable				*/

void control()
{
	VideoRunning ^= TRUE;
}

/* This is the SIGUSR2 handler.  It toggles the VideoStep variable					*/

void stepping()
{
	VideoStep ^= TRUE;
}

/* This is the SIGQUIT handler.  It toggles the VideoExit variable					*/

void getout()
{
	VideoRunning = FALSE;
	VideoStep    = FALSE;
	VideoExit	 = TRUE;
}

/* Compute some interesting playback numbers for the user */

void FindRate(tp,count)
struct timeval *tp;
int count;
{
	float d,e;
	struct timeval tt;
	int i,j;

	gettimeofday(&tt,NULL);
	i = tt.tv_sec * 1000000 + tt.tv_usec;
	j = tp->tv_sec * 1000000 + tp->tv_usec;
	i = i - j;
	e = ((float) i) / ((float) count);
	d = 1000000.0 / e;
	printf("Processed %d frames at %2.2f frames/sec\n",count,d);
	d = ((float) i) / 1000000;
	e = ((float) i) / ((float) count * 1000);
	printf("in %2.4f seconds at %2.2f msecs/frame\n",d,e);
}

/* Initialize our Network Video specific colormap (for OpenWindows 2.0) */

static int CreateColormap()
{
	register int i,j,k,l,m;

	CMap.index = 0;
	CMap.count = MAP4;
	CMap.red   = red;
	CMap.green = green;
	CMap.blue  = blue;

	if (ioctl(FrameBuffer,FBIOGETCMAP,&CMap) < 0) {
		perror("ioctl FBIOGETCMAP failed");
		return(-1);
	}

	i = 224;

	for (j=0; j<32; j++) {
		red[i]   = j * 8;
		green[i] = j * 8;
		blue[i]  = j * 8;
		i++;
	}

	CMap.index = 0;
	CMap.count = MAP4;
	CMap.red   = red;
	CMap.green = green;
	CMap.blue  = blue;

	if (ioctl(FrameBuffer,FBIOPUTCMAP,&CMap) < 0) {
		perror("ioctl FBIOPUTCMAP failed");
		return(-1);
	}

	return(0);
}

int InitializeAudio(device)
char *device;
{

/* Open up port for communicating with the audio driver */

	if ((Audio = open(device,O_WRONLY|O_NDELAY)) < 0) {
		Audio = NULL;
		if (errno == EBUSY) {
			fprintf(stderr,"Audio device is currently busy, try again later...");
			return(-1);
		}
		else {
			perror("Audio device open failed");
			return(-1);
		}
	}

	AudioHeader.sample_rate			= 8000;
	AudioHeader.samples_per_unit	= 1;
	AudioHeader.bytes_per_unit		= 1;
	AudioHeader.channels			= 1;
	AudioHeader.encoding			= AUDIO_ENCODING_ULAW;
	AudioHeader.data_size			= AUDIO_UNKNOWN_SIZE;

/* Make sure audio device matches audio file requirements */

	if (audio_set_play_config(Audio,&(AudioHeader)) != AUDIO_SUCCESS) {
		fprintf(stderr,"Couldn't reset audio device encoding to match video file");
		return(-1);
	}

/* Make sure audio device doesn't have any residue flags */

	return(NULL);
}

/* Use the audio device to set the requested volume */

static int SetAudioVolume(value)
int	value;
{
	double Vol;

	if (Audio == NULL) {
		fprintf(stderr,"SetAudioVolume: Audio device is not initialized!");
		return(-1);
	}

	if ((value < 0) || (value > 100)) {
		fprintf(stderr,"SetAudioVolume: Invalid volume request : %d",value);
		return(-1);
	}

	Vol = (double) value / (double) 100;

	if (audio_set_play_gain(Audio,&Vol) != AUDIO_SUCCESS) {
		fprintf(stderr,"SetAudioVolume: Could not set audio gain!");
		return(-1);
	}
	return(NULL);
}

/* Switch output between headphones and speaker */

static int SwitchAudioOutput(value)
int	value;
{
	unsigned port;

	if (Audio == NULL) {
		fprintf(stderr,"SwitchAudioOutput: Audio device is not initialized!");
		return(-1);
	}

	port = (value == 0 ? AUDIO_SPEAKER : AUDIO_HEADPHONE);

	if (audio_set_play_port(Audio,&port) != AUDIO_SUCCESS) {
		fprintf(stderr,"SwitchAudioOutput: Unable to switch audio output port");
		return(-1);
	}
	return(NULL);
}

PlayVideo()
{
	struct timeval tp;
	int i,m;
	register unsigned char *a,*b,*c,*d;
	register unsigned char e,f,g;
	register unsigned int  j;

	m = 0;

/* Get the clock start time */
	
	gettimeofday(&tp,NULL);

cloop:

	if (VideoRunning == TRUE) {
		read(VideoSocket,Buffer,16384);
		bcopy(Buffer,&i,sizeof(int));
		if (i == AUDIO_DATA) {
/* FIX THIS */
			write(Audio,Buffer,1000);
			goto cloop;
		}
		else {
			j = i & 0x7;
			if ((VideoStep == TRUE) && (j == 0)) {
				VideoRunning = FALSE;
				VideoStep = FALSE;
			}
		}
	}
	else {
		if (j == 0) {
			if (VideoExit == TRUE) return(0);
			if (VideoStep == TRUE) {
				j = 1;
				while (j != 0) {
					read(VideoSocket,Buffer,16384);
					bcopy(Buffer,&i,sizeof(int));
					j = i & 0x7;
				}
				VideoRunning = TRUE;
			}
			if (VideoRunning == FALSE) {
				sleep(1);
				goto cloop;
			}
		}
		else {
			read(VideoSocket,Buffer,16384);
			bcopy(Buffer,&i,sizeof(int));
			if (i == AUDIO_DATA) {
/* FIX THIS */
				write(Audio,Buffer,1000);
				goto cloop;
			}
			else {
				j = i & 0x7;
			}
		}
	}

	a = aval[j];
	b = bval[j];
	c = Buffer + sizeof(int);
	d = a + 1152;
	g = 240;
	while (a < b) {
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		e  = *c++; f = e >> 4; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		f = e & 0xf; if (f & 8) { f -= 7; g += f; } else g -= f;
		*a++ = g; *a++ = g; *d++ = g; *d++ = g;
		a = d + 832; d = a + 1152;
	}

	a = bval[j] - 1152;
	b = bval[j];
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;

	goto cloop;

exit:

	if (Debug) FindRate(&tp,m);
	return(0);
}

#ifdef HIRES

PlayHiResVideo()
{
	struct timeval tp;
	int i,m;
	register unsigned char *a,*b,*c,*d;
	register unsigned char e,f,g;
	register unsigned int  j;

	m = 0;

/* Get the clock start time */
	
	gettimeofday(&tp,NULL);

cloop:

	if (VideoRunning == TRUE) {
		read(VideoSocket,Buffer,16384);
		bcopy(Buffer,&i,sizeof(int));
		j = i & 0x7;
		if ((VideoStep == TRUE) && (j == 0)) {
			VideoRunning = FALSE;
			VideoStep = FALSE;
		}
	}
	else {
		if (j == 0) {
			if (VideoExit == TRUE) return(0);
			if (VideoStep == TRUE) {
				j = 1;
				while (j != 0) {
					read(VideoSocket,Buffer,16384);
					bcopy(Buffer,&i,sizeof(int));
					j = i & 0x7;
				}
				VideoRunning = TRUE;
			}
			if (VideoRunning == FALSE) {
				sleep(1);
				goto cloop;
			}
		}
		else {
			read(VideoSocket,Buffer,16384);
			bcopy(Buffer,&i,sizeof(int));
			j = i & 0x7;
		}
	}

	a = aval[j];
	b = bval[j];
	c = Buffer + sizeof(int);
	g = 240;
	while (a < b) {
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		e  = *c++; f = e >> 4;   if (f & 8) { f -= 7; g += f; } else g -= f; *a = g;
		a += 2;    f = e & 0xf;  if (f & 8) { f -= 7; g += f; } else g -= f; *a = g; a += 2;
		a = a + 832 + 1152;
	}
	a = aval[j];
	b = bval[j];
	c = a + 1; d = c + 1;
	while (a < b) {
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		*c = (*a + *d) >> 1; c += 2; a += 4; *c = (*a + *d) >> 1; c += 2; d += 4;
		a = a + 832 + 1152; c = a + 1; d = a + 1;
	}
	a = aval[j];
	b = bval[j];
	c = a + 1152; d = c + 1152;
	while (c < b) {
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		*c++ = (*a++ + *d++) >> 1; *c++ = (*a++ + *d++) >> 1;
		a = c + 832; c = a + 1152; d = c + 1152;
	}
	a = bval[j] - 1152;
	b = bval[j];
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;
	*b++ = *a++; *b++ = *a++; *b++ = *a++; *b++ = *a++;

	goto cloop;

exit:

	if (Debug) FindRate(&tp,m);
	return(0);
}
#endif

static void OpenConnection()
{
	VideoSocket = socket(AF_INET, SOCK_DGRAM, 0);

	bzero(&Destination,sizeof(Destination));
	Destination.sin_family      = AF_INET;
	Destination.sin_addr.s_addr = htonl(INADDR_ANY);
	Destination.sin_port        = htons(IPPORT_USERRESERVED);

	if (bind(VideoSocket,&Destination,sizeof(Destination)) < 0) {
		perror("bind socket");
		exit(1);
	}
}

/* Create the memory mapped segments for the frame buffer */

static void MapSegments(x,y)
int x,y;
{
	struct fbtype data;
	unsigned fbstart;
	unsigned len;

	if ((FrameBuffer = open("/dev/fb",O_RDWR)) < 0) {
		perror("Couldn't open /dev/fb");
		exit(3);
	}

/* Verify a support frame buffer */

	if (ioctl(FrameBuffer,FBIOGTYPE,&data) < 0) {
		perror("Ioctl FBIOGTYPE failed on /dev/fb");
		exit(7);
	}

	switch (data.fb_type) {
	case FBTYPE_SUN4COLOR:
		break;
	default:
		fprintf(stderr,"Unsupported frame buffer for network video!\n");
		exit(8);
	}

/* Compute requested offset into the cgthree frame buffer */

	pagesize = getpagesize();
	fbadd    = y * 1152 + x;
	fbstart  = (fbadd / pagesize) * pagesize;
	fbadd   -= fbstart;
	fbstart += (pagesize * 64);

/* Need 72 pages total, most on a sun4 is 64, so will break into two sections of 36 */
	
	len = (unsigned) 36 * pagesize;
	
	One = (unsigned char *) mmap((caddr_t) 0, len, PROT_READ|PROT_WRITE,
		MAP_SHARED, FrameBuffer, fbstart);
	if (One == (unsigned char *) -1) {
		perror("One mmap failed");
		exit(4);
	}

/* Need to overlap the segments by 1 page, so increase the length of Two by 2 pages */

	fbstart += (len - pagesize);
	len     += pagesize * 2;

	Two = (unsigned char *) mmap((caddr_t) One, len, PROT_READ|PROT_WRITE,
		MAP_SHARED, FrameBuffer, fbstart);
	if (Two == (unsigned char *) -1) {
		perror("Two mmap failed");
		exit(5);
	}
}

/* Main Function */
	
main(argc,argv)
int argc;
char *argv[];
{
	unsigned char *a;
	int fd,i,x,y;

	x					= 448;
	y					= 322;
	Volume				= 6;

/* Get options */

	while ((i = getopt(argc, argv, "v:x:y:djh")) != EOF) {
		switch (i) {
		case 'x':
			sscanf(optarg,"%d",&x);
			break;
		case 'y':
			sscanf(optarg,"%d",&y);
			break;
		case 'j':
			Speaker = FALSE;
			break;
		case 'v':
			Volume = atoi(optarg);
			break;
		case 'd':
			Debug = 1;
			break;
		case 'h':
#ifdef HIRES
			HiRes = TRUE;
#endif
			break;
		case '?':
			fprintf(stderr,"usage: client [-djh] [-x NNN] [-y NNN]\n");
			exit(2);
			break;
		}
	}

/* Create the memory mapped segments for the framebuffer */

	MapSegments(x,y);

/* Create our Videophile specific colormap for the frame buffer */

	if (CreateColormap() < 0) {
		fprintf(stderr,"Colormap initialization failed!");
		exit(6);
	}

/* Open and initialize the Network Video connection */

	OpenConnection();

/* Initialize framebuffer offsets */

	aval[0] = One + fbadd;
	aval[0] = aval[0] + ( 5 * 1152);
	aval[1] = aval[0] + (30 * 1152);
	aval[2] = aval[1] + (30 * 1152);
	aval[3] = aval[2] + (30 * 1152);
	aval[4] = Two + fbadd + 640;
	aval[5] = aval[4] + (30 * 1152);
	aval[6] = aval[5] + (30 * 1152);
	aval[7] = aval[6] + (30 * 1152);

	bval[0] = aval[0] + (29 * 1152);
	bval[1] = aval[1] + (29 * 1152);
	bval[2] = aval[2] + (29 * 1152);
	bval[3] = aval[3] + (29 * 1152);
	bval[4] = aval[4] + (29 * 1152);
	bval[5] = aval[5] + (29 * 1152);
	bval[6] = aval[6] + (29 * 1152);
	bval[7] = aval[7] + (29 * 1152);

	if (InitializeAudio("/dev/audio")) {
		exit(10);
	}
	else SetAudioVolume(Volume * 10);
	if (Speaker == FALSE) SwitchAudioOutput(TRUE);
	else SwitchAudioOutput(FALSE);

/* Enable signal pause and continue handling */

	VideoRunning = TRUE;
	signal(SIGUSR1,control);

/* Enable signal single step handling */

	VideoStep = FALSE;
	signal(SIGUSR2,stepping);

/* Enable signal exit handling */

	VideoExit = FALSE;
	signal(SIGQUIT,getout);

/* Play our video */

#ifdef HIRES
	if (HiRes) PlayHiResVideo();
	else PlayVideo();
#else
	PlayVideo();
#endif

/* Release the video input socket */

	close(VideoSocket);

/* Unmap the device from memory */

	munmap(One, 36 * pagesize);
	munmap(Two, 38 * pagesize);
	
	exit(0);
}

/* end of client.c */
