/*
 * (c) Copyright 1991 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log: player.c,v $
 * Revision 1.6  1992/05/27  16:25:04  schen
 * Moved from player.c to /plx/S/source/Xclients/sun4/schen/src/lib/api/player.c
 *
 * Revision 1.5  92/05/21  15:09:31  schen
 * checkin demo19
 * 
 * Revision 1.2  91/08/19  13:09:35  eric
 * added rcs header
 * 
 *
 */

#ifndef lint
static char *rcsid = "@(#)$Id: player.c,v 1.6 1992/05/27 16:25:04 schen Deleted $ Parallax Graphics Inc";
#endif

/*
 *	player.c - high-level interface routines for a video source machine
 *
 *	Written by John Malleo-Roach
 *
 */

/* INCLUDE FILES */

#include	<stdio.h>
#include	"alert.h"
#include	"file.h"
#include	"live.h"
#include	"pioneer.h"
#include	"sony.h"
#include	"pc-vcr.h"
#include	"pod.h"

/* CONSTANTS */

#define INITIALIZE		0
#define SET_AUDIO		1
#define SET_VIDEO		2
#define SHOW_FRAME		3
#define SHOW_SEGMENT	4
#define PLAY			5
#define REVERSE			6
#define STOP			7
#define CONTINUE		8
#define STILL			9
#define STEP_FORWARD	10
#define STEP_REVERSE	11
#define SLOW_FORWARD	12
#define SLOW_REVERSE	13
#define FAST_FORWARD	14
#define FAST_REVERSE	15
#define INDEX			16
#define STATUS			17
#define FRAME_NUMBER	18
#define EJECT			19

/* Known video player types */

#define PIONEER_MODEL	0
#define SONY_MODEL		1
#define NEC_MODEL		2
#define POD_MODEL		3
#define LIVE_MODEL		4
#define FILE_MODEL		5

/* Player status codes, these are the same in all device driver files */

#define PLAYER_STATUS_UNKNOWN	0
#define PLAYER_STATUS_STOP		1
#define PLAYER_STATUS_STILL		2
#define PLAYER_STATUS_PLAY		3
#define PLAYER_STATUS_EMPTY		4

/* TYPEDEFS */

typedef struct {
	int fd;
	int model;
	int speed;
	char *device;
	int status;
} Player;

/* GLOBALS */

int PlayerId = -1;						/* File descriptor for the laserdisc player	*/

/* STATIC VARIABLES */

static int PlayerModel;					/* Player model we are using 				*/

static char buffer[256];				/* Scratch buffer							*/

static int (*PlayerFunctions[6][20])() = {
	{
		InitializePioneer,
		SetPioneerAudio,
		SetPioneerVideo,
		ShowPioneerFrame,
		ShowPioneerSegment,
		PioneerPlay,
		PioneerReverse,
		PioneerStop,
		PioneerContinue,
		PioneerStill,
		PioneerStepForward,
		PioneerStepReverse,
		PioneerSlowForward,
		PioneerSlowReverse,
		PioneerFastForward,
		PioneerFastReverse,
		PioneerIndex,
		PioneerStatus,
		PioneerFrameNumber,
		PioneerEject
	},
	{
		InitializeSony,
		SetSonyAudio,
		SetSonyVideo,
		ShowSonyFrame,
		ShowSonySegment,
		SonyPlay,
		SonyReverse,
		SonyStop,
		SonyContinue,
		SonyStill,
		SonyStepForward,
		SonyStepReverse,
		SonySlowForward,
		SonySlowReverse,
		SonyFastForward,
		SonyFastReverse,
		SonyIndex,
		SonyStatus,
		SonyFrameNumber,
		SonyEject
	},
	{
		InitializeNEC,
		SetNECAudio,
		SetNECVideo,
		ShowNECFrame,
		ShowNECSegment,
		NECPlay,
		NECReverse,
		NECStop,
		NECContinue,
		NECStill,
		NECStepForward,
		NECStepReverse,
		NECSlowForward,
		NECSlowReverse,
		NECFastForward,
		NECFastReverse,
		NECIndex,
		NECStatus,
		NECFrameNumber,
		NECEject
	},
	{
		InitializePod,
		SetPodAudio,
		SetPodVideo,
		ShowPodFrame,
		ShowPodSegment,
		PodPlay,
		PodReverse,
		PodStop,
		PodContinue,
		PodStill,
		PodStepForward,
		PodStepReverse,
		PodSlowForward,
		PodSlowReverse,
		PodFastForward,
		PodFastReverse,
		PodIndex,
		PodStatus,
		PodFrameNumber,
		PodEject
	},
	{
		InitializeLive,
		SetLiveAudio,
		SetLiveVideo,
		ShowLiveFrame,
		ShowLiveSegment,
		LivePlay,
		LiveReverse,
		LiveStop,
		LiveContinue,
		LiveStill,
		LiveStepForward,
		LiveStepReverse,
		LiveSlowForward,
		LiveSlowReverse,
		LiveFastForward,
		LiveFastReverse,
		LiveIndex,
		LiveStatus,
		LiveFrameNumber,
		LiveEject
	},
	{
		InitializeFile,
		SetFileAudio,
		SetFileVideo,
		ShowFileFrame,
		ShowFileSegment,
		FilePlay,
		FileReverse,
		FileStop,
		FileContinue,
		FileStill,
		FileStepForward,
		FileStepReverse,
		FileSlowForward,
		FileSlowReverse,
		FileFastForward,
		FileFastReverse,
		FileIndex,
		FileStatus,
		FileFrameNumber,
		FileEject
	}
};

/* FUNCTIONS */

/* Initialize the selected laserdisc player */

Player *InitializeVideo(device,player_type,speed)
char *device;
int player_type;
int speed;
{
	Player *p;
	int retcode;

	PlayerModel = LIVE_MODEL;

	if (player_type == PIONEER_MODEL) PlayerModel = player_type;
	if (player_type == SONY_MODEL   ) PlayerModel = player_type;
	if (player_type == NEC_MODEL    ) PlayerModel = player_type;
	if (player_type == POD_MODEL    ) PlayerModel = player_type;
	if (player_type == LIVE_MODEL   ) PlayerModel = player_type;
	if (player_type == FILE_MODEL   ) PlayerModel = player_type;

	retcode = PlayerFunctions[PlayerModel][INITIALIZE](device,speed);

	if (retcode) p = NULL;
	else {
		p = (Player *) calloc(1,sizeof(Player));
		p->model  = PlayerModel;
		p->fd     = PlayerId;
		p->speed  = speed;
		p->status = PLAYER_STATUS_UNKNOWN;
		p->device = device;
	}
	return(p);
}

/* Set Audio parameters */

int SetPlayerAudio(player,mode)
Player *player;
int mode;
{
	int retcode;

	PlayerId	= player->fd;
	PlayerModel	= player->model;
	retcode		= PlayerFunctions[PlayerModel][SET_AUDIO](mode);

	return(retcode);
}

int SetPlayerVideo(player,mode)
Player *player;
int mode;
{
	int retcode;

	PlayerId	= player->fd;
	PlayerModel	= player->model;
	retcode		= PlayerFunctions[PlayerModel][SET_VIDEO](mode);

	return(retcode);
}

int ShowDiscFrame(player,frame)
Player *player;
int frame;
{
	int retcode;

	PlayerId	= player->fd;
	PlayerModel	= player->model;

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("Video Player not initialized",NULL);
		return(-1);
	}

	retcode = PlayerFunctions[PlayerModel][SHOW_FRAME](frame);

	return(retcode);
}

int ShowDiscSegment(player,start,end,repeat)
Player *player;
int start,end;
int repeat;
{
	int retcode;

	PlayerId	= player->fd;
	PlayerModel	= player->model;

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("Video Player not initialized",NULL);
		return(-1);
	}

	retcode = PlayerFunctions[PlayerModel][SHOW_SEGMENT](start,end,repeat);

	return(retcode);
}

int PlayerPlay(player)
Player *player;
{
	int retcode;

	PlayerId	= player->fd;
	PlayerModel	= player->model;

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("Video Player not initialized",NULL);
		return(-1);
	}

	retcode = PlayerFunctions[PlayerModel][PLAY]();

	return(retcode);
}

int PlayerReverse(player)
Player *player;
{
	int retcode;

	PlayerId	= player->fd;
	PlayerModel	= player->model;

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("Video Player not initialized",NULL);
		return(-1);
	}

	retcode = PlayerFunctions[PlayerModel][REVERSE]();

	return(retcode);
}

int PlayerStop(player)
Player *player;
{
	int retcode;

	PlayerId	= player->fd;
	PlayerModel	= player->model;

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("Video Player not initialized",NULL);
		return(-1);
	}

	retcode = PlayerFunctions[PlayerModel][STOP]();

	return(retcode);
}

int PlayerContinue(player)
Player *player;
{
	int retcode;

	PlayerId	= player->fd;
	PlayerModel	= player->model;

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("Video Player not initialized",NULL);
		return(-1);
	}

	retcode = PlayerFunctions[PlayerModel][CONTINUE]();

	return(retcode);
}

int PlayerStill(player)
Player *player;
{
	int retcode;

	PlayerId	= player->fd;
	PlayerModel	= player->model;

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("Video Player not initialized",NULL);
		return(-1);
	}

	retcode = PlayerFunctions[PlayerModel][STILL]();

	return(retcode);
}

int PlayerStepForward(player)
Player *player;
{
	int retcode;

	PlayerId	= player->fd;
	PlayerModel	= player->model;

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("Video Player not initialized",NULL);
		return(-1);
	}

	retcode = PlayerFunctions[PlayerModel][STEP_FORWARD]();

	return(retcode);
}

int PlayerStepReverse(player)
Player *player;
{
	int retcode;

	PlayerId	= player->fd;
	PlayerModel	= player->model;

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("Video Player not initialized",NULL);
		return(-1);
	}

	retcode = PlayerFunctions[PlayerModel][STEP_REVERSE]();

	return(retcode);
}

int PlayerSlowForward(player)
Player *player;
{
	int retcode;

	PlayerId	= player->fd;
	PlayerModel	= player->model;

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("Video Player not initialized",NULL);
		return(-1);
	}

	retcode = PlayerFunctions[PlayerModel][SLOW_FORWARD]();

	return(retcode);
}

int PlayerSlowReverse(player)
Player *player;
{
	int retcode;

	PlayerId	= player->fd;
	PlayerModel	= player->model;

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("Video Player not initialized",NULL);
		return(-1);
	}

	retcode = PlayerFunctions[PlayerModel][SLOW_REVERSE]();

	return(retcode);
}

int PlayerFastForward(player)
Player *player;
{
	int retcode;

	PlayerId	= player->fd;
	PlayerModel	= player->model;

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("Video Player not initialized",NULL);
		return(-1);
	}

	retcode = PlayerFunctions[PlayerModel][FAST_FORWARD]();

	return(retcode);
}

int PlayerFastReverse(player)
Player *player;
{
	int retcode;

	PlayerId	= player->fd;
	PlayerModel	= player->model;

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("Video Player not initialized",NULL);
		return(-1);
	}

	retcode = PlayerFunctions[PlayerModel][FAST_REVERSE]();

	return(retcode);
}

int PlayerIndex(player,status)
Player *player;
int status;
{
	int retcode;

	PlayerId	= player->fd;
	PlayerModel	= player->model;

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("Video Player not initialized",NULL);
		return(-1);
	}

	retcode = PlayerFunctions[PlayerModel][INDEX](status);

	return(retcode);
}

int PlayerStatus(player)
Player *player;
{
	int retcode;

	PlayerId	= player->fd;
	PlayerModel	= player->model;

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("Video Player not initialized",NULL);
		return(-1);
	}

	retcode = PlayerFunctions[PlayerModel][STATUS]();

	return(retcode);
}

int PlayerFrameNumber(player)
Player *player;
{
	int retcode;

	PlayerId	= player->fd;
	PlayerModel	= player->model;

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("Video Player not initialized",NULL);
		return(-1);
	}

	retcode = PlayerFunctions[PlayerModel][FRAME_NUMBER]();

	return(retcode);
}

int PlayerEject(player)
Player *player;
{
	int retcode;

	PlayerId	= player->fd;
	PlayerModel	= player->model;

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("Video Player not initialized",NULL);
		return(-1);
	}

	retcode = PlayerFunctions[PlayerModel][EJECT]();

	return(retcode);
}

/* end of player.c */
