/*
 * (c) Copyright 1991 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log: sony.c,v $
 * Revision 1.6  1992/05/27  16:25:39  schen
 * Moved from sony.c to /plx/S/source/Xclients/sun4/schen/src/lib/api/sony.c
 *
 * Revision 1.5  92/05/21  15:09:35  schen
 * checkin demo19
 * 
 * Revision 1.2  91/08/19  13:09:45  eric
 * added rcs header
 * 
 *
 */

#ifndef lint
static char *rcsid = "@(#)$Id: sony.c,v 1.6 1992/05/27 16:25:39 schen Deleted $ Parallax Graphics Inc";
#endif

/*
 *	sony.c - low level routines for driving the Sony LDP-2000 player
 *
 *	Written by John Malleo-Roach
 *
 */

/* INCLUDE FILES */

#include	<stdio.h>
#include	<sys/types.h>
#include	<sys/termios.h>
#include	<sys/filio.h>
#include	<fcntl.h>
#include	<strings.h>
#include	<errno.h>
#include	"alert.h"

/* CONSTANTS */

#define BUFLEN			32

#define ModelNumber		"LDP2000"

/* Define the instructions for communicating to a LDM2000 */

#define Completion		'\001'
#define Error			'\002'
#define NotTarget		'\005'
#define NoFrame			'\006'
#define Ack				'\012'
#define Nak				'\013'

#define Enter			"\100"

#define Stop			"\077"
#define Reject			"\143"
#define Start			"\142"

#define Still			"\117"
#define Continue		"\141"

#define Play			"\072"
#define Reverse			"\112"
#define FastForward		"\073"
#define FastReverse		"\113"
#define SlowForward		"\074"
#define SlowReverse		"\114"
#define VarForward		"\075"
#define VarReverse		"\115"
#define ScanForward		"\076"
#define ScanReverse		"\116"

#define Search			"\103"
#define Repeat			"\104"
#define FrameMode		"\125"

#define AudioOn			"\044"
#define AudioOff		"\045"
#define Ch1On			"\106"
#define Ch1Off			"\107"
#define Ch2On			"\110"
#define Ch2Off			"\111"
#define IndexOn			"\120"
#define IndexOff		"\121"

#define Clear			"\126"
#define FrameRequest	"\140"
#define StatusRequest	"\147"
#define DiscRequest		"\150"
#define ModelRequest	""

/* Various other constants for the laser player */

#define AUDIO_OFF			401
#define LEFT_ONLY			402
#define RIGHT_ONLY			403
#define STEREO				404

#define VIDEO_OFF			410
#define VIDEO_ON			411

#define SearchOrRepeatFlag	64
#define MotorOffFlag		32
#define InitFlag			16
#define DiscPropFlag		8
#define NoDiscFlag			4
#define OutOfFocusFlag		2
#define ErrorFlag			1

#define StepModeFlag		8
#define PlayModeFlag		1

/* Player status codes, these are the same in all device driver files */

#define PLAYER_STATUS_UNKNOWN	0
#define PLAYER_STATUS_STOP		1
#define PLAYER_STATUS_STILL		2
#define PLAYER_STATUS_PLAY		3
#define PLAYER_STATUS_EMPTY		4

/* GLOBALS */

extern int PlayerId;		/* File descriptor for the Pioneer laservision player */

static char buffer[BUFLEN];	/* Buffer for messages from the player 				  */
static char message[256];	/* Scratch space for error messages					  */

/* FUNCTIONS */

/* Send a byte to the sony player */

static int Send(command,count)
char *command;
int count;
{
	int i;

/* Flush any exising characters in the queue */
	
	if (ioctl(PlayerId,TCFLSH,2) < 0) {
#ifdef DEBUG
		perror("Couldn't flush I/O queues!");
#endif
		return(-1);
	}
#ifdef DEBUG
	fprintf(stderr,"Sending command : %s\n",command);
#endif
	if (write(PlayerId,command,strlen(command)) < 0) {
		SystemError(errno,"Write to device failed!");
		return (-1);
	}
	while (count) {
		i = read(PlayerId,buffer,BUFLEN);
#ifdef DEBUG
		if (i != 1) fprintf(stderr,"Received %d bytes instead of 1\n",i);
#endif
		if (buffer[0] == Ack) {
#ifdef DEBUG
			fprintf(stderr,"Received ACK(%d) on previous command\n",count);
#endif
		}
		else if (buffer[0] == Completion) {
#ifdef DEBUG
			fprintf(stderr,"Received COMP(%d) on previous command\n",count);
#endif
		}
		else if (buffer[0] == NoFrame) {
			ShowAlert("Requested frame number not found!",NULL);
			return(-1);
		}
		else if (buffer[0] == NotTarget) {
			ShowAlert("Player returned NotTarget on previous command!",NULL);
			return(-1);
		}
		else {
			sprintf(message,"Communication error with player : %o",buffer[0]);
			ShowAlert(message,NULL);
#ifdef DEBUG
			if (buffer[0] == Nak) fprintf(stderr,"NAK received!");
			else if (buffer[0] == Error) fprintf(stderr,"ERROR received!");
			else fprintf(stderr,"Received unknown byte from player : %o",buffer[0]);
#endif
			return(-1);
		}
		count--;
	}
	return(NULL);
}

/* Read the 5 byte status message returned by the player */

static int ReadPlayerStatus(b)
char *b;
{
	int i;

/* Read the 5 byte status byte 1 byte at a time. */

	i = read(PlayerId,&(buffer[0]),1);
	if (i != 1) {
		ShowAlert("Reading player status word failed!",NULL);
		return(-1);
	}
	i = read(PlayerId,&(buffer[1]),1);
	if (i != 1) {
		ShowAlert("Reading player status word failed!",NULL);
		return(-1);
	}
	i = read(PlayerId,&(buffer[2]),1);
	if (i != 1) {
		ShowAlert("Reading player status word failed!",NULL);
		return(-1);
	}
	i = read(PlayerId,&(buffer[3]),1);
	if (i != 1) {
		ShowAlert("Reading player status word failed!",NULL);
		return(-1);
	}
	i = read(PlayerId,&(buffer[4]),1);
	if (i != 1) {
		ShowAlert("Reading player status word failed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* Initialize the Sony player to a known status */

int InitializeSony(device,speed)
char *device;
int speed;
{
	int retcode;
    struct termios ttbuf;

/* Open up port for communicating with the laser disc */

	if ((PlayerId = open(device, O_RDWR | O_NDELAY)) < 0) {
		SystemError(errno,"Couldn't open laserdisc player device!");
		return(-1);
	}

/* Set the default speed to 1200 baud and RAW character mode */
	
    (void) ioctl(PlayerId, TCGETS, &ttbuf);

    ttbuf.c_iflag		= (long) 0;
    ttbuf.c_oflag		= (long) 0;
    ttbuf.c_lflag		= (long) 0;

	switch (speed) {
	case 0:
	    ttbuf.c_cflag	= (long) B1200 | CS8 | CLOCAL | CREAD | (B1200 << IBSHIFT) ;
		break;
	case 1:
	    ttbuf.c_cflag	= (long) B1200 | CS8 | CLOCAL | CREAD | (B1200 << IBSHIFT) ;
		break;
	case 2:
	    ttbuf.c_cflag	= (long) B2400 | CS8 | CLOCAL | CREAD | (B2400 << IBSHIFT) ;
		break;
	case 3:
	    ttbuf.c_cflag	= (long) B4800 | CS8 | CLOCAL | CREAD | (B4800 << IBSHIFT) ;
		break;
	case 4:
	    ttbuf.c_cflag	= (long) B9600 | CS8 | CLOCAL | CREAD | (B9600 << IBSHIFT) ;
		break;
	}

    ttbuf.c_cc[VMIN]	= 0;
    ttbuf.c_cc[VTIME]	= 10;

	if ((retcode = ioctl(PlayerId, TCSETSW, &ttbuf)) < 0) {
		SystemError(errno,"Couldn't initialize device!");
		close(PlayerId);
		return(-1);
	}

/* Flush any garbage out of the I/O queues */
	
	if ((retcode = ioctl(PlayerId, TCFLSH, 2)) < 0) {
		SystemError(errno,"Couldn't flush I/O queues!");
		close(PlayerId);
		return(-1);
	}

/* See if we recognize this player: Not possible on this model

	if (Send(ModelRequest,0) < 0) {
		close(PlayerId);
		return(-1);
	}
	retcode = read(PlayerId,buffer,BUFLEN);
	if (strcmp(buffer,ModelNumber) != 0) {
		sprintf(message,"Unsupported model number %s",buffer);
		ShowAlert(message,NULL);
		close(PlayerId);
		return(-1);
	}
*/

/* Reset any previous errors */

	if (Send(Clear,1) < 0) {
		ShowAlert("Failed to reset Sony status flag",NULL);
		close(PlayerId);
		return(-1);
	}

/* Find out player status and get user to load laserdisc */

	if (Send(StatusRequest,0) < 0) {
		close(PlayerId);
		return(-1);
	}
	if (ReadPlayerStatus(buffer) < 0) {
		close(PlayerId);
		return(-1);
	}

/* Process on current status */

	while (buffer[0] & NoDiscFlag) {
		ShowNote("Select OKAY after loading VideoDisc into Player",NULL);
		if (Send(StatusRequest,0) < 0) {
			close(PlayerId);
			return(-1);
		}
 		if (ReadPlayerStatus(buffer) < 0) {
			close(PlayerId);
			return(-1);
		}
	}
	if (buffer[0] & DiscPropFlag) {
		if (Send(Start,20) < 0) {
			ShowAlert("Failed to complete player initialization",NULL);
			close(PlayerId);
			return(-1);
		}
	}

/* Make sure the player is in frame mode */

	if (Send(FrameMode,1) < 0) {
		ShowAlert("Could not establish frame # mode!",NULL);
		close(PlayerId);
		return(-1);
	}
	return(NULL);
}

/* Command : Return simple player status code */

int SonyStatus()
{

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("VideoDisc Player not initialized",NULL);
		return(-1);
	}
	if (Send(StatusRequest,0) < 0) {
		ShowAlert("Player status request failed!",NULL);
		return(-1);
	}
	if (ReadPlayerStatus(buffer) < 0) {
		close(PlayerId);
		return(-1);
	}
	return(PLAYER_STATUS_UNKNOWN);
}

/* Command : Return current frame number */

int SonyFrameNumber()
{
	int number,i;

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("VideoDisc Player not initialized",NULL);
		return(-1);
	}
	if (Send(FrameRequest,0) < 0) {
		ShowAlert("Player frame number request failed!",NULL);
		return(-1);
	}
	i = read(PlayerId,&(buffer[0]),1);
	if (i != 1) {
		ShowAlert("Reading player frame number failed!",NULL);
		return(-1);
	}
	i = read(PlayerId,&(buffer[1]),1);
	if (i != 1) {
		ShowAlert("Reading player frame number failed!",NULL);
		return(-1);
	}
	i = read(PlayerId,&(buffer[2]),1);
	if (i != 1) {
		ShowAlert("Reading player frame number failed!",NULL);
		return(-1);
	}
	i = read(PlayerId,&(buffer[3]),1);
	if (i != 1) {
		ShowAlert("Reading player frame number failed!",NULL);
		return(-1);
	}
	i = read(PlayerId,&(buffer[4]),1);
	if (i != 1) {
		ShowAlert("Reading player frame number failed!",NULL);
		return(-1);
	}
	buffer[5] = '\0';
	number = atoi(buffer);

	return(number);
}

/* Set Audio parameters */

int SetSonyAudio(mode)
int mode;
{

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("VideoDisc Player not initialized",NULL);
		return(-1);
	}
	switch (mode) {
	case AUDIO_OFF:
		if (Send(AudioOff,1) < 0) {
			ShowAlert("Unable to turn audio track off!",NULL);
			return(-1);
		}
		break;
	case LEFT_ONLY:
		if (Send(AudioOff,1) < 0) {
			ShowAlert("Unable to turn audio track on!",NULL);
			return(-1);
		}
		if (Send(Ch1Off,1) < 0) {
			ShowAlert("Unable to turn right audio track off!",NULL);
			return(-1);
		}
		if (Send(Ch2On,1) < 0) {
			ShowAlert("Unable to turn left audio track on!",NULL);
			return(-1);
		}
		break;
	case RIGHT_ONLY:
		if (Send(AudioOff,1) < 0) {
			ShowAlert("Unable to turn audio track on!",NULL);
			return(-1);
		}
		if (Send(Ch1On,1) < 0) {
			ShowAlert("Unable to turn right audio track on!",NULL);
			return(-1);
		}
		if (Send(Ch2Off,1) < 0) {
			ShowAlert("Unable to turn left audio track off!",NULL);
			return(-1);
		}
		break;
	case STEREO:
		if (Send(AudioOff,1) < 0) {
			ShowAlert("Unable to turn audio track on!",NULL);
			return(-1);
		}
		if (Send(Ch1On,1) < 0) {
			ShowAlert("Unable to turn right audio track on!",NULL);
			return(-1);
		}
		if (Send(Ch2On,1) < 0) {
			ShowAlert("Unable to turn left audio track on!",NULL);
			return(-1);
		}
		break;
	default:
		sprintf(message,"Unknown mode requested - %d",mode);
		ShowAlert(message,NULL);
		return(-1);
	}
	return(NULL);
}

/* This is a noop for now, as the player doesn't seem to support this option */

int SetSonyVideo(mode)
int mode;
{

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("VideoDisc Player not initialized",NULL);
		return(-1);
	}
	switch (mode) {
	case VIDEO_OFF:
	case VIDEO_ON:
		ShowNote("This option not supported for this unit",NULL);
		break;
	default:
		sprintf(message,"Unknown mode requested - %d",mode);
		ShowAlert(message,NULL);
		return(-1);
	}
	return(NULL);
}

/* Command : Seek and display a single frame */

int ShowSonyFrame(frame)
int frame;
{
	int i;
	char tmp[2];
	char number[8];

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("VideoDisc Player not initialized",NULL);
		return(-1);
	}

/* Check for valid parameter */

	if ((frame < 0) || (frame > 128000)) {
		sprintf(message,"Invalid Frame address requested : %d",frame);
		ShowAlert(message,NULL);
		return(-1);
	}

/* Send out a search frame request */

	if (Send(Search,1) < 0) {
		ShowAlert("Search command failed!",NULL);
		return(-1);
	}
	tmp[1] = NULL;
	sprintf(number,"%05d",frame);
	for (i=0; i<5; i++) {
		tmp[0] = number[i];
		if (Send(tmp,1) < 0) {
			sprintf(message,"Frame number (char %d) transmit failed!",i);
			ShowAlert(message,NULL);
			return(-1);
		}
	}

/* Wait here until frame is found (2 ACKS) */

	if (Send(Enter,2) < 0) {
		ShowAlert("Enter command failed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* Command : Seek and display a complete video segment */

int ShowSonySegment(start,end,repeat)
int start,end,repeat;
{
	int i;
	char tmp[2];
	char number[8];

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("VideoDisc Player not initialized",NULL);
		return(-1);
	}

/* Check for valid parameters */

	if ((start < 0) || (start > 128000)) {
		sprintf(message,"Invalid starting frame address requested - %d",start);
		ShowAlert(message);
		return(-1);
	}

	if ((end < 0) || (end > 128000)) {
		sprintf(message,"Invalid ending frame address requested - %d",end);
		ShowAlert(message);
		return(-1);
	}

	if (start >= end) {
		sprintf(message,"Starting frame (%d) not less than end (%d)",start,end);
		ShowAlert(message,NULL);
		return(-1);
	}

/* Send out a search frame request */

	if (Send(Search,1) < 0) {
		ShowAlert("Search command failed!",NULL);
		return(-1);
	}
	tmp[1] = NULL;
	sprintf(number,"%05d",start);
	for (i=0; i<5; i++) {
		tmp[0] = number[i];
		if (Send(tmp,1) < 0) {
			sprintf(message,"Frame number (char %d) transmit failed!",i);
			ShowAlert(message,NULL);
			return(-1);
		}
	}

/* Wait here until successful (2 ACKS) */

	if (Send(Enter,2) < 0) {
		ShowAlert("Enter command failed!",NULL);
		return(-1);
	}

/* Start play with stop point */

	if (Send(Repeat,1) < 0) {
		ShowAlert("Repeat command failed!",NULL);
		return(-1);
	}
	sprintf(number,"%05d",end);
	for (i=0; i<5; i++) {
		tmp[0] = number[i];
		if (Send(tmp,1) < 0) {
			sprintf(message,"Frame number (char %d) transmit failed!",i);
			ShowAlert(message);
			return(-1);
		}
	}
	if (Send(Enter,1) < 0) {
		ShowAlert("Enter command failed!",NULL);
		return(-1);
	}

	tmp[0] = NULL;
	if (repeat) strcpy(tmp,"0"); else strcpy(tmp,"1");
	if (Send(tmp,1) < 0) {
		ShowAlert("Repeat count transmit failed!",NULL);
		return(-1);
	}
	if (Send(Enter,1) < 0) {
		ShowAlert("Enter command failed!",NULL);
		return(-1);
	}

/* Skip the wait for the completion code */

/*	read(PlayerId,buffer,BUFLEN); */

	return(NULL);
}

/* Command : play forward */

int SonyPlay()
{
/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("VideoDisc Player not initialized",NULL);
		return(-1);
	}

/* See if we are in a reasonable mode */

	if (Send(StatusRequest,0) < 0) {
		ShowAlert("Player status request failed!",NULL);
		return(-1);
	}
	if (ReadPlayerStatus(buffer) < 0) {
		close(PlayerId);
		return(-1);
	}

/* Wait here until playing has started */

	if (Send(Play,1) < 0) {
		ShowAlert("Play command failed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* Command : play reverse */

int SonyReverse()
{

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("VideoDisc Player not initialized",NULL);
		return(-1);
	}

/* See if we are in a reasonable mode */

	if (Send(StatusRequest,0) < 0) {
		ShowAlert("Player status request failed!",NULL);
		return(-1);
	}
	if (ReadPlayerStatus(buffer) < 0) {
		close(PlayerId);
		return(-1);
	}

/* Wait here until playing has started */

	if (Send(Reverse,1) < 0) {
		ShowAlert("Play command failed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* Command : Continue an interrupted segment */

int SonyContinue()
{

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("VideoDisc Player not initialized",NULL);
		return(-1);
	}

/* See if we are in a reasonable mode */

	if (Send(StatusRequest,0) < 0) {
		ShowAlert("Player status request failed!",NULL);
		return(-1);
	}
	if (ReadPlayerStatus(buffer) < 0) {
		close(PlayerId);
		return(-1);
	}

/* Wait here until playing has started again */

	if (Send(Continue,1) < 0) {
		ShowAlert("Continue command failed!",NULL);
		return(-1);
	}

/* FIXUP: We send this to make sure the player is at normal speed */

	if (Send(Play,1) < 0) {
		ShowAlert("Continue command failed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* Command : pause the player */

int SonyStill()
{

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("VideoDisc Player not initialized",NULL);
		return(-1);
	}

/* See if we are in a reasonable mode */

	if (Send(StatusRequest,0) < 0) {
		ShowAlert("Player status request failed!",NULL);
		return(-1);
	}
	if (ReadPlayerStatus(buffer) < 0) {
		close(PlayerId);
		return(-1);
	}

/* Wait here for return confirmation */

	if (buffer[0] & SearchOrRepeatFlag) {
		if (Send(Still,1) < 0) {
			ShowAlert("Still command failed!",NULL);
			return(-1);
		}
	}
	else {
		if (Send(Clear,1) < 0) {
			ShowAlert("Still command failed!",NULL);
			return(-1);
		}
	}
	return(NULL);
}

/* Command : Stop all playback */

int SonyStop()
{

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("VideoDisc Player not initialized",NULL);
		return(-1);
	}

/* See if we are in a reasonable mode */

	if (Send(StatusRequest,0) < 0) {
		ShowAlert("Player status request failed!",NULL);
		return(-1);
	}
	if (ReadPlayerStatus(buffer) < 0) {
		close(PlayerId);
		return(-1);
	}

/* Wait here for return confirmation */

	if (buffer[0] & SearchOrRepeatFlag) {
		if (Send(Stop,1) < 0) {
			ShowAlert("Stop command failed!",NULL);
			return(-1);
		}
	}
	else {
		if (Send(Clear,1) < 0) {
			ShowAlert("Stop command failed!",NULL);
			return(-1);
		}
	}
	return(NULL);
}

/* Command : move 1 frame forward */

int SonyStepForward()
{

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("VideoDisc Player not initialized",NULL);
		return(-1);
	}

/* See if we are in a reasonable mode */

	if (Send(StatusRequest,0) < 0) {
		ShowAlert("Player status request failed!",NULL);
		return(-1);
	}
	if (ReadPlayerStatus(buffer) < 0) {
		close(PlayerId);
		return(-1);
	}

/* Take one step forward */

	if (Send(VarForward,1) < 0) {
		ShowAlert("Variable forward-play command failed!",NULL);
		return(-1);
	}
	if (Send(Still,1) < 0) {
		ShowAlert("Still command failed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* Move 1 frame backwards */

int SonyStepReverse()
{

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("VideoDisc Player not initialized",NULL);
		return(-1);
	}

/* See if we are in a reasonable mode */

	if (Send(StatusRequest,0) < 0) {
		ShowAlert("Player status request failed!",NULL);
		return(-1);
	}
	if (ReadPlayerStatus(buffer) < 0) {
		close(PlayerId);
		return(-1);
	}

/* Take one step back */

	if (Send(VarReverse,1) < 0) {
		ShowAlert("Variable reverse-play command failed!",NULL);
		return(-1);
	}
	if (Send(Still,1) < 0) {
		ShowAlert("Still command failed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* Command : move slowly forward */

int SonySlowForward()
{

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("VideoDisc Player not initialized",NULL);
		return(-1);
	}

/* See if we are in a reasonable mode */

	if (Send(StatusRequest,0) < 0) {
		ShowAlert("Player status request failed!",NULL);
		return(-1);
	}
	if (ReadPlayerStatus(buffer) < 0) {
		close(PlayerId);
		return(-1);
	}

/* Take slow steps forward */

	if (Send(SlowForward,1) < 0) {
		ShowAlert("Slow forward-play command failed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* Command : move slowly backwards */

int SonySlowReverse()
{

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("VideoDisc Player not initialized",NULL);
		return(-1);
	}

/* See if we are in a reasonable mode */

	if (Send(StatusRequest,0) < 0) {
		ShowAlert("Player status request failed!",NULL);
		return(-1);
	}
	if (ReadPlayerStatus(buffer) < 0) {
		close(PlayerId);
		return(-1);
	}

/* Take slow steps back */

	if (Send(SlowReverse,1) < 0) {
		ShowAlert("Variable reverse-play command failed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* Command : move fast forward */

int SonyFastForward()
{

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("VideoDisc Player not initialized",NULL);
		return(-1);
	}

/* See if we are in a reasonable mode */

	if (Send(StatusRequest,0) < 0) {
		ShowAlert("Player status request failed!",NULL);
		return(-1);
	}
	if (ReadPlayerStatus(buffer) < 0) {
		close(PlayerId);
		return(-1);
	}

/* Take fast steps forward */

	if (Send(FastForward,1) < 0) {
		ShowAlert("Fast forward-play command failed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* Command : move fast backwards */

int SonyFastReverse()
{

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("VideoDisc Player not initialized",NULL);
		return(-1);
	}

/* See if we are in a reasonable mode */

	if (Send(StatusRequest,0) < 0) {
		ShowAlert("Player status request failed!",NULL);
		return(-1);
	}
	if (ReadPlayerStatus(buffer) < 0) {
		close(PlayerId);
		return(-1);
	}

/* Take fast steps back */

	if (Send(FastReverse,1) < 0) {
		ShowAlert("Variable reverse-play command failed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* Command : Provide visual frame indicator */

int SonyIndex(status)
int status;
{

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("VideoDisc Player not initialized",NULL);
		return(-1);
	}

/* See if we are in a reasonable mode */

	if (Send(StatusRequest,0) < 0) {
		ShowAlert("Player status request failed!",NULL);
		return(-1);
	}
	if (ReadPlayerStatus(buffer) < 0) {
		close(PlayerId);
		return(-1);
	}

/* Turn on/off frame location display */

	if (status) {
		if (Send(IndexOn,1) < 0) {
			ShowAlert("Player frame display command failed!",NULL);
			return(-1);
		}
	}
	else {
		if (Send(IndexOff,1) < 0) {
			ShowAlert("Player frame display command failed!",NULL);
			return(-1);
		}
	}
	return(NULL);
}

/* Eject the video source */

int SonyEject()
{

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("VideoDisc Player not initialized",NULL);
		return(-1);
	}

/* See if we are in a reasonable mode */

	if (Send(StatusRequest,0) < 0) {
		ShowAlert("Player status request failed!",NULL);
		return(-1);
	}
	if (ReadPlayerStatus(buffer) < 0) {
		close(PlayerId);
		return(-1);
	}
	if (Send(Reject,1) < 0) {
		ShowAlert("Player ejection command failed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* end of sony.c */
