/*
 * (c) Copyright 1991 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log: vaa.c,v $
 * Revision 1.29  1994/12/14  02:09:53  drapeau
 * Minor cosmetic change to fix the title bar of the application
 * according to marketing's requirements.
 *
 * Revision 1.28  1994/12/13  17:41:46  drapeau
 * Minor cosmetic changes to increase the visibility of the corporate
 * name
 *
 * Revision 1.27  1994/04/19  16:56:29  drapeau
 * Fixed code in order to create a TrueColor X window for use with video
 * and compressed image data.
 *
 * Revision 1.26  1994/01/20  01:27:10  drapeau
 * Made inclusion of files more portable across Sun OS platforms.  Also
 * modified defined symbols to use in-house versions (e.g., "SVR4"
 * instead of "SOLARIS").
 *
 * Revision 1.25  1993/12/01  06:09:53  drapeau
 * Modified include statements to force compiler to look in
 * $PARALLAX_HOME/include instead of current directory.  The .h files in
 * the current directory should probably be deleted.
 *
 * Revision 1.24  1993/11/19  04:39:11  drapeau
 * Added support for SVR4.
 * Changes too numerous to describe here.
 *
 * Revision 1.7  91/08/20  20:47:52  janet
 * don't squeeze the charmin
 * 
 * Revision 1.6  91/08/19  16:15:27  eric
 * we don't need no sccs
 * 
 * Revision 1.5  91/08/19  13:09:49  eric
 * added rcs header
 * 
 *
 */

#ifndef lint
static char *rcsid = "@(#)$Id: vaa.c,v 1.29 1994/12/14 02:09:53 drapeau Exp $ Parallax Graphics Inc";
#endif

/*
 *
 *	vaa.c - Video Acquistion Application for Parallax Video Hardware
 *		aka videotool
 *
 *	Written by John Malleo-Roach
 *
 */

/* INCLUDE FILES */

#ifndef SVR4
#include	<a.out.h>
#endif
#include	<stdio.h>
#include	<math.h>
#include	<errno.h>
#include	<fcntl.h>
#include    <setjmp.h>
#include	<sys/ioctl.h>
#include	<sys/types.h>
#include	<sys/times.h>
#include	<sys/param.h>
#ifdef SVR4
#include	<sys/filio.h>
#endif
#include	<vaa.h>
#include	<audio.h>
#include	<video.h>
#include	<jpeg.h>

/* CONSTANTS */

#define BITS_WIDTH		32
#define BITS_HEIGHT		32

/* Define program icon and mask */

static short icon_bits[] = {
#include "videotool.icon"
};
static short mask_bits[] = {
#include "videotool.mask"
};

/* Define button glyphs for player control */

static short fastreverse_bits[] = {
#include "fastreverse.icon"
};
static short reverse_bits[] = {
#include "reverse.icon"
};
static short slowreverse_bits[] = {
#include "slowreverse.icon"
};
static short stepreverse_bits[] = {
#include "stepreverse.icon"
};
static short stop_bits[] = {
#include "stop.icon"
};
static short still_bits[] = {
#include "still.icon"
};
static short stepforward_bits[] = {
#include "stepforward.icon"
};
static short slowforward_bits[] = {
#include "slowforward.icon"
};
static short forward_bits[] = {
#include "forward.icon"
};
static short fastforward_bits[] = {
#include "fastforward.icon"
};
static short number_bits[] = {
#include "number.icon"
};
static short playsegment_bits[] = {
#include "playsegment.icon"
};
static short eject_bits[] = {
#include "eject.icon"
};
static short record_bits[] = {
#include "record.icon"
};

#define PANEL_DATA				1000
#define ITEM_DATA				1001

#define VAA_VIDEO				1
#define VAA_AUDIO				2
#define VAA_RUN					4
#define VAA_EXIT				8
#define VAA_STOP			   16
#define VAA_PLAY			   32
#define VAA_RECORD			   64
#define VAA_START			  128

/* Constants for the audio device */

#define DEFAULT_RECORD_GAIN  	50
#define DEFAULT_MONITOR_GAIN 	50

/* TYPEDEFS */

/*
 * Data that are passed from the sender to the receiver.
 */

typedef struct {
	XID     xid;
	int     x;
	int     y;
	Window  window;
	Atom    property;
} Drag_message;

/* STATIC VARIABLES */

static char version[] = "@(#)$Id: vaa.c,v 1.29 1994/12/14 02:09:53 drapeau Exp $";

static int PlayerModel=LIVE_VIDEO;	/* Player model we are using (defaults to live)	*/

static Panel VAAPanel;				/* Panel for main frame of VAA 					*/
static Panel_item ViewButton;		/* Pointer to panel item for View button		*/
static Panel_item OptionsButton;	/* Pointer to panel item for Options button		*/

static Xv_Window PlayerFrame;		/* SubFrame for player control					*/
static Panel PlayerPanel;			/* Panel for player control buttons				*/
static Panel_item ModelItem;		/* Pointer to panel item player model			*/
static Panel_item SpeedItem;		/* Pointer to panel item player speed			*/
static Panel_item DeviceItem;		/* Pointer to panel item device name			*/
static Panel_item StartItem;		/* Pointer to panel item for start frame		*/
static Panel_item EndItem;			/* Pointer to panel item for end frame			*/

static Xv_Window ConfigFrame;		/* SubFrame for Video configuration parameters	*/
static Panel ConfigPanel;			/* Panel for controlling Video config parameters*/
static Panel_item AspectItem;		/* Pointer to panel item aspect control			*/

static Xv_Window JPEGFrame;			/* SubFrame for JPEG configuration parameters	*/
static Panel JPEGPanel;				/* Panel for controlling JPEG config parameters */

static Xv_Window ColorFrame;		/* SubFrame for Video parameters				*/
static Panel ColorPanel;			/* Panel for controling Video parameters		*/

static Xv_Window AudioFrame;		/* SubFrame for Audio parameters				*/
static Panel AudioPanel;			/* Panel for controlling audio parameters		*/
static Canvas AudioCanvas;			/* Canvas for displaying audio volume			*/

static Panel OutputPanel;			/* Panel for output control buttons				*/
static Panel_item OSelectItem;		/* Pointer to panel item select region			*/
static Panel_item OOnOffItem;		/* Pointer to panel item video out on/off		*/

static Panel_item DirName;			/* Name of working directory					*/
static Panel_item FileName;			/* Name of requested file						*/

static int StartFrame=0;            /* Starting frame of video segment				*/
static int CurrentFrame=0;          /* Current frame of video segment				*/
static int EndFrame=0;              /* Ending frame of video segment				*/

static int VAAMode=0;				/* Status flag for VAA application				*/
static int CompressChoice=0;		/* Capture video and/or audio					*/
static int JPEGChoice=0;			/* Save with jpeg compression					*/
static int JPEGRate=0;				/* Frame rate for video capturing				*/

static char dirname[256];			/* Name of working directory					*/

#ifdef AUDIO
static Audio_hdr PlayHeader;		/* Audio play header for device					*/
#endif
static u_char AudioBuffer[8000];	/* Storage for audio data						*/

static struct timeval TimeIn;
static struct timeval TimeOut;

/* GLOBAL VARIABLES */

Display  *VAADisplay;				/* Local display, XView Screen and X11 Screen	*/
Xv_Screen VAAXVScreen;
int       VAAScreen;

Context *CurrentContext;			/* Pointer to current video context				*/
VideoOut VideoOutput;				/* Pointer to video output object				*/

Xv_Window VAAFrame;					/* Top level frame for VAA client program		*/
Xv_Window OutputFrame;				/* SubFrame for video output control			*/

Panel_item RecordVolume;			/* Pointer to panel item for record volume		*/
Panel_item MonitorVolume;			/* Pointer to panel item for monitor volume		*/
Panel_item FrameSpeed;				/* Pointer to panel item for frame speed		*/
Panel_item AudioOut;				/* Pointer to panel item for where audio goes	*/
Panel_item AudioIn;					/* Pointer to panel item for where audio goes	*/

Panel_item InterlaceButton;			/* Pointer to panel item for Deinterlace		*/
Panel_item VideoItem;				/* Pointer to panel item for live/still			*/
Panel_item FormatItem;				/* Pointer to panel item video format			*/
Panel_item TypeItem;				/* Pointer to panel item input type				*/
Panel_item FormatOutItem;			/* Pointer to panel item video output format	*/
Panel_item StandardOutItem;			/* Pointer to panel item video output standard	*/

Panel_item FileButton;				/* Pointer to panel item for File button		*/

Panel_item HueItem;					/* Pointer to panel item hue adjustment			*/
Panel_item BrightnessItem;			/* Pointer to panel item brightness adjustment	*/
Panel_item ContrastItem;			/* Pointer to panel item contrast adjustment	*/
Panel_item SaturationItem;			/* Pointer to panel item saturation adjustment	*/

/* EXTERNAL DEFINITIONS */

extern int optind;
extern char *optarg;

/* FUNCTIONS */

static int FrameDone(window)
Xv_Window window;
{
	double gain;

	if (CurrentContext) {
		xv_destroy_safe(CurrentContext->video);
#ifdef OW3
		xv_destroy_safe(CurrentContext->dropsite);
#endif
	}
	if (VAAMode & VAA_AUDIO) {
#ifdef AUDIO
		audio_pause_record(AudioFd);
		audio_flush_record(AudioFd);
		gain = 0.0;
		audio_set_monitor_gain(AudioFd,&gain);
		close(AudioFd);
#endif
		AudioFd = 0;
	}
	xv_destroy_safe(window);
	VAAMode = VAA_EXIT;
	return(XV_OK);
}

static int SubFrameDone(window)
Xv_Window window;
{
	Context *c;
	int channel;
	Menu menu;
	double gain;

	xv_set(window,WIN_SHOW,FALSE,NULL);

	if (c = (Context *) xv_get(window,XV_KEY_DATA,ITEM_DATA)) {
		VAAMode &= ~VAA_VIDEO;
		menu = (Menu) xv_get(ViewButton,PANEL_ITEM_MENU);
		channel = (int) xv_get(c->video,VIDEO_CHANNEL);
		switch (channel) {
		case VIDEO_IN_ONE:
			xv_set(xv_get(menu,MENU_NTH_ITEM,1,NULL),MENU_INACTIVE,FALSE,NULL);
			break;
		case VIDEO_IN_TWO:
			xv_set(xv_get(menu,MENU_NTH_ITEM,2,NULL),MENU_INACTIVE,FALSE,NULL);
			break;
		}
		if (c->player) {
			if (c->player->model == JPEG_FILE) PlayerEject(c->player);
			else if (c->player->fd) close(c->player->fd);
		}
		if (VAAMode & VAA_PLAY) jpeg_interrupt_movie(VAADisplay);
		xv_destroy_safe(c->video);
		xv_destroy_safe(c->frame);
#ifdef OW3
		xv_destroy_safe(c->dropsite);
#endif
		free(c);
		if (CurrentContext && (window == CurrentContext->frame)) {
			CurrentContext = NULL;
		}
	}
	else if (window == AudioFrame) {
		if (VAAMode & VAA_RUN) ;
		else CloseAudio(FALSE,TRUE);
		VAAMode &= ~VAA_AUDIO;
	}
	else if (window == OutputFrame) {
		xv_set(VideoOutput,VIDEO_OUTPUT,FALSE,NULL);
		menu    = (Menu) xv_get(ViewButton,PANEL_ITEM_MENU);
		channel = (int) xv_get(VideoOutput,VIDEO_OUT_CHANNEL);
		switch (channel) {
		case VIDEO_OUT_ONE:
			xv_set(xv_get(menu,MENU_NTH_ITEM,3,NULL),MENU_INACTIVE,FALSE,NULL);
			break;
		}
		xv_destroy_safe(VideoOutput);
		VideoOutput = 0;
		xv_set(OOnOffItem,PANEL_VALUE,0,NULL);
	}
	return(XV_OK);
}

Notify_value notice_quit(client,status)
Notify_client client;
Destroy_status status;
{
	switch (status) {
	case DESTROY_CHECKING:
		if (VAAMode & VAA_PLAY) jpeg_interrupt_movie(VAADisplay);
		if (VAAMode & VAA_RUN) notify_veto_destroy(client);
		else VAAMode |= VAA_EXIT;
		break;
	case DESTROY_CLEANUP:
		if (CurrentContext) {
			xv_destroy(CurrentContext->video);
#ifdef OW3
			xv_destroy(CurrentContext->dropsite);
#endif
		}
	case DESTROY_SAVE_YOURSELF:
	case DESTROY_PROCESS_DEATH:
		VAAMode |= VAA_EXIT;
		break;
	}
	return(NOTIFY_DONE);
}

/* Catch signals for emergency exit */

Notify_value SignalCatcher(client, sig, when)
Notify_client client;
int sig;
Notify_signal_mode when;
{
	double gain;

	if (VAAMode & VAA_AUDIO) {
		gain = 0.0;
#ifdef AUDIO
		audio_set_monitor_gain(AudioFd,&gain);
#endif
		CloseAudio(FALSE,TRUE);
	}
	if (CurrentContext) {
		xv_destroy(CurrentContext->video);
#ifdef OW3
		xv_destroy(CurrentContext->dropsite);
#endif
		if (CurrentContext->player) {
			if (CurrentContext->player->fd) close(CurrentContext->player->fd);
		}
		CurrentContext = NULL;
	}
	return(NOTIFY_DONE);
}
	
/* Pause for a while */

static nap(usec)
int usec;
{
	struct timeval timeout;
	 
	timeout.tv_sec = 0;
	timeout.tv_usec = usec;
	  
    (void) select(0, (fd_set *) 0, (fd_set *) 0, (fd_set *) 0, &timeout);
}  

/* Event function for laserdisc panel to handle all button presses */

static void PlayerPanelEvent(item,event)
Panel_item item;
Event *event;
{
	int who;

	if (CurrentContext == NULL) {
		ShowAlert("Please select a video input window for this operation",NULL);
		xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
		return;
	}
	who = (int) xv_get(item,XV_KEY_DATA,PANEL_DATA);
	switch (who) {
	case 3:
		if (event_is_down(event)) {
			PlayerSlowReverse(CurrentContext->player);
		}
		else {
			PlayerStill(CurrentContext->player);
		}
		break;
	case 5:
		if (event_is_down(event)) {
			PlayerSlowForward(CurrentContext->player);
		}
		else {
			PlayerStill(CurrentContext->player);
		}
		break;
	default:
		break;
	}
	panel_default_handle_event(item,event);
}

/* Notify function for Player panel to handle all button presses */

static void PlayerPanelProc(item, event)
Panel_item item;
Event *event;
{
	char buf[32];
	char *device;
	int speed,model;
	int who,what;

	if (CurrentContext == NULL) {
		ShowAlert("Please select a video input window for this operation",NULL);
		xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
		return;
	}
	who = (int) xv_get(item,XV_KEY_DATA,PANEL_DATA);
	
	switch (who) {
	case 1:
		PlayerFastReverse(CurrentContext->player);
		break;
	case 2:
		PlayerReverse(CurrentContext->player);
		break;
	case 3:
		break;
	case 4:
		PlayerStop(CurrentContext->player);
		break;
	case 5:
		break;
	case 6:
		PlayerPlay(CurrentContext->player);
		break;
	case 7:
		PlayerFastForward(CurrentContext->player);
		break;
	case 8:
		break;
	case 9:
		break;
	case 10:
		PlayerStepReverse(CurrentContext->player);
		break;
	case 11:
		PlayerStill(CurrentContext->player);
		break;
	case 12:
		PlayerStepForward(CurrentContext->player);
		break;
	case 13:
		StartFrame = (int) xv_get(StartItem,PANEL_VALUE);
		EndFrame   = (int) xv_get(EndItem,PANEL_VALUE);
		if (EndFrame < StartFrame) {
			EndFrame = StartFrame;
			xv_set(EndItem,PANEL_VALUE,EndFrame,NULL);
		}
		if (StartFrame == EndFrame) ShowDiscFrame(CurrentContext->player,StartFrame);
		else {
			if (event_shift_is_down(event))
				ShowDiscSegment(CurrentContext->player,StartFrame,EndFrame,TRUE);
			else ShowDiscSegment(CurrentContext->player,StartFrame,EndFrame,FALSE);
		}
		break;
	case 14:
		PlayerIndex(CurrentContext->player,TRUE);
		xv_set(item,XV_KEY_DATA,PANEL_DATA,15,NULL);
		break;
	case 15:
		PlayerIndex(CurrentContext->player,FALSE);
		xv_set(item,XV_KEY_DATA,PANEL_DATA,14,NULL);
		break;
	case 16:
		PlayerEject(CurrentContext->player);
		if (CurrentContext->player->model != JPEG_FILE) break;
/* If we have just ejected a JPEG file, just fall through to LIVE_VIDEO */
		xv_set(ModelItem,PANEL_VALUE,0,NULL);
		xv_set(SpeedItem,PANEL_INACTIVE,TRUE,NULL);
		xv_set(DeviceItem,PANEL_INACTIVE,TRUE,NULL);
	case 17:
		if (CurrentContext->player) {
			if (CurrentContext->player->model == JPEG_FILE) PlayerEject(CurrentContext->player);
			else if (CurrentContext->player->fd) close(CurrentContext->player->fd);
			free(CurrentContext->player);
			CurrentContext->player = NULL;
		}
		switch ((int) xv_get(ModelItem,PANEL_VALUE)) {
		case 0:
			model = LIVE_VIDEO;
			break;
		case 1:
			model = SONY_IP;
			break;
		case 2:
			model = SONY_VISCA;
			break;
		case 3:
			model = PIONEER;
			break;
		case 4:
			model = NEC;
			break;
		case 5:
			model = PANASONIC;
			break;
		case 6:
			model = JPEG_FILE;
			break;
		}
		speed  = (int) xv_get(SpeedItem,PANEL_VALUE);
		device = (char *) xv_get(DeviceItem,PANEL_VALUE);
		if (model == JPEG_FILE)
			CurrentContext->player = (Player *)
				InitializeVideo(device,model,((int) CurrentContext->video));
		else CurrentContext->player = (Player *) InitializeVideo(device,model,speed);
		if (CurrentContext->player == NULL) {
			ShowAlert("Unable to initialize video source","Open Failed",NULL);

/* If error occurs, default to live model */

			CurrentContext->player = (Player *) InitializeVideo("/dev/null",LIVE_VIDEO,NULL);
			xv_set(ModelItem,PANEL_VALUE,0,NULL);
			xv_set(SpeedItem,PANEL_VALUE,0,NULL);
			xv_set(DeviceItem,PANEL_VALUE,"/dev/null",NULL);
		}
		break;
	case 18:
/* Make sure we are not live (for now) */
		if (CurrentContext == NULL) {
			ShowAlert("Can't create JPEG movie file without video window",NULL);
			break;
		}
		if (CurrentContext->player->model == LIVE_VIDEO) {
			ShowAlert("Can't create JPEG movie file from live source",NULL);
			break;
		}
		if (CurrentContext->player->model == JPEG_FILE) {
			ShowAlert("Can't create JPEG movie file from JPEG source",NULL);
			break;
		}
/* Connect to the requested directory */
		strcpy(dirname,(char *) xv_get(DirName,PANEL_VALUE));
		if (chdir(dirname) < 0) {
			SystemError(errno,"Change working directory failed");
			break;
		}
/* Get the starting and ending frame values */
		StartFrame     = (int) xv_get(StartItem,PANEL_VALUE);
		EndFrame       = (int) xv_get(EndItem,PANEL_VALUE);
		if (StartFrame == EndFrame) {
			ShowNote("Use 'Save Still' to capture a single video frame",NULL);
			break;
		}
		CurrentFrame   = 0;
/* Discard any existing audio file */
		if (VAAMode & VAA_AUDIO) CloseAudio(FALSE,TRUE); sleep(1);
		unlink("AUDIO");
		if (CompressChoice & VAA_VIDEO) {
			ShowDiscFrame(CurrentContext->player,StartFrame);
			sprintf(buf,"Frame %d",StartFrame);
			xv_set(VAAFrame,FRAME_RIGHT_FOOTER,buf,NULL);
		}
/* Disable SAVE_STILL mode */
		xv_set(CurrentContext->video,VIDEO_SAVE_STILL,FALSE,NULL);
/* Let her rip! */
		VAAMode   |= VAA_RUN;
		break;
	case 19:
		what = (int) event;
		if (what == 0) {
			xv_set(SpeedItem,PANEL_INACTIVE,TRUE,NULL);
			xv_set(DeviceItem,PANEL_INACTIVE,TRUE,NULL);
		}
		else if (what == 6) {
			xv_set(SpeedItem,PANEL_INACTIVE,TRUE,NULL);
			xv_set(DeviceItem,PANEL_INACTIVE,FALSE,NULL);
			xv_set(DeviceItem,PANEL_LABEL_STRING,"File Name   : ",NULL);
		}
		else {
			xv_set(SpeedItem,PANEL_INACTIVE,FALSE,NULL);
			xv_set(DeviceItem,PANEL_INACTIVE,FALSE,NULL);
			xv_set(DeviceItem,PANEL_LABEL_STRING,"Device Name : ",NULL);
		}
		break;
	}
}

/* Create panel for controlling the video player on the screen */

static int CreatePlayerPanel()
{
	char *UserDefault;
	Server_image si;
	int i,j;

	PlayerFrame = xv_create(VAAFrame,FRAME_CMD,
		FRAME_LABEL,				"Video Device Control Panel",
		FRAME_DONE_PROC,			SubFrameDone,
		FRAME_CMD_PUSHPIN_IN,		TRUE,
		FRAME_SHOW_RESIZE_CORNER,	FALSE,
		FRAME_INHERIT_COLORS,		TRUE,
		XV_X,						300,
		XV_Y,						300,
		NULL);
	if (PlayerFrame == 0) {
		ShowAlert("xv_create of player frame failed!",NULL);
		exit(1);
	}
	PlayerPanel = (Panel) xv_get(PlayerFrame,FRAME_CMD_PANEL);
	if (PlayerPanel == 0) {
		ShowAlert("xv_create of player panel failed!",NULL);
		exit(1);
	}
	si = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,					BITS_WIDTH,
		XV_HEIGHT,					BITS_HEIGHT,
		SERVER_IMAGE_BITS,			fastreverse_bits,
		NULL);
	(void) xv_create(PlayerPanel, PANEL_BUTTON,
		PANEL_EVENT_PROC,			PlayerPanelEvent,
		PANEL_NOTIFY_PROC,			PlayerPanelProc,
		PANEL_LABEL_IMAGE,			si,
		XV_X,						xv_col(PlayerPanel,0),
		XV_Y,						xv_row(PlayerPanel,0),
		XV_KEY_DATA,				PANEL_DATA,			1,
		NULL);
	si = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,					BITS_WIDTH,
		XV_HEIGHT,					BITS_HEIGHT,
		SERVER_IMAGE_BITS,			reverse_bits,
		NULL);
	(void) xv_create(PlayerPanel, PANEL_BUTTON,
		PANEL_EVENT_PROC,			PlayerPanelEvent,
		PANEL_NOTIFY_PROC,			PlayerPanelProc,
		PANEL_LABEL_IMAGE,			si,
		XV_X,						xv_col(PlayerPanel,8),
		XV_Y,						xv_row(PlayerPanel,0),
		XV_KEY_DATA,				PANEL_DATA,			2,
		NULL);
	si = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,					BITS_WIDTH,
		XV_HEIGHT,					BITS_HEIGHT,
		SERVER_IMAGE_BITS,			slowreverse_bits,
		NULL);
	(void) xv_create(PlayerPanel, PANEL_BUTTON,
		PANEL_EVENT_PROC,			PlayerPanelEvent,
		PANEL_NOTIFY_PROC,			PlayerPanelProc,
		PANEL_LABEL_IMAGE,			si,
		XV_X,						xv_col(PlayerPanel,16),
		XV_Y,						xv_row(PlayerPanel,0),
		XV_KEY_DATA,				PANEL_DATA,			3,
		NULL);
	si = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,					BITS_WIDTH,
		XV_HEIGHT,					BITS_HEIGHT,
		SERVER_IMAGE_BITS,			stop_bits,
		NULL);
	(void) xv_create(PlayerPanel, PANEL_BUTTON,
		PANEL_EVENT_PROC,			PlayerPanelEvent,
		PANEL_NOTIFY_PROC,			PlayerPanelProc,
		PANEL_LABEL_IMAGE,			si,
		XV_X,						xv_col(PlayerPanel,24),
		XV_Y,						xv_row(PlayerPanel,0),
		XV_KEY_DATA,				PANEL_DATA,			4,
		NULL);
	si = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,					BITS_WIDTH,
		XV_HEIGHT,					BITS_HEIGHT,
		SERVER_IMAGE_BITS,			slowforward_bits,
		NULL);
	(void) xv_create(PlayerPanel, PANEL_BUTTON,
		PANEL_EVENT_PROC,			PlayerPanelEvent,
		PANEL_NOTIFY_PROC,			PlayerPanelProc,
		PANEL_LABEL_IMAGE,			si,
		XV_X,						xv_col(PlayerPanel,32),
		XV_Y,						xv_row(PlayerPanel,0),
		XV_KEY_DATA,				PANEL_DATA,			5,
		NULL);
	si = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,					BITS_WIDTH,
		XV_HEIGHT,					BITS_HEIGHT,
		SERVER_IMAGE_BITS,			forward_bits,
		NULL);
	(void) xv_create(PlayerPanel, PANEL_BUTTON,
		PANEL_EVENT_PROC,			PlayerPanelEvent,
		PANEL_NOTIFY_PROC,			PlayerPanelProc,
		PANEL_LABEL_IMAGE,			si,
		XV_X,						xv_col(PlayerPanel,40),
		XV_Y,						xv_row(PlayerPanel,0),
		XV_KEY_DATA,				PANEL_DATA,			6,
		NULL);
	si = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,					BITS_WIDTH,
		XV_HEIGHT,					BITS_HEIGHT,
		SERVER_IMAGE_BITS,			fastforward_bits,
		NULL);
	(void) xv_create(PlayerPanel, PANEL_BUTTON,
		PANEL_EVENT_PROC,			PlayerPanelEvent,
		PANEL_NOTIFY_PROC,			PlayerPanelProc,
		PANEL_LABEL_IMAGE,			si,
		XV_X,						xv_col(PlayerPanel,48),
		XV_Y,						xv_row(PlayerPanel,0),
		XV_KEY_DATA,				PANEL_DATA,			7,
		NULL);
	si = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,					BITS_WIDTH,
		XV_HEIGHT,					BITS_HEIGHT,
		SERVER_IMAGE_BITS,			eject_bits,
		NULL);
	(void) xv_create(PlayerPanel, PANEL_BUTTON,
		PANEL_EVENT_PROC,			PlayerPanelEvent,
		PANEL_NOTIFY_PROC,			PlayerPanelProc,
		PANEL_LABEL_IMAGE,			si,
		XV_X,						xv_col(PlayerPanel,0),
		XV_Y,						xv_row(PlayerPanel,1)+16,
		XV_KEY_DATA,				PANEL_DATA,			16,
		NULL);
	si = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,					BITS_WIDTH,
		XV_HEIGHT,					BITS_HEIGHT,
		SERVER_IMAGE_BITS,			record_bits,
		NULL);
	(void) xv_create(PlayerPanel, PANEL_BUTTON,
		PANEL_EVENT_PROC,			PlayerPanelEvent,
		PANEL_NOTIFY_PROC,			PlayerPanelProc,
		PANEL_LABEL_IMAGE,			si,
		XV_X,						xv_col(PlayerPanel,8),
		XV_Y,						xv_row(PlayerPanel,1)+16,
		XV_KEY_DATA,				PANEL_DATA,			18,
		NULL);
	si = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,					BITS_WIDTH,
		XV_HEIGHT,					BITS_HEIGHT,
		SERVER_IMAGE_BITS,			stepreverse_bits,
		NULL);
	(void) xv_create(PlayerPanel, PANEL_BUTTON,
		PANEL_EVENT_PROC,			PlayerPanelEvent,
		PANEL_NOTIFY_PROC,			PlayerPanelProc,
		PANEL_LABEL_IMAGE,			si,
		XV_X,						xv_col(PlayerPanel,16),
		XV_Y,						xv_row(PlayerPanel,1)+16,
		XV_KEY_DATA,				PANEL_DATA,			10,
		NULL);
	si = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,					BITS_WIDTH,
		XV_HEIGHT,					BITS_HEIGHT,
		SERVER_IMAGE_BITS,			still_bits,
		NULL);
	(void) xv_create(PlayerPanel, PANEL_BUTTON,
		PANEL_EVENT_PROC,			PlayerPanelEvent,
		PANEL_NOTIFY_PROC,			PlayerPanelProc,
		PANEL_LABEL_IMAGE,			si,
		XV_X,						xv_col(PlayerPanel,24),
		XV_Y,						xv_row(PlayerPanel,1)+16,
		XV_KEY_DATA,				PANEL_DATA,			11,
		NULL);
	si = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,					BITS_WIDTH,
		XV_HEIGHT,					BITS_HEIGHT,
		SERVER_IMAGE_BITS,			stepforward_bits,
		NULL);
	(void) xv_create(PlayerPanel, PANEL_BUTTON,
		PANEL_EVENT_PROC,			PlayerPanelEvent,
		PANEL_NOTIFY_PROC,			PlayerPanelProc,
		PANEL_LABEL_IMAGE,			si,
		XV_X,						xv_col(PlayerPanel,32),
		XV_Y,						xv_row(PlayerPanel,1)+16,
		XV_KEY_DATA,				PANEL_DATA,			12,
		NULL);
	si = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,					BITS_WIDTH,
		XV_HEIGHT,					BITS_HEIGHT,
		SERVER_IMAGE_BITS,			playsegment_bits,
		NULL);
	(void) xv_create(PlayerPanel, PANEL_BUTTON,
		PANEL_EVENT_PROC,			PlayerPanelEvent,
		PANEL_NOTIFY_PROC,			PlayerPanelProc,
		PANEL_LABEL_IMAGE,			si,
		XV_X,						xv_col(PlayerPanel,40),
		XV_Y,						xv_row(PlayerPanel,1)+16,
		XV_KEY_DATA,				PANEL_DATA,			13,
		NULL);
	si = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,					BITS_WIDTH,
		XV_HEIGHT,					BITS_HEIGHT,
		SERVER_IMAGE_BITS,			number_bits,
		NULL);
	(void) xv_create(PlayerPanel, PANEL_BUTTON,
		PANEL_EVENT_PROC,			PlayerPanelEvent,
		PANEL_NOTIFY_PROC,			PlayerPanelProc,
		PANEL_LABEL_IMAGE,			si,
		XV_X,						xv_col(PlayerPanel,48),
		XV_Y,						xv_row(PlayerPanel,1)+16,
		XV_KEY_DATA,				PANEL_DATA,			14,
		NULL);
	StartItem = xv_create(PlayerPanel,PANEL_NUMERIC_TEXT, 
		PANEL_LABEL_X,              xv_col(PlayerPanel,0),
		PANEL_LABEL_Y,              xv_row(PlayerPanel,3)+10,
		PANEL_VALUE_DISPLAY_LENGTH, 6,
		PANEL_LABEL_STRING,         "  Start Frame : ",
		PANEL_VALUE,                0,
		PANEL_MIN_VALUE,			0,
		PANEL_MAX_VALUE,			999999,
		XV_KEY_DATA,				PANEL_DATA,			0,
		NULL);
	EndItem = xv_create(PlayerPanel,PANEL_NUMERIC_TEXT, 
		PANEL_LABEL_X,              xv_col(PlayerPanel,30),
		PANEL_LABEL_Y,              xv_row(PlayerPanel,3)+10,
		PANEL_VALUE_DISPLAY_LENGTH, 6,
		PANEL_LABEL_STRING,         "End Frame : ",
		PANEL_VALUE,                0,
		PANEL_MIN_VALUE,			0,
		PANEL_MAX_VALUE,			999999,
		XV_KEY_DATA,				PANEL_DATA,			0,
		NULL);
	ModelItem = (Panel_item) xv_create(PlayerPanel, PANEL_CHOICE_STACK,
		PANEL_NOTIFY_PROC,			PlayerPanelProc,
		PANEL_LABEL_X,				xv_col(PlayerPanel,0),
		PANEL_LABEL_Y,				xv_row(PlayerPanel,4),
		PANEL_LABEL_STRING,			"Video Protocol : ",
		PANEL_CHOICE_STRINGS,		"Live",
									"Sony IP",
									"Sony VISCA",
									"Pioneer",
									"NEC",
									"Panasonic",
									"JPEG Movie",
									NULL,
		XV_KEY_DATA,				PANEL_DATA,			19,
		NULL);
	UserDefault = defaults_get_string("videotool.videoprotocol","Videotool.VideoProtocol","Live");
	i = 0;
	if (strcmp("Live",UserDefault) == 0)               i = 0;
	else if (strcmp("Sony IP",UserDefault) == 0)       i = 1;
	else if (strcmp("Sony VISCA",UserDefault) == 0)    i = 2;
	else if (strcmp("Pioneer",UserDefault) == 0)       i = 3;
	else if (strcmp("NEC",UserDefault) == 0)           i = 4;
	else if (strcmp("Panasonic",UserDefault) == 0)     i = 5;
	else if (strcmp("JPEG Movie",UserDefault) == 0)    i = 6;
	xv_set(ModelItem,PANEL_VALUE,i,NULL);
	SpeedItem = (Panel_item) xv_create(PlayerPanel, PANEL_CHOICE_STACK,
		PANEL_NOTIFY_PROC,			PlayerPanelProc,
		PANEL_LABEL_X,				xv_col(PlayerPanel,30),
		PANEL_LABEL_Y,				xv_row(PlayerPanel,4),
		PANEL_LABEL_STRING,			"Source Speed : ",
		PANEL_CHOICE_STRINGS,		"Default",
									"1200",
									"2400",
									"4800",
									"9600",
									NULL,
		PANEL_INACTIVE,				(i == 0),
		XV_KEY_DATA,				PANEL_DATA,			0,
		NULL);
	UserDefault = defaults_get_string("videotool.videospeed","Videotool.VideoSpeed","Default");
	j = 0;
	if (strcmp("Default",UserDefault) == 0)   j = 0;
	else if (strcmp("1200",UserDefault) == 0) j = 1;
	else if (strcmp("2400",UserDefault) == 0) j = 2;
	else if (strcmp("4800",UserDefault) == 0) j = 3;
	else if (strcmp("9600",UserDefault) == 0) j = 4;
	xv_set(SpeedItem,PANEL_VALUE,j,NULL);
	DeviceItem = (Panel_item) xv_create(PlayerPanel,PANEL_TEXT, 
		PANEL_LABEL_X,              xv_col(PlayerPanel,0),
		PANEL_LABEL_Y,              xv_row(PlayerPanel,5),
		PANEL_VALUE_DISPLAY_LENGTH, 40,
		PANEL_VALUE_STORED_LENGTH,	256,
		PANEL_LABEL_STRING,         ((i != 6) ? "Device Name : " : "File Name   : "),
		PANEL_VALUE,                "/dev/null",
		PANEL_INACTIVE,				(i == 0),
		XV_KEY_DATA,				PANEL_DATA,			0,
		NULL);
	UserDefault = defaults_get_string("videotool.videodevice","Videotool.VideoDevice","/dev/null");
	xv_set(DeviceItem,PANEL_VALUE,UserDefault,NULL);
	(void) xv_create(PlayerPanel, PANEL_BUTTON,
		PANEL_NOTIFY_PROC,			PlayerPanelProc,
		PANEL_LABEL_STRING,			"Initialize",
		XV_X,						xv_col(PlayerPanel,24),
		XV_Y,						xv_row(PlayerPanel,6),
		XV_KEY_DATA,				PANEL_DATA,			17,
		NULL);

	window_fit(PlayerPanel);
	window_fit(PlayerFrame);
	return(0);
}

/* Create panel for controlling the video configuration */

static int CreateConfigPanel()
{
	ConfigFrame = xv_create(VAAFrame,FRAME_CMD,
		FRAME_LABEL,				"Video Configuration Control Panel",
		FRAME_DONE_PROC,			SubFrameDone,
		FRAME_CMD_PUSHPIN_IN,		TRUE,
		FRAME_SHOW_RESIZE_CORNER,	FALSE,
		FRAME_INHERIT_COLORS,		TRUE,
		XV_X,						300,
		XV_Y,						300,
		NULL);
	if (ConfigFrame == 0) {
		ShowAlert("xv_create of configuration frame failed!",NULL);
		exit(1);
	}
	ConfigPanel = (Panel) xv_get(ConfigFrame,FRAME_CMD_PANEL);
	if (ConfigPanel == 0) {
		ShowAlert("xv_create of configuration panel failed!",NULL);
		exit(1);
	}
	VideoItem = (Panel_item) xv_create(ConfigPanel, PANEL_CHOICE,
		PANEL_NOTIFY_PROC,			ConfigPanelProc,
		PANEL_FEEDBACK,				PANEL_INVERTED,
		PANEL_LABEL_X,				xv_col(ConfigPanel,0),
		PANEL_LABEL_Y,				xv_row(ConfigPanel,1),
		PANEL_LABEL_STRING,			"Video Motion : ",
		PANEL_CHOICE_STRINGS,		" Live  ","    Still   ",NULL,
		XV_KEY_DATA,				PANEL_DATA,			1,
		NULL);
	(void) xv_create(ConfigPanel, PANEL_CHOICE,
		PANEL_NOTIFY_PROC,			ConfigPanelProc,
		PANEL_FEEDBACK,				PANEL_INVERTED,
		PANEL_LABEL_X,				xv_col(ConfigPanel,0),
		PANEL_LABEL_Y,				xv_row(ConfigPanel,2),
		PANEL_LABEL_STRING,			"Video Resize : ",
		PANEL_CHOICE_STRINGS,		" Crop  ","Squeeze",NULL,
		PANEL_VALUE,				1,
		XV_KEY_DATA,				PANEL_DATA,			2,
		NULL);
	(void) xv_create(ConfigPanel, PANEL_BUTTON,
		PANEL_NOTIFY_PROC,			ConfigPanelProc,
		PANEL_LABEL_STRING,			"Default Size",
		XV_X,						xv_col(ConfigPanel,32),
		XV_Y,						xv_row(ConfigPanel,1),
		XV_KEY_DATA,				PANEL_DATA,			3,
		NULL);
	AspectItem = (Panel_item) xv_create(ConfigPanel, PANEL_CHOICE,
		PANEL_NOTIFY_PROC,			ConfigPanelProc,
		PANEL_FEEDBACK,				PANEL_INVERTED,
		PANEL_LABEL_X,				xv_col(ConfigPanel,0),
		PANEL_LABEL_Y,				xv_row(ConfigPanel,3),
		PANEL_LABEL_STRING,			" Aspect Ratio : ",
		PANEL_CHOICE_STRINGS,		"  Any  ","  4 to 3  ",NULL,
		XV_KEY_DATA,				PANEL_DATA,			4,
		NULL);
	FormatItem = (Panel_item) xv_create(ConfigPanel, PANEL_CHOICE_STACK,
		PANEL_NOTIFY_PROC,			ConfigPanelProc,
		PANEL_LABEL_X,				xv_col(ConfigPanel,0),
		PANEL_LABEL_Y,				xv_row(ConfigPanel,4),
		PANEL_LABEL_STRING,			"Video Format : ",
		PANEL_CHOICE_STRINGS,		"Composite","SuperVHS","YUV","RGB",NULL,
		XV_KEY_DATA,				PANEL_DATA,			5,
		XV_KEY_DATA,				ITEM_DATA,			0,
		NULL);
	TypeItem = (Panel_item) xv_create(ConfigPanel, PANEL_CHOICE_STACK,
		PANEL_NOTIFY_PROC,			ConfigPanelProc,
		PANEL_LABEL_X,				xv_col(ConfigPanel,0),
		PANEL_LABEL_Y,				xv_row(ConfigPanel,5),
		PANEL_LABEL_STRING,			"   Input Type : ",
		PANEL_CHOICE_STRINGS,		"24 bit RGB","8 bit RGB",
			"8 bit GRAY","7 bit GRAY","7 bit YUV",NULL,
		XV_KEY_DATA,				PANEL_DATA,			6,
		XV_KEY_DATA,				ITEM_DATA,			0,
		NULL);
	InterlaceButton = (Panel_item) xv_create(ConfigPanel, PANEL_BUTTON,
		PANEL_NOTIFY_PROC,			ConfigPanelProc,
		PANEL_LABEL_STRING,			"De-interlace",
		PANEL_INACTIVE,				TRUE,
		XV_X,						xv_col(ConfigPanel,32),
		XV_Y,						xv_row(ConfigPanel,3),
		XV_KEY_DATA,				PANEL_DATA,			7,
		NULL);
	(void) xv_create(ConfigPanel, PANEL_BUTTON,
		PANEL_NOTIFY_PROC,			ConfigPanelProc,
		PANEL_LABEL_STRING,			"Quarter Size",
		XV_X,						xv_col(ConfigPanel,32),
		XV_Y,						xv_row(ConfigPanel,2),
		XV_KEY_DATA,				PANEL_DATA,			8,
		NULL);
	window_fit(ConfigPanel);
	window_fit(ConfigFrame);
	return(0);
}

void JPEGPanelProc(item, value, event)
Panel_item item;
int value;
Event *event;
{
	Xv_opaque parent;
	int who;

	if (CurrentContext == NULL) {
		ShowAlert("Please select a video input window for this operation",NULL);
		xv_set(item,PANEL_NOTIFY_STATUS,XV_ERROR,NULL);
		return;
	}
	who	   = (int) xv_get(item,XV_KEY_DATA,PANEL_DATA);
	parent = (Xv_opaque) xv_get(item,PANEL_PARENT_PANEL);

	xv_set(parent,FRAME_BUSY,TRUE,NULL);

	switch (who) {
	case 1:
		CompressChoice = value;
		break;
	case 2:
		JPEGRate = value;
		break;
	case 3:
		xv_set(CurrentContext->video,VIDEO_Q_FACTOR,value,NULL);
		break;
	case 4:
		xv_set(CurrentContext->video,VIDEO_JPEG_ZOOM,value,NULL);
		break;
	case 5:
		JPEGChoice = value + 1;
		break;
	}
	xv_set(parent,FRAME_BUSY,FALSE,NULL);
}

/* Create panel for controlling the JPEG compression configuration */

static int CreateJPEGPanel()
{
	JPEGFrame = xv_create(VAAFrame,FRAME_CMD,
		FRAME_LABEL,				"JPEG Compression Control Panel",
		FRAME_DONE_PROC,			SubFrameDone,
		FRAME_CMD_PUSHPIN_IN,		TRUE,
		FRAME_SHOW_RESIZE_CORNER,	FALSE,
		FRAME_INHERIT_COLORS,		TRUE,
		XV_X,						300,
		XV_Y,						300,
		NULL);
	if (JPEGFrame == 0) {
		ShowAlert("xv_create of JPEG compression frame failed!",NULL);
		exit(1);
	}
	JPEGPanel = (Panel) xv_get(JPEGFrame,FRAME_CMD_PANEL);
	if (JPEGPanel == 0) {
		ShowAlert("xv_create of JPEG compression panel failed!",NULL);
		exit(1);
	}
	CompressChoice = VAA_VIDEO | VAA_AUDIO;
	(void) xv_create(JPEGPanel,PANEL_TOGGLE,
		PANEL_NOTIFY_PROC,			JPEGPanelProc,
		PANEL_FEEDBACK,				PANEL_INVERTED,
		PANEL_LABEL_X,				xv_col(VAAPanel,0),
		PANEL_LABEL_Y,				xv_row(VAAPanel,1),
		PANEL_LABEL_STRING,			"     Compress : ",
		PANEL_CHOICE_STRINGS,		" Video "," Audio ",NULL,
		PANEL_VALUE,				CompressChoice,
		XV_KEY_DATA,				PANEL_DATA,			1,
		NULL);
	(void) xv_create(JPEGPanel, PANEL_CHOICE,
		PANEL_NOTIFY_PROC,			JPEGPanelProc,
		PANEL_FEEDBACK,				PANEL_INVERTED,
		PANEL_LABEL_X,				xv_col(JPEGPanel,0),
		PANEL_LABEL_Y,				xv_row(JPEGPanel,2),
		PANEL_LABEL_STRING,			"Capture Rate : ",
		PANEL_CHOICE_STRINGS,		"1/2","2/3","4/5","5/6","1/1",NULL,	
		PANEL_VALUE,				JPEGRate,
		XV_KEY_DATA,				PANEL_DATA,			2,
		NULL);
	(void) xv_create(JPEGPanel, PANEL_SLIDER,
		PANEL_NOTIFY_PROC,			JPEGPanelProc,
		PANEL_LABEL_X,				xv_col(JPEGPanel,0),
		PANEL_LABEL_Y,				xv_row(JPEGPanel,3),
		PANEL_MIN_VALUE,			30,
		PANEL_MAX_VALUE,			300,
		PANEL_VALUE,				150,
		PANEL_LABEL_STRING,			"Q Factor : ",
		PANEL_TICKS,				28,
		XV_KEY_DATA,				PANEL_DATA,			3,
		NULL);
	(void) xv_create(JPEGPanel, PANEL_CHOICE,
		PANEL_NOTIFY_PROC,			JPEGPanelProc,
		PANEL_FEEDBACK,				PANEL_INVERTED,
		PANEL_LABEL_X,				xv_col(JPEGPanel,0),
		PANEL_LABEL_Y,				xv_row(JPEGPanel,4),
		PANEL_LABEL_STRING,			"   Frame Size : ",
		PANEL_CHOICE_STRINGS,		" Current "," Full ",NULL,	
		PANEL_VALUE,				0,
		XV_KEY_DATA,				PANEL_DATA,			4,
		NULL);
	(void) xv_create(JPEGPanel, PANEL_CHOICE,
		PANEL_NOTIFY_PROC,			JPEGPanelProc,
		PANEL_FEEDBACK,				PANEL_INVERTED,
		PANEL_LABEL_X,				xv_col(JPEGPanel,0),
		PANEL_LABEL_Y,				xv_row(JPEGPanel,5),
		PANEL_LABEL_STRING,			"   File Format : ",
		PANEL_CHOICE_STRINGS,		" JPEG Movie "," JFIF ",NULL,	
		PANEL_VALUE,				0,
		XV_KEY_DATA,				PANEL_DATA,			5,
		NULL);
	window_fit(JPEGPanel);
	window_fit(JPEGFrame);
	return(0);
}

/* Create panel for controlling the video output configuration */

static int CreateOutputPanel()
{
	OutputFrame = xv_create(VAAFrame,FRAME_CMD,
		FRAME_LABEL,				"Video Output Control Panel",
		FRAME_DONE_PROC,			SubFrameDone,
		FRAME_CMD_PUSHPIN_IN,		TRUE,
		FRAME_SHOW_RESIZE_CORNER,	FALSE,
		FRAME_INHERIT_COLORS,		TRUE,
		FRAME_SHOW_FOOTER,			TRUE,
		XV_X,						300,
		XV_Y,						300,
		NULL);
	if (OutputFrame == 0) {
		ShowAlert("xv_create of video output frame failed!",NULL);
		exit(1);
	}
	OutputPanel = (Panel) xv_get(OutputFrame,FRAME_CMD_PANEL);
	if (OutputPanel == 0) {
		ShowAlert("xv_create of video output panel failed!",NULL);
		exit(1);
	}
	OOnOffItem = (Panel_item) xv_create(OutputPanel, PANEL_CHOICE,
		PANEL_NOTIFY_PROC,			OutputPanelProc,
		PANEL_FEEDBACK,				PANEL_INVERTED,
		PANEL_LABEL_X,				xv_col(OutputPanel,0),
		PANEL_LABEL_Y,				xv_row(OutputPanel,0),
		PANEL_LABEL_STRING,			"     Video Output : ",
		PANEL_CHOICE_STRINGS,		"   Off   ","   On   ",NULL,
		XV_KEY_DATA,				PANEL_DATA,			1,
		NULL);
	(void) xv_create(OutputPanel, PANEL_BUTTON,
		PANEL_NOTIFY_PROC,			OutputPanelProc,
		PANEL_LABEL_STRING,			"Configure",
		XV_X,						xv_col(OutputPanel,30),
		XV_Y,						xv_row(OutputPanel,4),
		XV_KEY_DATA,				PANEL_DATA,			2,
		NULL);
	(void) xv_create(OutputPanel, PANEL_CHOICE,
		PANEL_NOTIFY_PROC,			OutputPanelProc,
		PANEL_FEEDBACK,				PANEL_INVERTED,
		PANEL_LABEL_X,				xv_col(OutputPanel,0),
		PANEL_LABEL_Y,				xv_row(OutputPanel,1),
		PANEL_LABEL_STRING,			"      Output Size : ",
		PANEL_CHOICE_STRINGS,		"Scaled4to3","Fixed","Any",NULL,
		PANEL_VALUE,				2,
		XV_KEY_DATA,				PANEL_DATA,			3,
		NULL);
	OSelectItem = (Panel_item) xv_create(OutputPanel, PANEL_CHOICE,
		PANEL_NOTIFY_PROC,			OutputPanelProc,
		PANEL_FEEDBACK,				PANEL_INVERTED,
		PANEL_LABEL_X,				xv_col(OutputPanel,0),
		PANEL_LABEL_Y,				xv_row(OutputPanel,2),
		PANEL_LABEL_STRING,			"     Select Mode : ",
		PANEL_CHOICE_STRINGS,		"Window","Region",NULL,
		PANEL_VALUE,				1,
		XV_KEY_DATA,				PANEL_DATA,			4,
		NULL);
	FormatOutItem = (Panel_item) xv_create(OutputPanel, PANEL_CHOICE_STACK,
		PANEL_NOTIFY_PROC,			OutputPanelProc,
		PANEL_LABEL_X,				xv_col(OutputPanel,0),
		PANEL_LABEL_Y,				xv_row(OutputPanel,3),
		PANEL_LABEL_STRING,			"   Video Format : ",
		PANEL_CHOICE_STRINGS,		"Composite","SuperVHS","YUV","RGB",NULL,
		XV_KEY_DATA,				PANEL_DATA,			5,
		XV_KEY_DATA,				ITEM_DATA,			0,
		NULL);
	StandardOutItem = (Panel_item) xv_create(OutputPanel, PANEL_CHOICE_STACK,
		PANEL_NOTIFY_PROC,			OutputPanelProc,
		PANEL_LABEL_X,				xv_col(OutputPanel,0),
		PANEL_LABEL_Y,				xv_row(OutputPanel,4),
		PANEL_LABEL_STRING,			"Video Standard : ",
		PANEL_CHOICE_STRINGS,		"NTSC","PAL","SECAM",NULL,
		XV_KEY_DATA,				PANEL_DATA,			6,
		XV_KEY_DATA,				ITEM_DATA,			0,
		NULL);
	window_fit(OutputPanel);
	window_fit(OutputFrame);
	return(0);
}

/* Create panel for controlling the video colors */

static int CreateColorPanel()
{
	ColorFrame = xv_create(VAAFrame,FRAME_CMD,
		FRAME_LABEL,				"Video Color Control Panel",
		FRAME_DONE_PROC,			SubFrameDone,
		FRAME_CMD_PUSHPIN_IN,		TRUE,
		FRAME_SHOW_RESIZE_CORNER,	FALSE,
		FRAME_INHERIT_COLORS,		TRUE,
		XV_X,						300,
		XV_Y,						300,
		NULL);
	if (ColorFrame == 0) {
		ShowAlert("xv_create of color control frame failed!",NULL);
		exit(1);
	}
	ColorPanel = (Panel) xv_get(ColorFrame,FRAME_CMD_PANEL);
	if (ColorPanel == 0) {
		ShowAlert("xv_create of color control panel failed!",NULL);
		exit(1);
	}
	HueItem = (Panel_item) xv_create(ColorPanel, PANEL_SLIDER,
		PANEL_NOTIFY_PROC,			ColorPanelProc,
		PANEL_NOTIFY_LEVEL,         PANEL_ALL,
		PANEL_LABEL_X,				xv_col(ColorPanel,0),
		PANEL_LABEL_Y,				xv_row(ColorPanel,1),
		PANEL_MIN_VALUE,			0,
		PANEL_MAX_VALUE,			255,
		PANEL_VALUE,				128,
		PANEL_LABEL_STRING,			"            Hue : ",
		PANEL_TICKS,				17,
		PANEL_SLIDER_WIDTH,			150,
		XV_KEY_DATA,				PANEL_DATA,			1,
		NULL);
	BrightnessItem = (Panel_item) xv_create(ColorPanel, PANEL_SLIDER,
		PANEL_NOTIFY_PROC,			ColorPanelProc,
		PANEL_NOTIFY_LEVEL,         PANEL_ALL,
		PANEL_LABEL_X,				xv_col(ColorPanel,0),
		PANEL_LABEL_Y,				xv_row(ColorPanel,2),
		PANEL_MIN_VALUE,			0,
		PANEL_MAX_VALUE,			255,
		PANEL_VALUE,				128,
		PANEL_LABEL_STRING,			"Brightness : ",
		PANEL_TICKS,				17,
		PANEL_SLIDER_WIDTH,			150,
		XV_KEY_DATA,				PANEL_DATA,			2,
		NULL);
	ContrastItem = (Panel_item) xv_create(ColorPanel, PANEL_SLIDER,
		PANEL_NOTIFY_PROC,			ColorPanelProc,
		PANEL_NOTIFY_LEVEL,         PANEL_ALL,
		PANEL_LABEL_X,				xv_col(ColorPanel,0),
		PANEL_LABEL_Y,				xv_row(ColorPanel,3),
		PANEL_MIN_VALUE,			0,
		PANEL_MAX_VALUE,			255,
		PANEL_VALUE,				128,
		PANEL_LABEL_STRING,			"    Contrast : ",
		PANEL_TICKS,				17,
		PANEL_SLIDER_WIDTH,			150,
		XV_KEY_DATA,				PANEL_DATA,			3,
		NULL);
	SaturationItem = (Panel_item) xv_create(ColorPanel, PANEL_SLIDER,
		PANEL_NOTIFY_PROC,			ColorPanelProc,
		PANEL_NOTIFY_LEVEL,         PANEL_ALL,
		PANEL_LABEL_X,				xv_col(ColorPanel,0),
		PANEL_LABEL_Y,				xv_row(ColorPanel,4),
		PANEL_MIN_VALUE,			0,
		PANEL_MAX_VALUE,			255,
		PANEL_VALUE,				128,
		PANEL_LABEL_STRING,			" Saturation : ",
		PANEL_TICKS,				17,
		PANEL_SLIDER_WIDTH,			150,
		XV_KEY_DATA,				PANEL_DATA,			4,
		NULL);
	(void) xv_create(ColorPanel, PANEL_BUTTON,
		PANEL_NOTIFY_PROC,			ColorPanelProc,
		PANEL_LABEL_STRING,			"Factory",
		XV_X,						xv_col(ColorPanel,8),
		XV_Y,						xv_row(ColorPanel,6),
		XV_KEY_DATA,				PANEL_DATA,			5,
		NULL);
	(void) xv_create(ColorPanel, PANEL_BUTTON,
		PANEL_NOTIFY_PROC,			ColorPanelProc,
		PANEL_LABEL_STRING,			" Save  ",
		XV_X,						xv_col(ColorPanel,19),
		XV_Y,						xv_row(ColorPanel,6),
		XV_KEY_DATA,				PANEL_DATA,			6,
		NULL);
	(void) xv_create(ColorPanel, PANEL_BUTTON,
		PANEL_NOTIFY_PROC,			ColorPanelProc,
		PANEL_LABEL_STRING,			"Recall ",
		XV_X,						xv_col(ColorPanel,30),
		XV_Y,						xv_row(ColorPanel,6),
		XV_KEY_DATA,				PANEL_DATA,			7,
		NULL);
	window_fit(ColorPanel);
	window_fit(ColorFrame);
	return(0);
}

/* Create panel for controlling the audio source on the screen */

static int CreateAudioPanel()
{
	Xv_Font font;
	XGCValues gcv;
	int height;

	AudioFrame = xv_create(VAAFrame,FRAME_CMD,
		FRAME_LABEL,				"Audio Control Panel",
		FRAME_DONE_PROC,			SubFrameDone,
		FRAME_CMD_PUSHPIN_IN,		TRUE,
		FRAME_SHOW_RESIZE_CORNER,	FALSE,
		FRAME_INHERIT_COLORS,		TRUE,
		XV_X,						300,
		XV_Y,						300,
		NULL);

	if (AudioFrame == 0) {
		ShowAlert("xv_create of audio frame failed!",NULL);
		exit(1);
	}

	AudioPanel = (Panel) xv_get(AudioFrame,FRAME_CMD_PANEL);

	if (AudioPanel == 0) {
		ShowAlert("xv_create of audio panel failed!",NULL);
		exit(1);
	}
	AudioOut = xv_create(AudioPanel, PANEL_CHOICE_STACK,
		PANEL_NOTIFY_PROC,			AudioPanelProc,
		PANEL_LAYOUT,				PANEL_VERTICAL,
		PANEL_LABEL_X,				xv_col(AudioPanel,0),
		PANEL_LABEL_Y,				xv_row(AudioPanel,0),
		PANEL_LABEL_STRING,			"Audio Out",
		PANEL_CHOICE_STRINGS,		"Speaker","External",NULL,
		XV_KEY_DATA,				PANEL_DATA,			1,
		NULL);
	FrameSpeed = xv_create(AudioPanel, PANEL_NUMERIC_TEXT,
		PANEL_LAYOUT,				PANEL_VERTICAL,
		PANEL_LABEL_X,				xv_col(AudioPanel,0),
		PANEL_LABEL_Y,				xv_row(AudioPanel,2),
		PANEL_LABEL_STRING,			"Frame Rate",
		PANEL_VALUE_DISPLAY_LENGTH,	2,
		PANEL_VALUE,				30,
		PANEL_MAX_VALUE,			30,
		NULL);
	AudioIn = xv_create(AudioPanel, PANEL_CHOICE_STACK,
		PANEL_NOTIFY_PROC,			AudioPanelProc,
		PANEL_LAYOUT,				PANEL_VERTICAL,
		PANEL_LABEL_X,				xv_col(AudioPanel,0),
		PANEL_LABEL_Y,				xv_row(AudioPanel,4),
		PANEL_LABEL_STRING,			"Audio In",
		PANEL_CHOICE_STRINGS,		"Microphone","Line In",NULL,
		XV_KEY_DATA,				PANEL_DATA,			4,
		NULL);
	RecordVolume = xv_create(AudioPanel, PANEL_SLIDER,
		PANEL_NOTIFY_PROC,			AudioPanelProc,
		PANEL_NOTIFY_LEVEL,         PANEL_ALL,
		PANEL_LABEL_X,				xv_col(AudioPanel,15),
		PANEL_LABEL_Y,				xv_row(AudioPanel,0),
		PANEL_MIN_VALUE,			0,
		PANEL_MAX_VALUE,			100,
		PANEL_VALUE,				DEFAULT_RECORD_GAIN,
		PANEL_DIRECTION,			PANEL_VERTICAL,
		PANEL_SLIDER_END_BOXES,		TRUE,
		PANEL_TICKS,				11,
		PANEL_SHOW_VALUE,			FALSE,
		XV_KEY_DATA,				PANEL_DATA,			2,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"R",
		PANEL_LABEL_X,				xv_col(AudioPanel,12),
		PANEL_LABEL_Y,				xv_row(AudioPanel,1)+16,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"E",
		PANEL_LABEL_X,				xv_col(AudioPanel,12),
		PANEL_LABEL_Y,				xv_row(AudioPanel,1)+28,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"C",
		PANEL_LABEL_X,				xv_col(AudioPanel,12),
		PANEL_LABEL_Y,				xv_row(AudioPanel,1)+40,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"O",
		PANEL_LABEL_X,				xv_col(AudioPanel,12),
		PANEL_LABEL_Y,				xv_row(AudioPanel,1)+52,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"R",
		PANEL_LABEL_X,				xv_col(AudioPanel,12),
		PANEL_LABEL_Y,				xv_row(AudioPanel,1)+64,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"D",
		PANEL_LABEL_X,				xv_col(AudioPanel,12),
		PANEL_LABEL_Y,				xv_row(AudioPanel,1)+76,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);
	MonitorVolume = xv_create(AudioPanel, PANEL_SLIDER,
		PANEL_NOTIFY_PROC,			AudioPanelProc,
		PANEL_NOTIFY_LEVEL,         PANEL_ALL,
		PANEL_LABEL_X,				xv_col(AudioPanel,25),
		PANEL_LABEL_Y,				xv_row(AudioPanel,0),
		PANEL_MIN_VALUE,			0,
		PANEL_MAX_VALUE,			100,
		PANEL_VALUE,				DEFAULT_MONITOR_GAIN,
		PANEL_DIRECTION,			PANEL_VERTICAL,
		PANEL_SLIDER_END_BOXES,		TRUE,
		PANEL_TICKS,				11,
		PANEL_SHOW_VALUE,			FALSE,
		XV_KEY_DATA,				PANEL_DATA,			3,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"M",
		PANEL_LABEL_X,				xv_col(AudioPanel,22),
		PANEL_LABEL_Y,				xv_row(AudioPanel,1)+12,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"O",
		PANEL_LABEL_X,				xv_col(AudioPanel,22),
		PANEL_LABEL_Y,				xv_row(AudioPanel,1)+24,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"N",
		PANEL_LABEL_X,				xv_col(AudioPanel,22),
		PANEL_LABEL_Y,				xv_row(AudioPanel,1)+36,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"I",
		PANEL_LABEL_X,				xv_col(AudioPanel,22)+2,
		PANEL_LABEL_Y,				xv_row(AudioPanel,1)+48,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"T",
		PANEL_LABEL_X,				xv_col(AudioPanel,22),
		PANEL_LABEL_Y,				xv_row(AudioPanel,1)+60,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"O",
		PANEL_LABEL_X,				xv_col(AudioPanel,22),
		PANEL_LABEL_Y,				xv_row(AudioPanel,1)+72,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);
	(void) xv_create(AudioPanel,PANEL_MESSAGE,
		PANEL_LABEL_STRING,			"R",
		PANEL_LABEL_X,				xv_col(AudioPanel,22),
		PANEL_LABEL_Y,				xv_row(AudioPanel,1)+84,
		PANEL_LABEL_BOLD,			TRUE,
		NULL);

	window_fit(AudioPanel);

	height = xv_get(AudioPanel,XV_HEIGHT);
	AudioCanvas = xv_create(AudioFrame,CANVAS,
		CANVAS_WIDTH,				70,
		CANVAS_HEIGHT,				height,
		CANVAS_RETAINED,			FALSE,
		CANVAS_REPAINT_PROC,		AudioRepaintProc,
		XV_Y,						0,
		XV_WIDTH,					70,
		XV_HEIGHT,					height,
		WIN_RIGHT_OF,				AudioPanel,
		NULL);


	window_fit(AudioCanvas);
	window_fit(AudioFrame);

	AudioWidth     = (int) xv_get(AudioCanvas,XV_WIDTH);
	AudioHeight    = (int) xv_get(AudioCanvas,XV_HEIGHT);
	AudioXID       = (Window) xv_get(canvas_paint_window(AudioCanvas),XV_XID);
	AudioLevel	   = 0.0;
	font = (Xv_Font) xv_find(AudioFrame,FONT,
		FONT_FAMILY,		FONT_FAMILY_DEFAULT,
		FONT_STYLE,			FONT_STYLE_BOLD,
		NULL);

	gcv.function   = GXcopy;
	gcv.font       = (Font) xv_get(font,XV_XID);
	gcv.foreground = WhitePixel(VAADisplay,VAAScreen);
	gcv.background = BlackPixel(VAADisplay,VAAScreen);
	AudioGCW       = XCreateGC(VAADisplay,AudioXID,
						GCFunction|GCForeground|GCBackground,&gcv);
	gcv.foreground = BlackPixel(VAADisplay,VAAScreen);
	gcv.background = WhitePixel(VAADisplay,VAAScreen);
	AudioGCB       = XCreateGC(VAADisplay,AudioXID,
						GCFunction|GCForeground|GCBackground|GCFont,&gcv);

/* An index into the colormap which is our purest red */

	gcv.foreground = 12;
	AudioGCR       = XCreateGC(VAADisplay,AudioXID,
						GCFunction|GCForeground|GCBackground,&gcv);

	return(0);
}
#ifdef OW3
static void OW3DropHandler(canvas, event, arg)
Xv_Window canvas;
Event *event;
Notify_arg arg;
{
	int length;
	int format;
	char *data=NULL;
	Xv_drop_site ds;
	char value[1024];
	Selection_requestor selector;

	switch (event_action(event)) {
	case ACTION_DRAG_MOVE:
	case ACTION_DRAG_COPY:
		selector = xv_create(CurrentContext->video, SELECTION_REQUESTOR, NULL);
		if ((ds = dnd_decode_drop(selector, event)) != XV_ERROR) {
			xv_set(selector, SEL_TYPE_NAME, "FILE_NAME", NULL);
			data = (char *) xv_get(selector, SEL_DATA, &length, &format);
		}
		dnd_done(selector);
		if (data) {
			value[0] = 0;
			strcpy(value,data);
			data = value + strlen(value);
			while (*data != '/') data--;
			*data = '\0'; data++;
			xv_set(DirName,PANEL_VALUE,value,NULL);
			xv_set(FileName,PANEL_VALUE,data,NULL);
			if (CurrentContext) LoadStillFrame(value,data);
		}
	}
}
#endif
/* Handle drag and drops into video window */

static void DragHandler(canvas, event, arg)
Xv_Window canvas;
Event *event;
Notify_arg arg;
{
	Drag_message *msg           = (Drag_message *) xv_get(canvas, WIN_MESSAGE_DATA);
	Display      *display       = (Display *) XV_DISPLAY_FROM_WINDOW(canvas);
	long          long_offset   = 0L;
	long          long_length   = (long) (MAXPATHLEN / 4);
	int           delete_flag   = TRUE;
	Atom          actual_type;
	int           actual_format;
	u_long        nitems;
	u_long        bytes_after;
	u_char       *prop=NULL;
	char         *data=NULL;
	char          value[1024];
	Xv_Window     frame;
	int           channel;
	Rect          rect;
	float         e;
	int           i;

	switch (event_action(event)) {
	case ACTION_DRAG_LOAD:
		if (msg == NULL || msg->property == None) return;
		if (XGetWindowProperty(display, msg->window, msg->property,
							   long_offset, long_length, delete_flag,
							   AnyPropertyType, &actual_type, &actual_format,
							   &nitems, &bytes_after, &prop) != Success)
			return;
		if (prop) {
			value[0] = 0;
			strcpy(value,prop);
			XFree(prop);
			data = value + strlen(value);
			while (*data != '/') data--;
			*data = '\0'; data++;
			xv_set(DirName,PANEL_VALUE,value,NULL);
			xv_set(FileName,PANEL_VALUE,data,NULL);
			if (CurrentContext) LoadStillFrame(value,data);
		}
		break;
	case KBD_USE:
		frame = (Xv_Window) xv_get(canvas,XV_OWNER);
		frame = (Xv_Window) xv_get(frame,XV_OWNER);
		CurrentContext = (Context *) xv_get(frame,XV_KEY_DATA,ITEM_DATA);
		channel = (int) xv_get(CurrentContext->video,VIDEO_CHANNEL);
		switch (channel) {
		case VIDEO_IN_ONE:
			sprintf(value,"Input 1");
			break;
		case VIDEO_IN_TWO:
			sprintf(value,"Input 2");
			break;
		}
		xv_set(VAAFrame,FRAME_RIGHT_FOOTER,value,NULL);
		break;
	case ACTION_ADJUST:
		if (event_is_down(event)) {
			if (VAAMode & VAA_RECORD) {
				VAAMode |= VAA_START;
				if (VAAMode & VAA_AUDIO)  CloseAudio(FALSE,TRUE);
				xv_set(CurrentContext->video,
					   VIDEO_SAVE_STILL,     FALSE,
					   VIDEO_USE_SHM,	     TRUE,
					   NULL);
				xv_set(VAAFrame,FRAME_RIGHT_FOOTER,"Movie Started",NULL);
				gettimeofday(&TimeIn,NULL);
			}
		}
		else {
			if (VAAMode & VAA_RECORD) {
				VAAMode &= ~VAA_START;
				VAAMode &= ~VAA_RECORD;
				jpeg_stop_movie(VAADisplay);
				CloseAudio(FALSE,FALSE);
				if (VAAMode & VAA_AUDIO) OpenAudio(FALSE,TRUE);
				xv_set(CurrentContext->video,
					   VIDEO_LIVE,		     TRUE,
					   VIDEO_SAVE_STILL,     TRUE,
					   VIDEO_USE_SHM,	     FALSE,
					   NULL);
				xv_set(VAAFrame,FRAME_RIGHT_FOOTER,"Movie Completed",NULL);
				gettimeofday(&TimeOut,NULL);
				i  = TimeOut.tv_sec * 1000000 + TimeOut.tv_usec;
				i -= TimeIn.tv_sec  * 1000000 + TimeIn.tv_usec;
				e  = 1000000.0 / (((float) i) / ((float) CurrentFrame));
				sprintf(value,"%d frames @ %2.2f fps, ",CurrentFrame,e);
				xv_set(VAAFrame,FRAME_LEFT_FOOTER,value,NULL);
			}
		}
		break;
#ifdef OW3
	case WIN_RESIZE:
		rect.r_top    = 0;
		rect.r_left   = 0;
		rect.r_width  = (int) xv_get(CurrentContext->video,VIDEO_WIDTH);
		rect.r_height = (int) xv_get(CurrentContext->video,VIDEO_HEIGHT);
		xv_set(CurrentContext->dropsite,DROP_SITE_REGION,&rect,NULL);
#endif	
		break;
	}
}

/* Create canvas for displaying video frames from the video board */

static int CreateVideoInput(channel)
int channel;
{
	int h;
#ifdef OW3
	Rect rect;
#endif
	Menu menu;
	Context *c;

	c = (Context *) calloc(1,sizeof(Context));

	c->frame = xv_create(VAAFrame,FRAME_CMD,
		FRAME_DONE_PROC,			SubFrameDone,
		FRAME_SHOW_RESIZE_CORNER,	TRUE,
		FRAME_CMD_PUSHPIN_IN,		TRUE,
		FRAME_LABEL,				(channel == VIDEO_IN_ONE ?
							 "Parallax Graphics: Input 1" :
							 "Parallax Graphics: Input 2"),
/*
#ifdef OW3
		XV_VISUAL_CLASS,            TrueColor,
		WIN_DEPTH,                  
						 (strncmp(XServerVendor(VAADisplay),"X11/NeWS",8) ? 32 : 24),
#endif
*/
		XV_WIDTH,					768,
		XV_HEIGHT,					576,
		XV_X,						200,
		XV_Y,						200,
		NULL);

	if (c->frame == 0) {
		ShowAlert("xv_create of video frame failed!",NULL);
		exit(1);
	}

/* Destroy the panel we get for free (but not needed here) */

	xv_destroy(xv_get(c->frame,FRAME_CMD_PANEL));

/* Create a video canvas for display off the board */

	c->video = xv_create(c->frame,VIDEO_IN,
#ifdef OW3
		WIN_EVENT_PROC,         OW3DropHandler,
#endif
			     XV_VISUAL_CLASS,	TrueColor,
		VIDEO_CHANNEL,			channel,
		VIDEO_SQUEEZE,			TRUE,
		VIDEO_SAVE_STILL,       TRUE,
		NULL);

	if (c->video == 0) {
		if (Xv_VideoStatus == VIDEO_CHANNEL_IN_USE) {
			ShowAlert("Requested channel already in use",NULL);
		}
		else if (Xv_VideoStatus == VIDEO_NO_MORE_CHANNELS) {
			ShowAlert("Requested channel is not available",NULL);
		}
        else if (Xv_VideoStatus == VIDEO_NO_SYNC_DETECTED) {
			ShowAlert("No Sync Detected - Check Input Connection ",NULL);
		}
		else {
			ShowAlert("xv_create of video canvas failed!",NULL);
		}
		xv_destroy(c->frame);
		free(c); return(0);
	}
	if (xv_get(c->video,VIDEO_CHANNEL) != channel) {
		ShowAlert("No Sync Detected - Check Input Connection ",NULL);
		xv_destroy(c->video);
		xv_destroy(c->frame);
		free(c); return(0);
	}
#ifdef OW3
	rect.r_top    = 0;
	rect.r_left   = 0;
	rect.r_width  = (int) xv_get(c->video,VIDEO_WIDTH);
	rect.r_height = (int) xv_get(c->video,VIDEO_HEIGHT);
	c->dropsite   = xv_create(c->video, DROP_SITE_ITEM,
			  DROP_SITE_ID,     c->video,
			  DROP_SITE_REGION, &rect,
			  NULL);
#endif	
/* Attach an event handler to the video in object */

	xv_set(c->video,VIDEO_EVENT_PROC,DragHandler,NULL);

/* What kind of board is this */

	h = (int) xv_get(c->video,VIDEO_HARDWARE);
	switch (h) {
	case VIDEO_1280:
		xv_set(VAAFrame,FRAME_LEFT_FOOTER,"1280",NULL);
		break;
	case VIDEO_VIPER:
		xv_set(VAAFrame,FRAME_LEFT_FOOTER,"Viper",NULL);
		break;
	case VIDEO_VIDEOVIEW:
		xv_set(VAAFrame,FRAME_LEFT_FOOTER,"VideoView",NULL);
		break;
	case VIDEO_XVIDEO_24:
		xv_set(VAAFrame,FRAME_LEFT_FOOTER,"XVideo-24",NULL);
		break;
	case VIDEO_XVIDEO_8:
		xv_set(VAAFrame,FRAME_LEFT_FOOTER,"XVideo-8",NULL);
		break;
	default:
		xv_set(VAAFrame,FRAME_LEFT_FOOTER,"Unknown",NULL);
		break;
	}

/* Initialize a default video source for this channel */

	c->player = (Player *) InitializeVideo("/dev/null",LIVE_VIDEO,NULL);

/* Disable the menu item that got us here */

	menu = (Menu) xv_get(ViewButton,PANEL_ITEM_MENU);
	switch (channel) {
	case VIDEO_IN_ONE:
		xv_set(xv_get(menu,MENU_NTH_ITEM,1,NULL),MENU_INACTIVE,TRUE,NULL);
		break;
	case VIDEO_IN_TWO:
		xv_set(xv_get(menu,MENU_NTH_ITEM,2,NULL),MENU_INACTIVE,TRUE,NULL);
		break;
	}
	xv_set(c->frame,WIN_SHOW,TRUE,NULL);

	CurrentContext = c;
	xv_set(c->video,XV_KEY_DATA,ITEM_DATA,c,NULL);
	xv_set(c->frame,XV_KEY_DATA,ITEM_DATA,c,NULL);	
	return(0);
}

/* Create canvas for displaying video frames from the video board */

static int CreateVideoOutput(channel)
int channel;
{
	Menu menu;
	int h,value;

/* Create a video output channel for display off the board */

	VideoOutput = xv_create(XV_NULL,VIDEO_OUT,
		VIDEO_OUT_CHANNEL,		channel,
		VIDEO_OUT_ASPECT,		FALSE,
		VIDEO_OUT_SCALE,		TRUE,
		NULL);

	if (VideoOutput == 0) {
		if (Xv_VideoStatus == VIDEO_CHANNEL_IN_USE) {
			ShowAlert("Requested channel already in use",NULL);
		}
		else if (Xv_VideoStatus == VIDEO_NO_MORE_CHANNELS) {
			ShowAlert("Requested channel is not available",NULL);
		}
		else {
			ShowAlert("xv_create of video output failed!",NULL);
		}
		return(0);
	}

/* Disable the menu item that got us here */

	menu = (Menu) xv_get(ViewButton,PANEL_ITEM_MENU);
	switch (channel) {
	case VIDEO_OUT_ONE:
		xv_set(xv_get(menu,MENU_NTH_ITEM,3,NULL),MENU_INACTIVE,TRUE,NULL);
		break;
	}
	value = (int) xv_get(VideoOutput,VIDEO_OUT_FORMAT);
	switch (value) {
	case VIDEO_COMPOSITE:
		xv_set(FormatOutItem,PANEL_VALUE,0,NULL);
		xv_set(FormatOutItem,XV_KEY_DATA,ITEM_DATA,0,NULL);
		break;
	case VIDEO_SUPER_VHS:
		xv_set(FormatOutItem,PANEL_VALUE,1,NULL);
		xv_set(FormatOutItem,XV_KEY_DATA,ITEM_DATA,1,NULL);
		break;
	case VIDEO_YUV:
		xv_set(FormatOutItem,PANEL_VALUE,2,NULL);
		xv_set(FormatOutItem,XV_KEY_DATA,ITEM_DATA,2,NULL);
		break;
	case VIDEO_RGB:
		xv_set(FormatOutItem,PANEL_VALUE,3,NULL);
		xv_set(FormatOutItem,XV_KEY_DATA,ITEM_DATA,3,NULL);
		break;
	default:
		xv_set(FormatOutItem,PANEL_VALUE,0,NULL);
		xv_set(FormatOutItem,XV_KEY_DATA,ITEM_DATA,0,NULL);
		break;
	}
	value = (int) xv_get(VideoOutput,VIDEO_OUT_STANDARD);
	switch (value) {
	case VIDEO_NTSC:
		xv_set(StandardOutItem,PANEL_VALUE,0,NULL);
		xv_set(StandardOutItem,XV_KEY_DATA,ITEM_DATA,0,NULL);
		break;
	case VIDEO_PAL:
		xv_set(StandardOutItem,PANEL_VALUE,1,NULL);
		xv_set(StandardOutItem,XV_KEY_DATA,ITEM_DATA,1,NULL);
		break;
	case VIDEO_SECAM:
		xv_set(StandardOutItem,PANEL_VALUE,2,NULL);
		xv_set(StandardOutItem,XV_KEY_DATA,ITEM_DATA,2,NULL);
		break;
	default:
		xv_set(StandardOutItem,PANEL_VALUE,0,NULL);
		xv_set(StandardOutItem,XV_KEY_DATA,ITEM_DATA,0,NULL);
		break;
	}
	return(1);
}

/* Capture the specified video segement for the video source */

static int CaptureVideo()
{
	int speed;
	JPEGImage *image;

/* Open new movie file */

	if (CurrentFrame == StartFrame) {
		speed = (int) xv_get(FrameSpeed,PANEL_VALUE);
		switch (JPEGRate) {
		case 0:
			speed = (speed * 1) / 2; break;
		case 1:
			speed = (speed * 2) / 3; break;
		case 2:
			speed = (speed * 4) / 5; break;
		case 3:
			speed = (speed * 5) / 6; break;
		}
		if (CompressChoice & VAA_AUDIO) {
			if ((AudioFd = open("AUDIO",O_RDONLY)) < 0) {
				ShowAlert("Unable to access cached audio, movie failed",NULL);
				CurrentFrame = EndFrame + 1;
				return(FALSE);
			}
			if (jpeg_open_movie(VAADisplay,xv_get(FileName,PANEL_VALUE),speed,
								(int) xv_get(CurrentContext->video,VIDEO_Q_FACTOR),
								&PlayHeader)) {
				ShowAlert("Unable to open movie file, aborting...",NULL);
				CurrentFrame = EndFrame + 1;
				return(FALSE);
			}
		}
		else {
			if (jpeg_open_movie(VAADisplay,xv_get(FileName,PANEL_VALUE),speed,
								(int) xv_get(CurrentContext->video,VIDEO_Q_FACTOR),
								NULL)) {
				ShowAlert("Unable to open movie file, aborting...",NULL);
				CurrentFrame = EndFrame + 1;
				return(FALSE);
			}
		}
	}

/* Place the audio we want into the movie file */

	if (CompressChoice & VAA_AUDIO) {
		if (read(AudioFd,AudioBuffer,jpeg_get_slice()) < 0) {
			SystemError(errno,"Read of audio data from file failed");
			CurrentFrame = EndFrame + 1;
			return(FALSE);
		}
		if (jpeg_add_audio(VAADisplay,AudioBuffer,0)) {
			ShowAlert("Unable to write audio to movie file, aborting...",NULL);
			CurrentFrame = EndFrame + 1;
			return(FALSE);
		}
	}

/* Grab a single JPEG compressed image */

	if (CurrentContext->player->model == NEC) {
		if (PlayerStatus(CurrentContext->player) == PLAYER_STATUS_PLAY) {
			ShowDiscFrame(CurrentContext->player,CurrentFrame);
		}
	}
	xv_set(CurrentContext->video,VIDEO_LIVE,FALSE,NULL);
	image = (JPEGImage *) xv_get(CurrentContext->video,VIDEO_JPEG_IMAGE);
	if (jpeg_add_still(VAADisplay,image)) {
		ShowAlert("Unable to write frame to movie file, aborting...",NULL);
		CurrentFrame = EndFrame + 1;
		return(FALSE);
	}
	if (jpeg_end_frame(VAADisplay)) {
		ShowAlert("Unable to write EOF to movie file, aborting...",NULL);
		CurrentFrame = EndFrame + 1;
		return(FALSE);
	}
	jpeg_free_image(VAADisplay,image);
	xv_set(CurrentContext->video,VIDEO_LIVE,TRUE,NULL);

/* Continue */

	if (CurrentContext->player->model == NEC) {
		if (PlayerStatus(CurrentContext->player) == PLAYER_STATUS_PLAY) {
			ShowDiscFrame(CurrentContext->player,CurrentFrame);
		}
	}
	switch (JPEGRate) {
	case 0:
		if ((CurrentFrame % 2) == 0) {
			PlayerStepForward(CurrentContext->player);
			CurrentFrame++;
		}
		PlayerStepForward(CurrentContext->player);
		break;
	case 1:
		if ((CurrentFrame % 3) == 0) {
			PlayerStepForward(CurrentContext->player);
			CurrentFrame++;
		}
		PlayerStepForward(CurrentContext->player);
		break;
	case 2:
		if ((CurrentFrame % 5) == 0) {
			PlayerStepForward(CurrentContext->player);
			CurrentFrame++;
		}
		PlayerStepForward(CurrentContext->player);
		break;
	case 3:
		if ((CurrentFrame % 6) == 0) {
			PlayerStepForward(CurrentContext->player);
			CurrentFrame++;
		}
		PlayerStepForward(CurrentContext->player);
		break;
	default:
		PlayerStepForward(CurrentContext->player);
		break;
	}
	CurrentFrame++;

	return(TRUE);
}

/* Capture the specifed audio track from a video segment */

static int CaptureAudio()
{
	char buf[32];
	int length;

	if (!AudioFd) {
		if (ShowDiscSegment(CurrentContext->player,StartFrame,EndFrame+2,FALSE) < 0)
			CurrentFrame = StartFrame;
		else {
			length = EndFrame - StartFrame + 1;
			if (length % 2) length++;
			OpenAudio(length,TRUE);
			if (audio_get_record_config(AudioFd,&(PlayHeader)) != AUDIO_SUCCESS) {
				ShowNote("Couldn't get audio device record encoding!",NULL);
				return(FALSE);
			}
			xv_set(VAAFrame,FRAME_RIGHT_FOOTER,"Recording soundtrack",NULL);
		}
	}
	else if (AudioDone()) {
		CloseAudio(TRUE,TRUE);
		CurrentFrame = StartFrame;
		if (CompressChoice & VAA_VIDEO) {
			ShowDiscFrame(CurrentContext->player,StartFrame);
			sprintf(buf,"Frame %d",StartFrame);
			xv_set(VAAFrame,FRAME_RIGHT_FOOTER,buf,NULL);
		}
	}
	return(TRUE);
}

/* Capture the current (live) video segement for the video source */

static int CaptureLiveVideo()
{
	int speed,value;
	static int delay;
	JPEGImage *image;

/* Open new movie file */

	if (CurrentFrame == 0) {
		speed = (int) xv_get(FrameSpeed,PANEL_VALUE);
		switch (JPEGRate) {
		case 0:
			speed = (speed * 1) / 2; break;
		case 1:
			speed = (speed * 2) / 3; break;
		case 2:
			speed = (speed * 4) / 5; break;
		case 3:
			speed = (speed * 5) / 6; break;
		}
		if (OpenAudio(FALSE,FALSE) == FALSE) return(FALSE);
		if (CompressChoice & VAA_AUDIO) {
			if (audio_get_record_config(AudioFd,&(PlayHeader)) != AUDIO_SUCCESS) {
				ShowNote("Couldn't get audio device record encoding!",NULL);
				return(FALSE);
			}
/* Open jpeg movie file with audio information */
			if (jpeg_open_movie(VAADisplay,xv_get(FileName,PANEL_VALUE),speed,
								(int) xv_get(CurrentContext->video,VIDEO_Q_FACTOR),
								&PlayHeader)) {
				ShowAlert("Unable to open movie file, aborting...",NULL);
				return(FALSE);
			}
		}
		else {
/* Open jpeg movie file without audio information */
			if (jpeg_open_movie(VAADisplay,xv_get(FileName,PANEL_VALUE),speed,
								(int) xv_get(CurrentContext->video,VIDEO_Q_FACTOR),
								NULL)) {
				ShowAlert("Unable to open movie file, aborting...",NULL);
				return(FALSE);
			}
		}
		xv_set(CurrentContext->video,VIDEO_LIVE,FALSE,NULL);
	}

/* Place the audio we want into the movie file */

loop:
	value = 0;
	if (ioctl(AudioFd,FIONREAD,&value) < 0) {
		SystemError(errno,"FIONREAD ioctl failed");
		return;
	}
	if (value < jpeg_get_slice()) goto loop;
	if (CompressChoice & VAA_AUDIO) {
		if (read(AudioFd,AudioBuffer,jpeg_get_slice()) < 0) {
			SystemError(errno,"Read of audio data from device failed");
			return(FALSE);
		}
		if (jpeg_add_audio(VAADisplay,AudioBuffer,0)) {
			ShowAlert("Unable to write audio to movie file, aborting...",NULL);
			return(FALSE);
		}
	}

/* Grab a single JPEG compressed image */

	image = (JPEGImage *) xv_get(CurrentContext->video,VIDEO_JPEG_IMAGE);
	if (jpeg_add_still(VAADisplay,image)) {
		ShowAlert("Unable to write frame to movie file, aborting...",NULL);
		return(FALSE);
	}
	if (jpeg_end_frame(VAADisplay)) {
		ShowAlert("Unable to write EOF to movie file, aborting...",NULL);
		return(FALSE);
	}
	xv_set(CurrentContext->video,VIDEO_FAST_FRAME,TRUE,NULL);
	CurrentFrame++;
	return(TRUE);
}

/* Event function for VAA panel to handle all button presses */

static int VAAPanelProc(item, event)
Panel_item item;
Event *event;
{
	Menu menu;
	int who;

	who = (int) xv_get(item,XV_KEY_DATA,PANEL_DATA);
	
	switch (who) {
	case 1:
		break;
	case 2:
		JPEGChoice = (int) xv_get(item,PANEL_VALUE);
		menu = (Menu) xv_get(OptionsButton,PANEL_ITEM_MENU);
		if (JPEGChoice)
			 xv_set(xv_get(menu,MENU_NTH_ITEM,4,NULL),MENU_INACTIVE,FALSE,NULL);
		else xv_set(xv_get(menu,MENU_NTH_ITEM,4,NULL),MENU_INACTIVE,TRUE,NULL);
		menu = (Menu) xv_get(FileButton,PANEL_ITEM_MENU);
		if (JPEGChoice)
			 xv_set(xv_get(menu,MENU_NTH_ITEM,4,NULL),MENU_INACTIVE,FALSE,NULL);
		else xv_set(xv_get(menu,MENU_NTH_ITEM,4,NULL),MENU_INACTIVE,TRUE,NULL);
		break;		
	default:
		break;
	}
	return(XV_OK);
}

/* Handle menu selections for options buton */

static void OptionsMenuProc(menu,item)
Menu menu;
Menu_item item;
{
	int choice;
	int value;

	choice = (int) xv_get(item,MENU_VALUE);

	switch (choice) {
	case 1:
/* Load still frame */
		if (CurrentContext) {
			LoadStillFrame(xv_get(DirName,PANEL_VALUE),xv_get(FileName,PANEL_VALUE));
		}
		else ShowAlert("No open video channel available!",NULL);
		break;
	case 2:
/* Save still frame */
		if (CurrentContext) {
			switch (JPEGChoice) {
			case 0:
				SaveSunRasterFile(xv_get(DirName,PANEL_VALUE),xv_get(FileName,PANEL_VALUE));
				break;
			case 1:
				SaveJPEGFile(xv_get(DirName,PANEL_VALUE),xv_get(FileName,PANEL_VALUE));
				break;
			case 2:
				SaveJFIFFile(xv_get(DirName,PANEL_VALUE),xv_get(FileName,PANEL_VALUE));
				break;
			}
		}
		break;
	case 3:
		if (CurrentContext == NULL) {
			ShowAlert("Can't display JPEG movie file without video window",NULL);
			break;
		}
		VAAMode |= VAA_PLAY;
		break;
	case 4:
/* Make sure we are not live (for now) */
		if (CurrentContext == NULL) {
			ShowAlert("Can't create JPEG movie file without video window",NULL);
			break;
		}
/* Connect to the requested directory */
		strcpy(dirname,(char *) xv_get(DirName,PANEL_VALUE));
		if (chdir(dirname) < 0) {
			SystemError(errno,"Change working directory failed");
			break;
		}
/* Discard any existing audio file */
		if (VAAMode & VAA_AUDIO) { CloseAudio(FALSE,TRUE); sleep(1); }
		CurrentFrame = 0;
/* Give instructions */
		ShowNote("Place mouse inside the currently active video input window",
				 "Use MIDDLE mouse button to start (down) and stop (up) movie",
				 NULL);
/* Enable recording */
		VAAMode |= VAA_RECORD;
		break;
	case 5:
/* Open input channel 1 */
		CreateVideoInput(VIDEO_IN_ONE);
		VAAMode |= VAA_VIDEO;
		break;
	case 6:
/* Open input channel 2	*/
		CreateVideoInput(VIDEO_IN_TWO);
		VAAMode |= VAA_VIDEO;
		break;
	case 7:
/* Open output channel 1 */
		if (CreateVideoOutput(VIDEO_OUT_ONE)) xv_set(OutputFrame,WIN_SHOW,TRUE,NULL);
		break;
	case 8:
/* Open Audio input 1 */
		xv_set(AudioFrame,WIN_SHOW,TRUE,NULL);
		VAAMode |= VAA_AUDIO;
		break;
	case 9:
/* Show video config control planel */
		if (CurrentContext) {
			value = (int) xv_get(CurrentContext->video,VIDEO_LIVE);
			if (value) xv_set(VideoItem,PANEL_VALUE,0,NULL);
			else xv_set(VideoItem,PANEL_VALUE,1,NULL);
			value = (int) xv_get(CurrentContext->video,VIDEO_NORMAL_ASPECT);
			xv_set(AspectItem,PANEL_VALUE,value,NULL);
			value = (int) xv_get(CurrentContext->video,VIDEO_FORMAT);
			switch (value) {
			case VIDEO_COMPOSITE:
				xv_set(FormatItem,PANEL_VALUE,0,NULL);
				xv_set(FormatItem,XV_KEY_DATA,ITEM_DATA,0,NULL);
				break;
			case VIDEO_SUPER_VHS:
				xv_set(FormatItem,PANEL_VALUE,1,NULL);
				xv_set(FormatItem,XV_KEY_DATA,ITEM_DATA,1,NULL);
				break;
			case VIDEO_YUV:
				xv_set(FormatItem,PANEL_VALUE,2,NULL);
				xv_set(FormatItem,XV_KEY_DATA,ITEM_DATA,2,NULL);
				break;
			case VIDEO_RGB:
				xv_set(FormatItem,PANEL_VALUE,3,NULL);
				xv_set(FormatItem,XV_KEY_DATA,ITEM_DATA,3,NULL);
				break;
			default:
				xv_set(FormatItem,PANEL_VALUE,0,NULL);
				xv_set(FormatItem,XV_KEY_DATA,ITEM_DATA,0,NULL);
				break;
			}
			value = (int) xv_get(CurrentContext->video,VIDEO_INPUT_TYPE);
			switch (value) {
			case VIDEO_RGB24:
				xv_set(TypeItem,PANEL_VALUE,0,NULL);
				xv_set(TypeItem,XV_KEY_DATA,ITEM_DATA,0,NULL);
				break;
			case VIDEO_RGB8:
				xv_set(TypeItem,PANEL_VALUE,1,NULL);
				xv_set(TypeItem,XV_KEY_DATA,ITEM_DATA,1,NULL);
				break;
			case VIDEO_GRAY8:
				xv_set(TypeItem,PANEL_VALUE,2,NULL);
				xv_set(TypeItem,XV_KEY_DATA,ITEM_DATA,2,NULL);
				break;
			case VIDEO_GRAY7:
				xv_set(TypeItem,PANEL_VALUE,3,NULL);
				xv_set(TypeItem,XV_KEY_DATA,ITEM_DATA,3,NULL);
				break;
			case VIDEO_YUV7:
				xv_set(TypeItem,PANEL_VALUE,4,NULL);
				xv_set(TypeItem,XV_KEY_DATA,ITEM_DATA,4,NULL);
				break;
			default:
				xv_set(TypeItem,PANEL_VALUE,0,NULL);
				xv_set(TypeItem,XV_KEY_DATA,ITEM_DATA,0,NULL);
				break;
			}
			value = (int) xv_get(CurrentContext->video,VIDEO_HUE);
			xv_set(HueItem,PANEL_VALUE,value,NULL);
			value = (int) xv_get(CurrentContext->video,VIDEO_BRIGHTNESS);
			xv_set(BrightnessItem,PANEL_VALUE,value,NULL);
			value = (int) xv_get(CurrentContext->video,VIDEO_CONTRAST);
			xv_set(ContrastItem,PANEL_VALUE,value,NULL);
			value = (int) xv_get(CurrentContext->video,VIDEO_SATURATION);
			xv_set(SaturationItem,PANEL_VALUE,value,NULL);
			xv_set(ConfigFrame,WIN_SHOW,TRUE,NULL);
		}
		else ShowAlert("Can't display Video Config Control Panel",
			"without at least one active video channel!",NULL);
		break;
	case 10:
		if (CurrentContext) {
			value = (int) xv_get(CurrentContext->video,VIDEO_HUE);
			xv_set(HueItem,PANEL_VALUE,value,NULL);
			value = (int) xv_get(CurrentContext->video,VIDEO_BRIGHTNESS);
			xv_set(BrightnessItem,PANEL_VALUE,value,NULL);
			value = (int) xv_get(CurrentContext->video,VIDEO_CONTRAST);
			xv_set(ContrastItem,PANEL_VALUE,value,NULL);
			value = (int) xv_get(CurrentContext->video,VIDEO_SATURATION);
			xv_set(SaturationItem,PANEL_VALUE,value,NULL);
			xv_set(ColorFrame,WIN_SHOW,TRUE,NULL);
		}
		else ShowAlert("Can't display Video Color Control Panel",
			"without at least one active video channel!",NULL);
		break;
	case 11:
/* Show video player control panel */
		if (CurrentContext) {
			xv_set(PlayerFrame,WIN_SHOW,TRUE,NULL);
		}
		else ShowAlert("Can't display Video Device Control Panel",
			"without at least one active video channel!",NULL);
		break;
	case 12:
/* Show JPEG control panel */
		if (CurrentContext) {
			xv_set(JPEGFrame,WIN_SHOW,TRUE,NULL);
		}
		else ShowAlert("Can't display JPEG Compression Control Panel",
			"without at least one active video channel!",NULL);
		break;
	default:
		break;
	}
}

/* Create panel for controling video input for VAA */

static int CreateVAAPanel()
{
	Server_image si;
	Menu menu;
	char *buf;

	VAAPanel = xv_create(VAAFrame,PANEL,NULL);

	if (VAAPanel == 0) {
		ShowAlert("xv_create of base frame panel failed!",NULL);
		exit(1);
	}

	menu = (Menu) xv_create(XV_NULL,MENU,
		MENU_NOTIFY_PROC,			OptionsMenuProc,
		MENU_ITEM,
			MENU_STRING,			"Load Still",
			MENU_VALUE,				1,
			NULL,
		MENU_ITEM,
			MENU_STRING,			"Save Still",
			MENU_INACTIVE,			TRUE,
			MENU_VALUE,				2,
			NULL,
		MENU_ITEM,
			MENU_STRING,			"Load Movie",
			MENU_VALUE,				3,
			NULL,
		MENU_ITEM,
			MENU_STRING,			"Save Movie",
			MENU_INACTIVE,			TRUE,
			MENU_VALUE,				4,
			NULL,
		NULL);

	FileButton = xv_create(VAAPanel, PANEL_BUTTON,
		PANEL_NOTIFY_PROC,			VAAPanelProc,
		PANEL_LABEL_STRING,			"File",
		PANEL_ITEM_MENU,			menu,
		XV_X,						xv_col(VAAPanel,0),
		XV_Y,						xv_row(VAAPanel,0),
		XV_KEY_DATA,				PANEL_DATA,			0,
		NULL);

	menu = (Menu) xv_create(XV_NULL,MENU,
		MENU_NOTIFY_PROC,			OptionsMenuProc,
		MENU_ITEM,
			MENU_STRING,			"Input  1",
			MENU_VALUE,				5,
			NULL,
		MENU_ITEM,
			MENU_STRING,			"Input  2",
			MENU_VALUE,				6,
			NULL,
		MENU_ITEM,
			MENU_STRING,			"Output 1",
			MENU_VALUE,				7,
			NULL,
		MENU_ITEM,
			MENU_STRING,			"Audio  1",
			MENU_VALUE,				8,
			NULL,
		NULL);

	ViewButton = (Panel_item) xv_create(VAAPanel, PANEL_BUTTON,
		PANEL_NOTIFY_PROC,			VAAPanelProc,
		PANEL_LABEL_STRING,			"View",
		PANEL_ITEM_MENU,			menu,
		XV_X,						xv_col(VAAPanel,10),
		XV_Y,						xv_row(VAAPanel,0),
		XV_KEY_DATA,				PANEL_DATA,			0,
		NULL);

	menu = (Menu) xv_create(XV_NULL,MENU,
		MENU_NOTIFY_PROC,			OptionsMenuProc,
		MENU_ITEM,
			MENU_STRING,			"Config",
			MENU_VALUE,				9,
			NULL,
		MENU_ITEM,
			MENU_STRING,			"Color",
			MENU_VALUE,				10,
			NULL,
		MENU_ITEM,
			MENU_STRING,			"Device",
			MENU_VALUE,				11,
			NULL,
		MENU_ITEM,
			MENU_STRING,			"JPEG",
			MENU_INACTIVE,			TRUE,
			MENU_VALUE,				12,
			NULL,
		NULL);

	OptionsButton = (Panel_item) xv_create(VAAPanel, PANEL_BUTTON,
		PANEL_NOTIFY_PROC,			VAAPanelProc,
		PANEL_LABEL_STRING,			"Edit",
		PANEL_ITEM_MENU,			menu,
		XV_X,						xv_col(VAAPanel,21),
		XV_Y,						xv_row(VAAPanel,0),
		XV_KEY_DATA,				PANEL_DATA,			0,
		NULL);
#ifdef SVR4
	(void) getcwd(dirname,256);
#else
	getwd(dirname);
#endif
	DirName = xv_create(VAAPanel,PANEL_TEXT, 
		PANEL_LABEL_X,              xv_col(VAAPanel,0),
		PANEL_LABEL_Y,              xv_row(VAAPanel,2),
		PANEL_VALUE_DISPLAY_LENGTH, 34,
		PANEL_VALUE_STORED_LENGTH,	256,
		PANEL_LABEL_STRING,         " Directory : ",
		PANEL_VALUE,                dirname,
		XV_KEY_DATA,				PANEL_DATA,			0,
		NULL);

	FileName = xv_create(VAAPanel,PANEL_TEXT, 
		PANEL_LABEL_X,              xv_col(VAAPanel,0),
		PANEL_LABEL_Y,              xv_row(VAAPanel,3),
		PANEL_VALUE_DISPLAY_LENGTH, 34,
		PANEL_VALUE_STORED_LENGTH,	256,
		PANEL_LABEL_STRING,         "File Name : ",
		PANEL_VALUE,                "image",
		XV_KEY_DATA,				PANEL_DATA,			0,
		NULL);

	JPEGChoice = FALSE;
	(void) xv_create(VAAPanel,PANEL_CHOICE,
		PANEL_NOTIFY_PROC,			VAAPanelProc,
		PANEL_FEEDBACK,				PANEL_INVERTED,
		PANEL_LABEL_STRING,			"JPEG Comp : ",
		PANEL_CHOICE_STRINGS,		" Off "," On  ",NULL,
		PANEL_VALUE,				JPEGChoice,
		XV_KEY_DATA,				PANEL_DATA,			2,
		XV_X,						xv_col(VAAPanel,0),
		XV_Y,						xv_row(VAAPanel,4),
		NULL);

	window_fit(VAAPanel);
	window_fit(VAAFrame);

	return(0);
}

main(argc,argv)
int argc;
char *argv[];
{
	Server_image a,b;
	Icon icon;
	char buf[32];
	int i;

/* Initialize the xview toolkit */

	xv_init(XV_INIT_ARGC_PTR_ARGV,&argc,argv,NULL);

/* Grab off any user supplied arguments */

	while ((i = getopt(argc, argv, "?")) != EOF) {
		switch (i) {
		case '?':
			fprintf(stderr,"usage: %s\n", argv[0]);
			exit(1);
			break;
		}
	}

/* Create our base frame */

	a = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,           64, 
		XV_HEIGHT,          64, 
		SERVER_IMAGE_BITS,  icon_bits,
		NULL);

	b = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
		XV_WIDTH,           64, 
		XV_HEIGHT,          64, 
		SERVER_IMAGE_BITS,  mask_bits,
		NULL);
		    
	icon = (Icon) xv_create(XV_NULL,ICON,
		ICON_IMAGE,         a,  
		ICON_MASK_IMAGE,    b,  
		ICON_TRANSPARENT,   TRUE,
		NULL);
		
	VAAFrame = xv_create(XV_NULL,FRAME,
			FRAME_SHOW_LABEL,			TRUE,
			FRAME_DONE_PROC,			FrameDone,
			FRAME_SHOW_FOOTER,			TRUE,
			FRAME_ICON,					icon,
			FRAME_SHOW_RESIZE_CORNER,	FALSE,
			XV_LABEL,					"Parallax Graphics  VideoTool (XView version)",
			XV_X,						300,
			XV_Y,						300,
			NULL);

	VAADisplay  = (Display *) xv_get(VAAFrame,XV_DISPLAY);
	VAAXVScreen = (Xv_Screen) xv_get(VAAFrame,XV_SCREEN);
	VAAScreen   = (int) xv_get(VAAXVScreen,SCREEN_NUMBER);

/* Break loop when user wants out */

	notify_interpose_destroy_func(VAAFrame,notice_quit);

/* Need to know when to close video channels on bad signals */

	(void) notify_set_signal_func(VAAFrame, SignalCatcher,
		SIGINT, NOTIFY_SYNC);
	(void) notify_set_signal_func(VAAFrame, SignalCatcher,
		SIGBUS, NOTIFY_SYNC);
	(void) notify_set_signal_func(VAAFrame, SignalCatcher,
		SIGHUP, NOTIFY_SYNC);
	(void) notify_set_signal_func(VAAFrame, SignalCatcher,
		SIGSEGV, NOTIFY_SYNC);
		
/* Create base frame control panel */

	CreateVAAPanel();

/* Create video player control panel */

	CreatePlayerPanel();

/* Create video board control panels */

	CreateConfigPanel();
	CreateColorPanel();
	CreateOutputPanel();
	CreateJPEGPanel();

/* Create audio control panel */

	CreateAudioPanel();

/* Our own event dispatching for XView events */

	xv_set(VAAFrame,WIN_SHOW,TRUE,NULL);

	while (TRUE) {
		if (VAAMode & VAA_STOP) {
			VAAMode   &= ~VAA_RUN;
			VAAMode   &= ~VAA_STOP;
			if (AudioFd) CloseAudio(TRUE,TRUE);
			unlink("AUDIO");
			if (VAAMode & VAA_AUDIO) OpenAudio(FALSE,TRUE);
			xv_set(CurrentContext->video,VIDEO_SAVE_STILL,TRUE,NULL);
			xv_set(VAAFrame,FRAME_RIGHT_FOOTER,"Movie Canceled",NULL);
		}
		else if (VAAMode & VAA_RUN) {
			if (CurrentFrame == 0) {
				if (CompressChoice & VAA_AUDIO) {
					CaptureAudio();
					if (VAAMode & VAA_AUDIO) UpdateAudioDisplay();
				}
				else CurrentFrame = StartFrame;
			}
			else if (CurrentFrame <= EndFrame) {
				if (CompressChoice & VAA_VIDEO) {
					CaptureVideo();
					sprintf(buf,"Frame %d",CurrentFrame);
					xv_set(VAAFrame,FRAME_RIGHT_FOOTER,buf,NULL);
				}
				else CurrentFrame = EndFrame + 1;
			}
			else {
				if (CompressChoice & VAA_VIDEO) {
					jpeg_stop_movie(VAADisplay);
					if (CompressChoice & VAA_AUDIO) {
						close(AudioFd); AudioFd = 0;
					}
					unlink("AUDIO");
				}
				VAAMode   &= ~VAA_RUN;
				if (VAAMode & VAA_AUDIO) OpenAudio(FALSE,TRUE);
				xv_set(CurrentContext->video,VIDEO_SAVE_STILL,TRUE,NULL);
				xv_set(VAAFrame,FRAME_RIGHT_FOOTER,"Movie Completed",NULL);
			}
			nap(50000);
		}
		else if (VAAMode & VAA_START) {
			if (CaptureLiveVideo() == FALSE) {
				VAAMode &= ~VAA_START;
				VAAMode &= ~VAA_RECORD;
				jpeg_stop_movie(VAADisplay);
				CloseAudio(FALSE,TRUE);
				if (VAAMode & VAA_AUDIO) OpenAudio(FALSE,TRUE);
				xv_set(CurrentContext->video,
					   VIDEO_LIVE,		 TRUE,
					   VIDEO_SAVE_STILL, TRUE,
					   VIDEO_USE_SHM,	 FALSE,
					   NULL);
				xv_set(VAAFrame,FRAME_RIGHT_FOOTER,"Movie Failed",NULL);
			}
		}
		else if (VAAMode & VAA_EXIT) break;
		else if (VAAMode & VAA_VIDEO) {
			if (VAAMode & VAA_AUDIO) {
				if (AudioFd == 0) {
					if (!OpenAudio(FALSE,TRUE)) VAAMode &= ~VAA_AUDIO;
				}
				UpdateAudioDisplay();
			}
			nap(50000);
		}
		else if (VAAMode & VAA_AUDIO) {
			if (AudioFd == 0) {
				if (!OpenAudio(FALSE,TRUE)) VAAMode &= ~VAA_AUDIO;
			}
			UpdateAudioDisplay();
			nap(50000);
		}
		else nap(50000);
		notify_dispatch();
		if (QLength(VAADisplay))
			xv_input_pending(VAADisplay,XConnectionNumber(VAADisplay));
		XFlush(VAADisplay);
		if (VAAMode & VAA_PLAY) {
			if (VAAMode & VAA_AUDIO) CloseAudio(FALSE,TRUE);
			LoadJPEGMovie(xv_get(DirName,PANEL_VALUE),xv_get(FileName,PANEL_VALUE));
			if (VAAMode & VAA_AUDIO) OpenAudio(FALSE,TRUE);
			VAAMode &= ~VAA_PLAY;
		}
	}
	exit(0);
}

/* end of vaa.c */
