/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Header: /plx/source/Xclients/generic/RCS/WindowToPortID.c,v 1.2 1994/10/03 16:15:00 drapeau Exp $ */
/* $Log: WindowToPortID.c,v $
 * Revision 1.2  1994/10/03  16:15:00  drapeau
 * Two changes:
 *
 * 1) Added a Delete() method to this module, allowing port-to-window
 * mappings to be removed from the specified list.
 *
 * 2) Fixed an error in the Print() method: the "d" in "%d" was missing
 * in printing the window and Port ID's.
 *
 * Revision 1.1  1993/11/20  00:40:15  drapeau
 * Initial revision
 * */
static char windowToPortIDRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/WindowToPortID.c,v 1.2 1994/10/03 16:15:00 drapeau Exp $";

#include "WindowToPortID.h"

WindowToPortID*	windowToPortIDList = (WindowToPortID*)NULL;	    /* Used by the XPlx library to keep track of Windows to... */
								    /* ...XvPortID's */

WindowToPortID* NewWindowToPortIDList(void)
{
  WindowToPortID*	newWindowToPortIDList;
  
  newWindowToPortIDList = (WindowToPortID*)malloc(sizeof(WindowToPortID));
  if (newWindowToPortIDList == (WindowToPortID*)NULL)
    return((WindowToPortID*)NULL);
  newWindowToPortIDList->window = (Window)0;
  newWindowToPortIDList->xvPortID = (XvPortID)0;
  newWindowToPortIDList->next = NULL;
  return(newWindowToPortIDList);
}								    /* end function newWindowToPortIDList */



void DestroyWindowToPortIDList(WindowToPortID* windowToPortIDList)
{
  WindowToPortID*	tempHead;
  WindowToPortID*	tempNext;
  
  if (windowToPortIDList == NULL)				    /* Was the WindowToPortID passed in empty? */
  {
    return;							    /* Yes, don't bother trying to free up the WindowToPortID */
  }
  else
    for (tempHead = windowToPortIDList;				    /* Traverse WindowToPortID, deleting one member at a time */
	 tempHead != (WindowToPortID*)NULL;
	 tempHead = tempNext)
    {
      tempNext = tempHead->next;				    /* Point to the next element of the list */
      free(tempHead);						    /* Free this element */
    }
  return;
}								    /* end function 	DestroyWindowToPortID */



void WindowToPortIDAdd(Window		window,
		       XvPortID		xvPortID,
		       WindowToPortID**	windowToPortIDList)
{
  WindowToPortID*		tempPtr;
  
  if (*windowToPortIDList == NULL)				    /* Has a WindowToPortID list been created yet? */
  {
    *windowToPortIDList = NewWindowToPortIDList();		    /* No, get new WindowToPortID and make it the 1st element...*/
    tempPtr = *windowToPortIDList;				    /* ...of the WindowToPortID */
  }
  else								    /* Yes, a WindowToPortID already exists; find the end and... */
  {								    /* ...insert a new element there */
    for (tempPtr = *windowToPortIDList;				    /* Traverse the WindowToPortID until the end is reached */
	 tempPtr->next; tempPtr = tempPtr->next)
      ;
    tempPtr->next = NewWindowToPortIDList();			    /* Create a new element and insert it at the end of the list */
    tempPtr = tempPtr->next;
  }
  tempPtr->window = window;					    /* Fill in the new element on the list */
  tempPtr->xvPortID = xvPortID;
  return;
}								    /* end function WindowToPortIDAddPortToList*/


void WindowToPortIDPrint(WindowToPortID* windowToPortIDList)
{
  WindowToPortID*	current;
  
  if (windowToPortIDList == NULL)
  {
    printf("No Window-to-XvPortID mappings yet exist.\n");
    return;
  }
  printf("\n\n\nList of Window-to-XvPortID mappings:\n");
  for (current = windowToPortIDList; current; current = current->next)
  {
    printf("Window ID: %d, XvPortID: %d.\n",
	   current->window,
	   current->xvPortID);
  }								    /* end for...*/
  return;
}								    /* end WindowToPortIDPrint*/



WindowToPortID* WindowToPortIDFind(Window 		window,
				   WindowToPortID* 	windowToPortIDList)
{
  WindowToPortID*	tempList;
  
  if (windowToPortIDList == (WindowToPortID*)NULL)		    /* Does the mapping list yet exist? */
  {								    /* No, return a NULL element */
    return((WindowToPortID*)NULL);
  }
  for (tempList = windowToPortIDList;				    /* Examine each element of the mapping list, searching... */
       tempList != (WindowToPortID*)NULL;			    /* ...for a match */
       tempList = tempList->next)
  {
    if (tempList->window == window)				    /* Was a match found? */
      return(tempList);						    /* Yes, return a pointer to the matching element */
  }
  return((WindowToPortID*)NULL);				    /* No match was found, return a NULL element */
} /* end function WindowToPortIDFind */



WindowToPortID* WindowToPortIDDelete(Window		window,
				     WindowToPortID*	windowToPortIDList)
{
  WindowToPortID*	thisItem;
  WindowToPortID*	lastItem;
  
  if (windowToPortIDList == (WindowToPortID*)NULL)		    /* Does the mapping list yet exist? */
  {								    /* No, return immediately */
    return;
  }
  lastItem = windowToPortIDList;				    /* Initialize the previous item examined to top of list */
  for (thisItem = windowToPortIDList ;				    /* Examine each element of the mapping list, searching... */
       thisItem != (WindowToPortID*)NULL;			    /* ...for a match */
       thisItem = thisItem->next)
  {
    if (thisItem->window == window)				    /* Was a match found? */
    {
      if (thisItem == windowToPortIDList)			    /* Is this the first item on the list? */
      {
	lastItem = thisItem->next;				    /* Yes, point to the 2nd item on the list (or NULL) */
	free(thisItem);						    /* Get rid of space taken by the 1st item */
	return(lastItem);					    /* Return the new head of the list */
      }
      else							    /* No, this item is somewhere buried in the list */
      {
	lastItem->next = thisItem->next;			    /* Make the last item looked at point to this item's next */
	free(thisItem);						    /* Get rid of space taken by the 1st item */
	return(windowToPortIDList);				    /* Return the same pointer to the head of the list */
      }
    } /* end if (window was found) */
  } /* end for... */
  return;
} /* end function WindowToPortIDDelete */
