/*
 * (c) Copyright 1991 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log: XPlxGetImage.c,v $
 * Revision 1.8  1995/12/20  00:54:49  drapeau
 * Enhanced the library to look for both video extensions: the library
 * functions now first look for the ParallaxVideo (i.e., "XPlx")
 * extension, using the XVideo (i.e., "Xv") extension as a fallback.
 *
 * Also, removed extraneous code now that the functions correctly look
 * for the proper video extension to use in the Display's extension data
 * structure.
 *
 * Also, reformatted code to make it more readable.
 *
 * Revision 1.7  1995/02/24  03:09:32  drapeau
 * Two changes:
 *
 * 1) Added code to help this library work with multiple displays (when a
 *    process using this library is working with mulitple displays, it is
 *    not guaranteed that the Plx extension will be loaded in the same
 *    spot and therefore might occupy a different base event code for Plx
 *    protocol events.  So, this library needs to keep a mapping of
 *    Display-to-Protocol-Event-Base-Codes so the "PREAMBLE" macro will
 *    always work correctly and allow this library to communicate with
 *    the proper X extension no matter what the Display.)
 *
 * 2) Cosmetic changes to make the code more readable and editable.
 *
 * Revision 1.6  1993/12/03  19:34:26  drapeau
 * Fixed the way include files are done to make them more flexible.
 *
 * Revision 1.5  1992/09/02  16:20:37  schen
 * removie the include of copyright.h because X11R5 does not have it.
 *
 * Revision 1.4  91/08/07  11:15:18  eric
 * second parameter to XCreateImage is a pointer to a visual
 * 
 * Revision 1.3  91/08/03  14:10:06  eric
 * many changes
 * 
 * Revision 1.2  91/07/24  14:17:08  eric
 * added rcs header
 * 
 *
 */

#ifndef lint
static char *rcsid = "@(#)$Id: XPlxGetImage.c,v 1.8 1995/12/20 00:54:49 drapeau Exp $ Parallax Graphics Inc";
#endif

/* $XConsortium: XGetImage.c,v 11.24 89/11/08 17:06:49 converse Exp $ */
/* Copyright    Massachusetts Institute of Technology    1986	*/

#define NEED_REPLIES
#include <X11/Xlibint.h>
#include <plxvideo.h>

#define ROUNDUP(nbytes, pad) (((((nbytes) - 1) + (pad)) / (pad)) * (pad))

extern XImage *XCreateImage();
extern Display*	plxLastOpenDpy;


static int VideoReqCode = 0;
static int first_event, first_error;


/*
 * set a drawable to be of type video
 */

#define	PREAMBLE(result)	\
		LockDisplay(dpy); \
		result = XPlxFindExtension(dpy, &VideoReqCode);

#define POSTAMBLE	\
		UnlockDisplay(dpy); \
		SyncHandle();

static unsigned int Ones(mask)                /* HACKMEM 169 */
    unsigned long mask;
{
    register unsigned long y;
    
    y = (mask >> 1) &033333333333;
    y = mask - y - ((y >>1) & 033333333333);
    return ((unsigned int) (((y + (y >> 3)) & 030707070707) % 077));
}



XImage* XPlxGetImage(register Display*		dpy,
		     Drawable			d,
		     int			x,
		     int			y,
		     unsigned int		width,
		     unsigned int		height,
		     unsigned long		plane_mask,
		     int			format)		    /* /either XYPixmap or ZPixmap */
{
  register XExtCodes*		codes;
  xGetImageReply		rep;
  register xPlxImageReq*	req;
  char*				data;
  long				nbytes;
  XImage*			image;
  static GC			gc = NULL;
  
  PREAMBLE(codes);
    
  GetReq(PlxImage, req);
  
  if(!gc)
    gc = XDefaultGC(dpy, DefaultScreen(dpy));
  
  /*
   * first set up the standard stuff in the request
   */
  req->drawable = d;
  req->gc = gc->gid;
  req->src_x = x;
  req->src_y = y;
  req->src_w = width;
  req->src_h = height;
  req->planeMask = plane_mask;
  req->format = format;
  req->reqType = VideoReqCode;
  req->videoReqType = X_PlxGetImage;
  
  if (_XReply (dpy, (xReply *) &rep, 0, xFalse) == 0) {
    UnlockDisplay(dpy);
    SyncHandle();
    return (XImage *)NULL;
  }
  
  nbytes = (long)rep.length << 2;
  data = (char *) Xmalloc((unsigned) nbytes);
  if (! data) {
    _XEatData(dpy, (unsigned long) nbytes);
    UnlockDisplay(dpy);
    SyncHandle();
    return (XImage *) NULL;
  }
  _XReadPad (dpy, data, nbytes);
  if (format == XYPixmap)
    image = XCreateImage(dpy, _XVIDtoVisual(dpy, rep.visual),
			 Ones (plane_mask & ((1 << rep.depth) - 1)),
			 format, 0, data, width, height, dpy->bitmap_pad, 0);
  else /* format == ZPixmap */
#ifdef NOTDEF
    image = XCreateImage (dpy, _XVIDtoVisual(dpy, rep.visual),
#endif
  image = XCreateImage (dpy, NULL,	    /* need a faked visual here */
			rep.depth, ZPixmap, 0, data, width, height,
			_XGetScanlinePad(dpy, (int) rep.depth), 0);
			  
  POSTAMBLE
			  
  return (image);
} /* end function XPlxGetImage */



#ifdef NOTDEF
XImage* XGetSubImage(register Display*		dpy,
		     Drawable			d,
		     int			x,
		     int			y,
		     unsigned int		width,
		     unsigned int		height,
		     unsigned long		plane_mask,
		     int			format,		    /* either XYFormat or ZFormat */
		     XImage*			dest_image,
		     int			dest_x,
		     int			dest_y)
{
  XImage *temp_image;
  temp_image = XGetImage(dpy, d, x, y, width, height, 
			 plane_mask, format);
  if (!temp_image)
    return (XImage *)NULL;
  _XSetImage(temp_image, dest_image, dest_x, dest_y);
  XDestroyImage(temp_image);
  return (dest_image);
} /* end function XGetSubImage */
#endif
