/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Log: XPlxRtv.c,v $
 * Revision 1.4  1996/01/03  21:56:14  lovvik
 * Added an argument for the signal standard to the XPlxRtvPutCImage()
 * call.
 *
 * Revision 1.3  1995/12/20  00:58:50  drapeau
 * Enhanced the library to look for both video extensions: the library
 * functions now first look for the ParallaxVideo (i.e., "XPlx")
 * extension, using the XVideo (i.e., "Xv") extension as a fallback.
 *
 * Also, removed extraneous code now that the functions correctly look
 * for the proper video extension to use in the Display's extension data
 * structure.
 *
 * Revision 1.2  1995/11/01  02:52:33  lovvik
 * Modified XPlxRtvPutCImage() and XPlxRtvGetCImage() to expect a return
 * value.  The return value is not actually used, but this ensures that the
 * operation will be completed by the server immediately.
 *
 * Revision 1.1  1995/10/11  02:21:22  drapeau
 * Initial revision
 * */

#ifndef lint
static char *xPlxRtvRcsId = "$Id: XPlxRtv.c,v 1.4 1996/01/03 21:56:14 lovvik Exp $";
#endif

#define NEED_REPLIES
#include	<X11/Xlibint.h>
#include	<X11/Xutil.h>
#include	<plxvideo.h>

static int VideoReqCode = 0;
static int first_event, first_error;
extern Display*	plxLastOpenDpy;

/*
 * set a drawable to be of type video
 */

#define	PREAMBLE(result)	\
		LockDisplay(dpy); \
		result = XPlxFindExtension(dpy, &VideoReqCode);

#define POSTAMBLE	\
		UnlockDisplay(dpy); \
		SyncHandle();



int XPlxRtvGetCImage(Display*		dpy,
		     Drawable		d,
		     GC			gc,
		     int		src_x,
		     int		src_y,
		     unsigned int	src_w,
		     unsigned int	src_h,
		     unsigned int	dst_w,
		     unsigned int	dst_h)
{
  register XExtCodes*		codes;
  static Display*		lastDpy = (Display*)NULL;
  register xPlxRtvImageReq*	req;
  xPlxRtvImageReply		reply;
  char*				data;
  long				nbytes;
  XPlxCImage*			image = NULL;
  
  PREAMBLE(codes);
    
  GetReq(PlxRtvImage, req);
  req->drawable = d;						    /* Set up the standard stuff in the request */
  req->gc = gc->gid;
  req->src_x = src_x;
  req->src_y = src_y;
  req->src_w = src_w;
  req->src_h = src_h;
  req->dst_x = 0;
  req->dst_y = 0;
  req->dst_w = dst_w;
  req->dst_h = dst_h;
  
  req->reqType = VideoReqCode;
  req->videoReqType = X_PlxRtvGetCImage;
  
  if(!_XReply(dpy, (xReply *) &reply, 0, xFalse))
  {
    UnlockDisplay(dpy);
    SyncHandle();
    return (0);
  }
  POSTAMBLE
} /* end function XPlxRtvGetCImage */



int		XPlxRtvPutCImage(register Display*	dpy,
				 Drawable		d,
				 GC			gc,
				 int			src_x,
				 int			src_y,
				 unsigned int		src_w,
				 unsigned int		src_h,
				 int			dst_x,
				 int			dst_y,
				 unsigned int		dst_w,
				 unsigned int		dst_h,
				 int			standard)
{
  register XExtCodes*		codes;
  register xPlxRtvImageReq*	req;
  xPlxRtvImageReply		reply;
  long				length;
  unsigned char*		src;
  
  PREAMBLE(codes);
    
  GetReq(PlxRtvImage, req);
  
  req->drawable = d;
  req->gc = gc->gid;
  req->reqType = VideoReqCode;
  req->videoReqType = X_PlxRtvPutCImage;
  req->src_x = src_x;
  req->src_y = src_y;
  req->src_w = src_w;
  req->src_h = src_h;
  req->dst_x = 0;
  req->dst_y = 0;
  req->dst_w = dst_w;
  req->dst_h = dst_h;
  req->format = standard;
  
  if(!_XReply(dpy, (xReply *) &reply, 0, xFalse))
  {
    UnlockDisplay(dpy);
    SyncHandle();
    return (0);
  }
  
  POSTAMBLE
} /* end function XPlxRtvPutCImage */


int XPlxSetRtvWindow(Display*		dpy,
		     Drawable		d,
		     GC			gc)
{
  register XExtCodes*		codes;
  static Display*		lastDpy = (Display*)NULL;
  register xPlxRtvImageReq*	req;
  xPlxRtvImageReply		reply;
  char*				data;
  long				nbytes;
  XPlxCImage*			image = NULL;
  
  PREAMBLE(codes);
    
  GetReq(PlxRtvImage, req);
  req->drawable = d;						    /* Set up the standard stuff in the request */
  req->gc = gc->gid;
  req->src_x = 0;
  req->src_y = 0;
  req->src_w = 1;
  req->src_h = 1;
  req->dst_x = 0;
  req->dst_y = 0;
  req->dst_w = 1;
  req->dst_h = 1;
  req->reqType = VideoReqCode;
  req->videoReqType = X_PlxSetRtvWindow;
  
  if(!_XReply(dpy, (xReply *) &reply, 0, xFalse))
  {
    UnlockDisplay(dpy);
    SyncHandle();
    return (0);
  }
  
  POSTAMBLE
} /* end function XPlxSetRtvWindow */



int		XPlxUnsetRtvWindow(register Display*	dpy,
				   Drawable		d,
				   GC			gc)
{
  register XExtCodes*		codes;
  register xPlxRtvImageReq*	req;
  xPlxRtvImageReply		reply;
  long				length;
  unsigned char*		src;
  
  PREAMBLE(codes);
    
  GetReq(PlxRtvImage, req);
  
  req->drawable = d;
  req->gc = gc->gid;
  req->src_x = 0;
  req->src_y = 0;
  req->src_w = 1;
  req->src_h = 1;
  req->dst_x = 0;
  req->dst_y = 0;
  req->dst_w = 1;
  req->dst_h = 1;
  req->reqType = VideoReqCode;
  req->videoReqType = X_PlxUnsetRtvWindow;
  
  if(!_XReply(dpy, (xReply *) &reply, 0, xFalse))
  {
    UnlockDisplay(dpy);
    SyncHandle();
    return (0);
  }
  
  POSTAMBLE
} /* end function XPlxRtvPutCImage */
