/*
 * (c) Copyright 1991 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log: XPlxShmComp.c,v $
 * Revision 1.72  1995/12/23  02:32:50  drapeau
 * Modified code to round-down to the nearest 16 pixel boundary on compress
 * widths.  Rounding down on height is unnecessary.
 *
 * Revision 1.71  1995/12/20  00:53:36  drapeau
 * Enhanced the library to look for both video extensions: the library
 * functions now first look for the ParallaxVideo (i.e., "XPlx")
 * extension, using the XVideo (i.e., "Xv") extension as a fallback.
 *
 * Also, removed extraneous code now that the functions correctly look
 * for the proper video extension to use in the Display's extension data
 * structure.
 *
 * Revision 1.70  1995/10/02  22:26:18  drapeau
 * Added Parallax-specific versions of XShm...() shared memory extension
 * routines to work on the Solaris-based window server.  This version is
 * not yet fully debugged.
 *
 * Revision 1.69  1995/08/30  04:49:21  drapeau
 * Fixed function declarations to support ANSI-C prototypes.  Also,
 * cosmetic changes to the code to enhance readability and editing ease.
 * Also, removed old code that supported the DEC platform.
 *
 * Revision 1.68  1995/02/24  03:07:59  drapeau
 * Two changes:
 *
 * 1) Added code to help this library work with multiple displays (when a
 *    process using this library is working with mulitple displays, it is
 *    not guaranteed that the Plx extension will be loaded in the same
 *    spot and therefore might occupy a different base event code for Plx
 *    protocol events.  So, this library needs to keep a mapping of
 *    Display-to-Protocol-Event-Base-Codes so the "PREAMBLE" macro will
 *    always work correctly and allow this library to communicate with
 *    the proper X extension no matter what the Display.)
 *
 * 2) Cosmetic changes to make the code more readable and editable.
 *
 * Revision 1.67  1994/10/22  04:36:06  drapeau
 * Fixed ANSI-C incompatibilies.
 *
 * Revision 1.66  1994/08/27  22:56:20  drapeau
 * Removed an unused variable from XPlxGetShmInfo() and made arguments
 * match function calls better.
 *
 * Revision 1.65  1994/08/25  05:10:37  drapeau
 * Removed obsolete MIPS conditional code.
 *
 * Also, removed all memory leaks found by Purify.  All known memory
 * leaks are now gone.
 *
 * Revision 1.64  1994/04/29  22:47:31  drapeau
 * Minor change, changed the way the "shmstr.h" include file is being
 * included.
 *
 * Revision 1.63  1994/01/20  01:30:15  drapeau
 * Removed the last remnants of the dst_w and dst_h arguments to
 * XPlxShmGetCImage(), which had caused compiler problems.
 *
 * Revision 1.62  1994/01/12  17:41:49  drapeau
 * Removed the last two arguments (destination width and height) to
 * XPlxShmGetCImage(); after talking with John Malleo-Roach, they were
 * deemed unnecessary and redundant (the image->width and image->height
 * parameters serve the same purpose I was trying to serve by adding two
 * arguments).
 *
 * Revision 1.61  1993/11/20  01:18:01  drapeau
 * Cleaned up inclusion of header files to increase portability.
 *
 * Revision 1.6  1993/11/17  02:22:56  drapeau
 * Two changes:
 * 1) Added two arguments to XPlxShmGetCImage() to match the functionality of
 *    the non-shared memory version.  This appears to have been a simple
 *    omission that is easily corrected; the code was already there to support
 *    the extra two arguments (allowing an image to be sized up before
 *    compressing).
 * 2) Cosmetic changes for better code readability.
 *
 * Revision 1.5  92/03/08  09:53:02  dave
 * fixed an include
 * 
 * Revision 1.4  92/03/08  09:18:55  dave
 * Added two new functions
 * 
 * Revision 1.3  91/11/14  13:56:21  schen
 * fixed the offset for get and put compressed image.
 * 
 * Revision 1.3  91/11/06  15:12:23  shirley
 * fixed the offset for get and put compressed image.
 *
 * Revision 1.2  91/11/06  15:12:23  dave
 * fixed planemask to have size in bytes
 * 
 * Revision 1.1  91/10/17  11:20:13  dave
 * Initial revision
 * 
 * Revision 1.3  91/08/03  14:09:59  eric
 * many changes
 * 
 * Revision 1.2  91/07/24  14:16:56  eric
 * added rcs header
 * 
 *
 */

#ifndef lint
static char *rcsid = "@(#)$Id: XPlxShmComp.c,v 1.72 1995/12/23 02:32:50 drapeau Exp $ Parallax Graphics Inc";
#endif

#define NEED_EVENTS
#define NEED_REPLIES

#include <stdio.h>

#ifndef mips
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#else
#include <fcntl.h>
#include <sys/ioctl.h>
#include "plxioctl.h"
#endif

#include	<X11/Xlibint.h>
#include	<X11/Xutil.h>
#include	<X11/extensions/XShm.h>
#include	<X11/extensions/shmstr.h>
#include	<X11/extensions/Xext.h>
#include	<extutil.h>
#include	<plxvideo.h>

/* #include <X11/Xlib.h> */

static int VideoReqCode = 0;
static int first_event, first_error;
extern Display*	plxLastOpenDpy;

/*
 * set a drawable to be of type video
 */

#define	PREAMBLE(result)	\
		LockDisplay(dpy); \
		result = XPlxFindExtension(dpy, &VideoReqCode);

#define POSTAMBLE	\
		UnlockDisplay(dpy); \
		SyncHandle();

static XExtensionInfo *shm_info;
static /* const */ char *shm_extension_name = PlxSharedMemoryExtensionName;

#define ShmCheckExtension(dpy,i,val) \
  XextCheckExtension (dpy, i, shm_extension_name, val)

/*****************************************************************************
 *                                                                           *
 *			   private utility routines                          *
 *                                                                           *
 *****************************************************************************/

static int close_display();
static char *error_string();
static Bool wire_to_event();
static Status event_to_wire();
static /* const */ XExtensionHooks shm_extension_hooks = {
    NULL,				/* create_gc */
    NULL,				/* copy_gc */
    NULL,				/* flush_gc */
    NULL,				/* free_gc */
    NULL,				/* create_font */
    NULL,				/* free_font */
    close_display,			/* close_display */
    wire_to_event,			/* wire_to_event */
    event_to_wire,			/* event_to_wire */
    NULL,				/* error */
    error_string,			/* error_string */
};

static /* const */ char *shm_error_list[] = {
    "BadShmSeg",			/* BadShmSeg */
};

static XEXT_GENERATE_FIND_DISPLAY (find_display, shm_info, shm_extension_name, 
				   &shm_extension_hooks, ShmNumberEvents, NULL)

static XEXT_GENERATE_CLOSE_DISPLAY (close_display, shm_info)

static XEXT_GENERATE_ERROR_STRING (error_string, shm_extension_name,
				   ShmNumberErrors, shm_error_list)


static Bool wire_to_event (dpy, re, event)
    Display *dpy;
    XEvent  *re;
    xEvent  *event;
{
    XExtDisplayInfo *info = find_display (dpy);
    XShmCompletionEvent	*se;
    xShmCompletionEvent	*sevent;
    
    ShmCheckExtension (dpy, info, False);
    
    switch ((event->u.u.type & 0x7f) - info->codes->first_event) {
    case ShmCompletion:
	se = (XShmCompletionEvent *) re;
	sevent = (xShmCompletionEvent *) event;
	se->type = sevent->type & 0x7f;
	se->serial = _XSetLastRequestRead(dpy,(xGenericReply *) event);
	se->send_event = (sevent->type & 0x80) != 0;
	se->display = dpy;
	se->drawable = sevent->drawable;
	se->major_code = sevent->majorEvent;
	se->minor_code = sevent->minorEvent;
	se->shmseg = sevent->shmseg;
	se->offset = sevent->offset;
    	return True;
    }
    return False;
}

static Status event_to_wire (dpy, re, event)
    Display *dpy;
    XEvent  *re;
    xEvent  *event;
{
    XExtDisplayInfo *info = find_display (dpy);
    XShmCompletionEvent	*se;
    xShmCompletionEvent	*sevent;
    
    ShmCheckExtension (dpy, info, 0);
    
    switch ((re->type & 0x7f) - info->codes->first_event) {
    case ShmCompletion:
    	se = (XShmCompletionEvent *) re;
	sevent = (xShmCompletionEvent *) event;
    	sevent->type = se->type | (se->send_event ? 0x80 : 0);
    	sevent->sequenceNumber = se->serial & 0xffff;
    	sevent->drawable = se->drawable;
    	sevent->majorEvent = se->major_code;
    	sevent->minorEvent = se->minor_code;
    	sevent->shmseg = se->shmseg;
    	sevent->offset = se->offset;
    	return True;
    }
    return False;
}

/*
 * CreateImage
 * 
 * Allocates the memory necessary for an XPlxCImage data structure. 
 * Initializes the structure with "default" values and returns XPlxCImage. 
 * 
 */

XPlxCImage *XPlxShmCreateCImage(register Display*	dpy,
				char*			data,
				XShmSegmentInfo*	shminfo,
				int			size,
				unsigned int		width,
				unsigned int		height)
{
  register XPlxCImage*	image;
  
  if ((image = (XPlxCImage *)
       Xcalloc(1, (unsigned) sizeof(XPlxCImage))) == NULL)
    return (XPlxCImage *) NULL;
  image->width = width;
  image->height = height;
  image->data = data;
  image->info = shminfo;
  image->size = size;
  return image;
} /* end function XPlxShmCreateCImage */



Status XPlxShmGetCImage(register Display*	dpy,
			Drawable		d,
			GC			gc,
			XPlxCImage*		image,
			int			src_x,
			int			src_y,
			unsigned int		src_w,
			unsigned int		src_h)
{
  register XExtCodes*		codes;
  XExtDisplayInfo*		info = (XExtDisplayInfo *) find_display (dpy);
  XShmSegmentInfo*		shminfo = (XShmSegmentInfo *) image->info;
  xPlxGetImageReply		rep;
  register xPlxImageReq*	req;
  char*				data;
  long				nbytes;
  
  ShmCheckExtension(dpy, info, 0);
  if(!shminfo) return 0;
  PREAMBLE(codes);
  if ((src_w < 16) || (image->width < 16) ||			    /* Is requested image size too small any parameter? */
      (src_h < 8) || (image->height < 8))
  if ((src_w % 16) != 0)					    /* Assure that all width sit on a... */
    src_w -= (src_w % 16);					    /* ...16 pixel boundary. Not sure why, but Dave Arnold... */
  if ((image->width % 16) != 0)					    /* ...says so. */
    image->width -= (image->width % 16);

  if (0 == VideoReqCode)					    /* Using the Xv extension? */
  {								    /* Yes, call the proper Xv function instead of the XPlx one */
    return(XPlxXvShmGetCImage(dpy, d, gc, image,
			      src_x, src_y, src_w, src_h));
  }
    
  GetReq(PlxImage, req);
  /*
   * first set up the standard stuff in the request
   */
  req->reqType = VideoReqCode;
  req->videoReqType = X_PlxShmGetCImage;
  req->drawable = d;
  req->gc = gc->gid;
  req->src_x = src_x;
  req->src_y = src_y;
  req->src_w = src_w;
  req->src_h = src_h;
  req->dst_x = 0;
  req->dst_y = 0;
  req->dst_w = image->width;
  req->dst_h = image->height;
  req->shmseg = shminfo->shmseg;
  req->offset = image->data - shminfo->shmaddr;
  if (_XReply (dpy, (xReply *) &rep, 0, xFalse) == 0) {
    UnlockDisplay(dpy);
    SyncHandle();
    return (0);
  }
  image->size = rep.size;
  
  POSTAMBLE
    
  return (1);
} /* end function XPlxShmGetCImage */



Status XPlxShmPutCImage(register Display*	dpy,
			Drawable		d,
			GC			gc,
			register XPlxCImage*	image,
			int			src_x,
			int			src_y,
			unsigned int		src_w,
			unsigned int		src_h,
			int			dst_x,
			int			dst_y,
			unsigned int		dst_w,
			unsigned int		dst_h,
			int			flag,
			Bool			send_event)
{
  register XExtCodes*		codes;
  XExtDisplayInfo*		info = (XExtDisplayInfo *) find_display (dpy);
  XShmSegmentInfo*		shminfo = (XShmSegmentInfo *) image->info;
  register xPlxImageReq*	req;
  
  ShmCheckExtension (dpy, info, 0);
  if (!shminfo) return 0;
  
  if ((image->info == NULL) || (image->size == 0))
    return 0;
  
  PREAMBLE(codes);
  if (0 == VideoReqCode)					    /* Using the Xv extension? */
  {								    /* Yes, call the proper Xv function instead of the XPlx one */
    return(XPlxXvShmPutCImage(dpy, d, gc, image,
			      src_x, src_y, src_w, src_h,
			      dst_x, dst_y, dst_w, dst_h,
			      flag, send_event));
  }
    
  GetReq(PlxImage, req);
  
  req->reqType = VideoReqCode;
  req->videoReqType = X_PlxShmPutCImage;
  req->drawable = d;
  req->gc = gc->gid;
  req->src_x = src_x;
  req->src_y = src_y;
  req->src_w = src_w;
  req->src_h = src_h;
  req->dst_x = dst_x;
  req->dst_y = dst_y;
  req->dst_w = dst_w;
  req->dst_h = dst_h;
  req->format = flag;						    /* Whether to squeeze the JPEG image or not */
  req->shmseg = shminfo->shmseg;
  req->offset = image->data - shminfo->shmaddr;
  req->sendEvent = send_event;					    /* Whether to send event or not */
  
  req->planeMask = image->size;
  
  POSTAMBLE
    
  return(1);
} /* end function XPlxShmPutCImage */



XShmSegmentInfo* XPlxGetShmInfo(void)
{
  XShmSegmentInfo*	shminfo;
  
  shminfo = (XShmSegmentInfo *) malloc(sizeof(XShmSegmentInfo));
  
  shminfo->shmid = shmget(IPC_PRIVATE, 256 * 1024, IPC_CREAT|0777);
  if (shminfo->shmid < 0)
  {
    perror("shmget");
    free(shminfo);
    return(NULL);
  }
  
  shminfo->shmaddr = (char *)shmat(shminfo->shmid, 0, 0);
  if (shminfo->shmaddr == ((char *)-1))
  {
    perror("shmat");
    free(shminfo);
    return(NULL);
  }
  shminfo->readOnly = False;					    /* This shared memory segment is read-write */
  return(shminfo);
} /* end function XPlxGetShmInfo */



void XPlxFreeShmInfo(XShmSegmentInfo* shminfo)
{
  int buf;
  
  if(shminfo == (XShmSegmentInfo*)NULL)
    return;
  shmdt((void*)shminfo->shmaddr);
  shmctl(shminfo->shmid, IPC_RMID, 0);
  free(shminfo);
} /* end function XPlxFreeShmInfo */



Bool XPlxShmQueryVersion(Display*	dpy,
			 int*		majorVersion,
			 int*		minorVersion,
			 Bool*		sharedPixmaps)
{
  XExtDisplayInfo*		info = find_display (dpy);
  xShmQueryVersionReply		rep;
  register xShmQueryVersionReq*	req;
  
  ShmCheckExtension (dpy, info, False);
  
  LockDisplay(dpy);
  GetReq(ShmQueryVersion, req);
  req->reqType = info->codes->major_opcode;
  req->shmReqType = X_ShmQueryVersion;
  if (!_XReply(dpy, (xReply *)&rep, 0, xFalse)) {
    UnlockDisplay(dpy);
    SyncHandle();
    return False;
  }
  *majorVersion = rep.majorVersion;
  *minorVersion = rep.minorVersion;
  *sharedPixmaps = rep.sharedPixmaps ? True : False;
  UnlockDisplay(dpy);
  SyncHandle();
  return True;
} /* end function XPlxShmQueryVersion */



int XPlxShmGetEventBase(Display*	dpy)
{
  XExtDisplayInfo *info = find_display (dpy);
    
  if (XextHasExtension(info))
  {
    return info->codes->first_event;
  }
  else
  {
    return -1;
  }
} /* end function XPlxShmGetEventBase */



Status XPlxShmAttach(Display*		dpy,
		     XShmSegmentInfo*	shminfo)
{
  XExtDisplayInfo*		info = find_display (dpy);
  register xShmAttachReq*	req;
  
  ShmCheckExtension (dpy, info, 0);
  
  shminfo->shmseg = XAllocID(dpy);
  LockDisplay(dpy);
  GetReq(ShmAttach, req);
  req->reqType = info->codes->major_opcode;
  req->shmReqType = X_ShmAttach;
  req->shmseg = shminfo->shmseg;
  req->shmid = shminfo->shmid;
  req->readOnly = shminfo->readOnly ? xTrue : xFalse;
  UnlockDisplay(dpy);
  SyncHandle();
  return 1;
} /* end function XPlxShmAttach */



Status XPlxShmDetach(Display*		dpy,
		     XShmSegmentInfo*	shminfo)
{
  XExtDisplayInfo*		info = find_display (dpy);
  register xShmDetachReq*	req;
  
  ShmCheckExtension (dpy, info, 0);
  
  LockDisplay(dpy);
  GetReq(ShmDetach, req);
  req->reqType = info->codes->major_opcode;
  req->shmReqType = X_ShmDetach;
  req->shmseg = shminfo->shmseg;
  UnlockDisplay(dpy);
  SyncHandle();
  return 1;
} /* end function XPlxShmDetach */
