/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Header: /plx/source/Xclients/generic/RCS/XPlxXvShmComp.c,v 1.19 1995/12/20 00:59:15 drapeau Exp $ */
/* $Log: XPlxXvShmComp.c,v $
 * Revision 1.19  1995/12/20  00:59:15  drapeau
 * Enhanced the library to look for both video extensions: the library
 * functions now first look for the ParallaxVideo (i.e., "XPlx")
 * extension, using the XVideo (i.e., "Xv") extension as a fallback.
 *
 * Also, removed extraneous code now that the functions correctly look
 * for the proper video extension to use in the Display's extension data
 * structure.
 *
 * Also, reformatted code to make it easier to read and edit.
 *
 * Revision 1.18  1995/06/24  00:19:12  drapeau
 * Added support for the function XPlxXPutImage() to replace XPutImage()
 * for the overlay product on the HP.
 *
 * Revision 1.17  1995/05/25  01:59:46  drapeau
 * Optimized the MakeXvShmCompAtoms() function so that Atoms are only
 * created once per Display.  The Atoms are now stored in the Display's
 * extension data structure.
 *
 * Revision 1.16  1995/05/23  06:53:26  drapeau
 * Major changes to this module to better support multiple open Display's
 * within a single process.
 *
 * Also, cleaned out obsolete code for unsupported platforms.
 *
 * Revision 1.15  1994/10/22  05:30:55  drapeau
 * Made the code more ANSI-compatible.
 *
 * Revision 1.14  1994/10/14  01:02:44  drapeau
 * Fixed XPlxShmGetCImage(), assuring that compression requests will
 * conform to the JPEG baseline specification of 8x8 blocks of pixels
 * (the width & height dimensions are rounded off to the nearest non-zero
 * multiple of 8).
 *
 * Revision 1.13  1994/09/03  05:58:18  drapeau
 * Changed the order of includes to support function prototypes.
 *
 * Revision 1.12  1994/03/29  01:35:03  drapeau
 * Major changes to re-implement the XPlx->Xv protocol translation for
 * compression requests.  Andy added support for compression in the Xv
 * library, so the corresponding XPlx requests changed.
 *
 * Revision 1.11  1993/12/15  06:12:46  drapeau
 * Slight modifications to make the code more portable, and self-reliant
 * upon the Parallax software release.  Currently, not all workstations
 * release the Xv X extension as part of their distributed software, so
 * Parallax must distribute its own source and header files.  These small
 * changes allow this software to use the Parallax version of Xv.
 *
 * Revision 1.1  1993/11/20  00:40:15  drapeau
 * Initial revision
 * */
static char xPlxXvShmCompRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/XPlxXvShmComp.c,v 1.19 1995/12/20 00:59:15 drapeau Exp $";

#define NEED_EVENTS
#define NEED_REPLIES

#include <stdio.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>

#include	<Xvlibint.h>
#include	<Xvplx.h>
#include	<Xvplxproto.h>					    /* Pick up Parallax-specific protocol requests */
#include	"WindowToPortID.h"
#include	<X11/Xutil.h>
#include	<X11/extensions/XShm.h>
#include	<shmstr.h>
#include	<Xext.h>
#include	<extutil.h>
#include	<XPlxExt.h>


XExtCodes*		_XvCodes;
extern XvPortID		xvJpegPort;				    /* Initialized by "XPlxPutTable()" */


#define	PREAMBLE(stat) \
printf("Commented out PreAmble in XPlxXvShmComp.c\n"); \
return(1);

#define POSTAMBLE \
  UnlockDisplay(dpy); \
  SyncHandle()

static char*	PlxDebugEnvVar = NULL;				    /* Library variable indicating whether to print debug msgs. */
static Bool	plxDebuggingOn = False;				    /* Accompanying internal variable to examine for debug msgs */
static Atom	getCImageAtom;
static Atom	putCImageAtom;
static Atom	putFirstCImageAtom;



static void MakeXvShmCompAtoms(Display* dpy)			    /* Create Atoms to be used for setting Xv attributes */
{
  XvAtomsRec*		atomInfo;
  XExtData*		extensionData;
  XExtData*		newExtensionData;
  Bool			atomInfoFound = False;
  static Bool		firstTimeCalled = True;
  
  if (firstTimeCalled == True)					    /* Only do this work once */
  {
    firstTimeCalled = False;
    PlxDebugEnvVar = getenv("PlxDebug");			    /* Determine whether debugging should be enabled */
    if ((char*)NULL != PlxDebugEnvVar)				    /* Did the process have debugging enabled? */
    {
      plxDebuggingOn = True;					    /* Yes, enable debugging for this library */
    }
    else
      plxDebuggingOn = False;					    /* No, don't bother printing debugging msgs for this library */
  }
  for (extensionData = dpy->ext_data;				    /* Look at this Display's extension data for Xv Atoms info */
       extensionData != (XExtData*)NULL;
       extensionData = extensionData->next)
  {
    if (XvPlxAtomsCode == extensionData->number)		    /* Was the Xv Atoms info found? */
    {
      atomInfoFound = True;					    /* Yes, mark a flag saying so,... */
      atomInfo = (XvAtomsRec*)extensionData->private_data;	    /* ...point to the Xv Atoms data,... */
      break;							    /* ...and quit looking. */
    }
    if ((XExtData*)NULL == extensionData->next)			    /* The end of the list was reached */
      break;
  } /* end for (extensionData traversal) */
  if (False == atomInfoFound)					    /* No Xv Atoms data were found, create them and add them... */
  {								    /* ...to the Display's extension data for future use. */
    newExtensionData = Xmalloc(sizeof(XExtData));
    newExtensionData->private_data = Xmalloc(sizeof(XvAtomsRec));
    newExtensionData->free_private = (int (*)())free;
    newExtensionData->number = XvPlxAtomsCode;
    newExtensionData->next = (struct _XExtData*)NULL;
    if (extensionData != (XExtData*)NULL)			    /* If this Display already had extension info, add the... */
    {								    /* ...Xv Atoms info to the end of the extension info list. */
      newExtensionData->next = extensionData->next;
      extensionData->next = newExtensionData;
    }
    else							    /* This Display had no extension info, so add the Plx... */
      dpy->ext_data = newExtensionData;				    /* ...info as the only extension info so far. */
    atomInfo = (XvAtomsRec*)newExtensionData->private_data;	    /* Point to the Xv Atoms extension info for filling in */
    atomInfo->tableCompression =
      XInternAtom(dpy,"XV_PLX_TABLE_COMPRESSION",False);
    atomInfo->tableDecompression =
      XInternAtom(dpy,"XV_PLX_TABLE_DECOMPRESSION",False);
    atomInfo->getCImage = XInternAtom(dpy,"XV_PLX_GETCIMAGE",False);
    atomInfo->putCImage = XInternAtom(dpy,"XV_PLX_PUTCIMAGE",False);
    atomInfo->firstPutCImage =
      XInternAtom(dpy,"XV_PLX_FIRST_PUTCIMAGE",False);
    atomInfo->status = XInternAtom(dpy, "XV_STATUS", False);
    atomInfo->hue = XInternAtom(dpy,"XV_HUE",False);
    atomInfo->saturation = XInternAtom(dpy,"XV_SATURATION",False);
    atomInfo->brightness = XInternAtom(dpy,"XV_BRIGHTNESS",False);
    atomInfo->contrast = XInternAtom(dpy,"XV_CONTRAST",False);
    atomInfo->encoding = XInternAtom(dpy, "XV_ENCODING", False);
    atomInfo->valueRecall = XInternAtom(dpy, "XV_PLX_VALUE_RECALL", False);
    atomInfo->valueSave = XInternAtom(dpy, "XV_PLX_VALUE_SAVE", False);
    atomInfo->videoParameterString = XInternAtom(dpy, "XV_PLX_VIDEO_PARAMETER_STRING", False);
    atomInfo->querySignals = XInternAtom(dpy, "XV_PLX_QUERY_SIGNALS", False);
    atomInfo->crop = XInternAtom(dpy, "XV_PLX_CROP", False);
    atomInfo->zoom = XInternAtom(dpy, "XV_PLX_ZOOM", False);
    atomInfo->xImage = XInternAtom(dpy, "XV_IMAGE_XIMAGE", False);
  } /* end if atom info wasn't found */
  getCImageAtom = atomInfo->getCImage;
  putCImageAtom = atomInfo->putCImage;
  putFirstCImageAtom = atomInfo->firstPutCImage;
  return;
} /* end function MakeXvShmCompAtoms */


static XExtensionInfo *shm_info;
static /* const */ char *shm_extension_name = SHMNAME;

#define ShmCheckExtension(dpy,i,val) \
  XextCheckExtension (dpy, i, shm_extension_name, val)

/*****************************************************************************
 *                                                                           *
 *			   private utility routines                          *
 *                                                                           *
 *****************************************************************************/

static int close_display();
static char *error_string();
static Bool wire_to_event();
static Status event_to_wire();
static /* const */ XExtensionHooks shm_extension_hooks = {
    NULL,				/* create_gc */
    NULL,				/* copy_gc */
    NULL,				/* flush_gc */
    NULL,				/* free_gc */
    NULL,				/* create_font */
    NULL,				/* free_font */
    close_display,			/* close_display */
    wire_to_event,			/* wire_to_event */
    event_to_wire,			/* event_to_wire */
    NULL,				/* error */
    error_string,			/* error_string */
};

static /* const */ char *shm_error_list[] = {
    "BadShmSeg",			/* BadShmSeg */
};

static XEXT_GENERATE_FIND_DISPLAY (find_display, shm_info, shm_extension_name, 
				   &shm_extension_hooks, ShmNumberEvents, NULL)

static XEXT_GENERATE_CLOSE_DISPLAY (close_display, shm_info)

static XEXT_GENERATE_ERROR_STRING (error_string, shm_extension_name,
				   ShmNumberErrors, shm_error_list)


static Bool wire_to_event(Display*	dpy,
			  XEvent*	re,
			  xEvent*	event)
{
  XExtDisplayInfo *info = find_display (dpy);
  XShmCompletionEvent	*se;
  xShmCompletionEvent	*sevent;
  
  ShmCheckExtension (dpy, info, False);
  
  switch ((event->u.u.type & 0x7f) - info->codes->first_event)
  {
   case ShmCompletion:
    se = (XShmCompletionEvent *) re;
    sevent = (xShmCompletionEvent *) event;
    se->type = sevent->type & 0x7f;
    se->serial = _XSetLastRequestRead(dpy,(xGenericReply *) event);
    se->send_event = (sevent->type & 0x80) != 0;
    se->display = dpy;
    se->drawable = sevent->drawable;
    se->major_code = sevent->majorEvent;
    se->minor_code = sevent->minorEvent;
    se->shmseg = sevent->shmseg;
    se->offset = sevent->offset;
    return True;
  }
  return False;
} /* end function wire_to_event */



static Status event_to_wire(Display*	dpy,
			    XEvent*	re,
			    xEvent*	event)
{
  XExtDisplayInfo *info = find_display (dpy);
  XShmCompletionEvent	*se;
  xShmCompletionEvent	*sevent;
  
  ShmCheckExtension (dpy, info, 0);
  
  switch ((re->type & 0x7f) - info->codes->first_event)
  {
   case ShmCompletion:
    se = (XShmCompletionEvent *) re;
    sevent = (xShmCompletionEvent *) event;
    sevent->type = se->type | (se->send_event ? 0x80 : 0);
    sevent->sequenceNumber = se->serial & 0xffff;
    sevent->drawable = se->drawable;
    sevent->majorEvent = se->major_code;
    sevent->minorEvent = se->minor_code;
    sevent->shmseg = se->shmseg;
    sevent->offset = se->offset;
    return True;
  }
  return False;
} /* end function event_to_wire */



Status XPlxXvShmGetCImage(Display*	dpy,
			  Drawable	d,
			  GC		gc,
			  XPlxCImage*	image,
			  int		src_x,
			  int		src_y,
			  unsigned int	src_w,
			  unsigned int	src_h)
{
  XExtDisplayInfo*		info = (XExtDisplayInfo *) find_display (dpy);
  XShmSegmentInfo*		shminfo = (XShmSegmentInfo *) image->info;
  xvPlxGetImageReply		rep;				    /* Reply header buffer. */
  register xvPlxImageReq*	req;
  char*				data;
  long				nbytes;
  WindowToPortID*		windowToPortIDMap;
  
  if ((src_w < 8) || (src_h < 8))				    /* Is requested image size too small any parameter? */
    return((Status)False);					    /* Yes, don't bother trying to compress */
  windowToPortIDMap = WindowToPortIDFind(d, windowToPortIDList);    /* Is there a mapping b/w requested Window and an XvPort? */
  if (windowToPortIDMap == (WindowToPortID*)NULL)		    /* No, this Window is not on the Parallax side of the... */
    return(0);							    /* ...frame buffer, so it cannot be compressed.  Return now. */
  
  ShmCheckExtension(dpy, info, 0);
  if(!shminfo) return 0;
  
  MakeXvShmCompAtoms(dpy);					    /* Assure that the Xv Atoms are known to the X server */
  PREAMBLE(XvBadExtension);
  XvGetReq(PlxImage, req);
  printf("XPlxShmGetCImage() May not work because length is set incorrectly.\n");
  printf("Need to check with Andy about what req->length should be set to.\n");
  /*
   * first set up the standard stuff in the request
   */
  printf("XPlxShmGetCImage(): Need to make image conform to 8x8 pixel blocks.\n");
  req->drawable = d;
  req->gc = gc->gid;
  req->src_x = src_x;
  req->src_y = src_y;
  req->src_w = src_w;
  req->src_h = src_h;
  req->dst_x = 0;
  req->dst_y = 0;
  req->dst_w = image->width;
  req->dst_h = image->height;
  req->shmseg = shminfo->shmseg;
  req->offset = image->data - shminfo->shmaddr;
  req->attribute = getCImageAtom;
  req->xvReqType = xv_PlxImage;
  req->port = windowToPortIDMap->xvPortID;			    /* Use the port for this window for the compression */
  
  if (_XReply (dpy, (xReply *) &rep, 0, xFalse) == 0)
  {
    UnlockDisplay(dpy);
    SyncHandle();
    return (0);
  }
  image->size = rep.size;
  POSTAMBLE;
  return (1);
} /* end function XPlxXvShmGetCImage */



Status XPlxXvShmPutCImage(Display*	dpy,
			  Drawable	d,
			  GC		gc,
			  XPlxCImage*	image,
			  int		src_x,
			  int		src_y,
			  unsigned int	src_w,
			  unsigned int	src_h,
			  int		dst_x,
			  int		dst_y,
			  unsigned int	dst_w,
			  unsigned int	dst_h,
			  int		flag,
			  Bool		send_event)
{
  XExtDisplayInfo*		info = (XExtDisplayInfo *) find_display (dpy);
  XShmSegmentInfo*		shminfo = (XShmSegmentInfo *) image->info;
  register xvPlxImageReq*	req;
  WindowToPortID*		windowToPortIDMap;
  
  ShmCheckExtension (dpy, info, 0);
  if (!shminfo) return 0;
  
  if ((image->info == NULL) || (image->size == 0))
    return 0;
  windowToPortIDMap = WindowToPortIDFind(d, windowToPortIDList);    /* Is there a mapping b/w requested Window and an XvPort? */
  if (windowToPortIDMap == (WindowToPortID*)NULL)		    /* No, this Window is not on the Parallax side of the... */
    return(0);							    /* ...frame buffer, so it cannot be compressed.  Return now. */
  
  MakeXvShmCompAtoms(dpy);					    /* Assure that the Xv Atoms are known to the X server */
  PREAMBLE(XvBadExtension);
  XvGetReq(PlxImage, req);
  
  req->drawable = d;
  req->gc = gc->gid;
  req->src_x = src_x;
  req->src_y = src_y;
  req->src_w = src_w;
  req->src_h = src_h;
  req->dst_x = dst_x;
  req->dst_y = dst_y;
  req->dst_w = dst_w;
  req->dst_h = dst_h;
  req->format = flag;					    /* Squeezed or not */
  req->shmseg = shminfo->shmseg;
  req->offset = image->data - shminfo->shmaddr;
  req->sendEvent = send_event;				    /* Whether to send event or not */
  req->planeMask = image->size;
  req->attribute = putCImageAtom;
  req->xvReqType = xv_PlxImage;
  req->port = windowToPortIDMap->xvPortID;			    /* Use the port for this window for the compression */
  
  POSTAMBLE;
  return(1);
} /* end function XPlxXvShmPutCImage */
