/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Dave Arnold
 */
/* $Header: /plx/source/Xclients/generic/RCS/InputCascade.c,v 1.8 1996/01/11 01:39:35 lovvik Exp $ */
/* $Log: InputCascade.c,v $
 * Revision 1.8  1996/01/11  01:39:35  lovvik
 *   Made a couple of modifications to XPlxCreateInputCascade():
 *     1.  If the hardware information passed into this function
 * 	is NULL, return a NULL structure.  (This fixes a
 * 	seg-fault on non-Parallax hardware).
 *     2.  Get the format instance information from the format_list.
 * 	This was moved to the input_list at one point.
 *
 * Revision 1.7  1996/01/08  19:38:16  lovvik
 *   Changed PlxInputCascadeCompress() to PlxCompressInputCascade().
 *
 * Revision 1.6  1996/01/07  04:40:58  lovvik
 *   2 changes:
 *     1. Added a new function PlxInputCascadeCompress() that removes
 *        unnecessary menu cascade buttons.
 *     2. Changed the logic in PlxInputCascadeGetChild() so it can
 *        correctly traverse the menu even if the menu has been
 *        compressed.
 *
 * Revision 1.5  1995/12/29  20:19:51  lovvik
 * Modified PlxCreateInputCascade() to use unique widget names
 * for all pulldown menus.
 *
 * Added PlxInputCascadeGetChild(), which will retrieve any widget
 * in the input cascade that was created with PlxCreateInputCascade.
 *
 * Fixed compiler warnings.
 *
 * Revision 1.4  1995/12/23  02:21:49  drapeau
 * Part-way modification of the InputCascade creation to attempt to
 * handle instance information in its new location.  The new location is
 * in the channel data structure rather than in the signal's interlaced
 * field.
 *
 * Note: This modification seems to work for overlay platforms (at least,
 * for the HP platform on which it was tested) but not for intergrated
 * frame buffers and the XVideo card.
 *
 * Revision 1.3  1995/10/08  05:52:16  drapeau
 * Fixed error in include directive: was picking up the wrong include
 * file.
 *
 * Revision 1.2  1995/05/16  02:48:21  drapeau
 * Made some cosmetic variable name changes so as to better avoid
 * conflict with variable names for buttons and menus that might be used
 * by application programmers.
 *
 * Also, commented out a call to XtManageChild() on the plxInputMenu,
 * since it was causing a warning on HP systems.  The removal of this
 * call seems to have no effect.
 *
 * Revision 1.1  95/05/15  18:28:32  18:28:32  lovvik ()
 * Changed 'pvic.h' to InputCascade.h to reflect the
 * filename change.
 * 
 * Revision 1.0  1995/05/15  17:02:30  drapeau
 * Initial revision
 * */
static char pvicRcsId[] = "$Header: /plx/source/Xclients/generic/RCS/InputCascade.c,v 1.8 1996/01/11 01:39:35 lovvik Exp $";

#include <stdio.h>
#include <stdlib.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <Xm/Xm.h>
#include <Xm/Label.h>
#include <Xm/CascadeB.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <XPlxExt.h>
#include <IOCascade.h>

static char *inputnames[MAXINPUTS] =
{
    "input0Button",
    "input1Button",
    "input2Button",
    "input3Button",
    "input4Button",
    "input5Button",
    "input6Button",
    "input7Button",
};

static char* inputStrings[MAXINPUTS] =
{
    "Off",
    "Input 1",
    "Input 2",
    "Input 3",
    "Input 4",
    "Input 5",
    "Input 6",
    "Input 7",
};

static char *formatnames[MAXFORMATS] =
{
	"format0Button",
	"format1Button",
	"format2Button",
	"format3Button",
	"format4Button",
	"format5Button",
	"format6Button",
	"format7Button",
};

static char *formatStrings[MAXFORMATS] =
{
	"(unknown format 0)",
	"(unknown format 1)",
	"Composite",
	"S-VHS (YC)",
	"YUV",
	"RGB",
	"(unknown format 6)",
	"(unknown format 7)",
};

static char *instancenames[MAXINSTANCES] =
{
	"instance0Button",
	"instance1Button",
	"instance2Button",
	"instance3Button",
	"instance4Button",
	"instance5Button",
	"instance6Button",
	"instance7Button",
};

static char* instanceStrings[MAXINSTANCES] =
{
	"1",
	"2",
	"3",
	"4",
	"5",
	"6",
	"7",
	"8",
};


static char *standardnames[MAXSTANDARDS] =
{
	"standard0Button",
	"standard1Button",
	"standard2Button",
	"standard3Button",
	"standard4Button",
	"standard5Button",
	"standard6Button",
	"standard7Button",
};

static char* standardStrings[MAXSTANDARDS] =
{
	"(unknown standard 0)",
	"(unknown standard 1)",
	"NTSC",
	"(unknown standard 3)",
	"PAL",
	"SECAM",
	"(unknown standard 6)",
	"(unknown standard 7)",
};


static Widget plxInputMenuBar = NULL;
static Widget inputCascade = NULL;
static Widget inputTypeCascade = NULL;
static Widget inputTypeMenuBar = NULL;
static Widget signalStandardCascade = NULL;
static Widget signalInstanceCascade = NULL;
static Widget inputChannelCascade[MAXINPUTS];

void tcallback(w, cd, ca)
Widget w;
void *cd, *ca;
{
	printf("in tcallback, cd = %x\n", (int)cd);
}

void PlxShowInputSelections(p_me, ss)
     PlxInputCascade *p_me;
     signalSpec ss;
{
  int ac;
  Arg al[64];							    /* Arg List */
  XmString      label;
  tinput *input;
  tformat *formatCascades;
  input = p_me->inputs[ss.s.channel];
  formatCascades = &input->formats[0];
  
  if((ss.s.channel > MAXINPUTS) ||(ss.s.format >= MAXFORMATS) ||    /* bounds check */
     (ss.s.instance >= MAXINSTANCES) || (ss.s.standard >= MAXSTANDARDS))
    return;
  if(inputCascade)
  {
    ac = 0;
    XtSetArg(al[ac], XmNlabelString, &label); ac++;
    XtGetValues(inputChannelCascade[ss.s.channel], al, ac);
    ac = 0;
    XtSetArg(al[ac], XmNlabelString, label); ac++;
    XtSetValues(inputCascade, al, ac);
  }
  if(inputTypeCascade)
  {
    ac = 0;
    XtSetArg(al[ac], XmNlabelString, &label); ac++;
    XtGetValues(formatCascades[ss.s.format].w, al, ac);
    ac = 0;
    XtSetArg(al[ac], XmNlabelString, label); ac++;
    XtSetArg(al[ac], XmNsubMenuId, input->cascade); ac++;
    XtSetValues(inputTypeCascade, al, ac);
  }
  if(signalInstanceCascade)
  {
    ac = 0;
    XtSetArg(al[ac], XmNlabelString, &label); ac++;
    XtGetValues(formatCascades[ss.s.format].instances[ss.s.instance].w, al, ac);
    ac = 0;
    XtSetArg(al[ac], XmNlabelString, label); ac++;
    XtSetArg(al[ac], XmNsubMenuId, formatCascades[ss.s.format].pw); ac++;
    XtSetValues(signalInstanceCascade, al, ac);
  }
  if(signalStandardCascade)
  {
    ac = 0;
    XtSetArg(al[ac], XmNlabelString, &label); ac++;
    XtGetValues(formatCascades[ss.s.format].instances[ss.s.instance].standards[ss.s.standard], al, ac);
    ac = 0;
    XtSetArg(al[ac], XmNlabelString, label); ac++;
    XtSetArg(al[ac], XmNsubMenuId, formatCascades[ss.s.format].instances[ss.s.instance].pw); ac++;
    XtSetValues(signalStandardCascade, al, ac);
  }
}



PlxInputCascade *PlxCreateInputCascade(Widget mparent, Widget fparent, Widget iparent, 
				       Widget sparent, plx_IO* plxio, XtCallbackProc callback)
{
  Arg 			al[64];					    /* Arg List */
  char 			menuName[255];
  int 			i;
  int			j;
  int			k;
  int 			channel;
  int 			instance;
  PlxInputCascade 	*ic;
  plx_IO_list 		*ip;
  plx_signal_list 	*sp;
  plx_format_list 	*fp;
  register int 		ac = 0;					    /* Arg Count */
  signalSpec		ss;
  tformat 		*formatCascades;
  unsigned char 	rctype;
  Widget 		inputLabel 		=	(Widget)NULL;
  Widget 		inputTypeLabel 		= 	(Widget)NULL;
  Widget 		signalInstanceMenuBar 	= 	(Widget)NULL;
  Widget 		signalInstanceLabel 	= 	(Widget)NULL;
  Widget 		signalStandardMenu 	= 	(Widget)NULL;
  Widget 		signalStandardLabel 	= 	(Widget)NULL;
  Widget 		inputTypeMenu 		= 	(Widget)NULL;
  Widget 		plxInputMenu 		= 	(Widget)NULL;
  Widget		parentWidget 		= 	(Widget)NULL;
  Widget 		firstAvailableButton 	= 	(Widget)NULL;
  XmString      	label;

  if(plxio == NULL)
  {
    /*
     * No hardware information - This is not a Parallax video board.
     */
    return((PlxInputCascade*)NULL);
  }
  
  if((mparent == NULL) && (fparent == NULL)) 
  {
    fprintf(stderr, "PlxCreateInputCascade called with no parents\n");
    return NULL;
  }
  ic = (PlxInputCascade *) malloc(sizeof(PlxInputCascade));
  plxInputMenu = NULL;

  if(mparent)
  {
    ac = 0;
    XtSetArg(al[ac], XmNrowColumnType, &rctype); ++ac;
    XtGetValues(mparent, al, ac);
    if(rctype == XmWORK_AREA)
    {
      inputLabel = (Widget) XmCreateLabel ( mparent, "inputLabel", NULL, 0 );
      label = XmStringCreateLocalized("Input:");
      XtVaSetValues(inputLabel, XmNlabelString, label, NULL);
      XmStringFree(label);
      plxInputMenuBar = (Widget) XmCreateMenuBar ( mparent, "plxInputMenuBar", NULL, 0 );
      inputCascade = XmCreateCascadeButton ( plxInputMenuBar, "inputCascade", NULL, 0 );
      plxInputMenu = XmCreatePulldownMenu( plxInputMenuBar, "plxInputMenu", NULL, 0 );
      label = XmStringCreateLocalized("Input George");
      XtVaSetValues(inputCascade, XmNlabelString, label, NULL);
      XmStringFree(label);
    }
  }
  
  if(fparent)
  {
    inputTypeLabel = (Widget) XmCreateLabel ( fparent, "inputTypeLabel", NULL, 0 );
    inputTypeMenuBar = (Widget) XmCreateMenuBar ( fparent, "inputTypeMenuBar", NULL, 0 );
    inputTypeCascade = XmCreateCascadeButton ( inputTypeMenuBar, "inputTypeCascade", NULL, 0 );
  }
  
  if(iparent)
  {
    signalInstanceLabel = (Widget) XmCreateLabel ( iparent, "signalInstanceLabel", NULL, 0 );
    signalInstanceMenuBar = XmCreateMenuBar ( iparent, "signalInstanceMenuBar", al, ac );
    signalInstanceCascade = XmCreateCascadeButton ( signalInstanceMenuBar, "signalInstanceCascade", NULL, 0 );
  }
  
  if(sparent)
  {
    signalStandardLabel = (Widget) XmCreateLabel ( sparent, "signalStandardLabel", NULL, 0 );
    signalStandardMenu = XmCreateMenuBar ( sparent, "signalStandardMenu", al, ac );
    signalStandardCascade = XmCreateCascadeButton ( signalStandardMenu, "signalStandardCascade", NULL, 0 );
  }
  
  for(channel = 0; channel < MAXINPUTS; ++channel)
  {
    inputChannelCascade[channel] = NULL;
    ic->inputs[channel] = NULL;
    ss.s.channel = channel;
    inputTypeMenu = NULL;
    for(ip = plxio->inputs; ip; ip = ip->next_IO)
    {
      if(ip->bundle != 0)					    /* Does this channel have bundle information? */
      {
	fprintf(stderr,						    /* Yes - code not implemented yet. */
		"Error! Bundle information not yet implemented.\n");
      }
      if(ip->channel != channel)				    /* channels don't match, check next */
	continue;
      
      if(ic->inputs[channel] == NULL)
      {
	ic->inputs[channel] = (tinput *) malloc(sizeof(tinput));
	formatCascades = &ic->inputs[channel]->formats[0];
	for(i = 0; i < MAXFORMATS; ++i)
	  formatCascades[i].w = NULL;
      }
      
      if(mparent && (inputChannelCascade[channel] == NULL))
      {
	if(plxInputMenu)
	  inputChannelCascade[channel] = XmCreateCascadeButton ( plxInputMenu, inputnames[channel], NULL, 0 );
	else
	  inputChannelCascade[channel] = XmCreateCascadeButton ( mparent, inputnames[channel], NULL, 0 );
	label = XmStringCreateLocalized(inputStrings[channel]);
	XtVaSetValues(inputChannelCascade[channel], XmNlabelString, label, NULL);
	XmStringFree(label);
	inputTypeMenu = XmCreatePulldownMenu( mparent, inputnames[channel], NULL, 0);
      }
      else if(inputTypeMenu == NULL)
      {
	inputTypeMenu = XmCreatePulldownMenu( inputTypeMenuBar, inputnames[channel], NULL, 0);
      }
      
      for(fp = ip->format_list; fp ; fp = fp->next_format)
      {
	i = fp->format;
	ss.s.format = i;
	if(formatCascades[i].w == NULL)
	{
	  formatCascades[i].w = XmCreateCascadeButton ( inputTypeMenu, formatnames[i], NULL, 0 );
	  sprintf(menuName, "%s %s", inputnames[channel], formatnames[i]);
	  formatCascades[i].pw = XmCreatePulldownMenu ( inputTypeMenu, menuName, NULL, 0 );
	  label = XmStringCreateLocalized(formatStrings[i]);
	  XtVaSetValues(formatCascades[i].w, XmNlabelString, label, NULL);
	  XmStringFree(label);
	  for(j = 0; j < MAXINSTANCES; ++j)
	    formatCascades[i].instances[j].w = NULL;
	}
	for(sp = ip->signal_list; sp; sp = sp->next_signal)
	{
	  j = fp->instance;
	  ss.s.instance = j;
	  if(formatCascades[i].instances[j].w == NULL)
	  {
	    formatCascades[i].instances[j].w = XmCreateCascadeButton(formatCascades[i].pw, instancenames[j], NULL, 0);
	    sprintf(menuName, "%s %s %s", inputnames[channel], formatnames[i], instancenames[j]);
	    formatCascades[i].instances[j].pw = XmCreatePulldownMenu(formatCascades[i].pw, menuName, NULL, 0);
	    label = XmStringCreateLocalized(instanceStrings[j]);
	    XtVaSetValues(formatCascades[i].instances[j].w, XmNlabelString, label, NULL);
	    XmStringFree(label);
	    for(k = 0; k < MAXSTANDARDS; ++k)
	      formatCascades[i].instances[j].standards[k] = NULL;
	  }
	  k = sp->signal->standard;
	  ss.s.standard = k;
	  if(formatCascades[i].instances[j].standards[k] == NULL)
	  {
	    formatCascades[i].instances[j].standards[k] =
	      (Widget) XmCreatePushButton(formatCascades[i].instances[j].pw, standardnames[k], NULL, 0 );
	    label = XmStringCreateLocalized(standardStrings[k]);
	    XtVaSetValues(formatCascades[i].instances[j].standards[k], XmNlabelString, label, NULL);
	    XmStringFree(label);
	    XtAddCallback (formatCascades[i].instances[j].standards[k],
			   XmNactivateCallback, callback, (XtPointer)ss.i);
	    XtManageChild(formatCascades[i].instances[j].standards[k]);
	  }
	}
      }
    }
    
    if(ic->inputs[channel])
    {
      for(i = 0; i < MAXFORMATS; ++i)
      {
	if(formatCascades[i].w)
	{
	  for(j = 0; j < MAXINSTANCES; ++j)
	  {
	    if(formatCascades[i].instances[j].w)
	    {
	      ac = 0;
	      XtSetArg(al[ac], XmNsubMenuId, formatCascades[i].instances[j].pw); ac++;
	      XtSetValues ( formatCascades[i].instances[j].w, al, ac );
	      XtManageChild(formatCascades[i].instances[j].w);
	    }
	  }
	  ac = 0;
	  XtSetArg(al[ac], XmNsubMenuId, formatCascades[i].pw); ac++;
	  XtSetValues ( formatCascades[i].w, al, ac );
	  XtManageChild(formatCascades[i].w);
	}
      }
    }
    if(ic->inputs[channel])
    {
      ic->inputs[channel]->channel = channel;
      ic->inputs[channel]->cascade = inputTypeMenu;
    }
    if(inputChannelCascade[channel])
    {
      ac = 0;
      XtSetArg(al[ac], XmNsubMenuId, inputTypeMenu); ac++;
      XtSetValues(inputChannelCascade[channel], al, ac);
      XtManageChild(inputChannelCascade[channel]);
    }
  }
  if (mparent)							    /* Create a "First Available" widget */
  {
    if (plxInputMenu != (Widget)NULL)
      parentWidget = plxInputMenu;
    else
      parentWidget = mparent;
    label = XmStringCreateLocalized("First Available");
    firstAvailableButton = XtVaCreateWidget("firstAvailableInputButton",
					    xmPushButtonWidgetClass,
					    parentWidget,
					    XmNlabelString, label,
					    NULL);
    XmStringFree(label);
    XtAddCallback(firstAvailableButton,				    /* Create a callback that passed a zero in when called;... */
		  XmNactivateCallback,				    /* ...the zero is a special code meaning "take the 1st... */
		  callback,					    /* ...available possible input if you can". */
		  (XtPointer)0);
    XtManageChild(firstAvailableButton);
  }
  if(plxInputMenuBar)
  {
    XtManageChild(inputLabel);
    XtManageChild(plxInputMenuBar);
/*
    XtManageChild(plxInputMenu);
*/
    ac = 0;
    XtSetArg(al[ac], XmNsubMenuId, plxInputMenu); ac++;
    XtSetValues(inputCascade, al, ac);
    XtManageChild(inputCascade);
  }
  
  if(inputTypeMenuBar)
  {
    XtManageChild(inputTypeLabel);
    XtManageChild(inputTypeMenuBar);
    XtManageChild(inputTypeCascade);
  }
  
  if(signalInstanceMenuBar)
  {
    XtManageChild(signalInstanceLabel);
    XtManageChild(signalInstanceMenuBar);
    XtManageChild(signalInstanceCascade);
  }
  
  if(signalStandardMenu)
  {
    XtManageChild(signalStandardLabel);
    XtManageChild(signalStandardMenu);
    XtManageChild(signalStandardCascade);
  }
  return(ic);
}


/*
 * PlxInputCascadeGetChild() is a function that will get any
 * child of the input menu that was generated by this library.
 * Because XtNameToWidget() cannot traverse pulldown menus, it
 * is necessary to extract the XmNsubMenuId from each cascade
 * button and use that as the parent passed into XtNameToWidget().
 *
 * The #defines to specify each widget can be found in IOCascade.h
 */

Widget PlxInputCascadeGetChild(Widget parent, int childId)
{
  /*
   * childId is a composite of:  Input | Format | Instance | Standard
   */

  Widget returnWidget = parent;
  Widget menuWidget;
  char widgetNameString[255] = "\0";
  char menuNameString[255] = "\0";
  char nameString[255] = "\0";
  int inputId;
  int formatId;
  int instanceId;
  int standardId;
  
  inputId = (childId >> 12) & 0x000F;
  formatId = (childId >> 8) & 0x000F;
  instanceId = (childId >> 4) & 0x000F;
  standardId = childId & 0x000F;
  menuWidget = parent;

  switch(childId)
  {
    case IOC_FirstAvailable:
    {
      return(XtNameToWidget(parent, "firstAvailableInputButton"));
      break;
    }
  }
  
  /*
   * Was an input id specified?  If so, get the widget id
   * of that input button.
   */
  if(0 != inputId)
  {
    inputId = 15 - inputId;
    sprintf(widgetNameString, "%s", inputnames[inputId]);
    menuWidget = parent;
    if(inputId >= MAXINPUTS)
    {
      /*
       * Requested the menu.
       */
      return(menuWidget);
    }
  }

  if(0 != formatId)
  {
    Widget widget;
    
    formatId = 15 - formatId;
    widget = XtNameToWidget(menuWidget, widgetNameString);
    if(widget != (Widget)NULL)
    {
      XtVaGetValues(widget,
		    XmNsubMenuId, &menuWidget,
		    NULL);

    }      
    /*
     * Before we look for format buttons, be sure we have the right menu.
     * The menu could be incorrect if menu compression was used.
     */
    sprintf(menuNameString, "%s", inputnames[inputId]);
    if(strcmp(menuNameString, XtName(menuWidget)) == 0)
    {
      /*
       * We have the correct menu.
       */
      sprintf(widgetNameString, "%s", formatnames[formatId]);
      if(formatId >= MAXFORMATS)
      {
	/*
	 * Requested the menu.
	 */
	return(menuWidget);
      }
    }
  }

  if(0 != instanceId)
  {
    Widget widget;
    
    instanceId = 15 - instanceId;

    /*
     * Before we look for instance buttons, be sure we have the format menu.
     * The menu could be incorrect if menu compression was used.
     */
    sprintf(menuNameString, "%s", inputnames[inputId]);
    if(strcmp(menuNameString, XtName(menuWidget)) == 0)
    {
      /*
       * We have the correct menu.  Now, we can get the instance menu.
       */
      sprintf(widgetNameString, "%s", formatnames[formatId]);
      widget = XtNameToWidget(menuWidget, widgetNameString);
      if(widget != (Widget)NULL)
      {
	XtVaGetValues(widget,
		      XmNsubMenuId, &menuWidget,
		      NULL);
      }      
      sprintf(widgetNameString, "%s", instancenames[instanceId]);
      if(instanceId >= MAXINSTANCES)
      {
	/*
	 * Requested the menu.
	 */
	return(menuWidget);
      }
    }
  }

  if(0 != standardId)
  {
    Widget widget;
    
    standardId = 15 - standardId;

    /*
     * Before we look for standard buttons, be sure we have the standard menu.
     * The menu could be incorrect if menu compression was used.
     */
    sprintf(menuNameString, "%s %s", inputnames[inputId], formatnames[formatId]);
    if(strcmp(menuNameString, XtName(menuWidget)) == 0)
    {
      /*
       * We have the Instance menu.  We must get the standards menu.
       */
      sprintf(widgetNameString, "%s", instancenames[instanceId]);
      widget = XtNameToWidget(menuWidget, widgetNameString);
      if(widget != (Widget)NULL)
      {
	XtVaGetValues(widget,
		      XmNsubMenuId, &menuWidget,
		      NULL);
      } 
    }     
    /*
     * Before we look for standard buttons, be sure we have the right menu.
     * The menu could be incorrect if menu compression was used.
     */
    sprintf(menuNameString, "%s %s %s", inputnames[inputId], formatnames[formatId], instancenames[instanceId]);
    if(strcmp(menuNameString, XtName(menuWidget)) == 0)
    {
      /*
       * We have the correct menu.
       */
      sprintf(widgetNameString, "%s", standardnames[standardId]);
      if(standardId >= MAXSTANDARDS)
      {
	/*
	 * Requested the menu.
	 */
	return(menuWidget);
      }      
    }
  }

  sprintf(nameString, "*%s", widgetNameString);

  if(menuWidget != (Widget)NULL)
  {
    returnWidget = XtNameToWidget(menuWidget, nameString);
    return(returnWidget);
  }
  else
  {
    return((Widget)NULL);
  }    
}


void PlxCompressInputCascade(Widget parent)
{
  int		input;
  int		format;
  int		instance;
  int		formatCount;
  int		instanceCount;
  Widget	inputCascadeButton;
  Widget	formatCascadeButton;
  Widget	instanceCascadeButton;
  Widget	formatMenu;
  Widget	instanceMenu;
  Widget	standardMenu;

  for(input = 0; input < MAXINPUTS; input++)
  {
    inputCascadeButton = XtNameToWidget(parent, inputnames[input]);
    if(inputCascadeButton != (Widget)NULL)
    {
      /*
       * The input button exists.  Check it's children to see if
       * they are all needed.
       */
      XtVaGetValues(inputCascadeButton,
		    XmNsubMenuId, &formatMenu,
		    NULL);
      /*
       * Count the format buttons.
       */
      formatCount = 0;
      for(format = 0; format < MAXFORMATS; format++)
      {
	Widget buttonWidget;
	
	buttonWidget = XtNameToWidget(formatMenu, formatnames[format]);
	if((Widget)NULL != buttonWidget)
	{
	  /*
	   * Got a valid button - increment the format button count.
	   */
	  formatCount++;
	}
      }
      
      /*
       * For each format, count the instances.
       */
      for (format = 0; format < MAXFORMATS; format++)
      {
	formatCascadeButton = XtNameToWidget(formatMenu, formatnames[format]);
	if((Widget)NULL != formatCascadeButton)
	{
	  /*
	   * Count the instance buttons.
	   */
	  XtVaGetValues(formatCascadeButton,
			XmNsubMenuId, &instanceMenu,
			NULL);
	  instanceCount = 0;
	  for(instance = 0; instance < MAXINSTANCES; instance++)
	  {
	    if((Widget)NULL != XtNameToWidget(instanceMenu, instancenames[instance]))
	    {
	      /*
	       * Got a valid button - increment the instance button count.
	       */
	      instanceCount++;
	    }
	  }
	  

	  /*
	   * At this point, we know how many formats and instances there are.
	   * We can now do the actual compression.
	   */
	  if(instanceCount < 2)
	  {
	    /*
	     * There is only one instance button.  It can be removed, but first
	     * we have to know which one it is.
	     */
	    for(instance = 0; instance < MAXINSTANCES; instance++)
	    {
	      instanceCascadeButton = XtNameToWidget(instanceMenu, instancenames[instance]);
	      if(instanceCascadeButton != (Widget)NULL)
	      {
		/*
		 * Found the only instance button.  Compress by removing the instance menu.
		 */
		XtVaGetValues(instanceCascadeButton,
			      XmNsubMenuId, &standardMenu,
			      NULL);
		XtVaSetValues(formatCascadeButton,
			      XmNsubMenuId, standardMenu,
			      NULL);
	      }
	    }
	  }
	  
	  if(formatCount < 2)
	  {
	    /*
	     * There is only one format button.
	     */
	    if(inputCascadeButton != (Widget)NULL)
	    {
	      XtVaGetValues(formatCascadeButton,
			    XmNsubMenuId, &instanceMenu,
			    NULL);
	      XtVaSetValues(inputCascadeButton,
			    XmNsubMenuId, instanceMenu,
			    NULL);
	    }
	  }
	}
      }
    }
  }
}
