/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Dave Arnold
 */
/* $Header: /plx/source/Xclients/generic/RCS/OutputCascade.c,v 1.4 1996/01/11 01:39:04 lovvik Exp $ */
/* $Log: OutputCascade.c,v $
 * Revision 1.4  1996/01/11  01:39:04  lovvik
 *   Made a couple of modifications to XPlxCreateOutputCascade():
 *     1.  If the hardware information passed into this function
 * 	is NULL, return a NULL structure.  (This fixes a
 * 	seg-fault on non-Parallax hardware).
 *     2.  Get the format instance information from the format_list.
 * 	This was moved to the input_list at one point.
 *
 * Revision 1.3  1996/01/08  19:39:19  lovvik
 *   Changed PlxOutputCascadeCompress() to PlxCompressOutputCascade().
 *
 * Revision 1.2  1996/01/07  04:46:18  lovvik
 *    2 changes:
 *      1. Added a new function PlxOutputCascadeCompress() that removes
 *         unnecessary menu cascade buttons.
 *      2. Added a new function PlxOutputGetChild() that can return any
 * 	menu component that was generated with PlxCreateOutputCascade()
 * 	even if the menu has been compressed.
 *
 * Revision 1.1  1995/10/07  00:56:09  drapeau
 * Initial revision
 * */
static char outputCascadeRcsId[] = "$Id: OutputCascade.c,v 1.4 1996/01/11 01:39:04 lovvik Exp $";


#include <stdio.h>
#include <stdlib.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <Xm/Xm.h>
#include <Xm/Label.h>
#include <Xm/CascadeB.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <XPlxExt.h>
#include "IOCascade.h"

static char *outputnames[MAXOUTPUTS] =
{
    "output0Button",
    "output1Button",
    "output2Button",
    "output3Button",
    "output4Button",
    "output5Button",
    "output6Button",
    "output7Button",
};

static char* outputStrings[MAXOUTPUTS] =
{
    "Off",
    "Output 1",
    "Output 2",
    "Output 3",
    "Output 4",
    "Output 5",
    "Output 6",
    "Output 7",
};

static char *formatnames[MAXFORMATS] =
{
	"format0Button",
	"format1Button",
	"format2Button",
	"format3Button",
	"format4Button",
	"format5Button",
	"format6Button",
	"format7Button",
};

static char *formatStrings[MAXFORMATS] =
{
	"(unknown format 0)",
	"(unknown format 1)",
	"Composite",
	"S-VHS (YC)",
	"YUV",
	"RGB",
	"(unknown format 6)",
	"(unknown format 7)",
};

static char *instancenames[MAXINSTANCES] =
{
	"instance0Button",
	"instance1Button",
	"instance2Button",
	"instance3Button",
	"instance4Button",
	"instance5Button",
	"instance6Button",
	"instance7Button",
};

static char* instanceStrings[MAXINSTANCES] =
{
	"1",
	"2",
	"3",
	"4",
	"5",
	"6",
	"7",
	"8",
};

static char *standardnames[MAXSTANDARDS] =
{
	"standard0Button",
	"standard1Button",
	"standard2Button",
	"standard3Button",
	"standard4Button",
	"standard5Button",
	"standard6Button",
	"standard7Button",
};

static char* standardStrings[MAXSTANDARDS] =
{
	"(unknown standard 0)",
	"(unknown standard 1)",
	"NTSC",
	"(unknown standard 3)",
	"PAL",
	"SECAM",
	"(unknown standard 6)",
	"(unknown standard 7)",
};


static Widget plxOutputMenuBar = NULL;
static Widget outputCascade = NULL;
static Widget outputTypeCascade = NULL;
static Widget outputTypeMenuBar = NULL;
static Widget signalStandardCascade = NULL;
static Widget signalInstanceCascade = NULL;
static Widget outputChannelCascade[MAXOUTPUTS];

void tocallback(w, cd, ca)
Widget w;
void *cd, *ca;
{
	printf("in tocallback, cd = %x\n", (int)cd);
}

void PlxShowOutputSelections(PlxOutputCascade*	p_me,
			     signalSpec		ss)
{
  int		ac;
  Arg		al[64];						    /* Arg List */
  int		firstOutputIndex;
  XmString      label;
  toutput*	output;
  tformat*	formatCascades;
  
  firstOutputIndex = (0 == ss.s.channel) ? 0 :			    /* Since PLX_OUTPUT_0 begins at 32, adjust so that the... */
    ss.s.channel - (PLX_OUTPUT_0 - 1);				    /* ...1st output is the 2nd element of the outputs array. */
  output = p_me->outputs[firstOutputIndex];
  formatCascades = &output->formats[0];
  if(((firstOutputIndex) > MAXOUTPUTS) ||			    /* bounds check */
     (ss.s.format >= MAXFORMATS) ||
     (ss.s.instance >= MAXINSTANCES) ||
     (ss.s.standard >= MAXSTANDARDS))
    return;
  if(outputCascade)
  {
    ac = 0;
    XtSetArg(al[ac], XmNlabelString, &label); ac++;
    XtGetValues(outputChannelCascade[firstOutputIndex], al, ac);
    ac = 0;
    XtSetArg(al[ac], XmNlabelString, label); ac++;
    XtSetValues(outputCascade, al, ac);
  }
  if(outputTypeCascade)
  {
    ac = 0;
    XtSetArg(al[ac], XmNlabelString, &label); ac++;
    XtGetValues(formatCascades[ss.s.format].w, al, ac);
    ac = 0;
    XtSetArg(al[ac], XmNlabelString, label); ac++;
    XtSetArg(al[ac], XmNsubMenuId, output->cascade); ac++;
    XtSetValues(outputTypeCascade, al, ac);
  }
  if(signalInstanceCascade)
  {
    ac = 0;
    XtSetArg(al[ac], XmNlabelString, &label); ac++;
    XtGetValues(formatCascades[ss.s.format].instances[ss.s.instance].w, al, ac);
    ac = 0;
    XtSetArg(al[ac], XmNlabelString, label); ac++;
    XtSetArg(al[ac], XmNsubMenuId, formatCascades[ss.s.format].pw); ac++;
    XtSetValues(signalInstanceCascade, al, ac);
  }
  if(signalStandardCascade)
  {
    ac = 0;
    XtSetArg(al[ac], XmNlabelString, &label); ac++;
    XtGetValues(formatCascades[ss.s.format].instances[ss.s.instance].standards[ss.s.standard], al, ac);
    ac = 0;
    XtSetArg(al[ac], XmNlabelString, label); ac++;
    XtSetArg(al[ac], XmNsubMenuId, formatCascades[ss.s.format].instances[ss.s.instance].pw); ac++;
    XtSetValues(signalStandardCascade, al, ac);
  }
} /* end function PlxShowOutputSelections */



PlxOutputCascade *PlxCreateOutputCascade(mparent, fparent, iparent, sparent, plxio, callback)
     Widget mparent;
     Widget fparent;
     Widget iparent;
     Widget sparent;
     plx_IO *plxio;
     XtCallbackProc callback;
{
  Arg			al[64];					    /* Arg List */
  char			menuName[255];
  int 			i;
  int			j;
  int			k;
  int			channel;
  int			outputIndex;
  PlxOutputCascade*	ic;
  plx_IO_list*		ip;
  plx_signal_list*	sp;
  plx_format_list*	fp;
  register int		ac = 0;					    /* Arg Count */
  signalSpec   		ss;
  tformat*		formatCascades;
  unsigned char		rctype;
  Widget		outputLabel = (Widget)NULL;
  Widget		outputTypeLabel = (Widget)NULL;
  Widget		signalInstanceMenuBar = (Widget)NULL;
  Widget		signalInstanceLabel = (Widget)NULL;
  Widget		signalStandardMenu = (Widget)NULL;
  Widget		signalStandardLabel = (Widget)NULL;
  Widget		outputTypeMenu = (Widget)NULL;
  Widget		plxOutputMenu = (Widget)NULL;
  XmString		label;

  if(plxio == NULL)
  {
    /*
     * No hardware information - This is not a Parallax video board.
     */
    return((PlxOutputCascade*)NULL);
  }
  
  if((mparent == NULL) && (fparent == NULL)) 
  {
    fprintf(stderr, "PlxCreateOutputCascade called with no parents\n");
    return NULL;
  }
  ic = (PlxOutputCascade *) malloc(sizeof(PlxOutputCascade));
  plxOutputMenu = NULL;
  
  if(mparent)
  {
    ac = 0;
    XtSetArg(al[ac], XmNrowColumnType, &rctype); ++ac;
    XtGetValues(mparent, al, ac);
    if(rctype == XmWORK_AREA)
    {
      outputLabel = (Widget) XmCreateLabel ( mparent, "outputLabel", NULL, 0 );
      label = XmStringCreateLocalized("Output:");
      XtVaSetValues(outputLabel, XmNlabelString, label, NULL);
      XmStringFree(label);
      plxOutputMenuBar = (Widget) XmCreateMenuBar ( mparent, "plxOutputMenuBar", NULL, 0 );
      outputCascade = XmCreateCascadeButton ( plxOutputMenuBar, "outputCascade", NULL, 0 );
      plxOutputMenu = XmCreatePulldownMenu( plxOutputMenuBar, "plxOutputMenu", NULL, 0 );
      label = XmStringCreateLocalized("Output...");
      XtVaSetValues(outputCascade, XmNlabelString, label, NULL);
      XmStringFree(label);
    }
  }
  
  if(fparent)
  {
    outputTypeLabel = (Widget) XmCreateLabel ( fparent, "outputTypeLabel", NULL, 0 );
    outputTypeMenuBar = (Widget) XmCreateMenuBar ( fparent, "outputTypeMenuBar", NULL, 0 );
    outputTypeCascade = XmCreateCascadeButton ( outputTypeMenuBar, "outputTypeCascade", NULL, 0 );
  }
  
  if(iparent)
  {
    signalInstanceLabel = (Widget) XmCreateLabel ( iparent, "signalInstanceLabel", NULL, 0 );
    signalInstanceMenuBar = XmCreateMenuBar ( iparent, "signalInstanceMenuBar", al, ac );
    signalInstanceCascade = XmCreateCascadeButton ( signalInstanceMenuBar, "signalInstanceCascade", NULL, 0 );
  }
  
  if(sparent)
  {
    signalStandardLabel = (Widget) XmCreateLabel ( sparent, "signalStandardLabel", NULL, 0 );
    signalStandardMenu = XmCreateMenuBar ( sparent, "signalStandardMenu", al, ac );
    signalStandardCascade = XmCreateCascadeButton ( signalStandardMenu, "signalStandardCascade", NULL, 0 );
  }
  
  for(channel = 0; channel < MAXOUTPUTS; ++channel)
  {
    outputIndex = (0 == channel) ? 0 :				    /* Since PLX_OUTPUT_0 begins at 32, adjust so that the... */
      channel + (PLX_OUTPUT_0 - 1);				    /* ...1st output is the 2nd element of the outputs array. */
    
    outputChannelCascade[channel] = NULL;
    ic->outputs[channel] = NULL;
    ss.s.channel = outputIndex;
    outputTypeMenu = NULL;
    for(ip = plxio->outputs; ip; ip = ip->next_IO)
    {
      if(ip->bundle != 0)					    /* Does this channel have bundle information? */
      {
	fprintf(stderr,						    /* Yes - code not implemented yet. */
		"Error! Bundle information not yet implemented.\n");
      }
      if(ip->channel != outputIndex)				    /* channels don't match, check next */
	continue;
      
      if(ic->outputs[channel] == NULL)
      {
	ic->outputs[channel] = (toutput *) malloc(sizeof(toutput));
	formatCascades = &ic->outputs[channel]->formats[0];
	for(i = 0; i < MAXFORMATS; ++i)
	  formatCascades[i].w = NULL;
      }
      
      if(mparent && (outputChannelCascade[channel] == NULL))
      {
	if(plxOutputMenu)
	  outputChannelCascade[channel] = XmCreateCascadeButton ( plxOutputMenu, outputnames[channel], NULL, 0 );
	else
	  outputChannelCascade[channel] = XmCreateCascadeButton ( mparent, outputnames[channel], NULL, 0 );
	label = XmStringCreateLocalized(outputStrings[channel]);
	XtVaSetValues(outputChannelCascade[channel], XmNlabelString, label, NULL);
	XmStringFree(label);
	outputTypeMenu = XmCreatePulldownMenu( mparent, outputnames[channel], NULL, 0);
      }
      else if(outputTypeMenu == NULL)
      {
	outputTypeMenu = XmCreatePulldownMenu( outputTypeMenuBar, outputnames[channel], NULL, 0);
      }
      
      for(fp = ip->format_list; fp ; fp = fp->next_format)
      {
	i = fp->format;
	ss.s.format = i;
	if(formatCascades[i].w == NULL)
	{
	  formatCascades[i].w = XmCreateCascadeButton ( outputTypeMenu, formatnames[i], NULL, 0 );
	  sprintf(menuName, "%s %s", outputnames[channel], formatnames[i]);
	  formatCascades[i].pw = XmCreatePulldownMenu ( outputTypeMenu, menuName, NULL, 0 );
	  label = XmStringCreateLocalized(formatStrings[i]);
	  XtVaSetValues(formatCascades[i].w, XmNlabelString, label, NULL);
	  XmStringFree(label);
	  for(j = 0; j < MAXINSTANCES; ++j)
	    formatCascades[i].instances[j].w = NULL;
	}
	for(sp = ip->signal_list; sp; sp = sp->next_signal)
	{
	  j = fp->instance;
	  ss.s.instance = j;
	  if(formatCascades[i].instances[j].w == NULL)
	  {
	    formatCascades[i].instances[j].w = XmCreateCascadeButton(formatCascades[i].pw, instancenames[j], NULL, 0);
	    sprintf(menuName, "%s %s %s", outputnames[channel], formatnames[i], instancenames[j]);
	    formatCascades[i].instances[j].pw = XmCreatePulldownMenu(formatCascades[i].pw, menuName, NULL, 0);
	    label = XmStringCreateLocalized(instanceStrings[j]);
	    XtVaSetValues(formatCascades[i].instances[j].w, XmNlabelString, label, NULL);
	    XmStringFree(label);
	    for(k = 0; k < MAXSTANDARDS; ++k)
	      formatCascades[i].instances[j].standards[k] = NULL;
	  }
	  k = sp->signal->standard;
	  ss.s.standard = k;
	  if(formatCascades[i].instances[j].standards[k] == NULL)
	  {
	    formatCascades[i].instances[j].standards[k] =
	      (Widget) XmCreatePushButton(formatCascades[i].instances[j].pw, standardnames[k], NULL, 0 );
	    label = XmStringCreateLocalized(standardStrings[k]);
	    XtVaSetValues(formatCascades[i].instances[j].standards[k], XmNlabelString, label, NULL);
	    XmStringFree(label);
	    XtAddCallback (formatCascades[i].instances[j].standards[k],
			   XmNactivateCallback, callback, (XtPointer)ss.i);
	    XtManageChild(formatCascades[i].instances[j].standards[k]);
	  }
	}
      }
    }
    
    if(ic->outputs[channel])
    {
      for(i = 0; i < MAXFORMATS; ++i)
      {
	if(formatCascades[i].w)
	{
	  for(j = 0; j < MAXINSTANCES; ++j)
	  {
	    if(formatCascades[i].instances[j].w)
	    {
	      ac = 0;
	      XtSetArg(al[ac], XmNsubMenuId, formatCascades[i].instances[j].pw); ac++;
	      XtSetValues ( formatCascades[i].instances[j].w, al, ac );
	      XtManageChild(formatCascades[i].instances[j].w);
	    }
	  }
	  ac = 0;
	  XtSetArg(al[ac], XmNsubMenuId, formatCascades[i].pw); ac++;
	  XtSetValues ( formatCascades[i].w, al, ac );
	  XtManageChild(formatCascades[i].w);
	}
      }
    }
    if(ic->outputs[channel])
    {
      ic->outputs[channel]->channel = outputIndex;
      ic->outputs[channel]->cascade = outputTypeMenu;
    }
    if(outputChannelCascade[channel])
    {
      ac = 0;
      XtSetArg(al[ac], XmNsubMenuId, outputTypeMenu); ac++;
      XtSetValues(outputChannelCascade[channel], al, ac);
      XtManageChild(outputChannelCascade[channel]);
    }
  }
  if(plxOutputMenuBar)
  {
    XtManageChild(outputLabel);
    XtManageChild(plxOutputMenuBar);
/*
    XtManageChild(plxOutputMenu);
*/
    ac = 0;
    XtSetArg(al[ac], XmNsubMenuId, plxOutputMenu); ac++;
    XtSetValues(outputCascade, al, ac);
    XtManageChild(outputCascade);
  }
  
  if(outputTypeMenuBar)
  {
    XtManageChild(outputTypeLabel);
    XtManageChild(outputTypeMenuBar);
    XtManageChild(outputTypeCascade);
  }
  
  if(signalInstanceMenuBar)
  {
    XtManageChild(signalInstanceLabel);
    XtManageChild(signalInstanceMenuBar);
    XtManageChild(signalInstanceCascade);
  }
  
  if(signalStandardMenu)
  {
    XtManageChild(signalStandardLabel);
    XtManageChild(signalStandardMenu);
    XtManageChild(signalStandardCascade);
  }
  return(ic);
}


/*
 * PlxOutputCascadeGetChild() is a function that will get any
 * child of the output menu that was generated by this library.
 * Because XtNameToWidget() cannot traverse pulldown menus, it
 * is necessary to extract the XmNsubMenuId from each cascade
 * button and use that as the parent passed into XtNameToWidget().
 *
 * The #defines to specify each widget can be found in IOCascade.h
 */

Widget PlxOutputCascadeGetChild(Widget parent, int childId)
{
  /*
   * childId is a composite of:  Output | Format | Instance | Standard
   */

  Widget returnWidget = parent;
  Widget menuWidget;
  char widgetNameString[255] = "\0";
  char menuNameString[255] = "\0";
  char nameString[255] = "\0";
  int outputId;
  int formatId;
  int instanceId;
  int standardId;
  
  outputId = (childId >> 12) & 0x000F;
  formatId = (childId >> 8) & 0x000F;
  instanceId = (childId >> 4) & 0x000F;
  standardId = childId & 0x000F;
  menuWidget = parent;

  /*
   * Was an output id specified?  If so, get the widget id
   * of that output button.
   */
  if(0 != outputId)
  {
    outputId = 15 - outputId;
    sprintf(widgetNameString, "%s", outputnames[outputId]);
    menuWidget = parent;
    if(outputId >= MAXOUTPUTS)
    {
      /*
       * Requested the menu.
       */
      return(menuWidget);
    }
  }

  if(0 != formatId)
  {
    Widget widget;
    
    formatId = 15 - formatId;
    widget = XtNameToWidget(menuWidget, widgetNameString);
    if(widget != (Widget)NULL)
    {
      XtVaGetValues(widget,
		    XmNsubMenuId, &menuWidget,
		    NULL);

    }      
    /*
     * Before we look for format buttons, be sure we have the right menu.
     * The menu could be incorrect if menu compression was used.
     */
    sprintf(menuNameString, "%s", outputnames[outputId]);
    if(strcmp(menuNameString, XtName(menuWidget)) == 0)
    {
      /*
       * We have the correct menu.
       */
      sprintf(widgetNameString, "%s", formatnames[formatId]);
      if(formatId >= MAXFORMATS)
      {
	/*
	 * Requested the menu.
	 */
	return(menuWidget);
      }
    }
  }

  if(0 != instanceId)
  {
    Widget widget;
    
    instanceId = 15 - instanceId;

    /*
     * Before we look for instance buttons, be sure we have the format menu.
     * The menu could be incorrect if menu compression was used.
     */
    sprintf(menuNameString, "%s", outputnames[outputId]);
    if(strcmp(menuNameString, XtName(menuWidget)) == 0)
    {
      /*
       * We have the correct menu.  Now, we can get the instance menu.
       */
      sprintf(widgetNameString, "%s", formatnames[formatId]);
      widget = XtNameToWidget(menuWidget, widgetNameString);
      if(widget != (Widget)NULL)
      {
	XtVaGetValues(widget,
		      XmNsubMenuId, &menuWidget,
		      NULL);
      }      
      sprintf(widgetNameString, "%s", instancenames[instanceId]);
      if(instanceId >= MAXINSTANCES)
      {
	/*
	 * Requested the menu.
	 */
	return(menuWidget);
      }
    }
  }

  if(0 != standardId)
  {
    Widget widget;
    
    standardId = 15 - standardId;

    /*
     * Before we look for standard buttons, be sure we have the standard menu.
     * The menu could be incorrect if menu compression was used.
     */
    sprintf(menuNameString, "%s %s", outputnames[outputId], formatnames[formatId]);
    if(strcmp(menuNameString, XtName(menuWidget)) == 0)
    {
      /*
       * We have the Instance menu.  We must get the standards menu.
       */
      sprintf(widgetNameString, "%s", instancenames[instanceId]);
      widget = XtNameToWidget(menuWidget, widgetNameString);
      if(widget != (Widget)NULL)
      {
	XtVaGetValues(widget,
		      XmNsubMenuId, &menuWidget,
		      NULL);
      } 
    }     
    /*
     * Before we look for standard buttons, be sure we have the right menu.
     * The menu could be incorrect if menu compression was used.
     */
    sprintf(menuNameString, "%s %s %s", outputnames[outputId], formatnames[formatId], instancenames[instanceId]);
    if(strcmp(menuNameString, XtName(menuWidget)) == 0)
    {
      /*
       * We have the correct menu.
       */
      sprintf(widgetNameString, "%s", standardnames[standardId]);
      if(standardId >= MAXSTANDARDS)
      {
	/*
	 * Requested the menu.
	 */
	return(menuWidget);
      }      
    }
  }

  sprintf(nameString, "*%s", widgetNameString);

  if(menuWidget != (Widget)NULL)
  {
    returnWidget = XtNameToWidget(menuWidget, nameString);
    return(returnWidget);
  }
  else
  {
    return((Widget)NULL);
  }    
}


void PlxCompressOutputCascade(Widget parent)
{
  int		output;
  int		format;
  int		instance;
  int		formatCount;
  int		instanceCount;
  Widget	outputCascadeButton;
  Widget	formatCascadeButton;
  Widget	instanceCascadeButton;
  Widget	formatMenu;
  Widget	instanceMenu;
  Widget	standardMenu;

  for(output = 0; output < MAXOUTPUTS; output++)
  {
    outputCascadeButton = XtNameToWidget(parent, outputnames[output]);
    if(outputCascadeButton != (Widget)NULL)
    {
      /*
       * The output button exists.  Check it's children to see if
       * they are all needed.
       */
      XtVaGetValues(outputCascadeButton,
		    XmNsubMenuId, &formatMenu,
		    NULL);
      /*
       * Count the format buttons.
       */
      formatCount = 0;
      for(format = 0; format < MAXFORMATS; format++)
      {
	Widget buttonWidget;
	
	buttonWidget = XtNameToWidget(formatMenu, formatnames[format]);
	if((Widget)NULL != buttonWidget)
	{
	  /*
	   * Got a valid button - increment the format button count.
	   */
	  formatCount++;
	}
      }
      
      /*
       * For each format, count the instances.
       */
      for (format = 0; format < MAXFORMATS; format++)
      {
	formatCascadeButton = XtNameToWidget(formatMenu, formatnames[format]);
	if((Widget)NULL != formatCascadeButton)
	{
	  /*
	   * Count the instance buttons.
	   */
	  XtVaGetValues(formatCascadeButton,
			XmNsubMenuId, &instanceMenu,
			NULL);
	  instanceCount = 0;
	  for(instance = 0; instance < MAXINSTANCES; instance++)
	  {
	    if((Widget)NULL != XtNameToWidget(instanceMenu, instancenames[instance]))
	    {
	      /*
	       * Got a valid button - increment the instance button count.
	       */
	      instanceCount++;
	    }
	  }
	  

	  /*
	   * At this point, we know how many formats and instances there are.
	   * We can now do the actual compression.
	   */
	  if(instanceCount < 2)
	  {
	    /*
	     * There is only one instance button.  It can be removed, but first
	     * we have to know which one it is.
	     */
	    for(instance = 0; instance < MAXINSTANCES; instance++)
	    {
	      instanceCascadeButton = XtNameToWidget(instanceMenu, instancenames[instance]);
	      if(instanceCascadeButton != (Widget)NULL)
	      {
		/*
		 * Found the only instance button.  Compress by removing the instance menu.
		 */
		XtVaGetValues(instanceCascadeButton,
			      XmNsubMenuId, &standardMenu,
			      NULL);
		XtVaSetValues(formatCascadeButton,
			      XmNsubMenuId, standardMenu,
			      NULL);
	      }
	    }
	  }
	  
	  if(formatCount < 2)
	  {
	    /*
	     * There is only one format button.
	     */
	    if(outputCascadeButton != (Widget)NULL)
	    {
	      XtVaGetValues(formatCascadeButton,
			    XmNsubMenuId, &instanceMenu,
			    NULL);
	      XtVaSetValues(outputCascadeButton,
			    XmNsubMenuId, instanceMenu,
			    NULL);
	    }
	  }
	}
      }
    }
  }
}
