static char jpegRcsId[] = "$Id: jpeg.c,v 1.16 1995/12/22 23:54:29 drapeau Exp $";

/*
 *
 *	jpeg.c -- JPEG API library
 *
 *	Written by John Malleo-Roach
 *
 */

#define ASYNCWRITE

/* #define HYPERCMEDIA */

/* INCLUDE FILES */

#include	<stdio.h>
#include	<math.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<sys/types.h>
#include	<sys/times.h>
#include	<sys/param.h>
#include	<sys/time.h>
#include	<sys/asynch.h>
#include	<sys/ipc.h>
#include	<sys/shm.h>

#include	<unistd.h>

#include    <xview/xview.h>
#include    <xview/canvas.h>
#include    <X11/Xlib.h>
#include	<X11/extensions/XShm.h>

#ifdef AUDIO
#include    <multimedia/libaudio.h>
#include    <multimedia/audio_device.h>
#endif

#include	"Video.h"
#include	"movie.h"
#include	"alert.h"

/* CONSTANTS */

#define BUFFERSIZE	1024 * 128
#define MAXFRAMES	16384

#define aiowait(a)
#define aioread(a,b,c,d,e,f) (f)->aio_return = read(a,b,c)

/* STATIC VARIABLES */

static u_char *bufp   = NULL;
static u_char *bufend = NULL;

static unsigned char Abuffer[BUFFERSIZE];	/* Disk buffer A for reads/writes		*/
static unsigned char Bbuffer[BUFFERSIZE];	/* Disk buffer B for reads/writes		*/
static unsigned int  BufSync=0;				/* Which buffer is active				*/
static aio_result_t  AioResult;				/* Use for asynchronous reads/writes	*/
static unsigned int  BufOffset=0;			/* Offset into file being read/wrote	*/

static unsigned int Counter  = 0;			/* Used to sync with the interrupts		*/

static int		NewJPEGMovie = 0;			/* True when jpeg movie is being built	*/
static int		FrameCount   = 0;			/* Number of frames written out			*/
static int		MovieFd      = 0;			/* Movie file descriptor				*/
static BOOLEAN  StopMovie    = 0;			/* Flag to interrupt playback           */

static jpheader JPHeader;					/* JPEG header info						*/

static struct timeval TimePointer;			/* Starting time of day					*/

static unsigned char audiobuf[BUFFERSIZE];	/* Buffer for loading audio data		*/
static unsigned char  jpegbuf[BUFFERSIZE];	/* Buffer for loading jpeg data 	    */
static unsigned int  indexbuf[MAXFRAMES];	/* Buffer for storing frame offsets		*/

static JPEGImage *VideoImage;				/* The XPlxCImage structure				*/
static XShmSegmentInfo shminfo;				/* Info for any shared memory			*/
static BOOLEAN UsingSharedMemory=FALSE;

/* JFIF file header info */

/* For undoing a zigzaged load/store of qtable */

static int zigzag[64] = {
	0, 1, 8,16, 9, 2, 3,10,
	17,24,32,25,18,11, 4, 5,
	12,19,26,33,40,48,41,34,
	27,20,13, 6, 7,14,21,28,
	35,42,49,56,57,50,43,36,
	29,22,15,23,30,37,44,51,
	58,59,52,45,38,31,39,46,
	53,60,61,54,47,55,62,63
};

static u_char jfif_header[] = {
/* SOI    2 bytes */
	0xff, 0xd8,
/* APP0  18 bytes */
	            0xff, 0xe0, 0x00, 0x10, 0x4a, 0x46,
	0x49, 0x46, 0x00, 0x01,	0x01, 0x00, 0x00, 0x01,
	0x00, 0x01,	0x00, 0x00,
/* SOF   19 bytes */
	                        0xff, 0xc0, 0x00, 0x11,
	0x08, 0x00, 0x00, 0x00, 0x00, 0x03,	0x01, 0x21,
	0x00, 0x02, 0x11, 0x01,	0x03, 0x11, 0x01,
/* DQT  134 bytes */
	                                          0xff,
	0xdb, 0x00, 0x84, 0x00,	0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00,	0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00,	0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00,	0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00,	0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00,	0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00,	0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00,	0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00,	0x01, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00,
/* DHT 420 bytes */
	                              0xff, 0xc4, 0x01,
	0xa2, 0x00, 0x00, 0x01, 0x05, 0x01, 0x01, 0x01,
	0x01, 0x01,	0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00,	0x00, 0x01, 0x02, 0x03, 0x04, 0x05,
	0x06, 0x07,	0x08, 0x09, 0x0a, 0x0b,	0x10, 0x00,
	0x02, 0x01, 0x03, 0x03, 0x02, 0x04, 0x03, 0x05,
	0x05, 0x04, 0x04, 0x00, 0x00, 0x01, 0x7d, 0x01,
	0x02, 0x03, 0x00, 0x04, 0x11, 0x05, 0x12, 0x21,
	0x31, 0x41, 0x06, 0x13, 0x51, 0x61, 0x07, 0x22,
	0x71, 0x14, 0x32, 0x81, 0x91, 0xa1, 0x08,
	0x23, 0x42, 0xb1, 0xc1, 0x15, 0x52, 0xd1, 0xf0,
	0x24, 0x33, 0x62, 0x72, 0x82, 0x09, 0x0a, 0x16,
	0x17, 0x18, 0x19, 0x1a, 0x25, 0x26, 0x27, 0x28,
	0x29, 0x2a, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39,
	0x3a, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49,
	0x4a, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59,
	0x5a, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69,
	0x6a, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79,
	0x7a, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89,
	0x8a, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98,
	0x99, 0x9a, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7,
	0xa8, 0xa9, 0xaa, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6,
	0xb7, 0xb8, 0xb9, 0xba, 0xc2, 0xc3, 0xc4, 0xc5,
	0xc6, 0xc7, 0xc8, 0xc9, 0xca, 0xd2, 0xd3, 0xd4,
	0xd5, 0xd6, 0xd7, 0xd8, 0xd9, 0xda, 0xe1, 0xe2,
	0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea,
	0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8,
	0xf9, 0xfa,	0x01, 0x00, 0x03, 0x01, 0x01, 0x01,
	0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x03, 0x04,
	0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x11,
	0x00, 0x02, 0x01, 0x02, 0x04, 0x04, 0x03, 0x04,
	0x07, 0x05, 0x04, 0x04, 0x00, 0x01, 0x02, 0x77,
	0x00, 0x01, 0x02, 0x03, 0x11, 0x04, 0x05, 0x21,
	0x31, 0x06, 0x12, 0x41, 0x51, 0x07, 0x61, 0x71,
	0x13, 0x22, 0x32, 0x81, 0x08, 0x14, 0x42, 0x91,
	0xa1, 0xb1, 0xc1, 0x09, 0x23, 0x33, 0x52, 0xf0,
	0x15, 0x62, 0x72, 0xd1, 0x0a, 0x16, 0x24, 0x34,
	0xe1, 0x25, 0xf1, 0x17, 0x18, 0x19, 0x1a, 0x26,
	0x27, 0x28, 0x29, 0x2a, 0x35, 0x36, 0x37, 0x38,
	0x39, 0x3a, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48,
	0x49, 0x4a, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58,
	0x59, 0x5a, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68,
	0x69, 0x6a, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78,
	0x79, 0x7a, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87,
	0x88, 0x89, 0x8a, 0x92, 0x93, 0x94, 0x95, 0x96,
	0x97, 0x98, 0x99, 0x9a, 0xa2, 0xa3, 0xa4, 0xa5,
	0xa6, 0xa7, 0xa8, 0xa9, 0xaa, 0xb2, 0xb3, 0xb4,
	0xb5, 0xb6, 0xb7, 0xb8, 0xb9, 0xba, 0xc2, 0xc3,
	0xc4, 0xc5, 0xc6, 0xc7, 0xc8, 0xc9, 0xca, 0xd2,
	0xd3, 0xd4, 0xd5, 0xd6, 0xd7, 0xd8, 0xd9, 0xda,
	0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9,
	0xea, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8,
	0xf9, 0xfa,
/* SOS  14 bytes */
	            0xff, 0xda, 0x00, 0x0c,	0x03, 0x01,
	0x00, 0x02, 0x11, 0x03, 0x11, 0x00, 0x3f, 0x00
};

/* FUNCTIONS */

/* This is the no-op SIGALRM signal handler.  It just keeps track of the numer of	*/
/* times it has been called for sync purposes.										*/

static void handler()
{
	Counter = 1;
}

#ifdef SVR4
static ualarm(int value, int interval)
{
	struct itimerval timeout;

	timeout.it_value.tv_sec     = 0;
	timeout.it_value.tv_usec    = value;
	timeout.it_interval.tv_sec  = 0;
	timeout.it_interval.tv_usec = interval;
	setitimer(ITIMER_REAL,&timeout,NULL);
}
#endif

/* Create a shared memory version of JPEGImage structure */

static JPEGImage *CreateJPEGImage(display, width, height)
Display			*display;
int				 width;
int				 height;
{
	int			majorv;
	int			minorv;
	int			i;
	JPEGImage  *image;
	Bool		sharedPixmaps;

	if(!XPlxShmQueryVersion(display,&majorv,&minorv,&sharedPixmaps)) {
	  UsingSharedMemory = FALSE;
	  image = (JPEGImage *) XPlxCreateCImage(display,jpegbuf,0,width,height);
	  return(image);
	}
	shminfo.shmid = shmget(IPC_PRIVATE, 128 * 1024, IPC_CREAT|0777);

	if (shminfo.shmid < 0) {
		UsingSharedMemory = FALSE;
		image = (JPEGImage *) XPlxCreateCImage(display,jpegbuf,0,width,height);
		return(image);
	}
	shminfo.shmaddr = (char *)shmat(shminfo.shmid, 0, 0);
	if (shminfo.shmaddr == ((char *) -1)) {
		UsingSharedMemory = FALSE;
		image = (JPEGImage *) XPlxCreateCImage(display,jpegbuf,0,width,height);
		return(image);
	}
	image = (JPEGImage *) XPlxShmCreateCImage(display,shminfo.shmaddr,&shminfo,0,width,height);
	shminfo.readOnly  = False;
	XPlxShmAttach(display, &shminfo);
	UsingSharedMemory = TRUE;
	return(image);
}

/* Free up a XVideo jpeg image data structure */

int jpeg_free_image(dpy,i)
Display   *dpy;
JPEGImage *i;
{
	XPlxDestroyCImage(i);
	return(0);
}

/* Destroy the X11 shared memory JPEGImage structure */

static int DestroyJPEGImage(display,image)
Display *display;
JPEGImage *image;
{
	int rc;

	if (UsingSharedMemory) {
		UsingSharedMemory = FALSE;
		XPlxShmDetach(display, image->info);
		XSync(display,False);
		rc = shmdt(image->info->shmaddr);
#ifdef HYPERCMEDIA
		if (rc < 0) hCm_SystemError(errno,"shmdt failed");
		rc = shmctl(image->info->shmid, IPC_RMID, 0);
		if (rc < 0) hCm_SystemError(errno,"shmctl failed");
#else
		if (rc < 0) SystemError(errno,"shmdt failed");
		rc = shmctl(image->info->shmid, IPC_RMID, 0);
		if (rc < 0) SystemError(errno,"shmctl failed");
#endif
	}
	else jpeg_free_image(display,image);
}

static int LoadAudio(fd,audio,play)
int fd;
int audio;
int play;
{
	int i,j;
	u_char *f;

	i = JPHeader.audioslice;

	if ((bufp + i) < bufend) { memcpy(audiobuf,bufp,i); bufp += i; }
	else {
		f = audiobuf; j = bufend - bufp; memcpy(f,bufp,j); f += j;
#ifdef SVR4
		sighold(SIGALRM); aiowait(NULL); sigrelse(SIGALRM);
#else
		aiowait(NULL);
#endif
		if (BufSync) {
			bufp = Bbuffer; bufend = bufp + AioResult.aio_return;
			BufOffset += AioResult.aio_return; BufSync ^= 1;
			AioResult.aio_return = AIO_INPROGRESS;
			aioread(fd,Abuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
		}
		else {
			bufp = Abuffer; bufend = bufp + AioResult.aio_return;
			BufOffset += AioResult.aio_return; BufSync ^= 1;
			AioResult.aio_return = AIO_INPROGRESS;
			aioread(fd,Bbuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
		}
		i -= j; if (i) memcpy(f,bufp,i); bufp += i;
	}

/* Load up the audio associated with this frame */

	if (play && audio) write(audio,audiobuf,JPHeader.audioslice);
}

/* Decode a frame into a equivalend area of memory */

static int LoadFrame(dpy,fd,video,frame)
Display *dpy;
int     fd;
VideoIn video;
int     frame;
{
	int i,j;
	int size;
	u_char *f;

	i = sizeof(int);

	if ((bufp + i) < bufend) { memcpy(&size,bufp,i); bufp += i; }
	else {
		f = (u_char *) &size; j = bufend - bufp; memcpy(f,bufp,j); f += j;
#ifdef SVR4
		sighold(SIGALRM); aiowait(NULL); sigrelse(SIGALRM);
#else
		aiowait(NULL);
#endif
		if (BufSync) {
			bufp = Bbuffer; bufend = bufp + AioResult.aio_return;
			BufOffset += AioResult.aio_return; BufSync ^= 1;
			AioResult.aio_return = AIO_INPROGRESS;
			aioread(fd,Abuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
		}
		else {
			bufp = Abuffer; bufend = bufp + AioResult.aio_return;
			BufOffset += AioResult.aio_return; BufSync ^= 1;
			AioResult.aio_return = AIO_INPROGRESS;
			aioread(fd,Bbuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
		}
		i -= j; if (i) memcpy(f,bufp,i); bufp += i;
	}
	i = size;
	if ((bufp + i) < bufend) { memcpy(VideoImage->data,bufp,i); bufp += i; }
	else {
		f = (u_char *) VideoImage->data; j = bufend - bufp; memcpy(f,bufp,j); f += j;
#ifdef SVR4
		sighold(SIGALRM); aiowait(NULL); sigrelse(SIGALRM);
#else
		aiowait(NULL);
#endif
		if (BufSync) {
			bufp = Bbuffer; bufend = bufp + AioResult.aio_return;
			BufOffset += AioResult.aio_return; BufSync ^= 1;
			AioResult.aio_return = AIO_INPROGRESS;
			aioread(fd,Abuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
		}
		else {
			bufp = Abuffer; bufend = bufp + AioResult.aio_return;
			BufOffset += AioResult.aio_return; BufSync ^= 1;
			AioResult.aio_return = AIO_INPROGRESS;
			aioread(fd,Bbuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
		}
		i -= j; if (i) memcpy(f,bufp,i); bufp += i;
	}

/* Set the size field of the JPEGImage structure */

	VideoImage->size = size;

/* Pull in the decoded frame now

	if (frame && UsingSharedMemory) while (xv_get(video,VIDEO_SHM_DONE) == FALSE);
*/
	xv_set(video,VIDEO_JPEG_IMAGE,VideoImage,NULL);

/* Flush the X server */

	XSync(dpy,False);
}

static int DecodeFrame(dpy,fd,audio,video,frame,track)
Display *dpy;
int     fd;
int		audio;
VideoIn video;
int     frame;
int     track;
{
	unsigned char c;

/* FIXUP: This assumes that the first read returned BUFFERSIZE */

	if (frame == 0) {
		AioResult.aio_return = AIO_INPROGRESS;
		AioResult.aio_errno  = 0;
		AioResult.aio_return = AIO_INPROGRESS;
		aioread(fd,Bbuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
		BufSync  ^= 1;
	}

	while (1) {

/* Get next byte stream command until we get a frame loaded */

		c = *bufp++;
		if (bufp == bufend) {
#ifdef SVR4
			sighold(SIGALRM); aiowait(NULL); sigrelse(SIGALRM);
#else
			aiowait(NULL);
#endif
			if (BufSync) {
				bufp = Bbuffer; bufend = bufp + AioResult.aio_return;
				BufOffset += AioResult.aio_return; BufSync ^= 1;
				AioResult.aio_return = AIO_INPROGRESS;
				aioread(fd,Abuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			}
			else {
				bufp = Abuffer; bufend = bufp + AioResult.aio_return;
				BufOffset += AioResult.aio_return; BufSync ^= 1;
				AioResult.aio_return = AIO_INPROGRESS;
				aioread(fd,Bbuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			}
		}
		switch (c) {
		case LOAD_JPEG:
			LoadFrame(dpy,fd,video,frame);
			break;
		case END_FRAME:
			return;
			break;
		case LOAD_AUDIO_0:
			LoadAudio(fd,audio,(track == 0));
			break;
		case LOAD_AUDIO_1:
			LoadAudio(fd,audio,(track == 1));
			break;
		case LOAD_AUDIO_2:
			LoadAudio(fd,audio,(track == 2));
			break;
		case LOAD_AUDIO_3:
			LoadAudio(fd,audio,(track == 3));
			break;
		default:
			printf("Undefined j_movie byte stream command: %d\n",c);
			printf("Current frame number is %d\n",frame);
			return(-1);
		}
	}
}

jpeg_decode_file(dpy,fd,audio,video,track)
Display *dpy;
int      fd;
int		 audio;
VideoIn  video;
int      track;
{
	int j,frames,flag;

	frames    = 0;
	BufOffset = 0;
	BufSync   = 0;
	Counter   = 0;
	flag      = 0;
	StopMovie = FALSE;

/* Initialize the XPlxCImage structure */

	flag = (int) xv_get(video,VIDEO_USE_SHM);

	VideoImage = (JPEGImage *) CreateJPEGImage(dpy,JPHeader.width,JPHeader.height);

	if (UsingSharedMemory && !flag) xv_set(video,VIDEO_USE_SHM,TRUE,NULL);

/* Set the interrupt scheduler to hit us at the requested frames per second */

#ifdef SVR4
	sigset(SIGALRM,(void (*)(int)) handler);
#else
	signal(SIGALRM,handler);
#endif
	j = 1000000 / JPHeader.fps;
	ualarm(j,j);
		
/* Get the clock start time */
	
	gettimeofday(&TimePointer,NULL);

	while (frames < JPHeader.frames) {

		if (StopMovie) break;

/* Show the next available frame */

		DecodeFrame(dpy,fd,audio,video,frames,track);

/* Handle any application events */

		if (QLength(dpy)) {
			xv_input_pending(dpy,XConnectionNumber(dpy));
		}
#ifdef SVR4
		sighold(SIGALRM);
		if (Counter == 0) {
			sigpause(SIGALRM);
		}
		else sigrelse(SIGALRM);
#else
		if (Counter == 0) pause();
#endif
		Counter = 0; frames++;
	}
	ualarm(0,0);

	if (UsingSharedMemory && !flag) xv_set(video,VIDEO_USE_SHM,FALSE,NULL);

/* Release the XPlxCImage structure */

	DestroyJPEGImage(dpy,VideoImage);
}

/* Save one frame of video as a still jpeg movie file */

int jpeg_save_still(dpy,image,file)
Display   *dpy;
JPEGImage *image;
char      *file;
{
	u_char    c;
	int       i;
	int      fd;
	jpheader  h;

	if ((fd = open(file,O_CREAT|O_WRONLY|O_TRUNC,0644)) == -1) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Couldn't open jpeg file for output");
#else
		SystemError(errno,"Couldn't open jpeg file for output");
#endif
		return(-1);
	}

	strcpy(h.magic,"j_movie");
    
	h.version    = JPEG_VERSION_2;
	h.fps        = 0;
	h.frames     = 1;
	h.width      = image->width;
	h.height     = image->height;
	h.bandwidth  = image->size;
	h.qfactor	 = 0;
	h.mapsize	 = 256 * 256 * 256;
	h.tracks     = 0;
	h.volbase    = 0;
	h.audioslice = 0;
	
	for (i=0; i<FILLER; i++) h.filler[i] = 0;
	
/* Put out the header info */

	if (write(fd,(char *)&h,sizeof(jpheader)) < 0) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"JPEG file header write failed!");
#else
		SystemError(errno,"JPEG file header write failed!");
#endif
		close(fd);
		unlink(file);
		return(-1);
	}

/* Write the load jpeg frame instruction */

	c = LOAD_JPEG;
	write(fd,&c,1);

/* Save the compressed image data */

	if (write(fd,image->data,image->size) < 0) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"JPEG file image data write failed!");
#else
		SystemError(errno,"JPEG file image data write failed!");
#endif
		close(fd);
		unlink(file);
		return(-1);
	}
	close(fd);
	return(0);
}

/* Save one frame of video as a jfif file */

int jpeg_save_jfif(dpy,image,file,table)
Display   *dpy;
JPEGImage *image;
char      *file;
u_char    *table;
{
	u_char    c;
	int      fd,i;

	if ((fd = open(file,O_CREAT|O_WRONLY|O_TRUNC,0644)) == -1) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Couldn't open jfif file for output");
#else
		SystemError(errno,"Couldn't open jfif file for output");
#endif
		return(-1);
	}

/* Update with width and height of compressed image */

	jfif_header[25] = (image->height  >> 8) & 0xff;
	jfif_header[26] = (image->height)  & 0xff;
	jfif_header[27] = (image->width >> 8) & 0xff;
	jfif_header[28] = (image->width) & 0xff;

/* Load in the XVideo quanization tables specified by the user */

	for (i=0; i<64; i++) {
		jfif_header[i+ 44] = (table[zigzag[i]+ 5]);
		jfif_header[i+109] = (table[zigzag[i]+70]);
	}

/* Put out the jfif header info */

	if (write(fd,jfif_header,sizeof(jfif_header)) < 0) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"JFIF header write failed!");
#else
		SystemError(errno,"JFIF header write failed!");
#endif
		close(fd);
		unlink(file);
		return(-1);
	}

/* Save the compressed image data */

	if (write(fd,image->data,image->size) < 0) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"JFIF scan image data write failed!");
#else
		SystemError(errno,"JFIF scan data write failed!");
#endif
		close(fd);
		unlink(file);
		return(-1);
	}

/* end of JFIF file */

	close(fd);
	return(0);
}

/* Load one frame of video from a still jpeg movie file */

JPEGImage *jpeg_load_still(dpy,file)
Display *dpy;
char    *file;
{
	u_char    *data;
	int          fd;
	JPEGImage    *i;
	jpheader      h;

	if ((fd = open(file,O_RDONLY)) == -1) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Couldn't open jpeg file for input");
#else
		SystemError(errno,"Couldn't open jpeg file for input");
#endif
		return((JPEGImage *) NULL);
	}

/* Load the header info */

	if (read(fd,&h,sizeof(jpheader)) < 0) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"JPEG file header read failed!");
#else
		SystemError(errno,"JPEG file header read failed!");
#endif
		close(fd);
		return((JPEGImage *) NULL);
	}

/* Verify that the file is kosher */

	if (strcmp(h.magic,"j_movie")) {
#ifdef HYPERCMEDIA
		hCm_ShowAlert("%s is not a valid JPEG compressed still image",file,NULL);
#else
		ShowAlert("File is not a valid JPEG compressed still image","",file,NULL);
#endif
		close(fd);
		return((JPEGImage *) NULL);
	}
	if (h.frames != 1) {
#ifdef HYPERCMEDIA
		hCm_ShowAlert("%s is not a valid JPEG compressed still image",file,NULL);
#else
		ShowAlert("File is not a valid JPEG compressed still image","",file,NULL);
#endif
		close(fd);
		return((JPEGImage *) NULL);
	}

/* Check the load single frame sequence */

	data = (u_char *) malloc(h.bandwidth);
	read(fd,data,1);
	if (*data != LOAD_JPEG) {
#ifdef HYPERCMEDIA
		hCm_ShowAlert("JPEG movie contains invalid movie sequence",NULL,NULL);
#else
		ShowAlert("JPEG movie contains invalid movie sequence",NULL);
#endif
		free(data);
		close(fd);
		return((JPEGImage *) NULL);
	}

/* Get the compressed frame out */

	if (read(fd,data,h.bandwidth) < 0) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"JPEG file image data read failed!");
#else
		SystemError(errno,"JPEG file image data read failed!");
#endif
		free(data);
		close(fd);
		return((JPEGImage *) NULL);
	}
	close(fd);

	i = (JPEGImage *) XPlxCreateCImage(dpy,data,h.bandwidth,h.width,h.height);

	if (i == NULL) {
#ifdef HYPERCMEDIA
		hCm_ShowAlert("XPlxCreateCImage failed: Unknown errror",NULL,NULL);
#else
		ShowAlert("XPlxCreateCImage failed: Unknown errror",NULL);
#endif
		free(data);
		return((JPEGImage *) NULL);
	}
	return(i);
}

/* Parse the jfif data into a form the c3lib can handle */

static int ParseJFIF(buffer,len,width,height)
u_char *buffer;
int len;
int *width;
int *height;
{
	int done,count;
	u_char c,*p,*buf;
	int haveq,haveh;
	int i,j,k,l,m,n,rsti;
	u_char qtab[128];
	u_char htab[1024];

	buf = buffer; done = count = haveh = haveq = 0;
	while (!done) {
		c = *buf++; count++;
		if (c != 0xff) return(-1);
		c = *buf++; count++;
		switch (c) {
		case 0xd8:
/* SOI */
			break;
		case 0xfe:
			p = buf;
/* COM */
			i  = ((int) *p++) << 8; 
			i += ((int) *p++); 
			count += i;
			buf += i; break;
		case 0xdd:
			p = buf;
/* RST */
			i  = ((int) *p++) << 8; 
			i += ((int) *p++); 
			rsti  = ((int) *p++) << 8; 
			rsti += ((int) *p++); 
			count += i;
			buf += i; break;
		case 0xe0:
			p = buf;
/* APP0 */
			i  = ((int) *p++) << 8; 
			i += ((int) *p++); 
			count += i;
/* Verify that the file is JFIF */
			if (*p++ != 'J') return(-2);
			if (*p++ != 'F') return(-2);
			if (*p++ != 'I') return(-2);
			if (*p++ != 'F') return(-2);
			buf += i; break;
		case 0xdb:
			p = buf;
/* DQT */
			i  = ((int) *p++) << 8; 
			i += ((int) *p++); 
			count += i;
			j  = i - 2;
			while (j) {
				haveq++;
				l = (int) *p++;
				if (l > 1) return(-3);
				k = (l << 6) + l;
				qtab[k++] = l;
/* Unzigzag the table */
				for (l=0; l<64; l++) {
					qtab[l+k] = (*p++);
				}
				j -= 65;
			}
			buf += i; break;
		case 0xc0:
			p = buf;
/* SOF */
			i  = ((int) *p++) << 8; 
			i += ((int) *p++); 
			count += i;
			if (*p++ != 8) return(-4);
			*height  = ((int) *p++) << 8;
			*height += ((int) *p++);
			*width   = ((int) *p++) << 8;
			*width  += ((int) *p++);
			if ((j = *p++) != 3) return(-5);
			for (k=0; k<j; k++) {
				l = *p++; n = *p++; p++;
				m = (n >> 4) & 0xf;
				n = n & 0xf; 
				switch (l) {
				case 1:
					if ((m != 2) || (n != 1)) return(-5);
					break;
				case 2:
				case 3:
					if ((m != 1) || (n != 1)) return(-5);
					break;
				default:
					return(0);
				}
			}
			buf += i; break;
		case 0xc4:
			p = buf;
/* DHT */
			i  = ((int) *p++) << 8; 
			i += ((int) *p++); 
			count += i;
			j = i - 2;
			while (j) {
				htab[haveh++] = *p++;
				j--; l = 0;
				for (k=0; k<16; k++) {
					htab[haveh] = *p++;
					l += htab[haveh++];
				}
				j -= 16;
				for (k=0; k<l; k++) htab[haveh++] = *p++;
				j -= l;
			}
			buf += i; break;
		case 0xda:
			p = buf;
/* SOS */
			i  = ((int) *p++) << 8; 
			i += ((int) *p++); 
			count += i;
			if ((j = *p++) != 3) return(-5);
			for (k=0; k<j; k++) {
				l = *p++; n = *p++;
				m = (n >> 4) & 0xf;
				n = n & 0xf; 
				switch (l) {
				case 1:
					if ((m != 0) || (n != 0)) return(-5);
					break;
				case 2:
				case 3:
					if ((m != 1) || (n != 1)) return(-5);
					break;
				default:
					return(0);
				}
			}
			done = 1; buf += i; break;
		default:
			return(-1);
		}
	}
/* Make sure the file defines qtables and htables */
	if (!haveh || !haveq) return(-6);
/* Reassemble the data in the right form for the C3lib */
	i = 0;
	buffer[i++] = 0xff;
	buffer[i++] = 0xdb;
	j = haveq * 65 + 2;
	buffer[i++] = (j >> 8) & 0xff;
	buffer[i++] = j & 0xff;
	j -= 2;
	for (k=0; k<j; k++) buffer[i++] = qtab[k];
/* restart interval added */
	buffer[i++] = 0xff;
	buffer[i++] = 0xdd;
	j = 4;
	buffer[i++] = (j >> 8) & 0xff;
	buffer[i++] = j & 0xff;
	buffer[i++] = (rsti >> 8) & 0xff;
	buffer[i++] = rsti & 0xff;
/* end of restart interval */
	buffer[i++] = 0xff;
	buffer[i++] = 0xc4;
	j = haveh + 2;
	buffer[i++] = (j >> 8) & 0xff;
	buffer[i++] = j & 0xff;
	j -= 2;
	for (k=0; k<j; k++) buffer[i++] = htab[k];
	for (j=count; j<len; j++) buffer[i++] = buffer[j];
	return(i);
}

/* Load one frame of video from a JFIF file */

JPEGImage *jpeg_load_jfif(dpy,file)
Display *dpy;
char    *file;
{
	JPEGImage *i;
	u_char    *data;
	int        fd,size;
	int        width,height;

	if ((fd = open(file,O_RDONLY)) == -1) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Couldn't open JFIF file for input");
#else
		SystemError(errno,"Couldn't open JFIF file for input");
#endif
		return((JPEGImage *) NULL);
	}

/* Figure out the size of the JFIF data */

	size  = lseek(fd,0,SEEK_END);
	lseek(fd,0,SEEK_SET);
	data = (u_char *) malloc(size);

/* Get the compressed frame out */

	if (read(fd,data,size) < 0) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"JFIF file data read failed!");
#else
		SystemError(errno,"JFIF file data read failed!");
#endif
		free(data);
		close(fd);
		return((JPEGImage *) NULL);
	}
	close(fd);

	width = height = 0;
	size = ParseJFIF(data,size,&width,&height);

	if (size < 0) {
		switch (size) {
		case -1:
#ifdef HYPERCMEDIA
			hCm_ShowAlert("Invalid marker sequence in file!",NULL,NULL);
#else
			ShowAlert("Invalid marker sequence in file!",NULL);
#endif
			break;
		case -2:
#ifdef HYPERCMEDIA
			hCm_ShowAlert("Unrecognized APP0 marker in file!",NULL,NULL);
#else
			ShowAlert("Unrecognized APP0 marker in file!",NULL);
#endif
			break;
		case -3:
#ifdef HYPERCMEDIA
			hCm_ShowAlert("Too many qtables defined in file!",NULL,NULL);
#else
			ShowAlert("Too many qtables defined in file!",NULL);
#endif
			break;
		case -4:
#ifdef HYPERCMEDIA
			hCm_ShowAlert("Only 8 bit precision supported!",NULL,NULL);
#else
			ShowAlert("Only 8 bit precision supported!",NULL);
#endif
			break;
		case -5:
#ifdef HYPERCMEDIA
			hCm_ShowAlert("Only YCbCr 422 format supported!",NULL,NULL);
#else
			ShowAlert("Only YCbCr 422 format supported!",NULL);
#endif
			break;
		case -6:
#ifdef HYPERCMEDIA
			hCm_ShowAlert("JFIF file is missing DHT/DQT sequence!",NULL,NULL);
#else
			ShowAlert("JFIF file is missing DHT/DQT sequence!",NULL);
#endif
			break;
		}
		free(data); return((JPEGImage *) NULL);
	}
	if (!width || !height) {
#ifdef HYPERCMEDIA
		hCm_ShowAlert("%s is not a valid JFIF file",file,NULL);
#else
		ShowAlert("File is not a valid JFIF file","",file,NULL);
#endif
		free(data); return((JPEGImage *) NULL);
	}
	i = (JPEGImage *) XPlxCreateCImage(dpy,data,size,width,height);

	if (i == NULL) {
#ifdef HYPERCMEDIA
		hCm_ShowAlert("XPlxCreateCImage failed: Unknown errror",NULL,NULL);
#else
		ShowAlert("XPlxCreateCImage failed: Unknown errror",NULL);
#endif
		free(data);
		return((JPEGImage *) NULL);
	}
	return(i);
}

#undef aiowait(a)

int jpeg_open_movie(dpy,filename,fps,qfactor,audio)
Display   *dpy;
char      *filename;
int        fps;
int        qfactor;
Audio_hdr *audio;
{
	u_char    c;
	int       i;

	if (NewJPEGMovie) {
#ifdef HYPERCMEDIA
		hCm_ShowAlert("Please finish current movie before starting new one!",NULL,NULL);
#else
		ShowAlert("Please finish current jpeg movie before starting new one!",NULL);
#endif
		return(-1);
	}
	else {
		NewJPEGMovie = TRUE;
		FrameCount   = 0;
	}
	if ((MovieFd = open(filename,O_CREAT|O_WRONLY|O_TRUNC,0644)) == -1) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Couldn't open jpeg movie file for output");
#else
		SystemError(errno,"Couldn't open jpeg movie file for output");
#endif
		return(-1);
	}

	strcpy(JPHeader.magic,"j_movie");
    
	JPHeader.version    = JPEG_VERSION_2;
	JPHeader.fps        = fps;
	JPHeader.frames     = 0;
	JPHeader.width      = 0;
	JPHeader.height     = 0;
	JPHeader.bandwidth  = 0;
	JPHeader.qfactor	= qfactor;
	JPHeader.mapsize    = 256 * 256 * 256;
	JPHeader.tracks     = 1;
	JPHeader.volbase    = 0;
	if (audio) {
/* Compute the number of bytes per frame at desired sampling and fps */
		i = (audio->channels * audio->bytes_per_unit * audio->sample_rate)
			/ audio->samples_per_unit;
		JPHeader.audioslice = (i / fps);
		if (i % fps) JPHeader.audioslice++;
		memcpy(&(JPHeader.audio),audio,sizeof(Audio_hdr));
	}
	else {
		JPHeader.audioslice = 0;
		memset(&(JPHeader.audio),0,sizeof(Audio_hdr));
	}
	for (i=0; i<FILLER; i++) JPHeader.filler[i] = 0;
	
/* Put out the header info */

	if (write(MovieFd,(char *)&JPHeader,sizeof(jpheader)) < 0) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"JPEG file header write failed!");
#else
		SystemError(errno,"JPEG file header write failed!");
#endif
		close(MovieFd);
		unlink(filename);
		return(-1);
	}

/* Record offset to first frame */

	indexbuf[FrameCount] = sizeof(jpheader);

/* Set up data structures for asyncio writes */
#ifdef ASYNCWRITE
	bufp                 = Bbuffer;
	bufend               = Bbuffer + BUFFERSIZE;
	BufOffset            = 0;
	BufSync              = 0;
	AioResult.aio_return = AIO_INPROGRESS;
#endif
	return(0);
}

int jpeg_add_still(dpy,image)
Display *dpy;
JPEGImage *image;
{
	int     i,j;
	u_char *p;
	u_char  c;

	if (NewJPEGMovie == FALSE) {
#ifdef HYPERCMEDIA
		hCm_ShowAlert("Please open a JPEG movie before writing stills out!",NULL,NULL);
#else
		ShowAlert("Please open a JPEG movie before writing stills out!",NULL);
#endif
		return(-1);
	}
	if (FrameCount == 0) {
		JPHeader.width  = image->width;
		JPHeader.height = image->height;
	}
	else {
		if ((JPHeader.width != image->width) || (JPHeader.height != image->height)) {
#ifdef HYPERCMEDIA
			hCm_ShowAlert("Frame width and/or height differs from previous!",NULL,NULL);
#else
			ShowAlert("Frame width and/or height differs from previous!",NULL);
#endif
			return(-1);
		}
	}
	JPHeader.bandwidth += image->size;

/* Write the load jpeg still instruction */

#ifdef ASYNCWRITE
	if (bufp != bufend) *bufp++ = LOAD_JPEG;
	else {
#ifdef SVR4
		if (FrameCount) { sighold(SIGALRM); aiowait(NULL); sigrelse(SIGALRM); }
#endif
		if (BufSync) {
			aiowrite(MovieFd,Abuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			AioResult.aio_return   = AIO_INPROGRESS;
			bufp = Bbuffer; bufend = Bbuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		else {
			aiowrite(MovieFd,Bbuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			AioResult.aio_return   = AIO_INPROGRESS;
			bufp = Abuffer; bufend = Abuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		*bufp++ = LOAD_JPEG;
	}
#else
	c = LOAD_JPEG;
	if (write(MovieFd,&c,1) != 1) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Write to JPEG movie file failed");
#else
		SystemError(errno,"Write to JPEG movie file failed");
#endif
		close(MovieFd);
		return(-1);
	}
#endif

/* Store the size of the compressed data */

#ifdef ASYNCWRITE
	if ((bufp + sizeof(int)) < bufend) {
		memcpy(bufp,&(image->size),sizeof(int));
		bufp += sizeof(int);
	}
	else {
		i = sizeof(int); p = (u_char *) &(image->size);
		j = bufend - bufp; memcpy(bufp,p,j); p += j;
#ifdef SVR4
		if (FrameCount) { sighold(SIGALRM); aiowait(NULL); sigrelse(SIGALRM); }
#endif
		if (BufSync) {
			aiowrite(MovieFd,Abuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			AioResult.aio_return   = AIO_INPROGRESS;
			bufp = Bbuffer; bufend = Bbuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		else {
			aiowrite(MovieFd,Bbuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			AioResult.aio_return   = AIO_INPROGRESS;
			bufp = Abuffer; bufend = Abuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		i -= j; if (i) memcpy(bufp,p,i); bufp += i;
	}
#else
	if (write(MovieFd,&(image->size),sizeof(int)) != sizeof(int)) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Write to JPEG movie file failed");
#else
		SystemError(errno,"Write to JPEG movie file failed");
#endif
		close(MovieFd);
		return(-1);
	}
#endif

/* Store the frame data */

#ifdef ASYNCWRITE
	if ((bufp + image->size) < bufend) {
		memcpy(bufp,image->data,image->size);
		bufp += image->size;
	}
	else {
		i = image->size; p = (u_char *) image->data;
		j = bufend - bufp; memcpy(bufp,p,j); p += j;
#ifdef SVR4
		if (FrameCount) { sighold(SIGALRM); aiowait(NULL); sigrelse(SIGALRM); }
#endif
		if (BufSync) {
			aiowrite(MovieFd,Abuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			AioResult.aio_return   = AIO_INPROGRESS;
			bufp = Bbuffer; bufend = Bbuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		else {
			aiowrite(MovieFd,Bbuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			AioResult.aio_return   = AIO_INPROGRESS;
			bufp = Abuffer; bufend = Abuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		i -= j; if (i) memcpy(bufp,p,i); bufp += i;
	}
#else
	if (write(MovieFd,image->data,image->size) != image->size) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Write to JPEG movie file failed");
#else
		SystemError(errno,"Write to JPEG movie file failed");
#endif
		close(MovieFd);
		return(-1);
	}
#endif
	return(0);
}

int jpeg_get_slice(dpy)
Display *dpy;
{
	return(JPHeader.audioslice);
}

int jpeg_add_audio(dpy,audio,track)
Display *dpy;
u_char  *audio;
int      track;
{
	int     i,j;
	u_char  c;

	if (NewJPEGMovie == FALSE) {
#ifdef HYPERCMEDIA
		hCm_ShowAlert("Please open a JPEG movie before writing audio out!",NULL,NULL);
#else
		ShowAlert("Please open a JPEG movie before writing audio out!",NULL);
#endif
		return(-1);
	}
	if ((track < 0) || (track > 3)) {
#ifdef HYPERCMEDIA
		hCm_ShowAlert("Invalid track number requested!",NULL,NULL);
#else
		ShowAlert("Invalid track number requested!",NULL);
#endif
		return(-1);
	}

/* Write the load audio track instruction */

	c = LOAD_AUDIO_0 - track;
#ifdef ASYNCWRITE
	if (bufp != bufend) *bufp++ = c;
	else {
#ifdef SVR4
		if (FrameCount) { sighold(SIGALRM); aiowait(NULL); sigrelse(SIGALRM); }
#endif
		if (BufSync) {
			aiowrite(MovieFd,Abuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			AioResult.aio_return   = AIO_INPROGRESS;
			bufp = Bbuffer; bufend = Bbuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		else {
			aiowrite(MovieFd,Bbuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			AioResult.aio_return   = AIO_INPROGRESS;
			bufp = Abuffer; bufend = Abuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		*bufp++ = c;
	}
#else
	if (write(MovieFd,&c,1) != 1) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Write to LOAD_AUDIO to JPEG movie file failed");
#else
		SystemError(errno,"Write to LOAD_AUDIO to JPEG movie file failed");
#endif
		close(MovieFd);
		return(-1);
	}
#endif

/* Store the audio data */

#ifdef ASYNCWRITE
	if ((bufp + JPHeader.audioslice) < bufend) {
		memcpy(bufp,audio,JPHeader.audioslice);
		bufp += JPHeader.audioslice;
	}
	else {
		i = JPHeader.audioslice; j = bufend - bufp;
		memcpy(bufp,audio,j); audio += j;
#ifdef SVR4
		if (FrameCount) { sighold(SIGALRM); aiowait(NULL); sigrelse(SIGALRM); }
#endif
		if (BufSync) {
			aiowrite(MovieFd,Abuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			AioResult.aio_return   = AIO_INPROGRESS;
			bufp = Bbuffer; bufend = Bbuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		else {
			aiowrite(MovieFd,Bbuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			AioResult.aio_return   = AIO_INPROGRESS;
			bufp = Abuffer; bufend = Abuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		i -= j; if (i) memcpy(bufp,audio,i); bufp += i;
	}
#else
	if (write(MovieFd,audio,JPHeader.audioslice) != JPHeader.audioslice) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Write of audio to JPEG movie file failed");
#else
		SystemError(errno,"Write of audio to JPEG movie file failed");
#endif
		close(MovieFd);
		return(-1);
	}
#endif
	return(0);
}

int jpeg_end_frame(dpy)
{
	u_char c;
	int length;

	if (NewJPEGMovie == FALSE) {
#ifdef HYPERCMEDIA
		hCm_ShowAlert("No open JPEG movie file exists!",NULL,NULL);
#else
		ShowAlert("No open JPEG movie file exists!",NULL);
#endif
		return(-1);
	}

/* Write the end of movie frame instruction */

#ifdef ASYNCWRITE
	if (bufp != bufend) *bufp++ = END_FRAME;
	else {
#ifdef SVR4
		if (FrameCount) { sighold(SIGALRM); aiowait(NULL); sigrelse(SIGALRM); }
#endif
		if (BufSync) {
			aiowrite(MovieFd,Abuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			AioResult.aio_return   = AIO_INPROGRESS;
			bufp = Bbuffer; bufend = Bbuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		else {
			aiowrite(MovieFd,Bbuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			AioResult.aio_return   = AIO_INPROGRESS;
			bufp = Abuffer; bufend = Abuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		*bufp++ = END_FRAME;
	}
#else
	c = END_FRAME;
	if (write(MovieFd,&c,1) != 1) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Write of END_FRAME to JPEG movie file failed");
#else
		SystemError(errno,"Write of END_FRAME to JPEG movie file failed");
#endif
		close(MovieFd);
		return(-1);
	}
#endif
	FrameCount++;

/* Record offset to next frame */
#ifdef ASYNCWRITE
	indexbuf[FrameCount] = sizeof(jpheader) + BufOffset + BUFFERSIZE - (bufend - bufp);
#else
	indexbuf[FrameCount] = lseek(MovieFd,0L,SEEK_CUR);
#endif
/* Terminate if reached max frame count */

	if (FrameCount == MAXFRAMES) {
		jpeg_stop_movie(NULL);
#ifdef HYPERCMEDIA
		hCm_ShowAlert("Reached maximum frame count for movie!",NULL,NULL);
#else
		ShowAlert("Reached maximum frame count for movie!",
				  "Ending current movie file.",
				  NULL);
#endif
	}
	return(0);
}

int jpeg_stop_movie(dpy)
Display *dpy;
{
	int value;

	if (NewJPEGMovie == FALSE) {
#ifdef HYPERCMEDIA
		hCm_ShowAlert("No open JPEG movie file exists!",NULL,NULL);
#else
		ShowAlert("No open JPEG movie file exists!",NULL);
#endif
		return(-1);
	}

/* Dump the asyncio buffer to the file if required */

#ifdef ASYNCWRITE
	if ((bufend - bufp) != BUFFERSIZE) {
#ifdef SVR4
		if (FrameCount) { sighold(SIGALRM); aiowait(NULL); sigrelse(SIGALRM); }
#endif
		if (BufSync) {
			aiowrite(MovieFd,Abuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			AioResult.aio_return   = AIO_INPROGRESS;
			bufp = Bbuffer; bufend = Bbuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		else {
			aiowrite(MovieFd,Bbuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			AioResult.aio_return   = AIO_INPROGRESS;
			bufp = Abuffer; bufend = Abuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
	}
#endif

/* Dump out the frame index buffer */
#ifdef ASYNCWRITE
#ifdef SVR4
	if (FrameCount) {
		sighold(SIGALRM); aiowait(NULL); sigrelse(SIGALRM);
	}
#else
	if (FrameCount) aiowait(NULL);
#endif
	JPHeader.indexbuf = BufOffset + sizeof(jpheader);
	value = sizeof(int) * (FrameCount + 1);
	AioResult.aio_return = AIO_INPROGRESS;
	aiowrite(MovieFd,(char *)indexbuf,value,BufOffset,SEEK_CUR,&AioResult);
#ifdef SVR4
	if (FrameCount) { sighold(SIGALRM); aiowait(NULL); sigrelse(SIGALRM); }
#endif
#else
	JPHeader.indexbuf = lseek(MovieFd,0L,SEEK_CUR);
	write(MovieFd,(char *) indexbuf,(sizeof(int) * FrameCount));
#endif	
/* Update the header info in the file */

	JPHeader.frames = FrameCount;
	if (FrameCount)
		JPHeader.bandwidth = ((JPHeader.bandwidth / FrameCount) * JPHeader.fps) / 1024;
	else JPHeader.bandwidth = 0;

	if (lseek(MovieFd,0L,SEEK_SET) < 0) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Reset of MovieFd failed");
#else
		SystemError(errno,"Reset of MovieFd failed");
#endif
		close(MovieFd);
		return(-1);
	}
	
/* Update the header info */

	if (write(MovieFd,(char *)&JPHeader,sizeof(jpheader)) < 0) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"JPEG file header rewrite failed!");
#else
		SystemError(errno,"JPEG file header rewrite failed!");
#endif
		close(MovieFd);
		return(-1);
	}
	close(MovieFd);
	FrameCount    = 0;
	NewJPEGMovie  = 0;
}

int jpeg_play_movie(dpy,video,filename,track)
Display *dpy;
VideoIn  video;
char    *filename;
int      track;
{
	int fd;
	int live=0;
	int save=0;
	int fact=0;
	int offset;
	int audio;
	double vol;

/* Open the compressed video file */

	if ((fd = open(filename,O_RDONLY)) < 0) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Couldn't open j_movie file");
#else
		SystemError(errno,"Couldn't open j_movie file");
#endif
		return(-1);
	}

/* Load the j_movie header information */

	if ((offset = read(fd,Abuffer,BUFFERSIZE)) < 0) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Load of j_movie header information failed");
#else
		SystemError(errno,"Load of j_movie header information failed");
#endif
		close(fd);
		return(-1);
	}
	memcpy(&JPHeader,Abuffer,sizeof(jpheader));

	bufp   = Abuffer + sizeof(jpheader);
	bufend = Abuffer + offset;

/* Verify the file type and version */

	if (strcmp(JPHeader.magic,"j_movie")) {
#ifdef HYPERCMEDIA
		hCm_ShowAlert("Requested file (%s) is not a j_movie file!",filename,NULL);
#else
		ShowAlert("Requested file is not a j_movie file!","",filename,NULL);
#endif
		close(fd); return(-1);
	}
	
/* Open up port for communicating with the audio driver, if needed */

	if (JPHeader.audio.data_size != 0) {
		if ((audio = open("/dev/audio",O_WRONLY|O_NDELAY)) < 0) {
			if (errno == EBUSY) {
#ifdef HYPERCMEDIA
				hCm_ShowAlert("Audio device is currently busy, try again later...",NULL,NULL);
#else
				ShowAlert("Audio device is currently busy, try again later...",NULL);
#endif
				close(fd); return(-1);
			}
			else {
#ifdef HYPERCMEDIA
				hCm_SystemError(errno,"Audio device open failed");
#else
				SystemError(errno,"Audio device open failed");
#endif
				close(fd); return(-1);
			}
		}

/* Make sure audio device matches audio file requirements */

		if (audio_set_play_config(audio,&(JPHeader.audio)) != AUDIO_SUCCESS) {
#ifdef HYPERCMEDIA
			hCm_ShowAlert("Couldn't reset audio device encoding",NULL,NULL);
#else
			ShowAlert("Couldn't reset audio device encoding",NULL);
#endif
			close(audio); close(fd);
			return(-1);
		}

/* Set playback volume (currently disabled)

		vol =  60.0 / 100.0;

		if (audio_set_play_gain(audio,&vol) != AUDIO_SUCCESS) {
#ifdef HYPERCMEDIA
			hCm_ShowAlert("Could not set audio gain!",NULL,NULL);
#else
			ShowAlert("Could not set audio gain!",NULL);
#endif
			close(audio); close(fd);
			return(-1);
		}
*/
	}
	else audio = 0;

/* Install the q factor that was used for the movie */

	if (JPHeader.qfactor != 0) {
		fact = (int) xv_get(video,VIDEO_Q_FACTOR);
		xv_set(video,
			   VIDEO_Q_FACTOR,   JPHeader.qfactor,
			   NULL);
	}

/* Set the video object to still frames and no still save */

	live = (int) xv_get(video,VIDEO_LIVE);
	save = (int) xv_get(video,VIDEO_SAVE_STILL);

	xv_set(video,
	   VIDEO_SAVE_STILL, FALSE,
	   VIDEO_LIVE,		 FALSE,
	   NULL);

/* Decode the j_movie */

	jpeg_decode_file(dpy,fd,audio,video,track);

/* Set the video object to previous state */

	xv_set(video,
	   VIDEO_SAVE_STILL, save,
	   VIDEO_LIVE,		 live,
	   NULL);

	if (JPHeader.qfactor != 0) xv_set(video,VIDEO_Q_FACTOR,fact,NULL);

/* Make sure audio buffer has emptied out */

	if (audio) {
		audio_flush_play(audio);
		close(audio);
	}
	close(fd);
}

jpheader *jpeg_movie_header(dpy,filename)
Display *dpy;
char *filename;
{
	int fd;
	jpheader *j;

/* Open the JPEG compressed video file */

	if ((fd = open(filename,O_RDONLY)) < 0) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Couldn't open j_movie file");
#else
		SystemError(errno,"Couldn't open j_movie file");
#endif
		return(NULL);
	}

/* Load the j_movie header information */

	j = (jpheader *) calloc(1,sizeof(jpheader));

	if (read(fd,j,sizeof(jpheader)) < 0) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Load of j_movie header information failed");
#else
		SystemError(errno,"Load of j_movie header information failed");
#endif
		close(fd);
		free(j);
		return(NULL);
	}
	return(j);
}

int jpeg_interrupt_movie(dpy)
Display *dpy;
{
	StopMovie = TRUE;
}

/* end of jpeg.c */
