/*
 * (c) Copyright 1991 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log: jvc.c,v $
 * Revision 1.3  1995/08/30  18:49:55  drapeau
 * Reformatted the code to eliminate many annoying but minor compiler
 * warnings.
 *
 * Revision 1.2  1993/11/19  04:32:13  drapeau
 * Modified the code to add support for SVR4.
 *
 * Revision 1.1  1993/09/16  15:28:55  schen
 * Initial revision
 *
 * Revision 1.8  93/03/15  16:41:49  schen
 * new release from Paradise Software.
 * 
 * Revision 1.2  91/08/19  13:09:26  eric
 * added rcs header
 * 
 *
 */

#ifndef lint
static char *rcsid = "@(#)$Id: jvc.c,v 1.3 1995/08/30 18:49:55 drapeau Exp $ Parallax Graphics Inc";
#endif

/*
 *	jvc.c - low level routines for driving the JVC player
 *
 *	Written by Shirley Chen
 *
 */

/* #define DEBUG */

/* INCLUDE FILES */

#include	<stdio.h>
#include	<sys/types.h>
#include	<sys/termios.h>
#include	<sys/filio.h>
#include	<fcntl.h>
#include	<string.h>
#include	<errno.h>
#include	"alert.h"

/* CONSTANTS */

#define BUFLEN			64

/* Define the instructions for communicating to a JVC */

/* Basic commands */

#define Valid			0xa
#define NotValid		0xb
#define Completion		0x1
#define Clear			0x56
#define Enter			0x40
#define Eject			0xa3
#define Play			0x3a
#define Pause			0x4f
/* This is the slowest reverse we got here */
#define Reverse			0xb6
#define Stop			0x3f

#define StepForward		0xad
#define SlowForward		0xb5
#define FastForward		0xab

#define StepReverse		0xae
#define SlowReverse		0xb6
#define FastReverse		0xac

#define GotoFrame		0xb2
#define PlaytoFrame		0xb4
#define StartFrame		0xe3
#define EndFrame		0xe4

#define AudioStereo		""
#define AudioRight		""
#define AudioLeft		""
#define AudioNormal		""

#define AudioOff		""

#define VideoOn			""
#define VideoOff		""


#define InqRequest	0xfb
#define StatusRequest	0xd7
#define CtlRequest	0xd9
#define CounterReset	0xe2

/* Various other constants for the laser player */

#ifndef NEEDED
#define AUDIO_OFF			401
#define LEFT_ONLY			402
#define RIGHT_ONLY			403
#define STEREO				404

#define VIDEO_OFF			410
#define VIDEO_ON			411
#endif

/* GLOBALS */

extern int PlayerId;		/* File descriptor for the JVC player */
static char buffer[BUFLEN];	/* Buffer for messages from the player */
static char *contents;		/* Pointer to current location in buffer */
static char message[256];	/* Scratch buffer for strings */

/* FUNCTIONS */

/* Send a command to the JVC player and collect response if any */

static int Send(command,count,sleeptime)
     char *command;
     int count;
     int sleeptime;
{
  int i,j;
  
  if (ioctl(PlayerId,TCFLSH,2) < 0)				    /* Flush any exising characters in the queue */
  {
#ifdef DEBUG
    perror("Couldn't flush I/O queues!");
#endif
    return(-1);
  }
#ifdef DEBUG
  fprintf(stderr,"Sending command : %s\n",command);
#endif
  if (write(PlayerId,command,1) < 0) {
    SystemError(errno,"Write to device failed!");
    return (-1);
  }
  if (sleeptime) sleep(sleeptime);
  i = read(PlayerId,buffer,BUFLEN);
  if ((!i) || i < count) {
#ifdef DEBUG
    fprintf(stderr,"Request for %d bytes, received %d bytes in read in Send\n", count, i);
#endif
  }
  
  if (buffer[0] == NotValid) {
    char cmd = Clear;
    if (write(PlayerId,&cmd,1) < 0) {
      SystemError(errno,"Write to device failed!");
      return (-1);
    }
  }
  
  if (i < BUFLEN)
    buffer[i] = '\0';
#ifdef DEBUG
  buffer[i] = 0;
  fprintf(stderr,"Player returned (%d) chars= %s\n",i,buffer);
#endif
  return(0);
} /* end function Send */



int JVCStatus()							    /* Return a simple status code for the player */
{
  int status;
  char cmd;
	
  if (PlayerId < 0)						    /* Make sure the player is initialized correctly */
  {
    ShowAlert("JVCStatus: VideoTape Player not initialized",NULL);
    close(PlayerId);
    PlayerId = -1;
  }
	
  cmd = Clear;
  if (Send(&cmd,1,1) < 0) {
    ShowAlert("JVCStatus: Player status request failed!",NULL);
    close(PlayerId);
    PlayerId = -1;
    return(-1);
  }
  cmd = InqRequest;						    /* Check whether the VCR is connected */
  if (Send(&cmd,1,1) < 0)
  {
    ShowAlert("JVCStatus: Player status request failed!",NULL);
    close(PlayerId);
    PlayerId = -1;
    return(-1);
  }
	
  if (buffer[0] != Valid) {
    ShowAlert("JVCStatus: Player status request failed!",NULL);
    close(PlayerId);
    PlayerId = -1;
    return(-1);
  }
	
  cmd = StatusRequest;						    /* Get play mode status */
  if (Send(&cmd,5,1) < 0) {
    ShowAlert("JVCStatus: Player status request failed!",NULL);
    close(PlayerId);
    PlayerId = -1;
    return(-1);
  }
	
  if (buffer[0] & 0x1) {					    /* Check for error */
    ShowAlert("JVCStatus: Player status request failed!",NULL);
    close(PlayerId);
    PlayerId = -1;
    return(-1);
  }
  return(0);
} /* end function JVCStatus */



int InitializeJVC(device,speed)					    /* Initialize the JVC player to a known state */
     char *device;
     int speed;
{
  int i,retcode;
  struct termios ttbuf;
  char cmd;
	
  if ((PlayerId = open(device, O_RDWR | O_NDELAY)) < 0) {	    /* Open up port for communicating with the video player */
    SystemError(errno,"Couldn't open video player device!");
    PlayerId = -1;
    return(-1);
  }
  (void) ioctl(PlayerId, TCGETS, &ttbuf);			    /* Set the default speed to 1200 baud and RAW character mode */
  
  ttbuf.c_iflag	= (long) 0;
  ttbuf.c_oflag	= (long) 0;
  ttbuf.c_lflag	= (long) 0;
  
  switch (speed) {
   case 0:
   case 1:
    ttbuf.c_cflag = (long) B1200 | CS8 | CLOCAL | CREAD | (B1200 << IBSHIFT) ;
    break;
   case 2:
    ttbuf.c_cflag = (long) B2400 | CS8 | CLOCAL | CREAD | (B2400 << IBSHIFT) ;
    break;
   case 3:
    ttbuf.c_cflag = (long) B4800 | CS8 | CLOCAL | CREAD | (B4800 << IBSHIFT) ;
    break;
   case 4:
    ttbuf.c_cflag = (long) B9600 | CS8 | CLOCAL | CREAD | (B9600 << IBSHIFT) ;
    break;
  }
  
  ttbuf.c_cc[VMIN]	= 6;
  ttbuf.c_cc[VTIME]	= 10;
  
  if ((retcode = ioctl(PlayerId, TCSETSW, &ttbuf)) < 0) {
    SystemError(errno,"Couldn't initialize device!");
    close(PlayerId);
    PlayerId = -1;
    return(-1);
  }
  
  if ((retcode = ioctl(PlayerId, TCFLSH, 2)) < 0) {		    /* Flush any garbage out of the I/O queues */
    SystemError(errno,"Couldn't flush I/O queues!");
    close(PlayerId);
    PlayerId = -1;
    return(-1);
  }
  
  if (JVCStatus())						    /* Find out if video player is ready to go */
    return(-1);
  
  cmd = FastReverse;						    /* Rewind the tape */
  if (Send(&cmd,1,1) < 0) {
    close(PlayerId);
    PlayerId = -1;
    return(-1);
  }
  
  cmd = StatusRequest;
  while (1) {
    if (Send(&cmd,5,1) < 0) {
      return(-1);
    }
    if (!(buffer[3] & 0x60))
      break;
  }
  
  cmd = CounterReset;						    /* Set the tape counter to zero */
  if (Send(&cmd,1,1) < 0) {
    close(PlayerId);
    PlayerId = -1;
    return(-1);
  }
  
  cmd = Enter;
  if (Send(&cmd,1,1) < 0) {
    close(PlayerId);
    PlayerId = -1;
    return(-1);
  }
  return(0);
} /* end function InitializeJVC */



int JVCFrameNumber()						    /* Return current frame number from player */
{
  int status;
  int number;
  char cmd;
  char *b;
	
  cmd = CtlRequest;						    /* Make sure the player is initialized correctly */
  if (PlayerId < 0) {
    ShowAlert("JVCFrameNumber: VideoTape Player not initialized",NULL);
    return(-1);
  }
	
  /*
   * JVC returns 8 to 9 bytes of data for reason I don't know of.
   */
  if (Send(&cmd,9,1) < 0) {
    ShowAlert("JVCFrameNumber: Player address request failed!",NULL);
    return(-1);
  }
  if ((buffer[0] & 0x20) == Valid)
    b = &buffer[7];						    /* 9 bytes returned */
  else
    b = &buffer[6];						    /* 8 bytes returned */
	
  number = atoi(b);						    /* frames */
  *b = '\0';
  b -= 2;
  number += (atoi(b) * 25);					    /* seconds */
  *b = '\0';
  b -= 2;
  number += (atoi(b) * 25 * 60);				    /* minutes */
  *b = '\0';
  b -= 2;
  number += (atoi(b) * 25 * 60 * 60);				    /* hours */
  return(number);
} /* end function JVCFrameNumber */



int SetJVCAudio(mode)						    /* Set Audio parameters */
     int mode;
{
  char *command;
	
  if (PlayerId < 0) {						    /* Make sure the player is initialized correctly */
    ShowAlert("SetJVCAudio: VideoTape Player not initialized",NULL);
    return(-1);
  }
  ShowNote("JVCIndex: This function not supported by JVC",NULL);
  return(-1);
#ifdef notdef
  switch (mode) {
   case AUDIO_OFF:
    command = AudioOff;
    break;
   case LEFT_ONLY:
    command = AudioLeft;
    break;
   case RIGHT_ONLY:
    command = AudioRight;
    break;
   case STEREO:
    command = AudioStereo;
    break;
   default:
    sprintf(message,"SetJVCAudio: Unknown mode requested for audio - %d",mode);
    ShowAlert(message,NULL);
    return(-1);
  }
  if (Send(command,5,1) < 0) {
    ShowAlert("SetJVCAudio: Audio parameters not changed!",NULL);
    return(-1);
  }
  return(0);
#endif
} /* end function SetJVCAudio */



int SetJVCVideo(mode)						    /* Set whether the video is visible or not */
     int mode;
{
  char *command;
  
  if (PlayerId < 0) {						    /* Make sure the player is initialized correctly */
    ShowAlert("SetJVCVideo: VideoTape Player not initialized",NULL);
    return(-1);
  }
  ShowNote("JVCIndex: This function not supported by JVC",NULL);
  return(-1);
#ifdef notdef
  switch (mode) {
   case VIDEO_OFF:
    command = VideoOff;
    break;
   case VIDEO_ON:
    command = VideoOn;
    break;
   default:
    sprintf(message,"SetJVCVideo: Unknown mode requested for video - %d",mode);
    ShowAlert(message,NULL);
    return(-1);
  }
  if (Send(command,5,1) < 0) {
    ShowAlert("SetJVCVideo: Video parameters not changed!",NULL);
    return(-1);
  }
  return(NULL);
#endif
}



int ShowJVCFrame(frame)						    /* Seek and show a single frame */
     int frame;
{
  int i;
  int sec = 0;
  int min = 0;
  int hrs = 0;
  char cmd[30];
  char *c = cmd;
  
  if (PlayerId < 0) {						    /* Make sure the player is initialized correctly */
    ShowAlert("ShowJVCFrame: VideoTape Player not initialized",NULL);
    return(-1);
  }
  
  if ((frame < 0) || (frame > 900000)) {			    /* Check for valid parameter */
    sprintf(message,"ShowJVCFrame: Invalid Frame address - %d",frame);
    ShowAlert(message,NULL);
    return(-1);
  }
  
  sec = frame / 25;
  if (sec > 60) {
    min = sec / 60;
    sec = sec % 60;
  }
  if (min > 60) {
    hrs = min / 60;
    hrs %= 24;
    min = min % 60;
  }
  frame = frame % 25;
  
  *c++ = GotoFrame;
  sprintf(c, "%2d%2d%2d%2d\n", hrs, min, sec, frame);
  c += 8;
  
  for (i = 0; i < 9; ++i)
  {
    if (cmd[i] == 0x20)						    /* replace the blank with 0 */
      cmd[i] = 0x30;
    if (Send(&cmd[i],1,1) < 0) {
      ShowAlert("ShowJVCFrame: Seek frame failed",NULL);
      return(-1);
    }
  }
  
  while(1) {
    cmd[i] = StatusRequest;
    if (Send(&cmd[i],5,1) < 0) {
      ShowAlert("ShowJVCFrame: Seek frame failed",NULL);
      return(-1);
    }
    if ((buffer[4] & 0x80) == 0x80) break;
  }
  
  cmd[++i] = Enter;
  if (Send(&cmd[i],1,1) < 0) {
    ShowAlert("ShowJVCFrame: Seek frame failed",NULL);
    return(-1);
  }
} /* end function ShowJVCFrame */

#ifdef notdef
/* Seek and show a single frame by elapsed time */

static int ShowJVCTime(minutes,seconds)
     int minutes,seconds;
{
  int number;
  char command[16];
  
  if (PlayerId < 0) {						    /* Make sure the player is initialized correctly */
    ShowAlert("ShowJVCTime: VideoTape Player not initialized",NULL);
    return(-1);
  }
  
  if ((minutes < 0) || (minutes > 120)) {			    /* Check for valid parameters */
    sprintf(message,"ShowJVCTime: Invalid time (minutes) - %d",minutes);
    ShowAlert(message,NULL);
    return(-1);
  }
  
  if ((seconds < 0) || (seconds > 60)) {
    sprintf(message,"ShowJVCTime: Invalid time (seconds) - %d",seconds);
    ShowAlert(message,NULL);
    return(-1);
  }
  
  number = (minutes * 60 + seconds) * 30;			    /* Send out a search for time request */
  sprintf(command,"%s%d\r",GotoFrame,number);
  if (Send(command,5,1) < 0) {
    ShowAlert("ShowJVCTime: Frame selection failed!",NULL);
    return(-1);
  }
  return(NULL);
} /* end function ShowJVCTime */
#endif



int ShowJVCSegment(start,end)					    /* Seek and show a complete video segment */
     int start,end;
{
  int i;
  int sec = 0;
  int min = 0;
  int hrs = 0;
  char cmd[30];
  char *c = cmd;
  
  if (PlayerId < 0) {						    /* Make sure the player is initialized correctly */
    ShowAlert("ShowJVCSegment: VideoTape Player not initialized",NULL);
    return(-1);
  }
  
  if ((start < 0) || (start > 900000)) {			    /* Check for valid parameters */
    sprintf(message,"ShowJVCSegment: Invalid Frame address - %d",start);
    ShowAlert(message,NULL);
    return(-1);
  }
  if ((end < 0) || (end > 900000)) {
    sprintf(message,"ShowJVCSegment: Invalid Frame address - %d",end);
    ShowAlert(message,NULL);
    return(-1);
  }
  if (start >= end) {
    sprintf(message,"ShowJVCSegment: Starting frame (%d) not less than end (%d)",start,end);
    ShowAlert(message,NULL);
    return(-1);
  }
  
  sec = start / 25;
  if (sec > 60) {
    min = sec / 60;
    sec = sec % 60;
  }
  if (min > 60) {
    hrs = min / 60;
    hrs %= 24;
    min = min % 60;
  }
  start = start % 25;
  
  *c++ = StartFrame;
  sprintf(c, "%2d%2d%2d%2d\n", hrs, min, sec, start);
  c += 8;
  *c++ = Enter;
  
  sec = end / 25;
  if (sec > 60) {
    min = sec / 60;
    sec = sec % 60;
  }
  if (min > 60) {
    hrs = min / 60;
    hrs %= 24;
    min = min % 60;
  }
  end = end % 25;
  
  *c++ = EndFrame;
  sprintf(c, "%2d%2d%2d%2d\n", hrs, min, sec, end);
  c += 8;
  *c++ = Enter;
  *c++ = PlaytoFrame;
  *c++ = 0x31;
  *c++ = Enter;
  
  for (i = 0; i < 23; ++ i)
  {
    if (cmd[i] == 0x20)						    /* replace the blank with 0 */
      cmd[i] = 0x30;
    if (Send(&cmd[i],1,1) < 0) {
      ShowAlert("ShowJVCSegment: VideoTape Player not initialized",NULL);
      return(-1);
    }
  }
  
  while(1) {
    i = read(PlayerId,buffer,BUFLEN);
    if (buffer[0] == Completion)
      return(0);
    else
      sleep(1);
  }
} /* end function ShowJVCSegment */



int JVCPlay()							    /* Command : play forward */
{
  char cmd = Play;
  
  if (PlayerId < 0) {						    /* Make sure the player is initialized correctly */
    ShowAlert("JVCPlay: VideoTape Player not initialized",NULL);
    return(-1);
  }
  if (Send(&cmd,1,1) < 0) {
    ShowAlert("JVCPlay: Play activation failed!",NULL);
    return(-1);
  }
  return(0);
} /* end function JVCPlay */



int JVCReverse()						    /* Command : play reverse */
{
  char cmd = Reverse;
  
  if (PlayerId < 0) {						    /* Make sure the player is initialized correctly */
    ShowAlert("JVCReverse: VideoTape Player not initialized",NULL);
    return(-1);
  }
  if (Send(&cmd,1,1) < 0) {
    ShowAlert("JVCReverse: Reverse activation failed!",NULL);
    return(-1);
  }
  cmd = 0x35;							    /* speed */
  if (Send(&cmd,1,1) < 0) {
    ShowAlert("JVCSlowForward: Scan forward activation failed!",NULL);
    return(-1);
  }
  return(0);
} /* end function JVCReverse */



int JVCContinue()						    /* Command : Continue an interrupted segment */
{
  char cmd = Play;
  
  if (PlayerId < 0) {						    /* Make sure the player is initialized correctly */ 
    ShowAlert("JVCContinue: VideoTape Player not initialized",NULL);
    return(-1);
  }
  if (Send(&cmd,1,1) < 0) {
    ShowAlert("JVCContinue: Play continuation activation failed!",NULL);
    return(-1);
  }
  return(0);
} /* end function JVCContinue */



int JVCStill()							    /* Command : pause during playback */
{
  char cmd = Pause;
  
  if (PlayerId < 0) {						    /* Make sure the player is initialized correctly */
    ShowAlert("JVCStill: VideoTape Player not initialized",NULL);
    return(-1);
  }
  if (Send(&cmd,1,1) < 0) {
    ShowAlert("JVCStill: Freeze frame activation failed!",NULL);
    return(-1);
  }
  return(0);
} /* end function JVCStill */



int JVCStop()							    /* Command : Stop playback */
{
  char cmd = Stop;
  
  if (PlayerId < 0) {						    /* Make sure the player is initialized correctly */
    ShowAlert("JVCStop: VideoTape Player not initialized",NULL);
    return(-1);
  }
  if (Send(&cmd,1,1) < 0) {
    ShowAlert("JVCStop: Stop player activation failed!",NULL);
    return(-1);
  }
  return(0);
} /* end function JVCStop */



int JVCStepForward()						    /* Command : move 1 frame forward */
{
  char cmd = StepForward;
  
  if (PlayerId < 0) {						    /* Make sure the player is initialized correctly */
    ShowAlert("JVCStepForward: VideoTape Player not initialized",NULL);
    return(-1);
  }
  if (Send(&cmd,1,1) < 0) {
    ShowAlert("JVCStepForward: Step forward activation failed!",NULL);
    return(-1);
  }
  return(0);
} /* end function JVCStepForward */



int JVCStepReverse()						    /* Command : move 1 frame backward */
{
  char cmd = StepReverse;
  
  if (PlayerId < 0) {						    /* Make sure the player is initialized correctly */
    ShowAlert("JVCStepReverse: VideoTape Player not initialized",NULL);
    return(-1);
  }
  
  if (Send(&cmd,1,1) < 0) {
    ShowAlert("JVCStepReverse: Step reverse activation failed!",NULL);
    return(-1);
  }
  return(0);
} /* end function JVCStepReverse */



int JVCSlowForward()						    /* Command : move slow forward */
{
  char cmd = SlowForward;
  
  if (PlayerId < 0) {						    /* Make sure the player is initialized correctly */
    ShowAlert("JVCSlowForward: VideoTape Player not initialized",NULL);
    return(-1);
  }
  if (Send(&cmd,1,1) < 0) {
    ShowAlert("JVCSlowForward: Scan forward activation failed!",NULL);
    return(-1);
  }
  cmd = 0x36;							    /* speed */
  if (Send(&cmd,1,1) < 0) {
    ShowAlert("JVCSlowForward: Scan forward activation failed!",NULL);
    return(-1);
  }
  return(0);
} /* end function JVCSlowForward */



int JVCSlowReverse()						    /* Command : move slow reverse */
{
  char cmd = SlowReverse;
  
  if (PlayerId < 0) {						    /* Make sure the player is initialized correctly */
    ShowAlert("JVCSlowReverse: VideoTape Player not initialized",NULL);
    return(-1);
  }
  if (Send(&cmd,1,1) < 0) {
    ShowAlert("JVCSlowForward: Scan forward activation failed!",NULL);
    return(-1);
  }
  cmd = 0x36;							    /* speed */
  if (Send(&cmd,1,1) < 0) {
    ShowAlert("JVCSlowForward: Scan forward activation failed!",NULL);
    return(-1);
  }
  return(0);
} /* end function JVCSlowReverse */



int JVCFastForward()						    /* Command : move fast forward */
{
  char cmd = FastForward;
  
  if (PlayerId < 0) {						    /* Make sure the player is initialized correctly */
    ShowAlert("JVCFastForward: VideoTape Player not initialized",NULL);
    return(-1);
  }
  if (Send(&cmd,1,1) < 0) {
    ShowAlert("JVCFastForward: Scan forward activation failed!",NULL);
    return(-1);
  }
  return(0);
} /* end function JVCFastForward */



int JVCFastReverse()						    /* Command : move fast reverse */
{
  char cmd = FastReverse;
  
  if (PlayerId < 0) {						    /* Make sure the player is initialized correctly */
    ShowAlert("JVCFastReverse: VideoTape Player not initialized",NULL);
    return(-1);
  }
  if (Send(&cmd,1,1) < 0) {
    ShowAlert("JVCFastReverse: Scan reverse activation failed!",NULL);
    return(-1);
  }
  return(0);
} /* end function JVCFastReverse */



int JVCIndex(status)						    /* Command : display visual frame location */
     int status;
{
  char command[8];
  
  if (PlayerId < 0) {						    /* Make sure the player is initialized correctly */
    ShowAlert("JVCIndex: VideoTape Player not initialized",NULL);
    return(-1);
  }
  ShowNote("JVCIndex: This function not supported by JVC",NULL);
  return(0);
} /* end function JVCIndex */



int JVCEject()							    /* Command : eject video source */
{
  char cmd;
  
  if (PlayerId < 0) {						    /* Make sure the player is initialized correctly */ 
    ShowAlert("JVCEject: VideoTape Player not initialized",NULL);
    return(-1);
  }
  cmd = Stop;
  if (Send(&cmd,1,1) < 0) {
    ShowAlert("JVCEject: Player ejection activation failed!",NULL);
    return(-1);
  }
  cmd = Eject;
  if (Send(&cmd,1,1) < 0) {
    ShowAlert("JVCEject: Player ejection activation failed!",NULL);
    return(-1);
  }
  return(0);
} /* end function JVCEject */

/* end of jvc.c */
