/*
 * (c) Copyright 1991 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log: pc-vcr.c,v $
 * Revision 1.11  1994/01/20  01:32:54  drapeau
 * Removed extraneous conditional compilation directives, since including
 * "string.h" always works no matter what OS platform.
 *
 * Revision 1.10  1993/11/19  04:32:13  drapeau
 * Modified the code to add support for SVR4.
 *
 * Revision 1.2  91/08/19  13:09:26  eric
 * added rcs header
 * 
 *
 */

#ifndef lint
static char *rcsid = "@(#)$Id: pc-vcr.c,v 1.11 1994/01/20 01:32:54 drapeau Exp $ Parallax Graphics Inc";
#endif

/*
 *	pc-vcr.c - low level routines for driving the NEC PC-VCR player
 *
 *	Written by John Malleo-Roach
 *
 */

/* #define DEBUG */

/* INCLUDE FILES */

#include	<stdio.h>
#include	<sys/types.h>
#include	<sys/termios.h>
#include	<sys/filio.h>
#include	<fcntl.h>
#include	<string.h>
#include	<errno.h>
#include	"alert.h"

/* CONSTANTS */

#define BUFLEN			64

/* Define the instructions for communicating to a NEC PC-VCR */

/* Signaling status */

#define Valid			"CP\r"
#define Ready			"AO\r"
#define Clear			"CL\r"
#define Reset			"CV\r"
#define Interrupt		"EP\r"
#define Error			"EE0"

/* Returned errors */

#define EPowOff			"EE000"
#define EBadAddr		"EE001"
#define ENoTape			"EE002"
#define EBadCom			"EE004"
#define EBadParm		"EE005"
#define EOverFlow		"EE006"
#define EBadHead		"EE008"
#define EFailCom		"EE009"

/* Status codes */

#define SNoTape			"SS00"
#define STapeStop		"SS01"

/* Basic commands */

#define Eject			"EJ\r"
#define Play			"PL\r"
#define Pause			"PS\r"
/* This is the slowest reverse we got here */
#define Reverse			"RW1\r"
#define Stop			"ST\r"

#define StepForward		"FS\r"
#define SlowForward		"SL+\r"
#define MedForward		"FF1\r"
#define FastForward		"FF0\r"
#define ScanForward		"FF2\r"

#define StepReverse		""
#define SlowReverse		""
#define MedReverse		"RW1\r"
#define FastReverse		"RW0\r"
#define ScanReverse		"RW2\r"

#define GotoFrame		"JF"
#define PlaytoFrame		"SP"

#define AudioStereo		"RLS\r"
#define AudioRight		"RLR\r"
#define AudioLeft		"RLL\r"
#define AudioNormal		"RLN\r"

#define AudioOff		"AM0\r"
#define AudioOn			"AM1\r"

#define VideoOn			"VM1\r"
#define VideoOff		"VM0\r"

#define Initialize		"CV\r"

#define StatusRequest	"DS\r"
#define AddressRequest	"RP0\r"
#define FrameRequest	"RF\r"
#define CounterReset	"CR\r"

/* Various other constants for the laser player */

#define AUDIO_OFF			401
#define LEFT_ONLY			402
#define RIGHT_ONLY			403
#define STEREO				404

#define VIDEO_OFF			410
#define VIDEO_ON			411

/* Player status codes, these are the same in all device driver files */

#define PLAYER_STATUS_UNKNOWN	0
#define PLAYER_STATUS_STOP		1
#define PLAYER_STATUS_STILL		2
#define PLAYER_STATUS_PLAY		3
#define PLAYER_STATUS_EMPTY		4

/* GLOBALS */

extern int PlayerId;			/* File descriptor for the NEC PC-VCR player		*/
static char buffer[BUFLEN];		/* Buffer for messages from the player				*/
static char *contents;			/* Pointer to current location in buffer			*/
static char message[256];		/* Scratch buffer for strings						*/

/* FUNCTIONS */

/* Send a command to the NEC PC-VCR player and collect response if any */

static int Send(command,count,sleeptime)
char *command;
int count;
int sleeptime;
{
	int i,j;

/* Flush any exising characters in the queue */

	if (ioctl(PlayerId,TCFLSH,2) < 0) {
#ifdef DEBUG
		perror("Couldn't flush I/O queues!");
#endif
		return(-1);
	}
#ifdef DEBUG
	fprintf(stderr,"Sending command : %s\n",command);
#endif
	if (write(PlayerId,command,strlen(command)) < 0) {
		SystemError(errno,"Write to device failed!");
		return (-1);
	}
	if (sleeptime) sleep(sleeptime);
	while (count) {
		i = read(PlayerId,buffer,BUFLEN);
		if (i == 0) {
#ifdef DEBUG
			 fprintf(stderr,"Received 0 bytes in read in Send\n");
#endif
			count--;
			continue;
		}
#ifdef DEBUG
		buffer[i] = 0;
		fprintf(stderr,"Player returned (%d) chars= %s\n",i,buffer);
#endif
		contents = buffer;
		if (strncmp(contents,Valid,strlen(Valid)) == 0) {
#ifdef DEBUG
			fprintf(stderr,"Received Valid on count %d\n",count);
#endif
			contents = (char *) &(contents[3]);
			i -= 3;
		}
		else if (strncmp(contents,Error,strlen(Error)) == 0) {
			contents[6] = '\0';
			sprintf(message,"SEND: Error on previous command : %s",contents);
			ShowAlert(message,NULL);
			return(-1);
		}
		if (i == 0) continue;
		if (strncmp(contents,Ready,strlen(Ready)) == 0) {
#ifdef DEBUG
			fprintf(stderr,"Received Ready on count %d\n",count);
#endif
			contents = (char *) &(contents[3]);
			i -= 3;
			if (i == 0) break;
		}
		else if (strncmp(contents,Error,strlen(Error)) == 0) {
			contents[6] = '\0';
			sprintf(message,"SEND: Error on previous command : %s",contents);
			ShowAlert(message,NULL);
			return(-1);
		}
		if (i) {
			j = i - strlen(Ready);
			if (strncmp(&(contents[j]),Ready,strlen(Ready)) == 0) {
#ifdef DEBUG
				fprintf(stderr,"Received Ready (with data) on count %d\n",count);
#endif
				i = j;
				for (j=0; j<i; j++) buffer[j] = contents[j];
				if (buffer[i-1] == '\r') buffer[i-1] = '\0';
				else buffer[i] = '\0';
#ifdef DEBUG
				fprintf(stderr,"Data received is %s\n",buffer);
#endif
				break;
			}
			else {
				sprintf(message,"SEND: junk on previous command : %s",contents);
				ShowAlert(message,NULL);
				return(-1);
			}
		}
	}
	return(NULL);
}

/* Check status untill it equals request */

static int NECWait(status)
char *status;
{
	while (1) {
		if (Send(StatusRequest,5,2) < 0) {
			return(-1);
		}
		if (strcmp(buffer,status) == 0) break;
		sleep(2);
	}
	return(NULL);
}

/* Return a simple status code for the player */

int NECStatus()
{
	int status;

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("NECStatus: VideoTape Player not initialized",NULL);
		return(-1);
	}

/* Get play mode status */

	if (Send(StatusRequest,5,2) < 0) {
		ShowAlert("NECStatus: Player status request failed!",NULL);
		return(PLAYER_STATUS_UNKNOWN);
	}
	switch (buffer[3]) {
	case '0':
		status = PLAYER_STATUS_EMPTY;
		break;
	case '1':
		status = PLAYER_STATUS_STOP;
		break;
	case '7':
		status = PLAYER_STATUS_PLAY;
		break;
	case '9':
		status = PLAYER_STATUS_STILL;
		break;
	default:
		status = PLAYER_STATUS_UNKNOWN;
		break;
	}
	return(status);
}

/* Initialize the NEC player to a known state */

int InitializeNEC(device,speed)
char *device;
int speed;
{
	int i,retcode;
    struct termios ttbuf;

/* Open up port for communicating with the video player */

	if ((PlayerId = open(device, O_RDWR | O_NDELAY)) < 0) {
		SystemError(errno,"Couldn't open video player device!");
		PlayerId = -1;
		return(-1);
	}

/* Set the default speed to 1200 baud and RAW character mode */
	
    (void) ioctl(PlayerId, TCGETS, &ttbuf);

    ttbuf.c_iflag		= (long) 0;
    ttbuf.c_oflag		= (long) 0;
    ttbuf.c_lflag		= (long) 0;

	switch (speed) {
	case 0:
	    ttbuf.c_cflag = (long) B1200 | CS8 | CLOCAL | CREAD | (B1200 << IBSHIFT) ;
		break;
	case 1:
	    ttbuf.c_cflag = (long) B1200 | CS8 | CLOCAL | CREAD | (B1200 << IBSHIFT) ;
		break;
	case 2:
	    ttbuf.c_cflag = (long) B2400 | CS8 | CLOCAL | CREAD | (B2400 << IBSHIFT) ;
		break;
	case 3:
	    ttbuf.c_cflag = (long) B4800 | CS8 | CLOCAL | CREAD | (B4800 << IBSHIFT) ;
		break;
	case 4:
	    ttbuf.c_cflag = (long) B9600 | CS8 | CLOCAL | CREAD | (B9600 << IBSHIFT) ;
		break;
	}

    ttbuf.c_cc[VMIN]	= 6;
    ttbuf.c_cc[VTIME]	= 10;

	if ((retcode = ioctl(PlayerId, TCSETSW, &ttbuf)) < 0) {
		SystemError(errno,"Couldn't initialize device!");
		close(PlayerId);
		PlayerId = -1;
		return(-1);
	}

/* Flush any garbage out of the I/O queues */
	
	if ((retcode = ioctl(PlayerId, TCFLSH, 2)) < 0) {
		SystemError(errno,"Couldn't flush I/O queues!");
		close(PlayerId);
		PlayerId = -1;
		return(-1);
	}

/* Find out if video player is ready to go */

	i = NECStatus();
	if (i == PLAYER_STATUS_EMPTY) {
		ShowNote("Select OKAY after loading videotape into Player",NULL);
		if (NECStatus() == PLAYER_STATUS_EMPTY) {
			ShowNote("InitializeNEC: No tape found! Giving up",NULL);
			close(PlayerId);
			PlayerId = -1;
			return(-1);
		}
	}
	else {
		if (i != PLAYER_STATUS_STOP) {
			if (Send(Stop,5,1) < 0) {
				close(PlayerId);
				PlayerId = -1;
				return(-1);
			}
		}
	}

/* Initialize the unit */

	if (Send(Initialize,5,1) < 0) {
		close(PlayerId);
		PlayerId = -1;
		return(-1);
	}

/* Rewind the tape */

	if (Send(FastReverse,5,1) < 0) {
		close(PlayerId);
		PlayerId = -1;
		return(-1);
	}

/* Wait till rewind is done */

	if (NECWait(STapeStop) < 0) {
		close(PlayerId);
		PlayerId = -1;
		return(-1);
	}

/* Set the tape counter to zero */

	if (Send(CounterReset,5,1) < 0) {
		close(PlayerId);
		PlayerId = -1;
		return(-1);
	}
	return(NULL);
}

/* Return current frame number from player */

int NECFrameNumber()
{
	int status;
	int number;

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("NECFrameNumber: VideoTape Player not initialized",NULL);
		return(-1);
	}
	if (Send(AddressRequest,5,1) < 0) {
		ShowAlert("NECFrameNumber: Player address request failed!",NULL);
		return(-1);
	}
	number = atoi(buffer);
	number = number * 30;
	if (Send(FrameRequest,5,1) < 0) {
		ShowAlert("NECFrameNumber: Player frame request failed!",NULL);
		return(-1);
	}
	number += atoi(buffer);
	return(number);
}

/* Set Audio parameters */

int SetNECAudio(mode)
int mode;
{
	char *command;

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("SetNECAudio: VideoTape Player not initialized",NULL);
		return(-1);
	}
	switch (mode) {
	case AUDIO_OFF:
		command = AudioOff;
		break;
	case LEFT_ONLY:
		command = AudioLeft;
		break;
	case RIGHT_ONLY:
		command = AudioRight;
		break;
	case STEREO:
		command = AudioStereo;
		break;
	default:
		sprintf(message,"SetNECAudio: Unknown mode requested for audio - %d",mode);
		ShowAlert(message,NULL);
		return(-1);
	}
	if (Send(command,5,1) < 0) {
		ShowAlert("SetNECAudio: Audio parameters not changed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* Set whether the video is visible or not */

int SetNECVideo(mode)
int mode;
{
	char *command;

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("SetNECVideo: VideoTape Player not initialized",NULL);
		return(-1);
	}
	switch (mode) {
	case VIDEO_OFF:
		command = VideoOff;
		break;
	case VIDEO_ON:
		command = VideoOn;
		break;
	default:
		sprintf(message,"SetNECVideo: Unknown mode requested for video - %d",mode);
		ShowAlert(message,NULL);
		return(-1);
	}
	if (Send(command,5,1) < 0) {
		ShowAlert("SetNECVideo: Video parameters not changed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* Seek and show a single frame */

int ShowNECFrame(frame)
int frame;
{
	char command[16];

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("ShowNECFrame: VideoTape Player not initialized",NULL);
		return(-1);
	}

/* Check for valid parameter */

	if ((frame < 0) || (frame > 999999)) {
		sprintf(message,"ShowNECFrame: Invalid Frame address - %d",frame);
		ShowAlert(message,NULL);
		return(-1);
	}

/* Send out a search frame request */

	sprintf(command,"%s%d\r",GotoFrame,frame);
	if (Send(command,5,1) < 0) {
		ShowAlert("ShowNECFrame: Frame selection failed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* Seek and show a single frame by elapsed time */

static int ShowNECTime(minutes,seconds)
int minutes,seconds;
{
	int number;
	char command[16];

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("ShowNECTime: VideoTape Player not initialized",NULL);
		return(-1);
	}

/* Check for valid parameters */

	if ((minutes < 0) || (minutes > 120)) {
		sprintf(message,"ShowNECTime: Invalid time (minutes) - %d",minutes);
		ShowAlert(message,NULL);
		return(-1);
	}

	if ((seconds < 0) || (seconds > 60)) {
		sprintf(message,"ShowNECTime: Invalid time (seconds) - %d",seconds);
		ShowAlert(message,NULL);
		return(-1);
	}

/* Send out a search for time request */

	number = (minutes * 60 + seconds) * 30;
	sprintf(command,"%s%d\r",GotoFrame,number);
	if (Send(command,5,1) < 0) {
		ShowAlert("ShowNECTime: Frame selection failed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* Seek and show a complete video segment */

int ShowNECSegment(start,end)
int start,end;
{
	int addr,frame;
	char command[16];

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("ShowNECSegment: VideoTape Player not initialized",NULL);
		return(-1);
	}

/* Check for valid parameters */

	if ((start < 0) || (start > 999999)) {
		sprintf(message,"ShowNECSegment: Invalid Frame address - %d",start);
		ShowAlert(message,NULL);
		return(-1);
	}
	if ((end < 0) || (end > 999999)) {
		sprintf(message,"ShowNECSegment: Invalid Frame address - %d",end);
		ShowAlert(message,NULL);
		return(-1);
	}
	if (start >= end) {
		sprintf(message,"ShowNECSegment: Starting frame (%d) not less than end (%d)",start,end);
		ShowAlert(message,NULL);
		return(-1);
	}
	if ((start+90) >= end) {
		ShowNote("ShowNECSegment: NEC PC-VCR requires",
			"segments to be at least 90 frames",NULL);
		return(0);
	}

/* Send out a search frame request */

	sprintf(command,"%s%d\r",GotoFrame,start);
	if (Send(command,5,1) < 0) {
		ShowAlert("ShowNECSegment: Frame selection failed!",NULL);
		return(-1);
	}

/* Start play with stop point */
	
	addr  = end / 30;
	frame = end % 30;
	sprintf(command,"%s%d:%d\r",PlaytoFrame,addr,frame);
	if (Send(command,5,1) < 0) {
		ShowAlert("ShowNECSegment: Segment play activation failed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* Seek and show a complete video segment by times */

static int ShowNECTimeSegment(smin,ssec,emin,esec)
int smin,ssec,emin,esec;
{
	char number[8];

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("ShowNECTimeSegment: VideoTape Player not initialized",NULL);
		return(-1);
	}

/* Check for valid parameters */

	if ((smin < 0) || (smin > 60)) {
		sprintf(message,"ShowNECTimeSegment: Invalid time (start minutes) - %d",smin);
		ShowAlert(message,NULL);
		return(-1);
	}

	if ((ssec < 0) || (ssec > 60)) {
		sprintf(message,"ShowNECTimeSegment: Invalid time (start seconds) - %d",ssec);
		ShowAlert(message,NULL);
		return(-1);
	}

	if ((emin < 0) || (emin > 60)) {
		sprintf(message,"ShowNECTimeSegment: Invalid time (end minutes) - %d",emin);
		ShowAlert(message,NULL);
		return(-1);
	}

	if ((esec < 0) || (esec > 60)) {
		sprintf(message,"ShowNECTimeSegment: Invalid time (end seconds) - %d",esec);
		ShowAlert(message,NULL);
		return(-1);
	}

/* Send out a search frame request	*/

/* Wait here until sucessful 		*/

/* Start play with stop point		*/
	
	return(NULL);
}

/* Command : play forward */

int NECPlay()
{

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("NECPlay: VideoTape Player not initialized",NULL);
		return(-1);
	}
	if (Send(Play,5,1) < 0) {
		ShowAlert("NECPlay: Play activation failed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* Command : play reverse */

int NECReverse()
{
	char command[12];

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("NECReverse: VideoTape Player not initialized",NULL);
		return(-1);
	}
	if (Send(Reverse,5,1) < 0) {
		ShowAlert("NECReverse: Reverse activation failed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* Command : Continue an interrupted segment */

int NECContinue()
{

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("NECContinue: VideoTape Player not initialized",NULL);
		return(-1);
	}
	if (Send(Play,5,1) < 0) {
		ShowAlert("NECContinue: Play continuation activation failed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* Command : pause during playback */

int NECStill()
{

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("NECStill: VideoTape Player not initialized",NULL);
		return(-1);
	}
	if (Send(Pause,5,1) < 0) {
		ShowAlert("NECStill: Freeze frame activation failed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* Command : Stop playback */

int NECStop()
{

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("NECStop: VideoTape Player not initialized",NULL);
		return(-1);
	}
	if (Send(Stop,5,1) < 0) {
		ShowAlert("NECStop: Stop player activation failed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* Command : move 1 frame forward */

int NECStepForward()
{

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("NECStepForward: VideoTape Player not initialized",NULL);
		return(-1);
	}
	if (Send(StepForward,5,1) < 0) {
		ShowAlert("NECStepForward: Step forward activation failed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* Command : move 1 frame backward */

int NECStepReverse()
{

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("NECStepReverse: VideoTape Player not initialized",NULL);
		return(-1);
	}
	ShowNote("NECStepReverse: This function not supported by NEC PC-VCR",NULL);

	return(NULL);
}

/* Command : move slow forward */

int NECSlowForward()
{
	char command[12];

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("NECSlowForward: VideoTape Player not initialized",NULL);
		return(-1);
	}
	if (Send(SlowForward,5,1) < 0) {
		ShowAlert("NECSlowForward: Scan forward activation failed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* Command : move slow reverse */

int NECSlowReverse()
{
	char command[12];

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("NECSlowReverse: VideoTape Player not initialized",NULL);
		return(-1);
	}
	ShowNote("NECSlowReverse: This function not supported by NEC PC-VCR",NULL);

	return(NULL);
}

/* Command : move fast forward */

int NECFastForward()
{
	char command[12];

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("NECFastForward: VideoTape Player not initialized",NULL);
		return(-1);
	}
	if (Send(ScanForward,5,1) < 0) {
		ShowAlert("NECFastForward: Scan forward activation failed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* Command : move fast reverse */

int NECFastReverse()
{
	char command[12];

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("NECFastReverse: VideoTape Player not initialized",NULL);
		return(-1);
	}
	if (Send(ScanReverse,5,1) < 0) {
		ShowAlert("NECFastReverse: Scan reverse activation failed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* Command : display visual frame location */

int NECIndex(status)
int status;
{
	char command[8];

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("NECIndex: VideoTape Player not initialized",NULL);
		return(-1);
	}
	ShowNote("NECIndex: This function not supported by NEC PC-VCR",NULL);

	return(NULL);
}

/* Command : eject video source */

int NECEject()
{
	char command[12];

/* Make sure the player is initialized correctly */

	if (PlayerId < 0) {
		ShowAlert("NECEject: VideoTape Player not initialized",NULL);
		return(-1);
	}
	if (Send(Eject,5,1) < 0) {
		ShowAlert("NECEject: Player ejection activation failed!",NULL);
		return(-1);
	}
	return(NULL);
}

/* end of pc-vcr.c */
