/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Andy Goodrich
 */
/* $Header: /plx/source/Xclients/generic/RCS/O_GCinfo.c,v 1.2 1995/04/12 23:00:36 drapeau Exp $ */
/* $Log: O_GCinfo.c,v $
 * Revision 1.2  1995/04/12  23:00:36  drapeau
 * Added a key value that is stored with the GC-Window pair, to help
 * verify which widget registered the GC-Window pair.
 *
 * Revision 1.1  1995/04/12  05:01:29  drapeau
 * Initial revision
 * */
static char O_GCinfoRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/O_GCinfo.c,v 1.2 1995/04/12 23:00:36 drapeau Exp $";

#include <stdio.h>
#include <sys/types.h>
#include "O_GCinfo.h"

typedef struct _GCpair {
	int		gc;		/* Graphics context identifier for pair. */
	int		owner;	/* Entity creating pair. */
	int		win;	/* Window identifier for pair. */
} GCpair;

typedef struct _I_GCinfo {
	O_GCinfo	base;	/* Public object. */
	int			max_pairs;	/* Maximum # of window/gc pairs we can hold. */
	int			n_pairs;	/* Current # of window/gc pairs we are holding. */
	GCpair		pairs[1];	/* First element in registry. */
} I_GCinfo;

/*--------------------------------------------------------------------------*
 |"m_GCinfo_add"
 |
 |  Add a graphics context / window pair to the GC information registry.
 |
 |    = 1 on success, 0 if registry is full.
 *--------------------------------------------------------------------------*/
int m_GCinfo_add( p_me, owner, win, gc )
	I_GCinfo*	p_me;	/* -> context for this object. */
	int         owner;  /* =  entity registering this pair. */
	int			win;	/* =  window id for window to be registered. */
	int			gc;		/* =  gc id for graphics context to be registered. */
{
	if ( p_me->n_pairs >= p_me->max_pairs ) return 0;

	p_me->pairs[p_me->n_pairs].win = win;
	p_me->pairs[p_me->n_pairs].gc = gc;
	p_me->pairs[p_me->n_pairs].owner = owner;
	p_me->n_pairs++;
		
	return 1;
}

/*--------------------------------------------------------------------------*
 |"m_GCinfo_del_by_gc"
 |
 |  Delete graphics context / window pair(s) from the GC information 
 |  registry given the gc.
 |
 |    = number of pairs found for this gc
 *--------------------------------------------------------------------------*/
int m_GCinfo_del_by_gc( p_me, gc )
	I_GCinfo*	p_me;	/* -> context for this object. */
	int			gc;		/* =  gc id for pair(s) to be removed. */
{
	int		i_pair;		/* Window id/gc pair now examining. */
	int		i_shuffle;	/* Next location to move a wid/gc pair into. */
	int		n_found;	/* Number of pairs found for this gc. */

	for ( i_pair = 0, n_found = 0; i_pair < p_me->n_pairs; i_pair++ )
	{
		if ( p_me->pairs[i_pair].gc == gc ) 
		{
			p_me->n_pairs--;
			n_found++;
			for ( i_shuffle = i_pair; i_shuffle < p_me->n_pairs; i_shuffle++ )
			{
				p_me->pairs[i_shuffle] = p_me->pairs[i_shuffle+1];
			}
			i_pair--;
		}
	}
	return n_found;
}

/*--------------------------------------------------------------------------*
 |"m_GCinfo_del_by_win"
 |
 |  Delete graphics context / window pair(s) from the GC information 
 |  registry given the window.
 |
 |    = number of pairs found for this window
 *--------------------------------------------------------------------------*/
int m_GCinfo_del_by_win( p_me, win )
	I_GCinfo*	p_me;	/* -> context for this object. */
	int			win;	/* =  window id for pair(s) to be removed. */
{
	int		i_pair;		/* Window id/gc pair now examining. */
	int		i_shuffle;	/* Next location to move a wid/gc pair into. */
	int		n_found;	/* Number of pairs found for this gc. */

	for ( i_pair = 0, n_found = 0; i_pair < p_me->n_pairs; i_pair++ )
	{
		if ( p_me->pairs[i_pair].win == win ) 
		{
			p_me->n_pairs--;
			n_found++;
			for ( i_shuffle = i_pair; i_shuffle < p_me->n_pairs; i_shuffle++ )
			{
				p_me->pairs[i_shuffle] = p_me->pairs[i_shuffle+1];
			}
			i_pair--;
		}
	}
	return n_found;
}

/*--------------------------------------------------------------------------*
 |"m_GCinfo_find_by_gc"
 |
 |  Find the window associated with a graphics context in the GC information 
 |  registry.
 |
 |    = 1 on success, 0 on failure
 *--------------------------------------------------------------------------*/
int m_GCinfo_find_by_gc( p_me, gc, p_win, p_owner )
	I_GCinfo*	p_me;		/* -> context for this object. */
	int			gc;			/* =  graphics context id for pair to be found. */
	int*		p_win;		/* -> where to store window id, or 0. */
	int*		p_owner;	/* -> where to store owner of pair, or 0. */
{
	int		i_pair;		/* window id/gc pair now examining. */

	for ( i_pair = 0; i_pair < p_me->n_pairs; i_pair++ )
	{
		if ( p_me->pairs[i_pair].gc == gc ) 
		{
			if ( p_win ) *p_win =  p_me->pairs[i_pair].win;
			if ( p_owner ) *p_owner =  p_me->pairs[i_pair].owner;
			return 1;
		}
	}

	return 0;
}

/*--------------------------------------------------------------------------*
 |"m_GCinfo_find_by_win"
 |
 |  Find the graphics context associated with a window in the GC information 
 |  registry.
 |
 |    = 1 on success, 0 on failure
 *--------------------------------------------------------------------------*/
int m_GCinfo_find_by_win( p_me, win, p_gc, p_owner )
	I_GCinfo*	p_me;		/* -> context for this object. */
	int			win;		/* =  window id for pair to be found. */
	int*		p_gc;		/* -> where to store window id, or 0. */
	int*		p_owner;	/* -> where to store owner of pair, or 0. */
{
	int		i_pair;		/* window id/gc pair now examining. */

	for ( i_pair = 0; i_pair < p_me->n_pairs; i_pair++ )
	{
		if ( p_me->pairs[i_pair].win == win ) 
		{
			if ( p_gc ) *p_gc =  p_me->pairs[i_pair].gc;
			if ( p_owner ) *p_owner =  p_me->pairs[i_pair].owner;
			return 1;
		}
	}

	return 0;
}

/*--------------------------------------------------------------------------*
 |"m_GCinfo_terminate"
 |
 |  Free a GC information registry object.
 *--------------------------------------------------------------------------*/
void m_GCinfo_terminate( p_me )
	I_GCinfo*	p_me;	/* -> context for this object. */
{
	free((char*)p_me);
}

static TV_GCinfo O_GCinfo_methods = {
	m_GCinfo_add,
	m_GCinfo_del_by_gc,
	m_GCinfo_del_by_win,
	m_GCinfo_find_by_gc,
	m_GCinfo_find_by_win,
	m_GCinfo_terminate
};

/*--------------------------------------------------------------------------*
 |"O_GCinfo_O_GCinfo"
 |
 |  Find the graphics context associated with a window in the GC information 
 |  registry.
 |
 |    = GC id on success, -1 if not found
 *--------------------------------------------------------------------------*/
O_GCinfo* O_GCinfo_O_GCinfo( max_pairs )
	int		max_pairs;	/* = maximum number of pairs to register. */
{
	int			n_alloc;	/* Number of bytes to allocate for context. */
	I_GCinfo*	p_me;		/* Context for this object. */

	n_alloc = sizeof(I_GCinfo) + ( (max_pairs-1)*sizeof(GCpair) );
	if ( !(p_me = (I_GCinfo*)(malloc(n_alloc))) )
	{
		fprintf(stderr, "*** Cannot allocate O_GCinfo object with %n pairs\n",
			max_pairs);
		return 0;
	}
	p_me->max_pairs = max_pairs;
	p_me->n_pairs = 0;
	p_me->base.atv = &O_GCinfo_methods;
	return (O_GCinfo*)p_me;
}
