/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Header: /plx/source/Xclients/generic/RCS/TCWindow.c,v 1.2 1995/10/31 01:42:42 lovvik Exp $ */
/* $Log: TCWindow.c,v $
 * Revision 1.2  1995/10/31  01:42:42  lovvik
 * The call to XPlxGetVideoVisual was being passed the widget.  This call was changed to
 * pass in the screen number and the display.
 *
 * Revision 1.1  1995/10/10  06:42:36  drapeau
 * Initial revision
 * */

static char tcWindowRcsid[] = "$Id: TCWindow.c,v 1.2 1995/10/31 01:42:42 lovvik Exp $";

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include "TCWindoP.h"

#define offset(field) XtOffsetOf(TCWindowRec, tc.field)

static XtResource resources[] =					    /* {name, class, type, size, offset, ... */
{								    /* ...default_type, default_addr}, */
{
  XtNtrueColor,
  XtCTrueColor,
  XtRBoolean,
  sizeof(Boolean),
  offset(trueColor),
  XtRImmediate,
  (XtPointer) True
  },
{
  XtNdrawMethod,
  XtCDrawMethod,
  XtRPointer,
  sizeof(XtPointer),
  offset(drawMethod),
  XtRImmediate,
  (XtPointer) NULL
  },
};

#undef offset




static void	Realize(Widget, XtValueMask*, XSetWindowAttributes*);
static void	Redisplay(Widget, XEvent*, Region);
static Boolean	SetValues(Widget, Widget, Widget, ArgList, Cardinal*);

TCWindowClassRec tcWindowClassRec = {
  { /* core fields */
    /* superclass		*/	(WidgetClass) &widgetClassRec,
    /* class_name		*/	"TCWindow",
    /* widget_size		*/	sizeof(TCWindowRec),
    /* class_initialize		*/	NULL,
    /* class_part_initialize	*/	NULL,
    /* class_inited		*/	FALSE,
    /* initialize		*/	NULL,
    /* initialize_hook		*/	NULL,
    /* realize			*/	Realize,
    /* actions			*/	NULL,
    /* num_actions		*/	0,
    /* resources		*/	resources,
    /* num_resources		*/	XtNumber(resources),
    /* xrm_class		*/	NULLQUARK,
    /* compress_motion		*/	TRUE,
    /* compress_exposure	*/	TRUE,
    /* compress_enterleave	*/	TRUE,
    /* visible_interest		*/	FALSE,
    /* destroy			*/	NULL,
    /* resize			*/	NULL,
    /* expose			*/	Redisplay,
    /* set_values		*/	SetValues,
    /* set_values_hook		*/	NULL,
    /* set_values_almost	*/	XtInheritSetValuesAlmost,
    /* get_values_hook		*/	NULL,
    /* accept_focus		*/	NULL,
    /* version			*/	XtVersion,
    /* callback_private		*/	NULL,
    /* tm_table			*/	NULL,
    /* query_geometry		*/	XtInheritQueryGeometry,
    /* display_accelerator	*/	XtInheritDisplayAccelerator,
    /* extension		*/	NULL
  },
  { /* TCWindow fields */
    /* empty			*/	0
  }
};

WidgetClass tcWindowWidgetClass = (WidgetClass)&tcWindowClassRec;


static void TCWindowWidgetDebug(void)				    /* Used to make it easy to tell stupid debuggers what... */
{								    /* ...file to switch to for debugging. */
  return;
}



static void Realize(Widget			widget,
		    XtValueMask*		valueMask,
		    XSetWindowAttributes*	attributes)
{
  int			screenNumber;
  Display*		display = XtDisplay(widget);
  Colormap		colormap;
  XVisualInfo*		theVisual;
  TCWindowWidget	newWidget = (TCWindowWidget)widget;
  
  TCWindowWidgetDebug();					    /* Make it easy to set breakpoints in stupid debuggers */
  screenNumber = XScreenNumberOfScreen(XtScreen(widget));
  theVisual = (XVisualInfo*)XPlxGetVideoVisual(screenNumber,	    /* Try to get the best TrueColor visual for this widget */
					       display);
  widget->core.colormap = XCreateColormap(display,		    /* Create a default colormap to install for this window. */
					  XtWindow(XtParent(widget)), /* See O'Reilly Xlib Reference Manual entry for... */
					  theVisual->visual,	    /* ..."XCreateWindow()" to learn why a default colormap... */
					  AllocNone);		    /* ...needs to be created at all. */
  attributes->colormap = widget->core.colormap;			    /* Set necessary attributes for a 24-bit video window.  ... */
  attributes->background_pixmap = None;				    /* ...Absent from this set of attributes are the... */
  attributes->background_pixel = BlackPixel(display, screenNumber); /* ..."border_pixmap" and "border_pixel" attributes; I... */
  attributes->bit_gravity = CenterGravity;			    /* ...don't know why they can't seem to be set here.  ... */
  attributes->win_gravity = NorthWestGravity;			    /* ...Using the values recommended by the O'Reilly book... */
  attributes->backing_store = NotUseful;			    /* ...(Volume 1) of "None" and "BlackPixel(etc.)"... */
  attributes->backing_pixel = BlackPixel(display, screenNumber);    /* ...causes a "BadMatch" protocol error at run-time.  ... */
  attributes->save_under = False;				    /* ...However, these initial values seem to be sufficient. */
  *valueMask |= (CWColormap | CWBackPixmap | CWBackPixel |
		 CWBitGravity | CWWinGravity |
		 CWBackingStore | CWBackingPixel |
		 CWSaveUnder);
  widget->core.depth = theVisual->depth;			    /* Reset the widget's depth to the TrueColor depth */
  XtCreateWindow((Widget)widget, InputOutput, theVisual->visual,    /* Yes, create window into which video will be displayed */
		 *valueMask, attributes);
  if (TrueColor == theVisual->class)				    /* Did the widget actually obtain a TrueColor visual? */
  {
    newWidget->tc.trueColor = True;				    /* Yes, set the appropriate public widget resource */
  }
  else								    /* No, the widget didn't obtain a TrueColor visual */
  {
    newWidget->tc.trueColor = False;
  }
  return;
}								    /* end function Realize */



static void Redisplay(Widget	w,
		      XEvent*	event,
		      Region	region)
{
  TCWindowWidget	theWidget = (TCWindowWidget) w;
  
  if (theWidget->tc.drawMethod != (XtExposeProc)NULL)		    /* Did the app pgmr specify a custom drawing method? */
  {								    /* Yes, call the pgmr's drawing method to draw over video */
    (*theWidget->tc.drawMethod)(w, event, region);		    /* Perform the method call */
  }
  return;
} /* end function Redisplay */



static Boolean SetValues(Widget		current,
			 Widget		request,
			 Widget		new,
			 ArgList	args,
			 Cardinal*	numArgs)
{
  TCWindowWidget	currentWidget = (TCWindowWidget)current;
  TCWindowWidget	newWidget = (TCWindowWidget)new;
  
  if (newWidget->tc.trueColor != currentWidget->tc.trueColor)	    /* Did the programmer try to set this read-only resource? */
  {								    /* Yes, report the error and reset the resource */
    XtWarning("TrueColor Widget: the \"trueColor\" resource is not read-only.\n");
    newWidget->tc.trueColor = currentWidget->tc.trueColor;
  }
  return;
} /* end function SetValues */
