/*
 *   Copyright (c) 1990 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Murali V. Srinivasan.
 */

#ifndef lint
static char *sid_XVideowidget_c = "@(#)XVideo.c	1.12 	9/21/90 Parallax Graphics Inc";
#endif

/*
Copyright 1988 by the Massachusetts Institute of Technology

Permission to use, copy, modify, and distribute this
software and its documentation for any purpose and without
fee is hereby granted, provided that the above copyright
notice appear in all copies and that both that copyright
notice and this permission notice appear in supporting
documentation, and that the name of M.I.T. not be used in
advertising or publicity pertaining to distribution of the
software without specific, written prior permission.
M.I.T. makes no representations about the suitability of
this software for any purpose.  It is provided "as is"
without express or implied warranty.
*/

#include <stdio.h>
#ifdef SYSTYPES
#include <sys/types.h>
#endif
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#ifndef OPENLOOK
#ifdef NOTDEF
#include <X11/Xmu.h>
#endif
#endif
#include "XPlxExt.h"
#include "XVideoP.h"

#ifdef VideoView
#include <pixrect/pixrect.h>
#include <pixrect/pr_planegroups.h>
#include <pixrect/memvar.h>
#include <X11/cursorfont.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sun/fbio.h>
static int fb_fd;
static Pixrect *tmp_pix;
static Pixrect *old_pixs;
static Pixrect *fb_pr;
static void Mouseme();
static void LeaveMe();
#define ON 1
#define OFF 0
#endif

#ifndef HACK
/*
 * work around the 24/32 depth problem of the X-server,
 * needs to be taken out when the 24-bit-depth is implemented.
*/
static hack_depth = 0;
#endif

static Dimension videowidth = 640;
static Dimension videoheight = 482;

static Position sx = 0;
static Position sy = 0;

static int hue = 0x80 ;
static int contrast =  0xc0;
static int brightness =  0x60;
static int lgain = 0xb0;
static int cgain = 0xa0 ;
static int saturation =  0xc0;

static int true = 1;
static short strue = 1;
static int false = 0;
static short sfalse = 0;
static short defwtype = PLX_YUV7;
static short defitype = PLX_COMP;
static short defformat = PLX_NTSC;

static void  ConfigVideo();

plx_IO *XPlxQueryConfig();
plx_signal *XPlxQueryVideo();
GC defgc;

static void Redisplay();  
static void TopConfig();
static void vidconfig();

#ifdef VideoView
static void VisIt();
static void ExposeIt();
#endif

static char defaultTranslations[] =
     "<Configure>:		 configvideo()\n\
      <Expose>:                  exposevideo()";

static XtActionsRec videoactions[] = {
	{ "configvideo",	vidconfig,},
        { "exposevideo",        Redisplay,},
};


static XtResource resources[] = {
	{
		XtNsx, XtCSx, XtRPosition, sizeof(Position),
		XtOffset(VideoWidget,video.source.x),XtRPosition,(caddr_t)&sx,
	},
	{
		XtNsy, XtCSy, XtRPosition, sizeof(Position),
		XtOffset(VideoWidget,video.source.y),XtRPosition,(caddr_t)&sy,
	},
	{
		XtNswidth, XtCSwidth, XtRDimension, sizeof(Dimension),
                XtOffset(VideoWidget,video.source.width),
		XtRDimension, (caddr_t)&videowidth,
	},
        {
                XtNsheight, XtCSheight, XtRDimension, sizeof(Dimension),
                XtOffset(VideoWidget,video.source.height),
		XtRDimension,(caddr_t)&videoheight,
        },
        {
                XtNlive, XtCLive, XtRBoolean, sizeof (Boolean),
                XtOffset(VideoWidget,video.live), XtRInt, (caddr_t)&true,
        },
        {
                XtNscale, XtCScale, XtRBoolean, sizeof (Boolean),
                XtOffset(VideoWidget,video.scale), XtRInt, (caddr_t)&false,
        },
        {
                XtNsqueeze, XtCSqueeze, XtRBoolean, sizeof (Boolean),
                XtOffset(VideoWidget,video.squeeze), XtRInt, (caddr_t)&false,
        },
        {
                XtNaspect, XtCAspect, XtRBoolean, sizeof (Boolean),
                XtOffset(VideoWidget,video.aspect), XtRInt, (caddr_t)&false,
        },
        {
                XtNstillsave, XtCStillsave, XtRBoolean, sizeof (Boolean),
                XtOffset(VideoWidget,video.stillsave), XtRInt, (caddr_t)&true,
        },
        {
                XtNnewframe, XtCNewframe, XtRBoolean, sizeof (Boolean),
                XtOffset(VideoWidget,video.newframe), XtRInt, (caddr_t)&true,
        },
        {
                XtNzoom, XtCZoom, XtRBoolean, sizeof (Boolean),
                XtOffset(VideoWidget,video.zoom), XtRInt, (caddr_t)&false,
        },
        {
                XtNzoomWidget, XtCZoomWidget, XtRPointer, sizeof (Widget),
                XtOffset(VideoWidget,video.zoomwidget),
			XtRImmediate, (caddr_t)0,
        },
        {
                XtNconfigWidget, XtCConfigWidget, XtRPointer, sizeof (Widget),
                XtOffset(VideoWidget,video.configwidget),
			XtRImmediate, (caddr_t)0,
        },
	{
		XtNinputFormat, XtCInputFormat, XtRShort, sizeof (short),
		XtOffset (VideoWidget, video.inputformat), XtRShort, (caddr_t)&defformat,
	},
	{
		XtNoutputFormat, XtCOutputFormat, XtRShort, sizeof (short),
		XtOffset (VideoWidget, video.outputformat), XtRShort, (caddr_t)&defformat,
	},
	{
		XtNwindowType, XtCWindowType, XtRShort, sizeof (short),
		XtOffset (VideoWidget, video.windowtype), XtRShort, (caddr_t)&defwtype,
	},
	{
		XtNinputType, XtCInputType, XtRShort, sizeof (short),
		XtOffset (VideoWidget, video.inputtype), XtRShort, (caddr_t)&defitype,
	},
	{
		XtNoutputType, XtCOutputType, XtRShort, sizeof (short),
		XtOffset (VideoWidget, video.outputtype), XtRShort, (caddr_t)&defitype,
	},
	{
		XtNchannel, XtCChannel, XtRShort, sizeof (short),
		XtOffset (VideoWidget, video.channel), XtRShort, (caddr_t)&strue,
	},
	{
		XtNsubchannel, XtCSubchannel, XtRShort, sizeof (short),
		XtOffset (VideoWidget, video.subchannel), XtRShort, (caddr_t)&sfalse,
	},
	{
		XtNoutputchannel, XtCOutputChannel, XtRShort, sizeof (short),
		XtOffset (VideoWidget, video.outputchannel), XtRShort, (caddr_t)&sfalse,
	},
	{
		XtNsaturation, XtCSaturation, XtRInt, sizeof (int),
		XtOffset (VideoWidget, video.saturation), XtRInt,(caddr_t) &saturation,
	},
	{
		XtNcontrast, XtCContrast, XtRInt, sizeof(int),
		XtOffset(VideoWidget, video.contrast), XtRInt, (caddr_t)&contrast,
	},
	{
		XtNhue, XtCHue, XtRInt, sizeof(int),
		XtOffset(VideoWidget, video.hue), XtRInt, (caddr_t)&hue,
	},
	{
		XtNbrightness, XtCBrightness, XtRInt, sizeof(int),
		XtOffset(VideoWidget, video.brightness), XtRInt, (caddr_t) &brightness,
	},
        {
                XtNfactory, XtCFactory, XtRBoolean, sizeof (Boolean),
                XtOffset(VideoWidget,video.factory), XtRInt, (caddr_t)&false,
        },
        {
                XtNsaveuser0, XtCSaveuser0, XtRBoolean, sizeof (Boolean),
                XtOffset(VideoWidget,video.saveuser0), XtRInt, (caddr_t)&false,
        },
        {
                XtNloaduser0, XtCLoaduser0, XtRBoolean, sizeof (Boolean),
                XtOffset(VideoWidget,video.loaduser0), XtRInt, (caddr_t)&false,
        },
        {
                XtNsaveuser1, XtCSaveuser1, XtRBoolean, sizeof (Boolean),
                XtOffset(VideoWidget,video.saveuser1), XtRInt, (caddr_t)&false,
        },
        {
                XtNloaduser1, XtCLoaduser1, XtRBoolean, sizeof (Boolean),
                XtOffset(VideoWidget,video.loaduser1), XtRInt, (caddr_t)&false,
        },
        {
                XtNplx_IO, XtCPlx_IO, XtRPointer, sizeof(plx_IO *),
                XtOffset(VideoWidget,video.plxio), XtRImmediate, (caddr_t)0,
        },
        {
                XtNresizeParent, XtCResizeParent, XtRBoolean, sizeof (Boolean),
                XtOffset(VideoWidget,video.resizeparent),XtRInt,(caddr_t)&false,
        },
        {
                XtNredisplay, XtCRedisplay, XtRBoolean, sizeof (Boolean),
                XtOffset(VideoWidget,video.redisplay), XtRInt, (caddr_t)&false,
        },
        {
                XtNwindowGravity, XtCWindowGravity, XtRInt, sizeof (int),
                XtOffset(VideoWidget,video.window_gravity), XtRImmediate,
			(caddr_t)NorthWestGravity,
        },
	{
		XtNgc,XtCGc,XtRPointer,sizeof(GC *),
		XtOffset(VideoWidget,video.gc), XtRImmediate,
			(caddr_t)&defgc,
	},
        { 
                XtNcursor, XtCCursor, XtRPointer, sizeof (int *),
                XtOffset(VideoWidget,video.cursorpixmap), XtRImmediate,
                        (caddr_t)0,
        },
};



#define ROUNDUP16(x)    (((x)+15)&(~15))        /* round up */
#define ROUNDDN16(x)    (((x))&(~15))        	/* round up */
#define ISROUND16(x)    (((x)&0x000f) == 0)     /* already on a 16 boundry */

static 	void Initialize();
static	void Realize();
static 	void Destroy();
static 	void Resize();

static Boolean SetValues();
static XtGeometryResult QueryGeometry();

static Dimension vidx, vidy;

VideoClassRec videoClassRec = {
    { /* core fields */
    /* superclass		*/	&widgetClassRec,
    /* class_name		*/	"Video",
    /* widget_size		*/	sizeof(VideoRec),
    /* class_initialize		*/	NULL,
    /* class_part_initialize	*/	NULL,
    /* class_inited		*/	FALSE,
    /* initialize		*/	Initialize,
    /* initialize_hook		*/	NULL,
    /* realize			*/	Realize,
    /* actions                  */	videoactions,
    /* num_actions		*/	XtNumber(videoactions),
    /* resources		*/	resources,
    /* resource_count		*/	XtNumber(resources),
    /* xrm_class		*/	NULL,
    /* compress_motion		*/	TRUE,
    /* compress_exposure	*/	TRUE,
    /* compress_enterleave	*/	TRUE,
    /* visible_interest		*/	TRUE,
    /* destroy			*/	Destroy,
    /* resize			*/	Resize,
    /* expose			*/	NULL,
    /* set_values		*/	SetValues,
    /* set_values_hook		*/	NULL,
    /* set_values_almost	*/	XtInheritSetValuesAlmost,
    /* get_values_hook		*/	NULL,
    /* accept_focus		*/	NULL,
    /* version			*/	XtVersion,
    /* callback_private		*/	NULL,
    /* tm_table			*/	defaultTranslations,
    /* query_geometry		*/	(XtGeometryHandler) QueryGeometry,
    /* display_accelarator	*/	NULL,
    /* extension		*/	NULL
    },
    {
    /* dummy			*/	0
    }
};

WidgetClass videoWidgetClass = (WidgetClass) &videoClassRec;

/* ARGSUSED */
static void Initialize (request, new)
Widget request, new;
{
	VideoWidget w = (VideoWidget)new;
	register int bw = w->core.border_width;
	register Display *disp = XtDisplay(new);
	register int scr = (int) XDefaultScreen(disp);
	Window root = XRootWindow(disp, XDefaultScreen(disp));
	Window win;
	int standard;
        GC gc = XDefaultGC(disp, scr);

#ifdef VideoView
           fb_fd = open("/dev/tvone0", 2);
           if(fb_fd == -1)
            {
             printf("/dev/fb open error\n");
             exit(-1);
             }
            fb_pr = pr_open("/dev/tvone0",2);
            video_switch(ON);
            pixrect_init();
#endif

#ifndef HACK
/*
 * work around the 24/32 depth problem of the X-server,
 * needs to be taken out when the 24-bit-depth is implemented.
 */
	if (strncmp(XServerVendor(disp),"X11/NeWS",8) == 0)
	    hack_depth = 24;
	else
	    hack_depth = 32;
#endif

	vidx = ROUNDUP16 (bw) - bw;
	vidy = vidx;

/* Just to get at the QueryConfig */

	win =  XCreateSimpleWindow (disp, root, 0, 0, 100, 100, 1, 1, 0);
	XFlush(disp);
	w->video.plxio = XPlxQueryConfig(disp, win, gc);
	XFlush(disp);
	XPlxVideoValueQuery(disp, win, gc, &saturation, &contrast, &hue, &brightness);
	XFlush(disp);

	XDestroyWindow(disp, win);

	XFlush(disp);

	if(w->video.configwidget)
	{
	    w->video.conf = (Config *) malloc(sizeof(Config));
	    w->video.conf->windowtype = w->video.windowtype;
	    w->video.conf->plxio = w->video.plxio;
	    XtAddEventHandler(w->video.configwidget, StructureNotifyMask, False,
			  TopConfig, w->video.conf);
	}
	else
	    w->video.conf = NULL;

#ifdef VideoView
	XtAddEventHandler(new, VisibilityChangeMask, False,
			  VisIt, NULL);
	XtAddEventHandler(new, ExposureMask, False,
			  ExposeIt, NULL);
       XtAddEventHandler(new,LeaveWindowMask,FALSE,LeaveMe,0);
        XtAddEventHandler(new,PointerMotionMask,FALSE,Mouseme,0);
#endif
	
	w->video.videowidth = w->video.plxio->inputs->signal_list->signal->w;

        w->video.videoheight = (w->video.videowidth * 3) / 4;
        w->video.source.y =
          w->video.plxio->inputs->signal_list->signal->h - w->video.videoheight;

	w->core.width = w->video.videowidth + (2 * vidx) ;
	w->core.height = w->video.videoheight + (2 * vidy) ;
	w->video.source.width = w->video.videowidth;
	w->video.source.height = w->video.videoheight;
	w->video.gc = XDefaultGC(disp, scr);

	w->video.channel = w->video.plxio->inputs->channel;
	w->video.inputtype = w->video.plxio->inputs->format_list->format;
	w->video.inputformat = w->video.plxio->inputs->signal_list->signal->standard;
	if(w->video.plxio->outputs)
	{
	    w->video.outputchannel = w->video.plxio->outputs->channel;
	    w->video.outputtype = w->video.plxio->outputs->format_list->format;
	    w->video.inputformat = w->video.plxio->outputs->signal_list->signal->standard;
	}

	w->video.saturation = saturation;
	w->video.hue = hue;
	w->video.brightness = brightness;
	w->video.contrast = contrast;
}

static void Realize (gw, valueMask, attrs)
Widget gw;
XtValueMask *valueMask;
XSetWindowAttributes *attrs;
{
	int bw = gw->core.border_width;
	int width;
	int height;
        VideoWidget w = (VideoWidget) gw;
	Display *disp = XtDisplay(w);
	Window win;
	GC gc;
	XGCValues gc_val;
	int screen;
        Cursor curs;

	int scr = DefaultScreen(disp);
	int depth = DefaultDepth(disp, scr);

	width =  w->video.videowidth + (2 * bw);
	height = w->video.videoheight + (2 * bw);

	w->core.width = width;
	w->core.height = height;
	screen = DefaultScreen(disp);

#ifdef NOTDEF
	attrs->backing_store = NotUseful;
	attrs->save_under = False;
	*valueMask |= (CWBackingStore | CWSaveUnder);
#endif
	if(w->video.plxio->hardware == PLX_XVIDEO_24)
	{
	    XVisualInfo vinfo, *vinfos;
	    Visual *visual = NULL;
	    Colormap colormap;
	    int j;

	    vinfo.screen = screen;
#ifndef HACK
/*
 * work around the 24/32 depth problem of the X-server,
 * needs to be taken out when the 24-bit-depth is implemented.
 */
	    vinfo.depth = hack_depth;
#endif
	    vinfo.class = TrueColor;
/*
	    vinfo.red_mask = 0xff;
	    vinfo.green_mask = 0xff00;
	    vinfo.blue_mask = 0xff0000;
	    vinfo.colormap_size = 256;
	    vinfo.bits_per_rgb = 8;
*/

	    vinfos = (XVisualInfo *)
	      XGetVisualInfo(disp,
			     VisualScreenMask | VisualClassMask,
			     &vinfo, &j);
	    if(j == 0)
	    {
		fprintf(stderr, "Unable to get TrueColor bit window\n");
		goto normw;
	    }
	    visual = vinfos[0].visual;
	    colormap = XCreateColormap(disp, RootWindow(disp, screen), visual,
					AllocNone);
	    attrs->colormap = colormap;
	    attrs->background_pixel = BlackPixel(disp, screen);
	    attrs->border_pixel = WhitePixel(disp, screen);
	    *valueMask |= (CWBorderPixel | CWBackPixel | CWColormap);
#ifndef HACK
/*
 * work around the 24/32 depth problem of the X-server,
 * needs to be taken out when the 24-bit-depth is implemented.
 */
	    depth = w->core.depth = hack_depth;
#endif
	    printf("Setting the depth of the window to %d bits, overriding hack_depth.\n",
		   vinfos[0].depth);
	    depth = vinfos[0].depth;
	    w->core.depth = vinfos[0].depth;
	    
	    XtCreateWindow( w, InputOutput, visual,
			*valueMask, attrs);
	    
/*
	    w->core.window = XCreateWindow(disp, XtWindow(XtParent(w)),
			w->core.x, w->core.y,
			w->core.width, w->core.height,
			w->core.border_width, 32, InputOutput, visual,
			*valueMask, attrs);
*/
	}
	else
	{
normw:	    XtCreateWindow( w, InputOutput, (Visual *)CopyFromParent,
			*valueMask, attrs);
	}

	win = XtWindow(w);

	if(w->video.conf)
	{
	    w->video.conf->window = win;
	}

	gc_val.foreground = (unsigned long) BlackPixel (disp, screen);
	gc_val.background = w->core.background_pixel;
        gc_val.plane_mask = -1;

	w->video.gc = XCreateGC (disp, win, GCPlaneMask|GCForeground|GCBackground, &gc_val);

	gc = w->video.gc;

	w->core.width = width;
	w->core.height = height;

        if(w->video.stillsave && (w->video.plxio->hardware != PLX_VIDEOVIEW))
#ifdef NOTDEF
	if(w->video.stillsave && 
           (w->video.plxio->hardware == PLX_VIPER || 
            w->video.plxio->hardware == PLX_1280))
#endif
	{
	    w->video.pixmap = XCreatePixmap (disp, win,
			w->video.source.width, w->video.source.height, depth);

	    XPlxVideoTag(disp, w->video.pixmap, gc, PLX_VIDEO);
	}
	else
	   w->video.pixmap = NULL;
	XPlxVideoTag(disp, win, gc, PLX_VIDEO);
	XVideoChannel(w);
#ifdef VideoView
          curs = XCreateFontCursor(XtDisplay(w),XC_left_ptr);
          XDefineCursor(XtDisplay(w),XtWindow(w),curs);
#endif

}

static void Destroy (gw)
Widget gw;
{
        VideoWidget w = (VideoWidget) gw;
#ifdef VideoView
	if(w->video.plxio->hardware == PLX_VIDEOVIEW)
	    plxvvmapout(XtDisplay(gw), XtWindow(gw)); 
#endif
}

static void 
Redisplay (gw, event, region)
Widget gw;
XEvent *event;
Region region;
{
	VideoWidget w = (VideoWidget) gw;
	register Display *disp = XtDisplay(gw);
	register Window win = XtWindow(gw);
	register int width = (int) gw->core.width - (gw->core.border_width * 2);
	register int height =(int)gw->core.height - (gw->core.border_width * 2);

	register int x = w->core.border_width;
	register int y = w->core.border_width;

	register XExposeEvent *xev = (XExposeEvent *) event;
	GC gc = w->video.gc;

	if(w->video.plxio->hardware == PLX_VIDEOVIEW)
	{
	    if(w->video.zoom == 0)
	    {
		if(width > w->video.videowidth)
		    width = w->video.videowidth;
		if(height > w->video.videoheight)
		    height = w->video.videoheight;
	    }
	}
	else
	{
	    if(w->video.scale == 0)
	    {
		if(width > w->video.videowidth)
		    width = w->video.videowidth;
		if(height > w->video.videoheight)
		    height = w->video.videoheight;
	    }
	}

	if (w->video.live)		/* Live */
	{
		if (w->video.squeeze)
		{
			XPlxVideoSqueezeLive (disp, win, gc,
				w->video.source.x, w->video.source.y,
				w->video.source.width, w->video.source.height,
				x, y, width, height);
		}
		else
		{
			XPlxVideoLive(disp, win, gc,
				w->video.source.x, w->video.source.y,
				x, y, width, height);
		}
	}
	else
	{
          if ((!w->video.stillsave) ||
                ((w->video.plxio->hardware == PLX_XVIDEO_24) &&
                 (w->video.newframe)))
	   {
#ifdef NOTDEF
		if (w->video.scale)
		{
			XPlxVideoScale (disp, win, gc,
				w->video.source.x, w->video.source.y,
				w->video.source.width, w->video.source.height,
				x, y, width, height);

		}
		else
#endif
		{
			if (w->video.squeeze)
			{
				XPlxVideoSqueezeStill (disp, win, gc, 
					w->video.source.x, w->video.source.y,
					w->video.source.width, w->video.source.height,
					x, y, width, height);

			}
			else 	/* Both Not Scale and Not Squeeze */
			{
				XPlxVideoStill(disp, win, gc,
					w->video.source.x, w->video.source.y,
					x, y, width, height);
			}
		}	
	   }
/*	   else 		/* Still needs to stored */
           if (w->video.stillsave)
	   {
		XPlxVideoStop(disp, win, gc);
		if(w->video.pixmap)
		{
		    if(w->video.newframe)
		    {
                     if(w->video.plxio->hardware != PLX_XVIDEO_24)
                      {
 			XPlxVideoStill (disp, w->video.pixmap, gc,
			w->video.source.x, w->video.source.y,
			0, 0, w->video.source.width, w->video.source.height);
		    }
		    else
		    {
                       XCopyArea (disp, win, w->video.pixmap, gc,
                                x, y, width, height, 0, 0);
                        goto nocop;
                      }
	            }
		    XFlush(disp);

                    if ((w->video.scale || w->video.squeeze) &&
                         (w->video.plxio->hardware != PLX_XVIDEO_24))
	 	    {
			XPlxScaleCopyArea (disp, w->video.pixmap, win, gc,
				0, 0, w->video.source.width, w->video.source.height,
				x, y, width, height);
		    }
		    else
		    {
			XCopyArea (disp, w->video.pixmap,  win, gc,
				0, 0, w->video.source.width, w->video.source.height,
				x, y);
		    }
        	}
	    }
	}
nocop:	XFlush(disp);
	w->video.newframe = 0;
}

/* ARGSUSED */
static Boolean SetValues(gcurrent, request, gnew)
Widget gcurrent, request, gnew;
{
	VideoWidget current = (VideoWidget) gcurrent;
	VideoWidget new = (VideoWidget) gnew;
	Boolean redisplay = FALSE;
	Display *disp = XtDisplay(new);
	
/* things do not need redisplay 

	controls -- call a separate procedure.
	channels -- call a separate procedure.
	zoom -- call a separate procedure.
	factory - call a separate procedure.
	save and load users - call a separate procedure.
**/
	
	if (!XtIsRealized(gnew)) 	/* Redisplay will take care of this */
		return(0);

	if (new->video.newframe)
	{
		redisplay = TRUE;
	}

	if (new->video.live != current->video.live)
	{
		redisplay = TRUE;
	}
	
	if (new->video.scale != current->video.scale)
	{
		new->video.newframe = 1;
		redisplay = TRUE;	
	}

	if (new->video.squeeze != current->video.squeeze)
	{
		redisplay = TRUE;
	}
	
	if (new->video.zoom != current->video.zoom)
	{
		XVideoZoom(new);
	}

	if ((new->video.saturation != current->video.saturation) ||
	    (new->video.brightness != current->video.brightness) ||
	    (new->video.hue != current->video.hue) ||
	    (new->video.contrast != current->video.contrast))
	{
		XVideoControls(new);
	}

	if ((new->video.channel != current->video.channel) ||
	   (new->video.subchannel != current->video.subchannel) ||
	   (new->video.inputtype != current->video.inputtype) ||
	   (new->video.windowtype != current->video.windowtype) ||
	   (new->video.inputformat != current->video.inputformat))
	{
		XVideoChannel(new);
		if ( new->video.redisplay)
		redisplay = TRUE;
	}
	if ((new->video.outputchannel != current->video.outputchannel) ||
	   (new->video.outputtype != current->video.outputtype) ||
	   (new->video.outputformat != current->video.outputformat))
	{
		XVideoOutputChannel(new);
	}
	if ((new->video.source.x != current->video.source.x) ||
		(new->video.source.y != current->video.source.y) ||
		(new->video.source.width != current->video.source.width) ||
		(new->video.source.height != current->video.source.height))
	{
		new->video.videowidth = new->video.source.width;
		new->video.videoheight = new->video.source.height;
		redisplay = TRUE;
	}
	if (new->video.factory)
	{
		new->video.factory = 0;
		XVideoFactory(new);
	}
	
	if (new->video.saveuser0)
	{
		new->video.saveuser0 = 0;
		XVideoSaveuser(new, USER_0);
	}
	if (new->video.saveuser1)
	{
		new->video.saveuser1 = 0;
		XVideoSaveuser(new, USER_1);
	}
	
	if (new->video.loaduser0)
	{
		new->video.loaduser0 = 0;
		XVideoLoaduser(new, USER_0);
	}
	
	if (new->video.loaduser1)
	{
		new->video.loaduser1 = 0;
		XVideoLoaduser(new, USER_1);
	}
	
	if (new->video.redisplay)
	{
		new->video.redisplay = 0;
		redisplay = TRUE;
	}

	if (redisplay)
	{
		redisplay = 0;
		Redisplay(new, 0, 0);
		XFlush(disp);
	}
	return (redisplay);
}

/* ARGSUSED */
static XtGeometryResult
QueryGeometry(w, request, ret)
Widget w;
XtWidgetGeometry *request, *ret;
{
	
	register XtGeometryMask mask = request->request_mode;
	short changed = 0;


	ret->request_mode = 0;

	if (mask & CWX)
	{
		if (!ISROUND16(request->x))
		{
			ret->x = ROUNDUP16(request->x);
			ret->request_mode |= CWX;
			changed = 1;
		}
	}
	return ((changed)? XtGeometryAlmost: XtGeometryYes);
}

static XVideoControls(w)
VideoWidget w;
{
	Display *disp = XtDisplay (w);
	Window win = XtWindow(w);
	
	XPlxVideoValueLoad(disp, win, w->video.gc,
		(int) w->video.saturation,
		(int) w->video.contrast,
		(int) w->video.hue,
		(int) w->video.brightness);
	XFlush(disp);
}

static XVideoZoom(new)
VideoWidget new;
{
	Display *disp = XtDisplay (new);
	Window w;
	if(new->video.zoomwidget)
	  w = XtWindow(new->video.zoomwidget);
	else
	  w = XtWindow(new);
	
	if (new->video.zoom)
	{
		XPlxZoomOn(disp, w, new->video.gc,
			0, 0, new->core.width, new->core.height);
	}
	else
		XPlxZoomOff(disp, w, new->video.gc);

	XFlush(disp);
}

static XVideoChannel(new)
VideoWidget new;
{
	Display *disp = XtDisplay (new);
	Window win = XtWindow (new);
	plx_signal *sig;

	if(new->video.conf)
		new->video.conf->windowtype = new->video.windowtype;

	XPlxVideoInputSelect(disp, win, new->video.gc, new->video.channel, 
			new->video.inputformat, new->video.inputtype,
			new->video.windowtype);

/*
	sig = (plx_signal *) XPlxQueryVideo(disp, win, new->video.gc);
*/

	if (new->video.pixmap && new->video.stillsave)
	{
	    XPlxVideoInputSelect(disp, new->video.pixmap , new->video.gc,
				new->video.channel, new->video.inputformat,
				new->video.inputtype, new->video.windowtype);
	    new->video.newframe = 1;
	}
			
	XFlush(disp);
}
			
static XVideoOutputChannel(new)
VideoWidget new;
{
	Display *disp = XtDisplay (new);
	Window win = XtWindow (new);

	XPlxVideoOutputSelect(disp, win, new->video.gc,
			new->video.outputchannel, new->video.outputformat,
			new->video.outputtype, PLX_RGB24);
	XFlush(disp);
}
			
static XVideoFactory(new)
VideoWidget new;
{
	Display *disp = XtDisplay (new);
	Window win = XtWindow (new);
	GC gc = new->video.gc;


	XPlxVideoValueRecall(disp, win, gc, FACTORY);
	
	XPlxVideoValueQuery(disp, win, gc,
			    &saturation, &contrast, &hue, &brightness);
	XFlush(disp);
	new->video.brightness = brightness;
	new->video.saturation = saturation;
	new->video.hue = hue;
	new->video.contrast = contrast;
}

static XVideoSaveuser(new, user)
VideoWidget new;
int user;
{
	
	Display *disp = XtDisplay (new);
	Window win = XtWindow (new);
	GC gc = new->video.gc;

	XPlxVideoValueSave(disp, win, gc, user);
	XFlush(disp);
}


static XVideoLoaduser(new, user)
VideoWidget new;
int user;
{
	
	Display *disp = XtDisplay (new);
	Window win = XtWindow (new);
	GC gc = new->video.gc;

	XPlxVideoValueRecall(disp, win, gc, user);
	XFlush(disp);

	
	XPlxVideoValueQuery(disp, win, gc,
			    &saturation, &contrast, &hue, &brightness);
	XFlush(disp);
	
	new->video.brightness = brightness;
	new->video.saturation = saturation;
	new->video.hue = hue;
	new->video.contrast = contrast;
}


static void get_position(dpy, win, x, y, bw)
Display *dpy;
Window win;
int *x, *y, *bw;
{
	Window root, parent, current, *children;
	int tx, ty, i, chan = 0;
	XWindowAttributes attrib;

	XGetWindowAttributes(dpy, win, &attrib);
	*bw = attrib.border_width;
	tx = attrib.x + attrib.border_width;
	ty = attrib.y + attrib.border_width;
	current = win;
	do
	{
	    XQueryTree(dpy, current, &root, &parent, &children, &i);
	    XGetWindowAttributes(dpy, parent, &attrib);
	    tx += (attrib.x + attrib.border_width);
	    ty += (attrib.y + attrib.border_width);
	    current = parent;
	} while (current != root);
	*x = tx;
	*y = ty;
}

static void get_size(dpy, win, w, h, bw)
Display *dpy;
Window win;
int *w, *h, *bw;
{
	XWindowAttributes attrib;

	XGetWindowAttributes(dpy, win, &attrib);
	*w = attrib.width;
	*h = attrib.height;
	*bw = attrib.border_width;
}

static int factable[16] =
{
	1, 2, 4, 4, 8, 8, 8, 8,
	16, 16, 16, 16, 16, 16, 16, 16
};

static int mapped = 0;

static void Resize(gw)
Widget gw;
{
	VideoWidget w = (VideoWidget) gw;
	int fact, facty, neww, newh, granular, bw, mw, mh;
	static int inhere = 0;
	XtGeometryResult xtgr;
	Arg args[10];
	unsigned int n;
	int tw, th, tbw;

	if(inhere) return;

	if((w->video.oldw == w->core.width) &&
	   (w->video.oldh == w->core.height))
		return;

	inhere = 1;

	bw = w->core.border_width * 2;
	mw = w->video.videowidth + bw;
	mh = w->video.videoheight + bw;

	if(w->video.plxio->hardware == PLX_VIDEOVIEW)
	{
	    w->video.oldw = w->core.width;
	    w->video.oldh = w->core.height;
	    if (w->video.zoom || (w->video.squeeze == 0) ||
		(w->video.aspect == 0))
	    {
		if(w->video.zoom == 0)
		{
		    if(w->video.oldw > mw)
			w->video.oldw = mw;
		    if(w->video.oldh > mh)
			w->video.oldh = mh;
		}
	    }
	    else
	    {
	      fact = 640 / ((w->video.oldw - bw) + 1);
	      if(fact > 16) fact = 16;
	      facty = 480 / ((w->video.oldh - bw) + 1);
	      if(facty > 16) facty = 16;
	      if(facty > fact) fact = facty;
	      fact = factable[fact];
	      w->video.oldw = (640 / fact) + bw;
	      w->video.oldh = ((480 + (fact - 1)) / fact) + bw;
	    }
	}
	else if((w->video.plxio->hardware == PLX_XVIDEO_24) ||
		(w->video.plxio->hardware == PLX_XVIDEO_8))
	{
	    tw = w->core.width - bw;
	    th = w->core.height - bw;
	    w->video.oldw = w->core.width;
	    w->video.oldh = w->core.height;
rfact:	    if(w->video.aspect)
	    {
		if((fact = ((tw * 3) / 4)) <= th)
		    w->video.oldh = fact + bw;
		else
		    w->video.oldw = ((th * 4) / 3) + bw;
	    }
	    if(w->video.oldw > mw)
	    {
		w->video.oldw = mw - bw;
		tw = mw - bw;  
		goto rfact;
	    }
	    if(w->video.oldh > mh)
	    {
		w->video.oldh = mh - bw;
		th = mh - bw; 
		goto rfact;
	    }
	}
	else
	{
	    if (w->video.windowtype == PLX_GRAY7)
		granular = 0;
	    else
		granular = 15;

	    tw = (w->core.width - bw) & ~granular;
	    th = w->core.height - bw;
	    w->video.oldw = tw + bw;
	    w->video.oldh = w->core.height;
	    if(w->video.aspect)
	    {
refact:		if((fact = ((tw * 3) / 4)) <= th)
		    w->video.oldh = fact + bw;
		else
		{
		    w->video.oldw = (((th * 4) / 3) & ~granular) + bw;
		    tw = w->video.oldw - bw;
		    th = w->video.oldh - bw;
		    goto refact;
		}
	    }
	    if(w->video.scale == 0)
	    {
		if(w->video.oldw > mw)
			w->video.oldw = mw;
		if(w->video.oldh > mh)
			w->video.oldh = mh;
	    }
	}

	xtgr=XtMakeResizeRequest(w, w->video.oldw, w->video.oldh, &neww, &newh);

	if((xtgr == XtGeometryYes) || (xtgr == XtGeometryDone))
	{
possmove:   if(w->core.x || w->core.y)
		XtMoveWidget(w, 0, 0);
allok:	    w->core.width = w->video.oldw;
	    w->core.height = w->video.oldh;
allok1:	    inhere = 0;
	    if(!mapped) return;
	    Redisplay(w, 0, 0);
	    return;
	}

	if(w->video.resizeparent)
	{
	    get_size(XtDisplay(XtParent(w)), XtWindow(XtParent(w)),
			 &tw, &th, &tbw);

	    tw += (w->video.oldw - tw);
	    th += (w->video.oldh - th);

	    XtResizeWidget(XtParent(w), tw, th, tbw);

	    xtgr=XtMakeResizeRequest(w, w->video.oldw, w->video.oldh,
					 &neww, &newh);

	    if((xtgr == XtGeometryYes) || (xtgr == XtGeometryDone))
		    goto possmove;
	}

	if((w->core.width <= w->video.oldw)&&(w->core.height <= w->video.oldh))
	{
	    w->video.oldw = w->core.width;
	    w->video.oldh = w->core.height;
	    goto allok1;
	}
	XResizeWindow(XtDisplay(w), XtWindow(w), w->video.oldw, w->video.oldh);
	switch(w->video.window_gravity)
	{
	    case NorthGravity:
		tw = (w->core.width - w->video.oldw) / 2;
		th = 0;
		break;
	    case NorthEastGravity:
		tw = (w->core.width - w->video.oldw);
		th = 0;
		break;
	    case WestGravity:
		tw = 0;
		th = (w->core.height - w->video.oldh) / 2;
		break;
	    case CenterGravity:
		tw = (w->core.width - w->video.oldw) / 2;
		th = (w->core.height - w->video.oldh) / 2;
		break;
	    case EastGravity:
		tw = (w->core.width - w->video.oldw);
		th = (w->core.height - w->video.oldh) / 2;
		break;
	    case SouthWestGravity:
		tw = 0;
		th = (w->core.height - w->video.oldh);
		break;
	    case SouthGravity:
		tw = (w->core.width - w->video.oldw) / 2;
		th = (w->core.height - w->video.oldh);
		break;
	    case SouthEastGravity:
		tw = (w->core.width - w->video.oldw);
		th = (w->core.height - w->video.oldh);
		break;
	    default:
		tw = 0;
		th = 0;
		break;
	}
	if((tw != w->core.x) || (th != w->core.y))
	    XtMoveWidget(w, tw, th);
	goto allok;

/* NOT REACHED */

}

static void TopConfig(widget, conf, event)
Widget widget;
Config *conf;
XEvent *event;
{

	Arg args[25];
	unsigned int n;

	int i,j;
	int newx, newy, bw;
	static short redisplay = 0;
	static int oldx = -1, oldy = -1;
	int offx, offy;
	int offset_x, offset_y;
	Display *disp = XtDisplay(widget);
	XConfigureEvent *xc = (XConfigureEvent *) event;
	static XExposeEvent eevent;

	if(event->type == UnmapNotify)
	{
#ifdef VideoView
	    if(mapped)
	        plxvvmapout(disp, conf->window);
#endif
	    mapped = 0;
	    return;
	}
	if(event->type == MapNotify)
	{
	    mapped = 1;
	    return;
	}

	if(event->type != ConfigureNotify)
	    return;

/*
	if((conf->plxio->hardware == PLX_XVIDEO_24) ||
	   (conf->plxio->hardware == PLX_XVIDEO_8))
	    return;

	if(((conf->plxio->hardware == PLX_VIPER) ||
	   (conf->plxio->hardware == PLX_1280)) &&
	   (conf->windowtype == PLX_GRAY7)) return;
*/

	get_position(disp, XtWindow(widget), &offset_x, &offset_y, &bw);
	get_position(disp, conf->window, &newx, &newy, &bw);
	offset_x = newx - offset_x;
	offset_y = newy - offset_y;

	if (redisplay)
	{
	    if((redisplay == 1) &&
	       ((conf->plxio->hardware == PLX_VIPER) ||
		(conf->plxio->hardware == PLX_1280)) &&
	       ((oldx != newx) || (oldy != newy)))
	    {
		++redisplay;
		offx = oldx - newx;
		offy = oldy - newy;
	        XtMoveWidget(widget, (oldx + offx) - offset_x,
			     (oldy + offy) - offset_y);
		return;
	    }
redisp:	    eevent.type = Expose;
	    eevent.display = disp;
	    eevent.window = conf->window;
	    eevent.x = 0;
	    eevent.y = 0;
	    eevent.width = 10;
	    eevent.height = 10;
	    eevent.count = 0;
	    XSendEvent(disp, conf->window, False, ExposureMask, &eevent);
	    redisplay = 0;
	    return;
	}
	if ((oldx != newx) || (oldy != newy))
	{
	    if((conf->plxio->hardware != PLX_VIPER) &&
		(conf->plxio->hardware != PLX_1280))
	    {
		oldx = newx;
		oldy = newy;
		goto redisp;
	    }
	    else
	    {
		redisplay = 1;
		oldx = ((newx + bw) & ~15) - bw;
		oldy = newy;
		XtMoveWidget(widget, oldx - offset_x, oldy - offset_y);
	    }
	}
}

static void vidconfig(widget, event, data)
VideoWidget widget;
XEvent *event;
caddr_t data;
{
	if(!mapped) return;
	TopConfig(widget->video.configwidget, widget->video.conf, event);
}

#ifdef VideoView

static void VisIt(w, data, event)
Widget w;
caddr_t data;
XVisibilityEvent *event;
{
	plxvvvisnote(XtDisplay(w), XtWindow(w),
		     (event->state == VisibilityFullyObscured) ? 1 : 0);
}

static void ExposeIt(w, data, event)
Widget w;
caddr_t data;
XExposeEvent *event;
{
	XRectangle rect;

	rect.x = event->x;
	rect.y = event->y;
	rect.width = event->width;
	rect.height = event->height;
	plxvvexpose(XtDisplay(w), XtWindow(w), &rect, event->count);
}

#define VXSIZE 640
#define VYSIZE 480
#define CURXSIZE 16
#define CURYSIZE 16
 
static Pixrect *cursor_pixrect;
static int init_done = 0;
static int inited = 0;
static int p_x,p_y;
static struct pr_pos pos;
static  char* white[VXSIZE*VYSIZE/8]={0};
static  unsigned char cursor_data[CURXSIZE*CURYSIZE/8]={
   0x0f, 0xff, /* 0000111111 */
   0x07, 0xff, /* 0000011111 */
   0x03, 0xff, /* 0000001111 */
   0x01, 0xff, /* 0000000111 */
   0x00, 0xff, /* 0000000011 */
   0x00, 0x7f, /* 0000000001 */
   0x00, 0x3f, /* 0000000000 */
   0x00, 0x3f, /* 0000000000 */
   0x00, 0x3f, /* 0000000000 */
   0x00, 0x3f, /* 0000000000 */
   0x00, 0xff, /* 0000000011 */
   0x00, 0xff, /* 0000000011 */
   0x30, 0x7f, /* 0011000001 */
   0xf0, 0x7f, /* 1111000001 */
   0xf8, 0x3f, /* 1111100000 */
   0xf8, 0x3f, /* 1111100000 */
};
 
static void Mouseme(w, data, event)
Widget w;
caddr_t data;
XEvent *event;
{
XMotionEvent *em;
 
          XtNextEvent(event);
          if(event->type == MotionNotify)
            {
               em = (XMotionEvent *)event;
               if ( init_done == 0)
                 { p_x = em->x_root;
                   p_y = em->y_root;
                   init_done = 1;
                 }
               tvone_WarpMouse(p_x, p_y, em->x_root, em->y_root);
               p_x = em->x_root;
               p_y = em->y_root;
             }
/*        if(event->type == LeaveNotify)
             {
                tvone_ClearMouse(p_x, p_y);
                init_done = 0;
              }
*/
      (void) XtDispatchEvent(event);
}
static void LeaveMe(w, data, event)
Widget w;
caddr_t data;
XEvent *event;
{
XMotionEvent *em;
        if(event->type == LeaveNotify)
            {
                tvone_ClearMouse(p_x, p_y);
                init_done = 0;
              }
 
}
 
tvone_WarpMouse(p_x, p_y, x, y)
int x, y;
{
 
        if (inited == 0)
        {
            inited = 1;
            tmp_pix = mem_create(CURXSIZE,CURYSIZE,1);
            old_pixs= mem_create(CURXSIZE, CURYSIZE, 1);
        }
        else
        {
/*      write back old cursor pixels    */
 
            pr_rop(fb_pr,p_x-3,p_y-1,CURXSIZE,CURYSIZE,PIX_SRC,old_pixs,0,0);
        }
/*      save current cursor pixels      */
 
        pr_rop(old_pixs,0,0,CURXSIZE,CURYSIZE,PIX_SRC,fb_pr,x-3,y-1);
 
/*      write cursor    */
        pr_rop(tmp_pix,0,0,CURXSIZE,CURYSIZE,PIX_SRC,old_pixs,0,0);
        pr_rop(tmp_pix,0,0,CURXSIZE,CURYSIZE,PIX_SRC&PIX_DST,cursor_pixrect,0,0);
 
        pr_rop(fb_pr, x-3, y-1, CURXSIZE, CURYSIZE, PIX_SRC, tmp_pix, 0, 0);
}
tvone_ClearMouse(p_x, p_y)
{
        pr_rop(fb_pr,p_x-3,p_y-1,CURXSIZE,CURYSIZE,PIX_SRC,old_pixs,0,0);
        inited = 0;
}
video_switch(off_on)
{
   int ret;
 
        pr_set_plane_group(fb_pr, PIXPG_VIDEO_ENABLE);
 
        if(off_on){
        pr_rop(fb_pr, pos.x, pos.y+16, VXSIZE, VYSIZE, PIX_SET|PIX_COLOR(1), NULL, 0, 0);
        } else {
        pr_rop(fb_pr, pos.x, pos.y+16, VXSIZE, VYSIZE, PIX_CLR|PIX_COLOR(1), NULL, 0, 0);
        }
}
 
pixrect_init(){
        cursor_pixrect = mem_create(CURXSIZE, CURYSIZE, 1);
        (unsigned char *)(mpr_d(cursor_pixrect)->md_image) = cursor_data;
}
 

#endif
