/****************************************************************************
 *
 *	VideoIn.c - Define XView interface for Parallax Graphics XLIB extensions
 *
 *	(C) Copyright 1991 by Paradise Software, Inc.
 *
 *	Written by John Malleo-Roach
 *
 *	Version %G% - %I%
 *
 ****************************************************************************/

/* #define TRACE		/* Print out informative messages during execution			*/
#define SHARE_CHANNELS	/* Enable multiple video canvases to share the same channel	*/
#define ALTERNATE		/* Use the parallax alternate way of getting video params   */

/* INCLUDE FILES */

#include	<string.h>
#include	<sys/types.h>
#include	<X11/Xlib.h>
#include	<X11/Xutil.h>
#include	"XPlxExt.h"
#include	"VideoImpl.h"
#include <xpm.h>

/* CONSTANTS */

#define CHAN_INUSE			0xff00

#define DEFAULT_CHANNEL		PLX_INPUT_0
#define DEFAULT_STANDARD	PLX_NTSC
#define DEFAULT_FORMAT		PLX_COMP
#define DEFAULT_TYPE		PLX_RGB24

#define DEFAULT_INTERVAL	50
#define DEFAULT_WIDTH		768
#define DEFAULT_HEIGHT		625

#define MAX_VALUE			0xff
#define MIN_VALUE			0x00

#define VIDEO_KEY			0xfe7a
#define WIN_KEY				0x7afe

#define VIDEO_MIN_WIDTH		40
#define VIDEO_MIN_HEIGHT	30

/* TYPEDEFS */

/* STATIC VARIABLES */

static int		 VideoInInit();
static Xv_opaque VideoInSet();
static Xv_opaque VideoInGet();
static int		 VideoInDestroy();

static int XViewVersionNumber=0;			/* What version of XView are we */

static struct itimerval logoTimeValue;				    /* Logo variables: variables having to do with...	*/
static GC		logoGC;					    /* ...drawing a logo on the video input window. */
static XGCValues	logoGCValues;
static Pixmap		logoPixmap;
static Pixmap		logoShapeMask;
static Window		logoWindow;
static XpmAttributes	logoAttrs;
static int		logoDuration;
static Boolean		repaintLogo;
static char*		logoFileName;
static Colormap		logoColormap;


/* GLOBAL VARIABLES */

Xv_pkg video_in_pkg = {
	"VideoIn",
	ATTR_PKG_VIDEO_IN,
	sizeof(VideoInPublic),
	CANVAS,
	VideoInInit,
	VideoInSet,
	VideoInGet,
	VideoInDestroy,
	NULL
};

int Xv_VideoStatus=VIDEO_OKAY;				/* Storage for error codes		*/

plx_IO *PlxBoardSetup=NULL;					/* Pointer to hardware setup	*/

/* EXTERNAL VARIABLES */

extern char *xv_version;

/* FUNCTIONS */

/* Convert a sun pixrect structure into a ximage structure */

static XImage *PR2XI(vp,p)
VideoInPrivate *vp;
Pixrect *p;
{
	XWindowAttributes	attr;
	XImage*			i;
	XVisualInfo*		visualInfo;
	
	XGetWindowAttributes(vp->dpy,vp->pw_xid,&attr);
	switch(p->pr_depth) {
	case 1:
		i = XCreateImage(vp->dpy,attr.visual,1,XYBitmap,0,
				(char *) mpr_d(p)->md_image,p->pr_width,p->pr_height,
				MPR_LINEBITPAD,mpr_d(p)->md_linebytes);
		break;
	case 8:
		i = XCreateImage(vp->dpy,attr.visual,8,ZPixmap,0,
				(char *) mpr_d(p)->md_image,p->pr_width,p->pr_height,
				MPR_LINEBITPAD,mpr_d(p)->md_linebytes);
		break;
	case 32:
		if (XViewVersionNumber == 3)
		{
		  visualInfo = XPlxGetVideoVisual(vp->dpy,	    /* Find the best visual for video */
						  attr.screen);
		  i = XCreateImage(vp->dpy, attr.visual,	    /* Create an X image of the correct depth for... */
				   visualInfo->depth,		    /* ...TrueColor visuals. */
				   ZPixmap, 0,
				   (char *) mpr_d(p)->md_image,
				   p->pr_width,p->pr_height,
				   32,mpr_d(p)->md_linebytes);
		}
		else
		{
		  i = XCreateImage(vp->dpy,attr.visual,32,ZPixmap,0,
				   (char *) mpr_d(p)->md_image,p->pr_width,p->pr_height,
				   32,mpr_d(p)->md_linebytes);
		}
		break;
	}
	i->byte_order       = MSBFirst;
	i->bitmap_bit_order = MSBFirst;
	return(i);
}

/* Convert a XImage structure into a Sun pixrect */

#ifdef SVR4
static Pixrect *XI2PR(image)
XImage *image; 
{
	XFree(image->data);
	return((Pixrect *) NULL);
}
#else
static Pixrect *XI2PR(image)
XImage *image; 
{
	struct mpr_data *d;
	Pixrect *p;
	 
#ifdef PLXSERVER
	switch (image->depth) {
	case 24:
		p = mem_point(image->width,image->height,image->bitmap_unit,image->data);
		break;
	default:
		p = mem_point(image->width,image->height,image->depth,image->data);
		break;
	}
#else
	p = mem_point(image->width,image->height,image->bitmap_unit,image->data);
#endif
	(mpr_d(p))->md_linebytes = image->bytes_per_line;
	(mpr_d(p))->md_primary   = TRUE;

	return(p);
}
#endif
/* Create a shared memory version of JPEGImage structure */

static JPEGImage *CreateImageCache(display, width, height)
Display			*display;
int				 width;
int				 height;
{
	int			majorv;
	int			minorv;
	int			i;
	JPEGImage  *image;
	Bool		sharedPixmaps;
	XShmSegmentInfo *shminfo;

	if(!XPlxShmQueryVersion(display,&majorv,&minorv,&sharedPixmaps)) return(NULL);
	shminfo = (XShmSegmentInfo *) calloc(1,sizeof(XShmSegmentInfo));
	shminfo->shmid = shmget(IPC_PRIVATE, 128 * 1024, IPC_CREAT|0777);
	if (shminfo->shmid < 0) return(NULL);
	shminfo->shmaddr = (char *) shmat(shminfo->shmid, 0, 0);
	if (shminfo->shmaddr == ((char *) -1)) {
		shmctl(shminfo->shmid, IPC_RMID, 0);
		return(NULL);
	}
	image = (JPEGImage *) XPlxShmCreateCImage(display,shminfo->shmaddr,shminfo,0,width,height);
	shminfo->readOnly  = False;
	XPlxShmAttach(display, shminfo);
	return(image);
}

/* Destroy the X11 shared memory JPEGImage structure */

static int DestroyImageCache(display,image)
Display *display;
JPEGImage *image;
{
	int rc;

	XPlxShmDetach(display, image->info);
	XSync(display,False);
	rc = shmdt(image->info->shmaddr);
/*
	if (rc < 0) perror("shmdt failed");
*/
	rc = shmctl(image->info->shmid, IPC_RMID, 0);
	if (rc < 0) perror("shmctl failed");
	free(image->info);
	free(image);
}

/* Verify good values for some parameters */

/* Translate VIDEO_FORMAT into a XPlxVideo format */

static int TranslateFormat(value)
int value;
{
	switch (value) {
	case VIDEO_COMPOSITE:
		return(PLX_COMP);
	case VIDEO_SUPER_VHS:
		return(PLX_YC);
	case VIDEO_YUV:
		return(PLX_YUV);
	case VIDEO_RGB:
		return(PLX_RGB);
	default:
		return(-1);
	}
}

/* Translate a XPlxVideo format into a VIDEO_FORMAT */

static int UnTranslateFormat(value)
int value;
{
	switch (value) {
	case PLX_COMP:
		return(VIDEO_COMPOSITE);
	case PLX_YC:
		return(VIDEO_SUPER_VHS);
	case PLX_YUV:
		return(VIDEO_YUV);
	case PLX_RGB:
		return(VIDEO_RGB);
	default:
		return(0);
	}
}

/* Translate VIDEO_STANDARD into a XPlxVideo standard */

static int TranslateStandard(value)
int value;
{
	switch (value) {
	case VIDEO_NTSC:
		return(PLX_NTSC);
	case VIDEO_PAL:
		return(PLX_PAL);
	case VIDEO_SECAM:
		return(PLX_SECAM);
	default:
		return(-1);
	}
}

/* Translate a XPlxVideo standard into VIDEO_STANDARD values */

static int UnTranslateStandard(value)
int value;
{
	switch (value) {
	case PLX_NTSC:
		return(VIDEO_NTSC);
	case PLX_PAL:
		return(VIDEO_PAL);
	case PLX_SECAM:
		return(VIDEO_SECAM);
	default:
		return(0);
	}
}

/* Translate a VIDEO_INPUT_TYPE into a XPlxVIdeo input type */

static int TranslateType(value)
int value;
{
	switch (value) {
	case VIDEO_YUV7:
		return(PLX_YUV7);
	case VIDEO_RGB24:
		return(PLX_RGB24);
	case VIDEO_RGB8:
		return(PLX_RGB8);
	case VIDEO_GRAY7:
		return(PLX_GRAY7);
	case VIDEO_GRAY8:
		return(PLX_GRAY8);
	default:
		return(-1);
	}
}

/* Translate a XPlxVideo input type into a VIDEO_INPUT_TYPE value */

static int UnTranslateType(value)
int value;
{
	switch (value) {
	case PLX_YUV7:
		return(VIDEO_YUV7);
	case PLX_RGB24:
		return(VIDEO_RGB24);
	case PLX_RGB8:
		return(VIDEO_RGB8);
	case PLX_GRAY7:
		return(VIDEO_GRAY7);
	case PLX_GRAY8:
		return(VIDEO_GRAY8);
	default:
		return(0);
	}
}

/* Translate a VIDEO_IO into a XPlxVideo channel */

static int TranslateChannel(value)
int value;
{
	switch (value) {
	case VIDEO_IO_OFF:
		return(PLX_IO_OFF);
	case VIDEO_IN_ONE:
		return(PLX_INPUT_0);
	case VIDEO_IN_TWO:
		return(PLX_INPUT_1);
	case VIDEO_OUT_ONE:
		return(PLX_OUTPUT_0);
	default:
		return(-1);
	}
}

/* Translate a XPlxVideo channel into a VIDEO_IO value */

static int UnTranslateChannel(value)
int value;
{
	switch (value) {
	case PLX_IO_OFF:
		return(VIDEO_IO_OFF);
	case PLX_INPUT_0:
		return(VIDEO_IN_ONE);
	case PLX_INPUT_1:
		return(VIDEO_IN_TWO);
	case PLX_OUTPUT_0:
		return(VIDEO_OUT_ONE);
	default:
		return(0);
	}
}

/* VIDEO_SATURATION, VIDEO_HUE, VIDEO_CONTRAST, VIDEO_BRIGHTNESS */

static Boolean CheckValue(value)
int value;
{
	if (value < MIN_VALUE) return(FALSE);
	if (value > MAX_VALUE) return(FALSE);
	return(TRUE);
}

/* Translate a VIDEO_GRAPHICS type into XPlxVideo value */

static int TranslateGraphics(value)
int value;
{
	switch (value) {
	case VIDEO_GRAPHICS_1:
		return(PLX_GRAPHICS_1);
	case VIDEO_GRAPHICS_8:
		return(PLX_GRAPHICS_8);
	case VIDEO_GRAPHICS_24:
		return(PLX_GRAPHICS_24);
	case VIDEO_PIXELS:
		return(PLX_VIDEO);
	case VIDEO_OVERLAY:
		return(PLX_VIDEO_OVR);
	default:
		return(-1);
	}
}

/* UnTranslate a XPlxVideo value into a VIDEO_GRAPHICS value */

static int UnTranslateGraphics(value)
int value;
{
	switch (value) {
	case PLX_GRAPHICS_1:
		return(VIDEO_GRAPHICS_1);
	case PLX_GRAPHICS_8:
		return(VIDEO_GRAPHICS_8);
	case PLX_GRAPHICS_24:
		return(VIDEO_GRAPHICS_24);
	case PLX_VIDEO:
		return(VIDEO_PIXELS);
	case PLX_VIDEO_OVR:
		return(VIDEO_OVERLAY);
/* This case should never happen */
	default:
		return(0);
	}
}

/* UnTranslate a XPlxVideo value into a VIDEO_HARDWARE value */

static int UnTranslateHardware(value)
int value;
{
	switch (value) {
	case PLX_1280:
		return(VIDEO_1280);
	case PLX_VIPER:
		return(VIDEO_VIPER);
	case PLX_VIDEOVIEW:
		return(VIDEO_VIDEOVIEW);
	case PLX_XVIDEO_24:
		return(VIDEO_XVIDEO_24);
	case PLX_XVIDEO_8:
		return(VIDEO_XVIDEO_8);
/* This case should never happen */
	default:
		return(0);
	}
}

/* Specific video display functions support in the XLIB extensions */

static void ShowLiveVideo(canvas,vp,x,y,w,h)
Canvas canvas;
VideoInPrivate *vp;
int x,y,w,h;
{
	if (w > (vp->width - vp->vx))  w = (vp->width - vp->vx);
#ifdef ALTERNATE
	if (h > (vp->height - vp->vy + 1)) h = (vp->height - vp->vy + 1);
#else
	if (h > (vp->height - vp->vy)) h = (vp->height - vp->vy);
#endif
	if (vp->vx > (vp->width - w)) vp->vx = (vp->width - w);
 	if (vp->vy > (vp->height - h)) vp->vy = (vp->height - h);
	if (PlxBoardSetup->hardware == PLX_VIPER) {
		if (vp->input_type == PLX_YUV7) {
			x = ((x >> 4) << 4);
			w = ((w >> 4) << 4) + 15;
		}
	}
	vp->dx = x;	vp->dy = y;	vp->dw = w;	vp->dh = h;
	if ((vp->flags & VideoJPEGFlag) == 0) {
#ifdef TRACE
		fprintf(stderr,"TRACE: XPlxVideoLive(vx=%d,vy=%d,x=%d,y=%d,w=%d,h=%d)\n",
				vp->vx,vp->vy,x,y,w,h);
#endif
		XPlxVideoLive(vp->dpy,vp->pw_xid,vp->pw_gc,vp->vx,vp->vy,x,y,w,h);
		if ((vp->flags & VideoSaveStillFlag) && vp->SavedStill) {
			XDestroyImage(vp->SavedStill);
			vp->SavedStill = NULL;
		}
	}
	else if (vp->flags & VideoTrackWinFlag) XClearWindow(vp->dpy,vp->pw_xid);
}

static void ShowStillVideo(canvas,vp,x,y,w,h)
Canvas canvas;
VideoInPrivate *vp;
int x,y,w,h;
{
	if (w > (vp->width - vp->vx))  w = (vp->width - vp->vx);
#ifdef ALTERNATE
	if (h > (vp->height - vp->vy + 1)) h = (vp->height - vp->vy + 1);
#else
	if (h > (vp->height - vp->vy)) h = (vp->height - vp->vy);
#endif
	if (vp->vx > (vp->width - w)) vp->vx = (vp->width - w);
	if (vp->vy > (vp->height + vp->interval - h))
		vp->vy = (vp->height + vp->interval - h);

	if (PlxBoardSetup->hardware == PLX_VIPER) {
		if (vp->input_type == PLX_YUV7) {
			x = ((x >> 4) << 4);
			w = ((w >> 4) << 4) + 15;
		}
	}
	vp->dx = x;	vp->dy = y;	vp->dw = w;	vp->dh = h;
	if ((vp->flags & VideoJPEGFlag) == 0) {
#ifdef TRACE
		fprintf(stderr,"TRACE: XPlxVideoStill(vx=%d,vy=%d,x=%d,y=%d,w=%d,h=%d)\n",
				vp->vx,vp->vy,x,y,w,h);
#endif
		if (vp->flags & VideoSaveStillFlag) {
			if (vp->SavedStill) {
				XPutImage(vp->dpy,vp->pw_xid,vp->pw_gc,vp->SavedStill,x,y,x,y,w,h);
			}
			else {
				XPlxVideoStill(vp->dpy,vp->pw_xid,vp->pw_gc,vp->vx,vp->vy,x,y,w,h);
				vp->SavedStill = (XImage *) XGetImage(vp->dpy,vp->pw_xid,
/* FIXUP: Why -2 for width and height? */
					 vp->dx,vp->dy,vp->dw-2,vp->dh-2,AllPlanes,ZPixmap);
			}
		}
		else XPlxVideoStill(vp->dpy,vp->pw_xid,vp->pw_gc,vp->vx,vp->vy,x,y,w,h);
	}
	else if (vp->flags & VideoTrackWinFlag) XClearWindow(vp->dpy,vp->pw_xid);
}

static void ShowSqueezeVideo(canvas,vp,x,y,w,h)
Canvas canvas;
VideoInPrivate *vp;
int x,y,w,h;
{
	if (vp->vw > (vp->width - vp->vx)) vp->vx = (vp->width - vp->vx);
#ifdef ALTERNATE
 	if (vp->vh > (vp->height - vp->vy + 1)) vp->vh = (vp->height - vp->vy + 1);
#else
 	if (vp->vh > (vp->height - vp->vy)) vp->vh = (vp->height - vp->vy);
#endif
	if (w > (vp->width - vp->vx)) w = (vp->width - vp->vx);
#ifdef ALTERNATE
	if (h > (vp->height - vp->vy + 1)) h = (vp->height - vp->vy + 1);
#else
	if (h > (vp->height - vp->vy)) h = (vp->height - vp->vy);
#endif
	if (vp->vx > (vp->width - vp->vw)) vp->vx = (vp->width - vp->vw);
	if (vp->vy > (vp->height + vp->interval - vp->vh))
		vp->vy = (vp->height + vp->interval - vp->vh);

	switch (PlxBoardSetup->hardware) {
	case PLX_VIPER:
		switch (vp->input_type) {
		case PLX_YUV7:
			vp->vx = ((vp->vx >> 4) << 4);
			vp->vw = ((vp->vw >> 4) << 4) + 15;
			x      = ((x >> 4) << 4);
			w     = ((w >> 4) << 4) + 15;
			if ((5 * w) < vp->vw) w = vp->vw / 5;
			if (w > vp->vw) w = vp->vw;
			break;
		case PLX_GRAY7:
			if ((w << 8) < vp->vw) w = vp->vw >> 8;
			if (w > vp->vw) w = vp->vw;
			break;
		}
		break;
	case PLX_VIDEOVIEW:
		if (w < 60 ) {
			x = (w - 40) / 2; if (x < 0) x = 0;
			y = (h - 30) / 2; if (y < 0) y = 0;
			w = 40; h = 30;
		}
		else if (w < 120) {
			x = (w - 80) / 2; if (x < 0) x = 0;
			y = (h - 60) / 2; if (y < 0) y = 0;
			w = 80; h = 60;
		}
		else if (w < 240) {
			x = (w - 160) / 2; if (x < 0) x = 0;
			y = (h - 120) / 2; if (y < 0) y = 0;
			w = 160; h = 120;
		}
		else if (w < 480) {
			x = (w - 320) / 2; if (x < 0) x = 0;
			y = (h - 240) / 2; if (y < 0) y = 0;
			w = 320; h = 240;
		}
		else {
			x = (w - 640) / 2; if (x < 0) x = 0;
			y = (h - 480) / 2; if (y < 0) y = 0;
			w = 640; h = 480;
		}
		break;
	case PLX_XVIDEO_8:
	case PLX_XVIDEO_24:
		if ((w << 12) < vp->vw) w = vp->vw >> 12;
		if (w > vp->vw) w = vp->vw;
		break;
	}
	vp->dx = x;	vp->dy = y;	vp->dw = w;	vp->dh = h;
	if ((vp->flags & VideoJPEGFlag) == 0) {
#ifdef TRACE
		fprintf(stderr,"TRACE: XPlxVideoSqueezeLive(vx=%d,vy=%d,vw=%d,vh=%d,x=%d,y=%d,w=%d,h=%d)\n",
				vp->vx,vp->vy,vp->vw,vp->vh,x,y,w,h);
#endif
		XPlxVideoSqueezeLive(vp->dpy,vp->pw_xid,vp->pw_gc,
							 vp->vx,vp->vy,vp->vw,vp->vh,x,y,w,h);
		if ((vp->flags & VideoSaveStillFlag) && vp->SavedStill) {
			XDestroyImage(vp->SavedStill);
			vp->SavedStill = NULL;
		}
	}
	else if (vp->flags & VideoTrackWinFlag) XClearWindow(vp->dpy,vp->pw_xid);
}

static void ShowSqueezeStillVideo(canvas,vp,x,y,w,h)
Canvas canvas;
VideoInPrivate *vp;
int x,y,w,h;
{
	if (vp->vw > (vp->width - vp->vx)) vp->vx = (vp->width - vp->vx);
#ifdef ALTERNATE
	if (vp->vh > (vp->height + vp->interval - vp->vy))
		vp->vh = (vp->height + vp->interval - vp->vy);
#else
	if (vp->vh > (vp->height + vp->interval - vp->vy + 1))
		vp->vh = (vp->height + vp->interval - vp->vy + 1);
#endif
	if (w > (vp->width - vp->vx)) w = (vp->width - vp->vx);
#ifdef ALTERNATE
	if (h > (vp->height - vp->vy + 1)) h = (vp->height - vp->vy + 1);
#else
	if (h > (vp->height - vp->vy)) h = (vp->height - vp->vy);
#endif
	if (vp->vx > (vp->width - vp->vw)) vp->vx = (vp->width - vp->vw);
	if (vp->vy > (vp->height + vp->interval - vp->vh))
		vp->vy = (vp->height + vp->interval - vp->vh);

	switch (PlxBoardSetup->hardware) {
	case PLX_VIPER:
		switch (vp->input_type) {
		case PLX_YUV7:
			vp->vx = ((vp->vx >> 4) << 4);
			vp->vw = ((vp->vw >> 4) << 4) + 15;
			x      = ((x >> 4) << 4);
			w     = ((w >> 4) << 4) + 15;
			if ((5 * w) < vp->vw) w = vp->vw / 5;
			if (w > vp->vw) w = vp->vw;
			break;
		case PLX_GRAY7:
			if ((w << 8) < vp->vw) w = vp->vw >> 8;
			if (w > vp->vw) w = vp->vw;
			break;
		}
		break;
	case PLX_VIDEOVIEW:
		if (w < 60 ) {
			x = (w - 40) / 2; if (x < 0) x = 0;
			y = (h - 30) / 2; if (y < 0) y = 0;
			w = 40; h = 30;
		}
		else if (w < 120) {
			x = (w - 80) / 2; if (x < 0) x = 0;
			y = (h - 60) / 2; if (y < 0) y = 0;
			w = 80; h = 60;
		}
		else if (w < 240) {
			x = (w - 160) / 2; if (x < 0) x = 0;
			y = (h - 120) / 2; if (y < 0) y = 0;
			w = 160; h = 120;
		}
		else if (w < 480) {
			x = (w - 320) / 2; if (x < 0) x = 0;
			y = (h - 240) / 2; if (y < 0) y = 0;
			w = 320; h = 240;
		}
		else {
			x = (w - 640) / 2; if (x < 0) x = 0;
			y = (h - 480) / 2; if (y < 0) y = 0;
			w = 640; h = 480;
		}
		break;
	case PLX_XVIDEO_8:
	case PLX_XVIDEO_24:
		if ((w << 12) < vp->vw) w = vp->vw >> 12;
		if (w > vp->vw) w = vp->vw;
		break;
	}
	vp->dx = x;	vp->dy = y;	vp->dw = w;	vp->dh = h;
	if ((vp->flags & VideoJPEGFlag) == 0) {
/* If the video conferencing paramters are set, revise video region */
		w = w / vp->conf_size;
#ifdef TRACE
		fprintf(stderr,"TRACE: XPlxVideoSqueezeStill(vx=%d,vy=%d,vw=%d,vh=%d,x=%d,y=%d,w=%d,h=%d)\n",
				vp->vx,vp->vy,vp->vw,vp->vh,x,y,w,h);
#endif
		if (vp->flags & VideoSaveStillFlag) {
			if (vp->SavedStill) {
				XPutImage(vp->dpy,vp->pw_xid,vp->pw_gc,vp->SavedStill,x,y,x,y,w,h);
			}
			else {
				XPlxVideoSqueezeStill(vp->dpy,vp->pw_xid,vp->pw_gc,vp->vx,
									  vp->vy,vp->vw,vp->vh,x,y,w,h);
				vp->SavedStill = (XImage *) XGetImage(vp->dpy,vp->pw_xid,
/* FIXUP: Why -2 for width and height? */
					 vp->dx,vp->dy,vp->dw-2,vp->dh-2,AllPlanes,ZPixmap);
			}
		}
		else XPlxVideoSqueezeStill(vp->dpy,vp->pw_xid,vp->pw_gc,vp->vx,
								   vp->vy,vp->vw,vp->vh,x,y,w,h);
	}
	else if (vp->flags & VideoTrackWinFlag) XClearWindow(vp->dpy,vp->pw_xid);
}

static void ShowScaleStillVideo(canvas,vp,x,y,w,h)
Canvas canvas;
VideoInPrivate *vp;
int x,y,w,h;
{
	if (vp->vw > (vp->width - vp->vx)) vp->vx = (vp->width - vp->vx);
#ifdef ALTERNATE
	if (vp->vh > (vp->height + vp->interval - vp->vy + 1))
		vp->vh = (vp->height + vp->interval - vp->vy + 1);
#else
	if (vp->vh > (vp->height + vp->interval - vp->vy))
		vp->vh = (vp->height + vp->interval - vp->vy);
#endif
	if (vp->vx > (vp->width - vp->vw)) vp->vx = (vp->width - vp->vw);
	if (vp->vy > (vp->height + vp->interval - vp->vh))
		vp->vy = (vp->height + vp->interval - vp->vh);

	switch (PlxBoardSetup->hardware) {
	case PLX_VIPER:
		vp->vx = ((vp->vx >> 4) << 4);
		vp->vw = ((vp->vw >> 4) << 4) + 15;
		x      = ((x >> 4) << 4);
		w      = ((w >> 4) << 4) + 15;
		if ((w << 8) < vp->vw) w = vp->vw >> 8;
		if (w > vp->vw) w = vp->vw;
		break;
	case PLX_VIDEOVIEW:
		if (w < 60 ) {
			x = (w - 40) / 2; if (x < 0) x = 0;
			y = (h - 30) / 2; if (y < 0) y = 0;
			w = 40; h = 30;
		}
		else if (w < 120) {
			x = (w - 80) / 2; if (x < 0) x = 0;
			y = (h - 60) / 2; if (y < 0) y = 0;
			w = 80; h = 60;
		}
		else if (w < 240) {
			x = (w - 160) / 2; if (x < 0) x = 0;
			y = (h - 120) / 2; if (y < 0) y = 0;
			w = 160; h = 120;
		}
		else if (w < 480) {
			x = (w - 320) / 2; if (x < 0) x = 0;
			y = (h - 240) / 2; if (y < 0) y = 0;
			w = 320; h = 240;
		}
		else {
			x = (w - 640) / 2; if (x < 0) x = 0;
			y = (h - 480) / 2; if (y < 0) y = 0;
			w = 640; h = 480;
		}
		break;
	case PLX_XVIDEO_8:
	case PLX_XVIDEO_24:
		if ((w << 12) < vp->vw) w = vp->vw >> 12;
		if (w > vp->vw) w = vp->vw;
		break;
	}
	vp->dx = x;	vp->dy = y;	vp->dw = w;	vp->dh = h;
	if ((vp->flags & VideoJPEGFlag) == 0) {
#ifdef TRACE
		fprintf(stderr,"TRACE: XPlxVideoScale(vx=%d,vy=%d,vw=%d,vh=%d,x=%d,y=%d,w=%d,h=%d)\n",
				vp->vx,vp->vy,vp->vw,vp->vh,x,y,w,h);
#endif
		if (vp->flags & VideoSaveStillFlag) {
			if (vp->SavedStill) {
				XPutImage(vp->dpy,vp->pw_xid,vp->pw_gc,vp->SavedStill,x,y,x,y,w,h);
			}
			else {
				XPlxVideoScale(vp->dpy,vp->pw_xid,vp->pw_gc,vp->vx,vp->vy,vp->vw,vp->vh,x,y,w,h);
				vp->SavedStill = (XImage *) XGetImage(vp->dpy,vp->pw_xid,
/* FIXUP: Why -2 for width and height? */
					 vp->dx,vp->dy,vp->dw-2,vp->dh-2,AllPlanes,ZPixmap);
			}
		}
		else XPlxVideoScale(vp->dpy,vp->pw_xid,vp->pw_gc,vp->vx,vp->vy,vp->vw,vp->vh,x,y,w,h);
	}
	else if (vp->flags & VideoTrackWinFlag) XClearWindow(vp->dpy,vp->pw_xid);
}

/* Track canvas resizes and enforce constraints */

static void VideoResize(canvas,width,height)
Canvas canvas;
int width;
int height;
{
	int i,j,w,h;
	Xv_Window win;
	VideoInPrivate *vp;

	vp		   = (VideoInPrivate *) xv_get(canvas,XV_KEY_DATA,VIDEO_KEY);
	vp->flags &= ~VideoAdjustFlag;
#ifdef ALTERNATE
	if (width  != DEFAULT_WIDTH)  width  += 2;
	if (height != (DEFAULT_HEIGHT - DEFAULT_INTERVAL)) height += 2;
#endif
	w          = width;
	h          = height;
#ifdef TRACE
		fprintf(stderr,"TRACE: VideoResize(width=%d,height=%d)\n",w,h);
#endif
	if (width  > vp->width)  w = vp->width;
#ifdef ALTERNATE
	if (height > (vp->height - vp->interval)) h = vp->height - vp->interval;
#else
	if (height > (vp->height - vp->interval)) h = vp->height - vp->interval;
#endif
	if (vp->flags & VideoAspectFlag) {
		i = (w - 2) / 4;
		j = (h - 2) / 3;
		if (i != j) {
			i = (i + j) / 2;
			w = i * 4;
			h = i * 3;
		}
	}
	if ((w != width) || (h != height)) {
#ifndef PLXSERVER
		vp->flags |= VideoAdjustFlag;
#endif
		if (vp->flags & VideoTrackWinFlag) {
#ifdef TRACE
			fprintf(stderr,"TRACE: Resizing video canvas to %d by %d\n",w,h);
#endif
			win = (Xv_Window) xv_get(canvas,XV_OWNER);
			w += (int) xv_get(canvas,XV_X);
			h += (int) xv_get(canvas,XV_Y);
			xv_set(win,XV_WIDTH,w,XV_HEIGHT,h,NULL);
		}
		/* vp->dw = w; vp->dh = h; */
	}
	vp->flags |= VideoResizeFlag;
}


static void RepaintLogo(VideoInPrivate*	vp)
{
  int			startY;
  int			startX;
  
  startX = xv_get(vp->public_self, XV_WIDTH) - (logoAttrs.width + 5);
  startY = xv_get(vp->public_self, XV_HEIGHT) - (logoAttrs.height + 5);
  XSetClipOrigin(vp->dpy, logoGC, startX, startY);
  XFillRectangle(vp->dpy, vp->pw_xid, logoGC,
		 startX, startY,
		 logoAttrs.width, logoAttrs.height);
  XCopyArea(vp->dpy, logoPixmap, vp->pw_xid, logoGC,
	    0, 0, logoAttrs.width, logoAttrs.height,
	    startX, startY);
  return;
} /* end function RepaintLogo */



static void LogoTimerFunction(Notify_client	client,
			      int		which)
{
  Display*	theDisplay;
  
  repaintLogo = False;
  theDisplay = (Display*)xv_get(client, XV_DISPLAY);
  XClearArea(theDisplay, logoWindow, 0, 0, 0, 0, True);
  return;
} /* end function LogoTimerFunction */



/* The workhorse function, clip and expose regions of video */

static void VideoRepaint(canvas,pw,rects)
Canvas canvas;
Xv_Window pw;
Rectlist *rects;
{
	XRectangle clip;
	Rectnode *r;
	Rect *Viewable;
	Boolean seen;
	VideoInPrivate *vp;
	int x,y,w,h;

/* If no exposure areas generated, skip this call */

	if (rects == NULL) return;

	vp		 = (VideoInPrivate *) xv_get(canvas,XV_KEY_DATA,VIDEO_KEY);

/* If we are in an active resize process, skip for now */

	if (vp->flags & VideoAdjustFlag) {
#ifdef TRACE
		fprintf(stderr,"TRACE: Detected VideoAdjustFlag in VideoRepaint\n");
#endif
		return;
	}

/* Dig out the view canvas area on the window */

	Viewable = (Rect *) xv_get(canvas,CANVAS_VIEWABLE_RECT,pw);
	x		 = Viewable->r_left;
	y		 = Viewable->r_top;
#ifdef ALTERNATE
	w		 = Viewable->r_width  + 2;
	h		 = Viewable->r_height + 2;
#else
	w		 = Viewable->r_width;
	h		 = Viewable->r_height;
#endif
	vp->flags |= VideoCalledFlag;
#ifdef TRACE
	fprintf(stderr,"TRACE: VideoRepaint(vx=%d,vy=%d,vw=%d,vh=%d,x=%d,y=%d,w=%d,h=%d)\n",
		vp->vx,vp->vy,vp->vw,vp->vh,x,y,w,h);
#endif
/* This is a impossible value used to indicate a call generated by a WIN_RESIZE event */
	if (rects != (Rectlist *) 1) {
		r    = rects->rl_head;
		seen = TRUE;
		while (r) {
			clip.x      = rects->rl_x + r->rn_rect.r_left;
			if (clip.x != vp->exposure.r_left) {
				vp->exposure.r_left = clip.x;
				seen = FALSE;
			}
			clip.y      = rects->rl_y + r->rn_rect.r_top;
			if (clip.y != vp->exposure.r_top) {
				vp->exposure.r_top = clip.y;
				seen = FALSE;
			}
			clip.width  = r->rn_rect.r_width;
			if (clip.width != vp->exposure.r_width) {
				vp->exposure.r_width = clip.width;
				seen = FALSE;
			}
			clip.height = r->rn_rect.r_height;
			if (clip.height != vp->exposure.r_height) {
				vp->exposure.r_height = clip.height;
				seen = FALSE;
			}
#ifdef TRACE
			fprintf(stderr,"TRACE: Exposure(x=%d,y=%d,w=%d,h=%d)\n",
				clip.x,clip.y,clip.width,clip.height);
#endif
			r = r->rn_next;
/* XView generates a unecessary EXPOSURE when shrinking the window, we will */
/* try to detect this event and ignore its operation.						*/
			if ((x == clip.x) && (y == clip.y) && (r == NULL)) {
				if (((w + 1) == clip.width) && ((h + 1) == clip.height)) {
#ifdef TRACE
					fprintf(stderr,"TRACE: Unecessary video exposure, skipping\n");
#endif
					return;
				}
			}
#ifdef CLIENTVV
			if (PlxBoardSetup->hardware == PLX_VIDEOVIEW) {
				if (r) plxvvexpose(vp->dpy,vp->pw_xid,&clip,TRUE);
				else plxvvexpose(vp->dpy,vp->pw_xid,&clip,FALSE);
			}
#endif
			if (PlxBoardSetup->hardware == PLX_XVIDEO_24) {

/* If we are handling a CCube board, then reset the TAG bits if necessary */

				if ((vp->flags & VideoFirstDeComp) == 0) {
#ifdef TRACE
					fprintf(stderr,"TRACE: XPlxVideoTag(bits=%d)\n",vp->tagbits);
#endif
					vp->flags |= VideoFirstDeComp;
					vp->flags |= VideoFirstComp;
					vp->tagbits = PLX_VIDEO;
					XPlxVideoTag(vp->dpy,vp->pw_xid,vp->pw_gc,vp->tagbits);
				}
				else if ((vp->flags & VideoFirstComp) == 0) {
#ifdef TRACE
					fprintf(stderr,"TRACE: XPlxVideoTag(bits=%d)\n",vp->tagbits);
#endif
					vp->flags |= VideoFirstDeComp;
					vp->flags |= VideoFirstComp;
					vp->tagbits = PLX_VIDEO;
					XPlxVideoTag(vp->dpy,vp->pw_xid,vp->pw_gc,vp->tagbits);
				}
			}
		}
		if (seen) {
#ifdef TRACE
			fprintf(stderr,"TRACE: No new video exposures, skipping\n");
#endif
#ifdef CLIENTVV
			return;
#endif
		}
	}
	else if (vp->flags & VideoJPEGFlag) {
		if (PlxBoardSetup->hardware == PLX_XVIDEO_24) {

/* If we are handling a CCube board, then reset the TAG bits if necessary */

			if ((vp->flags & VideoFirstDeComp) == 0) {
#ifdef TRACE
				fprintf(stderr,"TRACE: XPlxVideoTag(bits=%d)\n",vp->tagbits);
#endif
				vp->flags |= VideoFirstDeComp;
				vp->flags |= VideoFirstComp;
				vp->tagbits = PLX_VIDEO;
				XPlxVideoTag(vp->dpy,vp->pw_xid,vp->pw_gc,vp->tagbits);
			}
		}
	}

/* Start new video */

	if (vp->vx < 0) vp->vx = 0;	if (vp->vy < 0) vp->vy = 0;
	if (vp->vw < 1) vp->vw = 1;	if (vp->vh < 1) vp->vh = 1;
	if (x < 0)           x = 0;	if (y < 0)           y = 0;
	if (w < 1)           w = 1;	if (h < 1)           h = 1;
	if (vp->flags & VideoLiveFlag) {
		if (vp->flags & VideoSqueezeFlag) ShowSqueezeVideo(canvas,vp,x,y,w,h);
		else ShowLiveVideo(canvas,vp,x,y,w,h);
	}
	else {
		if (vp->flags & VideoSqueezeFlag) ShowSqueezeStillVideo(canvas,vp,x,y,w,h);
		else if (vp->flags & VideoScaleFlag) ShowScaleStillVideo(canvas,vp,x,y,w,h);
		else ShowStillVideo(canvas,vp,x,y,w,h);
	}
	if (0 != repaintLogo)					    /* Should a logo be shown on top of the video? */
	{							    /* Yes, repaint it after video is painted */
	  RepaintLogo(vp);
	}

/* Clear this flag (in case we are in a visibility reconfiguration event) */

	vp->flags &= ~VideoResizeFlag;
}

static int VideoInInit(owner,video_public,avlist)
Xv_opaque     owner;
VideoInPublic *video_public;
Attr_avlist   avlist;
{
	VideoInPrivate *video_private = xv_alloc(VideoInPrivate);

	if (!video_private) {
		xv_error((Xv_object) video_public,
			ERROR_STRING,	"Unable to allocate video data structure",
			ERROR_SEVERITY,	ERROR_RECOVERABLE,
			ERROR_PKG,		VIDEO_IN,
			NULL);
		return(XV_ERROR);
	}

/* Figure out what version of XView we are using */

	if (!XViewVersionNumber)
		XViewVersionNumber = (strcmp(xv_version,"XView Version 2 FCS") == 0) ? 2 : 3;

/* Setup inheritance links */

	video_public->private_data = (Xv_opaque) video_private;
	video_private->public_self = (Xv_opaque) video_public;

/* Initialize the defaults for the private variables */

	video_private->flags		= VideoCreateFlag  | VideoLiveFlag | VideoFirstComp |
								  VideoFirstDeComp | VideoTrackWinFlag;
	video_private->vx			= 0;
	video_private->dx			= 0;
	video_private->vy			= DEFAULT_INTERVAL;
	video_private->dy			= DEFAULT_INTERVAL;
	video_private->vw			= DEFAULT_WIDTH;
	video_private->dw			= DEFAULT_WIDTH;
	video_private->vh			= DEFAULT_HEIGHT - DEFAULT_INTERVAL;
	video_private->dh			= DEFAULT_HEIGHT - DEFAULT_INTERVAL;
	video_private->width		= DEFAULT_WIDTH;
	video_private->height		= DEFAULT_HEIGHT;
	video_private->interval		= DEFAULT_INTERVAL;
	video_private->standard		= DEFAULT_STANDARD;
	video_private->p_standard	= DEFAULT_STANDARD;
	video_private->format		= DEFAULT_FORMAT;
	video_private->p_format		= DEFAULT_FORMAT;
	video_private->input_type	= DEFAULT_TYPE;
	video_private->p_input_type	= DEFAULT_TYPE;
	video_private->channel		= DEFAULT_CHANNEL;
	video_private->p_channel	= DEFAULT_CHANNEL;
	video_private->saturation	= 0x80;
	video_private->hue			= 0x80;
	video_private->brightness	= 0x80;
	video_private->contrast		= 0x80;
	video_private->tagbits		= PLX_VIDEO;
	video_private->QFactor		= 100;
	video_private->ImageCache	= NULL;
	video_private->SavedStill	= NULL;
	video_private->conf_size    = 1;
	video_private->dpy			= (Display *) xv_get((Xv_object) video_public,XV_DISPLAY);

/* The paint window, its XID, and its GC will have to be initialized at the */
/* end of the xv_create phase, as the paint window doesn't exist until the  */
/* second pass of the xview object creation process.						*/

/* Customize some canvas and window attributes */

	xv_set((Xv_object) video_public,
		CANVAS_AUTO_SHRINK,			TRUE,
		CANVAS_AUTO_EXPAND,			TRUE,
		CANVAS_AUTO_CLEAR,			FALSE,
		CANVAS_RETAINED,			FALSE,
		CANVAS_REPAINT_PROC,		VideoRepaint,
		CANVAS_RESIZE_PROC,			VideoResize,
		CANVAS_MIN_PAINT_WIDTH,		VIDEO_MIN_WIDTH,
		CANVAS_MIN_PAINT_HEIGHT,	VIDEO_MIN_HEIGHT,
		CANVAS_FIXED_IMAGE,			FALSE,
		XV_WIDTH,					video_private->vw+2,
		XV_HEIGHT,					video_private->vh+2,
	       XV_VISUAL_CLASS,				TrueColor,
/*
	       XV_DEPTH,				24,
*/
		XV_KEY_DATA,				VIDEO_KEY,				video_private,
		0);

	Xv_VideoStatus = VIDEO_OKAY;
	return(XV_OK);
}

/* ReAdjust the requested board IO parameter */

static int AdjustBoardIO(vp)
VideoInPrivate *vp;
{
	plx_IO_list     *a,*f;
	plx_signal_list *b;
	plx_format_list *c;
	plx_type_list   *d;
	plx_signal		*e;

	Xv_VideoStatus = VIDEO_OKAY;

	if (vp->flags & VideoJPEGFlag) {
		Xv_VideoStatus = VIDEO_INVALID_OPERATION;
		return(XV_ERROR);
	}
	if (vp->p_channel != vp->channel) {
		a = PlxBoardSetup->inputs;
		while (a) {
			if (a->channel == vp->channel) break;
			a = a->next_IO;
		}
		if (a == NULL) {
			vp->channel = vp->p_channel;
			Xv_VideoStatus = VIDEO_CHANNEL_IN_USE;
			return(XV_ERROR);
		}
#ifndef SHARE_CHANNELS
		a->channel |= CHAN_INUSE;
#endif
		f = PlxBoardSetup->inputs;
		while (f) {
			if ((f->channel & 0xff) == vp->p_channel) break;
			f = f->next_IO;
		}
		if (f) f->channel &= 0xff;
		vp->p_channel = vp->channel;
	}
	else {
		a = PlxBoardSetup->inputs;
		while (a) {
			if ((a->channel & 0xff) == vp->channel) break;
			a = a->next_IO;
		}
		if (a == NULL) {
			Xv_VideoStatus = VIDEO_NO_MORE_CHANNELS;
			return(XV_ERROR);
		}
	}
	for (b=a->signal_list; b!=NULL; b=b->next_signal) {
		if (b->signal->standard == vp->standard) break;
	}
	if (b == NULL) {
		vp->standard = vp->p_standard;
		Xv_VideoStatus = VIDEO_BAD_STANDARD;
		return(XV_ERROR);
	}
	else vp->p_standard = vp->standard;
	for (c=a->format_list; c!=NULL; c=c->next_format) {
		if (c->format == vp->format) break;
	}
	if (c == NULL) {
		vp->format = vp->p_format;
		Xv_VideoStatus = VIDEO_BAD_FORMAT;
		return(XV_ERROR);
	}
	else vp->p_format = vp->format;
	for (d=a->type_list; d!=NULL; d=d->next_type) {
		if (d->type == vp->input_type) break;
	}
	if (d == NULL) {
		vp->input_type = vp->p_input_type;
		Xv_VideoStatus = VIDEO_BAD_INPUT_TYPE;
		return(XV_ERROR);
	}
	else vp->p_input_type = vp->input_type;

/* Set the desired input parameters we want (and should get) */

	XPlxVideoInputSelect(vp->dpy,vp->pw_xid,vp->pw_gc,vp->channel,
		vp->standard,vp->format,vp->input_type);

/* See what we really got off the board */

	e = (plx_signal *) XPlxQueryVideo(vp->dpy,vp->pw_xid,vp->pw_gc);

/* If not synced up, test for PAL signal */

	if(!e->sync_ok) {
		if (vp->standard == PLX_NTSC) vp->standard = PLX_PAL;
		else vp->standard = PLX_NTSC;
		XPlxVideoInputSelect(vp->dpy,vp->pw_xid,vp->pw_gc,vp->channel,
			vp->standard,vp->format,vp->input_type);
		e = (plx_signal *) XPlxQueryVideo(vp->dpy,vp->pw_xid,vp->pw_gc);

/* If not synced up, test for SECAM signal */

		if (!e->sync_ok) {
			if (vp->standard == PLX_PAL) vp->standard = PLX_SECAM;
			else vp->standard = PLX_PAL;
			XPlxVideoInputSelect(vp->dpy,vp->pw_xid,vp->pw_gc,vp->channel,
								 vp->standard,vp->format,vp->input_type);
			e = (plx_signal *) XPlxQueryVideo(vp->dpy,vp->pw_xid,vp->pw_gc);
			if (!e->sync_ok) {
				Xv_VideoStatus = VIDEO_NO_SYNC_DETECTED;
				return(XV_ERROR);
			}
			else {
				if (vp->standard == PLX_PAL) vp->standard = PLX_SECAM;
				else vp->standard = PLX_PAL;
			}
		}
		else {
			if (vp->standard == PLX_NTSC) vp->standard = PLX_PAL;
			else vp->standard = PLX_NTSC;
		}
	}
	if (e->interlaced) vp->flags |= VideoLacedFlag;
	if (e->sync_ok)    vp->flags |= VideoSyncFlag;
#ifdef ALTERNATE
	vp->vw		 = e->w;
	vp->width    = e->w;
	vp->vh		 = (e->w * 3) / 4;
#ifdef CLIENTVV
	vp->height   = (e->w * 3) / 4;
#else
	vp->height   = e->h;
#endif
	vp->interval = e->h - vp->vh;
	vp->vy       = vp->interval;
#else
	vp->vy       = e->b;
	vp->vw		 = e->w;
	vp->vh		 = e->h - e->b;
	vp->width    = e->w;
#ifdef CLIENTVV
	vp->height   = e->h - e->b;
#else
	vp->height   = e->h;
#endif
	vp->interval = e->b;
#endif
	vp->standard = e->standard;

	return(XV_OK);
}

/* Notice window movement */

static Notify_value frame_interposer(frame,event,arg,type)
Frame frame;
Event *event;
Notify_arg arg;
Notify_event_type type;
{
	XEvent *xevent;
	VideoInPrivate *video_private;
	Notify_value value;

	video_private = (VideoInPrivate *) xv_get(frame,XV_KEY_DATA,VIDEO_KEY);

/* Defensive programming */
	if (video_private == NULL)
		return(notify_next_event_func(frame,(Notify_event) event,arg,type));

	if (event_id(event) == WIN_RESIZE) {
		if (video_private->flags & VideoResizeFlag) {
			/*video_private->flags &= ~VideoResizeFlag;jamr*/
#ifdef TRACE
			fprintf(stderr,"TRACE: Frame interposer ignores WIN_RESIZE\n");
#endif
			return(notify_next_event_func(frame,(Notify_event) event,arg,type));
		}
		if (video_private->flags & VideoCalledFlag) {
			xevent = event_xevent(event);
			if (xevent->xconfigure.width) {
				if (xevent->xconfigure.height) {
#ifdef TRACE
					fprintf(stderr,"TRACE: Frame interposer sees WIN_RESIZE\n");
#endif
					VideoRepaint(video_private->public_self,video_private->pw,(Rectlist *)1);
				}
			}
		}
	}
	else if (event_id(event) == WIN_VISIBILITY_NOTIFY) {
		xevent = event_xevent(event);
		if (xevent->xvisibility.state != VisibilityUnobscured) {
			if (PlxBoardSetup->hardware == PLX_VIDEOVIEW) {
#ifdef CLIENTVV
#ifdef TRACE
				fprintf(stderr,"TRACE: Frame interposer calls plxvvvisnote\n");
#endif
				plxvvvisnote(video_private->dpy,video_private->pw_xid,
					(xevent->xvisibility.state == VisibilityFullyObscured) ? 1 : 0);
/* Skip the upcomming WIN_RESIZE event */
				video_private->flags |= VideoResizeFlag;
#endif
			}
		}
	}
/* jamr - commented out for now, don't remember why this was added.
   This causes a dragged and dropped still to be immediately erased
   because of the KBD event generated (for some people).
#ifdef SHARE_CHANNELS
	else if (event_id(event) == ACTION_NULL_EVENT) {
#ifdef TRACE
		fprintf(stderr,"TRACE: Frame interposer sees ACTION_NULL_EVENT\n");
#endif
		VideoRepaint(video_private->public_self,video_private->pw,(Rectlist *)1);
	}
#endif
*/
	return(notify_next_event_func(frame,(Notify_event) event,arg,type));
}




/* Called at the end of the initialization process */

static int FinishInit(vp)
     VideoInPrivate *vp;
{
  Xv_Window		 win;
  plx_IO_list     *a;
  plx_signal_list *b;
  plx_format_list *c;
  plx_type_list   *d;
  plx_signal		*e;
  int              f;
  int			result;
  static Boolean	logoAlreadyDone = False;
  char*			logoEnvResult;
  char			logoNameString[1024];
  
  Xv_VideoStatus = VIDEO_OKAY;
  
  /* Finish the initialization process for the VIDEO structure */
  
  vp->pw			= (Xv_Window) xv_get(vp->public_self,
						     CANVAS_NTH_PAINT_WINDOW,NULL);
  vp->pw_xid			= (Window) xv_get(vp->pw,XV_XID);
  vp->pw_gc 			= XCreateGC(vp->dpy,vp->pw_xid,(long) NULL,NULL);
  vp->exposure.r_left		= 0;
  vp->exposure.r_top		= 0;
  vp->exposure.r_width	= 0;
  vp->exposure.r_height	= 0;
  
  /* Change the window background to black */
  
  XSetWindowBackground(vp->dpy,vp->pw_xid,BlackPixel(vp->dpy,DefaultScreen(vp->dpy)));
  XSetForeground(vp->dpy,vp->pw_gc,0);
  if (logoAlreadyDone == False)					    /* Has the logo been initialized for this object already? */
  {								    /* If not, go through the process */
    logoAlreadyDone = True;
    logoWindow = vp->pw_xid;
    logoEnvResult = getenv("PARALLAX_HOME");			    /* See if the PARALLAX_HOME environment variable is set */
    if (NULL == logoEnvResult)					    /* The environment variable isn't set, so choose the... */
    {								    /* ...default location for installing Parallax software. */
      sprintf(logoNameString, "/opt/parallax/include/icon/.plxlogo");
    }
    else							    /* Use the PARALLAX_HOME environment variable and append... */
    {								    /* ...the correct suffix to get to the logo file. */
      sprintf(logoNameString, "%s/include/icon/.plxlogo",
	      logoEnvResult);
    }
    logoFileName =						    /* Look for an X resource indicating where to look for... */
      strdup((char*)defaults_get_string("logoFile", "LogoFile",	    /* ...a logo (in XPM format) to place on screen. */
					logoNameString));
    logoDuration = defaults_get_integer("logoTime", "LogoTime", 5); /* Look for an X resource stating how long logo should... */
    if (0 == logoDuration)					    /* ...remain on screen.  A duration of zero seconds means... */
    {								    /* ...never to show it; a duration of -1 means always to... */
      repaintLogo = False;					    /* show it; a duration of > 0 means show the logo that... */
    } /* ...many seconds. */
    else							    /* Just repaint the logo whenever the video window is... */
      repaintLogo = True;					    /* ...repainted. */
    if (0 != logoDuration)					    /* Is it okay to show the logo? */
    {								    /* Yes, continue preparing to do so */
      logoGCValues.plane_mask = AllPlanes;
      logoGC = XCreateGC(vp->dpy, vp->pw_xid,
			 (GCPlaneMask),
			 &logoGCValues);
      XPlxVideoTag(vp->dpy, vp->pw_xid, logoGC, PLX_GRAPHICS_24);
      logoAttrs.visual = (Visual*)xv_get(vp->public_self, XV_VISUAL);
      logoAttrs.depth = xv_get(vp->public_self, XV_DEPTH);
      logoAttrs.valuemask = (XpmVisual | XpmDepth | XpmColormap);
      logoColormap = XCreateColormap(vp->dpy,
				     vp->pw_xid,
				     logoAttrs.visual,
				     AllocNone);
      logoAttrs.colormap = logoColormap;
      result = XpmReadFileToPixmap(vp->dpy, vp->pw_xid,		    /* Try to create a couple of Pixmaps to use for... */
				   logoFileName,		    /* ...displaying the corporate logo in a video window;... */
				   &logoPixmap,			    /* ...use this call only for the one to be used as a... */
				   &logoShapeMask,		    /* ...clip mask. */
				   &logoAttrs);
      if (0 != result)						    /* Did the attempt to load in an XPM image fail? */
      {
	repaintLogo = False;
      }
      else							    /* The XPM image load succeeded; go ahead and finish... */
      {								    /* ...modifying things. */
	XSetClipMask(vp->dpy, logoGC, logoShapeMask);
      }
    } /* end (if it's okay to show the logo) */
    if ((logoDuration > 0) &&					    /* Repaint the logo for as long as specified by the default */
	(repaintLogo == True))
    {
      logoTimeValue.it_value.tv_sec = logoDuration;
      logoTimeValue.it_value.tv_usec    = 0;
      logoTimeValue.it_interval.tv_usec = 0;
      logoTimeValue.it_interval.tv_sec  = 0;
      notify_set_itimer_func(vp->public_self,			    /* Register a timer function to turn off the logo display */
			     (Notify_func)LogoTimerFunction,
			     ITIMER_REAL, &logoTimeValue,NULL);
    }
  } /* end if (logoAlreadyDone...) */
  /* Determine video configuration and setup if necessary */
  
  if (PlxBoardSetup == NULL)
    PlxBoardSetup = (plx_IO *) XPlxQueryConfig(vp->dpy,vp->pw_xid,vp->pw_gc);
  
  if (PlxBoardSetup->hardware == PLX_UNKNOWN) {
    Xv_VideoStatus = VIDEO_NO_HARDWARE;
    return(XV_ERROR);
  }
  
  /* Skip the channel initialization if only showing JPEG images */
  
  if (vp->flags & VideoJPEGFlag) goto skip;

/* Setup the channels and format parameters from the QueryConfig call */

start:
	a = PlxBoardSetup->inputs;
loop1:
	if (a == NULL) {
		if (vp->flags & VideoChannelFlag) {
			vp->flags &= ~VideoChannelFlag;
			a = PlxBoardSetup->inputs;
			while (a) {
				if ((a->channel & 0xff) == vp->channel) break;
				a = a->next_IO;
			}
			if (a) Xv_VideoStatus = VIDEO_CHANNEL_IN_USE;
			else Xv_VideoStatus = VIDEO_NO_MORE_CHANNELS;
			vp->channel = -1;
			return(XV_ERROR);
		}
		else {
			a = PlxBoardSetup->inputs;
			while (a) {
				if ((a->channel & CHAN_INUSE) == 0) break;
				a = a->next_IO;
			}
			if (a) {
				vp->channel = a->channel;
				goto start;
			}
			else {
				Xv_VideoStatus = VIDEO_NO_MORE_CHANNELS;
				vp->channel = -1;
				return(XV_ERROR);
			}
		}
	}
	if (vp->channel == a->channel) {
		for (b=a->signal_list; b!=NULL; b=b->next_signal) {
			if (b->signal->standard == vp->standard) break;

		}
		if (b == NULL) {
			if (vp->flags & VideoStandardFlag) {
				vp->flags &= ~VideoStandardFlag;
				Xv_VideoStatus = VIDEO_BAD_STANDARD;
				vp->channel = -1;
				return(XV_ERROR);
			}
			else b = a->signal_list;
		}
		for (c=a->format_list; c!=NULL; c=c->next_format) {
			if (c->format == vp->format) break;
		}
		if (c == NULL) {
			if (vp->flags & VideoFormatFlag) {
				vp->flags &= ~VideoFormatFlag;
				Xv_VideoStatus = VIDEO_BAD_FORMAT;
				vp->channel = -1;
				return(XV_ERROR);
			}
			else c = a->format_list;
		}
		for (d=a->type_list; d!=NULL; d=d->next_type) {
			if (d->type == vp->input_type) break;
		}
		if (d == NULL) {
			if (vp->flags & VideoInputFlag) {
				vp->flags &= ~VideoInputFlag;
				Xv_VideoStatus = VIDEO_BAD_INPUT_TYPE;
				vp->channel = -1;
				return(XV_ERROR);
			}
			else d = a->type_list;
		}
	}
	else {
		a = a->next_IO;
		goto loop1;
	}
done:
	vp->channel      = a->channel;
#ifndef SHARE_CHANNELS
	a->channel      |= CHAN_INUSE;
#endif
	vp->standard     = b->signal->standard;
	vp->format       = c->format;
	vp->input_type   = d->type;

/* Set the desired input parameters we want (and should get) */

	XPlxVideoInputSelect(vp->dpy,vp->pw_xid,vp->pw_gc,vp->channel,
		vp->standard,vp->format,vp->input_type);

/* See what we really got off the board */

	e = (plx_signal *) XPlxQueryVideo(vp->dpy,vp->pw_xid,vp->pw_gc);

/* If not synced up, try all the combinations we can think of */

	if (e->sync_ok) goto success;

	f = 0;
 top:

/* If not synced up, test for NTSC composite signal */

	vp->standard = PLX_NTSC;
	vp->format   = PLX_COMP;
	XPlxVideoInputSelect(vp->dpy,vp->pw_xid,vp->pw_gc,vp->channel,
						 vp->standard,vp->format,vp->input_type);
	e = (plx_signal *) XPlxQueryVideo(vp->dpy,vp->pw_xid,vp->pw_gc);
	if (e->sync_ok) goto success;

/* If not synced up, test for PAL composite signal */

	vp->standard = PLX_PAL;
	XPlxVideoInputSelect(vp->dpy,vp->pw_xid,vp->pw_gc,vp->channel,
						 vp->standard,vp->format,vp->input_type);
	e = (plx_signal *) XPlxQueryVideo(vp->dpy,vp->pw_xid,vp->pw_gc);
	if (e->sync_ok) goto success;

/* If not synced up, test for SECAM composite signal */

	vp->standard = PLX_SECAM;
	XPlxVideoInputSelect(vp->dpy,vp->pw_xid,vp->pw_gc,vp->channel,
						 vp->standard,vp->format,vp->input_type);
	e = (plx_signal *) XPlxQueryVideo(vp->dpy,vp->pw_xid,vp->pw_gc);
	if (e->sync_ok) goto success;

/* If not synced up, test for NTSC YC signal */

	vp->standard = PLX_NTSC;
	vp->format   = PLX_YC;
	XPlxVideoInputSelect(vp->dpy,vp->pw_xid,vp->pw_gc,vp->channel,
						 vp->standard,vp->format,vp->input_type);
	e = (plx_signal *) XPlxQueryVideo(vp->dpy,vp->pw_xid,vp->pw_gc);
	if (e->sync_ok) goto success;

/* If not synced up, test for PAL YC signal */

	vp->standard = PLX_PAL;
	XPlxVideoInputSelect(vp->dpy,vp->pw_xid,vp->pw_gc,vp->channel,
						 vp->standard,vp->format,vp->input_type);
	e = (plx_signal *) XPlxQueryVideo(vp->dpy,vp->pw_xid,vp->pw_gc);
	if (e->sync_ok) goto success;

/* If not synced up, test for SECAM YC signal */

	vp->standard = PLX_SECAM;
	XPlxVideoInputSelect(vp->dpy,vp->pw_xid,vp->pw_gc,vp->channel,
						 vp->standard,vp->format,vp->input_type);
	e = (plx_signal *) XPlxQueryVideo(vp->dpy,vp->pw_xid,vp->pw_gc);
	if (e->sync_ok) goto success;

/* If not synced up, test for NTSC RGB */

	vp->standard = PLX_NTSC;
	vp->format   = PLX_RGB;
	XPlxVideoInputSelect(vp->dpy,vp->pw_xid,vp->pw_gc,vp->channel,
						 vp->standard,vp->format,vp->input_type);
	e = (plx_signal *) XPlxQueryVideo(vp->dpy,vp->pw_xid,vp->pw_gc);
	if (e->sync_ok) goto success;

/* If not synced up, test for PAL RGB signal */

	vp->standard = PLX_PAL;
	XPlxVideoInputSelect(vp->dpy,vp->pw_xid,vp->pw_gc,vp->channel,
						 vp->standard,vp->format,vp->input_type);
	e = (plx_signal *) XPlxQueryVideo(vp->dpy,vp->pw_xid,vp->pw_gc);
	if (e->sync_ok) goto success;

/* If not synced up, test for SECAM RGB signal */

	vp->standard = PLX_SECAM;
	XPlxVideoInputSelect(vp->dpy,vp->pw_xid,vp->pw_gc,vp->channel,
						 vp->standard,vp->format,vp->input_type);
	e = (plx_signal *) XPlxQueryVideo(vp->dpy,vp->pw_xid,vp->pw_gc);
	if (e->sync_ok) goto success;

	switch (f) {
	case 0:
		if (vp->channel == PLX_INPUT_0) vp->channel = PLX_INPUT_1;
		else vp->channel = PLX_INPUT_0;
		f++; goto top;
		break;
	default:
		Xv_VideoStatus = VIDEO_NO_SYNC_DETECTED;
		return(XV_ERROR);
		break;
	}
 success:
	if (e->interlaced) vp->flags |= VideoLacedFlag;
	if (e->sync_ok)    vp->flags |= VideoSyncFlag;
#ifdef ALTERNATE
	vp->vw		 = e->w;
	vp->width    = e->w;
	vp->vh		 = (e->w * 3) / 4;
#ifdef CLIENTVV
	vp->height   = (e->w * 3) / 4;
#else
	vp->height   = e->h;
#endif
	vp->interval = e->h - vp->vh;
	vp->vy       = vp->interval;
#else
	vp->vy       = e->b;
	vp->vw		 = e->w;
	vp->vh		 = e->h - e->b;
	vp->width    = e->w;
#ifdef CLIENTVV
	vp->height   = e->h - e->b;
#else
	vp->height   = e->h;
#endif
	vp->interval = e->b;
#endif
	vp->standard = e->standard;

skip:

/* Initialize the tag bits for drawing */

	XPlxVideoTag(vp->dpy,vp->pw_xid,vp->pw_gc,vp->tagbits);

/* Interpose on the frame to notice WIN_RESIZE events */

	win				= (Xv_Window) xv_get(vp->public_self,XV_OWNER);
	xv_set(win,
		WIN_CONSUME_EVENTS,		WIN_CONFIGURE_REQUEST,WIN_VISIBILITY_NOTIFY,NULL,
		XV_KEY_DATA,			VIDEO_KEY,				vp,
		NULL);
	if (xv_get(win,XV_KEY_DATA,WIN_KEY) == 0) {
		(void) notify_interpose_event_func(win,frame_interposer,NOTIFY_SAFE);
		xv_set(win,XV_KEY_DATA,WIN_KEY,TRUE,NULL);
	}

/* Setup the JPEG quantization tables */

	if (PlxBoardSetup->hardware == PLX_XVIDEO_24) {
		vp->QSize = MakeQTables(vp->QFactor,&(vp->QTable));
		XPlxPutTable(vp->dpy,vp->pw_xid,vp->pw_gc,vp->QTable,vp->QSize,FALSE);
	}

/* If shared memory requested, allocate */

	if (((vp->flags & VideoJPEGFlag) == 0) && (vp->flags & VideoUseShmFlag)) {
		vp->ImageCache = CreateImageCache(vp->dpy,vp->dw,vp->dh);
		if (vp->ImageCache == NULL) {
			Xv_VideoStatus = VIDEO_UNKNOWN;
			return(XV_ERROR);
		}
	}

/* If any visual parameters changed, inform the board */

	if (vp->flags & VideoValueFlag) {
		vp->flags &= ~VideoValueFlag;
		XPlxVideoValueLoad(vp->dpy,vp->pw_xid,vp->pw_gc,vp->saturation,
			vp->contrast,vp->hue,vp->brightness);
	}
	return(XV_OK);
}

static Xv_opaque VideoInSet(video_public,avlist)
VideoInPublic *video_public;
Attr_avlist   avlist;
{
	XPlxCImage *jpeg;
	XImage *image;
	Pixrect *p;
	VideoInPrivate *video_private = VIDEO_IN_PRIVATE(video_public);
	Attr_attribute *attr;
	int save,x,w;

	Xv_VideoStatus = VIDEO_OKAY;

	for (attr = avlist; *attr; attr = attr_next(attr)) {
		switch ((int) attr[0]) {
		case VIDEO_FAST_FRAME:
			XPlxVideoSqueezeStill(video_private->dpy,video_private->pw_xid,
				video_private->pw_gc,video_private->vx,video_private->vy,
				video_private->vw,video_private->vh,video_private->dx,
				video_private->dy,video_private->dw,video_private->dh);
			break;
		case VIDEO_LIVE:
			if (video_private->flags & VideoJPEGFlag) break;
			if (attr[1]) {
				video_private->flags |= VideoLiveFlag;
				video_private->flags &= ~VideoJPEGFlag;
			}
			else video_private->flags &= ~VideoLiveFlag;
			video_private->flags |= VideoRepaintFlag;
			break;
		case VIDEO_SCALE:
			if (attr[1]) {
				video_private->flags |= VideoScaleFlag;
				video_private->flags &= ~VideoSqueezeFlag;
			}
			else {
				video_private->flags &= ~VideoScaleFlag;
			}
			video_private->flags |= VideoRepaintFlag;
			break;
		case VIDEO_SQUEEZE:
			if (attr[1]) {
				video_private->flags |= VideoSqueezeFlag;
				video_private->flags &= ~VideoScaleFlag;
			}
			else {
				video_private->flags &= ~VideoSqueezeFlag;
			}
			video_private->flags |= VideoRepaintFlag;
			break;
		case VIDEO_JPEG_ONLY:
			if (attr[1]) {
				video_private->flags |=  VideoJPEGFlag;
				video_private->flags &= ~VideoLiveFlag;
			}
			else {
				video_private->flags &= ~VideoJPEGFlag;
			}
			video_private->flags |= VideoRepaintFlag;
			break;
		case VIDEO_NEW_FRAME:
			video_private->flags |= VideoRepaintFlag;
			break;
		case VIDEO_TRACK_WIN:
			if (attr[1]) video_private->flags |=  VideoTrackWinFlag;
			else         video_private->flags &= ~VideoTrackWinFlag;
			break;
		case VIDEO_JPEG_ZOOM:
			if (attr[1]) video_private->flags |=  VideoJPEGZoomFlag;
			else         video_private->flags &= ~VideoJPEGZoomFlag;
			break;
		case VIDEO_SAVE_STILL:
/* Saved still can't be done correctly under OpenWindows 2.0 */
			if (XViewVersionNumber == 2) break;
			if (attr[1]) video_private->flags |=  VideoSaveStillFlag;
			else {
				video_private->flags &= ~VideoSaveStillFlag;
				if (video_private->SavedStill) {
					XDestroyImage(video_private->SavedStill);
					video_private->SavedStill = NULL;
				}
			}
			break;
		case VIDEO_USE_SHM:
			if (attr[1]) {
				if (video_private->flags & VideoUseShmFlag) break;
				video_private->flags |=  VideoUseShmFlag;
				if (video_private->flags & VideoCreateFlag)   break;
				if (video_private->flags & VideoJPEGFlag) break;
				video_private->ImageCache =
					CreateImageCache(video_private->dpy,video_private->dw,video_private->dh);
				if (video_private->ImageCache == NULL) {
					xv_error((Xv_object) video_public,
							 ERROR_STRING,		"Unable to use shared memory!",
							 ERROR_BAD_VALUE,	attr,
							 ERROR_PKG,			VIDEO_IN,
							 NULL);
				}
			}
			else {
				video_private->flags &= ~VideoUseShmFlag;
				if (video_private->ImageCache) {
					DestroyImageCache(video_private->dpy,video_private->ImageCache);
					video_private->ImageCache = NULL;
				}
			}
			break;
		case VIDEO_NORMAL_ASPECT:
			if (attr[1]) {
				video_private->flags |= VideoAspectFlag;
				video_private->flags |= VideoRepaintFlag;
			}
			else video_private->flags &= ~VideoAspectFlag;
			break;
		case VIDEO_USER0:
			if (video_private->flags & VideoCreateFlag) break;
			XPlxVideoValueLoad(video_private->dpy,video_private->pw_xid,
				video_private->pw_gc,video_private->saturation,
				video_private->contrast,video_private->hue,
				video_private->brightness);
			video_private->flags &= ~VideoValueFlag;
			XPlxVideoValueSave(video_private->dpy,video_private->pw_xid,
				video_private->pw_gc,USER_0);
			break;
		case VIDEO_USER1:
			if (video_private->flags & VideoCreateFlag) break;
			if (PlxBoardSetup->hardware != PLX_XVIDEO_24) 
				if (PlxBoardSetup->hardware != PLX_XVIDEO_8) break;
			XPlxVideoValueLoad(video_private->dpy,video_private->pw_xid,
				video_private->pw_gc,video_private->saturation,
				video_private->contrast,video_private->hue,
				video_private->brightness);
			video_private->flags &= ~VideoValueFlag;
			XPlxVideoValueSave(video_private->dpy,video_private->pw_xid,
				video_private->pw_gc,USER_1);
			break;
		case VIDEO_CHANNEL:
			save = TranslateChannel((int) attr[1]);
			if (save == -1) {
				xv_error((Xv_object) video_public,
					ERROR_STRING,		"Bad VIDEO_CHANNEL value supplied",
					ERROR_BAD_VALUE,	attr,
					ERROR_PKG,			VIDEO_IN,
					NULL);
				break;
			}
			video_private->p_channel = video_private->channel;
			video_private->channel   = save;
			if (video_private->p_channel == video_private->channel) break;
			if (video_private->flags & VideoCreateFlag)
				video_private->flags |= VideoChannelFlag;
			else video_private->flags |= VideoIOFlag;
			break;
		case VIDEO_STANDARD:
			save = TranslateStandard((int) attr[1]);
			if (save == -1) {
				xv_error((Xv_object) video_public,
					ERROR_STRING,		"Bad VIDEO_STANDARD value supplied",
					ERROR_BAD_VALUE,	attr,
					ERROR_PKG,			VIDEO_IN,
					NULL);
				break;
			}
			video_private->p_standard = video_private->standard;
			video_private->standard = save;
			if (video_private->p_standard == video_private->standard) break;
			if (video_private->flags & VideoCreateFlag)
				video_private->flags |= VideoStandardFlag;
			else video_private->flags |= VideoIOFlag;
			break;
		case VIDEO_FORMAT:
			save = TranslateFormat((int) attr[1]);
			if (save == -1) {
				xv_error((Xv_object) video_public,
					ERROR_STRING,		"Bad VIDEO_FORMAT value supplied",
					ERROR_BAD_VALUE,	attr,
					ERROR_PKG,			VIDEO_IN,
					NULL);
				break;
			}
			video_private->p_format = video_private->format;
			video_private->format = save;
			if (video_private->p_format == video_private->format) break;
			if (video_private->flags & VideoCreateFlag)
				video_private->flags |= VideoFormatFlag;
			else video_private->flags |= VideoIOFlag;
			break;
		case VIDEO_INPUT_TYPE:
			save = TranslateType((int) attr[1]);
			if (save == -1) {
				xv_error((Xv_object) video_public,
					ERROR_STRING,		"Bad VIDEO_INPUT_TYPE value supplied",
					ERROR_BAD_VALUE,	attr,
					ERROR_PKG,			VIDEO_IN,
					NULL);
				break;
			}
			video_private->p_input_type = video_private->input_type;
			video_private->input_type = save;
			if (video_private->p_input_type == video_private->input_type) break;
			if (video_private->flags & VideoCreateFlag)
				video_private->flags |= VideoInputFlag;
			else video_private->flags |= VideoIOFlag;
			break;
		case VIDEO_SATURATION:
			if (!CheckValue((int) attr[1])) {
				xv_error((Xv_object) video_public,
					ERROR_STRING,		"Bad VIDEO_SATURATION value supplied",
					ERROR_BAD_VALUE,	attr,
					ERROR_PKG,			VIDEO_IN,
					NULL);
				break;
			}
			video_private->saturation = (int) attr[1];
			video_private->flags |= VideoValueFlag;
			break;
		case VIDEO_HUE:
			if (!CheckValue((int) attr[1])) {
				xv_error((Xv_object) video_public,
					ERROR_STRING,		"Bad VIDEO_HUE value supplied",
					ERROR_BAD_VALUE,	attr,
					ERROR_PKG,			VIDEO_IN,
					NULL);
				break;
			}
			video_private->hue = (int) attr[1];
			video_private->flags |= VideoValueFlag;
			break;
		case VIDEO_CONTRAST:
			if (!CheckValue((int) attr[1])) {
				xv_error((Xv_object) video_public,
					ERROR_STRING,		"Bad VIDEO_CONTRAST value supplied",
					ERROR_BAD_VALUE,	attr,
					ERROR_PKG,			VIDEO_IN,
					NULL);
				break;
			}
			video_private->contrast = (int) attr[1];
			video_private->flags |= VideoValueFlag;
			break;
		case VIDEO_BRIGHTNESS:
			if (!CheckValue((int) attr[1])) {
				xv_error((Xv_object) video_public,
					ERROR_STRING,		"Bad VIDEO_BRIGHTNESS value supplied",
					ERROR_BAD_VALUE,	attr,
					ERROR_PKG,			VIDEO_IN,
					NULL);
				break;
			}
			video_private->brightness = (int) attr[1];
			video_private->flags |= VideoValueFlag;
			break;
		case VIDEO_X:
			video_private->vx = (int) attr[1];
			video_private->flags |= VideoSizeFlag;
			break;
		case VIDEO_Y:
			video_private->vy = (int) attr[1];
			video_private->flags |= VideoSizeFlag;
			break;
		case VIDEO_WIDTH:
			if (video_private->flags & VideoScaleFlag) {
				video_private->vw = (int) attr[1];
				video_private->flags |= VideoSizeFlag;
			}
			break;
		case VIDEO_HEIGHT:
			if (video_private->flags & VideoScaleFlag) {
				video_private->vh = (int) attr[1];
				video_private->flags |= VideoSizeFlag;
			}
			break;
		case VIDEO_GRAPHICS:
			save = TranslateGraphics((int) attr[1]);
			if (save == -1) {
				xv_error((Xv_object) video_public,
					ERROR_STRING,		"Bad VIDEO_GRAPHICS value supplied",
					ERROR_BAD_VALUE,	attr,
					ERROR_PKG,			VIDEO_IN,
					NULL);
				break;
			}
			video_private->tagbits = save;
			break;
		case VIDEO_PIXRECT:
			if (video_private->flags & VideoLiveFlag) break;
			p = (Pixrect *) attr[1];
			image = PR2XI(video_private,p);
#ifdef TRACE
			fprintf(stderr,"TRACE: XPutImage(sx=%d,sy=%d,vx=%d,vy=%d,vw=%d,vh=%d)\n",
				0,0,video_private->dx,video_private->dy,image->width,image->height);
#endif
			if (XViewVersionNumber == 2) {
				XPlxPutImage(video_private->dpy,video_private->pw_xid,video_private->pw_gc,
							 image,0,0,video_private->dx,video_private->dy,image->width,
							 image->height);
			}
			else {
				XPutImage(video_private->dpy,video_private->pw_xid,video_private->pw_gc,
							 image,0,0,video_private->dx,video_private->dy,image->width,
							 image->height);
			}
			XDestroyImage(image);
			break;
		case VIDEO_XIMAGE:
			if (video_private->flags & VideoLiveFlag) break;
			image = (XImage *) attr[1];
#ifdef TRACE
			fprintf(stderr,"TRACE: XPutImage(sx=%d,sy=%d,vx=%d,vy=%d,vw=%d,vh=%d)\n",
				0,0,video_private->dx,video_private->dy,image->width,image->height);
#endif
			if (XViewVersionNumber == 2) {
				XPlxPutImage(video_private->dpy,video_private->pw_xid,video_private->pw_gc,
							 image,0,0,video_private->dx,video_private->dy,image->width,
							 image->height);
			}
			else {
				XPutImage(video_private->dpy,video_private->pw_xid,video_private->pw_gc,
							 image,0,0,video_private->dx,video_private->dy,image->width,
							 image->height);
			}
			break;
		case VIDEO_EVENT_PROC:
			xv_set(video_private->pw,WIN_EVENT_PROC,attr[1],NULL);
			break;
		case VIDEO_Q_FACTOR:
			video_private->QFactor = (int) attr[1];
			if (video_private->flags & VideoCreateFlag) break;
			if (PlxBoardSetup->hardware == PLX_XVIDEO_24) {
				video_private->QSize = MakeQTables(video_private->QFactor,&(video_private->QTable));
				XPlxPutTable(video_private->dpy,video_private->pw_xid,video_private->pw_gc,
							 video_private->QTable,video_private->QSize,(video_private->flags & VideoGCIsOut));
			}
			break;
		case VIDEO_JPEG_IMAGE:
			if (video_private->flags & VideoLiveFlag) break;
			jpeg = (XPlxCImage *) attr[1];
			if ((video_private->flags & VideoGCIsOut) == 0) {
				video_private->flags |= VideoGCIsOut;
				XPlxPutTable(video_private->dpy,video_private->pw_xid,video_private->pw_gc,
							 video_private->QTable,video_private->QSize,TRUE);
			}
#ifdef TRACE
			fprintf(stderr,"TRACE: XPlxPutCImage(dx=%d,dy=%d,dw=%d,dh=%d)\n",
				video_private->dx,video_private->dy,video_private->dw,video_private->dh);
#endif
			switch (video_private->conf_size) {
			case 1:
				x = video_private->dx;
				w = video_private->dw;
				break;
			default:
/* This is necessary to account for using repaints to work correctly when
   compressing and decompressing to different parts of the same window
*/
				if ((video_private->tagbits == PLX_VIDEO_OVR) && 
					(video_private->flags & VideoFirstDeComp))
					XPlxVideoTag(video_private->dpy,video_private->pw_xid,
								 video_private->pw_gc,PLX_VIDEO);
				x = video_private->dx + video_private->dw / video_private->conf_size;
				w = video_private->dw - video_private->dw / video_private->conf_size;
				break;
			}
			if (video_private->flags & VideoUseShmFlag) {
				XPlxShmPutCImage(video_private->dpy,video_private->pw_xid,
					video_private->pw_gc,jpeg,0,0,jpeg->width,jpeg->height,
/* If the video conferencing paramter is set, revise video region
					video_private->dx,video_private->dy,video_private->dw,
*/
					x,video_private->dy,w,
					video_private->dh,(video_private->flags & VideoSqueezeFlag),False);
			}
			else {
				XPlxPutCImage(video_private->dpy,video_private->pw_xid,
					video_private->pw_gc,jpeg,0,0,jpeg->width,jpeg->height,
/* If the video conferencing paramter is set, revise video region
					video_private->dx,video_private->dy,video_private->dw,
*/
					x,video_private->dy,w,
					video_private->dh,(video_private->flags & VideoSqueezeFlag));
			}
			if (video_private->flags & VideoFirstDeComp) {
#ifdef TRACE
				fprintf(stderr,"TRACE: XPlxVideoTag(bits=%d)\n",video_private->tagbits);
#endif
				video_private->flags &= ~VideoFirstDeComp;
				video_private->tagbits = PLX_VIDEO_OVR;
				XPlxVideoTag(video_private->dpy,video_private->pw_xid,
					video_private->pw_gc,video_private->tagbits);
			}
			break;
		case VIDEO_CONF_SIZE:
			video_private->conf_size   = (int) attr[1];
			break;
		case VIDEO_COLOR:
			if (attr[1]) {
				XPlxSetVideoParameterString(video_private->dpy,video_private->pw_xid,
						video_private->pw_gc,"input_config COLOR") ;
			}
			else {
				XPlxSetVideoParameterString(video_private->dpy,video_private->pw_xid,
						video_private->pw_gc,"input_config COLORKILL") ;
			}
			break;
		case VIDEO_SHM_PTR:
			DestroyImageCache(video_private->dpy,(JPEGImage *) attr[1]);
			break;
		case XV_END_CREATE:
			video_private->flags &= ~VideoCreateFlag;
			return(FinishInit(video_private));
			break;

/* These attribute(s) are not settable */

		case VIDEO_FACTORY_RESET:
			xv_error((Xv_object) video_public,
				ERROR_STRING,		"Factory defaults are read only",
				ERROR_CANNOT_SET,	attr,
				ERROR_PKG,			VIDEO_IN,
				NULL);
			break;
		case VIDEO_SYNC:
			xv_error((Xv_object) video_public,
				ERROR_STRING,		"Video sync flag is read only!",
				ERROR_CANNOT_SET,	attr,
				ERROR_PKG,			VIDEO_IN,
				NULL);
			break;
		case VIDEO_INTERLACED:
			xv_error((Xv_object) video_public,
				ERROR_STRING,		"Video interlaced flag is read only!",
				ERROR_CANNOT_SET,	attr,
				ERROR_PKG,			VIDEO_IN,
				NULL);
			break;
		case VIDEO_HARDWARE:
			xv_error((Xv_object) video_public,
				ERROR_STRING,		"Video hardware type is read only!",
				ERROR_CANNOT_SET,	attr,
				ERROR_PKG,			VIDEO_IN,
				NULL);
			break;
		case VIDEO_SIGNAL_WIDTH:
			xv_error((Xv_object) video_public,
				ERROR_STRING,		"Video signal width is read only!",
				ERROR_CANNOT_SET,	attr,
				ERROR_PKG,			VIDEO_IN,
				NULL);
			break;
		case VIDEO_SIGNAL_HEIGHT:
			xv_error((Xv_object) video_public,
				ERROR_STRING,		"Video signal height is read only!",
				ERROR_CANNOT_SET,	attr,
				ERROR_PKG,			VIDEO_IN,
				NULL);
			break;
		case VIDEO_INTERVAL:
			xv_error((Xv_object) video_public,
				ERROR_STRING,		"Video signal interval is read only!",
				ERROR_CANNOT_SET,	attr,
				ERROR_PKG,			VIDEO_IN,
				NULL);
			break;
		case VIDEO_EXPOSED:
			xv_error((Xv_object) video_public,
				ERROR_STRING,		"Video exposed flag is read only!",
				ERROR_CANNOT_SET,	attr,
				ERROR_PKG,			VIDEO_IN,
				NULL);
			break;
		case VIDEO_SHM_DONE:
			xv_error((Xv_object) video_public,
				ERROR_STRING,		"Video SHM done is read only!",
				ERROR_CANNOT_SET,	attr,
				ERROR_PKG,			VIDEO_IN,
				NULL);
			break;
		case VIDEO_GC:
			xv_error((Xv_object) video_public,
				ERROR_STRING,		"Video GC is read only!",
				ERROR_CANNOT_SET,	attr,
				ERROR_PKG,			VIDEO_IN,
				NULL);
			break;
		case VIDEO_QTABLE:
			xv_error((Xv_object) video_public,
				ERROR_STRING,		"Video QTABLE ptr is read only!",
				ERROR_CANNOT_SET,	attr,
				ERROR_PKG,			VIDEO_IN,
				NULL);
			break;
		default:
			xv_check_bad_attr(VIDEO_IN,attr[0]);
			break;
		}
	}

/* If we are still in the create pass, then skip the rest */

	if (video_private->flags & VideoCreateFlag) return(XV_OK);

/* If any visual parameters changed, inform the board */

	if (video_private->flags & VideoValueFlag) {
		video_private->flags &= ~VideoValueFlag;
#ifdef TRACE
		fprintf(stderr,"TRACE: XPlxVideoValueLoad(s=%d,c=%d,h=%d,b=%d)\n",
			video_private->saturation,video_private->contrast,
			video_private->hue,video_private->brightness);
#endif
		XPlxVideoValueLoad(video_private->dpy,video_private->pw_xid,
			video_private->pw_gc,video_private->saturation,
			video_private->contrast,video_private->hue,
			video_private->brightness);
	}

/* If the input parameters have changed, try to do it */

	if (video_private->flags & VideoIOFlag) {
		video_private->flags &= ~VideoIOFlag;
		return(AdjustBoardIO(video_private));
	}

/* If any parameter has changed that requires a redislay, do it */

	if (video_private->flags & VideoSizeFlag) {
		video_private->flags &= ~VideoSizeFlag;
/*
		VideoResize(video_public,video_private->vw,video_private->vh);
*/
		VideoRepaint(video_public,video_private->pw,(Rectlist *)1);
	}
	if (video_private->flags & VideoRepaintFlag) {
		video_private->flags &= ~VideoRepaintFlag;
		VideoRepaint(video_public,video_private->pw,(Rectlist *)1);
	}

	return(XV_OK);
}

static Xv_opaque VideoInGet(video_public,status,attr,avlist)
VideoInPublic  *video_public;
int 		  *status;
Attr_attribute attr;
Attr_avlist    avlist;
{
	int w;
	XEvent event;
	XPlxCImage *jpeg;
	XImage *image;
	Pixrect *p;
	plx_signal *s;
	VideoInPrivate *video_private = VIDEO_IN_PRIVATE(video_public);

	Xv_VideoStatus = VIDEO_OKAY;

	switch ((int) attr) {
	case VIDEO_LIVE:
		if (video_private->flags & VideoLiveFlag) return(TRUE);
		else return(FALSE);
		break;
	case VIDEO_SCALE:
		if (video_private->flags & VideoScaleFlag) return(TRUE);
		else return(FALSE);
		break;
	case VIDEO_SQUEEZE:
		if (video_private->flags & VideoSqueezeFlag) return(TRUE);
		else return(FALSE);
		break;
	case VIDEO_JPEG_ONLY:
		if (video_private->flags & VideoJPEGFlag) return(TRUE);
		else return(FALSE);
		break;
	case VIDEO_JPEG_ZOOM:
		if (video_private->flags & VideoJPEGZoomFlag) return(TRUE);
		else return(FALSE);
		break;
	case VIDEO_TRACK_WIN:
		if (video_private->flags & VideoTrackWinFlag) return(TRUE);
		else return(FALSE);
		break;
	case VIDEO_NORMAL_ASPECT:
		if (video_private->flags & VideoAspectFlag) return(TRUE);
		else return(FALSE);
		break;
	case VIDEO_SAVE_STILL:
		if (video_private->flags & VideoSaveStillFlag) return(TRUE);
		else return(FALSE);
		break;
	case VIDEO_FACTORY_RESET:
		XPlxVideoValueRecall(video_private->dpy,video_private->pw_xid,
			video_private->pw_gc,FACTORY);
		XPlxVideoValueQuery(video_private->dpy,video_private->pw_xid,
			video_private->pw_gc,&(video_private->saturation),
			&(video_private->contrast),&(video_private->hue),
			&(video_private->brightness));
		return(TRUE);
		break;
	case VIDEO_USER0:
		XPlxVideoValueRecall(video_private->dpy,video_private->pw_xid,
			video_private->pw_gc,USER_0);
		XPlxVideoValueQuery(video_private->dpy,video_private->pw_xid,
			video_private->pw_gc,&(video_private->saturation),
			&(video_private->contrast),&(video_private->hue),
			&(video_private->brightness));
		return(TRUE);
		break;
	case VIDEO_USER1:
		if (PlxBoardSetup->hardware != PLX_XVIDEO_24) 
			if (PlxBoardSetup->hardware != PLX_XVIDEO_8) break;
		XPlxVideoValueRecall(video_private->dpy,video_private->pw_xid,
			video_private->pw_gc,USER_1);
		XPlxVideoValueQuery(video_private->dpy,video_private->pw_xid,
			video_private->pw_gc,&(video_private->saturation),
			&(video_private->contrast),&(video_private->hue),
			&(video_private->brightness));
		return(TRUE);
		break;
	case VIDEO_STANDARD:
		return(UnTranslateStandard(video_private->standard));
		break;
	case VIDEO_FORMAT:
		return(UnTranslateFormat(video_private->format));
		break;
	case VIDEO_INPUT_TYPE:
		return(UnTranslateType(video_private->input_type));
		break;
	case VIDEO_CHANNEL:
		return(UnTranslateChannel(video_private->channel));
		break;
	case VIDEO_SATURATION:
		return(video_private->saturation);
		break;
	case VIDEO_HUE:
		return(video_private->hue);
		break;
	case VIDEO_CONTRAST:
		return(video_private->contrast);
		break;
	case VIDEO_BRIGHTNESS:
		return(video_private->brightness);
		break;
	case VIDEO_X:
		return(video_private->vx);
		break;
	case VIDEO_Y:
		return(video_private->vy);
		break;
	case VIDEO_WIDTH:
		return(video_private->vw);
		break;
	case VIDEO_SIGNAL_WIDTH:
		return(video_private->width);
		break;
	case VIDEO_HEIGHT:
		return(video_private->vh);
		break;
	case VIDEO_SIGNAL_HEIGHT:
		return(video_private->height);
		break;
	case VIDEO_INTERVAL:
		return(video_private->interval);
		break;
	case VIDEO_EXPOSED:
		if (video_private->flags & VideoCalledFlag) return(TRUE);
		else return(FALSE);
		break;
	case VIDEO_USE_SHM:
		if (video_private->flags & VideoUseShmFlag) return(TRUE);
		else return(FALSE);
		break;
	case VIDEO_SHM_DONE:
		return(XCheckTypedWindowEvent(video_private->dpy,video_private->pw_xid,
			   XPlxShmGetEventBase(video_private->dpy)+ShmCompletion,&event));
		break;
	case VIDEO_GRAPHICS:
		return(UnTranslateGraphics(video_private->tagbits));
		break;
	case VIDEO_SYNC:
		s = (plx_signal *) XPlxQueryVideo(video_private->dpy,
				video_private->pw_xid,video_private->pw_gc);
		if (s->interlaced) video_private->flags |= VideoLacedFlag;
		else video_private->flags &= ~VideoLacedFlag;
		if (s->sync_ok) video_private->flags |= VideoSyncFlag;
		else video_private->flags &= ~VideoSyncFlag;
		if (video_private->flags & VideoSyncFlag) return(TRUE);
		else return(FALSE);
		break;
	case VIDEO_INTERLACED:
		s = (plx_signal *) XPlxQueryVideo(video_private->dpy,
				video_private->pw_xid,video_private->pw_gc);
		if (s->interlaced) video_private->flags |= VideoLacedFlag;
		else video_private->flags &= ~VideoLacedFlag;
		if (s->sync_ok) video_private->flags |= VideoSyncFlag;
		else video_private->flags &= ~VideoSyncFlag;
		if (video_private->flags & VideoLacedFlag) return(TRUE);
		else return(FALSE);
		break;
	case VIDEO_HARDWARE:
		return(UnTranslateHardware(PlxBoardSetup->hardware));
		break;
	case VIDEO_PIXRECT:
		if (video_private->flags & VideoLiveFlag) return(0);
#ifdef TRACE
			fprintf(stderr,"TRACE: XGetImage(vx=%d,vy=%d,vw=%d,vh=%d)\n",
				video_private->dx,video_private->dy,video_private->dw,video_private->dh);
#endif
		if (XViewVersionNumber == 2) {
			image = (XImage *) XPlxGetImage(video_private->dpy,video_private->pw_xid,
							0,0,video_private->dw,video_private->dh,AllPlanes,ZPixmap);
		}
		else {
			image = (XImage *) XGetImage(video_private->dpy,video_private->pw_xid,
/* FIXUP: Why -2 for the width and height? */
					 video_private->dx,video_private->dy,video_private->dw-2,
					 video_private->dh-2,AllPlanes,ZPixmap);
		}
		p = XI2PR(image);
		image->data = (char *) NULL;
		XDestroyImage(image);
		return((Xv_opaque) p);
	case VIDEO_XIMAGE:
		if (video_private->flags & VideoLiveFlag) return(0);
#ifdef TRACE
			fprintf(stderr,"TRACE: XGetImage(vx=%d,vy=%d,vw=%d,vh=%d)\n",
				video_private->dx,video_private->vy,video_private->dw,video_private->dh);
#endif
		if (XViewVersionNumber == 2) {
			image = (XImage *) XPlxGetImage(video_private->dpy,video_private->pw_xid,
							0,0,video_private->dw,video_private->dh,AllPlanes,ZPixmap);
		}
		else {
			image = (XImage *) XGetImage(video_private->dpy,video_private->pw_xid,
/* FIXUP: Why -2 for the width and height? */
					 video_private->dx,video_private->dy,video_private->dw-2,
					 video_private->dh-2,AllPlanes,ZPixmap);
		}
		return((Xv_opaque) image);
		break;
	case VIDEO_EVENT_PROC:
		return((Xv_opaque) xv_get(video_private->pw,WIN_EVENT_PROC));
		break;
	case VIDEO_Q_FACTOR:
		return((Xv_opaque) video_private->QFactor);
		break;
	case VIDEO_JPEG_IMAGE:
		if (video_private->flags & VideoLiveFlag)   return(0);
		if (video_private->flags & VideoGCIsOut) {
			video_private->flags &= ~VideoGCIsOut;
			XPlxPutTable(video_private->dpy,video_private->pw_xid,video_private->pw_gc,
						 video_private->QTable,video_private->QSize,FALSE);
		}
#ifdef TRACE
		fprintf(stderr,"TRACE: XPlxGetCImage(dx=%d,dy=%d,dw=%d,dh=%d)\n",
			video_private->dx,video_private->dy,video_private->dw,video_private->dh);
#endif
		if (video_private->flags & VideoUseShmFlag) {
			if (video_private->ImageCache == NULL) return(0);
/* If users request jpeg zoom, give it to them (really slow) */
			if (video_private->flags & VideoJPEGZoomFlag) {
				video_private->ImageCache->width  = video_private->vw;
				video_private->ImageCache->height = video_private->vh;
			}
			else {
				video_private->ImageCache->width  = video_private->dw;
				video_private->ImageCache->height = video_private->dh;
			}
			switch (video_private->conf_size) {
			case 1:
				w = video_private->dw;
				break;
			default:
				w = video_private->dw - video_private->dw / video_private->conf_size;
				video_private->ImageCache->width = w;
				break;
			}
			XPlxShmGetCImage(video_private->dpy,
					 video_private->pw_xid,
					 video_private->pw_gc,
					 (XPlxCImage*)video_private->ImageCache,
					 video_private->dx,
					 video_private->dy,
/* If the video conferencing paramter is set, revise video region
   video_private->dw,video_private->dh);
*/
					 w,video_private->dh);
			jpeg = (XPlxCImage *) video_private->ImageCache;
		}
		else {
/* If users request jpeg zoom, give it to them (really slow) */
			if (video_private->flags & VideoJPEGZoomFlag) {
				jpeg = (XPlxCImage *) XPlxGetCImage(video_private->dpy,
					video_private->pw_xid,video_private->pw_gc,video_private->dx,
					video_private->dy,video_private->dw,video_private->dh,
					video_private->vw,video_private->vh);
			}
			else {
				jpeg = (XPlxCImage *) XPlxGetCImage(video_private->dpy,
					video_private->pw_xid,video_private->pw_gc,video_private->dx,
					video_private->dy,video_private->dw,video_private->dh,
					video_private->dw,video_private->dh);
			}
		}
		if (video_private->flags & VideoFirstComp) {
#ifdef TRACE
			fprintf(stderr,"TRACE: XPlxVideoTag(bits=%d)\n",video_private->tagbits);
#endif
			video_private->flags &= ~VideoFirstComp;
			video_private->tagbits = PLX_VIDEO_OVR;
			XPlxVideoTag(video_private->dpy,video_private->pw_xid,
						 video_private->pw_gc,video_private->tagbits);
		}
		return((Xv_opaque) jpeg);
		break;
	case VIDEO_GC:
		return((Xv_opaque) video_private->pw_gc);
		break;
	case VIDEO_SHM_PTR:
/* Old definition
		if ((video_private->flags & VideoUseShmFlag) == 0) return(NULL);
		if  (video_private->ImageCache == NULL)            return(NULL);
		return((Xv_opaque) video_private->ImageCache->data);
*/
		return((Xv_opaque) CreateImageCache(video_private->dpy,
											video_private->dx,video_private->dy));
	case VIDEO_CONF_SIZE:
		return(video_private->conf_size);
		break;
	case VIDEO_QTABLE:
		return((Xv_opaque) video_private->QTable);
		break;
	case VIDEO_COLOR:
/* FIXUP: Get real value */
		return(TRUE);
		break;

/* These attributes can't be gotten */

	case VIDEO_NEW_FRAME:
	case VIDEO_FAST_FRAME:
		xv_error((Xv_object) video_public,
			ERROR_CANNOT_GET,	attr,
			ERROR_PKG,			VIDEO_IN,
			NULL);
		return(XV_OK);
		break;
	default:
		*status = xv_check_bad_attr(VIDEO_IN,attr);
		return(XV_OK);
		break;
	}
}

static int VideoInDestroy(video_public,status)
VideoInPublic  *video_public;
Destroy_status status;
{
	int save;
	Xv_window win;
	plx_IO_list *a;
	VideoInPrivate *video_private = VIDEO_IN_PRIVATE(video_public);

	if ((status == DESTROY_CLEANUP) || (status == DESTROY_PROCESS_DEATH)) {
#ifdef TRACE
		fprintf(stderr,"TRACE: video_destroy called on video canvas\n");
#endif
		if (video_private->flags & VideoCalledFlag) {
			XPlxVideoStop(video_private->dpy,video_private->pw_xid,
				video_private->pw_gc);
#ifdef CLIENTVV
			if (PlxBoardSetup->hardware == PLX_VIDEOVIEW)
				plxvvmapout(video_private->dpy,video_private->pw_xid);
#endif
		}
		XFreeGC(video_private->dpy,video_private->pw_gc);
		a = PlxBoardSetup->inputs;
		while (a) {
			if ((a->channel & 0xff) == video_private->channel) {
				a->channel &= 0xff;
				break;
			}
			a = a->next_IO;
		}
		if (video_private->ImageCache && (video_private->flags & VideoUseShmFlag)) {
			DestroyImageCache(video_private->dpy,video_private->ImageCache);
			video_private->ImageCache = NULL;
		}
		if (video_private->SavedStill) XDestroyImage(video_private->SavedStill);
/* If we are passing a valid error status back, preserve across this call */
		save = Xv_VideoStatus;
		win = (Xv_Window) xv_get(video_private->public_self,XV_OWNER);
		xv_set(win,XV_KEY_DATA,VIDEO_KEY,NULL,NULL);
		Xv_VideoStatus = save;
		free(video_private);
	}
	return(XV_OK);
}

/* end of VideoIn.c */
